import { isJSONObject } from '../helpers/json.js';
import { unmarshalDate, unmarshalArrayOfObject } from '../helpers/marshalling.js';

/**
 * Unmarshals {@link Money}
 *
 * @internal
 */
const unmarshalMoney = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'Money' failed as data isn't a dictionary.`);
  }
  return {
    currencyCode: data.currency_code,
    nanos: data.nanos,
    units: data.units
  };
};

/**
 * Unmarshals {@link ServiceInfo}.
 *
 * @internal
 */
const unmarshalServiceInfo = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ServiceInfo' failed as data isn't a dictionary.`);
  }
  return {
    description: data.description,
    documentationUrl: data.documentation_url,
    name: data.name,
    version: data.version
  };
};

/**
 * Unmarshals {@link ScwFile}.
 *
 * @internal
 */
const unmarshalScwFile = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'ScwFile' failed as data isn't a dictionary.`);
  }
  return {
    content: data.content,
    contentType: data.content_type,
    name: data.name
  };
};

/**
 * Unmarshals {@link TimeSeriesPoint}
 *
 * @remarks To optimize the size of this message,
 * the JSON is compressed in an array instead of a dictionary.
 * Example: `["2019-08-08T15:00:00Z", 0.2]`.
 *
 * @internal
 */
const unmarshalTimeSeriesPoint = data => {
  if (!Array.isArray(data)) {
    throw new TypeError(`Unmarshalling the type 'TimeSeriesPoint' failed as data isn't an array.`);
  }
  return {
    timestamp: unmarshalDate(data[0]),
    value: data[1]
  };
};

/**
 * Unmarshals {@link TimeSeries}
 *
 * @internal
 */
const unmarshalTimeSeries = data => {
  if (!isJSONObject(data)) {
    throw new TypeError(`Unmarshalling the type 'TimeSeries' failed as data isn't a dictionary.`);
  }
  return {
    metadata: data.metadata,
    name: data.name,
    points: unmarshalArrayOfObject(data.points, unmarshalTimeSeriesPoint)
  };
};

/**
 * Marshals {@link ScwFile}.
 *
 * @internal
 */
const marshalScwFile = obj => ({
  content: obj.content,
  content_type: obj.contentType,
  name: obj.name
});

/**
 * Marshals {@link Money}
 *
 * @internal
 */
const marshalMoney = obj => ({
  currency_code: obj.currencyCode,
  nanos: obj.nanos,
  units: obj.units
});

/**
 * Marshals {@link TimeSeriesPoint}
 *
 * @internal
 */
const marshalTimeSeriesPoint = obj => ({
  timestamp: obj.timestamp?.toISOString(),
  value: obj.value
});

/**
 * Marshals {@link TimeSeries}
 *
 * @internal
 */
const marshalTimeSeries = obj => ({
  metadata: obj.metadata,
  name: obj.name,
  points: obj.points.map(elt => marshalTimeSeriesPoint(elt))
});

export { marshalMoney, marshalScwFile, marshalTimeSeries, marshalTimeSeriesPoint, unmarshalMoney, unmarshalScwFile, unmarshalServiceInfo, unmarshalTimeSeries, unmarshalTimeSeriesPoint };
