/**
 * Function to retry logic between each interval.
 *
 * @returns The result and if it's done
 *
 * @throws An exception might be thrown by the logic being run
 *
 * @internal
 */
type Retry<T> = () => Promise<IteratorResult<T, T>>;
/**
 * Generated interval strategy iterator.
 *
 * @internal
 */
type IntervalStrategy = Generator<number, never | number, number>;
/**
 * Creates an exponential backoff interval strategy.
 *
 * @param minDelay - The minimum delay before the next try in seconds
 * @param maxDelay - The maximum delay before the next try in seconds
 * @returns An exponential backoff generator
 *
 * @internal
 */
declare function createExponentialBackoffStrategy(minDelay: number, maxDelay: number): IntervalStrategy;
/**
 * Tries a specific logic several times until it succeeds, timeouts, or throws an exception.
 *
 * @param retry - The function to retry logic between each interval
 * @param strategy - A generated interval strategy iterator
 * @param timeout - The maximum time elapsed before timeout error
 *
 * @throws An timeout exception or error thrown by the logic being run
 *
 * @internal
 */
declare const tryAtIntervals: <T>(retry: Retry<T>, strategy: IntervalStrategy, timeout?: number) => Promise<T>;
/**
 * Represents the condition to stop waiting for a resource.
 *
 * @public
 */
type WaitForStopCondition<T> = (obj: T) => Promise<boolean>;
/**
 * The options to wait until a resource is ready.
 *
 * @public
 */
type WaitForOptions<T> = {
    /**
     * Timeout in seconds.
     *
     * @defaultValue 300 seconds (5 minutes).
     */
    timeout?: number;
    /**
     * The minimum delay before the next try in seconds.
     *
     * @defaultValue 1 second.
     */
    minDelay?: number;
    /**
     * The maximum delay before the next try in seconds.
     *
     * @defaultValue 30 seconds.
     */
    maxDelay?: number;
    /**
     * The condition to stop trying.
     *
     * @defaultValue Waits for non-transient value.
     */
    stop?: WaitForStopCondition<T>;
};
type ResourceFetcher<T, R> = (request: R) => Promise<T>;
/**
 * Fetches resource several times until an expected condition is reached, timeouts, or throws an exception.
 *
 * @param stop - The condition to stop waiting
 * @param fetcher - The method to retrieve resource
 * @param request - The resource request options
 * @param options - The retry strategy options
 * @param strategy - An optional custom strategy
 *
 * @returns A promise of resource
 *
 * @public
 */
declare const waitForResource: <R, T>(stop: WaitForStopCondition<T>, fetcher: ResourceFetcher<T, R>, request: R, options?: WaitForOptions<T> | undefined, strategy?: IntervalStrategy) => Promise<Awaited<T>>;

/**
 * Defines the interceptor for a `Request`.
 * This allows you to intercept requests before starting.
 *
 * @example
 * Adds a custom header to a request:
 * ```
 * const addCustomHeaderInterceptor
 *   ({ key, value }: { key: string; value: string }): RequestInterceptor =>
 *   ({ request }) => {
 *     const clone = request.clone()
 *     clone.headers.set(key, value)
 *
 *     return clone
 * }
 * ```
 *
 * @public
 */
interface RequestInterceptor {
    ({ request }: {
        request: Readonly<Request>;
    }): Request | Promise<Request>;
}
/**
 * Defines the interceptor for a `Response`.
 * This allows you to intercept responses before unmarshalling.
 *
 * @example
 * Adds a delay before sending the response:
 * ```
 * const addDelayInterceptor: ResponseInterceptor = ({ response }) =>
 *   new Promise(resolve => {
 *     setTimeout(() => resolve(response), 1000)
 *   })
 * ```
 *
 * @public
 */
interface ResponseInterceptor {
    ({ response }: {
        response: Readonly<Response>;
    }): Response | Promise<Response>;
}
/**
 * Defines the interceptor for a `Response` error.
 * This allows you to intercept a response error before exiting the whole process.
 * You can either rethrow an error, and resolve with a different response.
 *
 * @remarks
 * You must return either:
 * 1. An error (`throw error` or `Promise.reject(error)`)
 * 2. Data (directly, or via a Promise)
 *
 * @example
 * Reports error to tracking service:
 * ```
 * const reportErrorToTrackingService: ResponseErrorInterceptor = async ({
 *   request,
 *   error,
 * }: {
 *   request: Request
 *   error: unknown
 * }) => {
 *   await sendErrorToErrorService(request, error)
 *   throw error
 * }
 * ```
 *
 * @public
 */
interface ResponseErrorInterceptor {
    ({ request, error, }: {
        request: Request;
        error: unknown;
    }): unknown | Promise<unknown>;
}
/**
 * Defines the network interceptors.
 * Please check the documentation of {@link RequestInterceptor},
 * {@link ResponseInterceptor} and {@link ResponseErrorInterceptor} for examples.
 *
 * @public
 */
type NetworkInterceptors = {
    request?: RequestInterceptor;
    response?: ResponseInterceptor;
    responseError?: ResponseErrorInterceptor;
};

declare enum LevelResolver {
    silent = 0,
    error = 1,
    warn = 2,
    info = 3,
    debug = 4
}
type LogLevel = keyof typeof LevelResolver;

/**
 * Logger.
 *
 * @public
 */
interface Logger {
    readonly logLevel: LogLevel;
    debug: (message: string) => void;
    info: (message: string) => void;
    warn: (message: string) => void;
    error: (message: string) => void;
}

/**
 * Sets a logger to be used within the SDK.
 *
 * @param logger - The Logger instance
 *
 * @public
 */
declare const setLogger: (logger: Readonly<Logger>) => void;
/**
 * Sets the logger to console logger with given logLevel (log is disabled by default).
 *
 * @param logLevel - The Log level (default to 'warn')
 * @param prefix - A Log message prefix (default to 'scaleway-sdk-js:')
 *
 * @public
 */
declare const enableConsoleLogger: (logLevel?: LogLevel, prefix?: string) => void;

/* eslint-disable @typescript-eslint/ban-types */
type Region = 'fr-par' | 'nl-ams' | 'pl-waw' | (string & {})

type Zone =
  | 'fr-par-1'
  | 'fr-par-2'
  | 'fr-par-3'
  | 'nl-ams-1'
  | 'nl-ams-2'
  | 'nl-ams-3'
  | 'pl-waw-1'
  | 'pl-waw-2'
  | (string & {})

/**
 * Holds access key and secret key.
 *
 * @public
 */
interface AuthenticationSecrets {
    /**
     * You need an access key and a secret key to connect to Scaleway API.
     * Generate your access key at the following address: {@link https://console.scaleway.com/project/credentials}.
     */
    accessKey: string;
    /**
     * The secret key is the value that can be used to authenticate against the API (the value used in X-Auth-Token HTTP-header).
     * The secret key MUST remain secret and not given to anyone or published online.
     * Generate your secret key at the following address: {@link https://console.scaleway.com/project/credentials}.
     */
    secretKey: string;
}
/**
 * Holds default values of a Scaleway profile.
 *
 * @public
 */
interface ProfileDefaultValues {
    /**
     * APIURL overrides the API URL of the Scaleway API to the given URL.
     * Change that if you want to direct requests to a different endpoint.
     *
     * @defaultValue `https://api.scaleway.com`
     */
    apiURL: string;
    /**
     * Your organization ID is the identifier of your account inside Scaleway infrastructure.
     */
    defaultOrganizationId?: string;
    /**
     * Your project ID is the identifier of the project your resources are attached to.
     */
    defaultProjectId?: string;
    /**
     * A region is represented as a geographical area such as France (Paris) or the Netherlands (Amsterdam).
     * It can contain multiple availability zones.
     *
     * Examples: fr-par, nl-ams.
     */
    defaultRegion?: Region;
    /**
     * A region can be split into many availability zones (AZ).
     * Latency between multiple AZ of the same region are low as they have a common network layer.
     *
     * Examples: fr-par-1, nl-ams-1
     */
    defaultZone?: Zone;
}
/**
 * Holds values of a Scaleway profile.
 *
 * @public
 */
type Profile = Partial<ProfileDefaultValues & AuthenticationSecrets>;

/**
 * Holds default values of settings.
 *
 * @public
 */
type DefaultValues = ProfileDefaultValues & {
    /**
     * The default number of results when requesting a paginated resource.
     */
    defaultPageSize?: number;
};
/**
 * Settings hold the values of all client options.
 *
 * @public
 */
interface Settings extends DefaultValues {
    /**
     * The default number of results when requesting a paginated resource.
     */
    defaultPageSize?: number;
    /**
     * HTTP Client doing the requests.
     */
    httpClient: typeof fetch;
    /**
     * The Request interceptors.
     *
     * @deprecated Please use `interceptors` instead.
     */
    requestInterceptors?: RequestInterceptor[];
    /**
     * The Response interceptors.
     *
     * @deprecated Please use `interceptors` instead.
     */
    responseInterceptors?: ResponseInterceptor[];
    /**
     * The Network interceptors.
     */
    interceptors: NetworkInterceptors[];
    /**
     * The User-Agent sent with each request.
     *
     * @defaultValue scaleway-sdk-js/version
     */
    userAgent: string;
}

/**
 * A factory to build {@link Settings}.
 *
 * @public
 */
type ClientConfig = (obj: Settings) => Settings;
/**
 * Instantiates the SDK from a configuration {@link Profile}.
 *
 * @param profile - The profile
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
declare const withProfile: (profile: Readonly<Profile>) => (settings: Readonly<Settings>) => Settings;
/**
 * Instantiates the SDK with a different HTTP client.
 *
 * @param httpClient - A fetch compatible HTTP client
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
declare const withHTTPClient: (httpClient: typeof fetch) => (settings: Readonly<Settings>) => Settings;
/**
 * Instantiates the SDK with a default page size.
 *
 * @param defaultPageSize - The default page size
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
declare const withDefaultPageSize: (defaultPageSize: number) => ClientConfig;
/**
 * Instantiates the SDK with a different default user agent.
 *
 * @param userAgent - The default user agent
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
declare const withUserAgent: (userAgent: string) => (settings: Readonly<Settings>) => Settings;
/**
 * Instantiates the SDK with an additional user agent.
 *
 * @param userAgent - The suffix to append to default user agent
 * @returns A factory {@link ClientConfig}
 *
 * @remarks This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @public
 */
declare const withUserAgentSuffix: (userAgent: string) => (settings: Readonly<Settings>) => Settings;
/**
 * Instantiates the SDK with additional interceptors.
 *
 * @param interceptors - The additional {@link NetworkInterceptors} interceptors
 * @returns A factory {@link ClientConfig}
 *
 * @remarks
 * It doesn't override the existing interceptors, but instead push more to the list.
 * This method should be used in conjunction with the initializer `createAdvancedClient`.
 *
 * @example
 * ```
 * withAdditionalInterceptors([
 *   {
 *     request: ({ request }) => {
 *       console.log(`Do something with ${JSON.stringify(request)}`)
 *       return request
 *     },
 *     response: ({ response }) => {
 *       console.log(`Do something with ${JSON.stringify(response)}`)
 *       return response
 *     },
 *     responseError: async ({
 *       request,
 *       error,
 *     }: {
 *       request: Request
 *       error: unknown
 *     }) => {
 *       console.log(
 *         `Do something with ${JSON.stringify(request)} and ${JSON.stringify(
 *           error,
 *         )}`,
 *       )
 *       throw error // or return Promise.resolve(someData)
 *     },
 *   },
 * ])
 * ```
 *
 * @public
 */
declare const withAdditionalInterceptors: (interceptors: NetworkInterceptors[]) => (settings: Readonly<Settings>) => Settings;

/** Scaleway Request. */
type ScwRequest = {
  method: 'GET' | 'POST' | 'PUT' | 'DELETE' | 'PATCH'
  path: string
  headers?: Record<string, string>
  body?: string
  urlParams?: URLSearchParams
  responseType?: 'json' | 'text' | 'blob'
}

/**
 * A factory to unmarshal a response.
 *
 * @param obj - The input object.
 * @returns The output object
 */
type ResponseUnmarshaller<T> = (obj: unknown) => T

type Fetcher = <T>(request: Readonly<ScwRequest>, unwrapper?: ResponseUnmarshaller<T>) => Promise<T>;

/**
 * Scaleway client.
 */
type Client = {
    fetch: Fetcher;
    settings: Settings;
};
/**
 * Creates a Scaleway client with advanced options.
 * You can either use existing factories
 * (like `withProfile`, `withUserAgentSuffix`, etc)
 * or write your own using the interface `ClientConfig`.
 *
 * @example
 * Creates a client with factories:
 * ```
 * createAdvancedClient(
 *   (obj: Settings) => ({
 *     ...obj,
 *     defaultPageSize: 100 ,
 *     httpClient: myFetchWrapper,
 *   }),
 *   withUserAgentSuffix('bot-name/1.0'),
 * )
 * ```
 *
 * @throws Error
 * Thrown if the setup fails.
 *
 * @public
 */
declare const createAdvancedClient: (...configs: ClientConfig[]) => Client;
/**
 * Creates a Scaleway client with a profile.
 *
 * @example
 * Creates a client with credentials & default values (see https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/):
 * ```
 * import { createClient } from '@scaleway/sdk'
 *
 * createClient({
 *   accessKey: 'SCWXXXXXXXXXXXXXXXXX',
 *   secretKey: 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx',
 *   defaultProjectId: 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx',
 *   defaultRegion: 'fr-par',
 *   defaultZone: 'fr-par-1',
 * })
 * ```
 *
 * @example
 * Creates a client by loading values from the environment (see https://www.scaleway.com/en/docs/identity-and-access-management/iam/how-to/create-api-keys/)
 * or the config file created by CLI `scw init` (see https://www.scaleway.com/en/cli/):
 * ```
 * import { loadProfileFromConfigurationFile } from '@scaleway/configuration-loader'
 * import { createClient } from '@scaleway/sdk'
 *
 * createClient({
 *   ...await loadProfileFromConfigurationFile(),
 *   defaultZone: 'fr-par-3',
 * })
 * ```
 *
 * @throws Error
 * Thrown if the setup fails.
 *
 * @public
 */
declare const createClient: (profile?: Profile) => Client;

/**
 * Represents API metadata.
 *
 * @remarks These metadata are only here for debugging. Do not rely on these values.
 *
 * @internal
 */
interface ServiceInfo {
  /** Name of the API */
  name: string
  /** Human readable description for the API. */
  description: string
  /** Version of the API. */
  version: string
  /** Web url where the documentation of the API can be found. */
  documentationUrl?: string
}

/**
 * Represents an amount of money with its currency type.
 */
interface Money {
  /** 3-letter currency code defined in ISO 4217. */
  currencyCode: string
  /**
   * Whole units of the amount.
   *
   * For example if `currencyCode` is `"USD"`, then 1 unit is one US dollar.
   */
  units: number
  /**
   * Number of nano (10^-9) units of the amount.
   *
   * The value must be between -999,999,999 and +999,999,999 inclusive.
   * If `units` is positive, `nanos` must be positive or zero.
   * If `units` is zero, `nanos` can be positive, zero, or negative.
   * If `units` is negative, `nanos` must be negative or zero.
   * For example $-1.75 is represented as `units`=-1 and `nanos`=-750,000,000.
   */
  nanos: number
}

/** Represents a point in a TimeSeries. */
interface TimeSeriesPoint {
  /** Date of the point. */
  timestamp?: Date
  /** Value of the point. */
  value: number
}

/** Represents a time series that could be used for graph purposes. */
interface TimeSeries {
  /** Name of the metric. */
  name: string
  /** Points contains all the points that composed the series. */
  points: TimeSeriesPoint[]
  /** Metadata contains some string metadata related to a metric. */
  metadata: Record<string, string>
}

/** Represents a Scaleway file. */
interface ScwFile {
  /** Name of the file. */
  name: string
  /** Content-type of the file. */
  contentType: string
  /** Content of the file in base64. */
  content: string
}

type JSON = string | number | boolean | null | JSON[] | {
    [key: string]: JSON;
};
type JSONObject = {
    [key: string]: JSON;
};
/**
 * Validates an unknown object is a JSON Object.
 *
 * @internal
 */
declare const isJSONObject: (obj: unknown) => obj is JSONObject;

/**
 * Scaleway error.
 *
 * @public
 */
declare class ScalewayError extends Error {
    /** The response status. */
    readonly status: number;
    /** The response payload. */
    readonly body: JSONObject | string;
    /** The augmented message. */
    readonly message: string;
    /** The message originating from the payload. */
    rawMessage: string | undefined;
    constructor(
    /** The response status. */
    status: number, 
    /** The response payload. */
    body: JSONObject | string, 
    /** The augmented message. */
    message?: string);
    static fromJSON(status: number, obj: Readonly<JSONObject>): ScalewayError | null;
    toString(): string;
}

/**
 * AlreadyExists error is used when a resource already exists.
 *
 * @public
 */
declare class AlreadyExistsError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly resource: string;
    readonly resourceId: string;
    readonly helpMessage: string;
    constructor(status: number, body: JSONObject, resource: string, resourceId: string, helpMessage: string);
    static fromJSON(status: number, obj: Readonly<JSONObject>): AlreadyExistsError | null;
}

/**
 * DeniedAuthentication error is used by the API Gateway auth service to deny a request.
 *
 * @public
 */
declare class DeniedAuthenticationError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly method: string;
    readonly reason: string;
    constructor(status: number, body: JSONObject, method: string, reason: string);
    static fromJSON(status: number, obj: Readonly<JSONObject>): DeniedAuthenticationError | null;
}

/**
 * Details of an {@link InvalidArgumentsError} error.
 *
 * @public
 */
type InvalidArgumentsErrorDetails = {
    readonly argumentName: string;
    readonly reason: string;
    readonly helpMessage?: string;
};
/**
 * InvalidArguments error happens when one or many fields are invalid in the request message.
 *
 * @public
 */
declare class InvalidArgumentsError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly details: InvalidArgumentsErrorDetails[];
    constructor(status: number, body: JSONObject, details: InvalidArgumentsErrorDetails[]);
    static fromJSON(status: number, obj: Readonly<JSONObject>): InvalidArgumentsError | null;
}

/**
 * OutOfStock error happens when stocks are empty for the resource.
 *
 * @public
 */
declare class OutOfStockError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly resource: string;
    constructor(status: number, body: JSONObject, resource: string);
    static fromJSON(status: number, obj: Readonly<JSONObject>): OutOfStockError | null;
}

/**
 * Details of an {@link PermissionsDeniedError} error.
 *
 * @public
 */
type PermissionsDeniedErrorDetails = {
    readonly resource: string;
    readonly action: string;
};
/**
 * PermissionsDenied error happens when one or many permissions are not accorded to the user making the request.
 *
 * @public
 */
declare class PermissionsDeniedError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly list: PermissionsDeniedErrorDetails[];
    constructor(status: number, body: JSONObject, list: PermissionsDeniedErrorDetails[]);
    static fromJSON(status: number, obj: Readonly<JSONObject>): ScalewayError | null;
}

/**
 * PreconditionFailed error is used when a precondition is not respected.
 *
 * @public
 */
declare class PreconditionFailedError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly precondition: string;
    readonly helpMessage: string;
    constructor(status: number, body: JSONObject, precondition: string, helpMessage: string);
    static fromJSON(status: number, obj: Readonly<JSONObject>): PreconditionFailedError | null;
}

/**
 * Scope of an {@link QuotasExceededErrorDetails} error.
 *
 * @public
 */
type QuotasExceededErrorScope = {
    kind: 'organization' | 'project';
    id: string;
};
/**
 * Details of an {@link QuotasExceededError} error.
 *
 * @public
 */
type QuotasExceededErrorDetails = {
    readonly resource: string;
    readonly quota: number;
    readonly current: number;
    readonly scope?: QuotasExceededErrorScope;
};
/**
 * QuotasExceeded error happens when one or many resource exceed quotas during the creation of a resource.
 *
 * @public
 */
declare class QuotasExceededError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly list: QuotasExceededErrorDetails[];
    constructor(status: number, body: JSONObject, list: QuotasExceededErrorDetails[]);
    static fromJSON(status: number, obj: Readonly<JSONObject>): QuotasExceededError | null;
}

/**
 * ResourceExpired error happens when trying to access a resource that has expired.
 *
 * @public
 */
declare class ResourceExpiredError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly resource: string;
    readonly resourceId: string;
    readonly expiredSince: Date;
    constructor(status: number, body: JSONObject, resource: string, resourceId: string, expiredSince: Date);
    static fromJSON(status: number, obj: Readonly<JSONObject>): ResourceExpiredError | null;
}

/**
 * ResourceLocked error happens when a resource is locked by trust and safety.
 *
 * @public
 */
declare class ResourceLockedError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly resource: string;
    readonly resourceId: string;
    constructor(status: number, body: JSONObject, resource: string, resourceId: string);
    static fromJSON(status: number, obj: Readonly<JSONObject>): ResourceLockedError | null;
}

/**
 * ResourceNotFound error happens when getting a resource that does not exist anymore.
 *
 * @public
 */
declare class ResourceNotFoundError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly resource: string;
    readonly resourceId: string;
    constructor(status: number, body: JSONObject, resource: string, resourceId: string);
    static fromJSON(status: number, obj: Readonly<JSONObject>): ResourceNotFoundError | null;
}

/**
 * Quota policy of {@link TooManyRequestsError}.
 *
 * @public
 */
type TooManyRequestsQuotaPolicy = {
    quota: number;
    windowSeconds?: number;
};
/**
 * TooManyRequestsError error happens when fetching too many times a resource.
 *
 * @public
 */
declare class TooManyRequestsError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly helpMessage: string;
    readonly limit?: TooManyRequestsQuotaPolicy | undefined;
    /** The number of seconds until the quota resets */
    readonly resetSeconds?: number | undefined;
    /** The timestamp when the quota resets */
    readonly resetAt?: Date | undefined;
    constructor(status: number, body: JSONObject, helpMessage: string, limit?: TooManyRequestsQuotaPolicy | undefined, 
    /** The number of seconds until the quota resets */
    resetSeconds?: number | undefined, 
    /** The timestamp when the quota resets */
    resetAt?: Date | undefined);
    static fromJSON(status: number, obj: Readonly<JSONObject>): TooManyRequestsError | null;
}

/**
 * TransientState error happens when trying to perform an action on a resource in a transient state.
 *
 * @public
 */
declare class TransientStateError extends ScalewayError {
    readonly status: number;
    readonly body: JSONObject;
    readonly resource: string;
    readonly resourceId: string;
    readonly currentState: string;
    constructor(status: number, body: JSONObject, resource: string, resourceId: string, currentState: string);
    static fromJSON(status: number, obj: Readonly<JSONObject>): ScalewayError | null;
}

type index$z_AlreadyExistsError = AlreadyExistsError;
declare const index$z_AlreadyExistsError: typeof AlreadyExistsError;
type index$z_DeniedAuthenticationError = DeniedAuthenticationError;
declare const index$z_DeniedAuthenticationError: typeof DeniedAuthenticationError;
type index$z_InvalidArgumentsError = InvalidArgumentsError;
declare const index$z_InvalidArgumentsError: typeof InvalidArgumentsError;
type index$z_InvalidArgumentsErrorDetails = InvalidArgumentsErrorDetails;
type index$z_OutOfStockError = OutOfStockError;
declare const index$z_OutOfStockError: typeof OutOfStockError;
type index$z_PermissionsDeniedError = PermissionsDeniedError;
declare const index$z_PermissionsDeniedError: typeof PermissionsDeniedError;
type index$z_PermissionsDeniedErrorDetails = PermissionsDeniedErrorDetails;
type index$z_PreconditionFailedError = PreconditionFailedError;
declare const index$z_PreconditionFailedError: typeof PreconditionFailedError;
type index$z_QuotasExceededError = QuotasExceededError;
declare const index$z_QuotasExceededError: typeof QuotasExceededError;
type index$z_QuotasExceededErrorDetails = QuotasExceededErrorDetails;
type index$z_QuotasExceededErrorScope = QuotasExceededErrorScope;
type index$z_ResourceExpiredError = ResourceExpiredError;
declare const index$z_ResourceExpiredError: typeof ResourceExpiredError;
type index$z_ResourceLockedError = ResourceLockedError;
declare const index$z_ResourceLockedError: typeof ResourceLockedError;
type index$z_ResourceNotFoundError = ResourceNotFoundError;
declare const index$z_ResourceNotFoundError: typeof ResourceNotFoundError;
type index$z_ScalewayError = ScalewayError;
declare const index$z_ScalewayError: typeof ScalewayError;
type index$z_TooManyRequestsError = TooManyRequestsError;
declare const index$z_TooManyRequestsError: typeof TooManyRequestsError;
type index$z_TooManyRequestsQuotaPolicy = TooManyRequestsQuotaPolicy;
type index$z_TransientStateError = TransientStateError;
declare const index$z_TransientStateError: typeof TransientStateError;
declare namespace index$z {
  export { index$z_AlreadyExistsError as AlreadyExistsError, index$z_DeniedAuthenticationError as DeniedAuthenticationError, index$z_InvalidArgumentsError as InvalidArgumentsError, type index$z_InvalidArgumentsErrorDetails as InvalidArgumentsErrorDetails, index$z_OutOfStockError as OutOfStockError, index$z_PermissionsDeniedError as PermissionsDeniedError, type index$z_PermissionsDeniedErrorDetails as PermissionsDeniedErrorDetails, index$z_PreconditionFailedError as PreconditionFailedError, index$z_QuotasExceededError as QuotasExceededError, type index$z_QuotasExceededErrorDetails as QuotasExceededErrorDetails, type index$z_QuotasExceededErrorScope as QuotasExceededErrorScope, index$z_ResourceExpiredError as ResourceExpiredError, index$z_ResourceLockedError as ResourceLockedError, index$z_ResourceNotFoundError as ResourceNotFoundError, index$z_ScalewayError as ScalewayError, index$z_TooManyRequestsError as TooManyRequestsError, type index$z_TooManyRequestsQuotaPolicy as TooManyRequestsQuotaPolicy, index$z_TransientStateError as TransientStateError };
}

/**
 * Adds asynchronously an header to a request through an interceptor.
 *
 * @param key - The header key
 * @param value - The header value as a Promise
 * @returns The Request interceptor
 *
 * @internal
 */
declare const addAsyncHeaderInterceptor: (key: string, getter: () => Promise<string | undefined>) => RequestInterceptor;

/**
 * Abstract class to instantiate API from a {@link Client}.
 *
 * @internal
 */
declare abstract class API$x {
    protected client: Client;
    constructor(client: Client);
}

interface TokenAccessor {
    (): Promise<string | undefined>;
}
/**
 * Authenticates with a session token.
 *
 * @param getToken - The token accessor
 * @returns The request interceptor
 *
 * @deprecated Please use addAsyncHeaderInterceptor instead.
 *
 * @internal
 */
declare const authenticateWithSessionToken: (getToken: TokenAccessor) => RequestInterceptor;

/**
 * Unmarshals {@link Money}
 *
 * @internal
 */
declare const unmarshalMoney: (data: unknown) => Money;
/**
 * Unmarshals {@link ServiceInfo}.
 *
 * @internal
 */
declare const unmarshalServiceInfo: (data: unknown) => ServiceInfo;
/**
 * Unmarshals {@link ScwFile}.
 *
 * @internal
 */
declare const unmarshalScwFile: (data: unknown) => ScwFile;
/**
 * Unmarshals {@link TimeSeriesPoint}
 *
 * @remarks To optimize the size of this message,
 * the JSON is compressed in an array instead of a dictionary.
 * Example: `["2019-08-08T15:00:00Z", 0.2]`.
 *
 * @internal
 */
declare const unmarshalTimeSeriesPoint: (data: unknown) => TimeSeriesPoint;
/**
 * Unmarshals {@link TimeSeries}
 *
 * @internal
 */
declare const unmarshalTimeSeries: (data: unknown) => TimeSeries;
/**
 * Marshals {@link ScwFile}.
 *
 * @internal
 */
declare const marshalScwFile: (obj: ScwFile) => Record<string, unknown>;
/**
 * Marshals {@link Money}
 *
 * @internal
 */
declare const marshalMoney: (obj: Money) => Record<string, unknown>;
/**
 * Marshals {@link TimeSeries}
 *
 * @internal
 */
declare const marshalTimeSeries: (obj: TimeSeries) => Record<string, unknown>;

/**
 * Returns the parameter if it's valid as path parameter
 * (string and not empty, or number), else throws an exception.
 *
 * @param name - The parameter name
 * @param param - The parameter value
 * @returns The parameter value
 *
 * @throws TypeError
 * Thrown if the parameter is invalid.
 *
 * @internal
 */
declare function validatePathParam(name: string, param: string | number | undefined): string;
/**
 * Resolves the ideal parameter and value amongst an optional list.
 *
 * @param list - The list to be looking into
 * @param isRequired - If at least one "one-of" should be found, false by default
 * @returns The parameter and value
 *
 * @throws TypeError
 * Thrown if isRequired is true, and no value or default value is specified.
 *
 * @internal
 */
declare const resolveOneOf: <T>(list: {
    param: string;
    value: T | undefined;
    default?: T | undefined;
}[], isRequired?: boolean) => Record<string, T>;
type URLParameterValue = string | number | boolean | Date | null;
/**
 * Filters defined parameters tuples and converts them to URLSearchParams.
 *
 * @param paramTuples - The key/value pairs
 * @returns URLSearchParams
 *
 * @internal
 */
declare const urlParams: (...paramTuples: Readonly<[
    string,
    URLParameterValue | URLParameterValue[] | undefined
]>[]) => URLSearchParams;
/**
 * Unmarshals data to Date object.
 *
 * @internal
 */
declare const unmarshalDate: (data: unknown) => Date | undefined;
/**
 * Unmarshals array of object.
 *
 * @internal
 */
declare const unmarshalArrayOfObject: <T, B extends boolean>(data: unknown, unmarshaller: (input: unknown) => T, emptyFallback?: B) => B extends true ? T[] | undefined : T[];
/**
 * Unmarshals map of object.
 *
 * @internal
 */
declare const unmarshalMapOfObject: <T, B extends boolean>(data: unknown, unmarshaller: (input: unknown) => T, emptyFallback?: B) => B extends true ? Record<string, T> | undefined : Record<string, T>;

interface PaginationOptions {
    page?: number;
    pageSize?: number;
}
interface PaginatedResponse {
    totalCount: number;
}
type PaginatedFetcher<T, R extends PaginationOptions = PaginationOptions> = (request: R) => Promise<T>;
type PaginatedContent<K extends string, T = unknown> = PaginatedResponse & {
    [key in K]: T[];
};
/**
 * Enriches a listing method with helpers.
 *
 * @param key - The resource key of values list
 * @param fetcher - The method to retrieve paginated resources
 * @param request - A request with pagination options
 * @returns A resource Promise with the pagination helpers
 *
 * @internal
 */
declare const enrichForPagination: <K extends string, T extends PaginatedContent<K, unknown>, R extends PaginationOptions>(key: K, fetcher: PaginatedFetcher<T, R>, request: R) => Promise<T> & {
    all: () => Promise<(Awaited<T[K]> extends infer T_1 ? T_1 extends Awaited<T[K]> ? T_1 extends readonly (infer InnerArr)[] ? InnerArr : T_1 : never : never)[]>;
    [Symbol.asyncIterator]: () => AsyncGenerator<T[K], void, void>;
};

type ListProjectsRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
interface Project$1 {
    /** ID of the Project. */
    id: string;
    /** Name of the Project. */
    name: string;
    /** Organization ID of the Project. */
    organizationId: string;
    /** Creation date of the Project. */
    createdAt?: Date;
    /** Update date of the Project. */
    updatedAt?: Date;
    /** Description of the Project. */
    description: string;
}
type CreateProjectRequest = {
    /** Name of the Project. */
    name?: string;
    /** Organization ID of the Project. */
    organizationId?: string;
    /** Description of the Project. */
    description?: string;
};
type DeleteProjectRequest = {
    /** Project ID of the Project. */
    projectId?: string;
};
type GetProjectRequest = {
    /** Project ID of the Project. */
    projectId?: string;
};
type ListProjectsRequest = {
    /** Organization ID of the Project. */
    organizationId?: string;
    /** Name of the Project. */
    name?: string;
    /** Page number for the returned Projects. */
    page?: number;
    /** Maximum number of Project per page. */
    pageSize?: number;
    /** Sort order of the returned Projects. */
    orderBy?: ListProjectsRequestOrderBy$1;
    /**
     * Project IDs to filter for. The results will be limited to any Projects with
     * an ID in this array.
     */
    projectIds?: string[];
};
interface ListProjectsResponse$1 {
    /** Total number of Projects. */
    totalCount: number;
    /** Paginated returned Projects. */
    projects: Project$1[];
}
type UpdateProjectRequest = {
    /** Project ID of the Project. */
    projectId?: string;
    /** Name of the Project. */
    name?: string;
    /** Description of the Project. */
    description?: string;
};

/**
 * Account API.
 *
 * User related data. This API allows you to manage projects.
 */
declare class API$w extends API$x {
    /**
     * Create a new Project for an Organization. Deprecated in favor of Account
     * API v3. Generate a new Project for an Organization, specifying its
     * configuration including name and description.
     *
     * @deprecated
     * @param request - The request {@link CreateProjectRequest}
     * @returns A Promise of Project
     */
    createProject: (request?: Readonly<CreateProjectRequest>) => Promise<Project$1>;
    protected pageOfListProjects: (request?: Readonly<ListProjectsRequest>) => Promise<ListProjectsResponse$1>;
    /**
     * List all Projects of an Organization. Deprecated in favor of Account API
     * v3. List all Projects of an Organization. The response will include the
     * total number of Projects as well as their associated Organizations, names
     * and IDs. Other information include the creation and update date of the
     * Project.
     *
     * @deprecated
     * @param request - The request {@link ListProjectsRequest}
     * @returns A Promise of ListProjectsResponse
     */
    listProjects: (request?: Readonly<ListProjectsRequest>) => Promise<ListProjectsResponse$1> & {
        all: () => Promise<Project$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Project$1[], void, void>;
    };
    /**
     * Get an existing Project. Deprecated in favor of Account API v3. Retrieve
     * information about an existing Project, specified by its Project ID. Its
     * full details, including ID, name and description, are returned in the
     * response object.
     *
     * @deprecated
     * @param request - The request {@link GetProjectRequest}
     * @returns A Promise of Project
     */
    getProject: (request?: Readonly<GetProjectRequest>) => Promise<Project$1>;
    /**
     * Delete an existing Project. Deprecated in favor of Account API v3. Delete
     * an existing Project, specified by its Project ID. The Project needs to be
     * empty (meaning there are no resources left in it) to be deleted
     * effectively. Note that deleting a Project is permanent, and cannot be
     * undone.
     *
     * @deprecated
     * @param request - The request {@link DeleteProjectRequest}
     */
    deleteProject: (request?: Readonly<DeleteProjectRequest>) => Promise<void>;
    /**
     * Update Project. Deprecated in favor of Account API v3. Update the
     * parameters of an existing Project, specified by its Project ID. These
     * parameters include the name and description.
     *
     * @deprecated
     * @param request - The request {@link UpdateProjectRequest}
     * @returns A Promise of Project
     */
    updateProject: (request?: Readonly<UpdateProjectRequest>) => Promise<Project$1>;
}

type index_gen$v_CreateProjectRequest = CreateProjectRequest;
type index_gen$v_DeleteProjectRequest = DeleteProjectRequest;
type index_gen$v_GetProjectRequest = GetProjectRequest;
type index_gen$v_ListProjectsRequest = ListProjectsRequest;
type index_gen$v_UpdateProjectRequest = UpdateProjectRequest;
declare namespace index_gen$v {
  export { API$w as API, type index_gen$v_CreateProjectRequest as CreateProjectRequest, type index_gen$v_DeleteProjectRequest as DeleteProjectRequest, type index_gen$v_GetProjectRequest as GetProjectRequest, type index_gen$v_ListProjectsRequest as ListProjectsRequest, type ListProjectsRequestOrderBy$1 as ListProjectsRequestOrderBy, type ListProjectsResponse$1 as ListProjectsResponse, type Project$1 as Project, type index_gen$v_UpdateProjectRequest as UpdateProjectRequest };
}

type ListProjectsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
interface Project {
    /** ID of the Project. */
    id: string;
    /** Name of the Project. */
    name: string;
    /** Organization ID of the Project. */
    organizationId: string;
    /** Creation date of the Project. */
    createdAt?: Date;
    /** Update date of the Project. */
    updatedAt?: Date;
    /** Description of the Project. */
    description: string;
}
interface ListProjectsResponse {
    /** Total number of Projects. */
    totalCount: number;
    /** Paginated returned Projects. */
    projects: Project[];
}
type ProjectApiCreateProjectRequest$1 = {
    /** Name of the Project. */
    name?: string;
    /** Organization ID of the Project. */
    organizationId?: string;
    /** Description of the Project. */
    description: string;
};
type ProjectApiDeleteProjectRequest = {
    /** Project ID of the Project. */
    projectId?: string;
};
type ProjectApiGetProjectRequest = {
    /** Project ID of the Project. */
    projectId?: string;
};
type ProjectApiListProjectsRequest$1 = {
    /** Organization ID of the Project. */
    organizationId?: string;
    /** Name of the Project. */
    name?: string;
    /** Page number for the returned Projects. */
    page?: number;
    /** Maximum number of Project per page. */
    pageSize?: number;
    /** Sort order of the returned Projects. */
    orderBy?: ListProjectsRequestOrderBy;
    /**
     * Project IDs to filter for. The results will be limited to any Projects with
     * an ID in this array.
     */
    projectIds?: string[];
};
type ProjectApiUpdateProjectRequest$1 = {
    /** Project ID of the Project. */
    projectId?: string;
    /** Name of the Project. */
    name?: string;
    /** Description of the Project. */
    description?: string;
};

/**
 * Account API.
 *
 * This API allows you to manage projects.
 */
declare class ProjectAPI extends API$x {
    /**
     * Create a new Project for an Organization. Generate a new Project for an
     * Organization, specifying its configuration including name and description.
     *
     * @param request - The request {@link ProjectApiCreateProjectRequest}
     * @returns A Promise of Project
     */
    createProject: (request: Readonly<ProjectApiCreateProjectRequest$1>) => Promise<Project>;
    protected pageOfListProjects: (request?: Readonly<ProjectApiListProjectsRequest$1>) => Promise<ListProjectsResponse>;
    /**
     * List all Projects of an Organization. List all Projects of an Organization.
     * The response will include the total number of Projects as well as their
     * associated Organizations, names, and IDs. Other information includes the
     * creation and update date of the Project.
     *
     * @param request - The request {@link ProjectApiListProjectsRequest}
     * @returns A Promise of ListProjectsResponse
     */
    listProjects: (request?: Readonly<ProjectApiListProjectsRequest$1>) => Promise<ListProjectsResponse> & {
        all: () => Promise<Project[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Project[], void, void>;
    };
    /**
     * Get an existing Project. Retrieve information about an existing Project,
     * specified by its Project ID. Its full details, including ID, name and
     * description, are returned in the response object.
     *
     * @param request - The request {@link ProjectApiGetProjectRequest}
     * @returns A Promise of Project
     */
    getProject: (request?: Readonly<ProjectApiGetProjectRequest>) => Promise<Project>;
    /**
     * Delete an existing Project. Delete an existing Project, specified by its
     * Project ID. The Project needs to be empty (meaning there are no resources
     * left in it) to be deleted effectively. Note that deleting a Project is
     * permanent, and cannot be undone.
     *
     * @param request - The request {@link ProjectApiDeleteProjectRequest}
     */
    deleteProject: (request?: Readonly<ProjectApiDeleteProjectRequest>) => Promise<void>;
    /**
     * Update Project. Update the parameters of an existing Project, specified by
     * its Project ID. These parameters include the name and description.
     *
     * @param request - The request {@link ProjectApiUpdateProjectRequest}
     * @returns A Promise of Project
     */
    updateProject: (request?: Readonly<ProjectApiUpdateProjectRequest$1>) => Promise<Project>;
}

declare const ProjectApiCreateProjectRequest: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const ProjectApiListProjectsRequest: {
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ProjectApiUpdateProjectRequest: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};

declare const validationRules_gen$e_ProjectApiCreateProjectRequest: typeof ProjectApiCreateProjectRequest;
declare const validationRules_gen$e_ProjectApiListProjectsRequest: typeof ProjectApiListProjectsRequest;
declare const validationRules_gen$e_ProjectApiUpdateProjectRequest: typeof ProjectApiUpdateProjectRequest;
declare namespace validationRules_gen$e {
  export { validationRules_gen$e_ProjectApiCreateProjectRequest as ProjectApiCreateProjectRequest, validationRules_gen$e_ProjectApiListProjectsRequest as ProjectApiListProjectsRequest, validationRules_gen$e_ProjectApiUpdateProjectRequest as ProjectApiUpdateProjectRequest };
}

type index_gen$u_ListProjectsRequestOrderBy = ListProjectsRequestOrderBy;
type index_gen$u_ListProjectsResponse = ListProjectsResponse;
type index_gen$u_Project = Project;
type index_gen$u_ProjectAPI = ProjectAPI;
declare const index_gen$u_ProjectAPI: typeof ProjectAPI;
type index_gen$u_ProjectApiDeleteProjectRequest = ProjectApiDeleteProjectRequest;
type index_gen$u_ProjectApiGetProjectRequest = ProjectApiGetProjectRequest;
declare namespace index_gen$u {
  export { type index_gen$u_ListProjectsRequestOrderBy as ListProjectsRequestOrderBy, type index_gen$u_ListProjectsResponse as ListProjectsResponse, type index_gen$u_Project as Project, index_gen$u_ProjectAPI as ProjectAPI, type ProjectApiCreateProjectRequest$1 as ProjectApiCreateProjectRequest, type index_gen$u_ProjectApiDeleteProjectRequest as ProjectApiDeleteProjectRequest, type index_gen$u_ProjectApiGetProjectRequest as ProjectApiGetProjectRequest, type ProjectApiListProjectsRequest$1 as ProjectApiListProjectsRequest, type ProjectApiUpdateProjectRequest$1 as ProjectApiUpdateProjectRequest, validationRules_gen$e as ValidationRules };
}

declare namespace index$y {
  export { index_gen$v as v2, index_gen$u as v3 };
}

type ListServersRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc';
type ServerStatus$1 = 'unknown_status' | 'starting' | 'ready' | 'error' | 'rebooting' | 'updating' | 'locking' | 'locked' | 'unlocking' | 'reinstalling';
type ServerTypeStock = 'unknown_stock' | 'no_stock' | 'low_stock' | 'high_stock';
interface ServerTypeCPU {
    name: string;
    coreCount: number;
}
interface ServerTypeDisk {
    capacity: number;
    type: string;
}
interface ServerTypeMemory {
    capacity: number;
    type: string;
}
interface OS$1 {
    /** Unique ID of the OS. */
    id: string;
    /** OS name. */
    name: string;
    /** OS name as it should be displayed. */
    label: string;
    /** URL of the image. */
    imageUrl: string;
    /** List of compatible server types. */
    compatibleServerTypes: string[];
}
interface ServerType$1 {
    /** CPU description. */
    cpu?: ServerTypeCPU;
    /** Size of the local disk of the server. */
    disk?: ServerTypeDisk;
    /** Name of the type. */
    name: string;
    /** Size of memory available. */
    memory?: ServerTypeMemory;
    /** Current stock. */
    stock: ServerTypeStock;
    /** Minimum duration of the lease in seconds (example. 3.4s). */
    minimumLeaseDuration?: string;
}
interface Server$2 {
    /** UUID of the server. */
    id: string;
    /** Type of the server. */
    type: string;
    /** Name of the server. */
    name: string;
    /** Project this server is associated with. */
    projectId: string;
    /** Organization this server is associated with. */
    organizationId: string;
    /** IPv4 address of the server. */
    ip: string;
    /** URL of the VNC. */
    vncUrl: string;
    /** Current status of the server. */
    status: ServerStatus$1;
    /** Date on which the server was created. */
    createdAt?: Date;
    /** Date on which the server was last updated. */
    updatedAt?: Date;
    /** Date on which the server was last deleted. */
    deletableAt?: Date;
    /** Zone of the server. */
    zone: Zone;
}
type CreateServerRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Create a server with this given name. */
    name?: string;
    /** Create a server in the given project ID. */
    projectId?: string;
    /** Create a server of the given type. */
    type: string;
};
type DeleteServerRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the server you want to delete. */
    serverId: string;
};
type GetOSRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the OS you want to get. */
    osId: string;
};
type GetServerRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the server you want to get. */
    serverId: string;
};
type GetServerTypeRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Server type identifier. */
    serverType: string;
};
type ListOSRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Positive integer to choose the page to return. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
    /** List of compatible server types. */
    serverType?: string;
    /**
     * Filter OS by name (note that "11.1" will return "11.1.2" and "11.1" but not
     * "12")).
     */
    name?: string;
};
interface ListOSResponse$1 {
    /** Total number of OS. */
    totalCount: number;
    /** List of OS. */
    os: OS$1[];
}
type ListServerTypesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
};
interface ListServerTypesResponse {
    /** Available server types. */
    serverTypes: ServerType$1[];
}
type ListServersRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Sort order of the returned servers. */
    orderBy?: ListServersRequestOrderBy$1;
    /** Only list servers of this project ID. */
    projectId?: string;
    /** Only list servers of this Organization ID. */
    organizationId?: string;
    /** Positive integer to choose the page to return. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
};
interface ListServersResponse$2 {
    /** Total number of servers. */
    totalCount: number;
    /** Paginated returned servers. */
    servers: Server$2[];
}
type RebootServerRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the server you want to reboot. */
    serverId: string;
};
type ReinstallServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the server you want to reinstall. */
    serverId: string;
};
type UpdateServerRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the server you want to update. */
    serverId: string;
    /** Updated name for your server. */
    name?: string;
};

/** Apple silicon API. */
declare class API$v extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    /**
     * List server types. List all technical details about Apple silicon server
     * types available in the specified zone. Since there is only one Availability
     * Zone for Apple silicon servers, the targeted value is `fr-par-3`.
     *
     * @param request - The request {@link ListServerTypesRequest}
     * @returns A Promise of ListServerTypesResponse
     */
    listServerTypes: (request?: Readonly<ListServerTypesRequest>) => Promise<ListServerTypesResponse>;
    /**
     * Get a server type. Get technical details (CPU, disk size etc.) of a server
     * type.
     *
     * @param request - The request {@link GetServerTypeRequest}
     * @returns A Promise of ServerType
     */
    getServerType: (request: Readonly<GetServerTypeRequest>) => Promise<ServerType$1>;
    /**
     * Create a server. Create a new server in the targeted zone, specifying its
     * configuration including name and type.
     *
     * @param request - The request {@link CreateServerRequest}
     * @returns A Promise of Server
     */
    createServer: (request: Readonly<CreateServerRequest$3>) => Promise<Server$2>;
    protected pageOfListServers: (request?: Readonly<ListServersRequest$3>) => Promise<ListServersResponse$2>;
    /**
     * List all servers. List all servers in the specified zone. By default,
     * returned servers in the list are ordered by creation date in ascending
     * order, though this can be modified via the `order_by` field.
     *
     * @param request - The request {@link ListServersRequest}
     * @returns A Promise of ListServersResponse
     */
    listServers: (request?: Readonly<ListServersRequest$3>) => Promise<ListServersResponse$2> & {
        all: () => Promise<Server$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Server$2[], void, void>;
    };
    protected pageOfListOS: (request?: Readonly<ListOSRequest$2>) => Promise<ListOSResponse$1>;
    /**
     * List all Operating Systems (OS). List all Operating Systems (OS). The
     * response will include the total number of OS as well as their associated
     * IDs, names and labels.
     *
     * @param request - The request {@link ListOSRequest}
     * @returns A Promise of ListOSResponse
     */
    listOS: (request?: Readonly<ListOSRequest$2>) => Promise<ListOSResponse$1> & {
        all: () => Promise<OS$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<OS$1[], void, void>;
    };
    /**
     * Get an Operating System (OS). Get an Operating System (OS). The response
     * will include the OS's unique ID as well as its name and label.
     *
     * @param request - The request {@link GetOSRequest}
     * @returns A Promise of OS
     */
    getOS: (request: Readonly<GetOSRequest$1>) => Promise<OS$1>;
    /**
     * Get a server. Retrieve information about an existing Apple silicon server,
     * specified by its server ID. Its full details, including name, status and IP
     * address, are returned in the response object.
     *
     * @param request - The request {@link GetServerRequest}
     * @returns A Promise of Server
     */
    getServer: (request: Readonly<GetServerRequest$2>) => Promise<Server$2>;
    /**
     * Waits for {@link Server} to be in a final state.
     *
     * @param request - The request {@link GetServerRequest}
     * @param options - The waiting options
     * @returns A Promise of Server
     */
    waitForServer: (request: Readonly<GetServerRequest$2>, options?: Readonly<WaitForOptions<Server$2>>) => Promise<Server$2>;
    /**
     * Update a server. Update the parameters of an existing Apple silicon server,
     * specified by its server ID.
     *
     * @param request - The request {@link UpdateServerRequest}
     * @returns A Promise of Server
     */
    updateServer: (request: Readonly<UpdateServerRequest$3>) => Promise<Server$2>;
    /**
     * Delete a server. Delete an existing Apple silicon server, specified by its
     * server ID. Deleting a server is permanent, and cannot be undone. Note that
     * the minimum allocation period for Apple silicon-as-a-service is 24 hours,
     * meaning you cannot delete your server prior to that.
     *
     * @param request - The request {@link DeleteServerRequest}
     */
    deleteServer: (request: Readonly<DeleteServerRequest$2>) => Promise<void>;
    /**
     * Reboot a server. Reboot an existing Apple silicon server, specified by its
     * server ID.
     *
     * @param request - The request {@link RebootServerRequest}
     * @returns A Promise of Server
     */
    rebootServer: (request: Readonly<RebootServerRequest$1>) => Promise<Server$2>;
    /**
     * Reinstall a server. Reinstall an existing Apple silicon server (specified
     * by its server ID) from a new image (OS). All the data on the disk is
     * deleted and all configuration is reset to the defailt configuration values
     * of the image (OS).
     *
     * @param request - The request {@link ReinstallServerRequest}
     * @returns A Promise of Server
     */
    reinstallServer: (request: Readonly<ReinstallServerRequest>) => Promise<Server$2>;
}

/** Lists transient statutes of the enum {@link ServerStatus}. */
declare const SERVER_TRANSIENT_STATUSES$2: ServerStatus$1[];

type index_gen$t_GetServerTypeRequest = GetServerTypeRequest;
type index_gen$t_ListServerTypesRequest = ListServerTypesRequest;
type index_gen$t_ListServerTypesResponse = ListServerTypesResponse;
type index_gen$t_ReinstallServerRequest = ReinstallServerRequest;
type index_gen$t_ServerTypeCPU = ServerTypeCPU;
type index_gen$t_ServerTypeDisk = ServerTypeDisk;
type index_gen$t_ServerTypeMemory = ServerTypeMemory;
type index_gen$t_ServerTypeStock = ServerTypeStock;
declare namespace index_gen$t {
  export { API$v as API, type CreateServerRequest$3 as CreateServerRequest, type DeleteServerRequest$2 as DeleteServerRequest, type GetOSRequest$1 as GetOSRequest, type GetServerRequest$2 as GetServerRequest, type index_gen$t_GetServerTypeRequest as GetServerTypeRequest, type ListOSRequest$2 as ListOSRequest, type ListOSResponse$1 as ListOSResponse, type index_gen$t_ListServerTypesRequest as ListServerTypesRequest, type index_gen$t_ListServerTypesResponse as ListServerTypesResponse, type ListServersRequest$3 as ListServersRequest, type ListServersRequestOrderBy$1 as ListServersRequestOrderBy, type ListServersResponse$2 as ListServersResponse, type OS$1 as OS, type RebootServerRequest$1 as RebootServerRequest, type index_gen$t_ReinstallServerRequest as ReinstallServerRequest, SERVER_TRANSIENT_STATUSES$2 as SERVER_TRANSIENT_STATUSES, type Server$2 as Server, type ServerStatus$1 as ServerStatus, type ServerType$1 as ServerType, type index_gen$t_ServerTypeCPU as ServerTypeCPU, type index_gen$t_ServerTypeDisk as ServerTypeDisk, type index_gen$t_ServerTypeMemory as ServerTypeMemory, type index_gen$t_ServerTypeStock as ServerTypeStock, type UpdateServerRequest$3 as UpdateServerRequest };
}

declare namespace index$x {
  export { index_gen$t as v1alpha1 };
}

type IPReverseStatus = 'unknown' | 'pending' | 'active' | 'error';
type IPVersion = 'IPv4' | 'IPv6';
type ListServerEventsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListServerPrivateNetworksRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc';
type ListServersRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListSettingsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type OfferStock = 'empty' | 'low' | 'available';
type OfferSubscriptionPeriod = 'unknown_subscription_period' | 'hourly' | 'monthly';
type ServerBootType = 'unknown_boot_type' | 'normal' | 'rescue';
type ServerInstallStatus = 'unknown' | 'to_install' | 'installing' | 'completed' | 'error';
type ServerOptionOptionStatus = 'option_status_unknown' | 'option_status_enable' | 'option_status_enabling' | 'option_status_disabling' | 'option_status_error';
type ServerPingStatus = 'ping_status_unknown' | 'ping_status_up' | 'ping_status_down';
type ServerPrivateNetworkStatus = 'unknown' | 'attaching' | 'attached' | 'error' | 'detaching' | 'locked';
type ServerStatus = 'unknown' | 'delivering' | 'ready' | 'stopping' | 'stopped' | 'starting' | 'error' | 'deleting' | 'locked' | 'out_of_stock' | 'ordered' | 'resetting';
type SettingType = 'unknown' | 'smtp';
interface OSOSField {
    editable: boolean;
    required: boolean;
    defaultValue?: string;
}
interface CPU {
    /** Name of the CPU. */
    name: string;
    /** Number of CPU cores. */
    coreCount: number;
    /** Number CPU threads. */
    threadCount: number;
    /** Frequency of the CPU in MHz. */
    frequency: number;
    /** Benchmark of the CPU. */
    benchmark: string;
}
interface Disk {
    /** Capacity of the disk in bytes. */
    capacity: number;
    /** Type of the disk. */
    type: string;
}
interface Memory {
    /** Capacity of the memory in bytes. */
    capacity: number;
    /** Type of the memory. */
    type: string;
    /** Frequency of the memory in MHz. */
    frequency: number;
    /** True if the memory is an error-correcting code memory. */
    isEcc: boolean;
}
interface OfferOptionOffer {
    /** ID of the option. */
    id: string;
    /** Name of the option. */
    name: string;
    /**
     * If true the option is enabled and included by default in the offer If false
     * the option is available for the offer but not included by default.
     */
    enabled: boolean;
    /** Period of subscription for the offer. */
    subscriptionPeriod: OfferSubscriptionPeriod;
    /** Price of the option. */
    price?: Money;
    /** Boolean to know if option could be managed. */
    manageable: boolean;
    /** ID of the OS linked to the option. */
    osId?: string;
}
interface PersistentMemory {
    /** Capacity of the memory in bytes. */
    capacity: number;
    /** Type of the memory. */
    type: string;
    /** Frequency of the memory in MHz. */
    frequency: number;
}
interface RaidController {
    model: string;
    raidLevel: string[];
}
interface IP$2 {
    /** ID of the IP. */
    id: string;
    /** Address of the IP. */
    address: string;
    /** Reverse IP value. */
    reverse: string;
    /** Version of IP (v4 or v6). */
    version: IPVersion;
    /** Status of the reverse. */
    reverseStatus: IPReverseStatus;
    /** A message related to the reverse status, e.g. in case of an error. */
    reverseStatusMessage: string;
}
interface ServerInstall {
    /** ID of the OS. */
    osId: string;
    /** Host defined during the server installation. */
    hostname: string;
    /** SSH public key IDs defined during server installation. */
    sshKeyIds: string[];
    /** Status of the server installation. */
    status: ServerInstallStatus;
    /**
     * User defined in the server installation, or the default user if none were
     * specified.
     */
    user: string;
    /**
     * Service user defined in the server installation, or the default user if
     * none were specified.
     */
    serviceUser: string;
    /** Address of the installed service. */
    serviceUrl: string;
}
interface ServerOption {
    /** ID of the option. */
    id: string;
    /** Name of the option. */
    name: string;
    /** Status of the option on this server. */
    status: ServerOptionOptionStatus;
    /** Defines whether the option can be managed (added or removed). */
    manageable: boolean;
    /** Auto expiration date for compatible options. */
    expiresAt?: Date;
}
interface ServerRescueServer {
    /** Rescue user name. */
    user: string;
    /** Rescue password. */
    password: string;
}
interface CreateServerRequestInstall$1 {
    /** ID of the OS to installation on the server. */
    osId: string;
    /** Hostname of the server. */
    hostname: string;
    /** SSH key IDs authorized on the server. */
    sshKeyIds: string[];
    /** User for the installation. */
    user?: string;
    /** Password for the installation. */
    password?: string;
    /** Regular user that runs the service to be installed on the server. */
    serviceUser?: string;
    /** Password used for the service to install. */
    servicePassword?: string;
}
interface OS {
    /** ID of the OS. */
    id: string;
    /** Name of the OS. */
    name: string;
    /** Version of the OS. */
    version: string;
    /** URL of this OS's logo. */
    logoUrl: string;
    /** Object defining the SSH requirements to install the OS. */
    ssh?: OSOSField;
    /** Object defining the username requirements to install the OS. */
    user?: OSOSField;
    /** Object defining the password requirements to install the OS. */
    password?: OSOSField;
    /** Object defining the username requirements to install the service. */
    serviceUser?: OSOSField;
    /** Object defining the password requirements to install the service. */
    servicePassword?: OSOSField;
    /** Defines if the operating system is enabled or not. */
    enabled: boolean;
    /** License required (check server options for pricing details). */
    licenseRequired: boolean;
    /** Defines if a specific Organization is allowed to install this OS type. */
    allowed: boolean;
}
interface Offer$1 {
    /** ID of the offer. */
    id: string;
    /** Name of the offer. */
    name: string;
    /** Stock level. */
    stock: OfferStock;
    /** Public bandwidth available (in bits/s) with the offer. */
    bandwidth: number;
    /** Commercial range of the offer. */
    commercialRange: string;
    /**
     * Price of the offer for the next 60 minutes (a server order at 11h32 will be
     * payed until 12h32).
     */
    pricePerHour?: Money;
    /** Monthly price of the offer, if subscribing on a monthly basis. */
    pricePerMonth?: Money;
    /** Disks specifications of the offer. */
    disks: Disk[];
    /** Defines whether the offer is currently available. */
    enable: boolean;
    /** CPU specifications of the offer. */
    cpus: CPU[];
    /** Memory specifications of the offer. */
    memories: Memory[];
    /** Name of the quota associated to the offer. */
    quotaName: string;
    /** Persistent memory specifications of the offer. */
    persistentMemories: PersistentMemory[];
    /** Raid controller specifications of the offer. */
    raidControllers: RaidController[];
    /** Array of OS images IDs incompatible with the server. */
    incompatibleOsIds: string[];
    /** Period of subscription for the offer. */
    subscriptionPeriod: OfferSubscriptionPeriod;
    /** Operation path of the service. */
    operationPath: string;
    /**
     * One time fee invoiced by Scaleway for the setup and activation of the
     * server.
     */
    fee?: Money;
    /** Available options for customization of the server. */
    options: OfferOptionOffer[];
    /** Private bandwidth available in bits/s with the offer. */
    privateBandwidth: number;
    /** Defines whether the offer's bandwidth is shared or not. */
    sharedBandwidth: boolean;
    /** Array of tags attached to the offer. */
    tags: string[];
}
interface Option {
    /** ID of the option. */
    id: string;
    /** Name of the option. */
    name: string;
    /** Defines whether the option is manageable (could be added or removed). */
    manageable: boolean;
}
interface ServerEvent {
    /** ID of the server to which the action will be applied. */
    id: string;
    /** The action that will be applied to the server. */
    action: string;
    /** Date of last modification of the action. */
    updatedAt?: Date;
    /** Date of creation of the action. */
    createdAt?: Date;
}
interface ServerPrivateNetwork {
    /** The Private Network ID. */
    id: string;
    /** The Private Network Project ID. */
    projectId: string;
    /** The server ID. */
    serverId: string;
    /** The Private Network ID. */
    privateNetworkId: string;
    /** The VLAN ID associated to the Private Network. */
    vlan?: number;
    /** The configuration status of the Private Network. */
    status: ServerPrivateNetworkStatus;
    /** The Private Network creation date. */
    createdAt?: Date;
    /** The date the Private Network was last modified. */
    updatedAt?: Date;
}
interface Server$1 {
    /** ID of the server. */
    id: string;
    /** Organization ID the server is attached to. */
    organizationId: string;
    /** Project ID the server is attached to. */
    projectId: string;
    /** Name of the server. */
    name: string;
    /** Description of the server. */
    description: string;
    /** Last modification date of the server. */
    updatedAt?: Date;
    /** Creation date of the server. */
    createdAt?: Date;
    /** Status of the server. */
    status: ServerStatus;
    /** Offer ID of the server. */
    offerId: string;
    /** Offer name of the server. */
    offerName: string;
    /** Array of custom tags attached to the server. */
    tags: string[];
    /** Array of IPs attached to the server. */
    ips: IP$2[];
    /** Domain of the server. */
    domain: string;
    /** Boot type of the server. */
    bootType: ServerBootType;
    /** Zone in which is the server located. */
    zone: Zone;
    /** Configuration of the installation. */
    install?: ServerInstall;
    /** Status of server ping. */
    pingStatus: ServerPingStatus;
    /** Options enabled on the server. */
    options: ServerOption[];
    /** Configuration of rescue boot. */
    rescueServer?: ServerRescueServer;
}
interface Setting {
    /** ID of the setting. */
    id: string;
    /** Type of the setting. */
    type: SettingType;
    /** ID of the Project ID. */
    projectId: string;
    /** Defines whether the setting is enabled. */
    enabled: boolean;
}
type AddOptionServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server. */
    serverId: string;
    /** ID of the option to add. */
    optionId: string;
    /** Auto expire the option after this date. */
    expiresAt?: Date;
};
interface BMCAccess {
    /** URL to access to the server console. */
    url: string;
    /**
     * The login to use for the BMC (Baseboard Management Controller) access
     * authentification.
     */
    login: string;
    /**
     * The password to use for the BMC (Baseboard Management Controller) access
     * authentification.
     */
    password: string;
    /**
     * The date after which the BMC (Baseboard Management Controller) access will
     * be closed.
     */
    expiresAt?: Date;
}
type CreateServerRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Offer ID of the new server. */
    offerId: string;
    /**
     * @deprecated Organization ID with which the server will be created.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Project ID with which the server will be created.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
    /** Name of the server (≠hostname). */
    name: string;
    /** Description associated with the server, max 255 characters. */
    description: string;
    /** Tags to associate to the server. */
    tags?: string[];
    /**
     * Object describing the configuration details of the OS installation on the
     * server.
     */
    install?: CreateServerRequestInstall$1;
    /** IDs of options to enable on server. */
    optionIds?: string[];
};
type DeleteOptionServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server. */
    serverId: string;
    /** ID of the option to delete. */
    optionId: string;
};
type DeleteServerRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server to delete. */
    serverId: string;
};
type GetBMCAccessRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server. */
    serverId: string;
};
type GetOSRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the OS. */
    osId: string;
};
type GetOfferRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the researched Offer. */
    offerId: string;
};
type GetOptionRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the option. */
    optionId: string;
};
type GetServerMetricsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Server ID to get the metrics. */
    serverId: string;
};
interface GetServerMetricsResponse {
    /** Timeseries object representing pings on the server. */
    pings?: TimeSeries;
}
type GetServerRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server. */
    serverId: string;
};
type InstallServerRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Server ID to install. */
    serverId: string;
    /** ID of the OS to installation on the server. */
    osId: string;
    /** Hostname of the server. */
    hostname: string;
    /** SSH key IDs authorized on the server. */
    sshKeyIds: string[];
    /** User used for the installation. */
    user?: string;
    /** Password used for the installation. */
    password?: string;
    /** User used for the service to install. */
    serviceUser?: string;
    /** Password used for the service to install. */
    servicePassword?: string;
};
type ListOSRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Page number. */
    page?: number;
    /** Number of OS per page. */
    pageSize?: number;
    /** Offer IDs to filter OSes for. */
    offerId?: string;
};
interface ListOSResponse {
    /** Total count of matching OS. */
    totalCount: number;
    /** OS that match filters. */
    os: OS[];
}
type ListOffersRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Page number. */
    page?: number;
    /** Number of offers per page. */
    pageSize?: number;
    /** Subscription period type to filter offers by. */
    subscriptionPeriod?: OfferSubscriptionPeriod;
};
interface ListOffersResponse$1 {
    /** Total count of matching offers. */
    totalCount: number;
    /** Offers that match filters. */
    offers: Offer$1[];
}
type ListOptionsRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Page number. */
    page?: number;
    /** Number of options per page. */
    pageSize?: number;
    /** Offer ID to filter options for. */
    offerId?: string;
    /** Name to filter options for. */
    name?: string;
};
interface ListOptionsResponse {
    /** Total count of matching options. */
    totalCount: number;
    /** Options that match filters. */
    options: Option[];
}
type ListServerEventsRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server events searched. */
    serverId: string;
    /** Page number. */
    page?: number;
    /** Number of server events per page. */
    pageSize?: number;
    /** Order of the server events. */
    orderBy?: ListServerEventsRequestOrderBy;
};
interface ListServerEventsResponse {
    /** Total count of matching events. */
    totalCount: number;
    /** Server events that match filters. */
    events: ServerEvent[];
}
interface ListServerPrivateNetworksResponse {
    serverPrivateNetworks: ServerPrivateNetwork[];
    totalCount: number;
}
type ListServersRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Page number. */
    page?: number;
    /** Number of servers per page. */
    pageSize?: number;
    /** Order of the servers. */
    orderBy?: ListServersRequestOrderBy;
    /** Tags to filter for. */
    tags?: string[];
    /** Status to filter for. */
    status?: string[];
    /** Names to filter for. */
    name?: string;
    /** Organization ID to filter for. */
    organizationId?: string;
    /** Project ID to filter for. */
    projectId?: string;
    /** Option ID to filter for. */
    optionId?: string;
};
interface ListServersResponse$1 {
    /** Total count of matching servers. */
    totalCount: number;
    /** Array of Elastic Metal server objects matching the filters in the request. */
    servers: Server$1[];
}
type ListSettingsRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Page number. */
    page?: number;
    /** Set the maximum list size. */
    pageSize?: number;
    /** Sort order for items in the response. */
    orderBy?: ListSettingsRequestOrderBy;
    /** ID of the Project. */
    projectId?: string;
};
interface ListSettingsResponse {
    /** Total count of matching settings. */
    totalCount: number;
    /** Settings that match filters. */
    settings: Setting[];
}
type PrivateNetworkApiAddServerPrivateNetworkRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** The ID of the server. */
    serverId: string;
    /** The ID of the Private Network. */
    privateNetworkId: string;
};
type PrivateNetworkApiDeleteServerPrivateNetworkRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** The ID of the server. */
    serverId: string;
    /** The ID of the Private Network. */
    privateNetworkId: string;
};
type PrivateNetworkApiListServerPrivateNetworksRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** The sort order for the returned Private Networks. */
    orderBy?: ListServerPrivateNetworksRequestOrderBy;
    /** The page number for the returned Private Networks. */
    page?: number;
    /** The maximum number of Private Networks per page. */
    pageSize?: number;
    /** Filter Private Networks by server ID. */
    serverId?: string;
    /** Filter Private Networks by Private Network ID. */
    privateNetworkId?: string;
    /** Filter Private Networks by Organization ID. */
    organizationId?: string;
    /** Filter Private Networks by Project ID. */
    projectId?: string;
};
type PrivateNetworkApiSetServerPrivateNetworksRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** The ID of the server. */
    serverId: string;
    /** The IDs of the Private Networks. */
    privateNetworkIds: string[];
};
type RebootServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server to reboot. */
    serverId: string;
    /** The type of boot. */
    bootType?: ServerBootType;
};
interface SetServerPrivateNetworksResponse {
    serverPrivateNetworks: ServerPrivateNetwork[];
}
type StartBMCAccessRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server. */
    serverId: string;
    /** The IP authorized to connect to the server. */
    ip: string;
};
type StartServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server to start. */
    serverId: string;
    /** The type of boot. */
    bootType?: ServerBootType;
};
type StopBMCAccessRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server. */
    serverId: string;
};
type StopServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server to stop. */
    serverId: string;
};
type UpdateIPRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server. */
    serverId: string;
    /** ID of the IP to update. */
    ipId: string;
    /** New reverse IP to update, not updated if null. */
    reverse?: string;
};
type UpdateServerRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the server to update. */
    serverId: string;
    /** Name of the server (≠hostname), not updated if null. */
    name?: string;
    /**
     * Description associated with the server, max 255 characters, not updated if
     * null.
     */
    description?: string;
    /** Tags associated with the server, not updated if null. */
    tags?: string[];
};
type UpdateSettingRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the setting. */
    settingId: string;
    /** Defines whether the setting is enabled. */
    enabled?: boolean;
};

/**
 * Elastic Metal API.
 *
 * This API allows to manage your Elastic Metal server.
 */
declare class API$u extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    protected pageOfListServers: (request?: Readonly<ListServersRequest$2>) => Promise<ListServersResponse$1>;
    /**
     * List Elastic Metal servers for an Organization. List Elastic Metal servers
     * for a specific Organization.
     *
     * @param request - The request {@link ListServersRequest}
     * @returns A Promise of ListServersResponse
     */
    listServers: (request?: Readonly<ListServersRequest$2>) => Promise<ListServersResponse$1> & {
        all: () => Promise<Server$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Server$1[], void, void>;
    };
    /**
     * Get a specific Elastic Metal server. Get full details of an existing
     * Elastic Metal server associated with the ID.
     *
     * @param request - The request {@link GetServerRequest}
     * @returns A Promise of Server
     */
    getServer: (request: Readonly<GetServerRequest$1>) => Promise<Server$1>;
    /**
     * Waits for {@link Server} to be in a final state.
     *
     * @param request - The request {@link GetServerRequest}
     * @param options - The waiting options
     * @returns A Promise of Server
     */
    waitForServer: (request: Readonly<GetServerRequest$1>, options?: Readonly<WaitForOptions<Server$1>>) => Promise<Server$1>;
    /**
     * Create an Elastic Metal server. Create a new Elastic Metal server. Once the
     * server is created, proceed with the [installation of an OS](#post-3e949e).
     *
     * @param request - The request {@link CreateServerRequest}
     * @returns A Promise of Server
     */
    createServer: (request: Readonly<CreateServerRequest$2>) => Promise<Server$1>;
    /**
     * Update an Elastic Metal server. Update the server associated with the ID.
     * You can update parameters such as the server's name, tags and description.
     * Any parameters left null in the request body are not updated.
     *
     * @param request - The request {@link UpdateServerRequest}
     * @returns A Promise of Server
     */
    updateServer: (request: Readonly<UpdateServerRequest$2>) => Promise<Server$1>;
    /**
     * Install an Elastic Metal server. Install an Operating System (OS) on the
     * Elastic Metal server with a specific ID.
     *
     * @param request - The request {@link InstallServerRequest}
     * @returns A Promise of Server
     */
    installServer: (request: Readonly<InstallServerRequest$1>) => Promise<Server$1>;
    /**
     * Return server metrics. Get the ping status of the server associated with
     * the ID.
     *
     * @param request - The request {@link GetServerMetricsRequest}
     * @returns A Promise of GetServerMetricsResponse
     */
    getServerMetrics: (request: Readonly<GetServerMetricsRequest>) => Promise<GetServerMetricsResponse>;
    /**
     * Delete an Elastic Metal server. Delete the server associated with the ID.
     *
     * @param request - The request {@link DeleteServerRequest}
     * @returns A Promise of Server
     */
    deleteServer: (request: Readonly<DeleteServerRequest$1>) => Promise<Server$1>;
    /**
     * Reboot an Elastic Metal server. Reboot the Elastic Metal server associated
     * with the ID, use the `boot_type` `rescue` to reboot the server in rescue
     * mode.
     *
     * @param request - The request {@link RebootServerRequest}
     * @returns A Promise of Server
     */
    rebootServer: (request: Readonly<RebootServerRequest>) => Promise<Server$1>;
    /**
     * Start an Elastic Metal server. Start the server associated with the ID.
     *
     * @param request - The request {@link StartServerRequest}
     * @returns A Promise of Server
     */
    startServer: (request: Readonly<StartServerRequest>) => Promise<Server$1>;
    /**
     * Stop an Elastic Metal server. Stop the server associated with the ID. The
     * server remains allocated to your account and all data remains on the local
     * storage of the server.
     *
     * @param request - The request {@link StopServerRequest}
     * @returns A Promise of Server
     */
    stopServer: (request: Readonly<StopServerRequest>) => Promise<Server$1>;
    protected pageOfListServerEvents: (request: Readonly<ListServerEventsRequest$1>) => Promise<ListServerEventsResponse>;
    /**
     * List server events. List event (i.e. start/stop/reboot) associated to the
     * server ID.
     *
     * @param request - The request {@link ListServerEventsRequest}
     * @returns A Promise of ListServerEventsResponse
     */
    listServerEvents: (request: Readonly<ListServerEventsRequest$1>) => Promise<ListServerEventsResponse> & {
        all: () => Promise<ServerEvent[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ServerEvent[], void, void>;
    };
    /**
     * Start BMC access. Start BMC (Baseboard Management Controller) access
     * associated with the ID. The BMC (Baseboard Management Controller) access is
     * available one hour after the installation of the server. You need first to
     * create an option Remote Access. You will find the ID and the price with a
     * call to listOffers
     * (https://developers.scaleway.com/en/products/baremetal/api/#get-78db92).
     * Then add the option
     * https://developers.scaleway.com/en/products/baremetal/api/#post-b14abd.
     * After adding the BMC option, you need to Get Remote Access to get the
     * login/password
     * https://developers.scaleway.com/en/products/baremetal/api/#get-cefc0f. Do
     * not forget to delete the Option after use.
     *
     * @param request - The request {@link StartBMCAccessRequest}
     * @returns A Promise of BMCAccess
     */
    startBMCAccess: (request: Readonly<StartBMCAccessRequest>) => Promise<BMCAccess>;
    /**
     * Get BMC access. Get the BMC (Baseboard Management Controller) access
     * associated with the ID, including the URL and login information needed to
     * connect.
     *
     * @param request - The request {@link GetBMCAccessRequest}
     * @returns A Promise of BMCAccess
     */
    getBMCAccess: (request: Readonly<GetBMCAccessRequest>) => Promise<BMCAccess>;
    /**
     * Stop BMC access. Stop BMC (Baseboard Management Controller) access
     * associated with the ID.
     *
     * @param request - The request {@link StopBMCAccessRequest}
     */
    stopBMCAccess: (request: Readonly<StopBMCAccessRequest>) => Promise<void>;
    /**
     * Update IP. Configure the IP address associated with the server ID and IP
     * ID. You can use this method to set a reverse DNS for an IP address.
     *
     * @param request - The request {@link UpdateIPRequest}
     * @returns A Promise of IP
     */
    updateIP: (request: Readonly<UpdateIPRequest$2>) => Promise<IP$2>;
    /**
     * Add server option. Add an option, such as Private Networks, to a specific
     * server.
     *
     * @param request - The request {@link AddOptionServerRequest}
     * @returns A Promise of Server
     */
    addOptionServer: (request: Readonly<AddOptionServerRequest>) => Promise<Server$1>;
    /**
     * Delete server option. Delete an option from a specific server.
     *
     * @param request - The request {@link DeleteOptionServerRequest}
     * @returns A Promise of Server
     */
    deleteOptionServer: (request: Readonly<DeleteOptionServerRequest>) => Promise<Server$1>;
    protected pageOfListOffers: (request?: Readonly<ListOffersRequest$2>) => Promise<ListOffersResponse$1>;
    /**
     * List offers. List all available Elastic Metal server configurations.
     *
     * @param request - The request {@link ListOffersRequest}
     * @returns A Promise of ListOffersResponse
     */
    listOffers: (request?: Readonly<ListOffersRequest$2>) => Promise<ListOffersResponse$1> & {
        all: () => Promise<Offer$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Offer$1[], void, void>;
    };
    /**
     * Get offer. Get details of an offer identified by its offer ID.
     *
     * @param request - The request {@link GetOfferRequest}
     * @returns A Promise of Offer
     */
    getOffer: (request: Readonly<GetOfferRequest>) => Promise<Offer$1>;
    /**
     * Get option. Return specific option for the ID.
     *
     * @param request - The request {@link GetOptionRequest}
     * @returns A Promise of Option
     */
    getOption: (request: Readonly<GetOptionRequest>) => Promise<Option>;
    protected pageOfListOptions: (request?: Readonly<ListOptionsRequest$1>) => Promise<ListOptionsResponse>;
    /**
     * List options. List all options matching with filters.
     *
     * @param request - The request {@link ListOptionsRequest}
     * @returns A Promise of ListOptionsResponse
     */
    listOptions: (request?: Readonly<ListOptionsRequest$1>) => Promise<ListOptionsResponse> & {
        all: () => Promise<Option[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Option[], void, void>;
    };
    protected pageOfListSettings: (request?: Readonly<ListSettingsRequest$1>) => Promise<ListSettingsResponse>;
    /**
     * List all settings. Return all settings for a Project ID.
     *
     * @param request - The request {@link ListSettingsRequest}
     * @returns A Promise of ListSettingsResponse
     */
    listSettings: (request?: Readonly<ListSettingsRequest$1>) => Promise<ListSettingsResponse> & {
        all: () => Promise<Setting[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Setting[], void, void>;
    };
    /**
     * Update setting. Update a setting for a Project ID (enable or disable).
     *
     * @param request - The request {@link UpdateSettingRequest}
     * @returns A Promise of Setting
     */
    updateSetting: (request: Readonly<UpdateSettingRequest>) => Promise<Setting>;
    protected pageOfListOS: (request?: Readonly<ListOSRequest$1>) => Promise<ListOSResponse>;
    /**
     * List available OSes. List all OSes that are available for installation on
     * Elastic Metal servers.
     *
     * @param request - The request {@link ListOSRequest}
     * @returns A Promise of ListOSResponse
     */
    listOS: (request?: Readonly<ListOSRequest$1>) => Promise<ListOSResponse> & {
        all: () => Promise<OS[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<OS[], void, void>;
    };
    /**
     * Get OS with an ID. Return the specific OS for the ID.
     *
     * @param request - The request {@link GetOSRequest}
     * @returns A Promise of OS
     */
    getOS: (request: Readonly<GetOSRequest>) => Promise<OS>;
}
/** Elastic Metal Private Network API. */
declare class PrivateNetworkAPI extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    /**
     * Add a server to a Private Network.
     *
     * @param request - The request
     *   {@link PrivateNetworkApiAddServerPrivateNetworkRequest}
     * @returns A Promise of ServerPrivateNetwork
     */
    addServerPrivateNetwork: (request: Readonly<PrivateNetworkApiAddServerPrivateNetworkRequest>) => Promise<ServerPrivateNetwork>;
    /**
     * Set multiple Private Networks on a server.
     *
     * @param request - The request
     *   {@link PrivateNetworkApiSetServerPrivateNetworksRequest}
     * @returns A Promise of SetServerPrivateNetworksResponse
     */
    setServerPrivateNetworks: (request: Readonly<PrivateNetworkApiSetServerPrivateNetworksRequest>) => Promise<SetServerPrivateNetworksResponse>;
    protected pageOfListServerPrivateNetworks: (request?: Readonly<PrivateNetworkApiListServerPrivateNetworksRequest>) => Promise<ListServerPrivateNetworksResponse>;
    /**
     * List the Private Networks of a server.
     *
     * @param request - The request
     *   {@link PrivateNetworkApiListServerPrivateNetworksRequest}
     * @returns A Promise of ListServerPrivateNetworksResponse
     */
    listServerPrivateNetworks: (request?: Readonly<PrivateNetworkApiListServerPrivateNetworksRequest>) => Promise<ListServerPrivateNetworksResponse> & {
        all: () => Promise<ServerPrivateNetwork[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ServerPrivateNetwork[], void, void>;
    };
    /**
     * Delete a Private Network.
     *
     * @param request - The request
     *   {@link PrivateNetworkApiDeleteServerPrivateNetworkRequest}
     */
    deleteServerPrivateNetwork: (request: Readonly<PrivateNetworkApiDeleteServerPrivateNetworkRequest>) => Promise<void>;
}

declare class BaremetalV1UtilsAPI extends API$u {
    /**
     * Waits for {@link ServerInstall} to be in a final state.
     *
     * @param request - The request {@link GetServerRequest}
     * @param options - The waiting options
     * @returns A Promise of ServerInstall
     */
    waitForServerInstall: (request: Readonly<GetServerRequest$1>, options?: Readonly<WaitForOptions<ServerInstall>>) => Promise<ServerInstall>;
}

/** Lists transient statutes of the enum {@link ServerInstallStatus}. */
declare const SERVER_INSTALL_TRANSIENT_STATUSES: ServerInstallStatus[];
/** Lists transient statutes of the enum {@link ServerPrivateNetworkStatus}. */
declare const SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES: ServerPrivateNetworkStatus[];
/** Lists transient statutes of the enum {@link ServerStatus}. */
declare const SERVER_TRANSIENT_STATUSES$1: ServerStatus[];

declare const CreateServerRequest$1: {
    description: {
        maxLength: number;
    };
    name: {
        minLength: number;
    };
};
declare const CreateServerRequestInstall: {
    hostname: {
        maxLength: number;
    };
    password: {
        maxLength: number;
    };
    servicePassword: {
        maxLength: number;
    };
    serviceUser: {
        maxLength: number;
    };
    user: {
        maxLength: number;
    };
};
declare const InstallServerRequest: {
    hostname: {
        maxLength: number;
    };
    password: {
        maxLength: number;
    };
    servicePassword: {
        maxLength: number;
    };
    serviceUser: {
        maxLength: number;
    };
    user: {
        maxLength: number;
    };
};
declare const ListOSRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListOffersRequest$1: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListOptionsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListServerEventsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListServersRequest$1: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListSettingsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const UpdateServerRequest$1: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
    };
};

declare const validationRules_gen$d_CreateServerRequestInstall: typeof CreateServerRequestInstall;
declare const validationRules_gen$d_InstallServerRequest: typeof InstallServerRequest;
declare const validationRules_gen$d_ListOSRequest: typeof ListOSRequest;
declare const validationRules_gen$d_ListOptionsRequest: typeof ListOptionsRequest;
declare const validationRules_gen$d_ListServerEventsRequest: typeof ListServerEventsRequest;
declare const validationRules_gen$d_ListSettingsRequest: typeof ListSettingsRequest;
declare namespace validationRules_gen$d {
  export { CreateServerRequest$1 as CreateServerRequest, validationRules_gen$d_CreateServerRequestInstall as CreateServerRequestInstall, validationRules_gen$d_InstallServerRequest as InstallServerRequest, validationRules_gen$d_ListOSRequest as ListOSRequest, ListOffersRequest$1 as ListOffersRequest, validationRules_gen$d_ListOptionsRequest as ListOptionsRequest, validationRules_gen$d_ListServerEventsRequest as ListServerEventsRequest, ListServersRequest$1 as ListServersRequest, validationRules_gen$d_ListSettingsRequest as ListSettingsRequest, UpdateServerRequest$1 as UpdateServerRequest };
}

type index$w_AddOptionServerRequest = AddOptionServerRequest;
type index$w_BMCAccess = BMCAccess;
type index$w_CPU = CPU;
type index$w_DeleteOptionServerRequest = DeleteOptionServerRequest;
type index$w_Disk = Disk;
type index$w_GetBMCAccessRequest = GetBMCAccessRequest;
type index$w_GetOSRequest = GetOSRequest;
type index$w_GetOfferRequest = GetOfferRequest;
type index$w_GetOptionRequest = GetOptionRequest;
type index$w_GetServerMetricsRequest = GetServerMetricsRequest;
type index$w_GetServerMetricsResponse = GetServerMetricsResponse;
type index$w_IPReverseStatus = IPReverseStatus;
type index$w_IPVersion = IPVersion;
type index$w_ListOSResponse = ListOSResponse;
type index$w_ListOptionsResponse = ListOptionsResponse;
type index$w_ListServerEventsRequestOrderBy = ListServerEventsRequestOrderBy;
type index$w_ListServerEventsResponse = ListServerEventsResponse;
type index$w_ListServerPrivateNetworksRequestOrderBy = ListServerPrivateNetworksRequestOrderBy;
type index$w_ListServerPrivateNetworksResponse = ListServerPrivateNetworksResponse;
type index$w_ListServersRequestOrderBy = ListServersRequestOrderBy;
type index$w_ListSettingsRequestOrderBy = ListSettingsRequestOrderBy;
type index$w_ListSettingsResponse = ListSettingsResponse;
type index$w_Memory = Memory;
type index$w_OS = OS;
type index$w_OSOSField = OSOSField;
type index$w_OfferOptionOffer = OfferOptionOffer;
type index$w_OfferStock = OfferStock;
type index$w_OfferSubscriptionPeriod = OfferSubscriptionPeriod;
type index$w_Option = Option;
type index$w_PersistentMemory = PersistentMemory;
type index$w_PrivateNetworkAPI = PrivateNetworkAPI;
declare const index$w_PrivateNetworkAPI: typeof PrivateNetworkAPI;
type index$w_PrivateNetworkApiAddServerPrivateNetworkRequest = PrivateNetworkApiAddServerPrivateNetworkRequest;
type index$w_PrivateNetworkApiDeleteServerPrivateNetworkRequest = PrivateNetworkApiDeleteServerPrivateNetworkRequest;
type index$w_PrivateNetworkApiListServerPrivateNetworksRequest = PrivateNetworkApiListServerPrivateNetworksRequest;
type index$w_PrivateNetworkApiSetServerPrivateNetworksRequest = PrivateNetworkApiSetServerPrivateNetworksRequest;
type index$w_RaidController = RaidController;
type index$w_RebootServerRequest = RebootServerRequest;
declare const index$w_SERVER_INSTALL_TRANSIENT_STATUSES: typeof SERVER_INSTALL_TRANSIENT_STATUSES;
declare const index$w_SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES: typeof SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES;
type index$w_ServerBootType = ServerBootType;
type index$w_ServerEvent = ServerEvent;
type index$w_ServerInstall = ServerInstall;
type index$w_ServerInstallStatus = ServerInstallStatus;
type index$w_ServerOption = ServerOption;
type index$w_ServerOptionOptionStatus = ServerOptionOptionStatus;
type index$w_ServerPingStatus = ServerPingStatus;
type index$w_ServerPrivateNetwork = ServerPrivateNetwork;
type index$w_ServerPrivateNetworkStatus = ServerPrivateNetworkStatus;
type index$w_ServerRescueServer = ServerRescueServer;
type index$w_ServerStatus = ServerStatus;
type index$w_SetServerPrivateNetworksResponse = SetServerPrivateNetworksResponse;
type index$w_Setting = Setting;
type index$w_SettingType = SettingType;
type index$w_StartBMCAccessRequest = StartBMCAccessRequest;
type index$w_StartServerRequest = StartServerRequest;
type index$w_StopBMCAccessRequest = StopBMCAccessRequest;
type index$w_StopServerRequest = StopServerRequest;
type index$w_UpdateSettingRequest = UpdateSettingRequest;
declare namespace index$w {
  export { BaremetalV1UtilsAPI as API, type index$w_AddOptionServerRequest as AddOptionServerRequest, type index$w_BMCAccess as BMCAccess, type index$w_CPU as CPU, type CreateServerRequest$2 as CreateServerRequest, type CreateServerRequestInstall$1 as CreateServerRequestInstall, type index$w_DeleteOptionServerRequest as DeleteOptionServerRequest, type DeleteServerRequest$1 as DeleteServerRequest, type index$w_Disk as Disk, type index$w_GetBMCAccessRequest as GetBMCAccessRequest, type index$w_GetOSRequest as GetOSRequest, type index$w_GetOfferRequest as GetOfferRequest, type index$w_GetOptionRequest as GetOptionRequest, type index$w_GetServerMetricsRequest as GetServerMetricsRequest, type index$w_GetServerMetricsResponse as GetServerMetricsResponse, type GetServerRequest$1 as GetServerRequest, type IP$2 as IP, type index$w_IPReverseStatus as IPReverseStatus, type index$w_IPVersion as IPVersion, type InstallServerRequest$1 as InstallServerRequest, type ListOSRequest$1 as ListOSRequest, type index$w_ListOSResponse as ListOSResponse, type ListOffersRequest$2 as ListOffersRequest, type ListOffersResponse$1 as ListOffersResponse, type ListOptionsRequest$1 as ListOptionsRequest, type index$w_ListOptionsResponse as ListOptionsResponse, type ListServerEventsRequest$1 as ListServerEventsRequest, type index$w_ListServerEventsRequestOrderBy as ListServerEventsRequestOrderBy, type index$w_ListServerEventsResponse as ListServerEventsResponse, type index$w_ListServerPrivateNetworksRequestOrderBy as ListServerPrivateNetworksRequestOrderBy, type index$w_ListServerPrivateNetworksResponse as ListServerPrivateNetworksResponse, type ListServersRequest$2 as ListServersRequest, type index$w_ListServersRequestOrderBy as ListServersRequestOrderBy, type ListServersResponse$1 as ListServersResponse, type ListSettingsRequest$1 as ListSettingsRequest, type index$w_ListSettingsRequestOrderBy as ListSettingsRequestOrderBy, type index$w_ListSettingsResponse as ListSettingsResponse, type index$w_Memory as Memory, type index$w_OS as OS, type index$w_OSOSField as OSOSField, type Offer$1 as Offer, type index$w_OfferOptionOffer as OfferOptionOffer, type index$w_OfferStock as OfferStock, type index$w_OfferSubscriptionPeriod as OfferSubscriptionPeriod, type index$w_Option as Option, type index$w_PersistentMemory as PersistentMemory, index$w_PrivateNetworkAPI as PrivateNetworkAPI, type index$w_PrivateNetworkApiAddServerPrivateNetworkRequest as PrivateNetworkApiAddServerPrivateNetworkRequest, type index$w_PrivateNetworkApiDeleteServerPrivateNetworkRequest as PrivateNetworkApiDeleteServerPrivateNetworkRequest, type index$w_PrivateNetworkApiListServerPrivateNetworksRequest as PrivateNetworkApiListServerPrivateNetworksRequest, type index$w_PrivateNetworkApiSetServerPrivateNetworksRequest as PrivateNetworkApiSetServerPrivateNetworksRequest, type index$w_RaidController as RaidController, type index$w_RebootServerRequest as RebootServerRequest, index$w_SERVER_INSTALL_TRANSIENT_STATUSES as SERVER_INSTALL_TRANSIENT_STATUSES, index$w_SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES as SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES, SERVER_TRANSIENT_STATUSES$1 as SERVER_TRANSIENT_STATUSES, type Server$1 as Server, type index$w_ServerBootType as ServerBootType, type index$w_ServerEvent as ServerEvent, type index$w_ServerInstall as ServerInstall, type index$w_ServerInstallStatus as ServerInstallStatus, type index$w_ServerOption as ServerOption, type index$w_ServerOptionOptionStatus as ServerOptionOptionStatus, type index$w_ServerPingStatus as ServerPingStatus, type index$w_ServerPrivateNetwork as ServerPrivateNetwork, type index$w_ServerPrivateNetworkStatus as ServerPrivateNetworkStatus, type index$w_ServerRescueServer as ServerRescueServer, type index$w_ServerStatus as ServerStatus, type index$w_SetServerPrivateNetworksResponse as SetServerPrivateNetworksResponse, type index$w_Setting as Setting, type index$w_SettingType as SettingType, type index$w_StartBMCAccessRequest as StartBMCAccessRequest, type index$w_StartServerRequest as StartServerRequest, type index$w_StopBMCAccessRequest as StopBMCAccessRequest, type index$w_StopServerRequest as StopServerRequest, type UpdateIPRequest$2 as UpdateIPRequest, type UpdateServerRequest$2 as UpdateServerRequest, type index$w_UpdateSettingRequest as UpdateSettingRequest, validationRules_gen$d as ValidationRules };
}

declare namespace index$v {
  export { index$w as v1 };
}

type DiscountDiscountMode$1 = 'unknown_discount_mode' | 'discount_mode_rate' | 'discount_mode_value' | 'discount_mode_splittable';
type DiscountFilterType$1 = 'unknown_type' | 'product_category' | 'product' | 'product_offer' | 'product_reference' | 'region' | 'zone';
type DownloadInvoiceRequestFileType$1 = 'pdf';
type InvoiceType$1 = 'unknown_type' | 'periodic' | 'purchase';
type ListDiscountsRequestOrderBy$1 = 'creation_date_desc' | 'creation_date_asc';
type ListInvoicesRequestOrderBy$1 = 'invoice_number_desc' | 'invoice_number_asc' | 'start_date_desc' | 'start_date_asc' | 'issued_date_desc' | 'issued_date_asc' | 'due_date_desc' | 'due_date_asc' | 'total_untaxed_desc' | 'total_untaxed_asc' | 'total_taxed_desc' | 'total_taxed_asc' | 'invoice_type_desc' | 'invoice_type_asc';
interface DiscountCoupon$1 {
    /** The description of the coupon. */
    description?: string;
}
interface DiscountFilter$1 {
    /** Type of the filter. */
    type: DiscountFilterType$1;
    /** Value of filter, it can be a product/range/region/zone value. */
    value: string;
}
interface GetConsumptionResponseConsumption {
    /** Monetary value of the consumption. */
    value?: Money;
    /** Description of the consumption. */
    description: string;
    /** Project ID of the consumption. */
    projectId: string;
    /** Category of the consumption. */
    category: string;
    /** Unique identifier of the product. */
    operationPath: string;
}
interface Discount$1 {
    /** The ID of the discount. */
    id: string;
    /** The creation date of the discount. */
    creationDate?: Date;
    /** The organization ID of the discount. */
    organizationId: string;
    /** The description of the discount. */
    description: string;
    /** The initial value of the discount. */
    value: number;
    /** The value indicating how much of the discount has been used. */
    valueUsed: number;
    /** The remaining value of the discount. */
    valueRemaining: number;
    /** The mode of the discount. */
    mode: DiscountDiscountMode$1;
    /** The start date of the discount. */
    startDate?: Date;
    /** The stop date of the discount. */
    stopDate?: Date;
    /** The description of the coupon. */
    coupon?: DiscountCoupon$1;
    /** List of products/ranges/regions/zones to limit the usability of discounts. */
    filters: DiscountFilter$1[];
}
interface Invoice$1 {
    /** Invoice ID. */
    id: string;
    /** Start date of the billing period. */
    startDate?: Date;
    /** Date when the invoice was sent to the customer. */
    issuedDate?: Date;
    /** Payment time limit, set according to the Organization's payment conditions. */
    dueDate?: Date;
    /** Total amount, untaxed. */
    totalUntaxed?: Money;
    /** Total amount, taxed. */
    totalTaxed?: Money;
    /** Type of invoice. */
    invoiceType: InvoiceType$1;
    /** Invoice number. */
    number: number;
}
type DownloadInvoiceRequest$1 = {
    /** Invoice ID. */
    invoiceId: string;
    /** Wanted file type. */
    fileType?: DownloadInvoiceRequestFileType$1;
};
type GetConsumptionRequest = {
    /** Filter by organization ID. */
    organizationId?: string;
};
interface GetConsumptionResponse {
    /** Detailed consumption list. */
    consumptions: GetConsumptionResponseConsumption[];
    /** Last consumption update date. */
    updatedAt?: Date;
}
type ListDiscountsRequest$1 = {
    /** Order discounts in the response by their description. */
    orderBy?: ListDiscountsRequestOrderBy$1;
    /** Positive integer to choose the page to return. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
    /** ID of the organization. */
    organizationId?: string;
};
interface ListDiscountsResponse$1 {
    /** Total number of discounts. */
    totalCount: number;
    /** Paginated returned discounts. */
    discounts: Discount$1[];
}
type ListInvoicesRequest$1 = {
    /**
     * Organization ID to filter for, only invoices from this Organization will be
     * returned.
     */
    organizationId?: string;
    /** Invoice's `start_date` is greater or equal to `started_after`. */
    startedAfter?: Date;
    /** Invoice's `start_date` precedes `started_before`. */
    startedBefore?: Date;
    /** Invoice type. It can either be `periodic` or `purchase`. */
    invoiceType?: InvoiceType$1;
    /** Positive integer to choose the page to return. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
    /** How invoices are ordered in the response. */
    orderBy?: ListInvoicesRequestOrderBy$1;
};
interface ListInvoicesResponse$1 {
    /** Total number of invoices. */
    totalCount: number;
    /** Paginated returned invoices. */
    invoices: Invoice$1[];
}

/**
 * Billing API.
 *
 * This API allows you to query your consumption.
 */
declare class API$t extends API$x {
    /**
     * Get current month's consumption. The consumption reflects the amount of
     * money you have spent for the products you have used. The consumption value
     * is monetary and is not computed in real time.
     *
     * @param request - The request {@link GetConsumptionRequest}
     * @returns A Promise of GetConsumptionResponse
     */
    getConsumption: (request?: Readonly<GetConsumptionRequest>) => Promise<GetConsumptionResponse>;
    protected pageOfListInvoices: (request?: Readonly<ListInvoicesRequest$1>) => Promise<ListInvoicesResponse$1>;
    /**
     * List invoices. List all your invoices, filtering by `start_date` and
     * `invoice_type`. Each invoice has its own ID.
     *
     * @param request - The request {@link ListInvoicesRequest}
     * @returns A Promise of ListInvoicesResponse
     */
    listInvoices: (request?: Readonly<ListInvoicesRequest$1>) => Promise<ListInvoicesResponse$1> & {
        all: () => Promise<Invoice$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Invoice$1[], void, void>;
    };
    /**
     * Download an invoice. Download a specific invoice, specified by its ID.
     *
     * @param request - The request {@link DownloadInvoiceRequest}
     * @returns A Promise of Blob
     */
    downloadInvoice: (request: Readonly<DownloadInvoiceRequest$1>) => Promise<Blob>;
    protected pageOfListDiscounts: (request?: Readonly<ListDiscountsRequest$1>) => Promise<ListDiscountsResponse$1>;
    /**
     * List all user's discounts. List all discounts for an organization and
     * usable categories/products/offers/references/regions/zones where the
     * discount can be applied.
     *
     * @param request - The request {@link ListDiscountsRequest}
     * @returns A Promise of ListDiscountsResponse
     */
    listDiscounts: (request?: Readonly<ListDiscountsRequest$1>) => Promise<ListDiscountsResponse$1> & {
        all: () => Promise<Discount$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Discount$1[], void, void>;
    };
}

type index_gen$s_GetConsumptionRequest = GetConsumptionRequest;
type index_gen$s_GetConsumptionResponse = GetConsumptionResponse;
type index_gen$s_GetConsumptionResponseConsumption = GetConsumptionResponseConsumption;
declare namespace index_gen$s {
  export { API$t as API, type Discount$1 as Discount, type DiscountCoupon$1 as DiscountCoupon, type DiscountDiscountMode$1 as DiscountDiscountMode, type DiscountFilter$1 as DiscountFilter, type DiscountFilterType$1 as DiscountFilterType, type DownloadInvoiceRequest$1 as DownloadInvoiceRequest, type DownloadInvoiceRequestFileType$1 as DownloadInvoiceRequestFileType, type index_gen$s_GetConsumptionRequest as GetConsumptionRequest, type index_gen$s_GetConsumptionResponse as GetConsumptionResponse, type index_gen$s_GetConsumptionResponseConsumption as GetConsumptionResponseConsumption, type Invoice$1 as Invoice, type InvoiceType$1 as InvoiceType, type ListDiscountsRequest$1 as ListDiscountsRequest, type ListDiscountsRequestOrderBy$1 as ListDiscountsRequestOrderBy, type ListDiscountsResponse$1 as ListDiscountsResponse, type ListInvoicesRequest$1 as ListInvoicesRequest, type ListInvoicesRequestOrderBy$1 as ListInvoicesRequestOrderBy, type ListInvoicesResponse$1 as ListInvoicesResponse };
}

type DiscountDiscountMode = 'unknown_discount_mode' | 'discount_mode_rate' | 'discount_mode_value' | 'discount_mode_splittable';
type DiscountFilterType = 'unknown_type' | 'category_name' | 'product_name' | 'product_range' | 'resource_name' | 'region' | 'zone';
type DownloadInvoiceRequestFileType = 'pdf';
type ExportInvoicesRequestFileType = 'csv';
type ExportInvoicesRequestOrderBy = 'invoice_number_desc' | 'invoice_number_asc' | 'start_date_desc' | 'start_date_asc' | 'issued_date_desc' | 'issued_date_asc' | 'due_date_desc' | 'due_date_asc' | 'total_untaxed_desc' | 'total_untaxed_asc' | 'total_taxed_desc' | 'total_taxed_asc' | 'invoice_type_desc' | 'invoice_type_asc';
type InvoiceType = 'unknown_type' | 'periodic' | 'purchase';
type ListConsumptionsRequestOrderBy = 'updated_at_desc' | 'updated_at_asc' | 'category_name_desc' | 'category_name_asc';
type ListDiscountsRequestOrderBy = 'creation_date_desc' | 'creation_date_asc';
type ListInvoicesRequestOrderBy = 'invoice_number_desc' | 'invoice_number_asc' | 'start_date_desc' | 'start_date_asc' | 'issued_date_desc' | 'issued_date_asc' | 'due_date_desc' | 'due_date_asc' | 'total_untaxed_desc' | 'total_untaxed_asc' | 'total_taxed_desc' | 'total_taxed_asc' | 'invoice_type_desc' | 'invoice_type_asc';
type ListTaxesRequestOrderBy = 'updated_at_desc' | 'updated_at_asc' | 'category_name_desc' | 'category_name_asc';
interface DiscountCoupon {
    /** The description of the coupon. */
    description?: string;
}
interface DiscountFilter {
    /**
     * Type of the filter (category name, product name, product range, resource
     * name, region or zone).
     */
    type: DiscountFilterType;
    /** Value of filter. */
    value: string;
}
interface ListConsumptionsResponseConsumption {
    /** Monetary value of the consumption. */
    value?: Money;
    /**
     * The product name. For example, "VPC Public Gateway S", "VPC Public Gateway
     * M" for the VPC product.
     */
    productName: string;
    /** Identifies the reference based on the category. */
    resourceName: string;
    /** Unique identifier of the product. */
    sku: string;
    /** Project ID of the consumption. */
    projectId: string;
    /** Name of consumption category. */
    categoryName: string;
    /** Unit of consumed quantity. */
    unit: string;
    /** Consumed quantity. */
    billedQuantity: string;
}
interface Discount {
    /** The ID of the discount. */
    id: string;
    /** The creation date of the discount. */
    creationDate?: Date;
    /** The organization ID of the discount. */
    organizationId: string;
    /** The description of the discount. */
    description: string;
    /** The initial value of the discount. */
    value: number;
    /** The value indicating how much of the discount has been used. */
    valueUsed: number;
    /** The remaining value of the discount. */
    valueRemaining: number;
    /** The mode of the discount. */
    mode: DiscountDiscountMode;
    /** The start date of the discount. */
    startDate?: Date;
    /** The stop date of the discount. */
    stopDate?: Date;
    /** The description of the coupon. */
    coupon?: DiscountCoupon;
    /** List of the discount scopes. */
    filters: DiscountFilter[];
}
interface Invoice {
    /** Invoice ID. */
    id: string;
    organizationId: string;
    organizationName: string;
    /** Start date of the billing period. */
    startDate?: Date;
    stopDate?: Date;
    /** The billing period of the invoice in the YYYY-MM format. */
    billingPeriod?: Date;
    /** Date when the invoice was sent to the customer. */
    issuedDate?: Date;
    /** Payment time limit, set according to the Organization's payment conditions. */
    dueDate?: Date;
    /** Total amount, untaxed. */
    totalUntaxed?: Money;
    /** Total amount, taxed. */
    totalTaxed?: Money;
    /** The total tax amount of the invoice. */
    totalTax?: Money;
    /** The total discount amount of the invoice. */
    totalDiscount?: Money;
    /** The total amount of the invoice before applying the discount. */
    totalUndiscount?: Money;
    /** Type of invoice, either periodic or purchase. */
    type: InvoiceType;
    /** The state of the Invoice. */
    state: string;
    /** Invoice number. */
    number: number;
    /** The name of the seller (Scaleway). */
    sellerName: string;
}
interface ListTaxesResponseTax {
    /** Description of the tax applied. */
    description: string;
    /** The three-letter currency code. */
    currency: string;
    /** Applied tax rate (0.2 means a VAT of 20%). */
    rate?: number;
    /** The total tax value of the consumption. */
    totalTaxValue?: number;
}
type DownloadInvoiceRequest = {
    /** Invoice ID. */
    invoiceId: string;
    /** File type. PDF by default. */
    fileType?: DownloadInvoiceRequestFileType;
};
type ExportInvoicesRequest = {
    /**
     * Organization ID. If specified, only invoices from this Organization will be
     * returned.
     */
    organizationId?: string;
    /** Return only invoice with start date greater than billing_period_start. */
    billingPeriodStartAfter?: Date;
    /** Return only invoice with start date less than billing_period_start. */
    billingPeriodStartBefore?: Date;
    /** Invoice type. It can either be `periodic` or `purchase`. */
    invoiceType?: InvoiceType;
    /** Page number. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
    /** How invoices are ordered in the response. */
    orderBy?: ExportInvoicesRequestOrderBy;
    /** File format for exporting the invoice list. */
    fileType?: ExportInvoicesRequestFileType;
};
type GetInvoiceRequest = {
    /** Invoice ID. */
    invoiceId: string;
};
type ListConsumptionsRequest = {
    /** Order consumptions list in the response by their update date. */
    orderBy?: ListConsumptionsRequestOrderBy;
    /** Positive integer to choose the page to return. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
    /**
     * Filter by Organization ID.
     *
     * One-of ('projectIdentifier'): at most one of 'organizationId', 'projectId'
     * could be set.
     */
    organizationId?: string;
    /**
     * Filter by Project ID.
     *
     * One-of ('projectIdentifier'): at most one of 'organizationId', 'projectId'
     * could be set.
     */
    projectId?: string;
    /**
     * Filter by name of a Category as they are shown in the invoice (Compute,
     * Network, Observability).
     */
    categoryName?: string;
    /**
     * Filter by the billing period in the YYYY-MM format. If it is empty the
     * current billing period will be used as default.
     */
    billingPeriod?: string;
};
interface ListConsumptionsResponse {
    /** Detailed consumption list. */
    consumptions: ListConsumptionsResponseConsumption[];
    /** Total number of returned items. */
    totalCount: number;
    /**
     * Sum of all discounts, displayed only when no category or project ID filter
     * is applied.
     */
    totalDiscountUntaxedValue: number;
    /** Last consumption update date. */
    updatedAt?: Date;
}
type ListDiscountsRequest = {
    /** Order discounts in the response by their description. */
    orderBy?: ListDiscountsRequestOrderBy;
    /** Positive integer to choose the page to return. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
    /** ID of the organization. */
    organizationId?: string;
};
interface ListDiscountsResponse {
    /** Total number of discounts. */
    totalCount: number;
    /** Paginated returned discounts. */
    discounts: Discount[];
}
type ListInvoicesRequest = {
    /**
     * Organization ID. If specified, only invoices from this Organization will be
     * returned.
     */
    organizationId?: string;
    /** Return only invoice with start date greater than billing_period_start. */
    billingPeriodStartAfter?: Date;
    /** Return only invoice with start date less than billing_period_start. */
    billingPeriodStartBefore?: Date;
    /** Invoice type. It can either be `periodic` or `purchase`. */
    invoiceType?: InvoiceType;
    /** Page number. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
    /** How invoices are ordered in the response. */
    orderBy?: ListInvoicesRequestOrderBy;
};
interface ListInvoicesResponse {
    /** Total number of invoices. */
    totalCount: number;
    /** Paginated returned invoices. */
    invoices: Invoice[];
}
type ListTaxesRequest = {
    /** Order consumed taxes list in the response by their update date. */
    orderBy?: ListTaxesRequestOrderBy;
    /** Page number. */
    page?: number;
    /**
     * Positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    pageSize?: number;
    /** Filter by Organization ID. */
    organizationId?: string;
    /**
     * Filter by the billing period in the YYYY-MM format. If it is empty the
     * current billing period will be used as default.
     */
    billingPeriod?: string;
};
interface ListTaxesResponse {
    /** Detailed consumption tax. */
    taxes: ListTaxesResponseTax[];
    /** Total number of returned items. */
    totalCount: number;
    /** Last consumption update date. */
    updatedAt?: Date;
}

/**
 * Billing API.
 *
 * This API allows you to query your consumption.
 */
declare class API$s extends API$x {
    protected pageOfListConsumptions: (request?: Readonly<ListConsumptionsRequest>) => Promise<ListConsumptionsResponse>;
    /**
     * Get monthly consumption. Consumption allows you to retrieve your past or
     * current consumption cost, by project or category.
     *
     * @param request - The request {@link ListConsumptionsRequest}
     * @returns A Promise of ListConsumptionsResponse
     */
    listConsumptions: (request?: Readonly<ListConsumptionsRequest>) => Promise<ListConsumptionsResponse> & {
        all: () => Promise<ListConsumptionsResponseConsumption[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ListConsumptionsResponseConsumption[], void, void>;
    };
    protected pageOfListTaxes: (request?: Readonly<ListTaxesRequest>) => Promise<ListTaxesResponse>;
    /**
     * Get monthly consumption taxes. Consumption Tax allows you to retrieve your
     * past or current tax charges, by project or category.
     *
     * @param request - The request {@link ListTaxesRequest}
     * @returns A Promise of ListTaxesResponse
     */
    listTaxes: (request?: Readonly<ListTaxesRequest>) => Promise<ListTaxesResponse> & {
        all: () => Promise<ListTaxesResponseTax[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ListTaxesResponseTax[], void, void>;
    };
    protected pageOfListInvoices: (request?: Readonly<ListInvoicesRequest>) => Promise<ListInvoicesResponse>;
    /**
     * List invoices. List all your invoices, filtering by `start_date` and
     * `invoice_type`. Each invoice has its own ID.
     *
     * @param request - The request {@link ListInvoicesRequest}
     * @returns A Promise of ListInvoicesResponse
     */
    listInvoices: (request?: Readonly<ListInvoicesRequest>) => Promise<ListInvoicesResponse> & {
        all: () => Promise<Invoice[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Invoice[], void, void>;
    };
    /**
     * Export invoices. Export invoices in a CSV file.
     *
     * @param request - The request {@link ExportInvoicesRequest}
     * @returns A Promise of Blob
     */
    exportInvoices: (request?: Readonly<ExportInvoicesRequest>) => Promise<Blob>;
    /**
     * Get an invoice. Get a specific invoice, specified by its ID.
     *
     * @param request - The request {@link GetInvoiceRequest}
     * @returns A Promise of Invoice
     */
    getInvoice: (request: Readonly<GetInvoiceRequest>) => Promise<Invoice>;
    /**
     * Download an invoice. Download a specific invoice, specified by its ID.
     *
     * @param request - The request {@link DownloadInvoiceRequest}
     * @returns A Promise of Blob
     */
    downloadInvoice: (request: Readonly<DownloadInvoiceRequest>) => Promise<Blob>;
    protected pageOfListDiscounts: (request?: Readonly<ListDiscountsRequest>) => Promise<ListDiscountsResponse>;
    /**
     * List discounts. List all discounts for your organization and usable
     * categories, products, offers, references, regions and zones where the
     * discount can be applied.
     *
     * @param request - The request {@link ListDiscountsRequest}
     * @returns A Promise of ListDiscountsResponse
     */
    listDiscounts: (request?: Readonly<ListDiscountsRequest>) => Promise<ListDiscountsResponse> & {
        all: () => Promise<Discount[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Discount[], void, void>;
    };
}

type index_gen$r_Discount = Discount;
type index_gen$r_DiscountCoupon = DiscountCoupon;
type index_gen$r_DiscountDiscountMode = DiscountDiscountMode;
type index_gen$r_DiscountFilter = DiscountFilter;
type index_gen$r_DiscountFilterType = DiscountFilterType;
type index_gen$r_DownloadInvoiceRequest = DownloadInvoiceRequest;
type index_gen$r_DownloadInvoiceRequestFileType = DownloadInvoiceRequestFileType;
type index_gen$r_ExportInvoicesRequest = ExportInvoicesRequest;
type index_gen$r_ExportInvoicesRequestFileType = ExportInvoicesRequestFileType;
type index_gen$r_ExportInvoicesRequestOrderBy = ExportInvoicesRequestOrderBy;
type index_gen$r_GetInvoiceRequest = GetInvoiceRequest;
type index_gen$r_Invoice = Invoice;
type index_gen$r_InvoiceType = InvoiceType;
type index_gen$r_ListConsumptionsRequest = ListConsumptionsRequest;
type index_gen$r_ListConsumptionsRequestOrderBy = ListConsumptionsRequestOrderBy;
type index_gen$r_ListConsumptionsResponse = ListConsumptionsResponse;
type index_gen$r_ListConsumptionsResponseConsumption = ListConsumptionsResponseConsumption;
type index_gen$r_ListDiscountsRequest = ListDiscountsRequest;
type index_gen$r_ListDiscountsRequestOrderBy = ListDiscountsRequestOrderBy;
type index_gen$r_ListDiscountsResponse = ListDiscountsResponse;
type index_gen$r_ListInvoicesRequest = ListInvoicesRequest;
type index_gen$r_ListInvoicesRequestOrderBy = ListInvoicesRequestOrderBy;
type index_gen$r_ListInvoicesResponse = ListInvoicesResponse;
type index_gen$r_ListTaxesRequest = ListTaxesRequest;
type index_gen$r_ListTaxesRequestOrderBy = ListTaxesRequestOrderBy;
type index_gen$r_ListTaxesResponse = ListTaxesResponse;
type index_gen$r_ListTaxesResponseTax = ListTaxesResponseTax;
declare namespace index_gen$r {
  export { API$s as API, type index_gen$r_Discount as Discount, type index_gen$r_DiscountCoupon as DiscountCoupon, type index_gen$r_DiscountDiscountMode as DiscountDiscountMode, type index_gen$r_DiscountFilter as DiscountFilter, type index_gen$r_DiscountFilterType as DiscountFilterType, type index_gen$r_DownloadInvoiceRequest as DownloadInvoiceRequest, type index_gen$r_DownloadInvoiceRequestFileType as DownloadInvoiceRequestFileType, type index_gen$r_ExportInvoicesRequest as ExportInvoicesRequest, type index_gen$r_ExportInvoicesRequestFileType as ExportInvoicesRequestFileType, type index_gen$r_ExportInvoicesRequestOrderBy as ExportInvoicesRequestOrderBy, type index_gen$r_GetInvoiceRequest as GetInvoiceRequest, type index_gen$r_Invoice as Invoice, type index_gen$r_InvoiceType as InvoiceType, type index_gen$r_ListConsumptionsRequest as ListConsumptionsRequest, type index_gen$r_ListConsumptionsRequestOrderBy as ListConsumptionsRequestOrderBy, type index_gen$r_ListConsumptionsResponse as ListConsumptionsResponse, type index_gen$r_ListConsumptionsResponseConsumption as ListConsumptionsResponseConsumption, type index_gen$r_ListDiscountsRequest as ListDiscountsRequest, type index_gen$r_ListDiscountsRequestOrderBy as ListDiscountsRequestOrderBy, type index_gen$r_ListDiscountsResponse as ListDiscountsResponse, type index_gen$r_ListInvoicesRequest as ListInvoicesRequest, type index_gen$r_ListInvoicesRequestOrderBy as ListInvoicesRequestOrderBy, type index_gen$r_ListInvoicesResponse as ListInvoicesResponse, type index_gen$r_ListTaxesRequest as ListTaxesRequest, type index_gen$r_ListTaxesRequestOrderBy as ListTaxesRequestOrderBy, type index_gen$r_ListTaxesResponse as ListTaxesResponse, type index_gen$r_ListTaxesResponseTax as ListTaxesResponseTax };
}

declare namespace index$u {
  export { index_gen$s as v2alpha1, index_gen$r as v2beta1 };
}

type ListSnapshotsRequestOrderBy$2 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListVolumesRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ReferenceStatus = 'unknown_status' | 'attaching' | 'attached' | 'detaching' | 'detached' | 'snapshotting' | 'error';
type ReferenceType = 'unknown_type' | 'link' | 'exclusive' | 'read_only';
type SnapshotStatus$2 = 'unknown_status' | 'creating' | 'available' | 'error' | 'deleting' | 'deleted' | 'in_use' | 'locked';
type StorageClass$2 = 'unknown_storage_class' | 'unspecified' | 'bssd' | 'sbs';
type VolumeStatus = 'unknown_status' | 'creating' | 'available' | 'in_use' | 'deleting' | 'deleted' | 'resizing' | 'error' | 'snapshotting' | 'locked';
interface Reference$1 {
    /** UUID of the reference. */
    id: string;
    /** Type of resource to which the reference is associated. */
    productResourceType: string;
    /**
     * UUID of the product resource it refers to (according to the
     * product_resource_type).
     */
    productResourceId: string;
    /** Creation date of the reference. */
    createdAt?: Date;
    /** Type of reference (link, exclusive, read_only). */
    type: ReferenceType;
    /** Status of reference (attaching, attached, detaching). */
    status: ReferenceStatus;
}
interface SnapshotParentVolume {
    /** Parent volume UUID (volume from which the snapshot originates). */
    id: string;
    /** Name of the parent volume. */
    name: string;
    /** Volume type of the parent volume. */
    type: string;
    /** Current status the parent volume. */
    status: VolumeStatus;
}
interface VolumeSpecifications {
    /**
     * The maximum IO/s expected, according to the different options available in
     * stock (`5000 | 15000`).
     */
    perfIops?: number;
    /** The storage class of the volume. */
    class: StorageClass$2;
}
interface CreateVolumeRequestFromEmpty {
    /** Must be compliant with the minimum (1 GB) and maximum (10 TB) allowed size. */
    size: number;
}
interface CreateVolumeRequestFromSnapshot {
    /**
     * Must be compliant with the minimum (1 GB) and maximum (10 TB) allowed size.
     * Size is optional and is used only if a resize of the volume is requested,
     * otherwise original snapshot size will be used.
     */
    size?: number;
    /** Source snapshot from which volume will be created. */
    snapshotId: string;
}
interface Snapshot$3 {
    /** UUID of the snapshot. */
    id: string;
    /** Name of the snapshot. */
    name: string;
    /** If the parent volume was deleted, value is null. */
    parentVolume?: SnapshotParentVolume;
    /** Size in bytes of the snapshot. */
    size: number;
    /** UUID of the project the snapshot belongs to. */
    projectId: string;
    /** Creation date of the snapshot. */
    createdAt?: Date;
    /** Last modification date of the properties of a snapshot. */
    updatedAt?: Date;
    /** List of the references to the snapshot. */
    references: Reference$1[];
    /** Current status of the snapshot (available, in_use, ...). */
    status: SnapshotStatus$2;
    /** List of tags assigned to the volume. */
    tags: string[];
    /** Snapshot zone. */
    zone: Zone;
    /** Storage class of the snapshot. */
    class: StorageClass$2;
}
interface VolumeType$3 {
    /** Volume type. */
    type: string;
    /** Price of the volume billed in GB/hour. */
    pricing?: Money;
    /** Price of the snapshot billed in GB/hour. */
    snapshotPricing?: Money;
    /** Volume specifications of the volume type. */
    specs?: VolumeSpecifications;
}
interface Volume$4 {
    /** UUID of the volume. */
    id: string;
    /** Name of the volume. */
    name: string;
    /** Volume type. */
    type: string;
    /** Volume size in bytes. */
    size: number;
    /** UUID of the project to which the volume belongs. */
    projectId: string;
    /** Creation date of the volume. */
    createdAt?: Date;
    /** Last update of the properties of a volume. */
    updatedAt?: Date;
    /** List of the references to the volume. */
    references: Reference$1[];
    /**
     * When a volume is created from a snapshot, is the UUID of the snapshot from
     * which the volume has been created.
     */
    parentSnapshotId?: string;
    /** Current status of the volume (available, in_use, ...). */
    status: VolumeStatus;
    /** List of tags assigned to the volume. */
    tags: string[];
    /** Volume zone. */
    zone: Zone;
    /** Specifications of the volume. */
    specs?: VolumeSpecifications;
    /** Last time the volume was detached. */
    lastDetachedAt?: Date;
}
type CreateSnapshotRequest$4 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the volume to snapshot. */
    volumeId: string;
    /** Name of the snapshot. */
    name: string;
    /** UUID of the project to which the volume and the snapshot belong. */
    projectId?: string;
    /** List of tags assigned to the snapshot. */
    tags?: string[];
};
type CreateVolumeRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Name of the volume. */
    name: string;
    /**
     * The maximum IO/s expected, according to the different options available in
     * stock (`5000 | 15000`).
     *
     * One-of ('requirements'): at most one of 'perfIops' could be set.
     */
    perfIops?: number;
    /** UUID of the project the volume belongs to. */
    projectId?: string;
    /**
     * Specify the size of the new volume if creating a new one from scratch.
     *
     * One-of ('from'): at most one of 'fromEmpty', 'fromSnapshot' could be set.
     */
    fromEmpty?: CreateVolumeRequestFromEmpty;
    /**
     * Specify the snapshot ID of the original snapshot.
     *
     * One-of ('from'): at most one of 'fromEmpty', 'fromSnapshot' could be set.
     */
    fromSnapshot?: CreateVolumeRequestFromSnapshot;
    /** List of tags assigned to the volume. */
    tags?: string[];
};
type DeleteSnapshotRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the snapshot. */
    snapshotId: string;
};
type DeleteVolumeRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the volume. */
    volumeId: string;
};
type GetSnapshotRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the snapshot. */
    snapshotId: string;
};
type GetVolumeRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the volume. */
    volumeId: string;
};
type ListSnapshotsRequest$4 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Criteria to use when ordering the list. */
    orderBy?: ListSnapshotsRequestOrderBy$2;
    /** Filter by Project ID. */
    projectId?: string;
    /** Filter by Organization ID. */
    organizationId?: string;
    /** Page number. */
    page?: number;
    /**
     * Page size, defines how many entries are returned in one page, must be lower
     * or equal to 100.
     */
    pageSize?: number;
    /** Filter snapshots by the ID of the original volume. */
    volumeId?: string;
    /** Filter snapshots by their names. */
    name?: string;
};
interface ListSnapshotsResponse$3 {
    /** Paginated returned list of snapshots. */
    snapshots: Snapshot$3[];
    /** Total number of snpashots in the project. */
    totalCount: number;
}
type ListVolumeTypesRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Page number. */
    page?: number;
    /**
     * Page size, defines how many entries are returned in one page, must be lower
     * or equal to 100.
     */
    pageSize?: number;
};
interface ListVolumeTypesResponse {
    /** Returns paginated list of volume-types. */
    volumeTypes: VolumeType$3[];
    /** Total number of volume-types currently available in stock. */
    totalCount: number;
}
type ListVolumesRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Criteria to use when ordering the list. */
    orderBy?: ListVolumesRequestOrderBy$1;
    /** Filter by Project ID. */
    projectId?: string;
    /** Filter by Organization ID. */
    organizationId?: string;
    /** Page number. */
    page?: number;
    /**
     * Page size, defines how many entries are returned in one page, must be lower
     * or equal to 100.
     */
    pageSize?: number;
    /** Filter the return volumes by their names. */
    name?: string;
    /**
     * Filter by a product resource ID linked to this volume (such as an Instance
     * ID).
     */
    productResourceId?: string;
};
interface ListVolumesResponse$2 {
    /** Paginated returned list of volumes. */
    volumes: Volume$4[];
    /** Total number of volumes in the project. */
    totalCount: number;
}
type UpdateSnapshotRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the snapshot. */
    snapshotId: string;
    /** When defined, is the name of the snapshot. */
    name?: string;
    /** List of tags assigned to the snapshot. */
    tags?: string[];
};
type UpdateVolumeRequest$2 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the volume. */
    volumeId: string;
    /** When defined, is the new name of the volume. */
    name?: string;
    /**
     * Size in bytes of the volume, with a granularity of 1 GB (10^9 bytes). Must
     * be compliant with the minimum (1GB) and maximum (10TB) allowed size.
     */
    size?: number;
    /** List of tags assigned to the volume. */
    tags?: string[];
    /**
     * The selected value must be available for the volume's current storage
     * class.
     */
    perfIops?: number;
};

/**
 * Scaleway Block Storage API.
 *
 * This API allows you to use and manage your Block Storage volumes.
 */
declare class API$r extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    protected pageOfListVolumeTypes: (request?: Readonly<ListVolumeTypesRequest$1>) => Promise<ListVolumeTypesResponse>;
    /**
     * List volume types. List all available volume types in a specified zone. The
     * volume types listed are ordered by name in ascending order.
     *
     * @param request - The request {@link ListVolumeTypesRequest}
     * @returns A Promise of ListVolumeTypesResponse
     */
    listVolumeTypes: (request?: Readonly<ListVolumeTypesRequest$1>) => Promise<ListVolumeTypesResponse> & {
        all: () => Promise<VolumeType$3[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<VolumeType$3[], void, void>;
    };
    protected pageOfListVolumes: (request?: Readonly<ListVolumesRequest$3>) => Promise<ListVolumesResponse$2>;
    /**
     * List volumes. List all existing volumes in a specified zone. By default,
     * the volumes listed are ordered by creation date in ascending order. This
     * can be modified via the `order_by` field.
     *
     * @param request - The request {@link ListVolumesRequest}
     * @returns A Promise of ListVolumesResponse
     */
    listVolumes: (request?: Readonly<ListVolumesRequest$3>) => Promise<ListVolumesResponse$2> & {
        all: () => Promise<Volume$4[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Volume$4[], void, void>;
    };
    /**
     * Create a volume. To create a new volume from scratch, you must specify
     * `from_empty` and the `size`. To create a volume from an existing snapshot,
     * specify `from_snapshot` and the `snapshot_id` in the request payload
     * instead, size is optional and can be specified if you need to extend the
     * original size. The volume will take on the same volume class and underlying
     * IOPS limitations as the original snapshot.
     *
     * @param request - The request {@link CreateVolumeRequest}
     * @returns A Promise of Volume
     */
    createVolume: (request: Readonly<CreateVolumeRequest$3>) => Promise<Volume$4>;
    /**
     * Get a volume. Retrieve technical information about a specific volume.
     * Details such as size, type, and status are returned in the response.
     *
     * @param request - The request {@link GetVolumeRequest}
     * @returns A Promise of Volume
     */
    getVolume: (request: Readonly<GetVolumeRequest$2>) => Promise<Volume$4>;
    /**
     * Waits for {@link Volume} to be in a final state.
     *
     * @param request - The request {@link GetVolumeRequest}
     * @param options - The waiting options
     * @returns A Promise of Volume
     */
    waitForVolume: (request: Readonly<GetVolumeRequest$2>, options?: Readonly<WaitForOptions<Volume$4>>) => Promise<Volume$4>;
    /**
     * Delete a detached volume. You must specify the `volume_id` of the volume
     * you want to delete. The volume must not be in the `in_use` status.
     *
     * @param request - The request {@link DeleteVolumeRequest}
     */
    deleteVolume: (request: Readonly<DeleteVolumeRequest$2>) => Promise<void>;
    /**
     * Update a volume. Update the technical details of a volume, such as its
     * name, tags, or its new size and `volume_type` (within the same Block
     * Storage class). You can only resize a volume to a larger size. It is
     * currently not possible to change your Block Storage Class.
     *
     * @param request - The request {@link UpdateVolumeRequest}
     * @returns A Promise of Volume
     */
    updateVolume: (request: Readonly<UpdateVolumeRequest$2>) => Promise<Volume$4>;
    protected pageOfListSnapshots: (request?: Readonly<ListSnapshotsRequest$4>) => Promise<ListSnapshotsResponse$3>;
    /**
     * List all snapshots. List all available snapshots in a specified zone. By
     * default, the snapshots listed are ordered by creation date in ascending
     * order. This can be modified via the `order_by` field.
     *
     * @param request - The request {@link ListSnapshotsRequest}
     * @returns A Promise of ListSnapshotsResponse
     */
    listSnapshots: (request?: Readonly<ListSnapshotsRequest$4>) => Promise<ListSnapshotsResponse$3> & {
        all: () => Promise<Snapshot$3[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Snapshot$3[], void, void>;
    };
    /**
     * Get a snapshot. Retrieve technical information about a specific snapshot.
     * Details such as size, volume type, and status are returned in the
     * response.
     *
     * @param request - The request {@link GetSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    getSnapshot: (request: Readonly<GetSnapshotRequest$3>) => Promise<Snapshot$3>;
    /**
     * Waits for {@link Snapshot} to be in a final state.
     *
     * @param request - The request {@link GetSnapshotRequest}
     * @param options - The waiting options
     * @returns A Promise of Snapshot
     */
    waitForSnapshot: (request: Readonly<GetSnapshotRequest$3>, options?: Readonly<WaitForOptions<Snapshot$3>>) => Promise<Snapshot$3>;
    /**
     * Create a snapshot of a volume. To create a snapshot, the volume must be in
     * the `in_use` or the `available` status. If your volume is in a transient
     * state, you need to wait until the end of the current operation.
     *
     * @param request - The request {@link CreateSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    createSnapshot: (request: Readonly<CreateSnapshotRequest$4>) => Promise<Snapshot$3>;
    /**
     * Delete a snapshot. You must specify the `snapshot_id` of the snapshot you
     * want to delete. The snapshot must not be in use.
     *
     * @param request - The request {@link DeleteSnapshotRequest}
     */
    deleteSnapshot: (request: Readonly<DeleteSnapshotRequest$3>) => Promise<void>;
    /**
     * Update a snapshot. Update the name or tags of the snapshot.
     *
     * @param request - The request {@link UpdateSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    updateSnapshot: (request: Readonly<UpdateSnapshotRequest$3>) => Promise<Snapshot$3>;
}

/** Lists transient statutes of the enum {@link ReferenceStatus}. */
declare const REFERENCE_TRANSIENT_STATUSES: ReferenceStatus[];
/** Lists transient statutes of the enum {@link SnapshotStatus}. */
declare const SNAPSHOT_TRANSIENT_STATUSES$3: SnapshotStatus$2[];
/** Lists transient statutes of the enum {@link VolumeStatus}. */
declare const VOLUME_TRANSIENT_STATUSES$1: VolumeStatus[];

declare const CreateSnapshotRequest$3: {
    name: {
        minLength: number;
    };
};
declare const CreateVolumeRequest$2: {
    name: {
        minLength: number;
    };
};
declare const ListSnapshotsRequest$3: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListVolumeTypesRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListVolumesRequest$2: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const Reference: {
    productResourceType: {
        minLength: number;
    };
};

declare const validationRules_gen$c_ListVolumeTypesRequest: typeof ListVolumeTypesRequest;
declare const validationRules_gen$c_Reference: typeof Reference;
declare namespace validationRules_gen$c {
  export { CreateSnapshotRequest$3 as CreateSnapshotRequest, CreateVolumeRequest$2 as CreateVolumeRequest, ListSnapshotsRequest$3 as ListSnapshotsRequest, validationRules_gen$c_ListVolumeTypesRequest as ListVolumeTypesRequest, ListVolumesRequest$2 as ListVolumesRequest, validationRules_gen$c_Reference as Reference };
}

type index_gen$q_CreateVolumeRequestFromEmpty = CreateVolumeRequestFromEmpty;
type index_gen$q_CreateVolumeRequestFromSnapshot = CreateVolumeRequestFromSnapshot;
type index_gen$q_ListVolumeTypesResponse = ListVolumeTypesResponse;
declare const index_gen$q_REFERENCE_TRANSIENT_STATUSES: typeof REFERENCE_TRANSIENT_STATUSES;
type index_gen$q_ReferenceStatus = ReferenceStatus;
type index_gen$q_ReferenceType = ReferenceType;
type index_gen$q_SnapshotParentVolume = SnapshotParentVolume;
type index_gen$q_VolumeSpecifications = VolumeSpecifications;
type index_gen$q_VolumeStatus = VolumeStatus;
declare namespace index_gen$q {
  export { API$r as API, type CreateSnapshotRequest$4 as CreateSnapshotRequest, type CreateVolumeRequest$3 as CreateVolumeRequest, type index_gen$q_CreateVolumeRequestFromEmpty as CreateVolumeRequestFromEmpty, type index_gen$q_CreateVolumeRequestFromSnapshot as CreateVolumeRequestFromSnapshot, type DeleteSnapshotRequest$3 as DeleteSnapshotRequest, type DeleteVolumeRequest$2 as DeleteVolumeRequest, type GetSnapshotRequest$3 as GetSnapshotRequest, type GetVolumeRequest$2 as GetVolumeRequest, type ListSnapshotsRequest$4 as ListSnapshotsRequest, type ListSnapshotsRequestOrderBy$2 as ListSnapshotsRequestOrderBy, type ListSnapshotsResponse$3 as ListSnapshotsResponse, type ListVolumeTypesRequest$1 as ListVolumeTypesRequest, type index_gen$q_ListVolumeTypesResponse as ListVolumeTypesResponse, type ListVolumesRequest$3 as ListVolumesRequest, type ListVolumesRequestOrderBy$1 as ListVolumesRequestOrderBy, type ListVolumesResponse$2 as ListVolumesResponse, index_gen$q_REFERENCE_TRANSIENT_STATUSES as REFERENCE_TRANSIENT_STATUSES, type Reference$1 as Reference, type index_gen$q_ReferenceStatus as ReferenceStatus, type index_gen$q_ReferenceType as ReferenceType, SNAPSHOT_TRANSIENT_STATUSES$3 as SNAPSHOT_TRANSIENT_STATUSES, type Snapshot$3 as Snapshot, type index_gen$q_SnapshotParentVolume as SnapshotParentVolume, type SnapshotStatus$2 as SnapshotStatus, type StorageClass$2 as StorageClass, type UpdateSnapshotRequest$3 as UpdateSnapshotRequest, type UpdateVolumeRequest$2 as UpdateVolumeRequest, VOLUME_TRANSIENT_STATUSES$1 as VOLUME_TRANSIENT_STATUSES, validationRules_gen$c as ValidationRules, type Volume$4 as Volume, type index_gen$q_VolumeSpecifications as VolumeSpecifications, type index_gen$q_VolumeStatus as VolumeStatus, type VolumeType$3 as VolumeType };
}

declare namespace index$t {
  export { index_gen$q as v1alpha1 };
}

type CockpitStatus = 'unknown_status' | 'creating' | 'ready' | 'deleting' | 'updating' | 'error';
type DatasourceType = 'unknown_datasource_type' | 'metrics' | 'logs' | 'traces' | 'alerts';
type GrafanaUserRole = 'unknown_role' | 'editor' | 'viewer';
type ListDatasourcesRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListGrafanaUsersRequestOrderBy = 'login_asc' | 'login_desc';
type ListPlansRequestOrderBy = 'name_asc' | 'name_desc';
type ListTokensRequestOrderBy$2 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type PlanName = 'unknown_name' | 'free' | 'premium' | 'custom';
interface ContactPointEmail {
    to: string;
}
interface TokenScopes {
    /** Permission to fetch metrics. */
    queryMetrics: boolean;
    /** Permission to write metrics. */
    writeMetrics: boolean;
    /** Permission to setup metrics rules. */
    setupMetricsRules: boolean;
    /** Permission to fetch logs. */
    queryLogs: boolean;
    /** Permission to write logs. */
    writeLogs: boolean;
    /** Permission to set up logs rules. */
    setupLogsRules: boolean;
    /** Permission to set up alerts. */
    setupAlerts: boolean;
    /** Permission to fetch traces. */
    queryTraces: boolean;
    /** Permission to write traces. */
    writeTraces: boolean;
}
interface CockpitEndpoints {
    /** URL for metrics. */
    metricsUrl: string;
    /** URL for logs. */
    logsUrl: string;
    /** URL for traces. */
    tracesUrl: string;
    /** URL for the alert manager. */
    alertmanagerUrl: string;
    /** URL for the Grafana dashboard. */
    grafanaUrl: string;
}
/** Pricing plan. */
interface Plan {
    /** ID of a given pricing plan. */
    id: string;
    /** Name of a given pricing plan. */
    name: PlanName;
    /** Interval of time during which Scaleway's Cockpit keeps your metrics. */
    retentionMetricsInterval?: string;
    /** Interval of time during which Scaleway's Cockpit keeps your logs. */
    retentionLogsInterval?: string;
    /** Interval of time during which Scaleway's Cockpit keeps your traces. */
    retentionTracesInterval?: string;
    /** Ingestion price in cents for 1 million samples. */
    sampleIngestionPrice: number;
    /** Ingestion price in cents for 1 GB of logs. */
    logsIngestionPrice: number;
    /** Ingestion price in cents for 1 GB of traces. */
    tracesIngestionPrice: number;
    /** Retention price in euros per month. */
    retentionPrice: number;
}
/** Contact point. */
interface ContactPoint {
    /**
     * Contact point configuration.
     *
     * One-of ('configuration'): at most one of 'email' could be set.
     */
    email?: ContactPointEmail;
}
/** Data source. */
interface Datasource {
    /** ID of the data source. */
    id: string;
    /** ID of the Project the Cockpit belongs to. */
    projectId: string;
    /** Data source name. */
    name: string;
    /** Data source URL. */
    url: string;
    /** Data source type. */
    type: DatasourceType;
    /**
     * Specifies that the data source receives data from Scaleway products and is
     * managed by Scaleway.
     */
    isManagedByScaleway: boolean;
}
/** Grafana dashboard. */
interface GrafanaProductDashboard {
    /** Name of the dashboard. */
    dashboardName: string;
    /** Title of the dashboard. */
    title: string;
    /** URL of the dashboard. */
    url: string;
    /** Tags of the dashboard. */
    tags: string[];
    /** Variables of the dashboard. */
    variables: string[];
}
/** Grafana user. */
interface GrafanaUser {
    /** ID of the Grafana user. */
    id: number;
    /** Username of the Grafana user. */
    login: string;
    /** Role assigned to the Grafana user. */
    role: GrafanaUserRole;
    /** The Grafana user's password. */
    password?: string;
}
interface Token$2 {
    /** ID of the token. */
    id: string;
    /** ID of the Project. */
    projectId: string;
    /** Name of the token. */
    name: string;
    /** Date and time of the token's creation. */
    createdAt?: Date;
    /** Date and time of the token's last update. */
    updatedAt?: Date;
    /** Token's permissions. */
    scopes?: TokenScopes;
    /** Token's secret key. */
    secretKey?: string;
}
type ActivateCockpitRequest = {
    /** ID of the Project the Cockpit belongs to. */
    projectId?: string;
};
/** Cockpit. */
interface Cockpit {
    /** ID of the Project the Cockpit belongs to. */
    projectId: string;
    /** Date and time of the Cockpit's creation. */
    createdAt?: Date;
    /** Date and time of the Cockpit's last update. */
    updatedAt?: Date;
    /** Endpoints of the Cockpit. */
    endpoints?: CockpitEndpoints;
    /** Status of the Cockpit. */
    status: CockpitStatus;
    /** Specifies whether managed alerts are enabled or disabled. */
    managedAlertsEnabled: boolean;
    /** Pricing plan information. */
    plan?: Plan;
}
/** Metrics for a given Cockpit. */
interface CockpitMetrics {
    /** Time series array. */
    timeseries: TimeSeries[];
}
/** Request to create a contact point. */
type CreateContactPointRequest = {
    /** ID of the Project in which to create the contact point. */
    projectId?: string;
    /** Contact point to create. */
    contactPoint?: ContactPoint;
};
/** Request to create a data source. */
type CreateDatasourceRequest = {
    /** ID of the Project the Cockpit belongs to. */
    projectId?: string;
    /** Data source name. */
    name: string;
    /** Data source type. */
    type?: DatasourceType;
    /** Specifies that the returned output is the default data source per type. */
    isDefault: boolean;
};
/** Request to create a Grafana user. */
type CreateGrafanaUserRequest = {
    /** ID of the Project. */
    projectId?: string;
    /** Username of the Grafana user. */
    login: string;
    /** Role assigned to the Grafana user. */
    role?: GrafanaUserRole;
};
type CreateTokenRequest$2 = {
    /** ID of the Project. */
    projectId?: string;
    /** Name of the token. */
    name?: string;
    /** Token's permissions. */
    scopes?: TokenScopes;
};
type DeactivateCockpitRequest = {
    /** ID of the Project the Cockpit belongs to. */
    projectId?: string;
};
/** Request to delete a contact point. */
type DeleteContactPointRequest = {
    /** ID of the Project. */
    projectId?: string;
    /** Contact point to delete. */
    contactPoint?: ContactPoint;
};
/** Request to delete a data source. */
type DeleteDatasourceRequest = {
    /** ID of the data source. */
    datasourceId: string;
};
/** Request to delete a Grafana user. */
type DeleteGrafanaUserRequest = {
    /** ID of the Grafana user. */
    grafanaUserId: number;
    /** ID of the Project. */
    projectId?: string;
};
type DeleteTokenRequest$2 = {
    /** ID of the token. */
    tokenId: string;
};
/** Request to disable the sending of managed alerts. */
type DisableManagedAlertsRequest = {
    /** ID of the Project. */
    projectId?: string;
};
/** Request to enable the sending of managed alerts. */
type EnableManagedAlertsRequest = {
    /** ID of the Project. */
    projectId?: string;
};
/** Request to get a given Cockpit's metrics. */
type GetCockpitMetricsRequest = {
    /** ID of the Project the Cockpit belongs to. */
    projectId?: string;
    /** Desired time range's start date for the metrics. */
    startDate?: Date;
    /** Desired time range's end date for the metrics. */
    endDate?: Date;
    /** Name of the metric requested. */
    metricName?: string;
};
type GetCockpitRequest = {
    /** ID of the Project the Cockpit belongs to. */
    projectId?: string;
};
/** Request to get a dashboard. */
type GetGrafanaProductDashboardRequest = {
    /** Name of the dashboard. */
    dashboardName: string;
    /** ID of the Project. */
    projectId?: string;
};
type GetTokenRequest$2 = {
    /** ID of the token. */
    tokenId: string;
};
/** Request to list all contact points. */
type ListContactPointsRequest = {
    /** Page number. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    /** ID of the Project from which to list the contact points. */
    projectId?: string;
};
/** Response returned when listing contact points. */
interface ListContactPointsResponse {
    /** Count of all contact points created. */
    totalCount: number;
    /** Array of contact points. */
    contactPoints: ContactPoint[];
    /**
     * Specifies whether the contact point has other receivers than the default
     * receiver.
     */
    hasAdditionalReceivers: boolean;
    /** Specifies whether there are unmanaged contact points. */
    hasAdditionalContactPoints: boolean;
}
type ListDatasourcesRequest = {
    /** Page number. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    /** How the response is ordered. */
    orderBy?: ListDatasourcesRequestOrderBy;
    /** ID of the Project. */
    projectId?: string;
    /** Filter by datasource types. */
    types?: DatasourceType[];
    /** Filter by managed datasources. */
    isManagedByScaleway?: boolean;
};
interface ListDatasourcesResponse {
    /** Count of all datasources corresponding to the request. */
    totalCount: number;
    /** List of the datasources within the pagination. */
    datasources: Datasource[];
}
/** Request to get a list of dashboards. */
type ListGrafanaProductDashboardsRequest = {
    /** ID of the Project. */
    projectId?: string;
    /** Page number. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    /** Tags to filter the dashboards. */
    tags?: string[];
};
/** Response returned when getting a list of dashboards. */
interface ListGrafanaProductDashboardsResponse {
    /** Count of grafana dasboards. */
    totalCount: number;
    /** Information on grafana dashboards. */
    dashboards: GrafanaProductDashboard[];
}
/** Request to list all Grafana users. */
type ListGrafanaUsersRequest = {
    /** Page number. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    orderBy?: ListGrafanaUsersRequestOrderBy;
    /** ID of the Project. */
    projectId?: string;
};
/** Response returned when listing Grafana users. */
interface ListGrafanaUsersResponse {
    /** Count of all Grafana users. */
    totalCount: number;
    /** Information on all Grafana users. */
    grafanaUsers: GrafanaUser[];
}
/** Request to list all pricing plans. */
type ListPlansRequest = {
    /** Page number. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    orderBy?: ListPlansRequestOrderBy;
};
/** Response returned when listing all pricing plans. */
interface ListPlansResponse {
    /** Count of all pricing plans. */
    totalCount: number;
    /** Information on plans. */
    plans: Plan[];
}
type ListTokensRequest$2 = {
    /** Page number. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    /** How the response is ordered. */
    orderBy?: ListTokensRequestOrderBy$2;
    /** ID of the Project. */
    projectId?: string;
};
interface ListTokensResponse$2 {
    /** Count of all tokens created. */
    totalCount: number;
    /** List of all tokens created. */
    tokens: Token$2[];
}
/** Request to reset a Grafana user's password. */
type ResetGrafanaUserPasswordRequest = {
    /** ID of the Grafana user. */
    grafanaUserId: number;
    /** ID of the Project. */
    projectId?: string;
};
/** Request to select a specific pricing plan. */
type SelectPlanRequest = {
    /** ID of the Project. */
    projectId?: string;
    /** ID of the pricing plan. */
    planId: string;
};
/** Response returned when selecting a pricing plan. */
interface SelectPlanResponse {
}
type TriggerTestAlertRequest = {
    projectId?: string;
};

/**
 * Cockpit API.
 *
 * The Cockpit API allows you to activate your Cockpit to store metrics and
 * logs. It also provides you with a dedicated Grafana for dashboarding to
 * visualize your metrics and logs.
 */
declare class API$q extends API$x {
    /**
     * Activate the Cockpit of a given Project specified by the Project ID.
     *
     * @param request - The request {@link ActivateCockpitRequest}
     * @returns A Promise of Cockpit
     */
    activateCockpit: (request?: Readonly<ActivateCockpitRequest>) => Promise<Cockpit>;
    /**
     * Retrieve the Cockpit of a given Project specified by the Project ID.
     *
     * @param request - The request {@link GetCockpitRequest}
     * @returns A Promise of Cockpit
     */
    getCockpit: (request?: Readonly<GetCockpitRequest>) => Promise<Cockpit>;
    /**
     * Waits for {@link Cockpit} to be in a final state.
     *
     * @param request - The request {@link GetCockpitRequest}
     * @param options - The waiting options
     * @returns A Promise of Cockpit
     */
    waitForCockpit: (request?: Readonly<GetCockpitRequest>, options?: Readonly<WaitForOptions<Cockpit>>) => Promise<Cockpit>;
    /**
     * Retrieve metrics from your Cockpit specified by the ID of the Project the
     * Cockpit belongs to.
     *
     * @param request - The request {@link GetCockpitMetricsRequest}
     * @returns A Promise of CockpitMetrics
     */
    getCockpitMetrics: (request?: Readonly<GetCockpitMetricsRequest>) => Promise<CockpitMetrics>;
    /**
     * Deactivate the Cockpit of a given Project specified by the Project ID.
     *
     * @param request - The request {@link DeactivateCockpitRequest}
     * @returns A Promise of Cockpit
     */
    deactivateCockpit: (request?: Readonly<DeactivateCockpitRequest>) => Promise<Cockpit>;
    /**
     * Create a data source for a given Project specified by the Project ID and
     * the data source type.
     *
     * @param request - The request {@link CreateDatasourceRequest}
     * @returns A Promise of Datasource
     */
    createDatasource: (request: Readonly<CreateDatasourceRequest>) => Promise<Datasource>;
    /**
     * Delete a given data source specified by the data source ID.
     *
     * @param request - The request {@link DeleteDatasourceRequest}
     */
    deleteDatasource: (request: Readonly<DeleteDatasourceRequest>) => Promise<void>;
    protected pageOfListDatasources: (request?: Readonly<ListDatasourcesRequest>) => Promise<ListDatasourcesResponse>;
    /**
     * Get a list of data sources for the specified Project ID.
     *
     * @param request - The request {@link ListDatasourcesRequest}
     * @returns A Promise of ListDatasourcesResponse
     */
    listDatasources: (request?: Readonly<ListDatasourcesRequest>) => Promise<ListDatasourcesResponse> & {
        all: () => Promise<Datasource[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Datasource[], void, void>;
    };
    /**
     * Create a token in a given Project specified by the Project ID.
     *
     * @param request - The request {@link CreateTokenRequest}
     * @returns A Promise of Token
     */
    createToken: (request?: Readonly<CreateTokenRequest$2>) => Promise<Token$2>;
    protected pageOfListTokens: (request?: Readonly<ListTokensRequest$2>) => Promise<ListTokensResponse$2>;
    /**
     * Get a list of tokens in a given Project specified by the Project ID.
     *
     * @param request - The request {@link ListTokensRequest}
     * @returns A Promise of ListTokensResponse
     */
    listTokens: (request?: Readonly<ListTokensRequest$2>) => Promise<ListTokensResponse$2> & {
        all: () => Promise<Token$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Token$2[], void, void>;
    };
    /**
     * Retrieve a given token specified by the token ID.
     *
     * @param request - The request {@link GetTokenRequest}
     * @returns A Promise of Token
     */
    getToken: (request: Readonly<GetTokenRequest$2>) => Promise<Token$2>;
    /**
     * Delete a given token specified by the token ID.
     *
     * @param request - The request {@link DeleteTokenRequest}
     */
    deleteToken: (request: Readonly<DeleteTokenRequest$2>) => Promise<void>;
    /**
     * Create a contact point associated with the default receiver, to receive
     * alerts.
     *
     * @param request - The request {@link CreateContactPointRequest}
     * @returns A Promise of ContactPoint
     */
    createContactPoint: (request?: Readonly<CreateContactPointRequest>) => Promise<ContactPoint>;
    protected pageOfListContactPoints: (request?: Readonly<ListContactPointsRequest>) => Promise<ListContactPointsResponse>;
    /**
     * Get a list of contact points created for a given Cockpit, specified by the
     * ID of the Project the Cockpit belongs to.
     *
     * @param request - The request {@link ListContactPointsRequest}
     * @returns A Promise of ListContactPointsResponse
     */
    listContactPoints: (request?: Readonly<ListContactPointsRequest>) => Promise<ListContactPointsResponse> & {
        all: () => Promise<ContactPoint[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ContactPoint[], void, void>;
    };
    /**
     * Delete a contact point associated with the default receiver.
     *
     * @param request - The request {@link DeleteContactPointRequest}
     */
    deleteContactPoint: (request?: Readonly<DeleteContactPointRequest>) => Promise<void>;
    /**
     * Enable the sending of managed alerts for a given Cockpit, specified by the
     * ID of the Project the Cockpit belongs to.
     *
     * @param request - The request {@link EnableManagedAlertsRequest}
     */
    enableManagedAlerts: (request?: Readonly<EnableManagedAlertsRequest>) => Promise<void>;
    /**
     * Disable the sending of managed alerts for a given Cockpit, specified by the
     * ID of the Project the Cockpit belongs to.
     *
     * @param request - The request {@link DisableManagedAlertsRequest}
     */
    disableManagedAlerts: (request?: Readonly<DisableManagedAlertsRequest>) => Promise<void>;
    /**
     * Send a test alert to make sure your contact points get notified when an
     * actual alert is triggered.
     *
     * @param request - The request {@link TriggerTestAlertRequest}
     */
    triggerTestAlert: (request?: Readonly<TriggerTestAlertRequest>) => Promise<void>;
    /**
     * Create a Grafana user for your Cockpit's Grafana. Make sure you save the
     * automatically-generated password and the Grafana user ID.
     *
     * @param request - The request {@link CreateGrafanaUserRequest}
     * @returns A Promise of GrafanaUser
     */
    createGrafanaUser: (request: Readonly<CreateGrafanaUserRequest>) => Promise<GrafanaUser>;
    protected pageOfListGrafanaUsers: (request?: Readonly<ListGrafanaUsersRequest>) => Promise<ListGrafanaUsersResponse>;
    /**
     * Get a list of all Grafana users created in your Cockpit's Grafana.
     *
     * @param request - The request {@link ListGrafanaUsersRequest}
     * @returns A Promise of ListGrafanaUsersResponse
     */
    listGrafanaUsers: (request?: Readonly<ListGrafanaUsersRequest>) => Promise<ListGrafanaUsersResponse> & {
        all: () => Promise<GrafanaUser[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<GrafanaUser[], void, void>;
    };
    /**
     * Delete a Grafana user from your Cockpit's Grafana, specified by the ID of
     * the Project the Cockpit belongs to, and the ID of the Grafana user.
     *
     * @param request - The request {@link DeleteGrafanaUserRequest}
     */
    deleteGrafanaUser: (request: Readonly<DeleteGrafanaUserRequest>) => Promise<void>;
    /**
     * Reset the password of a Grafana user, specified by the ID of the Project
     * the Cockpit belongs to, and the ID of the Grafana user.
     *
     * @param request - The request {@link ResetGrafanaUserPasswordRequest}
     * @returns A Promise of GrafanaUser
     */
    resetGrafanaUserPassword: (request: Readonly<ResetGrafanaUserPasswordRequest>) => Promise<GrafanaUser>;
    protected pageOfListPlans: (request?: Readonly<ListPlansRequest>) => Promise<ListPlansResponse>;
    /**
     * Get a list of all pricing plans available.
     *
     * @param request - The request {@link ListPlansRequest}
     * @returns A Promise of ListPlansResponse
     */
    listPlans: (request?: Readonly<ListPlansRequest>) => Promise<ListPlansResponse> & {
        all: () => Promise<Plan[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Plan[], void, void>;
    };
    /**
     * Select your chosen pricing plan for your Cockpit, specifying the Cockpit's
     * Project ID and the pricing plan's ID in the request.
     *
     * @param request - The request {@link SelectPlanRequest}
     * @returns A Promise of SelectPlanResponse
     */
    selectPlan: (request: Readonly<SelectPlanRequest>) => Promise<SelectPlanResponse>;
    protected pageOfListGrafanaProductDashboards: (request?: Readonly<ListGrafanaProductDashboardsRequest>) => Promise<ListGrafanaProductDashboardsResponse>;
    /**
     * List product dashboards. Get a list of available product dashboards.
     *
     * @param request - The request {@link ListGrafanaProductDashboardsRequest}
     * @returns A Promise of ListGrafanaProductDashboardsResponse
     */
    listGrafanaProductDashboards: (request?: Readonly<ListGrafanaProductDashboardsRequest>) => Promise<ListGrafanaProductDashboardsResponse> & {
        all: () => Promise<GrafanaProductDashboard[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<GrafanaProductDashboard[], void, void>;
    };
    /**
     * Get a product dashboard. Get a product dashboard specified by the dashboard
     * ID.
     *
     * @param request - The request {@link GetGrafanaProductDashboardRequest}
     * @returns A Promise of GrafanaProductDashboard
     */
    getGrafanaProductDashboard: (request: Readonly<GetGrafanaProductDashboardRequest>) => Promise<GrafanaProductDashboard>;
}

/** Lists transient statutes of the enum {@link CockpitStatus}. */
declare const COCKPIT_TRANSIENT_STATUSES: CockpitStatus[];

type index_gen$p_ActivateCockpitRequest = ActivateCockpitRequest;
declare const index_gen$p_COCKPIT_TRANSIENT_STATUSES: typeof COCKPIT_TRANSIENT_STATUSES;
type index_gen$p_Cockpit = Cockpit;
type index_gen$p_CockpitEndpoints = CockpitEndpoints;
type index_gen$p_CockpitMetrics = CockpitMetrics;
type index_gen$p_CockpitStatus = CockpitStatus;
type index_gen$p_ContactPoint = ContactPoint;
type index_gen$p_ContactPointEmail = ContactPointEmail;
type index_gen$p_CreateContactPointRequest = CreateContactPointRequest;
type index_gen$p_CreateDatasourceRequest = CreateDatasourceRequest;
type index_gen$p_CreateGrafanaUserRequest = CreateGrafanaUserRequest;
type index_gen$p_Datasource = Datasource;
type index_gen$p_DatasourceType = DatasourceType;
type index_gen$p_DeactivateCockpitRequest = DeactivateCockpitRequest;
type index_gen$p_DeleteContactPointRequest = DeleteContactPointRequest;
type index_gen$p_DeleteDatasourceRequest = DeleteDatasourceRequest;
type index_gen$p_DeleteGrafanaUserRequest = DeleteGrafanaUserRequest;
type index_gen$p_DisableManagedAlertsRequest = DisableManagedAlertsRequest;
type index_gen$p_EnableManagedAlertsRequest = EnableManagedAlertsRequest;
type index_gen$p_GetCockpitMetricsRequest = GetCockpitMetricsRequest;
type index_gen$p_GetCockpitRequest = GetCockpitRequest;
type index_gen$p_GetGrafanaProductDashboardRequest = GetGrafanaProductDashboardRequest;
type index_gen$p_GrafanaProductDashboard = GrafanaProductDashboard;
type index_gen$p_GrafanaUser = GrafanaUser;
type index_gen$p_GrafanaUserRole = GrafanaUserRole;
type index_gen$p_ListContactPointsRequest = ListContactPointsRequest;
type index_gen$p_ListContactPointsResponse = ListContactPointsResponse;
type index_gen$p_ListDatasourcesRequest = ListDatasourcesRequest;
type index_gen$p_ListDatasourcesRequestOrderBy = ListDatasourcesRequestOrderBy;
type index_gen$p_ListDatasourcesResponse = ListDatasourcesResponse;
type index_gen$p_ListGrafanaProductDashboardsRequest = ListGrafanaProductDashboardsRequest;
type index_gen$p_ListGrafanaProductDashboardsResponse = ListGrafanaProductDashboardsResponse;
type index_gen$p_ListGrafanaUsersRequest = ListGrafanaUsersRequest;
type index_gen$p_ListGrafanaUsersRequestOrderBy = ListGrafanaUsersRequestOrderBy;
type index_gen$p_ListGrafanaUsersResponse = ListGrafanaUsersResponse;
type index_gen$p_ListPlansRequest = ListPlansRequest;
type index_gen$p_ListPlansRequestOrderBy = ListPlansRequestOrderBy;
type index_gen$p_ListPlansResponse = ListPlansResponse;
type index_gen$p_Plan = Plan;
type index_gen$p_PlanName = PlanName;
type index_gen$p_ResetGrafanaUserPasswordRequest = ResetGrafanaUserPasswordRequest;
type index_gen$p_SelectPlanRequest = SelectPlanRequest;
type index_gen$p_SelectPlanResponse = SelectPlanResponse;
type index_gen$p_TokenScopes = TokenScopes;
type index_gen$p_TriggerTestAlertRequest = TriggerTestAlertRequest;
declare namespace index_gen$p {
  export { API$q as API, type index_gen$p_ActivateCockpitRequest as ActivateCockpitRequest, index_gen$p_COCKPIT_TRANSIENT_STATUSES as COCKPIT_TRANSIENT_STATUSES, type index_gen$p_Cockpit as Cockpit, type index_gen$p_CockpitEndpoints as CockpitEndpoints, type index_gen$p_CockpitMetrics as CockpitMetrics, type index_gen$p_CockpitStatus as CockpitStatus, type index_gen$p_ContactPoint as ContactPoint, type index_gen$p_ContactPointEmail as ContactPointEmail, type index_gen$p_CreateContactPointRequest as CreateContactPointRequest, type index_gen$p_CreateDatasourceRequest as CreateDatasourceRequest, type index_gen$p_CreateGrafanaUserRequest as CreateGrafanaUserRequest, type CreateTokenRequest$2 as CreateTokenRequest, type index_gen$p_Datasource as Datasource, type index_gen$p_DatasourceType as DatasourceType, type index_gen$p_DeactivateCockpitRequest as DeactivateCockpitRequest, type index_gen$p_DeleteContactPointRequest as DeleteContactPointRequest, type index_gen$p_DeleteDatasourceRequest as DeleteDatasourceRequest, type index_gen$p_DeleteGrafanaUserRequest as DeleteGrafanaUserRequest, type DeleteTokenRequest$2 as DeleteTokenRequest, type index_gen$p_DisableManagedAlertsRequest as DisableManagedAlertsRequest, type index_gen$p_EnableManagedAlertsRequest as EnableManagedAlertsRequest, type index_gen$p_GetCockpitMetricsRequest as GetCockpitMetricsRequest, type index_gen$p_GetCockpitRequest as GetCockpitRequest, type index_gen$p_GetGrafanaProductDashboardRequest as GetGrafanaProductDashboardRequest, type GetTokenRequest$2 as GetTokenRequest, type index_gen$p_GrafanaProductDashboard as GrafanaProductDashboard, type index_gen$p_GrafanaUser as GrafanaUser, type index_gen$p_GrafanaUserRole as GrafanaUserRole, type index_gen$p_ListContactPointsRequest as ListContactPointsRequest, type index_gen$p_ListContactPointsResponse as ListContactPointsResponse, type index_gen$p_ListDatasourcesRequest as ListDatasourcesRequest, type index_gen$p_ListDatasourcesRequestOrderBy as ListDatasourcesRequestOrderBy, type index_gen$p_ListDatasourcesResponse as ListDatasourcesResponse, type index_gen$p_ListGrafanaProductDashboardsRequest as ListGrafanaProductDashboardsRequest, type index_gen$p_ListGrafanaProductDashboardsResponse as ListGrafanaProductDashboardsResponse, type index_gen$p_ListGrafanaUsersRequest as ListGrafanaUsersRequest, type index_gen$p_ListGrafanaUsersRequestOrderBy as ListGrafanaUsersRequestOrderBy, type index_gen$p_ListGrafanaUsersResponse as ListGrafanaUsersResponse, type index_gen$p_ListPlansRequest as ListPlansRequest, type index_gen$p_ListPlansRequestOrderBy as ListPlansRequestOrderBy, type index_gen$p_ListPlansResponse as ListPlansResponse, type ListTokensRequest$2 as ListTokensRequest, type ListTokensRequestOrderBy$2 as ListTokensRequestOrderBy, type ListTokensResponse$2 as ListTokensResponse, type index_gen$p_Plan as Plan, type index_gen$p_PlanName as PlanName, type index_gen$p_ResetGrafanaUserPasswordRequest as ResetGrafanaUserPasswordRequest, type index_gen$p_SelectPlanRequest as SelectPlanRequest, type index_gen$p_SelectPlanResponse as SelectPlanResponse, type Token$2 as Token, type index_gen$p_TokenScopes as TokenScopes, type index_gen$p_TriggerTestAlertRequest as TriggerTestAlertRequest };
}

declare namespace index$s {
  export { index_gen$p as v1beta1 };
}

type ContainerHttpOption = 'unknown_http_option' | 'enabled' | 'redirected';
type ContainerPrivacy = 'unknown_privacy' | 'public' | 'private';
type ContainerProtocol = 'unknown_protocol' | 'http1' | 'h2c';
type ContainerStatus = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked' | 'creating' | 'pending' | 'created';
type CronStatus$1 = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked' | 'creating' | 'pending';
type DomainStatus$3 = 'unknown' | 'ready' | 'deleting' | 'error' | 'creating' | 'pending';
type ListContainersRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListCronsRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc';
type ListDomainsRequestOrderBy$2 = 'created_at_asc' | 'created_at_desc' | 'hostname_asc' | 'hostname_desc';
type ListLogsRequestOrderBy$2 = 'timestamp_desc' | 'timestamp_asc';
type ListNamespacesRequestOrderBy$2 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListTokensRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc';
type ListTriggersRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc';
type LogStream$1 = 'unknown' | 'stdout' | 'stderr';
type NamespaceStatus$2 = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked' | 'creating' | 'pending';
type TokenStatus$1 = 'unknown' | 'ready' | 'deleting' | 'error' | 'creating';
type TriggerInputType$1 = 'unknown_input_type' | 'sqs' | 'scw_sqs' | 'nats' | 'scw_nats';
type TriggerStatus$1 = 'unknown_status' | 'ready' | 'deleting' | 'error' | 'creating' | 'pending';
interface SecretHashedValue$1 {
    key: string;
    hashedValue: string;
}
interface TriggerMnqNatsClientConfig$1 {
    /** @deprecated */
    mnqNamespaceId?: string;
    /** Name of the NATS subject the trigger listens to. */
    subject: string;
    /** ID of the Messaging and Queuing NATS account. */
    mnqNatsAccountId: string;
    /** ID of the Messaging and Queuing project. */
    mnqProjectId: string;
    /** Currently, only the `fr-par` region is available. */
    mnqRegion: string;
    /**
     * ID of the Messaging and Queuing credentials used to subscribe to the NATS
     * subject.
     */
    mnqCredentialId?: string;
}
interface TriggerMnqSqsClientConfig$1 {
    /** @deprecated */
    mnqNamespaceId?: string;
    /** Name of the SQS queue the trigger listens to. */
    queue: string;
    /** ID of the Messaging and Queuing project. */
    mnqProjectId: string;
    /** Currently, only the `fr-par` region is available. */
    mnqRegion: string;
    /**
     * ID of the Messaging and Queuing credentials used to read from the SQS
     * queue.
     */
    mnqCredentialId?: string;
}
interface TriggerSqsClientConfig$1 {
    endpoint: string;
    queueUrl: string;
    accessKey: string;
    secretKey: string;
}
interface Secret$2 {
    key: string;
    value?: string;
}
interface CreateTriggerRequestMnqNatsClientConfig$3 {
    /** @deprecated */
    mnqNamespaceId?: string;
    /** Name of the NATS subject the trigger should listen to. */
    subject: string;
    /** ID of the Messaging and Queuing NATS account. */
    mnqNatsAccountId: string;
    /** ID of the Messaging and Queuing project. */
    mnqProjectId: string;
    /** Currently, only the `fr-par` region is available. */
    mnqRegion: string;
}
interface CreateTriggerRequestMnqSqsClientConfig$3 {
    /** @deprecated */
    mnqNamespaceId?: string;
    /** Name of the SQS queue the trigger should listen to. */
    queue: string;
    /** You must have activated SQS on this project. */
    mnqProjectId: string;
    /** Currently, only the `fr-par` region is available. */
    mnqRegion: string;
}
interface CreateTriggerRequestSqsClientConfig$1 {
    endpoint: string;
    queueUrl: string;
    accessKey: string;
    secretKey: string;
}
interface Container {
    /** UUID of the container. */
    id: string;
    /** Name of the container. */
    name: string;
    /** UUID of the namespace the container belongs to. */
    namespaceId: string;
    /** Status of the container. */
    status: ContainerStatus;
    /** Environment variables of the container. */
    environmentVariables: Record<string, string>;
    /** Minimum number of instances to scale the container to. */
    minScale: number;
    /** Maximum number of instances to scale the container to. */
    maxScale: number;
    /** Memory limit of the container in MB. */
    memoryLimit: number;
    /** CPU limit of the container in mvCPU. */
    cpuLimit: number;
    /** Processing time limit for the container. */
    timeout?: string;
    /** Last error message of the container. */
    errorMessage?: string;
    /** Privacy setting of the container. */
    privacy: ContainerPrivacy;
    /** Description of the container. */
    description?: string;
    /**
     * Name of the registry image (e.g.
     * "rg.fr-par.scw.cloud/something/image:tag").
     */
    registryImage: string;
    /** Number of maximum concurrent executions of the container. */
    maxConcurrency: number;
    /** Domain name attributed to the contaioner. */
    domainName: string;
    /** Protocol the container uses. */
    protocol: ContainerProtocol;
    /** Port the container listens on. */
    port: number;
    /** Secret environment variables of the container. */
    secretEnvironmentVariables: SecretHashedValue$1[];
    /**
     * Possible values:
     *
     * - Redirected: Responds to HTTP request with a 301 redirect to ask the clients
     *   to use HTTPS.
     * - Enabled: Serve both HTTP and HTTPS traffic.
     */
    httpOption: ContainerHttpOption;
    /** Region in which the container will be deployed. */
    region: Region;
}
interface Cron$1 {
    /** UUID of the cron. */
    id: string;
    /** UUID of the container invoked by this cron. */
    containerId: string;
    /** UNIX cron shedule. */
    schedule: string;
    /** Arguments to pass with the cron. */
    args?: Record<string, unknown>;
    /** Status of the cron. */
    status: CronStatus$1;
    /** Name of the cron. */
    name: string;
}
interface Domain$3 {
    /** UUID of the domain. */
    id: string;
    /** Domain assigned to the container. */
    hostname: string;
    /** UUID of the container. */
    containerId: string;
    /** URL (TBD). */
    url: string;
    /** Status of the domain. */
    status: DomainStatus$3;
    /** Last error message of the domain. */
    errorMessage?: string;
}
interface Log$2 {
    message: string;
    timestamp?: Date;
    id: string;
    /** Contains the severity of the log (info, debug, error, ...). */
    level: string;
    /** Source of the log (core runtime or user code). */
    source: string;
    /** Can be stdout or stderr. */
    stream: LogStream$1;
}
interface Namespace$2 {
    /** UUID of the namespace. */
    id: string;
    /** Name of the namespace. */
    name: string;
    /** Environment variables of the namespace. */
    environmentVariables: Record<string, string>;
    /** UUID of the Organization the namespace belongs to. */
    organizationId: string;
    /** UUID of the Project the namespace belongs to. */
    projectId: string;
    /** Status of the namespace. */
    status: NamespaceStatus$2;
    /** UUID of the registry namespace. */
    registryNamespaceId: string;
    /** Last error message of the namesace. */
    errorMessage?: string;
    /** Registry endpoint of the namespace. */
    registryEndpoint: string;
    /** Description of the endpoint. */
    description?: string;
    /** Secret environment variables of the namespace. */
    secretEnvironmentVariables: SecretHashedValue$1[];
    /** Region in which the namespace will be created. */
    region: Region;
}
interface Token$1 {
    /** UUID of the token. */
    id: string;
    /** Identifier of the token. */
    token: string;
    /**
     * UUID of the container the token belongs to.
     *
     * One-of ('scope'): at most one of 'containerId', 'namespaceId' could be set.
     */
    containerId?: string;
    /**
     * UUID of the namespace the token belongs to.
     *
     * One-of ('scope'): at most one of 'containerId', 'namespaceId' could be set.
     */
    namespaceId?: string;
    /** @deprecated Public key of the token. */
    publicKey?: string;
    /** Status of the token. */
    status: TokenStatus$1;
    /** Description of the token. */
    description?: string;
    /** Expiry date of the token. */
    expiresAt?: Date;
}
interface Trigger$1 {
    /** ID of the trigger. */
    id: string;
    /** Name of the trigger. */
    name: string;
    /** Description of the trigger. */
    description: string;
    /** ID of the container to trigger. */
    containerId: string;
    /** Type of the input. */
    inputType: TriggerInputType$1;
    /** Status of the trigger. */
    status: TriggerStatus$1;
    /** Error message of the trigger. */
    errorMessage?: string;
    /**
     * Configuration for a Scaleway Messaging and Queuing SQS queue.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    scwSqsConfig?: TriggerMnqSqsClientConfig$1;
    /**
     * Configuration for a Scaleway Messaging and Queuing NATS subject.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    scwNatsConfig?: TriggerMnqNatsClientConfig$1;
    /**
     * Configuration for an AWS SQS queue.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    sqsConfig?: TriggerSqsClientConfig$1;
}
interface UpdateTriggerRequestSqsClientConfig$1 {
    accessKey?: string;
    secretKey?: string;
}
type CreateContainerRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespace the container belongs to. */
    namespaceId: string;
    /** Name of the container. */
    name: string;
    /** Environment variables of the container. */
    environmentVariables?: Record<string, string>;
    /** Minimum number of instances to scale the container to. */
    minScale?: number;
    /** Maximum number of instances to scale the container to. */
    maxScale?: number;
    /** Memory limit of the container in MB. */
    memoryLimit?: number;
    /** CPU limit of the container in mvCPU. */
    cpuLimit?: number;
    /** Processing time limit for the container. */
    timeout?: string;
    /** Privacy setting of the container. */
    privacy?: ContainerPrivacy;
    /** Description of the container. */
    description?: string;
    /**
     * Name of the registry image (e.g.
     * "rg.fr-par.scw.cloud/something/image:tag").
     */
    registryImage?: string;
    /** Number of maximum concurrent executions of the container. */
    maxConcurrency?: number;
    /** Protocol the container uses. */
    protocol?: ContainerProtocol;
    /** Port the container listens on. */
    port?: number;
    /** Secret environment variables of the container. */
    secretEnvironmentVariables?: Secret$2[];
    /**
     * Possible values:
     *
     * - Redirected: Responds to HTTP request with a 301 redirect to ask the clients
     *   to use HTTPS.
     * - Enabled: Serve both HTTP and HTTPS traffic.
     */
    httpOption?: ContainerHttpOption;
};
type CreateCronRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the container to invoke by the cron. */
    containerId: string;
    /** UNIX cron shedule. */
    schedule: string;
    /** Arguments to pass with the cron. */
    args?: Record<string, unknown>;
    /** Name of the cron to create. */
    name?: string;
};
type CreateDomainRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Domain to assign. */
    hostname: string;
    /** UUID of the container to assign the domain to. */
    containerId: string;
};
type CreateNamespaceRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the namespace to create. */
    name?: string;
    /** Environment variables of the namespace to create. */
    environmentVariables?: Record<string, string>;
    /** UUID of the Project in which the namespace will be created. */
    projectId?: string;
    /** Description of the namespace to create. */
    description?: string;
    /** Secret environment variables of the namespace to create. */
    secretEnvironmentVariables?: Secret$2[];
};
type CreateTokenRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * UUID of the container to create the token for.
     *
     * One-of ('scope'): at most one of 'containerId', 'namespaceId' could be set.
     */
    containerId?: string;
    /**
     * UUID of the namespace to create the token for.
     *
     * One-of ('scope'): at most one of 'containerId', 'namespaceId' could be set.
     */
    namespaceId?: string;
    /** Description of the token. */
    description?: string;
    /** Expiry date of the token. */
    expiresAt?: Date;
};
type CreateTriggerRequest$3 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the trigger. */
    name: string;
    /** ID of the container to trigger. */
    containerId: string;
    /** Description of the trigger. */
    description?: string;
    /**
     * Configuration for a Scaleway Messaging and Queuing SQS queue.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    scwSqsConfig?: CreateTriggerRequestMnqSqsClientConfig$3;
    /**
     * Configuration for a Scaleway Messaging and Queuing NATS subject.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    scwNatsConfig?: CreateTriggerRequestMnqNatsClientConfig$3;
    /**
     * Configuration for an AWS SQS queue.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    sqsConfig?: CreateTriggerRequestSqsClientConfig$1;
};
type DeleteContainerRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the container to delete. */
    containerId: string;
};
type DeleteCronRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the cron to delete. */
    cronId: string;
};
type DeleteDomainRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the domain to delete. */
    domainId: string;
};
type DeleteNamespaceRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespace to delete. */
    namespaceId: string;
};
type DeleteTokenRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the token to delete. */
    tokenId: string;
};
type DeleteTriggerRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the trigger to delete. */
    triggerId: string;
};
type DeployContainerRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the container to deploy. */
    containerId: string;
};
type GetContainerRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the container to get. */
    containerId: string;
};
type GetCronRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the cron to get. */
    cronId: string;
};
type GetDomainRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the domain to get. */
    domainId: string;
};
type GetNamespaceRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespace to get. */
    namespaceId: string;
};
type GetTokenRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the token to get. */
    tokenId: string;
};
type GetTriggerRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the trigger to get. */
    triggerId: string;
};
type IssueJWTRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** One-of ('scope'): at most one of 'containerId', 'namespaceId' could be set. */
    containerId?: string;
    /** One-of ('scope'): at most one of 'containerId', 'namespaceId' could be set. */
    namespaceId?: string;
    expiresAt?: Date;
};
type ListContainersRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of containers per page. */
    pageSize?: number;
    /** Order of the containers. */
    orderBy?: ListContainersRequestOrderBy;
    /** UUID of the namespace the container belongs to. */
    namespaceId: string;
    /** Name of the container. */
    name?: string;
    /** UUID of the Organization the container belongs to. */
    organizationId?: string;
    /** UUID of the Project the container belongs to. */
    projectId?: string;
};
interface ListContainersResponse {
    /** Array of containers. */
    containers: Container[];
    /** Total number of containers. */
    totalCount: number;
}
type ListCronsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of crons per page. */
    pageSize?: number;
    /** Order of the crons. */
    orderBy?: ListCronsRequestOrderBy$1;
    /** UUID of the container invoked by the cron. */
    containerId: string;
};
interface ListCronsResponse$1 {
    /** Array of crons. */
    crons: Cron$1[];
    /** Total number of crons. */
    totalCount: number;
}
type ListDomainsRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of domains per page. */
    pageSize?: number;
    /** Order of the domains. */
    orderBy?: ListDomainsRequestOrderBy$2;
    /** UUID of the container the domain belongs to. */
    containerId: string;
};
interface ListDomainsResponse$3 {
    /** Array of domains. */
    domains: Domain$3[];
    /** Total number of domains. */
    totalCount: number;
}
type ListLogsRequest$3 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the container. */
    containerId: string;
    /** Page number. */
    page?: number;
    /** Number of logs per page. */
    pageSize?: number;
    /** Order of the logs. */
    orderBy?: ListLogsRequestOrderBy$2;
};
interface ListLogsResponse$2 {
    logs: Log$2[];
    totalCount: number;
}
type ListNamespacesRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of namespaces per page. */
    pageSize?: number;
    /** Order of the namespaces. */
    orderBy?: ListNamespacesRequestOrderBy$2;
    /** Name of the namespaces. */
    name?: string;
    /** UUID of the Organization the namespace belongs to. */
    organizationId?: string;
    /** UUID of the Project the namespace belongs to. */
    projectId?: string;
};
interface ListNamespacesResponse$2 {
    /** Array of the namespaces. */
    namespaces: Namespace$2[];
    /** Total number of namespaces. */
    totalCount: number;
}
type ListTokensRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of tokens per page. */
    pageSize?: number;
    /** Order of the tokens. */
    orderBy?: ListTokensRequestOrderBy$1;
    /** UUID of the container the token belongs to. */
    containerId?: string;
    /** UUID of the namespace the token belongs to. */
    namespaceId?: string;
};
interface ListTokensResponse$1 {
    tokens: Token$1[];
    totalCount: number;
}
type ListTriggersRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number to return. */
    page?: number;
    /** Maximum number of triggers to return per page. */
    pageSize?: number;
    /** Order in which to return results. */
    orderBy?: ListTriggersRequestOrderBy$1;
    /**
     * ID of the container the triggers belongs to.
     *
     * One-of ('scope'): at most one of 'containerId', 'namespaceId', 'projectId'
     * could be set.
     */
    containerId?: string;
    /**
     * ID of the namespace the triggers belongs to.
     *
     * One-of ('scope'): at most one of 'containerId', 'namespaceId', 'projectId'
     * could be set.
     */
    namespaceId?: string;
    /**
     * ID of the project the triggers belongs to.
     *
     * One-of ('scope'): at most one of 'containerId', 'namespaceId', 'projectId'
     * could be set.
     */
    projectId?: string;
};
interface ListTriggersResponse$1 {
    /** Total count of existing triggers (matching any filters specified). */
    totalCount: number;
    /** Triggers on this page. */
    triggers: Trigger$1[];
}
type UpdateContainerRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the container to update. */
    containerId: string;
    /** Environment variables of the container. */
    environmentVariables?: Record<string, string>;
    /** Minimum number of instances to scale the container to. */
    minScale?: number;
    /** Maximum number of instances to scale the container to. */
    maxScale?: number;
    /** Memory limit of the container in MB. */
    memoryLimit?: number;
    /** CPU limit of the container in mvCPU. */
    cpuLimit?: number;
    /** Processing time limit for the container. */
    timeout?: string;
    /** Defines whether to redeploy failed containers. */
    redeploy?: boolean;
    /** Privacy settings of the container. */
    privacy?: ContainerPrivacy;
    /** Description of the container. */
    description?: string;
    /**
     * Name of the registry image (e.g.
     * "rg.fr-par.scw.cloud/something/image:tag").
     */
    registryImage?: string;
    /** Number of maximum concurrent executions of the container. */
    maxConcurrency?: number;
    protocol?: ContainerProtocol;
    port?: number;
    secretEnvironmentVariables?: Secret$2[];
    /**
     * Possible values:
     *
     * - Redirected: Responds to HTTP request with a 301 redirect to ask the clients
     *   to use HTTPS.
     * - Enabled: Serve both HTTP and HTTPS traffic.
     */
    httpOption?: ContainerHttpOption;
};
type UpdateCronRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the cron to update. */
    cronId: string;
    /** UUID of the container invoked by the cron. */
    containerId?: string;
    /** UNIX cron schedule. */
    schedule?: string;
    /** Arguments to pass with the cron. */
    args?: Record<string, unknown>;
    /** Name of the cron. */
    name?: string;
};
type UpdateNamespaceRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespace to update. */
    namespaceId: string;
    /** Environment variables of the namespace to update. */
    environmentVariables?: Record<string, string>;
    /** Description of the namespace to update. */
    description?: string;
    /** Secret environment variables of the namespace to update. */
    secretEnvironmentVariables?: Secret$2[];
};
type UpdateTriggerRequest$3 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the trigger to update. */
    triggerId: string;
    /** Name of the trigger. */
    name?: string;
    /** Description of the trigger. */
    description?: string;
    /**
     * Configuration for an AWS SQS queue.
     *
     * One-of ('config'): at most one of 'sqsConfig' could be set.
     */
    sqsConfig?: UpdateTriggerRequestSqsClientConfig$1;
};

/** Serverless Containers API. */
declare class API$p extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListNamespaces: (request?: Readonly<ListNamespacesRequest$2>) => Promise<ListNamespacesResponse$2>;
    /**
     * List all your namespaces. List all namespaces in a specified region.
     *
     * @param request - The request {@link ListNamespacesRequest}
     * @returns A Promise of ListNamespacesResponse
     */
    listNamespaces: (request?: Readonly<ListNamespacesRequest$2>) => Promise<ListNamespacesResponse$2> & {
        all: () => Promise<Namespace$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Namespace$2[], void, void>;
    };
    /**
     * Get a namespace. Get the namespace associated with the specified ID.
     *
     * @param request - The request {@link GetNamespaceRequest}
     * @returns A Promise of Namespace
     */
    getNamespace: (request: Readonly<GetNamespaceRequest$2>) => Promise<Namespace$2>;
    /**
     * Waits for {@link Namespace} to be in a final state.
     *
     * @param request - The request {@link GetNamespaceRequest}
     * @param options - The waiting options
     * @returns A Promise of Namespace
     */
    waitForNamespace: (request: Readonly<GetNamespaceRequest$2>, options?: Readonly<WaitForOptions<Namespace$2>>) => Promise<Namespace$2>;
    /**
     * Create a new namespace. Create a new namespace in a specified region.
     *
     * @param request - The request {@link CreateNamespaceRequest}
     * @returns A Promise of Namespace
     */
    createNamespace: (request?: Readonly<CreateNamespaceRequest$2>) => Promise<Namespace$2>;
    /**
     * Update an existing namespace. Update the space associated with the
     * specified ID.
     *
     * @param request - The request {@link UpdateNamespaceRequest}
     * @returns A Promise of Namespace
     */
    updateNamespace: (request: Readonly<UpdateNamespaceRequest$2>) => Promise<Namespace$2>;
    /**
     * Delete an existing namespace. Delete the namespace associated with the
     * specified ID.
     *
     * @param request - The request {@link DeleteNamespaceRequest}
     * @returns A Promise of Namespace
     */
    deleteNamespace: (request: Readonly<DeleteNamespaceRequest$2>) => Promise<Namespace$2>;
    protected pageOfListContainers: (request: Readonly<ListContainersRequest>) => Promise<ListContainersResponse>;
    /**
     * List all your containers. List all containers for a specified region.
     *
     * @param request - The request {@link ListContainersRequest}
     * @returns A Promise of ListContainersResponse
     */
    listContainers: (request: Readonly<ListContainersRequest>) => Promise<ListContainersResponse> & {
        all: () => Promise<Container[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Container[], void, void>;
    };
    /**
     * Get a container. Get the container associated with the specified ID.
     *
     * @param request - The request {@link GetContainerRequest}
     * @returns A Promise of Container
     */
    getContainer: (request: Readonly<GetContainerRequest>) => Promise<Container>;
    /**
     * Waits for {@link Container} to be in a final state.
     *
     * @param request - The request {@link GetContainerRequest}
     * @param options - The waiting options
     * @returns A Promise of Container
     */
    waitForContainer: (request: Readonly<GetContainerRequest>, options?: Readonly<WaitForOptions<Container>>) => Promise<Container>;
    /**
     * Create a new container. Create a new container in the specified region.
     *
     * @param request - The request {@link CreateContainerRequest}
     * @returns A Promise of Container
     */
    createContainer: (request: Readonly<CreateContainerRequest>) => Promise<Container>;
    /**
     * Update an existing container. Update the container associated with the
     * specified ID.
     *
     * @param request - The request {@link UpdateContainerRequest}
     * @returns A Promise of Container
     */
    updateContainer: (request: Readonly<UpdateContainerRequest>) => Promise<Container>;
    /**
     * Delete a container. Delete the container associated with the specified ID.
     *
     * @param request - The request {@link DeleteContainerRequest}
     * @returns A Promise of Container
     */
    deleteContainer: (request: Readonly<DeleteContainerRequest>) => Promise<Container>;
    /**
     * Deploy a container. Deploy a container associated with the specified ID.
     *
     * @param request - The request {@link DeployContainerRequest}
     * @returns A Promise of Container
     */
    deployContainer: (request: Readonly<DeployContainerRequest>) => Promise<Container>;
    protected pageOfListCrons: (request: Readonly<ListCronsRequest$1>) => Promise<ListCronsResponse$1>;
    /**
     * List all your crons.
     *
     * @param request - The request {@link ListCronsRequest}
     * @returns A Promise of ListCronsResponse
     */
    listCrons: (request: Readonly<ListCronsRequest$1>) => Promise<ListCronsResponse$1> & {
        all: () => Promise<Cron$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Cron$1[], void, void>;
    };
    /**
     * Get a cron. Get the cron associated with the specified ID.
     *
     * @param request - The request {@link GetCronRequest}
     * @returns A Promise of Cron
     */
    getCron: (request: Readonly<GetCronRequest$1>) => Promise<Cron$1>;
    /**
     * Waits for {@link Cron} to be in a final state.
     *
     * @param request - The request {@link GetCronRequest}
     * @param options - The waiting options
     * @returns A Promise of Cron
     */
    waitForCron: (request: Readonly<GetCronRequest$1>, options?: Readonly<WaitForOptions<Cron$1>>) => Promise<Cron$1>;
    /**
     * Create a new cron.
     *
     * @param request - The request {@link CreateCronRequest}
     * @returns A Promise of Cron
     */
    createCron: (request: Readonly<CreateCronRequest$1>) => Promise<Cron$1>;
    /**
     * Update an existing cron. Update the cron associated with the specified ID.
     *
     * @param request - The request {@link UpdateCronRequest}
     * @returns A Promise of Cron
     */
    updateCron: (request: Readonly<UpdateCronRequest$1>) => Promise<Cron$1>;
    /**
     * Delete an existing cron. Delete the cron associated with the specified ID.
     *
     * @param request - The request {@link DeleteCronRequest}
     * @returns A Promise of Cron
     */
    deleteCron: (request: Readonly<DeleteCronRequest$1>) => Promise<Cron$1>;
    protected pageOfListLogs: (request: Readonly<ListLogsRequest$3>) => Promise<ListLogsResponse$2>;
    /**
     * Deprecated (replaced by
     * [Cockpit](https://www.scaleway.com/en/developers/api/cockpit/)). List your
     * container logs. Deprecated (replaced by
     * [Cockpit](https://www.scaleway.com/en/developers/api/cockpit/)). List the
     * logs of the container with the specified ID.
     *
     * @deprecated
     * @param request - The request {@link ListLogsRequest}
     * @returns A Promise of ListLogsResponse
     */
    listLogs: (request: Readonly<ListLogsRequest$3>) => Promise<ListLogsResponse$2> & {
        all: () => Promise<Log$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Log$2[], void, void>;
    };
    protected pageOfListDomains: (request: Readonly<ListDomainsRequest$2>) => Promise<ListDomainsResponse$3>;
    /**
     * List all domain name bindings. List all domain name bindings in a specified
     * region.
     *
     * @param request - The request {@link ListDomainsRequest}
     * @returns A Promise of ListDomainsResponse
     */
    listDomains: (request: Readonly<ListDomainsRequest$2>) => Promise<ListDomainsResponse$3> & {
        all: () => Promise<Domain$3[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Domain$3[], void, void>;
    };
    /**
     * Get a domain name binding. Get a domain name binding for the container with
     * the specified ID.
     *
     * @param request - The request {@link GetDomainRequest}
     * @returns A Promise of Domain
     */
    getDomain: (request: Readonly<GetDomainRequest$2>) => Promise<Domain$3>;
    /**
     * Waits for {@link Domain} to be in a final state.
     *
     * @param request - The request {@link GetDomainRequest}
     * @param options - The waiting options
     * @returns A Promise of Domain
     */
    waitForDomain: (request: Readonly<GetDomainRequest$2>, options?: Readonly<WaitForOptions<Domain$3>>) => Promise<Domain$3>;
    /**
     * Create a domain name binding. Create a domain name binding for the
     * container with the specified ID.
     *
     * @param request - The request {@link CreateDomainRequest}
     * @returns A Promise of Domain
     */
    createDomain: (request: Readonly<CreateDomainRequest$2>) => Promise<Domain$3>;
    /**
     * Delete a domain name binding. Delete the domain name binding with the
     * specific ID.
     *
     * @param request - The request {@link DeleteDomainRequest}
     * @returns A Promise of Domain
     */
    deleteDomain: (request: Readonly<DeleteDomainRequest$1>) => Promise<Domain$3>;
    /**
     * Create a JWT token. Deprecated in favor of CreateToken.
     *
     * @deprecated
     * @param request - The request {@link IssueJWTRequest}
     * @returns A Promise of Token
     */
    issueJWT: (request?: Readonly<IssueJWTRequest$1>) => Promise<Token$1>;
    /**
     * Create a new revocable token.
     *
     * @param request - The request {@link CreateTokenRequest}
     * @returns A Promise of Token
     */
    createToken: (request?: Readonly<CreateTokenRequest$1>) => Promise<Token$1>;
    /**
     * Get a token. Get a token with a specified ID.
     *
     * @param request - The request {@link GetTokenRequest}
     * @returns A Promise of Token
     */
    getToken: (request: Readonly<GetTokenRequest$1>) => Promise<Token$1>;
    /**
     * Waits for {@link Token} to be in a final state.
     *
     * @param request - The request {@link GetTokenRequest}
     * @param options - The waiting options
     * @returns A Promise of Token
     */
    waitForToken: (request: Readonly<GetTokenRequest$1>, options?: Readonly<WaitForOptions<Token$1>>) => Promise<Token$1>;
    protected pageOfListTokens: (request?: Readonly<ListTokensRequest$1>) => Promise<ListTokensResponse$1>;
    /**
     * List all tokens. List all tokens belonging to a specified Organization or
     * Project.
     *
     * @param request - The request {@link ListTokensRequest}
     * @returns A Promise of ListTokensResponse
     */
    listTokens: (request?: Readonly<ListTokensRequest$1>) => Promise<ListTokensResponse$1> & {
        all: () => Promise<Token$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Token$1[], void, void>;
    };
    /**
     * Delete a token. Delete a token with a specified ID.
     *
     * @param request - The request {@link DeleteTokenRequest}
     * @returns A Promise of Token
     */
    deleteToken: (request: Readonly<DeleteTokenRequest$1>) => Promise<Token$1>;
    /**
     * Create a trigger. Create a new trigger for a specified container.
     *
     * @param request - The request {@link CreateTriggerRequest}
     * @returns A Promise of Trigger
     */
    createTrigger: (request: Readonly<CreateTriggerRequest$3>) => Promise<Trigger$1>;
    /**
     * Get a trigger. Get a trigger with a specified ID.
     *
     * @param request - The request {@link GetTriggerRequest}
     * @returns A Promise of Trigger
     */
    getTrigger: (request: Readonly<GetTriggerRequest$1>) => Promise<Trigger$1>;
    /**
     * Waits for {@link Trigger} to be in a final state.
     *
     * @param request - The request {@link GetTriggerRequest}
     * @param options - The waiting options
     * @returns A Promise of Trigger
     */
    waitForTrigger: (request: Readonly<GetTriggerRequest$1>, options?: Readonly<WaitForOptions<Trigger$1>>) => Promise<Trigger$1>;
    protected pageOfListTriggers: (request?: Readonly<ListTriggersRequest$1>) => Promise<ListTriggersResponse$1>;
    /**
     * List all triggers. List all triggers belonging to a specified Organization
     * or Project.
     *
     * @param request - The request {@link ListTriggersRequest}
     * @returns A Promise of ListTriggersResponse
     */
    listTriggers: (request?: Readonly<ListTriggersRequest$1>) => Promise<ListTriggersResponse$1> & {
        all: () => Promise<Trigger$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Trigger$1[], void, void>;
    };
    /**
     * Update a trigger. Update a trigger with a specified ID.
     *
     * @param request - The request {@link UpdateTriggerRequest}
     * @returns A Promise of Trigger
     */
    updateTrigger: (request: Readonly<UpdateTriggerRequest$3>) => Promise<Trigger$1>;
    /**
     * Delete a trigger. Delete a trigger with a specified ID.
     *
     * @param request - The request {@link DeleteTriggerRequest}
     * @returns A Promise of Trigger
     */
    deleteTrigger: (request: Readonly<DeleteTriggerRequest$1>) => Promise<Trigger$1>;
}

/** Lists transient statutes of the enum {@link ContainerStatus}. */
declare const CONTAINER_TRANSIENT_STATUSES: ContainerStatus[];
/** Lists transient statutes of the enum {@link CronStatus}. */
declare const CRON_TRANSIENT_STATUSES$1: CronStatus$1[];
/** Lists transient statutes of the enum {@link DomainStatus}. */
declare const DOMAIN_TRANSIENT_STATUSES$3: DomainStatus$3[];
/** Lists transient statutes of the enum {@link NamespaceStatus}. */
declare const NAMESPACE_TRANSIENT_STATUSES$2: NamespaceStatus$2[];
/** Lists transient statutes of the enum {@link TokenStatus}. */
declare const TOKEN_TRANSIENT_STATUSES$1: TokenStatus$1[];
/** Lists transient statutes of the enum {@link TriggerStatus}. */
declare const TRIGGER_TRANSIENT_STATUSES$1: TriggerStatus$1[];

declare const CreateTriggerRequest$2: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const CreateTriggerRequestMnqNatsClientConfig$2: {
    mnqRegion: {
        maxLength: number;
        minLength: number;
    };
    subject: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const CreateTriggerRequestMnqSqsClientConfig$2: {
    mnqRegion: {
        maxLength: number;
        minLength: number;
    };
    queue: {
        maxLength: number;
        minLength: number;
    };
};
declare const UpdateTriggerRequest$2: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
    };
};

declare namespace validationRules_gen$b {
  export { CreateTriggerRequest$2 as CreateTriggerRequest, CreateTriggerRequestMnqNatsClientConfig$2 as CreateTriggerRequestMnqNatsClientConfig, CreateTriggerRequestMnqSqsClientConfig$2 as CreateTriggerRequestMnqSqsClientConfig, UpdateTriggerRequest$2 as UpdateTriggerRequest };
}

declare const index_gen$o_CONTAINER_TRANSIENT_STATUSES: typeof CONTAINER_TRANSIENT_STATUSES;
type index_gen$o_Container = Container;
type index_gen$o_ContainerHttpOption = ContainerHttpOption;
type index_gen$o_ContainerPrivacy = ContainerPrivacy;
type index_gen$o_ContainerProtocol = ContainerProtocol;
type index_gen$o_ContainerStatus = ContainerStatus;
type index_gen$o_CreateContainerRequest = CreateContainerRequest;
type index_gen$o_DeleteContainerRequest = DeleteContainerRequest;
type index_gen$o_DeployContainerRequest = DeployContainerRequest;
type index_gen$o_GetContainerRequest = GetContainerRequest;
type index_gen$o_ListContainersRequest = ListContainersRequest;
type index_gen$o_ListContainersRequestOrderBy = ListContainersRequestOrderBy;
type index_gen$o_ListContainersResponse = ListContainersResponse;
type index_gen$o_UpdateContainerRequest = UpdateContainerRequest;
declare namespace index_gen$o {
  export { API$p as API, index_gen$o_CONTAINER_TRANSIENT_STATUSES as CONTAINER_TRANSIENT_STATUSES, CRON_TRANSIENT_STATUSES$1 as CRON_TRANSIENT_STATUSES, type index_gen$o_Container as Container, type index_gen$o_ContainerHttpOption as ContainerHttpOption, type index_gen$o_ContainerPrivacy as ContainerPrivacy, type index_gen$o_ContainerProtocol as ContainerProtocol, type index_gen$o_ContainerStatus as ContainerStatus, type index_gen$o_CreateContainerRequest as CreateContainerRequest, type CreateCronRequest$1 as CreateCronRequest, type CreateDomainRequest$2 as CreateDomainRequest, type CreateNamespaceRequest$2 as CreateNamespaceRequest, type CreateTokenRequest$1 as CreateTokenRequest, type CreateTriggerRequest$3 as CreateTriggerRequest, type CreateTriggerRequestMnqNatsClientConfig$3 as CreateTriggerRequestMnqNatsClientConfig, type CreateTriggerRequestMnqSqsClientConfig$3 as CreateTriggerRequestMnqSqsClientConfig, type CreateTriggerRequestSqsClientConfig$1 as CreateTriggerRequestSqsClientConfig, type Cron$1 as Cron, type CronStatus$1 as CronStatus, DOMAIN_TRANSIENT_STATUSES$3 as DOMAIN_TRANSIENT_STATUSES, type index_gen$o_DeleteContainerRequest as DeleteContainerRequest, type DeleteCronRequest$1 as DeleteCronRequest, type DeleteDomainRequest$1 as DeleteDomainRequest, type DeleteNamespaceRequest$2 as DeleteNamespaceRequest, type DeleteTokenRequest$1 as DeleteTokenRequest, type DeleteTriggerRequest$1 as DeleteTriggerRequest, type index_gen$o_DeployContainerRequest as DeployContainerRequest, type Domain$3 as Domain, type DomainStatus$3 as DomainStatus, type index_gen$o_GetContainerRequest as GetContainerRequest, type GetCronRequest$1 as GetCronRequest, type GetDomainRequest$2 as GetDomainRequest, type GetNamespaceRequest$2 as GetNamespaceRequest, type GetTokenRequest$1 as GetTokenRequest, type GetTriggerRequest$1 as GetTriggerRequest, type IssueJWTRequest$1 as IssueJWTRequest, type index_gen$o_ListContainersRequest as ListContainersRequest, type index_gen$o_ListContainersRequestOrderBy as ListContainersRequestOrderBy, type index_gen$o_ListContainersResponse as ListContainersResponse, type ListCronsRequest$1 as ListCronsRequest, type ListCronsRequestOrderBy$1 as ListCronsRequestOrderBy, type ListCronsResponse$1 as ListCronsResponse, type ListDomainsRequest$2 as ListDomainsRequest, type ListDomainsRequestOrderBy$2 as ListDomainsRequestOrderBy, type ListDomainsResponse$3 as ListDomainsResponse, type ListLogsRequest$3 as ListLogsRequest, type ListLogsRequestOrderBy$2 as ListLogsRequestOrderBy, type ListLogsResponse$2 as ListLogsResponse, type ListNamespacesRequest$2 as ListNamespacesRequest, type ListNamespacesRequestOrderBy$2 as ListNamespacesRequestOrderBy, type ListNamespacesResponse$2 as ListNamespacesResponse, type ListTokensRequest$1 as ListTokensRequest, type ListTokensRequestOrderBy$1 as ListTokensRequestOrderBy, type ListTokensResponse$1 as ListTokensResponse, type ListTriggersRequest$1 as ListTriggersRequest, type ListTriggersRequestOrderBy$1 as ListTriggersRequestOrderBy, type ListTriggersResponse$1 as ListTriggersResponse, type Log$2 as Log, type LogStream$1 as LogStream, NAMESPACE_TRANSIENT_STATUSES$2 as NAMESPACE_TRANSIENT_STATUSES, type Namespace$2 as Namespace, type NamespaceStatus$2 as NamespaceStatus, type Secret$2 as Secret, type SecretHashedValue$1 as SecretHashedValue, TOKEN_TRANSIENT_STATUSES$1 as TOKEN_TRANSIENT_STATUSES, TRIGGER_TRANSIENT_STATUSES$1 as TRIGGER_TRANSIENT_STATUSES, type Token$1 as Token, type TokenStatus$1 as TokenStatus, type Trigger$1 as Trigger, type TriggerInputType$1 as TriggerInputType, type TriggerMnqNatsClientConfig$1 as TriggerMnqNatsClientConfig, type TriggerMnqSqsClientConfig$1 as TriggerMnqSqsClientConfig, type TriggerSqsClientConfig$1 as TriggerSqsClientConfig, type TriggerStatus$1 as TriggerStatus, type index_gen$o_UpdateContainerRequest as UpdateContainerRequest, type UpdateCronRequest$1 as UpdateCronRequest, type UpdateNamespaceRequest$2 as UpdateNamespaceRequest, type UpdateTriggerRequest$3 as UpdateTriggerRequest, type UpdateTriggerRequestSqsClientConfig$1 as UpdateTriggerRequestSqsClientConfig, validationRules_gen$b as ValidationRules };
}

declare namespace index$r {
  export { index_gen$o as v1beta1 };
}

type ACLRuleAction$1 = 'allow' | 'deny';
type ACLRuleDirection$1 = 'inbound' | 'outbound';
type ACLRuleProtocol$1 = 'tcp' | 'udp' | 'icmp';
type EngineSettingPropertyType$1 = 'BOOLEAN' | 'INT' | 'STRING' | 'FLOAT';
type InstanceLogStatus$1 = 'unknown' | 'ready' | 'creating' | 'error';
type InstanceStatus$2 = 'unknown' | 'ready' | 'provisioning' | 'configuring' | 'deleting' | 'error' | 'autohealing' | 'locked' | 'initializing' | 'disk_full' | 'backuping' | 'snapshotting' | 'restarting';
type ListDatabasesRequestOrderBy$2 = 'name_asc' | 'name_desc' | 'size_asc' | 'size_desc';
type ListInstanceLogsRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc';
type ListInstancesRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc' | 'region' | 'status_asc' | 'status_desc';
type ListPrivilegesRequestOrderBy$1 = 'user_name_asc' | 'user_name_desc' | 'database_name_asc' | 'database_name_desc';
type ListSnapshotsRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc' | 'expires_at_asc' | 'expires_at_desc';
type ListUsersRequestOrderBy$2 = 'name_asc' | 'name_desc' | 'is_admin_asc' | 'is_admin_desc';
type MaintenanceStatus$1 = 'unknown' | 'pending' | 'done' | 'canceled';
type NodeTypeGeneration$1 = 'unknown_generation' | 'generation_v1' | 'generation_v2';
type NodeTypeStock$2 = 'unknown' | 'low_stock' | 'out_of_stock' | 'available';
type Permission$1 = 'readonly' | 'readwrite' | 'all' | 'custom' | 'none';
type ReadReplicaStatus$1 = 'unknown' | 'provisioning' | 'initializing' | 'ready' | 'deleting' | 'error' | 'locked' | 'configuring' | 'promoting';
type SnapshotStatus$1 = 'unknown' | 'creating' | 'ready' | 'restoring' | 'deleting' | 'error' | 'locked';
type StorageClass$1 = 'unknown_storage_class' | 'lssd' | 'bssd' | 'sbs';
type VolumeType$2 = 'lssd' | 'bssd' | 'sbs_5k' | 'sbs_15k';
interface EndpointDirectAccessDetails$1 {
}
interface EndpointLoadBalancerDetails$1 {
}
interface EndpointPrivateNetworkDetails$1 {
    /** UUID of the Private Network. */
    privateNetworkId: string;
    /** CIDR notation of the endpoint IPv4 address. */
    serviceIp: string;
    /** Private network zone. */
    zone: Zone;
}
interface EndpointSpecPrivateNetworkIpamConfig$1 {
}
interface ReadReplicaEndpointSpecPrivateNetworkIpamConfig$1 {
}
interface EngineSetting$1 {
    /** Setting name from the database engine. */
    name: string;
    /** Value set when not specified. */
    defaultValue: string;
    /** Setting can be applied without restarting. */
    hotConfigurable: boolean;
    /** Setting description. */
    description: string;
    /** Setting type. */
    propertyType: EngineSettingPropertyType$1;
    /** Setting base unit. */
    unit?: string;
    /** Validation regex for string type settings. */
    stringConstraint?: string;
    /** Minimum value for int types. */
    intMin?: number;
    /** Maximum value for int types. */
    intMax?: number;
    /** Minimum value for float types. */
    floatMin?: number;
    /** Maximum value for float types. */
    floatMax?: number;
}
interface Endpoint$2 {
    /** UUID of the endpoint. */
    id: string;
    /**
     * IPv4 address of the endpoint.
     *
     * One-of ('address'): at most one of 'ip', 'hostname' could be set.
     */
    ip?: string;
    /** TCP port of the endpoint. */
    port: number;
    /** Name of the endpoint. */
    name?: string;
    /**
     * Private Network details. One maximum per Database Instance or Read Replica
     * (a Database Instance and its Read Replica can have different Private
     * Networks). Cannot be updated (has to be deleted and recreated).
     *
     * One-of ('details'): at most one of 'privateNetwork', 'loadBalancer',
     * 'directAccess' could be set.
     */
    privateNetwork?: EndpointPrivateNetworkDetails$1;
    /**
     * Load Balancer details. Public endpoint for Database Instance which is
     * systematically present. One per Database Instance.
     *
     * One-of ('details'): at most one of 'privateNetwork', 'loadBalancer',
     * 'directAccess' could be set.
     */
    loadBalancer?: EndpointLoadBalancerDetails$1;
    /**
     * Direct access details. Public endpoint reserved for Read Replicas. One per
     * Read Replica.
     *
     * One-of ('details'): at most one of 'privateNetwork', 'loadBalancer',
     * 'directAccess' could be set.
     */
    directAccess?: EndpointDirectAccessDetails$1;
    /**
     * Hostname of the endpoint.
     *
     * One-of ('address'): at most one of 'ip', 'hostname' could be set.
     */
    hostname?: string;
}
interface EndpointSpecLoadBalancer$1 {
}
interface EndpointSpecPrivateNetwork$1 {
    /** UUID of the Private Network to be connected to the Database Instance. */
    privateNetworkId: string;
    /**
     * Endpoint IPv4 address with a CIDR notation. Refer to the official Scaleway
     * documentation to learn more about IP and subnet limitations.
     *
     * One-of ('config'): at most one of 'serviceIp', 'ipamConfig' could be set.
     */
    serviceIp?: string;
    /**
     * Automated configuration of your Private Network endpoint with Scaleway IPAM
     * service. One at the most per Database Instance or Read Replica (a Database
     * Instance and its Read Replica can have different Private Networks). Cannot
     * be updated (has to be deleted and recreated).
     *
     * One-of ('config'): at most one of 'serviceIp', 'ipamConfig' could be set.
     */
    ipamConfig?: EndpointSpecPrivateNetworkIpamConfig$1;
}
interface ReadReplicaEndpointSpecDirectAccess$1 {
}
interface ReadReplicaEndpointSpecPrivateNetwork$1 {
    /** UUID of the Private Network to be connected to the Read Replica. */
    privateNetworkId: string;
    /**
     * Endpoint IPv4 address with a CIDR notation. Refer to the official Scaleway
     * documentation to learn more about IP and subnet limitations.
     *
     * One-of ('config'): at most one of 'serviceIp', 'ipamConfig' could be set.
     */
    serviceIp?: string;
    /**
     * Automated configuration of your Private Network endpoint with Scaleway IPAM
     * service. One at the most per Database Instance or Read Replica (a Database
     * Instance and its Read Replica can have different private networks). Cannot
     * be updated (has to be deleted and recreated).
     *
     * One-of ('config'): at most one of 'serviceIp', 'ipamConfig' could be set.
     */
    ipamConfig?: ReadReplicaEndpointSpecPrivateNetworkIpamConfig$1;
}
interface EngineVersion$1 {
    /** Database engine version. */
    version: string;
    /** Database engine name. */
    name: string;
    /** End of life date. */
    endOfLife?: Date;
    /** Engine settings available to be set. */
    availableSettings: EngineSetting$1[];
    /** Disabled versions cannot be created. */
    disabled: boolean;
    /** Beta status of engine version. */
    beta: boolean;
    /** Engine settings available to be set at database initialization. */
    availableInitSettings: EngineSetting$1[];
}
interface BackupSchedule$1 {
    /** Frequency of the backup schedule (in hours). */
    frequency: number;
    /** Default retention period of backups (in days). */
    retention: number;
    /** Defines whether the backup schedule feature is disabled. */
    disabled: boolean;
    /** Next run of the backup schedule (accurate to 10 minutes). */
    nextRunAt?: Date;
}
interface InstanceSetting$1 {
    name: string;
    value: string;
}
interface LogsPolicy$1 {
    /** Max age (in days) of remote logs to keep on the Database Instance. */
    maxAgeRetention?: number;
    /** Max disk size of remote logs to keep on the Database Instance. */
    totalDiskRetention?: number;
}
interface Maintenance$1 {
    /** Start date of the maintenance window. */
    startsAt?: Date;
    /** End date of the maintenance window. */
    stopsAt?: Date;
    /** Closed maintenance date. */
    closedAt?: Date;
    /** Maintenance information message. */
    reason: string;
    /** Status of the maintenance. */
    status: MaintenanceStatus$1;
}
interface ReadReplica$1 {
    /** UUID of the Read Replica. */
    id: string;
    /** Display Read Replica connection information. */
    endpoints: Endpoint$2[];
    /** Read replica status. */
    status: ReadReplicaStatus$1;
    /** Region the Read Replica is in. */
    region: Region;
    /**
     * Whether the replica is in the same Availability Zone as the main Database
     * Instance nodes or not.
     */
    sameZone: boolean;
}
interface UpgradableVersion$1 {
    id: string;
    name: string;
    version: string;
    minorVersion: string;
}
interface Volume$3 {
    type: VolumeType$2;
    size: number;
    class: StorageClass$1;
}
interface NodeTypeVolumeConstraintSizes$1 {
    /** [deprecated] Mimimum size required for the Volume. */
    minSize: number;
    /** [deprecated] Maximum size required for the Volume. */
    maxSize: number;
}
interface NodeTypeVolumeType$1 {
    /** Volume Type. */
    type: VolumeType$2;
    /** The description of the Volume. */
    description: string;
    /** Mimimum size required for the Volume. */
    minSize: number;
    /** Maximum size required for the Volume. */
    maxSize: number;
    /** Minimum increment level for a Block Storage volume size. */
    chunkSize: number;
    /** The storage class of the volume. */
    class: StorageClass$1;
}
interface SnapshotVolumeType$2 {
    type: VolumeType$2;
    class: StorageClass$1;
}
interface ACLRuleRequest$1 {
    ip: string;
    description: string;
}
interface ACLRule$2 {
    ip: string;
    /** @deprecated */
    port?: number;
    protocol: ACLRuleProtocol$1;
    direction: ACLRuleDirection$1;
    action: ACLRuleAction$1;
    description: string;
}
interface EndpointSpec$2 {
    /**
     * Load Balancer endpoint specifications. Public endpoint for Database
     * Instance which is systematically present. One per Document Database
     * Instance.
     *
     * One-of ('spec'): at most one of 'loadBalancer', 'privateNetwork' could be
     * set.
     */
    loadBalancer?: EndpointSpecLoadBalancer$1;
    /**
     * Private Network endpoint specifications. One maximum per Database Instance
     * or Read Replica (a Database Instance and its Read Replica can have
     * different Private Networks). Cannot be updated (has to be deleted and
     * recreated).
     *
     * One-of ('spec'): at most one of 'loadBalancer', 'privateNetwork' could be
     * set.
     */
    privateNetwork?: EndpointSpecPrivateNetwork$1;
}
interface ReadReplicaEndpointSpec$1 {
    /**
     * Direct access endpoint specifications. Public endpoint reserved for Read
     * Replicas. One per Read Replica.
     *
     * One-of ('spec'): at most one of 'directAccess', 'privateNetwork' could be
     * set.
     */
    directAccess?: ReadReplicaEndpointSpecDirectAccess$1;
    /**
     * Private Network endpoint specifications. One at most, per Read Replica.
     * Cannot be updated (has to be deleted and recreated).
     *
     * One-of ('spec'): at most one of 'directAccess', 'privateNetwork' could be
     * set.
     */
    privateNetwork?: ReadReplicaEndpointSpecPrivateNetwork$1;
}
interface DatabaseEngine$1 {
    /** Engine name. */
    name: string;
    /** Engine logo URL. */
    logoUrl: string;
    /** Available versions. */
    versions: EngineVersion$1[];
    /** Region of this Database Instance. */
    region: Region;
}
interface Database$3 {
    /** Name of the database. */
    name: string;
    /** Name of the database owner. */
    owner: string;
    /** Defines whether the database is managed or not. */
    managed: boolean;
    /** Size of the database. */
    size: number;
}
interface ListInstanceLogsDetailsResponseInstanceLogDetail$1 {
    logName: string;
    size: number;
}
interface InstanceLog$1 {
    /** Presigned S3 URL to download your log file. */
    downloadUrl?: string;
    /** UUID of the Database Instance log. */
    id: string;
    /** Status of the logs in a Database Instance. */
    status: InstanceLogStatus$1;
    /** Name of the underlying node. */
    nodeName: string;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
    /** Creation date (must follow the ISO 8601 format). */
    createdAt?: Date;
    /** Region the Database Instance is in. */
    region: Region;
}
interface Instance$2 {
    /** Creation date (must follow the ISO 8601 format). */
    createdAt?: Date;
    /** Volumes of the Database Instance. */
    volume?: Volume$3;
    /** Region the Database Instance is in. */
    region: Region;
    /** UUID of the Database Instance. */
    id: string;
    /** Name of the Database Instance. */
    name: string;
    /** Organization ID the Database Instance belongs to. */
    organizationId: string;
    /** Project ID the Database Instance belongs to. */
    projectId: string;
    /** Status of the Database Instance. */
    status: InstanceStatus$2;
    /** Database engine of the database. */
    engine: string;
    /** Available database engine versions for upgrade. */
    upgradableVersion: UpgradableVersion$1[];
    /** @deprecated Endpoint of the Database Instance. */
    endpoint?: Endpoint$2;
    /** List of tags applied to the Database Instance. */
    tags: string[];
    /** Advanced settings of the Database Instance. */
    settings: InstanceSetting$1[];
    /** Backup schedule of the Database Instance. */
    backupSchedule?: BackupSchedule$1;
    /** Defines whether or not High-Availability is enabled. */
    isHaCluster: boolean;
    /** Read Replicas of the Database Instance. */
    readReplicas: ReadReplica$1[];
    /** Node type of the Database Instance. */
    nodeType: string;
    /** List of engine settings to be set at Database Instance initialization. */
    initSettings: InstanceSetting$1[];
    /** List of Database Instance endpoints. */
    endpoints: Endpoint$2[];
    /** Logs policy of the Database Instance. */
    logsPolicy?: LogsPolicy$1;
    /** Store logical backups in the same region as the Database Instance. */
    backupSameRegion: boolean;
    /** List of Database Instance maintenance events. */
    maintenances: Maintenance$1[];
}
interface NodeType$2 {
    /** Node Type name identifier. */
    name: string;
    /** Current stock status for the Node Type. */
    stockStatus: NodeTypeStock$2;
    /** Current specs of the offer. */
    description: string;
    /** Number of virtual CPUs. */
    vcpus: number;
    /** Quantity of RAM. */
    memory: number;
    /** @deprecated {undefined} Deprecated Node Type volume constraints. */
    volumeConstraint?: NodeTypeVolumeConstraintSizes$1;
    /** @deprecated The Node Type is compliant with Block Storage. */
    isBssdCompatible?: boolean;
    /** The Node Type is currently disabled. */
    disabled: boolean;
    /** The Node Type is currently in beta. */
    beta: boolean;
    /** Available storage options for the Node Type. */
    availableVolumeTypes: NodeTypeVolumeType$1[];
    /** The Node Type can be used only with the High Availability option. */
    isHaRequired: boolean;
    /** Generation associated the NodeType offer. */
    generation: NodeTypeGeneration$1;
    /** Instance range associated with the NodeType offer. */
    instanceRange: string;
    /** Region the Node Type is in. */
    region: Region;
}
interface Privilege$1 {
    /** Permission (Read, Read/Write, All, Custom). */
    permission: Permission$1;
    /** Name of the database. */
    databaseName: string;
    /** Name of the user. */
    userName: string;
}
interface Snapshot$2 {
    /** UUID of the snapshot. */
    id: string;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the snapshot. */
    name: string;
    /** Status of the snapshot. */
    status: SnapshotStatus$1;
    /** Size of the snapshot. */
    size?: number;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
    /** Creation date (must follow the ISO 8601 format). */
    createdAt?: Date;
    /** Updated date (must follow the ISO 8601 format). */
    updatedAt?: Date;
    /** Name of the Database Instance of the snapshot. */
    instanceName: string;
    /** Source node type. */
    nodeType: string;
    /** Type of volume where data is stored (lssd, bssd or sbs). */
    volumeType?: SnapshotVolumeType$2;
    /** Region of this snapshot. */
    region: Region;
}
interface User$2 {
    /**
     * Name of the user (Length must be between 1 and 63 characters. First
     * character must be an alphabet character (a-zA-Z). Your username cannot
     * start with '_rdb' or 'pg_'. Only a-zA-Z0-9_$- characters are accepted).
     */
    name: string;
    /**
     * Defines whether or not a user got administrative privileges on the Database
     * Instance.
     */
    isAdmin: boolean;
}
interface UpgradeInstanceRequestMajorUpgradeWorkflow$1 {
    /**
     * This will create a new Database Instance with same specifications as the
     * current one and perform a Database Engine upgrade.
     */
    upgradableVersionId: string;
    /**
     * At the end of the migration procedure this option let you migrate all your
     * database endpoint to the upgraded instance.
     */
    withEndpoints: boolean;
}
type AddInstanceACLRulesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to add ACL rules to. */
    instanceId: string;
    /** ACL rules to add to the Database Instance. */
    rules: ACLRuleRequest$1[];
};
interface AddInstanceACLRulesResponse$1 {
    /** ACL Rules enabled for the Database Instance. */
    rules: ACLRule$2[];
}
type AddInstanceSettingsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to add settings to. */
    instanceId: string;
    /** Settings to add to the Database Instance. */
    settings: InstanceSetting$1[];
};
interface AddInstanceSettingsResponse$1 {
    /** Settings available on the Database Instance. */
    settings: InstanceSetting$1[];
}
type CloneInstanceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to clone. */
    instanceId: string;
    /** Name of the Database Instance clone. */
    name: string;
    /** Node type of the clone. */
    nodeType?: string;
};
type CreateDatabaseRequest$3 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance where to create the database. */
    instanceId: string;
    /** Name of the database. */
    name: string;
};
type CreateEndpointRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you to which you want to add an endpoint. */
    instanceId: string;
    /** Specification of the endpoint you want to create. */
    endpointSpec?: EndpointSpec$2;
};
type CreateInstanceFromSnapshotRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Block snapshot of the Database Instance. */
    snapshotId: string;
    /** Name of the Database Instance created with the snapshot. */
    instanceName: string;
    /**
     * Defines whether or not High Availability is enabled on the new Database
     * Instance.
     */
    isHaCluster?: boolean;
    /** The node type used to restore the snapshot. */
    nodeType?: string;
};
type CreateInstanceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * @deprecated Please use project_id instead.
     *
     *   One-of ('projectIdentifier'): at most one of 'organizationId', 'projectId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * The Project ID on which the Database Instance will be created.
     *
     * One-of ('projectIdentifier'): at most one of 'organizationId', 'projectId'
     * could be set.
     */
    projectId?: string;
    /** Name of the Database Instance. */
    name?: string;
    /** Database engine of the Database Instance. */
    engine: string;
    /** Username created when the Database Instance is created. */
    userName: string;
    /** Password of the user. */
    password: string;
    /** Type of node to use for the Database Instance. */
    nodeType: string;
    /** Defines whether or not High-Availability is enabled. */
    isHaCluster: boolean;
    /** Defines whether or not backups are disabled. */
    disableBackup: boolean;
    /** Tags to apply to the Database Instance. */
    tags?: string[];
    /** List of engine settings to be set upon Database Instance initialization. */
    initSettings?: InstanceSetting$1[];
    /** Type of volume where data is stored (lssd, bssd, ...). */
    volumeType?: VolumeType$2;
    /** Volume size when volume_type is not lssd. */
    volumeSize: number;
    /**
     * One or multiple EndpointSpec used to expose your Database Instance. A
     * load_balancer public endpoint is systematically created.
     */
    initEndpoints?: EndpointSpec$2[];
    /**
     * Defines whether to or not to store logical backups in the same region as
     * the Database Instance.
     */
    backupSameRegion: boolean;
};
type CreateReadReplicaEndpointRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
    /** Specification of the endpoint you want to create. */
    endpointSpec: ReadReplicaEndpointSpec$1[];
};
type CreateReadReplicaRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to create a Read Replica from. */
    instanceId: string;
    /** Specification of the endpoint you want to create. */
    endpointSpec?: ReadReplicaEndpointSpec$1[];
    /**
     * Defines whether or not to create the replica in the same Availability Zone
     * as the main Database Instance nodes.
     */
    sameZone?: boolean;
};
type CreateSnapshotRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the snapshot. */
    name?: string;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
};
type CreateUserRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance in which you want to create a user. */
    instanceId: string;
    /** Name of the user you want to create. */
    name: string;
    /** Password of the user you want to create. */
    password: string;
    /** Defines whether the user will have administrative privileges. */
    isAdmin: boolean;
};
type DeleteDatabaseRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance where to delete the database. */
    instanceId: string;
    /** Name of the database to delete. */
    name: string;
};
type DeleteEndpointRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** This endpoint can also be used to delete a Read Replica endpoint. */
    endpointId: string;
};
type DeleteInstanceACLRulesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to delete an ACL rule from. */
    instanceId: string;
    /** IP addresses defined in the ACL rules of the Database Instance. */
    aclRuleIps: string[];
};
interface DeleteInstanceACLRulesResponse$1 {
    /** IP addresses defined in the ACL rules of the Database Instance. */
    rules: ACLRule$2[];
}
type DeleteInstanceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to delete. */
    instanceId: string;
};
type DeleteInstanceSettingsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to delete settings from. */
    instanceId: string;
    /** Settings names to delete. */
    settingNames: string[];
};
interface DeleteInstanceSettingsResponse$1 {
    /** Settings names to delete from the Database Instance. */
    settings: InstanceSetting$1[];
}
type DeleteReadReplicaRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
};
type DeleteSnapshotRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the snapshot to delete. */
    snapshotId: string;
};
type DeleteUserRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to delete the user from. */
    instanceId: string;
    /** Name of the user. */
    name: string;
};
type GetEndpointRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the endpoint you want to get. */
    endpointId: string;
};
type GetInstanceCertificateRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
};
type GetInstanceLogRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the instance_log you want. */
    instanceLogId: string;
};
type GetInstanceMetricsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Start date to gather metrics from. */
    startDate?: Date;
    /** End date to gather metrics from. */
    endDate?: Date;
    /** Name of the metric to gather. */
    metricName?: string;
};
type GetInstanceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
};
type GetReadReplicaRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
};
type GetSnapshotRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the snapshot. */
    snapshotId: string;
};
interface InstanceMetrics$1 {
    /** Time series of metrics of a Database Instance. */
    timeseries: TimeSeries[];
}
type ListDatabaseEnginesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the database engine. */
    name?: string;
    /** Version of the database engine. */
    version?: string;
    page?: number;
    pageSize?: number;
};
interface ListDatabaseEnginesResponse$1 {
    /** List of the available database engines. */
    engines: DatabaseEngine$1[];
    /** Total count of database engines available. */
    totalCount: number;
}
type ListDatabasesRequest$3 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to list the databases of. */
    instanceId: string;
    /** Name of the database. */
    name?: string;
    /** Defines whether or not the database is managed. */
    managed?: boolean;
    /** User that owns this database. */
    owner?: string;
    /** Criteria to use when ordering database listing. */
    orderBy?: ListDatabasesRequestOrderBy$2;
    page?: number;
    pageSize?: number;
};
interface ListDatabasesResponse$2 {
    /** List of the databases. */
    databases: Database$3[];
    /** Total count of databases present on a Database Instance. */
    totalCount: number;
}
type ListInstanceACLRulesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    page?: number;
    pageSize?: number;
};
interface ListInstanceACLRulesResponse$1 {
    /** List of ACL rules present on a Database Instance. */
    rules: ACLRule$2[];
    /** Total count of ACL rules present on a Database Instance. */
    totalCount: number;
}
type ListInstanceLogsDetailsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
};
interface ListInstanceLogsDetailsResponse$1 {
    /** Remote Database Instance logs details. */
    details: ListInstanceLogsDetailsResponseInstanceLogDetail$1[];
}
type ListInstanceLogsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
    /** Criteria to use when ordering Database Instance logs listing. */
    orderBy?: ListInstanceLogsRequestOrderBy$1;
};
interface ListInstanceLogsResponse$1 {
    /** Available logs in a Database Instance. */
    instanceLogs: InstanceLog$1[];
}
type ListInstancesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** List Database Instances that have a given tag. */
    tags?: string[];
    /** Lists Database Instances that match a name pattern. */
    name?: string;
    /** Criteria to use when ordering Database Instance listings. */
    orderBy?: ListInstancesRequestOrderBy$1;
    /** Please use project_id instead. */
    organizationId?: string;
    /** Project ID to list the Database Instance of. */
    projectId?: string;
    page?: number;
    pageSize?: number;
};
interface ListInstancesResponse$1 {
    /** List of all Database Instances available in an Organization or Project. */
    instances: Instance$2[];
    /** Total count of Database Instances available in a Organization or Project. */
    totalCount: number;
}
type ListNodeTypesRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Defines whether or not to include disabled types. */
    includeDisabledTypes: boolean;
    page?: number;
    pageSize?: number;
};
interface ListNodeTypesResponse$2 {
    /** Types of the node. */
    nodeTypes: NodeType$2[];
    /** Total count of node-types available. */
    totalCount: number;
}
type ListPrivilegesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Criteria to use when ordering privileges listing. */
    orderBy?: ListPrivilegesRequestOrderBy$1;
    page?: number;
    pageSize?: number;
    /** Name of the database. */
    databaseName?: string;
    /** Name of the user. */
    userName?: string;
};
interface ListPrivilegesResponse$1 {
    /** Privileges of a user in a database in a Database Instance. */
    privileges: Privilege$1[];
    /** Total count of privileges present on a database. */
    totalCount: number;
}
type ListSnapshotsRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the snapshot. */
    name?: string;
    /** Criteria to use when ordering snapshot listing. */
    orderBy?: ListSnapshotsRequestOrderBy$1;
    /** UUID of the Database Instance. */
    instanceId?: string;
    /** Organization ID the snapshots belongs to. */
    organizationId?: string;
    /** Project ID the snapshots belongs to. */
    projectId?: string;
    page?: number;
    pageSize?: number;
};
interface ListSnapshotsResponse$2 {
    /** List of snapshots. */
    snapshots: Snapshot$2[];
    /** Total count of snapshots available. */
    totalCount: number;
}
type ListUsersRequest$3 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the user. */
    name?: string;
    /** Criteria to use when requesting user listing. */
    orderBy?: ListUsersRequestOrderBy$2;
    page?: number;
    pageSize?: number;
};
interface ListUsersResponse$2 {
    /** List of users in a Database Instance. */
    users: User$2[];
    /** Total count of users present on a Database Instance. */
    totalCount: number;
}
type MigrateEndpointRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the endpoint you want to migrate. */
    endpointId: string;
    /** UUID of the instance you want to attach the endpoint to. */
    instanceId: string;
};
type PromoteReadReplicaRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
};
type PurgeInstanceLogsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
    /** Given log name to purge. */
    logName?: string;
};
type RenewInstanceCertificateRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
};
type ResetReadReplicaRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
};
type RestartInstanceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to restart. */
    instanceId: string;
};
type SetInstanceACLRulesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance where the ACL rules must be set. */
    instanceId: string;
    /** ACL rules to define for the Database Instance. */
    rules: ACLRuleRequest$1[];
};
interface SetInstanceACLRulesResponse$1 {
    /** ACLs rules configured for a Database Instance. */
    rules: ACLRule$2[];
}
type SetInstanceSettingsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance where the settings must be set. */
    instanceId: string;
    /** Settings to define for the Database Instance. */
    settings: InstanceSetting$1[];
};
interface SetInstanceSettingsResponse$1 {
    /** Settings configured for a Database Instance. */
    settings: InstanceSetting$1[];
}
type SetPrivilegeRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the database. */
    databaseName: string;
    /** Name of the user. */
    userName: string;
    /** Permission to set (Read, Read/Write, All, Custom). */
    permission?: Permission$1;
};
type UpdateInstanceRequest$3 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to update. */
    instanceId: string;
    /** In hours. */
    backupScheduleFrequency?: number;
    /** In days. */
    backupScheduleRetention?: number;
    /** Defines whether or not the backup schedule is disabled. */
    isBackupScheduleDisabled?: boolean;
    /** Name of the Database Instance. */
    name?: string;
    /** Tags of a Database Instance. */
    tags?: string[];
    /** Logs policy of the Database Instance. */
    logsPolicy?: LogsPolicy$1;
    /** Store logical backups in the same region as the Database Instance. */
    backupSameRegion?: boolean;
    /** Defines the start time of the autobackup. */
    backupScheduleStartHour?: number;
};
type UpdateSnapshotRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the snapshot to update. */
    snapshotId: string;
    /** Name of the snapshot. */
    name?: string;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
};
type UpdateUserRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance the user belongs to. */
    instanceId: string;
    /** Name of the database user. */
    name: string;
    /** Password of the database user. */
    password?: string;
    /** Defines whether or not this user got administrative privileges. */
    isAdmin?: boolean;
};
type UpgradeInstanceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to upgrade. */
    instanceId: string;
    /**
     * Node type of the Database Instance you want to upgrade to.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    nodeType?: string;
    /**
     * Defines whether or not High Availability should be enabled on the Database
     * Instance.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    enableHa?: boolean;
    /**
     * Increase your Block volume size.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    volumeSize?: number;
    /**
     * Change your Database Instance storage type.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    volumeType?: VolumeType$2;
    /**
     * This will create a new Database Instance with same specifications as the
     * current one and perform a Database Engine upgrade.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    upgradableVersionId?: string;
    /**
     * Upgrade your database engine to a new major version including instance
     * endpoints.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    majorUpgradeWorkflow?: UpgradeInstanceRequestMajorUpgradeWorkflow$1;
};

/** Managed Document Databases API. */
declare class API$o extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListDatabaseEngines: (request?: Readonly<ListDatabaseEnginesRequest$1>) => Promise<ListDatabaseEnginesResponse$1>;
    /**
     * List available database engines. List the FerretDB database engines
     * available at Scaleway.
     *
     * @param request - The request {@link ListDatabaseEnginesRequest}
     * @returns A Promise of ListDatabaseEnginesResponse
     */
    listDatabaseEngines: (request?: Readonly<ListDatabaseEnginesRequest$1>) => Promise<ListDatabaseEnginesResponse$1> & {
        all: () => Promise<DatabaseEngine$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DatabaseEngine$1[], void, void>;
    };
    protected pageOfListNodeTypes: (request: Readonly<ListNodeTypesRequest$2>) => Promise<ListNodeTypesResponse$2>;
    /**
     * List available node types. List all available node types. By default, the
     * node types returned in the list are ordered by creation date in ascending
     * order, though this can be modified via the `order_by` field.
     *
     * @param request - The request {@link ListNodeTypesRequest}
     * @returns A Promise of ListNodeTypesResponse
     */
    listNodeTypes: (request: Readonly<ListNodeTypesRequest$2>) => Promise<ListNodeTypesResponse$2> & {
        all: () => Promise<NodeType$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<NodeType$2[], void, void>;
    };
    /**
     * Upgrade a Database Instance. Upgrade your current Database Instance
     * specifications like node type, high availability, volume, or the database
     * engine version. Note that upon upgrade the `enable_ha` parameter can only
     * be set to `true`.
     *
     * @param request - The request {@link UpgradeInstanceRequest}
     * @returns A Promise of Instance
     */
    upgradeInstance: (request: Readonly<UpgradeInstanceRequest$1>) => Promise<Instance$2>;
    protected pageOfListInstances: (request?: Readonly<ListInstancesRequest$1>) => Promise<ListInstancesResponse$1>;
    /**
     * List Database Instances. List all Database Instances in the specified
     * region, for a given Scaleway Organization or Scaleway Project. By default,
     * the Database Instances returned in the list are ordered by creation date in
     * ascending order, though this can be modified via the order_by field. You
     * can define additional parameters for your query, such as `tags` and `name`.
     * For the `name` parameter, the value you include will be checked against the
     * whole name string to see if it includes the string you put in the
     * parameter.
     *
     * @param request - The request {@link ListInstancesRequest}
     * @returns A Promise of ListInstancesResponse
     */
    listInstances: (request?: Readonly<ListInstancesRequest$1>) => Promise<ListInstancesResponse$1> & {
        all: () => Promise<Instance$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Instance$2[], void, void>;
    };
    /**
     * Get a Database Instance. Retrieve information about a given Database
     * Instance, specified by the `region` and `instance_id` parameters. Its full
     * details, including name, status, IP address and port, are returned in the
     * response object.
     *
     * @param request - The request {@link GetInstanceRequest}
     * @returns A Promise of Instance
     */
    getInstance: (request: Readonly<GetInstanceRequest$1>) => Promise<Instance$2>;
    /**
     * Waits for {@link Instance} to be in a final state.
     *
     * @param request - The request {@link GetInstanceRequest}
     * @param options - The waiting options
     * @returns A Promise of Instance
     */
    waitForInstance: (request: Readonly<GetInstanceRequest$1>, options?: Readonly<WaitForOptions<Instance$2>>) => Promise<Instance$2>;
    /**
     * Create a Database Instance. Create a new Database Instance. You must set
     * the `engine`, `user_name`, `password` and `node_type` parameters.
     * Optionally, you can specify the volume type and size.
     *
     * @param request - The request {@link CreateInstanceRequest}
     * @returns A Promise of Instance
     */
    createInstance: (request: Readonly<CreateInstanceRequest$1>) => Promise<Instance$2>;
    /**
     * Update a Database Instance. Update the parameters of a Database Instance,
     * including name, tags and backup schedule details.
     *
     * @param request - The request {@link UpdateInstanceRequest}
     * @returns A Promise of Instance
     */
    updateInstance: (request: Readonly<UpdateInstanceRequest$3>) => Promise<Instance$2>;
    /**
     * Delete a Database Instance. Delete a given Database Instance, specified by
     * the `region` and `instance_id` parameters. Deleting a Database Instance is
     * permanent, and cannot be undone. Note that upon deletion all your data will
     * be lost.
     *
     * @param request - The request {@link DeleteInstanceRequest}
     * @returns A Promise of Instance
     */
    deleteInstance: (request: Readonly<DeleteInstanceRequest$1>) => Promise<Instance$2>;
    /**
     * Clone a Database Instance. Clone a given Database Instance, specified by
     * the `region` and `instance_id` parameters. The clone feature allows you to
     * create a new Database Instance from an existing one. The clone includes all
     * existing databases, users and permissions. You can create a clone on a
     * Database Instance bigger than your current one.
     *
     * @param request - The request {@link CloneInstanceRequest}
     * @returns A Promise of Instance
     */
    cloneInstance: (request: Readonly<CloneInstanceRequest$1>) => Promise<Instance$2>;
    /**
     * Restart Database Instance. Restart a given Database Instance, specified by
     * the `region` and `instance_id` parameters. The status of the Database
     * Instance returned in the response.
     *
     * @param request - The request {@link RestartInstanceRequest}
     * @returns A Promise of Instance
     */
    restartInstance: (request: Readonly<RestartInstanceRequest$1>) => Promise<Instance$2>;
    /**
     * Get the TLS certificate of a Database Instance. Retrieve information about
     * the TLS certificate of a given Database Instance. Details like name and
     * content are returned in the response.
     *
     * @param request - The request {@link GetInstanceCertificateRequest}
     * @returns A Promise of Blob
     */
    getInstanceCertificate: (request: Readonly<GetInstanceCertificateRequest$1>) => Promise<Blob>;
    /**
     * Renew the TLS certificate of a Database Instance. Renew a TLS for a
     * Database Instance. Renewing a certificate means that you will not be able
     * to connect to your Database Instance using the previous certificate. You
     * will also need to download and update the new certificate for all database
     * clients.
     *
     * @param request - The request {@link RenewInstanceCertificateRequest}
     */
    renewInstanceCertificate: (request: Readonly<RenewInstanceCertificateRequest$1>) => Promise<void>;
    /**
     * Get Database Instance metrics. Retrieve the time series metrics of a given
     * Database Instance. You can define the period from which to retrieve metrics
     * by specifying the `start_date` and `end_date`.
     *
     * @param request - The request {@link GetInstanceMetricsRequest}
     * @returns A Promise of InstanceMetrics
     */
    getInstanceMetrics: (request: Readonly<GetInstanceMetricsRequest$1>) => Promise<InstanceMetrics$1>;
    /**
     * Create a Read Replica. Create a new Read Replica of a Database Instance.
     * You must specify the `region` and the `instance_id`. You can only create a
     * maximum of 3 Read Replicas per Database Instance.
     *
     * @param request - The request {@link CreateReadReplicaRequest}
     * @returns A Promise of ReadReplica
     */
    createReadReplica: (request: Readonly<CreateReadReplicaRequest$1>) => Promise<ReadReplica$1>;
    /**
     * Get a Read Replica. Retrieve information about a Database Instance Read
     * Replica. Full details about the Read Replica, like `endpoints`, `status`
     * and `region` are returned in the response.
     *
     * @param request - The request {@link GetReadReplicaRequest}
     * @returns A Promise of ReadReplica
     */
    getReadReplica: (request: Readonly<GetReadReplicaRequest$1>) => Promise<ReadReplica$1>;
    /**
     * Waits for {@link ReadReplica} to be in a final state.
     *
     * @param request - The request {@link GetReadReplicaRequest}
     * @param options - The waiting options
     * @returns A Promise of ReadReplica
     */
    waitForReadReplica: (request: Readonly<GetReadReplicaRequest$1>, options?: Readonly<WaitForOptions<ReadReplica$1>>) => Promise<ReadReplica$1>;
    /**
     * Delete a Read Replica. Delete a Read Replica of a Database Instance. You
     * must specify the `region` and `read_replica_id` parameters of the Read
     * Replica you want to delete.
     *
     * @param request - The request {@link DeleteReadReplicaRequest}
     * @returns A Promise of ReadReplica
     */
    deleteReadReplica: (request: Readonly<DeleteReadReplicaRequest$1>) => Promise<ReadReplica$1>;
    /**
     * Resync a Read Replica. When you resync a Read Replica, first it is reset,
     * then its data is resynchronized from the primary node. Your Read Replica
     * remains unavailable during the resync process. The duration of this process
     * is proportional to the size of your Database Instance. The configured
     * endpoints do not change.
     *
     * @param request - The request {@link ResetReadReplicaRequest}
     * @returns A Promise of ReadReplica
     */
    resetReadReplica: (request: Readonly<ResetReadReplicaRequest$1>) => Promise<ReadReplica$1>;
    /**
     * Promote a Read Replica. Promote a Read Replica to Database Instance
     * automatically.
     *
     * @param request - The request {@link PromoteReadReplicaRequest}
     * @returns A Promise of Instance
     */
    promoteReadReplica: (request: Readonly<PromoteReadReplicaRequest$1>) => Promise<Instance$2>;
    /**
     * Create an endpoint for a Read Replica. Create a new endpoint for a Read
     * Replica. Read Replicas can have at most one direct access and one Private
     * Network endpoint.
     *
     * @param request - The request {@link CreateReadReplicaEndpointRequest}
     * @returns A Promise of ReadReplica
     */
    createReadReplicaEndpoint: (request: Readonly<CreateReadReplicaEndpointRequest$1>) => Promise<ReadReplica$1>;
    /**
     * List available logs of a Database Instance. List the available logs of a
     * Database Instance. By default, the logs returned in the list are ordered by
     * creation date in ascending order, though this can be modified via the
     * order_by field.
     *
     * @param request - The request {@link ListInstanceLogsRequest}
     * @returns A Promise of ListInstanceLogsResponse
     */
    listInstanceLogs: (request: Readonly<ListInstanceLogsRequest$1>) => Promise<ListInstanceLogsResponse$1>;
    /**
     * Get given logs of a Database Instance. Retrieve information about the logs
     * of a Database Instance. Specify the `instance_log_id` and `region` in your
     * request to get information such as `download_url`, `status`, `expires_at`
     * and `created_at` about your logs in the response.
     *
     * @param request - The request {@link GetInstanceLogRequest}
     * @returns A Promise of InstanceLog
     */
    getInstanceLog: (request: Readonly<GetInstanceLogRequest$1>) => Promise<InstanceLog$1>;
    /**
     * Waits for {@link InstanceLog} to be in a final state.
     *
     * @param request - The request {@link GetInstanceLogRequest}
     * @param options - The waiting options
     * @returns A Promise of InstanceLog
     */
    waitForInstanceLog: (request: Readonly<GetInstanceLogRequest$1>, options?: Readonly<WaitForOptions<InstanceLog$1>>) => Promise<InstanceLog$1>;
    /**
     * Purge remote Database Instance logs. Purge a given remote log from a
     * Database Instance. You can specify the `log_name` of the log you wish to
     * clean from your Database Instance.
     *
     * @param request - The request {@link PurgeInstanceLogsRequest}
     */
    purgeInstanceLogs: (request: Readonly<PurgeInstanceLogsRequest$1>) => Promise<void>;
    /**
     * List remote Database Instance logs details. List remote log details. By
     * default, the details returned in the list are ordered by creation date in
     * ascending order, though this can be modified via the order_by field.
     *
     * @param request - The request {@link ListInstanceLogsDetailsRequest}
     * @returns A Promise of ListInstanceLogsDetailsResponse
     */
    listInstanceLogsDetails: (request: Readonly<ListInstanceLogsDetailsRequest$1>) => Promise<ListInstanceLogsDetailsResponse$1>;
    /**
     * Add Database Instance advanced settings. Add an advanced setting to a
     * Database Instance. You must set the `name` and the `value` of each
     * setting.
     *
     * @param request - The request {@link AddInstanceSettingsRequest}
     * @returns A Promise of AddInstanceSettingsResponse
     */
    addInstanceSettings: (request: Readonly<AddInstanceSettingsRequest$1>) => Promise<AddInstanceSettingsResponse$1>;
    /**
     * Delete Database Instance advanced settings. Delete an advanced setting in a
     * Database Instance. You must specify the names of the settings you want to
     * delete in the request.
     *
     * @param request - The request {@link DeleteInstanceSettingsRequest}
     * @returns A Promise of DeleteInstanceSettingsResponse
     */
    deleteInstanceSettings: (request: Readonly<DeleteInstanceSettingsRequest$1>) => Promise<DeleteInstanceSettingsResponse$1>;
    /**
     * Set Database Instance advanced settings. Update an advanced setting for a
     * Database Instance. Settings added upon database engine initalization can
     * only be defined once, and cannot, therefore, be updated.
     *
     * @param request - The request {@link SetInstanceSettingsRequest}
     * @returns A Promise of SetInstanceSettingsResponse
     */
    setInstanceSettings: (request: Readonly<SetInstanceSettingsRequest$1>) => Promise<SetInstanceSettingsResponse$1>;
    protected pageOfListInstanceACLRules: (request: Readonly<ListInstanceACLRulesRequest$1>) => Promise<ListInstanceACLRulesResponse$1>;
    /**
     * List ACL rules of a Database Instance. List the ACL rules for a given
     * Database Instance. The response is an array of ACL objects, each one
     * representing an ACL that denies, allows or redirects traffic based on
     * certain conditions.
     *
     * @param request - The request {@link ListInstanceACLRulesRequest}
     * @returns A Promise of ListInstanceACLRulesResponse
     */
    listInstanceACLRules: (request: Readonly<ListInstanceACLRulesRequest$1>) => Promise<ListInstanceACLRulesResponse$1> & {
        all: () => Promise<ACLRule$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ACLRule$2[], void, void>;
    };
    /**
     * Add an ACL rule to a Database Instance. Add an additional ACL rule to a
     * Database Instance.
     *
     * @param request - The request {@link AddInstanceACLRulesRequest}
     * @returns A Promise of AddInstanceACLRulesResponse
     */
    addInstanceACLRules: (request: Readonly<AddInstanceACLRulesRequest$1>) => Promise<AddInstanceACLRulesResponse$1>;
    /**
     * Set ACL rules for a Database Instance. Replace all the ACL rules of a
     * Database Instance.
     *
     * @param request - The request {@link SetInstanceACLRulesRequest}
     * @returns A Promise of SetInstanceACLRulesResponse
     */
    setInstanceACLRules: (request: Readonly<SetInstanceACLRulesRequest$1>) => Promise<SetInstanceACLRulesResponse$1>;
    /**
     * Delete ACL rules of a Database Instance. Delete one or more ACL rules of a
     * Database Instance.
     *
     * @param request - The request {@link DeleteInstanceACLRulesRequest}
     * @returns A Promise of DeleteInstanceACLRulesResponse
     */
    deleteInstanceACLRules: (request: Readonly<DeleteInstanceACLRulesRequest$1>) => Promise<DeleteInstanceACLRulesResponse$1>;
    protected pageOfListUsers: (request: Readonly<ListUsersRequest$3>) => Promise<ListUsersResponse$2>;
    /**
     * List users of a Database Instance. List all users of a given Database
     * Instance. By default, the users returned in the list are ordered by
     * creation date in ascending order, though this can be modified via the
     * order_by field.
     *
     * @param request - The request {@link ListUsersRequest}
     * @returns A Promise of ListUsersResponse
     */
    listUsers: (request: Readonly<ListUsersRequest$3>) => Promise<ListUsersResponse$2> & {
        all: () => Promise<User$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<User$2[], void, void>;
    };
    /**
     * Create a user for a Database Instance. Create a new user for a Database
     * Instance. You must define the `name`, `password` and `is_admin`
     * parameters.
     *
     * @param request - The request {@link CreateUserRequest}
     * @returns A Promise of User
     */
    createUser: (request: Readonly<CreateUserRequest$2>) => Promise<User$2>;
    /**
     * Update a user on a Database Instance. Update the parameters of a user on a
     * Database Instance. You can update the `password` and `is_admin` parameters,
     * but you cannot change the name of the user.
     *
     * @param request - The request {@link UpdateUserRequest}
     * @returns A Promise of User
     */
    updateUser: (request: Readonly<UpdateUserRequest$2>) => Promise<User$2>;
    /**
     * Delete a user on a Database Instance. Delete a given user on a Database
     * Instance. You must specify, in the endpoint, the `region`, `instance_id`
     * and `name` parameters of the user you want to delete.
     *
     * @param request - The request {@link DeleteUserRequest}
     */
    deleteUser: (request: Readonly<DeleteUserRequest$2>) => Promise<void>;
    protected pageOfListDatabases: (request: Readonly<ListDatabasesRequest$3>) => Promise<ListDatabasesResponse$2>;
    /**
     * List databases in a Database Instance. List all databases of a given
     * Database Instance. By default, the databases returned in the list are
     * ordered by creation date in ascending order, though this can be modified
     * via the order_by field. You can define additional parameters for your
     * query, such as `name`, `managed` and `owner`.
     *
     * @param request - The request {@link ListDatabasesRequest}
     * @returns A Promise of ListDatabasesResponse
     */
    listDatabases: (request: Readonly<ListDatabasesRequest$3>) => Promise<ListDatabasesResponse$2> & {
        all: () => Promise<Database$3[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Database$3[], void, void>;
    };
    /**
     * Create a database in a Database Instance. Create a new database. You must
     * define the `name` parameter in the request.
     *
     * @param request - The request {@link CreateDatabaseRequest}
     * @returns A Promise of Database
     */
    createDatabase: (request: Readonly<CreateDatabaseRequest$3>) => Promise<Database$3>;
    /**
     * Delete a database in a Database Instance. Delete a given database on a
     * Database Instance. You must specify, in the endpoint, the `region`,
     * `instance_id` and `name` parameters of the database you want to delete.
     *
     * @param request - The request {@link DeleteDatabaseRequest}
     */
    deleteDatabase: (request: Readonly<DeleteDatabaseRequest$2>) => Promise<void>;
    protected pageOfListPrivileges: (request: Readonly<ListPrivilegesRequest$1>) => Promise<ListPrivilegesResponse$1>;
    /**
     * List user privileges for a database. List privileges of a user on a
     * database. By default, the details returned in the list are ordered by
     * creation date in ascending order, though this can be modified via the
     * order_by field. You can define additional parameters for your query, such
     * as `database_name` and `user_name`.
     *
     * @param request - The request {@link ListPrivilegesRequest}
     * @returns A Promise of ListPrivilegesResponse
     */
    listPrivileges: (request: Readonly<ListPrivilegesRequest$1>) => Promise<ListPrivilegesResponse$1> & {
        all: () => Promise<Privilege$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Privilege$1[], void, void>;
    };
    /**
     * Set user privileges for a database. Set the privileges of a user on a
     * database. You must define `database_name`, `user_name` and `permission` in
     * the request body.
     *
     * @param request - The request {@link SetPrivilegeRequest}
     * @returns A Promise of Privilege
     */
    setPrivilege: (request: Readonly<SetPrivilegeRequest$1>) => Promise<Privilege$1>;
    protected pageOfListSnapshots: (request?: Readonly<ListSnapshotsRequest$2>) => Promise<ListSnapshotsResponse$2>;
    /**
     * List snapshots. List snapshots. You can include the `instance_id` or
     * `project_id` in your query to get the list of snapshots for specific
     * Database Instances and/or Projects. By default, the details returned in the
     * list are ordered by creation date in ascending order, though this can be
     * modified via the `order_by` field.
     *
     * @param request - The request {@link ListSnapshotsRequest}
     * @returns A Promise of ListSnapshotsResponse
     */
    listSnapshots: (request?: Readonly<ListSnapshotsRequest$2>) => Promise<ListSnapshotsResponse$2> & {
        all: () => Promise<Snapshot$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Snapshot$2[], void, void>;
    };
    /**
     * Get a Database Instance snapshot. Retrieve information about a given
     * snapshot, specified by its `snapshot_id` and `region`. Full details about
     * the snapshot, like size and expiration date, are returned in the response.
     *
     * @param request - The request {@link GetSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    getSnapshot: (request: Readonly<GetSnapshotRequest$2>) => Promise<Snapshot$2>;
    /**
     * Waits for {@link Snapshot} to be in a final state.
     *
     * @param request - The request {@link GetSnapshotRequest}
     * @param options - The waiting options
     * @returns A Promise of Snapshot
     */
    waitForSnapshot: (request: Readonly<GetSnapshotRequest$2>, options?: Readonly<WaitForOptions<Snapshot$2>>) => Promise<Snapshot$2>;
    /**
     * Create a Database Instance snapshot. Create a new snapshot of a Database
     * Instance. You must define the `name` parameter in the request.
     *
     * @param request - The request {@link CreateSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    createSnapshot: (request: Readonly<CreateSnapshotRequest$2>) => Promise<Snapshot$2>;
    /**
     * Update a Database Instance snapshot. Update the parameters of a snapshot of
     * a Database Instance. You can update the `name` and `expires_at`
     * parameters.
     *
     * @param request - The request {@link UpdateSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    updateSnapshot: (request: Readonly<UpdateSnapshotRequest$2>) => Promise<Snapshot$2>;
    /**
     * Delete a Database Instance snapshot. Delete a given snapshot of a Database
     * Instance. You must specify, in the endpoint, the `region` and `snapshot_id`
     * parameters of the snapshot you want to delete.
     *
     * @param request - The request {@link DeleteSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    deleteSnapshot: (request: Readonly<DeleteSnapshotRequest$2>) => Promise<Snapshot$2>;
    /**
     * Create a new Database Instance from a snapshot. Restore a snapshot. When
     * you restore a snapshot, a new Instance is created and billed to your
     * account. Note that is possible to select a larger node type for your new
     * Database Instance. However, the Block volume size will be the same as the
     * size of the restored snapshot. All Instance settings will be restored if
     * you chose a node type with the same or more memory size than the initial
     * Instance. Settings will be reset to the default if your node type has less
     * memory.
     *
     * @param request - The request {@link CreateInstanceFromSnapshotRequest}
     * @returns A Promise of Instance
     */
    createInstanceFromSnapshot: (request: Readonly<CreateInstanceFromSnapshotRequest$1>) => Promise<Instance$2>;
    /**
     * Create a new Database Instance endpoint. Create a new endpoint for a
     * Database Instance. You can add `load_balancer` and `private_network`
     * specifications to the body of the request.
     *
     * @param request - The request {@link CreateEndpointRequest}
     * @returns A Promise of Endpoint
     */
    createEndpoint: (request: Readonly<CreateEndpointRequest$1>) => Promise<Endpoint$2>;
    /**
     * Delete a Database Instance endpoint. Delete the endpoint of a Database
     * Instance. You must specify the `region` and `endpoint_id` parameters of the
     * endpoint you want to delete. Note that might need to update any environment
     * configurations that point to the deleted endpoint.
     *
     * @param request - The request {@link DeleteEndpointRequest}
     */
    deleteEndpoint: (request: Readonly<DeleteEndpointRequest$2>) => Promise<void>;
    /**
     * Get a Database Instance endpoint. Retrieve information about a Database
     * Instance endpoint. Full details about the endpoint, like `ip`, `port`,
     * `private_network` and `load_balancer` specifications are returned in the
     * response.
     *
     * @param request - The request {@link GetEndpointRequest}
     * @returns A Promise of Endpoint
     */
    getEndpoint: (request: Readonly<GetEndpointRequest$2>) => Promise<Endpoint$2>;
    /**
     * Migrate Database Instance endpoint. Migrate an existing Database Instance
     * endpoint to another Database Instance.
     *
     * @param request - The request {@link MigrateEndpointRequest}
     * @returns A Promise of Endpoint
     */
    migrateEndpoint: (request: Readonly<MigrateEndpointRequest$1>) => Promise<Endpoint$2>;
}

/** Lists transient statutes of the enum {@link InstanceLogStatus}. */
declare const INSTANCE_LOG_TRANSIENT_STATUSES$1: InstanceLogStatus$1[];
/** Lists transient statutes of the enum {@link InstanceStatus}. */
declare const INSTANCE_TRANSIENT_STATUSES$2: InstanceStatus$2[];
/** Lists transient statutes of the enum {@link MaintenanceStatus}. */
declare const MAINTENANCE_TRANSIENT_STATUSES$1: MaintenanceStatus$1[];
/** Lists transient statutes of the enum {@link ReadReplicaStatus}. */
declare const READ_REPLICA_TRANSIENT_STATUSES$1: ReadReplicaStatus$1[];
/** Lists transient statutes of the enum {@link SnapshotStatus}. */
declare const SNAPSHOT_TRANSIENT_STATUSES$2: SnapshotStatus$1[];

declare const UpdateInstanceRequest$2: {
    backupScheduleStartHour: {
        lessThanOrEqual: number;
    };
};

declare namespace validationRules_gen$a {
  export { UpdateInstanceRequest$2 as UpdateInstanceRequest };
}

declare namespace index_gen$n {
  export { type ACLRule$2 as ACLRule, type ACLRuleAction$1 as ACLRuleAction, type ACLRuleDirection$1 as ACLRuleDirection, type ACLRuleProtocol$1 as ACLRuleProtocol, type ACLRuleRequest$1 as ACLRuleRequest, API$o as API, type AddInstanceACLRulesRequest$1 as AddInstanceACLRulesRequest, type AddInstanceACLRulesResponse$1 as AddInstanceACLRulesResponse, type AddInstanceSettingsRequest$1 as AddInstanceSettingsRequest, type AddInstanceSettingsResponse$1 as AddInstanceSettingsResponse, type BackupSchedule$1 as BackupSchedule, type CloneInstanceRequest$1 as CloneInstanceRequest, type CreateDatabaseRequest$3 as CreateDatabaseRequest, type CreateEndpointRequest$1 as CreateEndpointRequest, type CreateInstanceFromSnapshotRequest$1 as CreateInstanceFromSnapshotRequest, type CreateInstanceRequest$1 as CreateInstanceRequest, type CreateReadReplicaEndpointRequest$1 as CreateReadReplicaEndpointRequest, type CreateReadReplicaRequest$1 as CreateReadReplicaRequest, type CreateSnapshotRequest$2 as CreateSnapshotRequest, type CreateUserRequest$2 as CreateUserRequest, type Database$3 as Database, type DatabaseEngine$1 as DatabaseEngine, type DeleteDatabaseRequest$2 as DeleteDatabaseRequest, type DeleteEndpointRequest$2 as DeleteEndpointRequest, type DeleteInstanceACLRulesRequest$1 as DeleteInstanceACLRulesRequest, type DeleteInstanceACLRulesResponse$1 as DeleteInstanceACLRulesResponse, type DeleteInstanceRequest$1 as DeleteInstanceRequest, type DeleteInstanceSettingsRequest$1 as DeleteInstanceSettingsRequest, type DeleteInstanceSettingsResponse$1 as DeleteInstanceSettingsResponse, type DeleteReadReplicaRequest$1 as DeleteReadReplicaRequest, type DeleteSnapshotRequest$2 as DeleteSnapshotRequest, type DeleteUserRequest$2 as DeleteUserRequest, type Endpoint$2 as Endpoint, type EndpointDirectAccessDetails$1 as EndpointDirectAccessDetails, type EndpointLoadBalancerDetails$1 as EndpointLoadBalancerDetails, type EndpointPrivateNetworkDetails$1 as EndpointPrivateNetworkDetails, type EndpointSpec$2 as EndpointSpec, type EndpointSpecLoadBalancer$1 as EndpointSpecLoadBalancer, type EndpointSpecPrivateNetwork$1 as EndpointSpecPrivateNetwork, type EndpointSpecPrivateNetworkIpamConfig$1 as EndpointSpecPrivateNetworkIpamConfig, type EngineSetting$1 as EngineSetting, type EngineSettingPropertyType$1 as EngineSettingPropertyType, type EngineVersion$1 as EngineVersion, type GetEndpointRequest$2 as GetEndpointRequest, type GetInstanceCertificateRequest$1 as GetInstanceCertificateRequest, type GetInstanceLogRequest$1 as GetInstanceLogRequest, type GetInstanceMetricsRequest$1 as GetInstanceMetricsRequest, type GetInstanceRequest$1 as GetInstanceRequest, type GetReadReplicaRequest$1 as GetReadReplicaRequest, type GetSnapshotRequest$2 as GetSnapshotRequest, INSTANCE_LOG_TRANSIENT_STATUSES$1 as INSTANCE_LOG_TRANSIENT_STATUSES, INSTANCE_TRANSIENT_STATUSES$2 as INSTANCE_TRANSIENT_STATUSES, type Instance$2 as Instance, type InstanceLog$1 as InstanceLog, type InstanceLogStatus$1 as InstanceLogStatus, type InstanceMetrics$1 as InstanceMetrics, type InstanceSetting$1 as InstanceSetting, type InstanceStatus$2 as InstanceStatus, type ListDatabaseEnginesRequest$1 as ListDatabaseEnginesRequest, type ListDatabaseEnginesResponse$1 as ListDatabaseEnginesResponse, type ListDatabasesRequest$3 as ListDatabasesRequest, type ListDatabasesRequestOrderBy$2 as ListDatabasesRequestOrderBy, type ListDatabasesResponse$2 as ListDatabasesResponse, type ListInstanceACLRulesRequest$1 as ListInstanceACLRulesRequest, type ListInstanceACLRulesResponse$1 as ListInstanceACLRulesResponse, type ListInstanceLogsDetailsRequest$1 as ListInstanceLogsDetailsRequest, type ListInstanceLogsDetailsResponse$1 as ListInstanceLogsDetailsResponse, type ListInstanceLogsDetailsResponseInstanceLogDetail$1 as ListInstanceLogsDetailsResponseInstanceLogDetail, type ListInstanceLogsRequest$1 as ListInstanceLogsRequest, type ListInstanceLogsRequestOrderBy$1 as ListInstanceLogsRequestOrderBy, type ListInstanceLogsResponse$1 as ListInstanceLogsResponse, type ListInstancesRequest$1 as ListInstancesRequest, type ListInstancesRequestOrderBy$1 as ListInstancesRequestOrderBy, type ListInstancesResponse$1 as ListInstancesResponse, type ListNodeTypesRequest$2 as ListNodeTypesRequest, type ListNodeTypesResponse$2 as ListNodeTypesResponse, type ListPrivilegesRequest$1 as ListPrivilegesRequest, type ListPrivilegesRequestOrderBy$1 as ListPrivilegesRequestOrderBy, type ListPrivilegesResponse$1 as ListPrivilegesResponse, type ListSnapshotsRequest$2 as ListSnapshotsRequest, type ListSnapshotsRequestOrderBy$1 as ListSnapshotsRequestOrderBy, type ListSnapshotsResponse$2 as ListSnapshotsResponse, type ListUsersRequest$3 as ListUsersRequest, type ListUsersRequestOrderBy$2 as ListUsersRequestOrderBy, type ListUsersResponse$2 as ListUsersResponse, type LogsPolicy$1 as LogsPolicy, MAINTENANCE_TRANSIENT_STATUSES$1 as MAINTENANCE_TRANSIENT_STATUSES, type Maintenance$1 as Maintenance, type MaintenanceStatus$1 as MaintenanceStatus, type MigrateEndpointRequest$1 as MigrateEndpointRequest, type NodeType$2 as NodeType, type NodeTypeGeneration$1 as NodeTypeGeneration, type NodeTypeStock$2 as NodeTypeStock, type NodeTypeVolumeConstraintSizes$1 as NodeTypeVolumeConstraintSizes, type NodeTypeVolumeType$1 as NodeTypeVolumeType, type Permission$1 as Permission, type Privilege$1 as Privilege, type PromoteReadReplicaRequest$1 as PromoteReadReplicaRequest, type PurgeInstanceLogsRequest$1 as PurgeInstanceLogsRequest, READ_REPLICA_TRANSIENT_STATUSES$1 as READ_REPLICA_TRANSIENT_STATUSES, type ReadReplica$1 as ReadReplica, type ReadReplicaEndpointSpec$1 as ReadReplicaEndpointSpec, type ReadReplicaEndpointSpecDirectAccess$1 as ReadReplicaEndpointSpecDirectAccess, type ReadReplicaEndpointSpecPrivateNetwork$1 as ReadReplicaEndpointSpecPrivateNetwork, type ReadReplicaEndpointSpecPrivateNetworkIpamConfig$1 as ReadReplicaEndpointSpecPrivateNetworkIpamConfig, type ReadReplicaStatus$1 as ReadReplicaStatus, type RenewInstanceCertificateRequest$1 as RenewInstanceCertificateRequest, type ResetReadReplicaRequest$1 as ResetReadReplicaRequest, type RestartInstanceRequest$1 as RestartInstanceRequest, SNAPSHOT_TRANSIENT_STATUSES$2 as SNAPSHOT_TRANSIENT_STATUSES, type SetInstanceACLRulesRequest$1 as SetInstanceACLRulesRequest, type SetInstanceACLRulesResponse$1 as SetInstanceACLRulesResponse, type SetInstanceSettingsRequest$1 as SetInstanceSettingsRequest, type SetInstanceSettingsResponse$1 as SetInstanceSettingsResponse, type SetPrivilegeRequest$1 as SetPrivilegeRequest, type Snapshot$2 as Snapshot, type SnapshotStatus$1 as SnapshotStatus, type SnapshotVolumeType$2 as SnapshotVolumeType, type StorageClass$1 as StorageClass, type UpdateInstanceRequest$3 as UpdateInstanceRequest, type UpdateSnapshotRequest$2 as UpdateSnapshotRequest, type UpdateUserRequest$2 as UpdateUserRequest, type UpgradableVersion$1 as UpgradableVersion, type UpgradeInstanceRequest$1 as UpgradeInstanceRequest, type UpgradeInstanceRequestMajorUpgradeWorkflow$1 as UpgradeInstanceRequestMajorUpgradeWorkflow, type User$2 as User, validationRules_gen$a as ValidationRules, type Volume$3 as Volume, type VolumeType$2 as VolumeType };
}

declare namespace index$q {
  export { index_gen$n as v1beta1 };
}

type LanguageCode = 'unknown_language_code' | 'en_US' | 'fr_FR' | 'de_DE';

type ContactEmailStatus = 'email_status_unknown' | 'validated' | 'not_validated' | 'invalid_email';
type ContactExtensionFRMode = 'mode_unknown' | 'individual' | 'company_identification_code' | 'duns' | 'local' | 'association' | 'trademark' | 'code_auth_afnic';
type ContactExtensionNLLegalForm = 'legal_form_unknown' | 'other' | 'non_dutch_eu_company' | 'non_dutch_legal_form_enterprise_subsidiary' | 'limited_company' | 'limited_company_in_formation' | 'cooperative' | 'limited_partnership' | 'sole_company' | 'european_economic_interest_group' | 'religious_entity' | 'partnership' | 'public_company' | 'mutual_benefit_company' | 'residential' | 'shipping_company' | 'foundation' | 'association' | 'trading_partnership' | 'natural_person';
type ContactLegalForm = 'legal_form_unknown' | 'individual' | 'corporate' | 'association' | 'other';
type ContactStatus = 'status_unknown' | 'active' | 'pending';
type DNSZoneStatus = 'unknown' | 'active' | 'pending' | 'error' | 'locked';
type DSRecordAlgorithm = 'rsamd5' | 'dh' | 'dsa' | 'rsasha1' | 'dsa_nsec3_sha1' | 'rsasha1_nsec3_sha1' | 'rsasha256' | 'rsasha512' | 'ecc_gost' | 'ecdsap256sha256' | 'ecdsap384sha384' | 'ed25519' | 'ed448';
type DSRecordDigestType = 'sha_1' | 'sha_256' | 'gost_r_34_11_94' | 'sha_384';
type DomainFeatureStatus = 'feature_status_unknown' | 'enabling' | 'enabled' | 'disabling' | 'disabled';
type DomainRecordHTTPServiceConfigStrategy = 'random' | 'hashed' | 'all';
type DomainRecordType = 'unknown' | 'A' | 'AAAA' | 'CNAME' | 'TXT' | 'SRV' | 'TLSA' | 'MX' | 'NS' | 'PTR' | 'CAA' | 'ALIAS' | 'LOC' | 'SSHFP' | 'HINFO' | 'RP' | 'URI' | 'DS' | 'NAPTR' | 'DNAME';
type DomainRegistrationStatusTransferStatus = 'status_unknown' | 'pending' | 'waiting_vote' | 'rejected' | 'processing' | 'done';
type DomainStatus$2 = 'status_unknown' | 'active' | 'creating' | 'create_error' | 'renewing' | 'renew_error' | 'xfering' | 'xfer_error' | 'expired' | 'expiring' | 'updating' | 'checking' | 'locked' | 'deleting';
type HostStatus = 'unknown_status' | 'active' | 'updating' | 'deleting';
type LinkedProduct = 'unknown_product' | 'vpc';
type ListContactsRequestRole = 'unknown_role' | 'owner' | 'administrative' | 'technical';
type ListDNSZoneRecordsRequestOrderBy = 'name_asc' | 'name_desc';
type ListDNSZonesRequestOrderBy = 'domain_asc' | 'domain_desc' | 'subdomain_asc' | 'subdomain_desc' | 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc';
type ListDomainsRequestOrderBy$1 = 'domain_asc' | 'domain_desc';
type ListRenewableDomainsRequestOrderBy = 'domain_asc' | 'domain_desc';
type ListTasksRequestOrderBy = 'domain_desc' | 'domain_asc' | 'type_asc' | 'type_desc' | 'status_asc' | 'status_desc' | 'updated_at_asc' | 'updated_at_desc';
type ListTldsRequestOrderBy = 'name_asc' | 'name_desc';
type RawFormat = 'unknown_raw_format' | 'bind';
type RenewableDomainStatus = 'unknown' | 'renewable' | 'late_reneweable' | 'not_renewable';
type SSLCertificateStatus = 'unknown' | 'new' | 'pending' | 'success' | 'error';
type TaskStatus$1 = 'unavailable' | 'new' | 'waiting_payment' | 'pending' | 'success' | 'error';
type TaskType = 'unknown' | 'create_domain' | 'create_external_domain' | 'renew_domain' | 'transfer_domain' | 'trade_domain' | 'lock_domain_transfer' | 'unlock_domain_transfer' | 'enable_dnssec' | 'disable_dnssec' | 'update_domain' | 'update_contact' | 'delete_domain' | 'cancel_task' | 'generate_ssl_certificate' | 'renew_ssl_certificate' | 'send_message' | 'delete_domain_expired' | 'delete_external_domain' | 'create_host' | 'update_host' | 'delete_host' | 'move_project' | 'transfer_online_domain';
interface DomainRecordGeoIPConfigMatch {
    countries: string[];
    continents: string[];
    data: string;
}
interface DomainRecordViewConfigView {
    subnet: string;
    data: string;
}
interface DomainRecordWeightedConfigWeightedIP {
    ip: string;
    weight: number;
}
interface DSRecordPublicKey {
    key: string;
}
interface DomainRecordGeoIPConfig {
    matches: DomainRecordGeoIPConfigMatch[];
    default: string;
}
interface DomainRecordHTTPServiceConfig {
    ips: string[];
    mustContain?: string;
    url: string;
    userAgent?: string;
    strategy: DomainRecordHTTPServiceConfigStrategy;
}
interface DomainRecordViewConfig {
    views: DomainRecordViewConfigView[];
}
interface DomainRecordWeightedConfig {
    weightedIps: DomainRecordWeightedConfigWeightedIP[];
}
interface ContactExtensionFRAssociationInfo {
    publicationJo?: Date;
    publicationJoPage: number;
}
interface ContactExtensionFRCodeAuthAfnicInfo {
    codeAuthAfnic: string;
}
interface ContactExtensionFRDunsInfo {
    dunsId: string;
    localId: string;
}
interface ContactExtensionFRIndividualInfo {
    whoisOptIn: boolean;
}
interface ContactExtensionFRTrademarkInfo {
    trademarkInpi: string;
}
interface DSRecordDigest {
    type: DSRecordDigestType;
    digest: string;
    publicKey?: DSRecordPublicKey;
}
interface DomainRecord {
    data: string;
    name: string;
    priority: number;
    ttl: number;
    type: DomainRecordType;
    comment?: string;
    /**
     * One-of ('dynamicData'): at most one of 'geoIpConfig', 'httpServiceConfig',
     * 'weightedConfig', 'viewConfig' could be set.
     */
    geoIpConfig?: DomainRecordGeoIPConfig;
    /**
     * One-of ('dynamicData'): at most one of 'geoIpConfig', 'httpServiceConfig',
     * 'weightedConfig', 'viewConfig' could be set.
     */
    httpServiceConfig?: DomainRecordHTTPServiceConfig;
    /**
     * One-of ('dynamicData'): at most one of 'geoIpConfig', 'httpServiceConfig',
     * 'weightedConfig', 'viewConfig' could be set.
     */
    weightedConfig?: DomainRecordWeightedConfig;
    /**
     * One-of ('dynamicData'): at most one of 'geoIpConfig', 'httpServiceConfig',
     * 'weightedConfig', 'viewConfig' could be set.
     */
    viewConfig?: DomainRecordViewConfig;
    id: string;
}
interface RecordIdentifier {
    name: string;
    type: DomainRecordType;
    data?: string;
    ttl?: number;
}
interface ContactExtensionEU {
    europeanCitizenship: string;
}
interface ContactExtensionFR {
    mode: ContactExtensionFRMode;
    /**
     * One-of ('modeFields'): at most one of 'individualInfo', 'dunsInfo',
     * 'associationInfo', 'trademarkInfo', 'codeAuthAfnicInfo' could be set.
     */
    individualInfo?: ContactExtensionFRIndividualInfo;
    /**
     * One-of ('modeFields'): at most one of 'individualInfo', 'dunsInfo',
     * 'associationInfo', 'trademarkInfo', 'codeAuthAfnicInfo' could be set.
     */
    dunsInfo?: ContactExtensionFRDunsInfo;
    /**
     * One-of ('modeFields'): at most one of 'individualInfo', 'dunsInfo',
     * 'associationInfo', 'trademarkInfo', 'codeAuthAfnicInfo' could be set.
     */
    associationInfo?: ContactExtensionFRAssociationInfo;
    /**
     * One-of ('modeFields'): at most one of 'individualInfo', 'dunsInfo',
     * 'associationInfo', 'trademarkInfo', 'codeAuthAfnicInfo' could be set.
     */
    trademarkInfo?: ContactExtensionFRTrademarkInfo;
    /**
     * One-of ('modeFields'): at most one of 'individualInfo', 'dunsInfo',
     * 'associationInfo', 'trademarkInfo', 'codeAuthAfnicInfo' could be set.
     */
    codeAuthAfnicInfo?: ContactExtensionFRCodeAuthAfnicInfo;
}
interface ContactExtensionNL {
    legalForm: ContactExtensionNLLegalForm;
    legalFormRegistrationNumber: string;
}
interface ContactQuestion {
    question: string;
    answer: string;
}
interface TldOffer {
    action: string;
    operationPath: string;
    price?: Money;
}
interface DSRecord {
    keyId: number;
    algorithm: DSRecordAlgorithm;
    /** One-of ('type'): at most one of 'digest', 'publicKey' could be set. */
    digest?: DSRecordDigest;
    /** One-of ('type'): at most one of 'digest', 'publicKey' could be set. */
    publicKey?: DSRecordPublicKey;
}
interface RecordChangeAdd {
    records: DomainRecord[];
}
interface RecordChangeClear {
}
interface RecordChangeDelete {
    /** One-of ('identifier'): at most one of 'id', 'idFields' could be set. */
    id?: string;
    /** One-of ('identifier'): at most one of 'id', 'idFields' could be set. */
    idFields?: RecordIdentifier;
}
interface RecordChangeSet {
    /** One-of ('identifier'): at most one of 'id', 'idFields' could be set. */
    id?: string;
    /** One-of ('identifier'): at most one of 'id', 'idFields' could be set. */
    idFields?: RecordIdentifier;
    records: DomainRecord[];
}
interface ImportRawDNSZoneRequestTsigKey {
    name: string;
    key: string;
    algorithm: string;
}
interface Contact {
    id: string;
    legalForm: ContactLegalForm;
    firstname: string;
    lastname: string;
    companyName: string;
    email: string;
    emailAlt: string;
    phoneNumber: string;
    faxNumber: string;
    addressLine1: string;
    addressLine2: string;
    zip: string;
    city: string;
    country: string;
    vatIdentificationCode: string;
    companyIdentificationCode: string;
    lang: LanguageCode;
    resale: boolean;
    /** @deprecated */
    questions?: ContactQuestion[];
    extensionFr?: ContactExtensionFR;
    extensionEu?: ContactExtensionEU;
    whoisOptIn: boolean;
    emailStatus: ContactEmailStatus;
    state: string;
    extensionNl?: ContactExtensionNL;
    status: ContactStatus;
}
interface ContactRolesRoles {
    isOwner: boolean;
    isAdministrative: boolean;
    isTechnical: boolean;
}
interface DomainRegistrationStatusExternalDomain {
    validationToken: string;
}
interface DomainRegistrationStatusTransfer {
    status: DomainRegistrationStatusTransferStatus;
    voteCurrentOwner: boolean;
    voteNewOwner: boolean;
}
interface Tld {
    name: string;
    dnssecSupport: boolean;
    durationInYearsMin: number;
    durationInYearsMax: number;
    idnSupport: boolean;
    offers: Record<string, TldOffer>;
    specifications: Record<string, string>;
}
interface NewContact {
    legalForm: ContactLegalForm;
    firstname: string;
    lastname: string;
    companyName?: string;
    email: string;
    emailAlt?: string;
    phoneNumber: string;
    faxNumber?: string;
    addressLine1: string;
    addressLine2?: string;
    zip: string;
    city: string;
    country: string;
    vatIdentificationCode?: string;
    companyIdentificationCode?: string;
    lang: LanguageCode;
    resale: boolean;
    /** @deprecated */
    questions?: ContactQuestion[];
    extensionFr?: ContactExtensionFR;
    extensionEu?: ContactExtensionEU;
    whoisOptIn: boolean;
    state?: string;
    extensionNl?: ContactExtensionNL;
}
interface CheckContactsCompatibilityResponseContactCheckResult {
    compatible: boolean;
    errorMessage?: string;
}
interface DNSZone {
    domain: string;
    subdomain: string;
    ns: string[];
    nsDefault: string[];
    nsMaster: string[];
    status: DNSZoneStatus;
    message?: string;
    updatedAt?: Date;
    projectId: string;
    linkedProducts: LinkedProduct[];
}
interface DomainDNSSEC {
    status: DomainFeatureStatus;
    dsRecords: DSRecord[];
}
interface RecordChange {
    /**
     * One-of ('change'): at most one of 'add', 'set', 'delete', 'clear' could be
     * set.
     */
    add?: RecordChangeAdd;
    /**
     * One-of ('change'): at most one of 'add', 'set', 'delete', 'clear' could be
     * set.
     */
    set?: RecordChangeSet;
    /**
     * One-of ('change'): at most one of 'add', 'set', 'delete', 'clear' could be
     * set.
     */
    delete?: RecordChangeDelete;
    /**
     * One-of ('change'): at most one of 'add', 'set', 'delete', 'clear' could be
     * set.
     */
    clear?: RecordChangeClear;
}
interface ImportProviderDNSZoneRequestOnlineV1 {
    token: string;
}
interface ImportRawDNSZoneRequestAXFRSource {
    nameServer: string;
    tsigKey?: ImportRawDNSZoneRequestTsigKey;
}
interface ImportRawDNSZoneRequestBindSource {
    content: string;
}
interface ContactRoles {
    contact?: Contact;
    roles: Record<string, ContactRolesRoles>;
}
interface Nameserver$1 {
    name: string;
    ip: string[];
}
interface DNSZoneVersion {
    id: string;
    createdAt?: Date;
}
interface Host {
    domain: string;
    name: string;
    ips: string[];
    status: HostStatus;
}
interface DomainSummary {
    domain: string;
    projectId: string;
    autoRenewStatus: DomainFeatureStatus;
    dnssecStatus: DomainFeatureStatus;
    eppCode: string[];
    expiredAt?: Date;
    updatedAt?: Date;
    registrar: string;
    isExternal: boolean;
    status: DomainStatus$2;
    /**
     * One-of ('registrationStatus'): at most one of
     * 'externalDomainRegistrationStatus', 'transferRegistrationStatus' could be
     * set.
     */
    externalDomainRegistrationStatus?: DomainRegistrationStatusExternalDomain;
    /**
     * One-of ('registrationStatus'): at most one of
     * 'externalDomainRegistrationStatus', 'transferRegistrationStatus' could be
     * set.
     */
    transferRegistrationStatus?: DomainRegistrationStatusTransfer;
    organizationId: string;
}
interface RenewableDomain {
    domain: string;
    projectId: string;
    organizationId: string;
    status: RenewableDomainStatus;
    renewableDurationInYears?: number;
    expiredAt?: Date;
    limitRenewAt?: Date;
    limitRedemptionAt?: Date;
    estimatedDeleteAt?: Date;
    tld?: Tld;
}
interface SSLCertificate {
    dnsZone: string;
    alternativeDnsZones: string[];
    status: SSLCertificateStatus;
    privateKey: string;
    certificateChain: string;
    createdAt?: Date;
    expiredAt?: Date;
}
interface Task$1 {
    /** The unique identifier of the task. */
    id: string;
    /** The project ID associated to the task. */
    projectId: string;
    /** The organization ID associated to the task. */
    organizationId: string;
    /** The domain name associated to the task. */
    domain?: string;
    /** The type of the task. */
    type: TaskType;
    /** The status of the task. */
    status: TaskStatus$1;
    /** Start date of the task. */
    startedAt?: Date;
    /** Last update of the task. */
    updatedAt?: Date;
    /** Error message associated to the task. */
    message?: string;
    /** Human-friendly contact identifier used when the task concerns a contact. */
    contactIdentifier?: string;
}
interface TransferInDomainRequestTransferRequest {
    domain: string;
    authCode: string;
}
interface UpdateContactRequestQuestion {
    question?: string;
    answer?: string;
}
interface AvailableDomain {
    domain: string;
    available: boolean;
    tld?: Tld;
}
interface CheckContactsCompatibilityResponse {
    compatible: boolean;
    ownerCheckResult?: CheckContactsCompatibilityResponseContactCheckResult;
    administrativeCheckResult?: CheckContactsCompatibilityResponseContactCheckResult;
    technicalCheckResult?: CheckContactsCompatibilityResponseContactCheckResult;
}
type ClearDNSZoneRecordsRequest = {
    /** DNS zone to clear. */
    dnsZone: string;
};
interface ClearDNSZoneRecordsResponse {
}
type CloneDNSZoneRequest = {
    /** DNS zone to clone. */
    dnsZone: string;
    /** Destination DNS zone in which to clone the chosen DNS zone. */
    destDnsZone: string;
    /** Specifies whether or not the destination DNS zone will be overwritten. */
    overwrite: boolean;
    /** Project ID of the destination DNS zone. */
    projectId?: string;
};
type CreateDNSZoneRequest = {
    /** Domain in which to crreate the DNS zone. */
    domain: string;
    /** Subdomain of the DNS zone to create. */
    subdomain: string;
    /** Project ID in which to create the DNS zone. */
    projectId?: string;
};
type CreateSSLCertificateRequest = {
    dnsZone: string;
    alternativeDnsZones?: string[];
};
type DeleteDNSZoneRequest = {
    /** DNS zone to delete. */
    dnsZone: string;
    /** Project ID of the DNS zone to delete. */
    projectId?: string;
};
interface DeleteDNSZoneResponse {
}
type DeleteDNSZoneTsigKeyRequest = {
    dnsZone: string;
};
interface DeleteExternalDomainResponse {
}
type DeleteSSLCertificateRequest = {
    dnsZone: string;
};
interface DeleteSSLCertificateResponse {
}
interface Domain$2 {
    domain: string;
    organizationId: string;
    projectId: string;
    autoRenewStatus: DomainFeatureStatus;
    dnssec?: DomainDNSSEC;
    eppCode: string[];
    expiredAt?: Date;
    updatedAt?: Date;
    registrar: string;
    isExternal: boolean;
    status: DomainStatus$2;
    dnsZones: DNSZone[];
    ownerContact?: Contact;
    technicalContact?: Contact;
    administrativeContact?: Contact;
    /**
     * One-of ('registrationStatus'): at most one of
     * 'externalDomainRegistrationStatus', 'transferRegistrationStatus' could be
     * set.
     */
    externalDomainRegistrationStatus?: DomainRegistrationStatusExternalDomain;
    /**
     * One-of ('registrationStatus'): at most one of
     * 'externalDomainRegistrationStatus', 'transferRegistrationStatus' could be
     * set.
     */
    transferRegistrationStatus?: DomainRegistrationStatusTransfer;
    tld?: Tld;
    linkedProducts: LinkedProduct[];
}
type ExportRawDNSZoneRequest = {
    /** DNS zone to export. */
    dnsZone: string;
    /** DNS zone format. */
    format?: RawFormat;
};
type GetDNSZoneTsigKeyRequest = {
    dnsZone: string;
};
interface GetDNSZoneTsigKeyResponse {
    name: string;
    key: string;
    algorithm: string;
}
type GetDNSZoneVersionDiffRequest = {
    dnsZoneVersionId: string;
};
interface GetDNSZoneVersionDiffResponse {
    changes: RecordChange[];
}
interface GetDomainAuthCodeResponse {
    authCode: string;
}
type GetSSLCertificateRequest = {
    dnsZone: string;
};
type ImportProviderDNSZoneRequest = {
    dnsZone: string;
    /** One-of ('provider'): at most one of 'onlineV1' could be set. */
    onlineV1?: ImportProviderDNSZoneRequestOnlineV1;
};
interface ImportProviderDNSZoneResponse {
    records: DomainRecord[];
}
type ImportRawDNSZoneRequest = {
    /** DNS zone to import. */
    dnsZone: string;
    /** @deprecated */
    content?: string;
    projectId?: string;
    /** @deprecated */
    format?: RawFormat;
    /**
     * Import a bind file format.
     *
     * One-of ('source'): at most one of 'bindSource', 'axfrSource' could be set.
     */
    bindSource?: ImportRawDNSZoneRequestBindSource;
    /**
     * Import from the name server given with TSIG, to use or not.
     *
     * One-of ('source'): at most one of 'bindSource', 'axfrSource' could be set.
     */
    axfrSource?: ImportRawDNSZoneRequestAXFRSource;
};
interface ImportRawDNSZoneResponse {
    records: DomainRecord[];
}
interface ListContactsResponse {
    totalCount: number;
    contacts: ContactRoles[];
}
type ListDNSZoneNameserversRequest = {
    /** DNS zone on which to filter the returned DNS zone name servers. */
    dnsZone: string;
    /** Project ID on which to filter the returned DNS zone name servers. */
    projectId?: string;
};
interface ListDNSZoneNameserversResponse {
    /** DNS zone name servers returned. */
    ns: Nameserver$1[];
}
type ListDNSZoneRecordsRequest = {
    /** DNS zone on which to filter the returned DNS zone records. */
    dnsZone: string;
    /** Project ID on which to filter the returned DNS zone records. */
    projectId?: string;
    /** Sort order of the returned DNS zone records. */
    orderBy?: ListDNSZoneRecordsRequestOrderBy;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Maximum number of DNS zone records per page. */
    pageSize?: number;
    /** Name on which to filter the returned DNS zone records. */
    name: string;
    /** Record type on which to filter the returned DNS zone records. */
    type?: DomainRecordType;
    /** Record ID on which to filter the returned DNS zone records. */
    id?: string;
};
interface ListDNSZoneRecordsResponse {
    /** Total number of DNS zone records. */
    totalCount: number;
    /** Paginated returned DNS zone records. */
    records: DomainRecord[];
}
type ListDNSZoneVersionRecordsRequest = {
    dnsZoneVersionId: string;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Maximum number of DNS zones versions records per page. */
    pageSize?: number;
};
interface ListDNSZoneVersionRecordsResponse {
    /** Total number of DNS zones versions records. */
    totalCount: number;
    records: DomainRecord[];
}
type ListDNSZoneVersionsRequest = {
    dnsZone: string;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Maximum number of DNS zones versions per page. */
    pageSize?: number;
};
interface ListDNSZoneVersionsResponse {
    /** Total number of DNS zones versions. */
    totalCount: number;
    versions: DNSZoneVersion[];
}
type ListDNSZonesRequest = {
    /** Organization ID on which to filter the returned DNS zones. */
    organizationId?: string;
    /** Project ID on which to filter the returned DNS zones. */
    projectId?: string;
    /** Sort order of the returned DNS zones. */
    orderBy?: ListDNSZonesRequestOrderBy;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Maximum number of DNS zones to return per page. */
    pageSize?: number;
    /** Domain on which to filter the returned DNS zones. */
    domain: string;
    /** @deprecated DNS zone on which to filter the returned DNS zones. */
    dnsZone?: string;
    /** DNS zones on which to filter the returned DNS zones. */
    dnsZones?: string[];
    /** Only list DNS zones created after this date. */
    createdAfter?: Date;
    /** Only list DNS zones created before this date. */
    createdBefore?: Date;
    /** Only list DNS zones updated after this date. */
    updatedAfter?: Date;
    /** Only list DNS zones updated before this date. */
    updatedBefore?: Date;
};
interface ListDNSZonesResponse {
    /** Total number of DNS zones matching the requested criteria. */
    totalCount: number;
    /** Paginated returned DNS zones. */
    dnsZones: DNSZone[];
}
interface ListDomainHostsResponse {
    totalCount: number;
    hosts: Host[];
}
interface ListDomainsResponse$2 {
    totalCount: number;
    domains: DomainSummary[];
}
interface ListRenewableDomainsResponse {
    totalCount: number;
    domains: RenewableDomain[];
}
type ListSSLCertificatesRequest = {
    dnsZone: string;
    page?: number;
    pageSize?: number;
    projectId?: string;
};
interface ListSSLCertificatesResponse {
    totalCount: number;
    certificates: SSLCertificate[];
}
interface ListTasksResponse {
    totalCount: number;
    tasks: Task$1[];
}
interface ListTldsResponse {
    /** Array of TLDs. */
    tlds: Tld[];
    /** Total count of TLDs returned. */
    totalCount: number;
}
interface OrderResponse {
    domains: string[];
    organizationId: string;
    projectId: string;
    taskId: string;
    createdAt?: Date;
}
type RefreshDNSZoneRequest = {
    /** DNS zone to refresh. */
    dnsZone: string;
    /** Specifies whether or not to recreate the DNS zone. */
    recreateDnsZone: boolean;
    /** Specifies whether or not to recreate the sub DNS zone. */
    recreateSubDnsZone: boolean;
};
interface RefreshDNSZoneResponse {
    /** DNS zones returned. */
    dnsZones: DNSZone[];
}
interface RegisterExternalDomainResponse {
    domain: string;
    organizationId: string;
    validationToken: string;
    createdAt?: Date;
    projectId: string;
}
type RegistrarApiBuyDomainsRequest = {
    domains: string[];
    durationInYears: number;
    projectId?: string;
    /**
     * One-of ('ownerContactType'): at most one of 'ownerContactId',
     * 'ownerContact' could be set.
     */
    ownerContactId?: string;
    /**
     * One-of ('ownerContactType'): at most one of 'ownerContactId',
     * 'ownerContact' could be set.
     */
    ownerContact?: NewContact;
    /**
     * One-of ('administrativeContactType'): at most one of
     * 'administrativeContactId', 'administrativeContact' could be set.
     */
    administrativeContactId?: string;
    /**
     * One-of ('administrativeContactType'): at most one of
     * 'administrativeContactId', 'administrativeContact' could be set.
     */
    administrativeContact?: NewContact;
    /**
     * One-of ('technicalContactType'): at most one of 'technicalContactId',
     * 'technicalContact' could be set.
     */
    technicalContactId?: string;
    /**
     * One-of ('technicalContactType'): at most one of 'technicalContactId',
     * 'technicalContact' could be set.
     */
    technicalContact?: NewContact;
};
type RegistrarApiCheckContactsCompatibilityRequest = {
    domains?: string[];
    tlds?: string[];
    /**
     * One-of ('ownerContactType'): at most one of 'ownerContactId',
     * 'ownerContact' could be set.
     */
    ownerContactId?: string;
    /**
     * One-of ('ownerContactType'): at most one of 'ownerContactId',
     * 'ownerContact' could be set.
     */
    ownerContact?: NewContact;
    /**
     * One-of ('administrativeContactType'): at most one of
     * 'administrativeContactId', 'administrativeContact' could be set.
     */
    administrativeContactId?: string;
    /**
     * One-of ('administrativeContactType'): at most one of
     * 'administrativeContactId', 'administrativeContact' could be set.
     */
    administrativeContact?: NewContact;
    /**
     * One-of ('technicalContactType'): at most one of 'technicalContactId',
     * 'technicalContact' could be set.
     */
    technicalContactId?: string;
    /**
     * One-of ('technicalContactType'): at most one of 'technicalContactId',
     * 'technicalContact' could be set.
     */
    technicalContact?: NewContact;
};
type RegistrarApiCreateDomainHostRequest = {
    domain: string;
    name: string;
    ips?: string[];
};
type RegistrarApiDeleteDomainHostRequest = {
    domain: string;
    name: string;
};
type RegistrarApiDeleteExternalDomainRequest = {
    domain: string;
};
type RegistrarApiDisableDomainAutoRenewRequest = {
    domain: string;
};
type RegistrarApiDisableDomainDNSSECRequest = {
    domain: string;
};
type RegistrarApiEnableDomainAutoRenewRequest = {
    domain: string;
};
type RegistrarApiEnableDomainDNSSECRequest = {
    domain: string;
    dsRecord?: DSRecord;
};
type RegistrarApiGetContactRequest = {
    contactId: string;
};
type RegistrarApiGetDomainAuthCodeRequest = {
    domain: string;
};
type RegistrarApiGetDomainRequest = {
    domain: string;
};
type RegistrarApiListContactsRequest = {
    page?: number;
    pageSize?: number;
    domain?: string;
    projectId?: string;
    organizationId?: string;
    role?: ListContactsRequestRole;
    emailStatus?: ContactEmailStatus;
};
type RegistrarApiListDomainHostsRequest = {
    domain: string;
    page?: number;
    pageSize?: number;
};
type RegistrarApiListDomainsRequest = {
    page?: number;
    pageSize?: number;
    orderBy?: ListDomainsRequestOrderBy$1;
    registrar?: string;
    status?: DomainStatus$2;
    projectId?: string;
    organizationId?: string;
    isExternal?: boolean;
    domain?: string;
};
type RegistrarApiListRenewableDomainsRequest = {
    page?: number;
    pageSize?: number;
    orderBy?: ListRenewableDomainsRequestOrderBy;
    projectId?: string;
    organizationId?: string;
};
type RegistrarApiListTasksRequest = {
    page?: number;
    pageSize?: number;
    projectId?: string;
    organizationId?: string;
    domain?: string;
    types?: TaskType[];
    statuses?: TaskStatus$1[];
    orderBy?: ListTasksRequestOrderBy;
};
type RegistrarApiListTldsRequest = {
    /** Array of TLDs to return. */
    tlds?: string[];
    /** Page number for the returned Projects. */
    page?: number;
    /** Maximum number of Project per page. */
    pageSize?: number;
    /** Sort order of the returned TLDs. */
    orderBy?: ListTldsRequestOrderBy;
};
type RegistrarApiLockDomainTransferRequest = {
    domain: string;
};
type RegistrarApiRegisterExternalDomainRequest = {
    domain: string;
    projectId?: string;
};
type RegistrarApiRenewDomainsRequest = {
    domains: string[];
    durationInYears: number;
    forceLateRenewal?: boolean;
};
type RegistrarApiSearchAvailableDomainsRequest = {
    /** A list of domain to search, TLD is optional. */
    domains: string[];
    /** Array of tlds to search on. */
    tlds?: string[];
    /** Search exact match. */
    strictSearch: boolean;
};
type RegistrarApiTradeDomainRequest = {
    domain: string;
    projectId?: string;
    /**
     * One-of ('newOwnerContactType'): at most one of 'newOwnerContactId',
     * 'newOwnerContact' could be set.
     */
    newOwnerContactId?: string;
    /**
     * One-of ('newOwnerContactType'): at most one of 'newOwnerContactId',
     * 'newOwnerContact' could be set.
     */
    newOwnerContact?: NewContact;
};
type RegistrarApiTransferInDomainRequest = {
    domains: TransferInDomainRequestTransferRequest[];
    projectId?: string;
    /**
     * One-of ('ownerContactType'): at most one of 'ownerContactId',
     * 'ownerContact' could be set.
     */
    ownerContactId?: string;
    /**
     * One-of ('ownerContactType'): at most one of 'ownerContactId',
     * 'ownerContact' could be set.
     */
    ownerContact?: NewContact;
    /**
     * One-of ('administrativeContactType'): at most one of
     * 'administrativeContactId', 'administrativeContact' could be set.
     */
    administrativeContactId?: string;
    /**
     * One-of ('administrativeContactType'): at most one of
     * 'administrativeContactId', 'administrativeContact' could be set.
     */
    administrativeContact?: NewContact;
    /**
     * One-of ('technicalContactType'): at most one of 'technicalContactId',
     * 'technicalContact' could be set.
     */
    technicalContactId?: string;
    /**
     * One-of ('technicalContactType'): at most one of 'technicalContactId',
     * 'technicalContact' could be set.
     */
    technicalContact?: NewContact;
};
type RegistrarApiUnlockDomainTransferRequest = {
    domain: string;
};
type RegistrarApiUpdateContactRequest = {
    contactId: string;
    email?: string;
    emailAlt?: string;
    phoneNumber?: string;
    faxNumber?: string;
    addressLine1?: string;
    addressLine2?: string;
    zip?: string;
    city?: string;
    country?: string;
    vatIdentificationCode?: string;
    companyIdentificationCode?: string;
    lang?: LanguageCode;
    resale?: boolean;
    /** @deprecated */
    questions?: UpdateContactRequestQuestion[];
    extensionFr?: ContactExtensionFR;
    extensionEu?: ContactExtensionEU;
    whoisOptIn?: boolean;
    state?: string;
    extensionNl?: ContactExtensionNL;
};
type RegistrarApiUpdateDomainHostRequest = {
    domain: string;
    name: string;
    ips?: string[];
};
type RegistrarApiUpdateDomainRequest = {
    domain: string;
    /**
     * One-of ('technicalContactInfo'): at most one of 'technicalContactId',
     * 'technicalContact' could be set.
     */
    technicalContactId?: string;
    /**
     * One-of ('technicalContactInfo'): at most one of 'technicalContactId',
     * 'technicalContact' could be set.
     */
    technicalContact?: NewContact;
    /**
     * @deprecated
     *
     *   One-of ('ownerContactInfo'): at most one of 'ownerContactId',
     *   'ownerContact' could be set.
     */
    ownerContactId?: string;
    /**
     * @deprecated
     *
     *   One-of ('ownerContactInfo'): at most one of 'ownerContactId',
     *   'ownerContact' could be set.
     */
    ownerContact?: NewContact;
    /**
     * One-of ('administrativeContactInfo'): at most one of
     * 'administrativeContactId', 'administrativeContact' could be set.
     */
    administrativeContactId?: string;
    /**
     * One-of ('administrativeContactInfo'): at most one of
     * 'administrativeContactId', 'administrativeContact' could be set.
     */
    administrativeContact?: NewContact;
};
type RestoreDNSZoneVersionRequest = {
    dnsZoneVersionId: string;
};
interface RestoreDNSZoneVersionResponse {
}
interface SearchAvailableDomainsResponse {
    /** Array of available domains. */
    availableDomains: AvailableDomain[];
}
type UpdateDNSZoneNameserversRequest = {
    /** DNS zone in which to update the DNS zone name servers. */
    dnsZone: string;
    /** New DNS zone name servers. */
    ns: Nameserver$1[];
};
interface UpdateDNSZoneNameserversResponse {
    /** DNS zone name servers returned. */
    ns: Nameserver$1[];
}
type UpdateDNSZoneRecordsRequest = {
    /** DNS zone in which to update the DNS zone records. */
    dnsZone: string;
    /** Changes made to the records. */
    changes: RecordChange[];
    /** Specifies whether or not to return all the records. */
    returnAllRecords?: boolean;
    /**
     * Disable the creation of the target zone if it does not exist. Target zone
     * creation is disabled by default.
     */
    disallowNewZoneCreation: boolean;
    /** Use the provided serial (0) instead of the auto-increment serial. */
    serial?: number;
};
interface UpdateDNSZoneRecordsResponse {
    /** DNS zone records returned. */
    records: DomainRecord[];
}
type UpdateDNSZoneRequest = {
    /** DNS zone to update. */
    dnsZone: string;
    /** Name of the new DNS zone to create. */
    newDnsZone?: string;
    /** Project ID in which to create the new DNS zone. */
    projectId?: string;
};

/**
 * Domains and DNS API.
 *
 * Manage your domains, DNS zones and records with the Domains and DNS API.
 */
declare class API$n extends API$x {
    protected pageOfListDNSZones: (request: Readonly<ListDNSZonesRequest>) => Promise<ListDNSZonesResponse>;
    /**
     * List DNS zones. Retrieve the list of DNS zones you can manage and filter
     * DNS zones associated with specific domain names.
     *
     * @param request - The request {@link ListDNSZonesRequest}
     * @returns A Promise of ListDNSZonesResponse
     */
    listDNSZones: (request: Readonly<ListDNSZonesRequest>) => Promise<ListDNSZonesResponse> & {
        all: () => Promise<DNSZone[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DNSZone[], void, void>;
    };
    /**
     * Create a DNS zone. Create a new DNS zone specified by the domain name, the
     * subdomain and the Project ID.
     *
     * @param request - The request {@link CreateDNSZoneRequest}
     * @returns A Promise of DNSZone
     */
    createDNSZone: (request: Readonly<CreateDNSZoneRequest>) => Promise<DNSZone>;
    /**
     * Update a DNS zone. Update the name and/or the Organizations for a DNS zone.
     *
     * @param request - The request {@link UpdateDNSZoneRequest}
     * @returns A Promise of DNSZone
     */
    updateDNSZone: (request: Readonly<UpdateDNSZoneRequest>) => Promise<DNSZone>;
    /**
     * Clone a DNS zone. Clone an existing DNS zone with all its records into a
     * new DNS zone.
     *
     * @param request - The request {@link CloneDNSZoneRequest}
     * @returns A Promise of DNSZone
     */
    cloneDNSZone: (request: Readonly<CloneDNSZoneRequest>) => Promise<DNSZone>;
    /**
     * Delete a DNS zone. Delete a DNS zone and all its records.
     *
     * @param request - The request {@link DeleteDNSZoneRequest}
     * @returns A Promise of DeleteDNSZoneResponse
     */
    deleteDNSZone: (request: Readonly<DeleteDNSZoneRequest>) => Promise<DeleteDNSZoneResponse>;
    protected pageOfListDNSZoneRecords: (request: Readonly<ListDNSZoneRecordsRequest>) => Promise<ListDNSZoneRecordsResponse>;
    /**
     * List records within a DNS zone. Retrieve a list of DNS records within a DNS
     * zone that has default name servers. You can filter records by type and
     * name.
     *
     * @param request - The request {@link ListDNSZoneRecordsRequest}
     * @returns A Promise of ListDNSZoneRecordsResponse
     */
    listDNSZoneRecords: (request: Readonly<ListDNSZoneRecordsRequest>) => Promise<ListDNSZoneRecordsResponse> & {
        all: () => Promise<DomainRecord[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DomainRecord[], void, void>;
    };
    /**
     * Update records within a DNS zone. Update records within a DNS zone that has
     * default name servers and perform several actions on your records.
     *
     * Actions include:
     *
     * - Add: allows you to add a new record or add a new IP to an existing A
     *   record, for example
     * - Set: allows you to edit a record or edit an IP from an existing A record,
     *   for example
     * - Delete: allows you to delete a record or delete an IP from an existing A
     *   record, for example
     * - Clear: allows you to delete all records from a DNS zone
     *
     * All edits will be versioned.
     *
     * @param request - The request {@link UpdateDNSZoneRecordsRequest}
     * @returns A Promise of UpdateDNSZoneRecordsResponse
     */
    updateDNSZoneRecords: (request: Readonly<UpdateDNSZoneRecordsRequest>) => Promise<UpdateDNSZoneRecordsResponse>;
    /**
     * List name servers within a DNS zone. Retrieve a list of name servers within
     * a DNS zone and their optional glue records.
     *
     * @param request - The request {@link ListDNSZoneNameserversRequest}
     * @returns A Promise of ListDNSZoneNameserversResponse
     */
    listDNSZoneNameservers: (request: Readonly<ListDNSZoneNameserversRequest>) => Promise<ListDNSZoneNameserversResponse>;
    /**
     * Update name servers within a DNS zone. Update name servers within a DNS
     * zone and set optional glue records.
     *
     * @param request - The request {@link UpdateDNSZoneNameserversRequest}
     * @returns A Promise of UpdateDNSZoneNameserversResponse
     */
    updateDNSZoneNameservers: (request: Readonly<UpdateDNSZoneNameserversRequest>) => Promise<UpdateDNSZoneNameserversResponse>;
    /**
     * Clear records within a DNS zone. Delete all records within a DNS zone that
     * has default name servers.<br/> All edits will be versioned.
     *
     * @param request - The request {@link ClearDNSZoneRecordsRequest}
     * @returns A Promise of ClearDNSZoneRecordsResponse
     */
    clearDNSZoneRecords: (request: Readonly<ClearDNSZoneRecordsRequest>) => Promise<ClearDNSZoneRecordsResponse>;
    /**
     * Export a raw DNS zone. Export a DNS zone with default name servers, in a
     * specific format.
     *
     * @param request - The request {@link ExportRawDNSZoneRequest}
     * @returns A Promise of Blob
     */
    exportRawDNSZone: (request: Readonly<ExportRawDNSZoneRequest>) => Promise<Blob>;
    /**
     * Import a raw DNS zone. Import and replace the format of records from a
     * given provider, with default name servers.
     *
     * @param request - The request {@link ImportRawDNSZoneRequest}
     * @returns A Promise of ImportRawDNSZoneResponse
     */
    importRawDNSZone: (request: Readonly<ImportRawDNSZoneRequest>) => Promise<ImportRawDNSZoneResponse>;
    /**
     * Import a DNS zone from another provider. Import and replace the format of
     * records from a given provider, with default name servers.
     *
     * @param request - The request {@link ImportProviderDNSZoneRequest}
     * @returns A Promise of ImportProviderDNSZoneResponse
     */
    importProviderDNSZone: (request: Readonly<ImportProviderDNSZoneRequest>) => Promise<ImportProviderDNSZoneResponse>;
    /**
     * Refresh a DNS zone. Refresh an SOA DNS zone to reload the records in the
     * DNS zone and update the SOA serial. You can recreate the given DNS zone and
     * its sub DNS zone if needed.
     *
     * @param request - The request {@link RefreshDNSZoneRequest}
     * @returns A Promise of RefreshDNSZoneResponse
     */
    refreshDNSZone: (request: Readonly<RefreshDNSZoneRequest>) => Promise<RefreshDNSZoneResponse>;
    protected pageOfListDNSZoneVersions: (request: Readonly<ListDNSZoneVersionsRequest>) => Promise<ListDNSZoneVersionsResponse>;
    /**
     * List versions of a DNS zone. Retrieve a list of a DNS zone's versions.<br/>
     * The maximum version count is 100. If the count reaches this limit, the
     * oldest version will be deleted after each new modification.
     *
     * @param request - The request {@link ListDNSZoneVersionsRequest}
     * @returns A Promise of ListDNSZoneVersionsResponse
     */
    listDNSZoneVersions: (request: Readonly<ListDNSZoneVersionsRequest>) => Promise<ListDNSZoneVersionsResponse> & {
        all: () => Promise<DNSZoneVersion[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DNSZoneVersion[], void, void>;
    };
    protected pageOfListDNSZoneVersionRecords: (request: Readonly<ListDNSZoneVersionRecordsRequest>) => Promise<ListDNSZoneVersionRecordsResponse>;
    /**
     * List records from a given version of a specific DNS zone. Retrieve a list
     * of records from a specific DNS zone version.
     *
     * @param request - The request {@link ListDNSZoneVersionRecordsRequest}
     * @returns A Promise of ListDNSZoneVersionRecordsResponse
     */
    listDNSZoneVersionRecords: (request: Readonly<ListDNSZoneVersionRecordsRequest>) => Promise<ListDNSZoneVersionRecordsResponse> & {
        all: () => Promise<DomainRecord[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DomainRecord[], void, void>;
    };
    /**
     * Access differences from a specific DNS zone version. Access a previous DNS
     * zone version to see the differences from another specific version.
     *
     * @param request - The request {@link GetDNSZoneVersionDiffRequest}
     * @returns A Promise of GetDNSZoneVersionDiffResponse
     */
    getDNSZoneVersionDiff: (request: Readonly<GetDNSZoneVersionDiffRequest>) => Promise<GetDNSZoneVersionDiffResponse>;
    /**
     * Restore a DNS zone version. Restore and activate a version of a specific
     * DNS zone.
     *
     * @param request - The request {@link RestoreDNSZoneVersionRequest}
     * @returns A Promise of RestoreDNSZoneVersionResponse
     */
    restoreDNSZoneVersion: (request: Readonly<RestoreDNSZoneVersionRequest>) => Promise<RestoreDNSZoneVersionResponse>;
    /**
     * Get a DNS zone's TLS certificate. Get the DNS zone's TLS certificate. If
     * you do not have a certificate, the ouptut returns `no certificate found`.
     *
     * @param request - The request {@link GetSSLCertificateRequest}
     * @returns A Promise of SSLCertificate
     */
    getSSLCertificate: (request: Readonly<GetSSLCertificateRequest>) => Promise<SSLCertificate>;
    /**
     * Waits for {@link SSLCertificate} to be in a final state.
     *
     * @param request - The request {@link GetSSLCertificateRequest}
     * @param options - The waiting options
     * @returns A Promise of SSLCertificate
     */
    waitForSSLCertificate: (request: Readonly<GetSSLCertificateRequest>, options?: Readonly<WaitForOptions<SSLCertificate>>) => Promise<SSLCertificate>;
    /**
     * Create or get the DNS zone's TLS certificate. Create a new TLS certificate
     * or retrieve information about an existing TLS certificate.
     *
     * @param request - The request {@link CreateSSLCertificateRequest}
     * @returns A Promise of SSLCertificate
     */
    createSSLCertificate: (request: Readonly<CreateSSLCertificateRequest>) => Promise<SSLCertificate>;
    protected pageOfListSSLCertificates: (request: Readonly<ListSSLCertificatesRequest>) => Promise<ListSSLCertificatesResponse>;
    /**
     * List a user's TLS certificates. List all the TLS certificates a user has
     * created, specified by the user's Project ID and the DNS zone.
     *
     * @param request - The request {@link ListSSLCertificatesRequest}
     * @returns A Promise of ListSSLCertificatesResponse
     */
    listSSLCertificates: (request: Readonly<ListSSLCertificatesRequest>) => Promise<ListSSLCertificatesResponse> & {
        all: () => Promise<SSLCertificate[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<SSLCertificate[], void, void>;
    };
    /**
     * Delete a TLS certificate. Delete an existing TLS certificate specified by
     * its DNS zone. Deleting a TLS certificate is permanent and cannot be
     * undone.
     *
     * @param request - The request {@link DeleteSSLCertificateRequest}
     * @returns A Promise of DeleteSSLCertificateResponse
     */
    deleteSSLCertificate: (request: Readonly<DeleteSSLCertificateRequest>) => Promise<DeleteSSLCertificateResponse>;
    /**
     * Get the DNS zone's TSIG key. Retrieve information about the TSIG key of a
     * given DNS zone to allow AXFR requests.
     *
     * @param request - The request {@link GetDNSZoneTsigKeyRequest}
     * @returns A Promise of GetDNSZoneTsigKeyResponse
     */
    getDNSZoneTsigKey: (request: Readonly<GetDNSZoneTsigKeyRequest>) => Promise<GetDNSZoneTsigKeyResponse>;
    /**
     * Delete the DNS zone's TSIG key. Delete an existing TSIG key specified by
     * its DNS zone. Deleting a TSIG key is permanent and cannot be undone.
     *
     * @param request - The request {@link DeleteDNSZoneTsigKeyRequest}
     */
    deleteDNSZoneTsigKey: (request: Readonly<DeleteDNSZoneTsigKeyRequest>) => Promise<void>;
}
/**
 * Domains and DNS - Registrar API.
 *
 * Manage your domains and contacts.
 */
declare class RegistrarAPI extends API$x {
    protected pageOfListTasks: (request?: Readonly<RegistrarApiListTasksRequest>) => Promise<ListTasksResponse>;
    /**
     * List tasks. List all operations performed on the account. You can filter
     * the list of tasks by domain name.
     *
     * @param request - The request {@link RegistrarApiListTasksRequest}
     * @returns A Promise of ListTasksResponse
     */
    listTasks: (request?: Readonly<RegistrarApiListTasksRequest>) => Promise<ListTasksResponse> & {
        all: () => Promise<Task$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Task$1[], void, void>;
    };
    /**
     * Purchase domains. Request the registration of domain names. You can provide
     * a domain's already existing contact or a new contact.
     *
     * @param request - The request {@link RegistrarApiBuyDomainsRequest}
     * @returns A Promise of OrderResponse
     */
    buyDomains: (request: Readonly<RegistrarApiBuyDomainsRequest>) => Promise<OrderResponse>;
    /**
     * Renew domains. Request the renewal of one or more domain names.
     *
     * @param request - The request {@link RegistrarApiRenewDomainsRequest}
     * @returns A Promise of OrderResponse
     */
    renewDomains: (request: Readonly<RegistrarApiRenewDomainsRequest>) => Promise<OrderResponse>;
    /**
     * Transfer a domain. Request the transfer of a domain from another registrar
     * to Scaleway Domains and DNS.
     *
     * @param request - The request {@link RegistrarApiTransferInDomainRequest}
     * @returns A Promise of OrderResponse
     */
    transferInDomain: (request: Readonly<RegistrarApiTransferInDomainRequest>) => Promise<OrderResponse>;
    /**
     * Trade a domain's contact. Request to change a domain's contact owner.<br/>
     * If you specify the `organization_id` of the domain's new owner, the contact
     * will change from the current owner's Scaleway account to the new owner's
     * Scaleway account.<br/> If the new owner's current contact information is
     * not available, the first ever contact they have created for previous
     * domains is taken into account to operate the change.<br/> If the new owner
     * has never created a contact to register domains before, an error message
     * displays.
     *
     * @param request - The request {@link RegistrarApiTradeDomainRequest}
     * @returns A Promise of OrderResponse
     */
    tradeDomain: (request: Readonly<RegistrarApiTradeDomainRequest>) => Promise<OrderResponse>;
    /**
     * Register an external domain. Request the registration of an external domain
     * name.
     *
     * @param request - The request
     *   {@link RegistrarApiRegisterExternalDomainRequest}
     * @returns A Promise of RegisterExternalDomainResponse
     */
    registerExternalDomain: (request: Readonly<RegistrarApiRegisterExternalDomainRequest>) => Promise<RegisterExternalDomainResponse>;
    /**
     * Delete an external domain. Delete an external domain name.
     *
     * @param request - The request {@link RegistrarApiDeleteExternalDomainRequest}
     * @returns A Promise of DeleteExternalDomainResponse
     */
    deleteExternalDomain: (request: Readonly<RegistrarApiDeleteExternalDomainRequest>) => Promise<DeleteExternalDomainResponse>;
    /**
     * Check if contacts are compatible with a domain or a TLD. Check whether
     * contacts are compatible with a domain or a TLD. If contacts are not
     * compatible with either the domain or the TLD, the information that needs to
     * be corrected is returned.
     *
     * @param request - The request
     *   {@link RegistrarApiCheckContactsCompatibilityRequest}
     * @returns A Promise of CheckContactsCompatibilityResponse
     */
    checkContactsCompatibility: (request?: Readonly<RegistrarApiCheckContactsCompatibilityRequest>) => Promise<CheckContactsCompatibilityResponse>;
    protected pageOfListContacts: (request?: Readonly<RegistrarApiListContactsRequest>) => Promise<ListContactsResponse>;
    /**
     * List contacts. Retrieve the list of contacts and their associated domains
     * and roles. You can filter the list by domain name.
     *
     * @param request - The request {@link RegistrarApiListContactsRequest}
     * @returns A Promise of ListContactsResponse
     */
    listContacts: (request?: Readonly<RegistrarApiListContactsRequest>) => Promise<ListContactsResponse> & {
        all: () => Promise<ContactRoles[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ContactRoles[], void, void>;
    };
    /**
     * Get a contact. Retrieve a contact's details from the registrar using the
     * given contact's ID.
     *
     * @param request - The request {@link RegistrarApiGetContactRequest}
     * @returns A Promise of Contact
     */
    getContact: (request: Readonly<RegistrarApiGetContactRequest>) => Promise<Contact>;
    /**
     * Update contact. Edit the contact's information.
     *
     * @param request - The request {@link RegistrarApiUpdateContactRequest}
     * @returns A Promise of Contact
     */
    updateContact: (request: Readonly<RegistrarApiUpdateContactRequest>) => Promise<Contact>;
    protected pageOfListDomains: (request?: Readonly<RegistrarApiListDomainsRequest>) => Promise<ListDomainsResponse$2>;
    /**
     * List domains. Retrieve the list of domains you own.
     *
     * @param request - The request {@link RegistrarApiListDomainsRequest}
     * @returns A Promise of ListDomainsResponse
     */
    listDomains: (request?: Readonly<RegistrarApiListDomainsRequest>) => Promise<ListDomainsResponse$2> & {
        all: () => Promise<DomainSummary[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DomainSummary[], void, void>;
    };
    protected pageOfListRenewableDomains: (request?: Readonly<RegistrarApiListRenewableDomainsRequest>) => Promise<ListRenewableDomainsResponse>;
    /**
     * List domains that can be renewed. Retrieve the list of domains you own that
     * can be renewed. You can also see the maximum renewal duration in years for
     * your domains that are renewable.
     *
     * @param request - The request {@link RegistrarApiListRenewableDomainsRequest}
     * @returns A Promise of ListRenewableDomainsResponse
     */
    listRenewableDomains: (request?: Readonly<RegistrarApiListRenewableDomainsRequest>) => Promise<ListRenewableDomainsResponse> & {
        all: () => Promise<RenewableDomain[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<RenewableDomain[], void, void>;
    };
    /**
     * Get domain. Retrieve a specific domain and display the domain's
     * information.
     *
     * @param request - The request {@link RegistrarApiGetDomainRequest}
     * @returns A Promise of Domain
     */
    getDomain: (request: Readonly<RegistrarApiGetDomainRequest>) => Promise<Domain$2>;
    /**
     * Waits for {@link Domain} to be in a final state.
     *
     * @param request - The request {@link RegistrarApiGetDomainRequest}
     * @param options - The waiting options
     * @returns A Promise of Domain
     */
    waitForDomain: (request: Readonly<RegistrarApiGetDomainRequest>, options?: Readonly<WaitForOptions<Domain$2>>) => Promise<Domain$2>;
    /**
     * Update a domain's contacts. Update contacts for a specific domain or create
     * a new contact.<br/> If you add the same contact for multiple roles (owner,
     * administrative, technical), only one ID will be created and used for all of
     * the roles.
     *
     * @param request - The request {@link RegistrarApiUpdateDomainRequest}
     * @returns A Promise of Domain
     */
    updateDomain: (request: Readonly<RegistrarApiUpdateDomainRequest>) => Promise<Domain$2>;
    /**
     * Lock the transfer of a domain. Lock the transfer of a domain. This means
     * that the domain cannot be transferred and the authorization code cannot be
     * requested to your current registrar.
     *
     * @param request - The request {@link RegistrarApiLockDomainTransferRequest}
     * @returns A Promise of Domain
     */
    lockDomainTransfer: (request: Readonly<RegistrarApiLockDomainTransferRequest>) => Promise<Domain$2>;
    /**
     * Unlock the transfer of a domain. Unlock the transfer of a domain. This
     * means that the domain can be transferred and the authorization code can be
     * requested to your current registrar.
     *
     * @param request - The request {@link RegistrarApiUnlockDomainTransferRequest}
     * @returns A Promise of Domain
     */
    unlockDomainTransfer: (request: Readonly<RegistrarApiUnlockDomainTransferRequest>) => Promise<Domain$2>;
    /**
     * Enable auto renew. Enable the `auto renew` feature for a domain. This means
     * the domain will be automatically renewed before its expiry date.
     *
     * @param request - The request
     *   {@link RegistrarApiEnableDomainAutoRenewRequest}
     * @returns A Promise of Domain
     */
    enableDomainAutoRenew: (request: Readonly<RegistrarApiEnableDomainAutoRenewRequest>) => Promise<Domain$2>;
    /**
     * Disable auto renew. Disable the `auto renew` feature for a domain. This
     * means the domain will not be renewed before its expiry date.
     *
     * @param request - The request
     *   {@link RegistrarApiDisableDomainAutoRenewRequest}
     * @returns A Promise of Domain
     */
    disableDomainAutoRenew: (request: Readonly<RegistrarApiDisableDomainAutoRenewRequest>) => Promise<Domain$2>;
    /**
     * Get a domain's authorization code. Retrieve the authorization code to
     * tranfer an unlocked domain. The output returns an error if the domain is
     * locked. Some TLDs may have a different procedure to retrieve the
     * authorization code. In that case, the information displays in the message
     * field.
     *
     * @param request - The request {@link RegistrarApiGetDomainAuthCodeRequest}
     * @returns A Promise of GetDomainAuthCodeResponse
     */
    getDomainAuthCode: (request: Readonly<RegistrarApiGetDomainAuthCodeRequest>) => Promise<GetDomainAuthCodeResponse>;
    /**
     * Update domain DNSSEC. If your domain has the default Scaleway NS and uses
     * another registrar, you have to update the DS record manually.
     *
     * @param request - The request {@link RegistrarApiEnableDomainDNSSECRequest}
     * @returns A Promise of Domain
     */
    enableDomainDNSSEC: (request: Readonly<RegistrarApiEnableDomainDNSSECRequest>) => Promise<Domain$2>;
    /**
     * Disable a domain's DNSSEC. Disable DNSSEC for a domain.
     *
     * @param request - The request {@link RegistrarApiDisableDomainDNSSECRequest}
     * @returns A Promise of Domain
     */
    disableDomainDNSSEC: (request: Readonly<RegistrarApiDisableDomainDNSSECRequest>) => Promise<Domain$2>;
    /**
     * Search available domains. Search a domain or a maximum of 10 domains that
     * are available.
     *
     * If the TLD list is empty or not set, the search returns the results from
     * the most popular TLDs.
     *
     * @param request - The request
     *   {@link RegistrarApiSearchAvailableDomainsRequest}
     * @returns A Promise of SearchAvailableDomainsResponse
     */
    searchAvailableDomains: (request: Readonly<RegistrarApiSearchAvailableDomainsRequest>) => Promise<SearchAvailableDomainsResponse>;
    protected pageOfListTlds: (request?: Readonly<RegistrarApiListTldsRequest>) => Promise<ListTldsResponse>;
    /**
     * List TLD offers. Retrieve the list of TLDs and offers associated with them.
     *
     * @param request - The request {@link RegistrarApiListTldsRequest}
     * @returns A Promise of ListTldsResponse
     */
    listTlds: (request?: Readonly<RegistrarApiListTldsRequest>) => Promise<ListTldsResponse> & {
        all: () => Promise<Tld[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Tld[], void, void>;
    };
    /**
     * Create a hostname for a domain. Create a hostname for a domain with glue
     * IPs.
     *
     * @param request - The request {@link RegistrarApiCreateDomainHostRequest}
     * @returns A Promise of Host
     */
    createDomainHost: (request: Readonly<RegistrarApiCreateDomainHostRequest>) => Promise<Host>;
    protected pageOfListDomainHosts: (request: Readonly<RegistrarApiListDomainHostsRequest>) => Promise<ListDomainHostsResponse>;
    /**
     * List a domain's hostnames. List a domain's hostnames using their glue IPs.
     *
     * @param request - The request {@link RegistrarApiListDomainHostsRequest}
     * @returns A Promise of ListDomainHostsResponse
     */
    listDomainHosts: (request: Readonly<RegistrarApiListDomainHostsRequest>) => Promise<ListDomainHostsResponse> & {
        all: () => Promise<Host[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Host[], void, void>;
    };
    /**
     * Update a domain's hostname. Update a domain's hostname with glue IPs.
     *
     * @param request - The request {@link RegistrarApiUpdateDomainHostRequest}
     * @returns A Promise of Host
     */
    updateDomainHost: (request: Readonly<RegistrarApiUpdateDomainHostRequest>) => Promise<Host>;
    /**
     * Delete a domain's hostname.
     *
     * @param request - The request {@link RegistrarApiDeleteDomainHostRequest}
     * @returns A Promise of Host
     */
    deleteDomainHost: (request: Readonly<RegistrarApiDeleteDomainHostRequest>) => Promise<Host>;
}

/** Lists transient statutes of the enum {@link DNSZoneStatus}. */
declare const DNS_ZONE_TRANSIENT_STATUSES: DNSZoneStatus[];
/** Lists transient statutes of the enum {@link DomainFeatureStatus}. */
declare const DOMAIN_FEATURE_TRANSIENT_STATUSES: DomainFeatureStatus[];
/**
 * Lists transient statutes of the enum
 * {@link DomainRegistrationStatusTransferStatus}.
 */
declare const DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES: DomainRegistrationStatusTransferStatus[];
/** Lists transient statutes of the enum {@link DomainStatus}. */
declare const DOMAIN_TRANSIENT_STATUSES$2: DomainStatus$2[];
/** Lists transient statutes of the enum {@link HostStatus}. */
declare const HOST_TRANSIENT_STATUSES: HostStatus[];
/** Lists transient statutes of the enum {@link SSLCertificateStatus}. */
declare const SSL_CERTIFICATE_TRANSIENT_STATUSES: SSLCertificateStatus[];
/** Lists transient statutes of the enum {@link TaskStatus}. */
declare const TASK_TRANSIENT_STATUSES$1: TaskStatus$1[];

type index_gen$m_AvailableDomain = AvailableDomain;
type index_gen$m_CheckContactsCompatibilityResponse = CheckContactsCompatibilityResponse;
type index_gen$m_CheckContactsCompatibilityResponseContactCheckResult = CheckContactsCompatibilityResponseContactCheckResult;
type index_gen$m_ClearDNSZoneRecordsRequest = ClearDNSZoneRecordsRequest;
type index_gen$m_ClearDNSZoneRecordsResponse = ClearDNSZoneRecordsResponse;
type index_gen$m_CloneDNSZoneRequest = CloneDNSZoneRequest;
type index_gen$m_Contact = Contact;
type index_gen$m_ContactEmailStatus = ContactEmailStatus;
type index_gen$m_ContactExtensionEU = ContactExtensionEU;
type index_gen$m_ContactExtensionFR = ContactExtensionFR;
type index_gen$m_ContactExtensionFRAssociationInfo = ContactExtensionFRAssociationInfo;
type index_gen$m_ContactExtensionFRCodeAuthAfnicInfo = ContactExtensionFRCodeAuthAfnicInfo;
type index_gen$m_ContactExtensionFRDunsInfo = ContactExtensionFRDunsInfo;
type index_gen$m_ContactExtensionFRIndividualInfo = ContactExtensionFRIndividualInfo;
type index_gen$m_ContactExtensionFRMode = ContactExtensionFRMode;
type index_gen$m_ContactExtensionFRTrademarkInfo = ContactExtensionFRTrademarkInfo;
type index_gen$m_ContactExtensionNL = ContactExtensionNL;
type index_gen$m_ContactExtensionNLLegalForm = ContactExtensionNLLegalForm;
type index_gen$m_ContactLegalForm = ContactLegalForm;
type index_gen$m_ContactQuestion = ContactQuestion;
type index_gen$m_ContactRoles = ContactRoles;
type index_gen$m_ContactRolesRoles = ContactRolesRoles;
type index_gen$m_ContactStatus = ContactStatus;
type index_gen$m_CreateDNSZoneRequest = CreateDNSZoneRequest;
type index_gen$m_CreateSSLCertificateRequest = CreateSSLCertificateRequest;
type index_gen$m_DNSZone = DNSZone;
type index_gen$m_DNSZoneStatus = DNSZoneStatus;
type index_gen$m_DNSZoneVersion = DNSZoneVersion;
declare const index_gen$m_DNS_ZONE_TRANSIENT_STATUSES: typeof DNS_ZONE_TRANSIENT_STATUSES;
declare const index_gen$m_DOMAIN_FEATURE_TRANSIENT_STATUSES: typeof DOMAIN_FEATURE_TRANSIENT_STATUSES;
declare const index_gen$m_DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES: typeof DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES;
type index_gen$m_DSRecord = DSRecord;
type index_gen$m_DSRecordAlgorithm = DSRecordAlgorithm;
type index_gen$m_DSRecordDigest = DSRecordDigest;
type index_gen$m_DSRecordDigestType = DSRecordDigestType;
type index_gen$m_DSRecordPublicKey = DSRecordPublicKey;
type index_gen$m_DeleteDNSZoneRequest = DeleteDNSZoneRequest;
type index_gen$m_DeleteDNSZoneResponse = DeleteDNSZoneResponse;
type index_gen$m_DeleteDNSZoneTsigKeyRequest = DeleteDNSZoneTsigKeyRequest;
type index_gen$m_DeleteExternalDomainResponse = DeleteExternalDomainResponse;
type index_gen$m_DeleteSSLCertificateRequest = DeleteSSLCertificateRequest;
type index_gen$m_DeleteSSLCertificateResponse = DeleteSSLCertificateResponse;
type index_gen$m_DomainDNSSEC = DomainDNSSEC;
type index_gen$m_DomainFeatureStatus = DomainFeatureStatus;
type index_gen$m_DomainRecord = DomainRecord;
type index_gen$m_DomainRecordGeoIPConfig = DomainRecordGeoIPConfig;
type index_gen$m_DomainRecordGeoIPConfigMatch = DomainRecordGeoIPConfigMatch;
type index_gen$m_DomainRecordHTTPServiceConfig = DomainRecordHTTPServiceConfig;
type index_gen$m_DomainRecordHTTPServiceConfigStrategy = DomainRecordHTTPServiceConfigStrategy;
type index_gen$m_DomainRecordType = DomainRecordType;
type index_gen$m_DomainRecordViewConfig = DomainRecordViewConfig;
type index_gen$m_DomainRecordViewConfigView = DomainRecordViewConfigView;
type index_gen$m_DomainRecordWeightedConfig = DomainRecordWeightedConfig;
type index_gen$m_DomainRecordWeightedConfigWeightedIP = DomainRecordWeightedConfigWeightedIP;
type index_gen$m_DomainRegistrationStatusExternalDomain = DomainRegistrationStatusExternalDomain;
type index_gen$m_DomainRegistrationStatusTransfer = DomainRegistrationStatusTransfer;
type index_gen$m_DomainRegistrationStatusTransferStatus = DomainRegistrationStatusTransferStatus;
type index_gen$m_DomainSummary = DomainSummary;
type index_gen$m_ExportRawDNSZoneRequest = ExportRawDNSZoneRequest;
type index_gen$m_GetDNSZoneTsigKeyRequest = GetDNSZoneTsigKeyRequest;
type index_gen$m_GetDNSZoneTsigKeyResponse = GetDNSZoneTsigKeyResponse;
type index_gen$m_GetDNSZoneVersionDiffRequest = GetDNSZoneVersionDiffRequest;
type index_gen$m_GetDNSZoneVersionDiffResponse = GetDNSZoneVersionDiffResponse;
type index_gen$m_GetDomainAuthCodeResponse = GetDomainAuthCodeResponse;
type index_gen$m_GetSSLCertificateRequest = GetSSLCertificateRequest;
declare const index_gen$m_HOST_TRANSIENT_STATUSES: typeof HOST_TRANSIENT_STATUSES;
type index_gen$m_Host = Host;
type index_gen$m_HostStatus = HostStatus;
type index_gen$m_ImportProviderDNSZoneRequest = ImportProviderDNSZoneRequest;
type index_gen$m_ImportProviderDNSZoneRequestOnlineV1 = ImportProviderDNSZoneRequestOnlineV1;
type index_gen$m_ImportProviderDNSZoneResponse = ImportProviderDNSZoneResponse;
type index_gen$m_ImportRawDNSZoneRequest = ImportRawDNSZoneRequest;
type index_gen$m_ImportRawDNSZoneRequestAXFRSource = ImportRawDNSZoneRequestAXFRSource;
type index_gen$m_ImportRawDNSZoneRequestBindSource = ImportRawDNSZoneRequestBindSource;
type index_gen$m_ImportRawDNSZoneRequestTsigKey = ImportRawDNSZoneRequestTsigKey;
type index_gen$m_ImportRawDNSZoneResponse = ImportRawDNSZoneResponse;
type index_gen$m_LinkedProduct = LinkedProduct;
type index_gen$m_ListContactsRequestRole = ListContactsRequestRole;
type index_gen$m_ListContactsResponse = ListContactsResponse;
type index_gen$m_ListDNSZoneNameserversRequest = ListDNSZoneNameserversRequest;
type index_gen$m_ListDNSZoneNameserversResponse = ListDNSZoneNameserversResponse;
type index_gen$m_ListDNSZoneRecordsRequest = ListDNSZoneRecordsRequest;
type index_gen$m_ListDNSZoneRecordsRequestOrderBy = ListDNSZoneRecordsRequestOrderBy;
type index_gen$m_ListDNSZoneRecordsResponse = ListDNSZoneRecordsResponse;
type index_gen$m_ListDNSZoneVersionRecordsRequest = ListDNSZoneVersionRecordsRequest;
type index_gen$m_ListDNSZoneVersionRecordsResponse = ListDNSZoneVersionRecordsResponse;
type index_gen$m_ListDNSZoneVersionsRequest = ListDNSZoneVersionsRequest;
type index_gen$m_ListDNSZoneVersionsResponse = ListDNSZoneVersionsResponse;
type index_gen$m_ListDNSZonesRequest = ListDNSZonesRequest;
type index_gen$m_ListDNSZonesRequestOrderBy = ListDNSZonesRequestOrderBy;
type index_gen$m_ListDNSZonesResponse = ListDNSZonesResponse;
type index_gen$m_ListDomainHostsResponse = ListDomainHostsResponse;
type index_gen$m_ListRenewableDomainsRequestOrderBy = ListRenewableDomainsRequestOrderBy;
type index_gen$m_ListRenewableDomainsResponse = ListRenewableDomainsResponse;
type index_gen$m_ListSSLCertificatesRequest = ListSSLCertificatesRequest;
type index_gen$m_ListSSLCertificatesResponse = ListSSLCertificatesResponse;
type index_gen$m_ListTasksRequestOrderBy = ListTasksRequestOrderBy;
type index_gen$m_ListTasksResponse = ListTasksResponse;
type index_gen$m_ListTldsRequestOrderBy = ListTldsRequestOrderBy;
type index_gen$m_ListTldsResponse = ListTldsResponse;
type index_gen$m_NewContact = NewContact;
type index_gen$m_OrderResponse = OrderResponse;
type index_gen$m_RawFormat = RawFormat;
type index_gen$m_RecordChange = RecordChange;
type index_gen$m_RecordChangeAdd = RecordChangeAdd;
type index_gen$m_RecordChangeClear = RecordChangeClear;
type index_gen$m_RecordChangeDelete = RecordChangeDelete;
type index_gen$m_RecordChangeSet = RecordChangeSet;
type index_gen$m_RecordIdentifier = RecordIdentifier;
type index_gen$m_RefreshDNSZoneRequest = RefreshDNSZoneRequest;
type index_gen$m_RefreshDNSZoneResponse = RefreshDNSZoneResponse;
type index_gen$m_RegisterExternalDomainResponse = RegisterExternalDomainResponse;
type index_gen$m_RegistrarAPI = RegistrarAPI;
declare const index_gen$m_RegistrarAPI: typeof RegistrarAPI;
type index_gen$m_RegistrarApiBuyDomainsRequest = RegistrarApiBuyDomainsRequest;
type index_gen$m_RegistrarApiCheckContactsCompatibilityRequest = RegistrarApiCheckContactsCompatibilityRequest;
type index_gen$m_RegistrarApiCreateDomainHostRequest = RegistrarApiCreateDomainHostRequest;
type index_gen$m_RegistrarApiDeleteDomainHostRequest = RegistrarApiDeleteDomainHostRequest;
type index_gen$m_RegistrarApiDeleteExternalDomainRequest = RegistrarApiDeleteExternalDomainRequest;
type index_gen$m_RegistrarApiDisableDomainAutoRenewRequest = RegistrarApiDisableDomainAutoRenewRequest;
type index_gen$m_RegistrarApiDisableDomainDNSSECRequest = RegistrarApiDisableDomainDNSSECRequest;
type index_gen$m_RegistrarApiEnableDomainAutoRenewRequest = RegistrarApiEnableDomainAutoRenewRequest;
type index_gen$m_RegistrarApiEnableDomainDNSSECRequest = RegistrarApiEnableDomainDNSSECRequest;
type index_gen$m_RegistrarApiGetContactRequest = RegistrarApiGetContactRequest;
type index_gen$m_RegistrarApiGetDomainAuthCodeRequest = RegistrarApiGetDomainAuthCodeRequest;
type index_gen$m_RegistrarApiGetDomainRequest = RegistrarApiGetDomainRequest;
type index_gen$m_RegistrarApiListContactsRequest = RegistrarApiListContactsRequest;
type index_gen$m_RegistrarApiListDomainHostsRequest = RegistrarApiListDomainHostsRequest;
type index_gen$m_RegistrarApiListDomainsRequest = RegistrarApiListDomainsRequest;
type index_gen$m_RegistrarApiListRenewableDomainsRequest = RegistrarApiListRenewableDomainsRequest;
type index_gen$m_RegistrarApiListTasksRequest = RegistrarApiListTasksRequest;
type index_gen$m_RegistrarApiListTldsRequest = RegistrarApiListTldsRequest;
type index_gen$m_RegistrarApiLockDomainTransferRequest = RegistrarApiLockDomainTransferRequest;
type index_gen$m_RegistrarApiRegisterExternalDomainRequest = RegistrarApiRegisterExternalDomainRequest;
type index_gen$m_RegistrarApiRenewDomainsRequest = RegistrarApiRenewDomainsRequest;
type index_gen$m_RegistrarApiSearchAvailableDomainsRequest = RegistrarApiSearchAvailableDomainsRequest;
type index_gen$m_RegistrarApiTradeDomainRequest = RegistrarApiTradeDomainRequest;
type index_gen$m_RegistrarApiTransferInDomainRequest = RegistrarApiTransferInDomainRequest;
type index_gen$m_RegistrarApiUnlockDomainTransferRequest = RegistrarApiUnlockDomainTransferRequest;
type index_gen$m_RegistrarApiUpdateContactRequest = RegistrarApiUpdateContactRequest;
type index_gen$m_RegistrarApiUpdateDomainHostRequest = RegistrarApiUpdateDomainHostRequest;
type index_gen$m_RegistrarApiUpdateDomainRequest = RegistrarApiUpdateDomainRequest;
type index_gen$m_RenewableDomain = RenewableDomain;
type index_gen$m_RenewableDomainStatus = RenewableDomainStatus;
type index_gen$m_RestoreDNSZoneVersionRequest = RestoreDNSZoneVersionRequest;
type index_gen$m_RestoreDNSZoneVersionResponse = RestoreDNSZoneVersionResponse;
type index_gen$m_SSLCertificate = SSLCertificate;
type index_gen$m_SSLCertificateStatus = SSLCertificateStatus;
declare const index_gen$m_SSL_CERTIFICATE_TRANSIENT_STATUSES: typeof SSL_CERTIFICATE_TRANSIENT_STATUSES;
type index_gen$m_SearchAvailableDomainsResponse = SearchAvailableDomainsResponse;
type index_gen$m_TaskType = TaskType;
type index_gen$m_Tld = Tld;
type index_gen$m_TldOffer = TldOffer;
type index_gen$m_TransferInDomainRequestTransferRequest = TransferInDomainRequestTransferRequest;
type index_gen$m_UpdateContactRequestQuestion = UpdateContactRequestQuestion;
type index_gen$m_UpdateDNSZoneNameserversRequest = UpdateDNSZoneNameserversRequest;
type index_gen$m_UpdateDNSZoneNameserversResponse = UpdateDNSZoneNameserversResponse;
type index_gen$m_UpdateDNSZoneRecordsRequest = UpdateDNSZoneRecordsRequest;
type index_gen$m_UpdateDNSZoneRecordsResponse = UpdateDNSZoneRecordsResponse;
type index_gen$m_UpdateDNSZoneRequest = UpdateDNSZoneRequest;
declare namespace index_gen$m {
  export { API$n as API, type index_gen$m_AvailableDomain as AvailableDomain, type index_gen$m_CheckContactsCompatibilityResponse as CheckContactsCompatibilityResponse, type index_gen$m_CheckContactsCompatibilityResponseContactCheckResult as CheckContactsCompatibilityResponseContactCheckResult, type index_gen$m_ClearDNSZoneRecordsRequest as ClearDNSZoneRecordsRequest, type index_gen$m_ClearDNSZoneRecordsResponse as ClearDNSZoneRecordsResponse, type index_gen$m_CloneDNSZoneRequest as CloneDNSZoneRequest, type index_gen$m_Contact as Contact, type index_gen$m_ContactEmailStatus as ContactEmailStatus, type index_gen$m_ContactExtensionEU as ContactExtensionEU, type index_gen$m_ContactExtensionFR as ContactExtensionFR, type index_gen$m_ContactExtensionFRAssociationInfo as ContactExtensionFRAssociationInfo, type index_gen$m_ContactExtensionFRCodeAuthAfnicInfo as ContactExtensionFRCodeAuthAfnicInfo, type index_gen$m_ContactExtensionFRDunsInfo as ContactExtensionFRDunsInfo, type index_gen$m_ContactExtensionFRIndividualInfo as ContactExtensionFRIndividualInfo, type index_gen$m_ContactExtensionFRMode as ContactExtensionFRMode, type index_gen$m_ContactExtensionFRTrademarkInfo as ContactExtensionFRTrademarkInfo, type index_gen$m_ContactExtensionNL as ContactExtensionNL, type index_gen$m_ContactExtensionNLLegalForm as ContactExtensionNLLegalForm, type index_gen$m_ContactLegalForm as ContactLegalForm, type index_gen$m_ContactQuestion as ContactQuestion, type index_gen$m_ContactRoles as ContactRoles, type index_gen$m_ContactRolesRoles as ContactRolesRoles, type index_gen$m_ContactStatus as ContactStatus, type index_gen$m_CreateDNSZoneRequest as CreateDNSZoneRequest, type index_gen$m_CreateSSLCertificateRequest as CreateSSLCertificateRequest, type index_gen$m_DNSZone as DNSZone, type index_gen$m_DNSZoneStatus as DNSZoneStatus, type index_gen$m_DNSZoneVersion as DNSZoneVersion, index_gen$m_DNS_ZONE_TRANSIENT_STATUSES as DNS_ZONE_TRANSIENT_STATUSES, index_gen$m_DOMAIN_FEATURE_TRANSIENT_STATUSES as DOMAIN_FEATURE_TRANSIENT_STATUSES, index_gen$m_DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES as DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES, DOMAIN_TRANSIENT_STATUSES$2 as DOMAIN_TRANSIENT_STATUSES, type index_gen$m_DSRecord as DSRecord, type index_gen$m_DSRecordAlgorithm as DSRecordAlgorithm, type index_gen$m_DSRecordDigest as DSRecordDigest, type index_gen$m_DSRecordDigestType as DSRecordDigestType, type index_gen$m_DSRecordPublicKey as DSRecordPublicKey, type index_gen$m_DeleteDNSZoneRequest as DeleteDNSZoneRequest, type index_gen$m_DeleteDNSZoneResponse as DeleteDNSZoneResponse, type index_gen$m_DeleteDNSZoneTsigKeyRequest as DeleteDNSZoneTsigKeyRequest, type index_gen$m_DeleteExternalDomainResponse as DeleteExternalDomainResponse, type index_gen$m_DeleteSSLCertificateRequest as DeleteSSLCertificateRequest, type index_gen$m_DeleteSSLCertificateResponse as DeleteSSLCertificateResponse, type Domain$2 as Domain, type index_gen$m_DomainDNSSEC as DomainDNSSEC, type index_gen$m_DomainFeatureStatus as DomainFeatureStatus, type index_gen$m_DomainRecord as DomainRecord, type index_gen$m_DomainRecordGeoIPConfig as DomainRecordGeoIPConfig, type index_gen$m_DomainRecordGeoIPConfigMatch as DomainRecordGeoIPConfigMatch, type index_gen$m_DomainRecordHTTPServiceConfig as DomainRecordHTTPServiceConfig, type index_gen$m_DomainRecordHTTPServiceConfigStrategy as DomainRecordHTTPServiceConfigStrategy, type index_gen$m_DomainRecordType as DomainRecordType, type index_gen$m_DomainRecordViewConfig as DomainRecordViewConfig, type index_gen$m_DomainRecordViewConfigView as DomainRecordViewConfigView, type index_gen$m_DomainRecordWeightedConfig as DomainRecordWeightedConfig, type index_gen$m_DomainRecordWeightedConfigWeightedIP as DomainRecordWeightedConfigWeightedIP, type index_gen$m_DomainRegistrationStatusExternalDomain as DomainRegistrationStatusExternalDomain, type index_gen$m_DomainRegistrationStatusTransfer as DomainRegistrationStatusTransfer, type index_gen$m_DomainRegistrationStatusTransferStatus as DomainRegistrationStatusTransferStatus, type DomainStatus$2 as DomainStatus, type index_gen$m_DomainSummary as DomainSummary, type index_gen$m_ExportRawDNSZoneRequest as ExportRawDNSZoneRequest, type index_gen$m_GetDNSZoneTsigKeyRequest as GetDNSZoneTsigKeyRequest, type index_gen$m_GetDNSZoneTsigKeyResponse as GetDNSZoneTsigKeyResponse, type index_gen$m_GetDNSZoneVersionDiffRequest as GetDNSZoneVersionDiffRequest, type index_gen$m_GetDNSZoneVersionDiffResponse as GetDNSZoneVersionDiffResponse, type index_gen$m_GetDomainAuthCodeResponse as GetDomainAuthCodeResponse, type index_gen$m_GetSSLCertificateRequest as GetSSLCertificateRequest, index_gen$m_HOST_TRANSIENT_STATUSES as HOST_TRANSIENT_STATUSES, type index_gen$m_Host as Host, type index_gen$m_HostStatus as HostStatus, type index_gen$m_ImportProviderDNSZoneRequest as ImportProviderDNSZoneRequest, type index_gen$m_ImportProviderDNSZoneRequestOnlineV1 as ImportProviderDNSZoneRequestOnlineV1, type index_gen$m_ImportProviderDNSZoneResponse as ImportProviderDNSZoneResponse, type index_gen$m_ImportRawDNSZoneRequest as ImportRawDNSZoneRequest, type index_gen$m_ImportRawDNSZoneRequestAXFRSource as ImportRawDNSZoneRequestAXFRSource, type index_gen$m_ImportRawDNSZoneRequestBindSource as ImportRawDNSZoneRequestBindSource, type index_gen$m_ImportRawDNSZoneRequestTsigKey as ImportRawDNSZoneRequestTsigKey, type index_gen$m_ImportRawDNSZoneResponse as ImportRawDNSZoneResponse, type index_gen$m_LinkedProduct as LinkedProduct, type index_gen$m_ListContactsRequestRole as ListContactsRequestRole, type index_gen$m_ListContactsResponse as ListContactsResponse, type index_gen$m_ListDNSZoneNameserversRequest as ListDNSZoneNameserversRequest, type index_gen$m_ListDNSZoneNameserversResponse as ListDNSZoneNameserversResponse, type index_gen$m_ListDNSZoneRecordsRequest as ListDNSZoneRecordsRequest, type index_gen$m_ListDNSZoneRecordsRequestOrderBy as ListDNSZoneRecordsRequestOrderBy, type index_gen$m_ListDNSZoneRecordsResponse as ListDNSZoneRecordsResponse, type index_gen$m_ListDNSZoneVersionRecordsRequest as ListDNSZoneVersionRecordsRequest, type index_gen$m_ListDNSZoneVersionRecordsResponse as ListDNSZoneVersionRecordsResponse, type index_gen$m_ListDNSZoneVersionsRequest as ListDNSZoneVersionsRequest, type index_gen$m_ListDNSZoneVersionsResponse as ListDNSZoneVersionsResponse, type index_gen$m_ListDNSZonesRequest as ListDNSZonesRequest, type index_gen$m_ListDNSZonesRequestOrderBy as ListDNSZonesRequestOrderBy, type index_gen$m_ListDNSZonesResponse as ListDNSZonesResponse, type index_gen$m_ListDomainHostsResponse as ListDomainHostsResponse, type ListDomainsRequestOrderBy$1 as ListDomainsRequestOrderBy, type ListDomainsResponse$2 as ListDomainsResponse, type index_gen$m_ListRenewableDomainsRequestOrderBy as ListRenewableDomainsRequestOrderBy, type index_gen$m_ListRenewableDomainsResponse as ListRenewableDomainsResponse, type index_gen$m_ListSSLCertificatesRequest as ListSSLCertificatesRequest, type index_gen$m_ListSSLCertificatesResponse as ListSSLCertificatesResponse, type index_gen$m_ListTasksRequestOrderBy as ListTasksRequestOrderBy, type index_gen$m_ListTasksResponse as ListTasksResponse, type index_gen$m_ListTldsRequestOrderBy as ListTldsRequestOrderBy, type index_gen$m_ListTldsResponse as ListTldsResponse, type Nameserver$1 as Nameserver, type index_gen$m_NewContact as NewContact, type index_gen$m_OrderResponse as OrderResponse, type index_gen$m_RawFormat as RawFormat, type index_gen$m_RecordChange as RecordChange, type index_gen$m_RecordChangeAdd as RecordChangeAdd, type index_gen$m_RecordChangeClear as RecordChangeClear, type index_gen$m_RecordChangeDelete as RecordChangeDelete, type index_gen$m_RecordChangeSet as RecordChangeSet, type index_gen$m_RecordIdentifier as RecordIdentifier, type index_gen$m_RefreshDNSZoneRequest as RefreshDNSZoneRequest, type index_gen$m_RefreshDNSZoneResponse as RefreshDNSZoneResponse, type index_gen$m_RegisterExternalDomainResponse as RegisterExternalDomainResponse, index_gen$m_RegistrarAPI as RegistrarAPI, type index_gen$m_RegistrarApiBuyDomainsRequest as RegistrarApiBuyDomainsRequest, type index_gen$m_RegistrarApiCheckContactsCompatibilityRequest as RegistrarApiCheckContactsCompatibilityRequest, type index_gen$m_RegistrarApiCreateDomainHostRequest as RegistrarApiCreateDomainHostRequest, type index_gen$m_RegistrarApiDeleteDomainHostRequest as RegistrarApiDeleteDomainHostRequest, type index_gen$m_RegistrarApiDeleteExternalDomainRequest as RegistrarApiDeleteExternalDomainRequest, type index_gen$m_RegistrarApiDisableDomainAutoRenewRequest as RegistrarApiDisableDomainAutoRenewRequest, type index_gen$m_RegistrarApiDisableDomainDNSSECRequest as RegistrarApiDisableDomainDNSSECRequest, type index_gen$m_RegistrarApiEnableDomainAutoRenewRequest as RegistrarApiEnableDomainAutoRenewRequest, type index_gen$m_RegistrarApiEnableDomainDNSSECRequest as RegistrarApiEnableDomainDNSSECRequest, type index_gen$m_RegistrarApiGetContactRequest as RegistrarApiGetContactRequest, type index_gen$m_RegistrarApiGetDomainAuthCodeRequest as RegistrarApiGetDomainAuthCodeRequest, type index_gen$m_RegistrarApiGetDomainRequest as RegistrarApiGetDomainRequest, type index_gen$m_RegistrarApiListContactsRequest as RegistrarApiListContactsRequest, type index_gen$m_RegistrarApiListDomainHostsRequest as RegistrarApiListDomainHostsRequest, type index_gen$m_RegistrarApiListDomainsRequest as RegistrarApiListDomainsRequest, type index_gen$m_RegistrarApiListRenewableDomainsRequest as RegistrarApiListRenewableDomainsRequest, type index_gen$m_RegistrarApiListTasksRequest as RegistrarApiListTasksRequest, type index_gen$m_RegistrarApiListTldsRequest as RegistrarApiListTldsRequest, type index_gen$m_RegistrarApiLockDomainTransferRequest as RegistrarApiLockDomainTransferRequest, type index_gen$m_RegistrarApiRegisterExternalDomainRequest as RegistrarApiRegisterExternalDomainRequest, type index_gen$m_RegistrarApiRenewDomainsRequest as RegistrarApiRenewDomainsRequest, type index_gen$m_RegistrarApiSearchAvailableDomainsRequest as RegistrarApiSearchAvailableDomainsRequest, type index_gen$m_RegistrarApiTradeDomainRequest as RegistrarApiTradeDomainRequest, type index_gen$m_RegistrarApiTransferInDomainRequest as RegistrarApiTransferInDomainRequest, type index_gen$m_RegistrarApiUnlockDomainTransferRequest as RegistrarApiUnlockDomainTransferRequest, type index_gen$m_RegistrarApiUpdateContactRequest as RegistrarApiUpdateContactRequest, type index_gen$m_RegistrarApiUpdateDomainHostRequest as RegistrarApiUpdateDomainHostRequest, type index_gen$m_RegistrarApiUpdateDomainRequest as RegistrarApiUpdateDomainRequest, type index_gen$m_RenewableDomain as RenewableDomain, type index_gen$m_RenewableDomainStatus as RenewableDomainStatus, type index_gen$m_RestoreDNSZoneVersionRequest as RestoreDNSZoneVersionRequest, type index_gen$m_RestoreDNSZoneVersionResponse as RestoreDNSZoneVersionResponse, type index_gen$m_SSLCertificate as SSLCertificate, type index_gen$m_SSLCertificateStatus as SSLCertificateStatus, index_gen$m_SSL_CERTIFICATE_TRANSIENT_STATUSES as SSL_CERTIFICATE_TRANSIENT_STATUSES, type index_gen$m_SearchAvailableDomainsResponse as SearchAvailableDomainsResponse, TASK_TRANSIENT_STATUSES$1 as TASK_TRANSIENT_STATUSES, type Task$1 as Task, type TaskStatus$1 as TaskStatus, type index_gen$m_TaskType as TaskType, type index_gen$m_Tld as Tld, type index_gen$m_TldOffer as TldOffer, type index_gen$m_TransferInDomainRequestTransferRequest as TransferInDomainRequestTransferRequest, type index_gen$m_UpdateContactRequestQuestion as UpdateContactRequestQuestion, type index_gen$m_UpdateDNSZoneNameserversRequest as UpdateDNSZoneNameserversRequest, type index_gen$m_UpdateDNSZoneNameserversResponse as UpdateDNSZoneNameserversResponse, type index_gen$m_UpdateDNSZoneRecordsRequest as UpdateDNSZoneRecordsRequest, type index_gen$m_UpdateDNSZoneRecordsResponse as UpdateDNSZoneRecordsResponse, type index_gen$m_UpdateDNSZoneRequest as UpdateDNSZoneRequest };
}

declare namespace index$p {
  export { index_gen$m as v2beta1 };
}

type FlexibleIPStatus = 'unknown' | 'ready' | 'updating' | 'attached' | 'error' | 'detaching' | 'locked';
type ListFlexibleIPsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type MACAddressStatus = 'unknown' | 'ready' | 'updating' | 'used' | 'error' | 'deleting';
type MACAddressType = 'unknown_type' | 'vmware' | 'xen' | 'kvm';
interface MACAddress {
    /** ID of the flexible IP. */
    id: string;
    /** MAC address of the Virtual MAC. */
    macAddress: string;
    /** Type of virtual MAC. */
    macType: MACAddressType;
    /** Status of virtual MAC. */
    status: MACAddressStatus;
    /** Date on which the virtual MAC was last updated. */
    updatedAt?: Date;
    /** Date on which the virtual MAC was created. */
    createdAt?: Date;
    /** MAC address IP Availability Zone. */
    zone: Zone;
}
interface FlexibleIP {
    /** ID of the flexible IP. */
    id: string;
    /** ID of the Organization the flexible IP is attached to. */
    organizationId: string;
    /** ID of the Project the flexible IP is attached to. */
    projectId: string;
    /** Flexible IP description. */
    description: string;
    /** Flexible IP tags. */
    tags: string[];
    /** Date on which the flexible IP was last updated. */
    updatedAt?: Date;
    /** Date on which the flexible IP was created. */
    createdAt?: Date;
    /**
     * - Ready : flexible IP is created and ready to be attached to a server or to
     *   be associated with a virtual MAC.
     * - Updating: flexible IP is being attached to a server or a virtual MAC
     *   operation is ongoing
     * - Attached: flexible IP is attached to a server
     * - Error: a flexible IP operation resulted in an error
     * - Detaching: flexible IP is being detached from a server
     * - Locked: the resource of the flexible IP is locked.
     */
    status: FlexibleIPStatus;
    /** IP of the flexible IP. */
    ipAddress: string;
    /** MAC address of the flexible IP. */
    macAddress?: MACAddress;
    /** ID of the server linked to the flexible IP. */
    serverId?: string;
    /** Reverse DNS value. */
    reverse: string;
    /** Availability Zone of the flexible IP. */
    zone: Zone;
}
type AttachFlexibleIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * Multiple IDs can be provided, but note that flexible IPs must belong to the
     * same MAC group (see details about MAC groups).
     */
    fipsIds: string[];
    /** ID of the server on which to attach the flexible IPs. */
    serverId: string;
};
interface AttachFlexibleIPsResponse {
    /** Total count of flexible IPs that are being updated. */
    totalCount: number;
    /** List of flexible IPs in an updating state. */
    flexibleIps: FlexibleIP[];
}
type CreateFlexibleIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the project to associate with the Flexible IP. */
    projectId?: string;
    /** Flexible IP description (max. of 255 characters). */
    description: string;
    /** Tags to associate to the flexible IP. */
    tags?: string[];
    /** ID of the server to which the newly created flexible IP will be attached. */
    serverId?: string;
    /** Value of the reverse DNS. */
    reverse?: string;
    /** Defines whether the flexible IP has an IPv6 address. */
    isIpv6: boolean;
};
type DeleteFlexibleIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the flexible IP to delete. */
    fipId: string;
};
type DeleteMACAddrRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * If the flexible IP belongs to a MAC group, the MAC will be removed from
     * both the MAC group and flexible IP.
     */
    fipId: string;
};
type DetachFlexibleIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * List of flexible IP IDs to detach from a server. Multiple IDs can be
     * provided. Note that flexible IPs must belong to the same MAC group.
     */
    fipsIds: string[];
};
interface DetachFlexibleIPsResponse {
    /** Total count of flexible IPs that are being detached. */
    totalCount: number;
    /** List of flexible IPs in a detaching state. */
    flexibleIps: FlexibleIP[];
}
type DuplicateMACAddrRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Note that the flexible IPs need to be attached to the same server. */
    fipId: string;
    /** Note that flexible IPs need to be attached to the same server. */
    duplicateFromFipId: string;
};
type GenerateMACAddrRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the flexible IP for which to generate a virtual MAC. */
    fipId: string;
    /** TODO. */
    macType: MACAddressType;
};
type GetFlexibleIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the flexible IP. */
    fipId: string;
};
type ListFlexibleIPsRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Sort order of the returned flexible IPs. */
    orderBy?: ListFlexibleIPsRequestOrderBy;
    /** Page number. */
    page?: number;
    /** Maximum number of flexible IPs per page. */
    pageSize?: number;
    /**
     * Filter by tag, only flexible IPs with one or more matching tags will be
     * returned.
     */
    tags?: string[];
    /** Filter by status, only flexible IPs with this status will be returned. */
    status?: FlexibleIPStatus[];
    /**
     * Filter by server IDs, only flexible IPs with these server IDs will be
     * returned.
     */
    serverIds?: string[];
    /**
     * Filter by Organization ID, only flexible IPs from this Organization will be
     * returned.
     */
    organizationId?: string;
    /** Filter by Project ID, only flexible IPs from this Project will be returned. */
    projectId?: string;
};
interface ListFlexibleIPsResponse {
    /** Total count of matching flexible IPs. */
    totalCount: number;
    /** List of all flexible IPs. */
    flexibleIps: FlexibleIP[];
}
type MoveMACAddrRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    fipId: string;
    dstFipId: string;
};
type UpdateFlexibleIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the flexible IP to update. */
    fipId: string;
    /** Flexible IP description (max. 255 characters). */
    description?: string;
    /** Tags associated with the flexible IP. */
    tags?: string[];
    /** Value of the reverse DNS. */
    reverse?: string;
};

/** Elastic Metal - Flexible IP API. */
declare class API$m extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    /**
     * Create a new flexible IP. Generate a new flexible IP within a given zone,
     * specifying its configuration including Project ID and description.
     *
     * @param request - The request {@link CreateFlexibleIPRequest}
     * @returns A Promise of FlexibleIP
     */
    createFlexibleIP: (request: Readonly<CreateFlexibleIPRequest>) => Promise<FlexibleIP>;
    /**
     * Get an existing flexible IP. Retrieve information about an existing
     * flexible IP, specified by its ID and zone. Its full details, including
     * Project ID, description and status, are returned in the response object.
     *
     * @param request - The request {@link GetFlexibleIPRequest}
     * @returns A Promise of FlexibleIP
     */
    getFlexibleIP: (request: Readonly<GetFlexibleIPRequest>) => Promise<FlexibleIP>;
    /**
     * Waits for {@link FlexibleIP} to be in a final state.
     *
     * @param request - The request {@link GetFlexibleIPRequest}
     * @param options - The waiting options
     * @returns A Promise of FlexibleIP
     */
    waitForFlexibleIP: (request: Readonly<GetFlexibleIPRequest>, options?: Readonly<WaitForOptions<FlexibleIP>>) => Promise<FlexibleIP>;
    protected pageOfListFlexibleIPs: (request?: Readonly<ListFlexibleIPsRequest$1>) => Promise<ListFlexibleIPsResponse>;
    /**
     * List flexible IPs. List all flexible IPs within a given zone.
     *
     * @param request - The request {@link ListFlexibleIPsRequest}
     * @returns A Promise of ListFlexibleIPsResponse
     */
    listFlexibleIPs: (request?: Readonly<ListFlexibleIPsRequest$1>) => Promise<ListFlexibleIPsResponse> & {
        /**
         * Waits for {@link FlexibleIP} to be in a final state.
         *
         * @param request - The request {@link GetFlexibleIPRequest}
         * @param options - The waiting options
         * @returns A Promise of FlexibleIP
         */
        all: () => Promise<FlexibleIP[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<FlexibleIP[], void, void>;
    };
    /**
     * Update an existing flexible IP. Update the parameters of an existing
     * flexible IP, specified by its ID and zone. These parameters include tags
     * and description.
     *
     * @param request - The request {@link UpdateFlexibleIPRequest}
     * @returns A Promise of FlexibleIP
     */
    updateFlexibleIP: (request: Readonly<UpdateFlexibleIPRequest>) => Promise<FlexibleIP>;
    /**
     * Delete an existing flexible IP. Delete an existing flexible IP, specified
     * by its ID and zone. Note that deleting a flexible IP is permanent and
     * cannot be undone.
     *
     * @param request - The request {@link DeleteFlexibleIPRequest}
     */
    deleteFlexibleIP: (request: Readonly<DeleteFlexibleIPRequest>) => Promise<void>;
    /**
     * Attach an existing flexible IP to a server. Attach an existing flexible IP
     * to a specified Elastic Metal server.
     *
     * @param request - The request {@link AttachFlexibleIPRequest}
     * @returns A Promise of AttachFlexibleIPsResponse
     */
    attachFlexibleIP: (request: Readonly<AttachFlexibleIPRequest>) => Promise<AttachFlexibleIPsResponse>;
    /**
     * Detach an existing flexible IP from a server. Detach an existing flexible
     * IP from a specified Elastic Metal server.
     *
     * @param request - The request {@link DetachFlexibleIPRequest}
     * @returns A Promise of DetachFlexibleIPsResponse
     */
    detachFlexibleIP: (request: Readonly<DetachFlexibleIPRequest>) => Promise<DetachFlexibleIPsResponse>;
    /**
     * Generate a virtual MAC address on an existing flexible IP. Generate a
     * virtual MAC (Media Access Control) address on an existing flexible IP.
     *
     * @param request - The request {@link GenerateMACAddrRequest}
     * @returns A Promise of FlexibleIP
     */
    generateMACAddr: (request: Readonly<GenerateMACAddrRequest>) => Promise<FlexibleIP>;
    /**
     * Duplicate a virtual MAC address to another flexible IP. Duplicate a virtual
     * MAC address from a given flexible IP to another flexible IP attached to the
     * same server.
     *
     * @param request - The request {@link DuplicateMACAddrRequest}
     * @returns A Promise of FlexibleIP
     */
    duplicateMACAddr: (request: Readonly<DuplicateMACAddrRequest>) => Promise<FlexibleIP>;
    /**
     * Relocate an existing virtual MAC address to a different flexible IP.
     * Relocate a virtual MAC (Media Access Control) address from an existing
     * flexible IP to a different flexible IP.
     *
     * @param request - The request {@link MoveMACAddrRequest}
     * @returns A Promise of FlexibleIP
     */
    moveMACAddr: (request: Readonly<MoveMACAddrRequest>) => Promise<FlexibleIP>;
    /**
     * Detach a given virtual MAC address from an existing flexible IP. Detach a
     * given MAC (Media Access Control) address from an existing flexible IP.
     *
     * @param request - The request {@link DeleteMACAddrRequest}
     */
    deleteMACAddr: (request: Readonly<DeleteMACAddrRequest>) => Promise<void>;
}

/** Lists transient statutes of the enum {@link FlexibleIPStatus}. */
declare const FLEXIBLE_IP_TRANSIENT_STATUSES: FlexibleIPStatus[];
/** Lists transient statutes of the enum {@link MACAddressStatus}. */
declare const MAC_ADDRESS_TRANSIENT_STATUSES: MACAddressStatus[];

declare const ListFlexibleIPsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};

declare const validationRules_gen$9_ListFlexibleIPsRequest: typeof ListFlexibleIPsRequest;
declare namespace validationRules_gen$9 {
  export { validationRules_gen$9_ListFlexibleIPsRequest as ListFlexibleIPsRequest };
}

type index_gen$l_AttachFlexibleIPRequest = AttachFlexibleIPRequest;
type index_gen$l_AttachFlexibleIPsResponse = AttachFlexibleIPsResponse;
type index_gen$l_CreateFlexibleIPRequest = CreateFlexibleIPRequest;
type index_gen$l_DeleteFlexibleIPRequest = DeleteFlexibleIPRequest;
type index_gen$l_DeleteMACAddrRequest = DeleteMACAddrRequest;
type index_gen$l_DetachFlexibleIPRequest = DetachFlexibleIPRequest;
type index_gen$l_DetachFlexibleIPsResponse = DetachFlexibleIPsResponse;
type index_gen$l_DuplicateMACAddrRequest = DuplicateMACAddrRequest;
declare const index_gen$l_FLEXIBLE_IP_TRANSIENT_STATUSES: typeof FLEXIBLE_IP_TRANSIENT_STATUSES;
type index_gen$l_FlexibleIP = FlexibleIP;
type index_gen$l_FlexibleIPStatus = FlexibleIPStatus;
type index_gen$l_GenerateMACAddrRequest = GenerateMACAddrRequest;
type index_gen$l_GetFlexibleIPRequest = GetFlexibleIPRequest;
type index_gen$l_ListFlexibleIPsRequestOrderBy = ListFlexibleIPsRequestOrderBy;
type index_gen$l_ListFlexibleIPsResponse = ListFlexibleIPsResponse;
type index_gen$l_MACAddress = MACAddress;
type index_gen$l_MACAddressStatus = MACAddressStatus;
type index_gen$l_MACAddressType = MACAddressType;
declare const index_gen$l_MAC_ADDRESS_TRANSIENT_STATUSES: typeof MAC_ADDRESS_TRANSIENT_STATUSES;
type index_gen$l_MoveMACAddrRequest = MoveMACAddrRequest;
type index_gen$l_UpdateFlexibleIPRequest = UpdateFlexibleIPRequest;
declare namespace index_gen$l {
  export { API$m as API, type index_gen$l_AttachFlexibleIPRequest as AttachFlexibleIPRequest, type index_gen$l_AttachFlexibleIPsResponse as AttachFlexibleIPsResponse, type index_gen$l_CreateFlexibleIPRequest as CreateFlexibleIPRequest, type index_gen$l_DeleteFlexibleIPRequest as DeleteFlexibleIPRequest, type index_gen$l_DeleteMACAddrRequest as DeleteMACAddrRequest, type index_gen$l_DetachFlexibleIPRequest as DetachFlexibleIPRequest, type index_gen$l_DetachFlexibleIPsResponse as DetachFlexibleIPsResponse, type index_gen$l_DuplicateMACAddrRequest as DuplicateMACAddrRequest, index_gen$l_FLEXIBLE_IP_TRANSIENT_STATUSES as FLEXIBLE_IP_TRANSIENT_STATUSES, type index_gen$l_FlexibleIP as FlexibleIP, type index_gen$l_FlexibleIPStatus as FlexibleIPStatus, type index_gen$l_GenerateMACAddrRequest as GenerateMACAddrRequest, type index_gen$l_GetFlexibleIPRequest as GetFlexibleIPRequest, type ListFlexibleIPsRequest$1 as ListFlexibleIPsRequest, type index_gen$l_ListFlexibleIPsRequestOrderBy as ListFlexibleIPsRequestOrderBy, type index_gen$l_ListFlexibleIPsResponse as ListFlexibleIPsResponse, type index_gen$l_MACAddress as MACAddress, type index_gen$l_MACAddressStatus as MACAddressStatus, type index_gen$l_MACAddressType as MACAddressType, index_gen$l_MAC_ADDRESS_TRANSIENT_STATUSES as MAC_ADDRESS_TRANSIENT_STATUSES, type index_gen$l_MoveMACAddrRequest as MoveMACAddrRequest, type index_gen$l_UpdateFlexibleIPRequest as UpdateFlexibleIPRequest, validationRules_gen$9 as ValidationRules };
}

declare namespace index$o {
  export { index_gen$l as v1alpha1 };
}

type CronStatus = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked' | 'creating' | 'pending';
type DomainStatus$1 = 'unknown' | 'ready' | 'deleting' | 'error' | 'creating' | 'pending';
type FunctionHttpOption = 'unknown_http_option' | 'enabled' | 'redirected';
type FunctionPrivacy = 'unknown_privacy' | 'public' | 'private';
type FunctionRuntime = 'unknown_runtime' | 'golang' | 'python' | 'python3' | 'node8' | 'node10' | 'node14' | 'node16' | 'node17' | 'python37' | 'python38' | 'python39' | 'python310' | 'go113' | 'go117' | 'go118' | 'node18' | 'rust165' | 'go119' | 'python311' | 'php82' | 'node19' | 'go120' | 'node20' | 'go121';
type FunctionStatus = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked' | 'creating' | 'pending' | 'created';
type ListCronsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListDomainsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'hostname_asc' | 'hostname_desc';
type ListFunctionsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListLogsRequestOrderBy$1 = 'timestamp_desc' | 'timestamp_asc';
type ListNamespacesRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListTokensRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListTriggersRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type LogStream = 'unknown' | 'stdout' | 'stderr';
type NamespaceStatus$1 = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked' | 'creating' | 'pending';
type RuntimeStatus = 'unknown_status' | 'beta' | 'available' | 'deprecated' | 'end_of_support' | 'end_of_life';
type TokenStatus = 'unknown' | 'ready' | 'deleting' | 'error' | 'creating';
type TriggerInputType = 'unknown_input_type' | 'sqs' | 'scw_sqs' | 'nats' | 'scw_nats';
type TriggerStatus = 'unknown_status' | 'ready' | 'deleting' | 'error' | 'creating' | 'pending';
interface SecretHashedValue {
    key: string;
    hashedValue: string;
}
interface TriggerMnqNatsClientConfig {
    /** @deprecated */
    mnqNamespaceId?: string;
    /** Name of the NATS subject the trigger listens to. */
    subject: string;
    /** ID of the Messaging and Queuing NATS account. */
    mnqNatsAccountId: string;
    /** ID of the Messaging and Queuing project. */
    mnqProjectId: string;
    /** Currently, only the `fr-par` region is available. */
    mnqRegion: string;
    /**
     * ID of the Messaging and Queuing credentials used to subscribe to the NATS
     * subject.
     */
    mnqCredentialId?: string;
}
interface TriggerMnqSqsClientConfig {
    /** @deprecated */
    mnqNamespaceId?: string;
    /** Name of the SQS queue the trigger listens to. */
    queue: string;
    /** ID of the Messaging and Queuing project. */
    mnqProjectId: string;
    /** Currently, only the `fr-par` region is available. */
    mnqRegion: string;
    /**
     * ID of the Messaging and Queuing credentials used to read from the SQS
     * queue.
     */
    mnqCredentialId?: string;
}
interface TriggerSqsClientConfig {
    endpoint: string;
    queueUrl: string;
    accessKey: string;
    secretKey: string;
}
interface Secret$1 {
    key: string;
    value?: string;
}
interface CreateTriggerRequestMnqNatsClientConfig$1 {
    /** @deprecated */
    mnqNamespaceId?: string;
    /** Name of the NATS subject the trigger should listen to. */
    subject: string;
    /** ID of the Messaging and Queuing NATS account. */
    mnqNatsAccountId: string;
    /** ID of the Messaging and Queuing project. */
    mnqProjectId: string;
    /** Currently, only the `fr-par` region is available. */
    mnqRegion: string;
}
interface CreateTriggerRequestMnqSqsClientConfig$1 {
    /** @deprecated */
    mnqNamespaceId?: string;
    /** Name of the SQS queue the trigger should listen to. */
    queue: string;
    /** You must have activated SQS on this project. */
    mnqProjectId: string;
    /** Currently, only the `fr-par` region is available. */
    mnqRegion: string;
}
interface CreateTriggerRequestSqsClientConfig {
    endpoint: string;
    queueUrl: string;
    accessKey: string;
    secretKey: string;
}
interface Cron {
    /** UUID of the cron. */
    id: string;
    /** UUID of the function the cron applies to. */
    functionId: string;
    /** Schedule of the cron. */
    schedule: string;
    /** Arguments to pass with the cron. */
    args?: Record<string, unknown>;
    /** Status of the cron. */
    status: CronStatus;
    /** Name of the cron. */
    name: string;
}
interface Domain$1 {
    /** UUID of the domain. */
    id: string;
    /** Hostname associated with the function. */
    hostname: string;
    /** UUID of the function the domain is associated with. */
    functionId: string;
    /** URL of the function. */
    url: string;
    /** State of the doamin. */
    status: DomainStatus$1;
    /** Error message if the domain is in "error" state. */
    errorMessage?: string;
}
interface Runtime$1 {
    name: string;
    language: string;
    version: string;
    defaultHandler: string;
    codeSample: string;
    status: RuntimeStatus;
    statusMessage: string;
    extension: string;
    implementation: string;
    logoUrl: string;
}
interface Function {
    /** UUID of the function. */
    id: string;
    /** Name of the function. */
    name: string;
    /** UUID of the namespace the function belongs to. */
    namespaceId: string;
    /** Status of the function. */
    status: FunctionStatus;
    /** Environment variables of the function. */
    environmentVariables: Record<string, string>;
    /** Minimum number of instances to scale the function to. */
    minScale: number;
    /** Maximum number of instances to scale the function to. */
    maxScale: number;
    /** Runtime of the function. */
    runtime: FunctionRuntime;
    /** Memory limit of the function in MB. */
    memoryLimit: number;
    /** CPU limit of the function. */
    cpuLimit: number;
    /** Request processing time limit for the function. */
    timeout?: string;
    /** Handler to use for the function. */
    handler: string;
    /** Error message if the function is in "error" state. */
    errorMessage?: string;
    /** Description of the current build step. */
    buildMessage?: string;
    /** Privacy setting of the function. */
    privacy: FunctionPrivacy;
    /** Description of the function. */
    description?: string;
    /** Domain name associated with the function. */
    domainName: string;
    /** Secret environment variables of the function. */
    secretEnvironmentVariables: SecretHashedValue[];
    /** Region in which the function is deployed. */
    region: Region;
    /**
     * Possible values:
     *
     * - Redirected: Responds to HTTP request with a 301 redirect to ask the clients
     *   to use HTTPS.
     * - Enabled: Serve both HTTP and HTTPS traffic.
     */
    httpOption: FunctionHttpOption;
    runtimeMessage: string;
}
interface Log$1 {
    /** Message of the log. */
    message: string;
    /** Timestamp of the log. */
    timestamp?: Date;
    /** UUID of the log. */
    id: string;
    /** Severity of the log (info, debug, error etc.). */
    level: string;
    /** Source of the log (core runtime or user code). */
    source: string;
    /** Can be stdout or stderr. */
    stream: LogStream;
}
interface Namespace$1 {
    /** UUID of the namespace. */
    id: string;
    /** Name of the namespace. */
    name: string;
    /** Environment variables of the namespace. */
    environmentVariables: Record<string, string>;
    /** UUID of the Organization the namespace belongs to. */
    organizationId: string;
    /** UUID of the Project the namespace belongs to. */
    projectId: string;
    /** Status of the namespace. */
    status: NamespaceStatus$1;
    /** UUID of the registry namespace. */
    registryNamespaceId: string;
    /** Error message if the namespace is in "error" state. */
    errorMessage?: string;
    /** Registry endpoint of the namespace. */
    registryEndpoint: string;
    /** Description of the namespace. */
    description?: string;
    /** Secret environment variables of the namespace. */
    secretEnvironmentVariables: SecretHashedValue[];
    /** Region in which the namespace is located. */
    region: Region;
}
interface Token {
    /** UUID of the token. */
    id: string;
    /** String of the token. */
    token: string;
    /**
     * UUID of the function the token is associated with.
     *
     * One-of ('scope'): at most one of 'functionId', 'namespaceId' could be set.
     */
    functionId?: string;
    /**
     * UUID of the namespace the token is assoicated with.
     *
     * One-of ('scope'): at most one of 'functionId', 'namespaceId' could be set.
     */
    namespaceId?: string;
    /** @deprecated Public key of the token. */
    publicKey?: string;
    /** Status of the token. */
    status: TokenStatus;
    /** Description of the token. */
    description?: string;
    /** Date on which the token expires. */
    expiresAt?: Date;
}
interface Trigger {
    /** ID of the trigger. */
    id: string;
    /** Name of the trigger. */
    name: string;
    /** Description of the trigger. */
    description: string;
    /** ID of the function to trigger. */
    functionId: string;
    /** Type of the input. */
    inputType: TriggerInputType;
    /** Status of the trigger. */
    status: TriggerStatus;
    /** Error message of the trigger. */
    errorMessage?: string;
    /**
     * Configuration for a Scaleway Messaging and Queuing SQS queue.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    scwSqsConfig?: TriggerMnqSqsClientConfig;
    /**
     * Configuration for a Scaleway Messaging and Queuing NATS subject.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    scwNatsConfig?: TriggerMnqNatsClientConfig;
    /**
     * Configuration for an AWS SQS queue.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    sqsConfig?: TriggerSqsClientConfig;
}
interface UpdateTriggerRequestSqsClientConfig {
    accessKey?: string;
    secretKey?: string;
}
type CreateCronRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the function to use the cron with. */
    functionId: string;
    /** Schedule of the cron in UNIX cron format. */
    schedule: string;
    /** Arguments to use with the cron. */
    args?: Record<string, unknown>;
    /** Name of the cron. */
    name?: string;
};
type CreateDomainRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hostame to create. */
    hostname: string;
    /** UUID of the function to associate the domain with. */
    functionId: string;
};
type CreateFunctionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the function to create. */
    name?: string;
    /** UUID of the namespace the function will be created in. */
    namespaceId: string;
    /** Environment variables of the function. */
    environmentVariables?: Record<string, string>;
    /** Minumum number of instances to scale the function to. */
    minScale?: number;
    /** Maximum number of instances to scale the function to. */
    maxScale?: number;
    /** Runtime to use with the function. */
    runtime?: FunctionRuntime;
    /** Memory limit of the function in MB. */
    memoryLimit?: number;
    /** Request processing time limit for the function. */
    timeout?: string;
    /** Handler to use with the function. */
    handler?: string;
    /** Privacy setting of the function. */
    privacy?: FunctionPrivacy;
    /** Description of the function. */
    description?: string;
    secretEnvironmentVariables?: Secret$1[];
    /**
     * Possible values:
     *
     * - Redirected: Responds to HTTP request with a 301 redirect to ask the clients
     *   to use HTTPS.
     * - Enabled: Serve both HTTP and HTTPS traffic.
     */
    httpOption?: FunctionHttpOption;
};
type CreateNamespaceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    name?: string;
    /** Environment variables of the namespace. */
    environmentVariables?: Record<string, string>;
    /** UUID of the project in which the namespace will be created. */
    projectId?: string;
    /** Description of the namespace. */
    description?: string;
    /** Secret environment variables of the namespace. */
    secretEnvironmentVariables?: Secret$1[];
};
type CreateTokenRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * UUID of the function to associate the token with.
     *
     * One-of ('scope'): at most one of 'functionId', 'namespaceId' could be set.
     */
    functionId?: string;
    /**
     * UUID of the namespace to associate the token with.
     *
     * One-of ('scope'): at most one of 'functionId', 'namespaceId' could be set.
     */
    namespaceId?: string;
    /** Description of the token. */
    description?: string;
    /** Date on which the token expires. */
    expiresAt?: Date;
};
type CreateTriggerRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the trigger. */
    name: string;
    /** ID of the function to trigger. */
    functionId: string;
    /** Description of the trigger. */
    description?: string;
    /**
     * Configuration for a Scaleway Messaging and Queuing SQS queue.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    scwSqsConfig?: CreateTriggerRequestMnqSqsClientConfig$1;
    /**
     * Configuration for a Scaleway Messaging and Queuing NATS subject.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    scwNatsConfig?: CreateTriggerRequestMnqNatsClientConfig$1;
    /**
     * Configuration for an AWS SQS queue.
     *
     * One-of ('config'): at most one of 'scwSqsConfig', 'scwNatsConfig',
     * 'sqsConfig' could be set.
     */
    sqsConfig?: CreateTriggerRequestSqsClientConfig;
};
type DeleteCronRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the cron to delete. */
    cronId: string;
};
type DeleteDomainRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the domain to delete. */
    domainId: string;
};
type DeleteFunctionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the function to delete. */
    functionId: string;
};
type DeleteNamespaceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespace. */
    namespaceId: string;
};
type DeleteTokenRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the token to delete. */
    tokenId: string;
};
type DeleteTriggerRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the trigger to delete. */
    triggerId: string;
};
type DeployFunctionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the function to deploy. */
    functionId: string;
};
interface DownloadURL {
    url: string;
    headers: Record<string, string[]>;
}
type GetCronRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the cron to get. */
    cronId: string;
};
type GetDomainRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the domain to get. */
    domainId: string;
};
type GetFunctionDownloadURLRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the function to get the the download URL for. */
    functionId: string;
};
type GetFunctionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the function. */
    functionId: string;
};
type GetFunctionUploadURLRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the function to get the upload URL for. */
    functionId: string;
    /** Size of the archive to upload in bytes. */
    contentLength: number;
};
type GetNamespaceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespace. */
    namespaceId: string;
};
type GetTokenRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the token to get. */
    tokenId: string;
};
type GetTriggerRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the trigger to get. */
    triggerId: string;
};
type IssueJWTRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** One-of ('scope'): at most one of 'functionId', 'namespaceId' could be set. */
    functionId?: string;
    /** One-of ('scope'): at most one of 'functionId', 'namespaceId' could be set. */
    namespaceId?: string;
    expiresAt?: Date;
};
type ListCronsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of crons per page. */
    pageSize?: number;
    /** Order of the crons. */
    orderBy?: ListCronsRequestOrderBy;
    /** UUID of the function. */
    functionId: string;
};
interface ListCronsResponse {
    /** Array of crons. */
    crons: Cron[];
    /** Total number of crons. */
    totalCount: number;
}
type ListDomainsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of domains per page. */
    pageSize?: number;
    /** Order of the domains. */
    orderBy?: ListDomainsRequestOrderBy;
    /** UUID of the function the domain is assoicated with. */
    functionId: string;
};
interface ListDomainsResponse$1 {
    /** Array of domains. */
    domains: Domain$1[];
    /** Total number of domains. */
    totalCount: number;
}
type ListFunctionRuntimesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
};
interface ListFunctionRuntimesResponse {
    /** Array of runtimes available. */
    runtimes: Runtime$1[];
    /** Total number of runtimes. */
    totalCount: number;
}
type ListFunctionsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of functions per page. */
    pageSize?: number;
    /** Order of the functions. */
    orderBy?: ListFunctionsRequestOrderBy;
    /** UUID of the namespace the function belongs to. */
    namespaceId: string;
    /** Name of the function. */
    name?: string;
    /** UUID of the Organziation the function belongs to. */
    organizationId?: string;
    /** UUID of the Project the function belongs to. */
    projectId?: string;
};
interface ListFunctionsResponse {
    /** Array of functions. */
    functions: Function[];
    /** Total number of functions. */
    totalCount: number;
}
type ListLogsRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the function to get the logs for. */
    functionId: string;
    /** Page number. */
    page?: number;
    /** Number of logs per page. */
    pageSize?: number;
    /** Order of the logs. */
    orderBy?: ListLogsRequestOrderBy$1;
};
interface ListLogsResponse$1 {
    /** Array of logs. */
    logs: Log$1[];
    /** Total number of logs. */
    totalCount: number;
}
type ListNamespacesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of namespaces per page. */
    pageSize?: number;
    /** Order of the namespaces. */
    orderBy?: ListNamespacesRequestOrderBy$1;
    /** Name of the namespace. */
    name?: string;
    /** UUID of the Organization the namespace belongs to. */
    organizationId?: string;
    /** UUID of the Project the namespace belongs to. */
    projectId?: string;
};
interface ListNamespacesResponse$1 {
    namespaces: Namespace$1[];
    /** Total number of namespaces. */
    totalCount: number;
}
type ListTokensRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number. */
    page?: number;
    /** Number of tokens per page. */
    pageSize?: number;
    /** Sort order for the tokens. */
    orderBy?: ListTokensRequestOrderBy;
    /** UUID of the function the token is assoicated with. */
    functionId?: string;
    /** UUID of the namespace the token is associated with. */
    namespaceId?: string;
};
interface ListTokensResponse {
    tokens: Token[];
    totalCount: number;
}
type ListTriggersRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number to return. */
    page?: number;
    /** Maximum number of triggers to return per page. */
    pageSize?: number;
    /** Order in which to return results. */
    orderBy?: ListTriggersRequestOrderBy;
    /**
     * ID of the function the triggers belongs to.
     *
     * One-of ('scope'): at most one of 'functionId', 'namespaceId', 'projectId'
     * could be set.
     */
    functionId?: string;
    /**
     * ID of the namespace the triggers belongs to.
     *
     * One-of ('scope'): at most one of 'functionId', 'namespaceId', 'projectId'
     * could be set.
     */
    namespaceId?: string;
    /**
     * ID of the project the triggers belongs to.
     *
     * One-of ('scope'): at most one of 'functionId', 'namespaceId', 'projectId'
     * could be set.
     */
    projectId?: string;
};
interface ListTriggersResponse {
    /** Total count of existing triggers (matching any filters specified). */
    totalCount: number;
    /** Triggers on this page. */
    triggers: Trigger[];
}
type UpdateCronRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the cron to update. */
    cronId: string;
    /** UUID of the function to use the cron with. */
    functionId?: string;
    /** Schedule of the cron in UNIX cron format. */
    schedule?: string;
    /** Arguments to use with the cron. */
    args?: Record<string, unknown>;
    /** Name of the cron. */
    name?: string;
};
type UpdateFunctionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the function to update. */
    functionId: string;
    /** Environment variables of the function to update. */
    environmentVariables?: Record<string, string>;
    /** Minumum number of instances to scale the function to. */
    minScale?: number;
    /** Maximum number of instances to scale the function to. */
    maxScale?: number;
    /** Runtime to use with the function. */
    runtime?: FunctionRuntime;
    /** Memory limit of the function in MB. */
    memoryLimit?: number;
    /** Processing time limit for the function. */
    timeout?: string;
    /** Redeploy failed function. */
    redeploy?: boolean;
    /** Handler to use with the function. */
    handler?: string;
    /** Privacy setting of the function. */
    privacy?: FunctionPrivacy;
    /** Description of the function. */
    description?: string;
    /** Secret environment variables of the function. */
    secretEnvironmentVariables?: Secret$1[];
    /**
     * Possible values:
     *
     * - Redirected: Responds to HTTP request with a 301 redirect to ask the clients
     *   to use HTTPS.
     * - Enabled: Serve both HTTP and HTTPS traffic.
     */
    httpOption?: FunctionHttpOption;
};
type UpdateNamespaceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespapce. */
    namespaceId: string;
    /** Environment variables of the namespace. */
    environmentVariables?: Record<string, string>;
    /** Description of the namespace. */
    description?: string;
    /** Secret environment variables of the namespace. */
    secretEnvironmentVariables?: Secret$1[];
};
type UpdateTriggerRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the trigger to update. */
    triggerId: string;
    /** Name of the trigger. */
    name?: string;
    /** Description of the trigger. */
    description?: string;
    /**
     * Configuration for an AWS SQS queue.
     *
     * One-of ('config'): at most one of 'sqsConfig' could be set.
     */
    sqsConfig?: UpdateTriggerRequestSqsClientConfig;
};
interface UploadURL {
    /** Upload URL to upload the function to. */
    url: string;
    /** HTTP headers. */
    headers: Record<string, string[]>;
}

/** Serverless Functions API. */
declare class API$l extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListNamespaces: (request?: Readonly<ListNamespacesRequest$1>) => Promise<ListNamespacesResponse$1>;
    /**
     * List all your namespaces. List all existing namespaces in the specified
     * region.
     *
     * @param request - The request {@link ListNamespacesRequest}
     * @returns A Promise of ListNamespacesResponse
     */
    listNamespaces: (request?: Readonly<ListNamespacesRequest$1>) => Promise<ListNamespacesResponse$1> & {
        all: () => Promise<Namespace$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Namespace$1[], void, void>;
    };
    /**
     * Get a namespace. Get the namespace associated with the specified ID.
     *
     * @param request - The request {@link GetNamespaceRequest}
     * @returns A Promise of Namespace
     */
    getNamespace: (request: Readonly<GetNamespaceRequest$1>) => Promise<Namespace$1>;
    /**
     * Waits for {@link Namespace} to be in a final state.
     *
     * @param request - The request {@link GetNamespaceRequest}
     * @param options - The waiting options
     * @returns A Promise of Namespace
     */
    waitForNamespace: (request: Readonly<GetNamespaceRequest$1>, options?: Readonly<WaitForOptions<Namespace$1>>) => Promise<Namespace$1>;
    /**
     * Create a new namespace. Create a new namespace in a specified Organization
     * or Project.
     *
     * @param request - The request {@link CreateNamespaceRequest}
     * @returns A Promise of Namespace
     */
    createNamespace: (request?: Readonly<CreateNamespaceRequest$1>) => Promise<Namespace$1>;
    /**
     * Update an existing namespace. Update the namespace associated with the
     * specified ID.
     *
     * @param request - The request {@link UpdateNamespaceRequest}
     * @returns A Promise of Namespace
     */
    updateNamespace: (request: Readonly<UpdateNamespaceRequest$1>) => Promise<Namespace$1>;
    /**
     * Delete an existing namespace. Delete the namespace associated with the
     * specified ID.
     *
     * @param request - The request {@link DeleteNamespaceRequest}
     * @returns A Promise of Namespace
     */
    deleteNamespace: (request: Readonly<DeleteNamespaceRequest$1>) => Promise<Namespace$1>;
    protected pageOfListFunctions: (request: Readonly<ListFunctionsRequest>) => Promise<ListFunctionsResponse>;
    /**
     * List all your functions.
     *
     * @param request - The request {@link ListFunctionsRequest}
     * @returns A Promise of ListFunctionsResponse
     */
    listFunctions: (request: Readonly<ListFunctionsRequest>) => Promise<ListFunctionsResponse> & {
        all: () => Promise<Function[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Function[], void, void>;
    };
    /**
     * Get a function. Get the function associated with the specified ID.
     *
     * @param request - The request {@link GetFunctionRequest}
     * @returns A Promise of Function
     */
    getFunction: (request: Readonly<GetFunctionRequest>) => Promise<Function>;
    /**
     * Waits for {@link Function} to be in a final state.
     *
     * @param request - The request {@link GetFunctionRequest}
     * @param options - The waiting options
     * @returns A Promise of Function
     */
    waitForFunction: (request: Readonly<GetFunctionRequest>, options?: Readonly<WaitForOptions<Function>>) => Promise<Function>;
    /**
     * Create a new function. Create a new function in the specified region for a
     * specified Organization or Project.
     *
     * @param request - The request {@link CreateFunctionRequest}
     * @returns A Promise of Function
     */
    createFunction: (request: Readonly<CreateFunctionRequest>) => Promise<Function>;
    /**
     * Update an existing function. Update the function associated with the
     * specified ID.
     *
     * @param request - The request {@link UpdateFunctionRequest}
     * @returns A Promise of Function
     */
    updateFunction: (request: Readonly<UpdateFunctionRequest>) => Promise<Function>;
    /**
     * Delete a function. Delete the function associated with the specified ID.
     *
     * @param request - The request {@link DeleteFunctionRequest}
     * @returns A Promise of Function
     */
    deleteFunction: (request: Readonly<DeleteFunctionRequest>) => Promise<Function>;
    /**
     * Deploy a function. Deploy a function associated with the specified ID.
     *
     * @param request - The request {@link DeployFunctionRequest}
     * @returns A Promise of Function
     */
    deployFunction: (request: Readonly<DeployFunctionRequest>) => Promise<Function>;
    /**
     * List function runtimes. List available function runtimes.
     *
     * @param request - The request {@link ListFunctionRuntimesRequest}
     * @returns A Promise of ListFunctionRuntimesResponse
     */
    listFunctionRuntimes: (request?: Readonly<ListFunctionRuntimesRequest>) => Promise<ListFunctionRuntimesResponse>;
    /**
     * Get an upload URL of a function. Get an upload URL of a function associated
     * with the specified ID.
     *
     * @param request - The request {@link GetFunctionUploadURLRequest}
     * @returns A Promise of UploadURL
     */
    getFunctionUploadURL: (request: Readonly<GetFunctionUploadURLRequest>) => Promise<UploadURL>;
    /**
     * Get a download URL of a function. Get a download URL for a function
     * associated with the specified ID.
     *
     * @param request - The request {@link GetFunctionDownloadURLRequest}
     * @returns A Promise of DownloadURL
     */
    getFunctionDownloadURL: (request: Readonly<GetFunctionDownloadURLRequest>) => Promise<DownloadURL>;
    protected pageOfListCrons: (request: Readonly<ListCronsRequest>) => Promise<ListCronsResponse>;
    /**
     * List all crons. List all the cronjobs in a specified region.
     *
     * @param request - The request {@link ListCronsRequest}
     * @returns A Promise of ListCronsResponse
     */
    listCrons: (request: Readonly<ListCronsRequest>) => Promise<ListCronsResponse> & {
        all: () => Promise<Cron[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Cron[], void, void>;
    };
    /**
     * Get a cron. Get the cron associated with the specified ID.
     *
     * @param request - The request {@link GetCronRequest}
     * @returns A Promise of Cron
     */
    getCron: (request: Readonly<GetCronRequest>) => Promise<Cron>;
    /**
     * Waits for {@link Cron} to be in a final state.
     *
     * @param request - The request {@link GetCronRequest}
     * @param options - The waiting options
     * @returns A Promise of Cron
     */
    waitForCron: (request: Readonly<GetCronRequest>, options?: Readonly<WaitForOptions<Cron>>) => Promise<Cron>;
    /**
     * Create a new cron. Create a new cronjob for a function with the specified
     * ID.
     *
     * @param request - The request {@link CreateCronRequest}
     * @returns A Promise of Cron
     */
    createCron: (request: Readonly<CreateCronRequest>) => Promise<Cron>;
    /**
     * Update an existing cron. Update the cron associated with the specified ID.
     *
     * @param request - The request {@link UpdateCronRequest}
     * @returns A Promise of Cron
     */
    updateCron: (request: Readonly<UpdateCronRequest>) => Promise<Cron>;
    /**
     * Delete an existing cron. Delete the cron associated with the specified ID.
     *
     * @param request - The request {@link DeleteCronRequest}
     * @returns A Promise of Cron
     */
    deleteCron: (request: Readonly<DeleteCronRequest>) => Promise<Cron>;
    protected pageOfListLogs: (request: Readonly<ListLogsRequest$2>) => Promise<ListLogsResponse$1>;
    /**
     * Deprecated (replaced by
     * [Cockpit](https://www.scaleway.com/en/developers/api/cockpit/)). List
     * application logs. Deprecated (replaced by
     * [Cockpit](https://www.scaleway.com/en/developers/api/cockpit/)). List the
     * application logs of the function with the specified ID.
     *
     * @deprecated
     * @param request - The request {@link ListLogsRequest}
     * @returns A Promise of ListLogsResponse
     */
    listLogs: (request: Readonly<ListLogsRequest$2>) => Promise<ListLogsResponse$1> & {
        all: () => Promise<Log$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Log$1[], void, void>;
    };
    protected pageOfListDomains: (request: Readonly<ListDomainsRequest$1>) => Promise<ListDomainsResponse$1>;
    /**
     * List all domain name bindings. List all domain name bindings in a specified
     * region.
     *
     * @param request - The request {@link ListDomainsRequest}
     * @returns A Promise of ListDomainsResponse
     */
    listDomains: (request: Readonly<ListDomainsRequest$1>) => Promise<ListDomainsResponse$1> & {
        all: () => Promise<Domain$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Domain$1[], void, void>;
    };
    /**
     * Get a domain name binding. Get a domain name binding for the function with
     * the specified ID.
     *
     * @param request - The request {@link GetDomainRequest}
     * @returns A Promise of Domain
     */
    getDomain: (request: Readonly<GetDomainRequest$1>) => Promise<Domain$1>;
    /**
     * Waits for {@link Domain} to be in a final state.
     *
     * @param request - The request {@link GetDomainRequest}
     * @param options - The waiting options
     * @returns A Promise of Domain
     */
    waitForDomain: (request: Readonly<GetDomainRequest$1>, options?: Readonly<WaitForOptions<Domain$1>>) => Promise<Domain$1>;
    /**
     * Create a domain name binding. Create a domain name binding for the function
     * with the specified ID.
     *
     * @param request - The request {@link CreateDomainRequest}
     * @returns A Promise of Domain
     */
    createDomain: (request: Readonly<CreateDomainRequest$1>) => Promise<Domain$1>;
    /**
     * Delete a domain name binding. Delete a domain name binding for the function
     * with the specified ID.
     *
     * @param request - The request {@link DeleteDomainRequest}
     * @returns A Promise of Domain
     */
    deleteDomain: (request: Readonly<DeleteDomainRequest>) => Promise<Domain$1>;
    /**
     * Create a JWT token. Deprecated in favor of CreateToken.
     *
     * @deprecated
     * @param request - The request {@link IssueJWTRequest}
     * @returns A Promise of Token
     */
    issueJWT: (request?: Readonly<IssueJWTRequest>) => Promise<Token>;
    /**
     * Create a new revocable token.
     *
     * @param request - The request {@link CreateTokenRequest}
     * @returns A Promise of Token
     */
    createToken: (request?: Readonly<CreateTokenRequest>) => Promise<Token>;
    /**
     * Get a token.
     *
     * @param request - The request {@link GetTokenRequest}
     * @returns A Promise of Token
     */
    getToken: (request: Readonly<GetTokenRequest>) => Promise<Token>;
    /**
     * Waits for {@link Token} to be in a final state.
     *
     * @param request - The request {@link GetTokenRequest}
     * @param options - The waiting options
     * @returns A Promise of Token
     */
    waitForToken: (request: Readonly<GetTokenRequest>, options?: Readonly<WaitForOptions<Token>>) => Promise<Token>;
    protected pageOfListTokens: (request?: Readonly<ListTokensRequest>) => Promise<ListTokensResponse>;
    /**
     * List all tokens.
     *
     * @param request - The request {@link ListTokensRequest}
     * @returns A Promise of ListTokensResponse
     */
    listTokens: (request?: Readonly<ListTokensRequest>) => Promise<ListTokensResponse> & {
        all: () => Promise<Token[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Token[], void, void>;
    };
    /**
     * Delete a token.
     *
     * @param request - The request {@link DeleteTokenRequest}
     * @returns A Promise of Token
     */
    deleteToken: (request: Readonly<DeleteTokenRequest>) => Promise<Token>;
    /**
     * Create a trigger. Create a new trigger for a specified function.
     *
     * @param request - The request {@link CreateTriggerRequest}
     * @returns A Promise of Trigger
     */
    createTrigger: (request: Readonly<CreateTriggerRequest$1>) => Promise<Trigger>;
    /**
     * Get a trigger. Get a trigger with a specified ID.
     *
     * @param request - The request {@link GetTriggerRequest}
     * @returns A Promise of Trigger
     */
    getTrigger: (request: Readonly<GetTriggerRequest>) => Promise<Trigger>;
    /**
     * Waits for {@link Trigger} to be in a final state.
     *
     * @param request - The request {@link GetTriggerRequest}
     * @param options - The waiting options
     * @returns A Promise of Trigger
     */
    waitForTrigger: (request: Readonly<GetTriggerRequest>, options?: Readonly<WaitForOptions<Trigger>>) => Promise<Trigger>;
    protected pageOfListTriggers: (request?: Readonly<ListTriggersRequest>) => Promise<ListTriggersResponse>;
    /**
     * List all triggers. List all triggers belonging to a specified Organization
     * or Project.
     *
     * @param request - The request {@link ListTriggersRequest}
     * @returns A Promise of ListTriggersResponse
     */
    listTriggers: (request?: Readonly<ListTriggersRequest>) => Promise<ListTriggersResponse> & {
        all: () => Promise<Trigger[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Trigger[], void, void>;
    };
    /**
     * Update a trigger. Update a trigger with a specified ID.
     *
     * @param request - The request {@link UpdateTriggerRequest}
     * @returns A Promise of Trigger
     */
    updateTrigger: (request: Readonly<UpdateTriggerRequest$1>) => Promise<Trigger>;
    /**
     * Delete a trigger. Delete a trigger with a specified ID.
     *
     * @param request - The request {@link DeleteTriggerRequest}
     * @returns A Promise of Trigger
     */
    deleteTrigger: (request: Readonly<DeleteTriggerRequest>) => Promise<Trigger>;
}

/** Lists transient statutes of the enum {@link CronStatus}. */
declare const CRON_TRANSIENT_STATUSES: CronStatus[];
/** Lists transient statutes of the enum {@link DomainStatus}. */
declare const DOMAIN_TRANSIENT_STATUSES$1: DomainStatus$1[];
/** Lists transient statutes of the enum {@link FunctionStatus}. */
declare const FUNCTION_TRANSIENT_STATUSES: FunctionStatus[];
/** Lists transient statutes of the enum {@link NamespaceStatus}. */
declare const NAMESPACE_TRANSIENT_STATUSES$1: NamespaceStatus$1[];
/** Lists transient statutes of the enum {@link TokenStatus}. */
declare const TOKEN_TRANSIENT_STATUSES: TokenStatus[];
/** Lists transient statutes of the enum {@link TriggerStatus}. */
declare const TRIGGER_TRANSIENT_STATUSES: TriggerStatus[];

declare const CreateTriggerRequest: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const CreateTriggerRequestMnqNatsClientConfig: {
    mnqRegion: {
        maxLength: number;
        minLength: number;
    };
    subject: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const CreateTriggerRequestMnqSqsClientConfig: {
    mnqRegion: {
        maxLength: number;
        minLength: number;
    };
    queue: {
        maxLength: number;
        minLength: number;
    };
};
declare const UpdateTriggerRequest: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
    };
};

declare const validationRules_gen$8_CreateTriggerRequest: typeof CreateTriggerRequest;
declare const validationRules_gen$8_CreateTriggerRequestMnqNatsClientConfig: typeof CreateTriggerRequestMnqNatsClientConfig;
declare const validationRules_gen$8_CreateTriggerRequestMnqSqsClientConfig: typeof CreateTriggerRequestMnqSqsClientConfig;
declare const validationRules_gen$8_UpdateTriggerRequest: typeof UpdateTriggerRequest;
declare namespace validationRules_gen$8 {
  export { validationRules_gen$8_CreateTriggerRequest as CreateTriggerRequest, validationRules_gen$8_CreateTriggerRequestMnqNatsClientConfig as CreateTriggerRequestMnqNatsClientConfig, validationRules_gen$8_CreateTriggerRequestMnqSqsClientConfig as CreateTriggerRequestMnqSqsClientConfig, validationRules_gen$8_UpdateTriggerRequest as UpdateTriggerRequest };
}

declare const index_gen$k_CRON_TRANSIENT_STATUSES: typeof CRON_TRANSIENT_STATUSES;
type index_gen$k_CreateCronRequest = CreateCronRequest;
type index_gen$k_CreateFunctionRequest = CreateFunctionRequest;
type index_gen$k_CreateTokenRequest = CreateTokenRequest;
type index_gen$k_CreateTriggerRequestSqsClientConfig = CreateTriggerRequestSqsClientConfig;
type index_gen$k_Cron = Cron;
type index_gen$k_CronStatus = CronStatus;
type index_gen$k_DeleteCronRequest = DeleteCronRequest;
type index_gen$k_DeleteDomainRequest = DeleteDomainRequest;
type index_gen$k_DeleteFunctionRequest = DeleteFunctionRequest;
type index_gen$k_DeleteTokenRequest = DeleteTokenRequest;
type index_gen$k_DeleteTriggerRequest = DeleteTriggerRequest;
type index_gen$k_DeployFunctionRequest = DeployFunctionRequest;
type index_gen$k_DownloadURL = DownloadURL;
declare const index_gen$k_FUNCTION_TRANSIENT_STATUSES: typeof FUNCTION_TRANSIENT_STATUSES;
type index_gen$k_Function = Function;
type index_gen$k_FunctionHttpOption = FunctionHttpOption;
type index_gen$k_FunctionPrivacy = FunctionPrivacy;
type index_gen$k_FunctionRuntime = FunctionRuntime;
type index_gen$k_FunctionStatus = FunctionStatus;
type index_gen$k_GetCronRequest = GetCronRequest;
type index_gen$k_GetFunctionDownloadURLRequest = GetFunctionDownloadURLRequest;
type index_gen$k_GetFunctionRequest = GetFunctionRequest;
type index_gen$k_GetFunctionUploadURLRequest = GetFunctionUploadURLRequest;
type index_gen$k_GetTokenRequest = GetTokenRequest;
type index_gen$k_GetTriggerRequest = GetTriggerRequest;
type index_gen$k_IssueJWTRequest = IssueJWTRequest;
type index_gen$k_ListCronsRequest = ListCronsRequest;
type index_gen$k_ListCronsRequestOrderBy = ListCronsRequestOrderBy;
type index_gen$k_ListCronsResponse = ListCronsResponse;
type index_gen$k_ListDomainsRequestOrderBy = ListDomainsRequestOrderBy;
type index_gen$k_ListFunctionRuntimesRequest = ListFunctionRuntimesRequest;
type index_gen$k_ListFunctionRuntimesResponse = ListFunctionRuntimesResponse;
type index_gen$k_ListFunctionsRequest = ListFunctionsRequest;
type index_gen$k_ListFunctionsRequestOrderBy = ListFunctionsRequestOrderBy;
type index_gen$k_ListFunctionsResponse = ListFunctionsResponse;
type index_gen$k_ListTokensRequest = ListTokensRequest;
type index_gen$k_ListTokensRequestOrderBy = ListTokensRequestOrderBy;
type index_gen$k_ListTokensResponse = ListTokensResponse;
type index_gen$k_ListTriggersRequest = ListTriggersRequest;
type index_gen$k_ListTriggersRequestOrderBy = ListTriggersRequestOrderBy;
type index_gen$k_ListTriggersResponse = ListTriggersResponse;
type index_gen$k_LogStream = LogStream;
type index_gen$k_RuntimeStatus = RuntimeStatus;
type index_gen$k_SecretHashedValue = SecretHashedValue;
declare const index_gen$k_TOKEN_TRANSIENT_STATUSES: typeof TOKEN_TRANSIENT_STATUSES;
declare const index_gen$k_TRIGGER_TRANSIENT_STATUSES: typeof TRIGGER_TRANSIENT_STATUSES;
type index_gen$k_Token = Token;
type index_gen$k_TokenStatus = TokenStatus;
type index_gen$k_Trigger = Trigger;
type index_gen$k_TriggerInputType = TriggerInputType;
type index_gen$k_TriggerMnqNatsClientConfig = TriggerMnqNatsClientConfig;
type index_gen$k_TriggerMnqSqsClientConfig = TriggerMnqSqsClientConfig;
type index_gen$k_TriggerSqsClientConfig = TriggerSqsClientConfig;
type index_gen$k_TriggerStatus = TriggerStatus;
type index_gen$k_UpdateCronRequest = UpdateCronRequest;
type index_gen$k_UpdateFunctionRequest = UpdateFunctionRequest;
type index_gen$k_UpdateTriggerRequestSqsClientConfig = UpdateTriggerRequestSqsClientConfig;
type index_gen$k_UploadURL = UploadURL;
declare namespace index_gen$k {
  export { API$l as API, index_gen$k_CRON_TRANSIENT_STATUSES as CRON_TRANSIENT_STATUSES, type index_gen$k_CreateCronRequest as CreateCronRequest, type CreateDomainRequest$1 as CreateDomainRequest, type index_gen$k_CreateFunctionRequest as CreateFunctionRequest, type CreateNamespaceRequest$1 as CreateNamespaceRequest, type index_gen$k_CreateTokenRequest as CreateTokenRequest, type CreateTriggerRequest$1 as CreateTriggerRequest, type CreateTriggerRequestMnqNatsClientConfig$1 as CreateTriggerRequestMnqNatsClientConfig, type CreateTriggerRequestMnqSqsClientConfig$1 as CreateTriggerRequestMnqSqsClientConfig, type index_gen$k_CreateTriggerRequestSqsClientConfig as CreateTriggerRequestSqsClientConfig, type index_gen$k_Cron as Cron, type index_gen$k_CronStatus as CronStatus, DOMAIN_TRANSIENT_STATUSES$1 as DOMAIN_TRANSIENT_STATUSES, type index_gen$k_DeleteCronRequest as DeleteCronRequest, type index_gen$k_DeleteDomainRequest as DeleteDomainRequest, type index_gen$k_DeleteFunctionRequest as DeleteFunctionRequest, type DeleteNamespaceRequest$1 as DeleteNamespaceRequest, type index_gen$k_DeleteTokenRequest as DeleteTokenRequest, type index_gen$k_DeleteTriggerRequest as DeleteTriggerRequest, type index_gen$k_DeployFunctionRequest as DeployFunctionRequest, type Domain$1 as Domain, type DomainStatus$1 as DomainStatus, type index_gen$k_DownloadURL as DownloadURL, index_gen$k_FUNCTION_TRANSIENT_STATUSES as FUNCTION_TRANSIENT_STATUSES, type index_gen$k_Function as Function, type index_gen$k_FunctionHttpOption as FunctionHttpOption, type index_gen$k_FunctionPrivacy as FunctionPrivacy, type index_gen$k_FunctionRuntime as FunctionRuntime, type index_gen$k_FunctionStatus as FunctionStatus, type index_gen$k_GetCronRequest as GetCronRequest, type GetDomainRequest$1 as GetDomainRequest, type index_gen$k_GetFunctionDownloadURLRequest as GetFunctionDownloadURLRequest, type index_gen$k_GetFunctionRequest as GetFunctionRequest, type index_gen$k_GetFunctionUploadURLRequest as GetFunctionUploadURLRequest, type GetNamespaceRequest$1 as GetNamespaceRequest, type index_gen$k_GetTokenRequest as GetTokenRequest, type index_gen$k_GetTriggerRequest as GetTriggerRequest, type index_gen$k_IssueJWTRequest as IssueJWTRequest, type index_gen$k_ListCronsRequest as ListCronsRequest, type index_gen$k_ListCronsRequestOrderBy as ListCronsRequestOrderBy, type index_gen$k_ListCronsResponse as ListCronsResponse, type ListDomainsRequest$1 as ListDomainsRequest, type index_gen$k_ListDomainsRequestOrderBy as ListDomainsRequestOrderBy, type ListDomainsResponse$1 as ListDomainsResponse, type index_gen$k_ListFunctionRuntimesRequest as ListFunctionRuntimesRequest, type index_gen$k_ListFunctionRuntimesResponse as ListFunctionRuntimesResponse, type index_gen$k_ListFunctionsRequest as ListFunctionsRequest, type index_gen$k_ListFunctionsRequestOrderBy as ListFunctionsRequestOrderBy, type index_gen$k_ListFunctionsResponse as ListFunctionsResponse, type ListLogsRequest$2 as ListLogsRequest, type ListLogsRequestOrderBy$1 as ListLogsRequestOrderBy, type ListLogsResponse$1 as ListLogsResponse, type ListNamespacesRequest$1 as ListNamespacesRequest, type ListNamespacesRequestOrderBy$1 as ListNamespacesRequestOrderBy, type ListNamespacesResponse$1 as ListNamespacesResponse, type index_gen$k_ListTokensRequest as ListTokensRequest, type index_gen$k_ListTokensRequestOrderBy as ListTokensRequestOrderBy, type index_gen$k_ListTokensResponse as ListTokensResponse, type index_gen$k_ListTriggersRequest as ListTriggersRequest, type index_gen$k_ListTriggersRequestOrderBy as ListTriggersRequestOrderBy, type index_gen$k_ListTriggersResponse as ListTriggersResponse, type Log$1 as Log, type index_gen$k_LogStream as LogStream, NAMESPACE_TRANSIENT_STATUSES$1 as NAMESPACE_TRANSIENT_STATUSES, type Namespace$1 as Namespace, type NamespaceStatus$1 as NamespaceStatus, type Runtime$1 as Runtime, type index_gen$k_RuntimeStatus as RuntimeStatus, type Secret$1 as Secret, type index_gen$k_SecretHashedValue as SecretHashedValue, index_gen$k_TOKEN_TRANSIENT_STATUSES as TOKEN_TRANSIENT_STATUSES, index_gen$k_TRIGGER_TRANSIENT_STATUSES as TRIGGER_TRANSIENT_STATUSES, type index_gen$k_Token as Token, type index_gen$k_TokenStatus as TokenStatus, type index_gen$k_Trigger as Trigger, type index_gen$k_TriggerInputType as TriggerInputType, type index_gen$k_TriggerMnqNatsClientConfig as TriggerMnqNatsClientConfig, type index_gen$k_TriggerMnqSqsClientConfig as TriggerMnqSqsClientConfig, type index_gen$k_TriggerSqsClientConfig as TriggerSqsClientConfig, type index_gen$k_TriggerStatus as TriggerStatus, type index_gen$k_UpdateCronRequest as UpdateCronRequest, type index_gen$k_UpdateFunctionRequest as UpdateFunctionRequest, type UpdateNamespaceRequest$1 as UpdateNamespaceRequest, type UpdateTriggerRequest$1 as UpdateTriggerRequest, type index_gen$k_UpdateTriggerRequestSqsClientConfig as UpdateTriggerRequestSqsClientConfig, type index_gen$k_UploadURL as UploadURL, validationRules_gen$8 as ValidationRules };
}

declare namespace index$n {
  export { index_gen$k as v1beta1 };
}

type BearerType = 'unknown_bearer_type' | 'user' | 'application';
type ListAPIKeysRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'expires_at_asc' | 'expires_at_desc' | 'access_key_asc' | 'access_key_desc';
type ListApplicationsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc';
type ListGroupsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc';
type ListJWTsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc';
type ListLogsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListPermissionSetsRequestOrderBy = 'name_asc' | 'name_desc' | 'created_at_asc' | 'created_at_desc';
type ListPoliciesRequestOrderBy = 'policy_name_asc' | 'policy_name_desc' | 'created_at_asc' | 'created_at_desc';
type ListQuotaRequestOrderBy = 'name_asc' | 'name_desc';
type ListSSHKeysRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc';
type ListUsersRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'email_asc' | 'email_desc' | 'last_login_asc' | 'last_login_desc';
type LogAction = 'unknown_action' | 'created' | 'updated' | 'deleted';
type LogResourceType = 'unknown_resource_type' | 'api_key' | 'user' | 'application' | 'group' | 'policy';
type PermissionSetScopeType = 'unknown_scope_type' | 'projects' | 'organization' | 'account_root_user';
type UserStatus = 'unknown_status' | 'invitation_pending' | 'activated';
type UserType = 'unknown_type' | 'guest' | 'owner';
interface RuleSpecs {
    /** Names of permission sets bound to the rule. */
    permissionSetNames?: string[];
    /**
     * List of Project IDs the rule is scoped to.
     *
     * One-of ('scope'): at most one of 'projectIds', 'organizationId' could be
     * set.
     */
    projectIds?: string[];
    /**
     * ID of Organization the rule is scoped to.
     *
     * One-of ('scope'): at most one of 'projectIds', 'organizationId' could be
     * set.
     */
    organizationId?: string;
}
interface JWT {
    /** JWT ID. */
    jti: string;
    /** ID of the user who issued the JWT. */
    issuerId: string;
    /** ID of the user targeted by the JWT. */
    audienceId: string;
    /** Creation date of the JWT. */
    createdAt?: Date;
    /** Last update date of the JWT. */
    updatedAt?: Date;
    /** Expiration date of the JWT. */
    expiresAt?: Date;
    /** IP address used during the creation of the JWT. */
    ip: string;
    /** User-agent used during the creation of the JWT. */
    userAgent: string;
}
interface APIKey {
    /** Access key of the API key. */
    accessKey: string;
    /** Secret key of the API Key. */
    secretKey?: string;
    /**
     * ID of application that bears the API key.
     *
     * One-of ('bearer'): at most one of 'applicationId', 'userId' could be set.
     */
    applicationId?: string;
    /**
     * ID of user that bears the API key.
     *
     * One-of ('bearer'): at most one of 'applicationId', 'userId' could be set.
     */
    userId?: string;
    /** Description of API key. */
    description: string;
    /** Date and time of API key creation. */
    createdAt?: Date;
    /** Date and time of last API key update. */
    updatedAt?: Date;
    /** Date and time of API key expiration. */
    expiresAt?: Date;
    /** Default Project ID specified for this API key. */
    defaultProjectId: string;
    /** Defines whether or not the API key is editable. */
    editable: boolean;
    /** IP address of the device that created the API key. */
    creationIp: string;
}
interface Application {
    /** ID of the application. */
    id: string;
    /** Name of the application. */
    name: string;
    /** Description of the application. */
    description: string;
    /** Date and time application was created. */
    createdAt?: Date;
    /** Date and time of last application update. */
    updatedAt?: Date;
    /** ID of the Organization. */
    organizationId: string;
    /** Defines whether or not the application is editable. */
    editable: boolean;
    /** Number of API keys attributed to the application. */
    nbApiKeys: number;
    /** Tags associated with the user. */
    tags: string[];
}
interface Group {
    /** ID of the group. */
    id: string;
    /** Date and time of group creation. */
    createdAt?: Date;
    /** Date and time of last group update. */
    updatedAt?: Date;
    /** ID of Organization linked to the group. */
    organizationId: string;
    /** Name of the group. */
    name: string;
    /** Description of the group. */
    description: string;
    /** IDs of users attached to this group. */
    userIds: string[];
    /** IDs of applications attached to this group. */
    applicationIds: string[];
    /** Tags associated to the group. */
    tags: string[];
}
interface Log {
    /** Log ID. */
    id: string;
    /** Creation date of the log. */
    createdAt?: Date;
    /** IP address of the HTTP request linked to the log. */
    ip: string;
    /** User-Agent of the HTTP request linked to the log. */
    userAgent: string;
    /** Action linked to the log. */
    action: LogAction;
    /** ID of the principal at the origin of the log. */
    bearerId: string;
    /** ID of Organization linked to the log. */
    organizationId: string;
    /** Type of the resource linked to the log. */
    resourceType: LogResourceType;
    /** ID of the resource linked to the log. */
    resourceId: string;
}
interface PermissionSet {
    /** Id of the permission set. */
    id: string;
    /** Name of the permission set. */
    name: string;
    /** Scope of the permission set. */
    scopeType: PermissionSetScopeType;
    /** Description of the permission set. */
    description: string;
    /** Categories of the permission set. */
    categories?: string[];
}
interface Policy {
    /** Id of the policy. */
    id: string;
    /** Name of the policy. */
    name: string;
    /** Description of the policy. */
    description: string;
    /** Organization ID of the policy. */
    organizationId: string;
    /** Date and time of policy creation. */
    createdAt?: Date;
    /** Date and time of last policy update. */
    updatedAt?: Date;
    /** Defines whether or not a policy is editable. */
    editable: boolean;
    /** Number of rules of the policy. */
    nbRules: number;
    /** Number of policy scopes. */
    nbScopes: number;
    /** Number of permission sets of the policy. */
    nbPermissionSets: number;
    /** Tags associated with the policy. */
    tags: string[];
    /**
     * ID of the user attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    userId?: string;
    /**
     * ID of the group attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    groupId?: string;
    /**
     * ID of the application attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    applicationId?: string;
    /**
     * Defines whether or not a policy is attributed to a principal.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    noPrincipal?: boolean;
}
interface Quotum {
    /** Name of the quota. */
    name: string;
    /**
     * Maximum limit of the quota.
     *
     * One-of ('value'): at most one of 'limit', 'unlimited' could be set.
     */
    limit?: number;
    /**
     * Defines whether or not the quota is unlimited.
     *
     * One-of ('value'): at most one of 'limit', 'unlimited' could be set.
     */
    unlimited?: boolean;
}
interface Rule {
    /** Id of rule. */
    id: string;
    /** Names of permission sets bound to the rule. */
    permissionSetNames?: string[];
    /** Permission_set_names have the same scope_type. */
    permissionSetsScopeType: PermissionSetScopeType;
    /**
     * List of Project IDs the rule is scoped to.
     *
     * One-of ('scope'): at most one of 'projectIds', 'organizationId',
     * 'accountRootUserId' could be set.
     */
    projectIds?: string[];
    /**
     * ID of Organization the rule is scoped to.
     *
     * One-of ('scope'): at most one of 'projectIds', 'organizationId',
     * 'accountRootUserId' could be set.
     */
    organizationId?: string;
    /**
     * ID of account root user the rule is scoped to.
     *
     * One-of ('scope'): at most one of 'projectIds', 'organizationId',
     * 'accountRootUserId' could be set.
     */
    accountRootUserId?: string;
}
interface SSHKey {
    /** ID of SSH key. */
    id: string;
    /** Name of SSH key. */
    name: string;
    /** Public key of SSH key. */
    publicKey: string;
    /** Fingerprint of the SSH key. */
    fingerprint: string;
    /** Creation date of SSH key. */
    createdAt?: Date;
    /** Last update date of SSH key. */
    updatedAt?: Date;
    /** ID of Organization linked to the SSH key. */
    organizationId: string;
    /** ID of Project linked to the SSH key. */
    projectId: string;
    /** SSH key status. */
    disabled: boolean;
}
interface User$1 {
    /** ID of user. */
    id: string;
    /** Email of user. */
    email: string;
    /** Date user was created. */
    createdAt?: Date;
    /** Date of last user update. */
    updatedAt?: Date;
    /** ID of the Organization. */
    organizationId: string;
    /** Deletion status of user. Owners cannot be deleted. */
    deletable: boolean;
    /** Date of the last login. */
    lastLoginAt?: Date;
    /** Type of user. */
    type: UserType;
    /** @deprecated Deprecated, use "mfa" instead. */
    twoFactorEnabled?: boolean;
    /** Status of user invitation. */
    status: UserStatus;
    /** Defines whether MFA is enabled. */
    mfa: boolean;
    /** ID of the account root user associated with the user. */
    accountRootUserId: string;
    /** Tags associated with the user. */
    tags: string[];
}
type AddGroupMemberRequest = {
    /** ID of the group. */
    groupId: string;
    /**
     * ID of the user to add.
     *
     * One-of ('member'): at most one of 'userId', 'applicationId' could be set.
     */
    userId?: string;
    /**
     * ID of the application to add.
     *
     * One-of ('member'): at most one of 'userId', 'applicationId' could be set.
     */
    applicationId?: string;
};
type AddGroupMembersRequest = {
    /** ID of the group. */
    groupId: string;
    /** IDs of the users to add. */
    userIds?: string[];
    /** IDs of the applications to add. */
    applicationIds?: string[];
};
type ClonePolicyRequest = {
    policyId: string;
};
type CreateAPIKeyRequest$1 = {
    /**
     * ID of the application.
     *
     * One-of ('bearer'): at most one of 'applicationId', 'userId' could be set.
     */
    applicationId?: string;
    /**
     * ID of the user.
     *
     * One-of ('bearer'): at most one of 'applicationId', 'userId' could be set.
     */
    userId?: string;
    /** Expiration date of the API key. */
    expiresAt?: Date;
    /** Default Project ID to use with Object Storage. */
    defaultProjectId?: string;
    /** Description of the API key (max length is 200 characters). */
    description: string;
};
type CreateApplicationRequest$1 = {
    /** Name of the application to create (max length is 64 characters). */
    name?: string;
    /** ID of the Organization. */
    organizationId?: string;
    /** Description of the application (max length is 200 characters). */
    description: string;
    /** Tags associated with the application (maximum of 10 tags). */
    tags?: string[];
};
type CreateGroupRequest$1 = {
    /** ID of Organization linked to the group. */
    organizationId?: string;
    /**
     * Name of the group to create (max length is 64 chars). MUST be unique inside
     * an Organization.
     */
    name?: string;
    /** Description of the group to create (max length is 200 chars). */
    description: string;
    /** Tags associated with the group (maximum of 10 tags). */
    tags?: string[];
};
type CreatePolicyRequest$1 = {
    /** Name of the policy to create (max length is 64 characters). */
    name?: string;
    /** Description of the policy to create (max length is 200 characters). */
    description: string;
    /** ID of the Organization. */
    organizationId?: string;
    /** Rules of the policy to create. */
    rules?: RuleSpecs[];
    /** Tags associated with the policy (maximum of 10 tags). */
    tags?: string[];
    /**
     * ID of user attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    userId?: string;
    /**
     * ID of group attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    groupId?: string;
    /**
     * ID of application attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    applicationId?: string;
    /**
     * Defines whether or not a policy is attributed to a principal.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    noPrincipal?: boolean;
};
type CreateSSHKeyRequest$1 = {
    /** Name of the SSH key. Max length is 1000. */
    name?: string;
    /**
     * SSH public key. Currently only the ssh-rsa, ssh-dss (DSA), ssh-ed25519 and
     * ecdsa keys with NIST curves are supported. Max length is 65000.
     */
    publicKey: string;
    /** Project the resource is attributed to. */
    projectId?: string;
};
type CreateUserRequest$1 = {
    /** ID of the Organization. */
    organizationId?: string;
    /** Email of the user. */
    email: string;
    /** Tags associated with the user. */
    tags?: string[];
};
type DeleteAPIKeyRequest = {
    /** Access key to delete. */
    accessKey: string;
};
type DeleteApplicationRequest = {
    /** ID of the application to delete. */
    applicationId: string;
};
type DeleteGroupRequest = {
    /** ID of the group to delete. */
    groupId: string;
};
type DeleteJWTRequest = {
    /** JWT ID of the JWT to delete. */
    jti: string;
};
type DeletePolicyRequest = {
    /** Id of policy to delete. */
    policyId: string;
};
type DeleteSSHKeyRequest = {
    sshKeyId: string;
};
type DeleteUserRequest$1 = {
    /** ID of the user to delete. */
    userId: string;
};
type GetAPIKeyRequest = {
    /** Access key to search for. */
    accessKey: string;
};
type GetApplicationRequest = {
    /** ID of the application to find. */
    applicationId: string;
};
type GetGroupRequest = {
    /** ID of the group. */
    groupId: string;
};
type GetJWTRequest = {
    /** JWT ID of the JWT to get. */
    jti: string;
};
type GetLogRequest = {
    /** ID of the log. */
    logId: string;
};
type GetPolicyRequest = {
    /** Id of policy to search. */
    policyId: string;
};
type GetQuotumRequest$1 = {
    /** Name of the quota to get. */
    quotumName: string;
    /** ID of the Organization. */
    organizationId?: string;
};
type GetSSHKeyRequest = {
    /** ID of the SSH key. */
    sshKeyId: string;
};
type GetUserRequest = {
    /** ID of the user to find. */
    userId: string;
};
type ListAPIKeysRequest$1 = {
    /** Criteria for sorting results. */
    orderBy?: ListAPIKeysRequestOrderBy;
    /** Page number. Value must be greater or equal to 1. */
    page?: number;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** ID of Organization. */
    organizationId?: string;
    /**
     * @deprecated ID of application that bears the API key.
     *
     *   One-of ('bearer'): at most one of 'applicationId', 'userId' could be set.
     */
    applicationId?: string;
    /**
     * @deprecated ID of user that bears the API key.
     *
     *   One-of ('bearer'): at most one of 'applicationId', 'userId' could be set.
     */
    userId?: string;
    /** Defines whether to filter out editable API keys or not. */
    editable?: boolean;
    /** Defines whether to filter out expired API keys or not. */
    expired?: boolean;
    /** @deprecated Filter by access key (deprecated in favor of `access_keys`). */
    accessKey?: string;
    /** Filter by description. */
    description?: string;
    /** Filter by bearer ID. */
    bearerId?: string;
    /** Filter by type of bearer. */
    bearerType?: BearerType;
    /** Filter by a list of access keys. */
    accessKeys?: string[];
};
interface ListAPIKeysResponse {
    /** List of API keys. */
    apiKeys: APIKey[];
    /** Total count of API Keys. */
    totalCount: number;
}
type ListApplicationsRequest$1 = {
    /** Criteria for sorting results. */
    orderBy?: ListApplicationsRequestOrderBy;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Page number. Value must be greater than 1. */
    page?: number;
    /** Name of the application to filter. */
    name?: string;
    /** ID of the Organization to filter. */
    organizationId?: string;
    /** Defines whether to filter out editable applications or not. */
    editable?: boolean;
    /** Filter by list of IDs. */
    applicationIds?: string[];
    /** Filter by tags containing a given string. */
    tag?: string;
};
interface ListApplicationsResponse {
    /** List of applications. */
    applications: Application[];
    /** Total count of applications. */
    totalCount: number;
}
type ListGroupsRequest$1 = {
    /** Sort order of groups. */
    orderBy?: ListGroupsRequestOrderBy;
    /** Requested page number. Value must be greater or equal to 1. */
    page?: number;
    /** Number of items per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Filter by Organization ID. */
    organizationId?: string;
    /** Name of group to find. */
    name?: string;
    /** Filter by a list of application IDs. */
    applicationIds?: string[];
    /** Filter by a list of user IDs. */
    userIds?: string[];
    /** Filter by a list of group IDs. */
    groupIds?: string[];
    /** Filter by tags containing a given string. */
    tag?: string;
};
interface ListGroupsResponse {
    /** List of groups. */
    groups: Group[];
    /** Total count of groups. */
    totalCount: number;
}
type ListJWTsRequest$1 = {
    /** Criteria for sorting results. */
    orderBy?: ListJWTsRequestOrderBy;
    /** ID of the user to search. */
    audienceId?: string;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Page number. Value must be greater to 1. */
    page?: number;
    /** Filter out expired JWTs or not. */
    expired?: boolean;
};
interface ListJWTsResponse {
    jwts: JWT[];
    totalCount: number;
}
type ListLogsRequest$1 = {
    /** Criteria for sorting results. */
    orderBy?: ListLogsRequestOrderBy;
    /** Filter by Organization ID. */
    organizationId?: string;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Page number. Value must be greater to 1. */
    page?: number;
    /** Defined whether or not to filter out logs created after this timestamp. */
    createdAfter?: Date;
    /** Defined whether or not to filter out logs created before this timestamp. */
    createdBefore?: Date;
    /** Defined whether or not to filter out by a specific action. */
    action?: LogAction;
    /** Defined whether or not to filter out by a specific type of resource. */
    resourceType?: LogResourceType;
    /** Defined whether or not to filter out log by bearer ID or resource ID. */
    search?: string;
};
interface ListLogsResponse {
    /** List of logs. */
    logs: Log[];
    /** Total count of logs. */
    totalCount: number;
}
type ListPermissionSetsRequest$1 = {
    /** Criteria for sorting results. */
    orderBy?: ListPermissionSetsRequestOrderBy;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Page number. Value must be greater than 1. */
    page?: number;
    /** Filter by Organization ID. */
    organizationId?: string;
};
interface ListPermissionSetsResponse {
    /** List of permission sets. */
    permissionSets: PermissionSet[];
    /** Total count of permission sets. */
    totalCount: number;
}
type ListPoliciesRequest$1 = {
    /** Criteria for sorting results. */
    orderBy?: ListPoliciesRequestOrderBy;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Page number. Value must be greater than 1. */
    page?: number;
    /** ID of the Organization to filter. */
    organizationId?: string;
    /** Defines whether or not filter out editable policies. */
    editable?: boolean;
    /** Defines whether or not to filter by list of user IDs. */
    userIds?: string[];
    /** Defines whether or not to filter by list of group IDs. */
    groupIds?: string[];
    /** Filter by a list of application IDs. */
    applicationIds?: string[];
    /** Defines whether or not the policy is attributed to a principal. */
    noPrincipal?: boolean;
    /** Name of the policy to fetch. */
    policyName?: string;
    /** Filter by tags containing a given string. */
    tag?: string;
    /** Filter by a list of IDs. */
    policyIds?: string[];
};
interface ListPoliciesResponse {
    /** List of policies. */
    policies: Policy[];
    /** Total count of policies. */
    totalCount: number;
}
type ListQuotaRequest$1 = {
    /** Criteria for sorting results. */
    orderBy?: ListQuotaRequestOrderBy;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Page number. Value must be greater than 1. */
    page?: number;
    /** Filter by Organization ID. */
    organizationId?: string;
};
interface ListQuotaResponse {
    /** List of quota. */
    quota: Quotum[];
    /** Total count of quota. */
    totalCount: number;
}
type ListRulesRequest$1 = {
    /** Id of policy to search. */
    policyId: string;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Page number. Value must be greater than 1. */
    page?: number;
};
interface ListRulesResponse {
    /** Rules of the policy. */
    rules: Rule[];
    /** Total count of rules. */
    totalCount: number;
}
type ListSSHKeysRequest$1 = {
    /** Sort order of the SSH keys. */
    orderBy?: ListSSHKeysRequestOrderBy;
    /** Requested page number. Value must be greater or equal to 1. */
    page?: number;
    /** Number of items per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Filter by Organization ID. */
    organizationId?: string;
    /** Name of group to find. */
    name?: string;
    /** Filter by Project ID. */
    projectId?: string;
    /** Defines whether to include disabled SSH keys or not. */
    disabled?: boolean;
};
interface ListSSHKeysResponse {
    /** List of SSH keys. */
    sshKeys: SSHKey[];
    /** Total count of SSH keys. */
    totalCount: number;
}
type ListUsersRequest$2 = {
    /** Criteria for sorting results. */
    orderBy?: ListUsersRequestOrderBy$1;
    /** Number of results per page. Value must be between 1 and 100. */
    pageSize?: number;
    /** Page number. Value must be greater or equal to 1. */
    page?: number;
    /** ID of the Organization to filter. */
    organizationId?: string;
    /** Filter by list of IDs. */
    userIds?: string[];
    /** Filter by MFA status. */
    mfa?: boolean;
    /** Filter by tags containing a given string. */
    tag?: string;
};
interface ListUsersResponse$1 {
    /** List of users. */
    users: User$1[];
    /** Total count of users. */
    totalCount: number;
}
type RemoveGroupMemberRequest = {
    /** ID of the group. */
    groupId: string;
    /**
     * ID of the user to remove.
     *
     * One-of ('member'): at most one of 'userId', 'applicationId' could be set.
     */
    userId?: string;
    /**
     * ID of the application to remove.
     *
     * One-of ('member'): at most one of 'userId', 'applicationId' could be set.
     */
    applicationId?: string;
};
type SetGroupMembersRequest = {
    groupId: string;
    userIds: string[];
    applicationIds: string[];
};
type SetRulesRequest = {
    /** Id of policy to update. */
    policyId: string;
    /** Rules of the policy to set. */
    rules: RuleSpecs[];
};
interface SetRulesResponse {
    /** Rules of the policy. */
    rules: Rule[];
}
type UpdateAPIKeyRequest$1 = {
    /** Access key to update. */
    accessKey: string;
    /** New default Project ID to set. */
    defaultProjectId?: string;
    /** New description to update. */
    description?: string;
};
type UpdateApplicationRequest$1 = {
    /** ID of the application to update. */
    applicationId: string;
    /** New name for the application (max length is 64 chars). */
    name?: string;
    /** New description for the application (max length is 200 chars). */
    description?: string;
    /** New tags for the application (maximum of 10 tags). */
    tags?: string[];
};
type UpdateGroupRequest$1 = {
    /** ID of the group to update. */
    groupId: string;
    /**
     * New name for the group (max length is 64 chars). MUST be unique inside an
     * Organization.
     */
    name?: string;
    /** New description for the group (max length is 200 chars). */
    description?: string;
    /** New tags for the group (maximum of 10 tags). */
    tags?: string[];
};
type UpdatePolicyRequest$1 = {
    /** Id of policy to update. */
    policyId: string;
    /** New name for the policy (max length is 64 characters). */
    name?: string;
    /** New description of policy (max length is 200 characters). */
    description?: string;
    /** New tags for the policy (maximum of 10 tags). */
    tags?: string[];
    /**
     * New ID of user attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    userId?: string;
    /**
     * New ID of group attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    groupId?: string;
    /**
     * New ID of application attributed to the policy.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    applicationId?: string;
    /**
     * Defines whether or not the policy is attributed to a principal.
     *
     * One-of ('principal'): at most one of 'userId', 'groupId', 'applicationId',
     * 'noPrincipal' could be set.
     */
    noPrincipal?: boolean;
};
type UpdateSSHKeyRequest$1 = {
    sshKeyId: string;
    /** Name of the SSH key. Max length is 1000. */
    name?: string;
    /** Enable or disable the SSH key. */
    disabled?: boolean;
};
type UpdateUserRequest$1 = {
    /** ID of the user to update. */
    userId: string;
    /** New tags for the user (maximum of 10 tags). */
    tags?: string[];
};

/** IAM API. */
declare class API$k extends API$x {
    protected pageOfListSSHKeys: (request?: Readonly<ListSSHKeysRequest$1>) => Promise<ListSSHKeysResponse>;
    /**
     * List SSH keys. List SSH keys. By default, the SSH keys listed are ordered
     * by creation date in ascending order. This can be modified via the
     * `order_by` field. You can define additional parameters for your query such
     * as `organization_id`, `name`, `project_id` and `disabled`.
     *
     * @param request - The request {@link ListSSHKeysRequest}
     * @returns A Promise of ListSSHKeysResponse
     */
    listSSHKeys: (request?: Readonly<ListSSHKeysRequest$1>) => Promise<ListSSHKeysResponse> & {
        all: () => Promise<SSHKey[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<SSHKey[], void, void>;
    };
    /**
     * Create an SSH key. Add a new SSH key to a Scaleway Project. You must
     * specify the `name`, `public_key` and `project_id`.
     *
     * @param request - The request {@link CreateSSHKeyRequest}
     * @returns A Promise of SSHKey
     */
    createSSHKey: (request: Readonly<CreateSSHKeyRequest$1>) => Promise<SSHKey>;
    /**
     * Get an SSH key. Retrieve information about a given SSH key, specified by
     * the `ssh_key_id` parameter. The SSH key's full details, including `id`,
     * `name`, `public_key`, and `project_id` are returned in the response.
     *
     * @param request - The request {@link GetSSHKeyRequest}
     * @returns A Promise of SSHKey
     */
    getSSHKey: (request: Readonly<GetSSHKeyRequest>) => Promise<SSHKey>;
    /**
     * Update an SSH key. Update the parameters of an SSH key, including `name`
     * and `disable`.
     *
     * @param request - The request {@link UpdateSSHKeyRequest}
     * @returns A Promise of SSHKey
     */
    updateSSHKey: (request: Readonly<UpdateSSHKeyRequest$1>) => Promise<SSHKey>;
    /**
     * Delete an SSH key. Delete a given SSH key, specified by the `ssh_key_id`.
     * Deleting an SSH is permanent, and cannot be undone. Note that you might
     * need to update any configurations that used the SSH key.
     *
     * @param request - The request {@link DeleteSSHKeyRequest}
     */
    deleteSSHKey: (request: Readonly<DeleteSSHKeyRequest>) => Promise<void>;
    protected pageOfListUsers: (request?: Readonly<ListUsersRequest$2>) => Promise<ListUsersResponse$1>;
    /**
     * List users of an Organization. List the users of an Organization. By
     * default, the users listed are ordered by creation date in ascending order.
     * This can be modified via the `order_by` field. You must define the
     * `organization_id` in the query path of your request. You can also define
     * additional parameters for your query such as `user_ids`.
     *
     * @param request - The request {@link ListUsersRequest}
     * @returns A Promise of ListUsersResponse
     */
    listUsers: (request?: Readonly<ListUsersRequest$2>) => Promise<ListUsersResponse$1> & {
        all: () => Promise<User$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<User$1[], void, void>;
    };
    /**
     * Get a given user. Retrieve information about a user, specified by the
     * `user_id` parameter. The user's full details, including `id`, `email`,
     * `organization_id`, `status` and `mfa` are returned in the response.
     *
     * @param request - The request {@link GetUserRequest}
     * @returns A Promise of User
     */
    getUser: (request: Readonly<GetUserRequest>) => Promise<User$1>;
    /**
     * Update a user. Update the parameters of a user, including `tags`.
     *
     * @param request - The request {@link UpdateUserRequest}
     * @returns A Promise of User
     */
    updateUser: (request: Readonly<UpdateUserRequest$1>) => Promise<User$1>;
    /**
     * Delete a guest user from an Organization. Remove a user from an
     * Organization in which they are a guest. You must define the `user_id` in
     * your request. Note that removing a user from an Organization automatically
     * deletes their API keys, and any policies directly attached to them become
     * orphaned.
     *
     * @param request - The request {@link DeleteUserRequest}
     */
    deleteUser: (request: Readonly<DeleteUserRequest$1>) => Promise<void>;
    /**
     * Create a new user. Create a new user. You must define the `organization_id`
     * and the `email` in your request.
     *
     * @param request - The request {@link CreateUserRequest}
     * @returns A Promise of User
     */
    createUser: (request: Readonly<CreateUserRequest$1>) => Promise<User$1>;
    protected pageOfListApplications: (request?: Readonly<ListApplicationsRequest$1>) => Promise<ListApplicationsResponse>;
    /**
     * List applications of an Organization. List the applications of an
     * Organization. By default, the applications listed are ordered by creation
     * date in ascending order. This can be modified via the `order_by` field. You
     * must define the `organization_id` in the query path of your request. You
     * can also define additional parameters for your query such as
     * `application_ids`.
     *
     * @param request - The request {@link ListApplicationsRequest}
     * @returns A Promise of ListApplicationsResponse
     */
    listApplications: (request?: Readonly<ListApplicationsRequest$1>) => Promise<ListApplicationsResponse> & {
        all: () => Promise<Application[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Application[], void, void>;
    };
    /**
     * Create a new application. Create a new application. You must define the
     * `name` parameter in the request.
     *
     * @param request - The request {@link CreateApplicationRequest}
     * @returns A Promise of Application
     */
    createApplication: (request: Readonly<CreateApplicationRequest$1>) => Promise<Application>;
    /**
     * Get a given application. Retrieve information about an application,
     * specified by the `application_id` parameter. The application's full
     * details, including `id`, `email`, `organization_id`, `status` and
     * `two_factor_enabled` are returned in the response.
     *
     * @param request - The request {@link GetApplicationRequest}
     * @returns A Promise of Application
     */
    getApplication: (request: Readonly<GetApplicationRequest>) => Promise<Application>;
    /**
     * Update an application. Update the parameters of an application, including
     * `name` and `description`.
     *
     * @param request - The request {@link UpdateApplicationRequest}
     * @returns A Promise of Application
     */
    updateApplication: (request: Readonly<UpdateApplicationRequest$1>) => Promise<Application>;
    /**
     * Delete an application. Delete an application. Note that this action is
     * irreversible and will automatically delete the application's API keys.
     * Policies attached to users and applications via this group will no longer
     * apply.
     *
     * @param request - The request {@link DeleteApplicationRequest}
     */
    deleteApplication: (request: Readonly<DeleteApplicationRequest>) => Promise<void>;
    protected pageOfListGroups: (request?: Readonly<ListGroupsRequest$1>) => Promise<ListGroupsResponse>;
    /**
     * List groups. List groups. By default, the groups listed are ordered by
     * creation date in ascending order. This can be modified via the `order_by`
     * field. You can define additional parameters to filter your query. Use
     * `user_ids` or `application_ids` to list all groups certain users or
     * applications belong to.
     *
     * @param request - The request {@link ListGroupsRequest}
     * @returns A Promise of ListGroupsResponse
     */
    listGroups: (request?: Readonly<ListGroupsRequest$1>) => Promise<ListGroupsResponse> & {
        all: () => Promise<Group[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Group[], void, void>;
    };
    /**
     * Create a group. Create a new group. You must define the `name` and
     * `organization_id` parameters in the request.
     *
     * @param request - The request {@link CreateGroupRequest}
     * @returns A Promise of Group
     */
    createGroup: (request: Readonly<CreateGroupRequest$1>) => Promise<Group>;
    /**
     * Get a group. Retrive information about a given group, specified by the
     * `group_id` parameter. The group's full details, including `user_ids` and
     * `application_ids` are returned in the response.
     *
     * @param request - The request {@link GetGroupRequest}
     * @returns A Promise of Group
     */
    getGroup: (request: Readonly<GetGroupRequest>) => Promise<Group>;
    /**
     * Update a group. Update the parameters of group, including `name` and
     * `description`.
     *
     * @param request - The request {@link UpdateGroupRequest}
     * @returns A Promise of Group
     */
    updateGroup: (request: Readonly<UpdateGroupRequest$1>) => Promise<Group>;
    /**
     * Overwrite users and applications of a group. Overwrite users and
     * applications configuration in a group. Any information that you add using
     * this command will overwrite the previous configuration.
     *
     * @param request - The request {@link SetGroupMembersRequest}
     * @returns A Promise of Group
     */
    setGroupMembers: (request: Readonly<SetGroupMembersRequest>) => Promise<Group>;
    /**
     * Add a user or an application to a group. Add a user or an application to a
     * group. You can specify a `user_id` and and `application_id` in the body of
     * your request. Note that you can only add one of each per request.
     *
     * @param request - The request {@link AddGroupMemberRequest}
     * @returns A Promise of Group
     */
    addGroupMember: (request: Readonly<AddGroupMemberRequest>) => Promise<Group>;
    /**
     * Add multiple users and applications to a group. Add multiple users and
     * applications to a group in a single call. You can specify an array of
     * `user_id`s and `application_id`s. Note that any existing users and
     * applications in the group will remain. To add new users/applications and
     * delete pre-existing ones, use the [Overwrite users and applications of a
     * group](#path-groups-overwrite-users-and-applications-of-a-group) method.
     *
     * @param request - The request {@link AddGroupMembersRequest}
     * @returns A Promise of Group
     */
    addGroupMembers: (request: Readonly<AddGroupMembersRequest>) => Promise<Group>;
    /**
     * Remove a user or an application from a group. Remove a user or an
     * application from a group. You can specify a `user_id` and and
     * `application_id` in the body of your request. Note that you can only remove
     * one of each per request. Removing a user from a group means that any
     * permissions given to them via the group (i.e. from an attached policy) will
     * no longer apply. Be sure you want to remove these permissions from the user
     * before proceeding.
     *
     * @param request - The request {@link RemoveGroupMemberRequest}
     * @returns A Promise of Group
     */
    removeGroupMember: (request: Readonly<RemoveGroupMemberRequest>) => Promise<Group>;
    /**
     * Delete a group. Delete a group. Note that this action is irreversible and
     * could delete permissions for group members. Policies attached to users and
     * applications via this group will no longer apply.
     *
     * @param request - The request {@link DeleteGroupRequest}
     */
    deleteGroup: (request: Readonly<DeleteGroupRequest>) => Promise<void>;
    protected pageOfListPolicies: (request?: Readonly<ListPoliciesRequest$1>) => Promise<ListPoliciesResponse>;
    /**
     * List policies of an Organization. List the policies of an Organization. By
     * default, the policies listed are ordered by creation date in ascending
     * order. This can be modified via the `order_by` field. You must define the
     * `organization_id` in the query path of your request. You can also define
     * additional parameters to filter your query, such as `user_ids`,
     * `groups_ids`, `application_ids`, and `policy_name`.
     *
     * @param request - The request {@link ListPoliciesRequest}
     * @returns A Promise of ListPoliciesResponse
     */
    listPolicies: (request?: Readonly<ListPoliciesRequest$1>) => Promise<ListPoliciesResponse> & {
        all: () => Promise<Policy[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Policy[], void, void>;
    };
    /**
     * Create a new policy. Create a new application. You must define the `name`
     * parameter in the request. You can specify parameters such as `user_id`,
     * `groups_id`, `application_id`, `no_principal`, `rules` and its child
     * attributes.
     *
     * @param request - The request {@link CreatePolicyRequest}
     * @returns A Promise of Policy
     */
    createPolicy: (request: Readonly<CreatePolicyRequest$1>) => Promise<Policy>;
    /**
     * Get an existing policy. Retrieve information about a policy, speficified by
     * the `policy_id` parameter. The policy's full details, including `id`,
     * `name`, `organization_id`, `nb_rules` and `nb_scopes`, `nb_permission_sets`
     * are returned in the response.
     *
     * @param request - The request {@link GetPolicyRequest}
     * @returns A Promise of Policy
     */
    getPolicy: (request: Readonly<GetPolicyRequest>) => Promise<Policy>;
    /**
     * Update an existing policy. Update the parameters of a policy, including
     * `name`, `description`, `user_id`, `group_id`, `application_id` and
     * `no_principal`.
     *
     * @param request - The request {@link UpdatePolicyRequest}
     * @returns A Promise of Policy
     */
    updatePolicy: (request: Readonly<UpdatePolicyRequest$1>) => Promise<Policy>;
    /**
     * Delete a policy. Delete a policy. You must define specify the `policy_id`
     * parameter in your request. Note that when deleting a policy, all
     * permissions it gives to its principal (user, group or application) will be
     * revoked.
     *
     * @param request - The request {@link DeletePolicyRequest}
     */
    deletePolicy: (request: Readonly<DeletePolicyRequest>) => Promise<void>;
    /**
     * Clone a policy. Clone a policy. You must define specify the `policy_id`
     * parameter in your request.
     *
     * @param request - The request {@link ClonePolicyRequest}
     * @returns A Promise of Policy
     */
    clonePolicy: (request: Readonly<ClonePolicyRequest>) => Promise<Policy>;
    /**
     * Set rules of a given policy. Overwrite the rules of a given policy. Any
     * information that you add using this command will overwrite the previous
     * configuration. If you include some of the rules you already had in your
     * previous configuration in your new one, but you change their order, the new
     * order of display will apply. While policy rules are ordered, they have no
     * impact on the access logic of IAM because rules are allow-only.
     *
     * @param request - The request {@link SetRulesRequest}
     * @returns A Promise of SetRulesResponse
     */
    setRules: (request: Readonly<SetRulesRequest>) => Promise<SetRulesResponse>;
    protected pageOfListRules: (request: Readonly<ListRulesRequest$1>) => Promise<ListRulesResponse>;
    /**
     * List rules of a given policy. List the rules of a given policy. By default,
     * the rules listed are ordered by creation date in ascending order. This can
     * be modified via the `order_by` field. You must define the `policy_id` in
     * the query path of your request.
     *
     * @param request - The request {@link ListRulesRequest}
     * @returns A Promise of ListRulesResponse
     */
    listRules: (request: Readonly<ListRulesRequest$1>) => Promise<ListRulesResponse> & {
        all: () => Promise<Rule[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Rule[], void, void>;
    };
    protected pageOfListPermissionSets: (request?: Readonly<ListPermissionSetsRequest$1>) => Promise<ListPermissionSetsResponse>;
    /**
     * List permission sets. List permission sets available for given
     * Organization. You must define the `organization_id` in the query path of
     * your request.
     *
     * @param request - The request {@link ListPermissionSetsRequest}
     * @returns A Promise of ListPermissionSetsResponse
     */
    listPermissionSets: (request?: Readonly<ListPermissionSetsRequest$1>) => Promise<ListPermissionSetsResponse> & {
        all: () => Promise<PermissionSet[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<PermissionSet[], void, void>;
    };
    protected pageOfListAPIKeys: (request?: Readonly<ListAPIKeysRequest$1>) => Promise<ListAPIKeysResponse>;
    /**
     * List API keys. List API keys. By default, the API keys listed are ordered
     * by creation date in ascending order. This can be modified via the
     * `order_by` field. You can define additional parameters for your query such
     * as `editable`, `expired`, `access_key` and `bearer_id`.
     *
     * @param request - The request {@link ListAPIKeysRequest}
     * @returns A Promise of ListAPIKeysResponse
     */
    listAPIKeys: (request?: Readonly<ListAPIKeysRequest$1>) => Promise<ListAPIKeysResponse> & {
        all: () => Promise<APIKey[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<APIKey[], void, void>;
    };
    /**
     * Create an API key. Create an API key. You must specify the `application_id`
     * or the `user_id` and the description. You can also specify the
     * `default_project_id` which is the Project ID of your preferred Project, to
     * use with Object Storage. The `access_key` and `secret_key` values are
     * returned in the response. Note that he secret key is only showed once. Make
     * sure that you copy and store both keys somewhere safe.
     *
     * @param request - The request {@link CreateAPIKeyRequest}
     * @returns A Promise of APIKey
     */
    createAPIKey: (request: Readonly<CreateAPIKeyRequest$1>) => Promise<APIKey>;
    /**
     * Get an API key. Retrive information about an API key, specified by the
     * `access_key` parameter. The API key's details, including either the
     * `user_id` or `application_id` of its bearer are returned in the response.
     * Note that the string value for the `secret_key` is nullable, and therefore
     * is not displayed in the response. The `secret_key` value is only displayed
     * upon API key creation.
     *
     * @param request - The request {@link GetAPIKeyRequest}
     * @returns A Promise of APIKey
     */
    getAPIKey: (request: Readonly<GetAPIKeyRequest>) => Promise<APIKey>;
    /**
     * Update an API key. Update the parameters of an API key, including
     * `default_project_id` and `description`.
     *
     * @param request - The request {@link UpdateAPIKeyRequest}
     * @returns A Promise of APIKey
     */
    updateAPIKey: (request: Readonly<UpdateAPIKeyRequest$1>) => Promise<APIKey>;
    /**
     * Delete an API key. Delete an API key. Note that this action is irreversible
     * and cannot be undone. Make sure you update any configurations using the API
     * keys you delete.
     *
     * @param request - The request {@link DeleteAPIKeyRequest}
     */
    deleteAPIKey: (request: Readonly<DeleteAPIKeyRequest>) => Promise<void>;
    protected pageOfListQuota: (request?: Readonly<ListQuotaRequest$1>) => Promise<ListQuotaResponse>;
    /**
     * List all quotas in the Organization. List all product and features quota
     * for an Organization, with their associated limits. By default, the quota
     * listed are ordered by creation date in ascending order. This can be
     * modified via the `order_by` field. You must define the `organization_id` in
     * the query path of your request.
     *
     * @param request - The request {@link ListQuotaRequest}
     * @returns A Promise of ListQuotaResponse
     */
    listQuota: (request?: Readonly<ListQuotaRequest$1>) => Promise<ListQuotaResponse> & {
        all: () => Promise<Quotum[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Quotum[], void, void>;
    };
    /**
     * Get a quota in the Organization. Retrieve information about a resource
     * quota, specified by the `quotum_name` parameter. The quota's `limit`, or
     * whether it is unlimited, is returned in the response.
     *
     * @param request - The request {@link GetQuotumRequest}
     * @returns A Promise of Quotum
     */
    getQuotum: (request: Readonly<GetQuotumRequest$1>) => Promise<Quotum>;
    protected pageOfListJWTs: (request?: Readonly<ListJWTsRequest$1>) => Promise<ListJWTsResponse>;
    /**
     * List JWTs.
     *
     * @param request - The request {@link ListJWTsRequest}
     * @returns A Promise of ListJWTsResponse
     */
    listJWTs: (request?: Readonly<ListJWTsRequest$1>) => Promise<ListJWTsResponse> & {
        all: () => Promise<JWT[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<JWT[], void, void>;
    };
    /**
     * Get a JWT.
     *
     * @param request - The request {@link GetJWTRequest}
     * @returns A Promise of JWT
     */
    getJWT: (request: Readonly<GetJWTRequest>) => Promise<JWT>;
    /**
     * Delete a JWT.
     *
     * @param request - The request {@link DeleteJWTRequest}
     */
    deleteJWT: (request: Readonly<DeleteJWTRequest>) => Promise<void>;
    protected pageOfListLogs: (request?: Readonly<ListLogsRequest$1>) => Promise<ListLogsResponse>;
    /**
     * List logs. List logs available for given Organization. You must define the
     * `organization_id` in the query path of your request.
     *
     * @param request - The request {@link ListLogsRequest}
     * @returns A Promise of ListLogsResponse
     */
    listLogs: (request?: Readonly<ListLogsRequest$1>) => Promise<ListLogsResponse> & {
        all: () => Promise<Log[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Log[], void, void>;
    };
    /**
     * Get a log. Retrieve information about a log, specified by the `log_id`
     * parameter. The log's full details, including `id`, `ip`, `user_agent`,
     * `action`, `bearer_id`, `resource_type` and `resource_id` are returned in
     * the response.
     *
     * @param request - The request {@link GetLogRequest}
     * @returns A Promise of Log
     */
    getLog: (request: Readonly<GetLogRequest>) => Promise<Log>;
}

declare const CreateAPIKeyRequest: {
    description: {
        ignoreEmpty: boolean;
        maxLength: number;
    };
};
declare const CreateApplicationRequest: {
    description: {
        ignoreEmpty: boolean;
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const CreateGroupRequest: {
    description: {
        ignoreEmpty: boolean;
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const CreatePolicyRequest: {
    description: {
        ignoreEmpty: boolean;
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const CreateSSHKeyRequest: {
    name: {
        ignoreEmpty: boolean;
        maxLength: number;
    };
    publicKey: {
        maxLength: number;
        minLength: number;
    };
};
declare const GetQuotumRequest: {
    quotumName: {
        minLength: number;
    };
};
declare const ListAPIKeysRequest: {
    description: {
        maxLength: number;
    };
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListApplicationsRequest: {
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListGroupsRequest: {
    name: {
        minLength: number;
    };
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListJWTsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListLogsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListPermissionSetsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListPoliciesRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
    policyName: {
        minLength: number;
    };
};
declare const ListQuotaRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListRulesRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListSSHKeysRequest: {
    name: {
        maxLength: number;
        minLength: number;
    };
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListUsersRequest$1: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const UpdateAPIKeyRequest: {
    description: {
        maxLength: number;
    };
};
declare const UpdateApplicationRequest: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const UpdateGroupRequest: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const UpdatePolicyRequest: {
    description: {
        maxLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
        pattern: RegExp;
    };
};
declare const UpdateSSHKeyRequest: {
    name: {
        maxLength: number;
    };
};

declare const validationRules_gen$7_CreateAPIKeyRequest: typeof CreateAPIKeyRequest;
declare const validationRules_gen$7_CreateApplicationRequest: typeof CreateApplicationRequest;
declare const validationRules_gen$7_CreateGroupRequest: typeof CreateGroupRequest;
declare const validationRules_gen$7_CreatePolicyRequest: typeof CreatePolicyRequest;
declare const validationRules_gen$7_CreateSSHKeyRequest: typeof CreateSSHKeyRequest;
declare const validationRules_gen$7_GetQuotumRequest: typeof GetQuotumRequest;
declare const validationRules_gen$7_ListAPIKeysRequest: typeof ListAPIKeysRequest;
declare const validationRules_gen$7_ListApplicationsRequest: typeof ListApplicationsRequest;
declare const validationRules_gen$7_ListGroupsRequest: typeof ListGroupsRequest;
declare const validationRules_gen$7_ListJWTsRequest: typeof ListJWTsRequest;
declare const validationRules_gen$7_ListLogsRequest: typeof ListLogsRequest;
declare const validationRules_gen$7_ListPermissionSetsRequest: typeof ListPermissionSetsRequest;
declare const validationRules_gen$7_ListPoliciesRequest: typeof ListPoliciesRequest;
declare const validationRules_gen$7_ListQuotaRequest: typeof ListQuotaRequest;
declare const validationRules_gen$7_ListRulesRequest: typeof ListRulesRequest;
declare const validationRules_gen$7_ListSSHKeysRequest: typeof ListSSHKeysRequest;
declare const validationRules_gen$7_UpdateAPIKeyRequest: typeof UpdateAPIKeyRequest;
declare const validationRules_gen$7_UpdateApplicationRequest: typeof UpdateApplicationRequest;
declare const validationRules_gen$7_UpdateGroupRequest: typeof UpdateGroupRequest;
declare const validationRules_gen$7_UpdatePolicyRequest: typeof UpdatePolicyRequest;
declare const validationRules_gen$7_UpdateSSHKeyRequest: typeof UpdateSSHKeyRequest;
declare namespace validationRules_gen$7 {
  export { validationRules_gen$7_CreateAPIKeyRequest as CreateAPIKeyRequest, validationRules_gen$7_CreateApplicationRequest as CreateApplicationRequest, validationRules_gen$7_CreateGroupRequest as CreateGroupRequest, validationRules_gen$7_CreatePolicyRequest as CreatePolicyRequest, validationRules_gen$7_CreateSSHKeyRequest as CreateSSHKeyRequest, validationRules_gen$7_GetQuotumRequest as GetQuotumRequest, validationRules_gen$7_ListAPIKeysRequest as ListAPIKeysRequest, validationRules_gen$7_ListApplicationsRequest as ListApplicationsRequest, validationRules_gen$7_ListGroupsRequest as ListGroupsRequest, validationRules_gen$7_ListJWTsRequest as ListJWTsRequest, validationRules_gen$7_ListLogsRequest as ListLogsRequest, validationRules_gen$7_ListPermissionSetsRequest as ListPermissionSetsRequest, validationRules_gen$7_ListPoliciesRequest as ListPoliciesRequest, validationRules_gen$7_ListQuotaRequest as ListQuotaRequest, validationRules_gen$7_ListRulesRequest as ListRulesRequest, validationRules_gen$7_ListSSHKeysRequest as ListSSHKeysRequest, ListUsersRequest$1 as ListUsersRequest, validationRules_gen$7_UpdateAPIKeyRequest as UpdateAPIKeyRequest, validationRules_gen$7_UpdateApplicationRequest as UpdateApplicationRequest, validationRules_gen$7_UpdateGroupRequest as UpdateGroupRequest, validationRules_gen$7_UpdatePolicyRequest as UpdatePolicyRequest, validationRules_gen$7_UpdateSSHKeyRequest as UpdateSSHKeyRequest };
}

type index_gen$j_APIKey = APIKey;
type index_gen$j_AddGroupMemberRequest = AddGroupMemberRequest;
type index_gen$j_AddGroupMembersRequest = AddGroupMembersRequest;
type index_gen$j_Application = Application;
type index_gen$j_BearerType = BearerType;
type index_gen$j_ClonePolicyRequest = ClonePolicyRequest;
type index_gen$j_DeleteAPIKeyRequest = DeleteAPIKeyRequest;
type index_gen$j_DeleteApplicationRequest = DeleteApplicationRequest;
type index_gen$j_DeleteGroupRequest = DeleteGroupRequest;
type index_gen$j_DeleteJWTRequest = DeleteJWTRequest;
type index_gen$j_DeletePolicyRequest = DeletePolicyRequest;
type index_gen$j_DeleteSSHKeyRequest = DeleteSSHKeyRequest;
type index_gen$j_GetAPIKeyRequest = GetAPIKeyRequest;
type index_gen$j_GetApplicationRequest = GetApplicationRequest;
type index_gen$j_GetGroupRequest = GetGroupRequest;
type index_gen$j_GetJWTRequest = GetJWTRequest;
type index_gen$j_GetLogRequest = GetLogRequest;
type index_gen$j_GetPolicyRequest = GetPolicyRequest;
type index_gen$j_GetSSHKeyRequest = GetSSHKeyRequest;
type index_gen$j_GetUserRequest = GetUserRequest;
type index_gen$j_Group = Group;
type index_gen$j_JWT = JWT;
type index_gen$j_ListAPIKeysRequestOrderBy = ListAPIKeysRequestOrderBy;
type index_gen$j_ListAPIKeysResponse = ListAPIKeysResponse;
type index_gen$j_ListApplicationsRequestOrderBy = ListApplicationsRequestOrderBy;
type index_gen$j_ListApplicationsResponse = ListApplicationsResponse;
type index_gen$j_ListGroupsRequestOrderBy = ListGroupsRequestOrderBy;
type index_gen$j_ListGroupsResponse = ListGroupsResponse;
type index_gen$j_ListJWTsRequestOrderBy = ListJWTsRequestOrderBy;
type index_gen$j_ListJWTsResponse = ListJWTsResponse;
type index_gen$j_ListLogsRequestOrderBy = ListLogsRequestOrderBy;
type index_gen$j_ListLogsResponse = ListLogsResponse;
type index_gen$j_ListPermissionSetsRequestOrderBy = ListPermissionSetsRequestOrderBy;
type index_gen$j_ListPermissionSetsResponse = ListPermissionSetsResponse;
type index_gen$j_ListPoliciesRequestOrderBy = ListPoliciesRequestOrderBy;
type index_gen$j_ListPoliciesResponse = ListPoliciesResponse;
type index_gen$j_ListQuotaRequestOrderBy = ListQuotaRequestOrderBy;
type index_gen$j_ListQuotaResponse = ListQuotaResponse;
type index_gen$j_ListRulesResponse = ListRulesResponse;
type index_gen$j_ListSSHKeysRequestOrderBy = ListSSHKeysRequestOrderBy;
type index_gen$j_ListSSHKeysResponse = ListSSHKeysResponse;
type index_gen$j_Log = Log;
type index_gen$j_LogAction = LogAction;
type index_gen$j_LogResourceType = LogResourceType;
type index_gen$j_PermissionSet = PermissionSet;
type index_gen$j_PermissionSetScopeType = PermissionSetScopeType;
type index_gen$j_Policy = Policy;
type index_gen$j_Quotum = Quotum;
type index_gen$j_RemoveGroupMemberRequest = RemoveGroupMemberRequest;
type index_gen$j_Rule = Rule;
type index_gen$j_RuleSpecs = RuleSpecs;
type index_gen$j_SSHKey = SSHKey;
type index_gen$j_SetGroupMembersRequest = SetGroupMembersRequest;
type index_gen$j_SetRulesRequest = SetRulesRequest;
type index_gen$j_SetRulesResponse = SetRulesResponse;
type index_gen$j_UserStatus = UserStatus;
type index_gen$j_UserType = UserType;
declare namespace index_gen$j {
  export { API$k as API, type index_gen$j_APIKey as APIKey, type index_gen$j_AddGroupMemberRequest as AddGroupMemberRequest, type index_gen$j_AddGroupMembersRequest as AddGroupMembersRequest, type index_gen$j_Application as Application, type index_gen$j_BearerType as BearerType, type index_gen$j_ClonePolicyRequest as ClonePolicyRequest, type CreateAPIKeyRequest$1 as CreateAPIKeyRequest, type CreateApplicationRequest$1 as CreateApplicationRequest, type CreateGroupRequest$1 as CreateGroupRequest, type CreatePolicyRequest$1 as CreatePolicyRequest, type CreateSSHKeyRequest$1 as CreateSSHKeyRequest, type CreateUserRequest$1 as CreateUserRequest, type index_gen$j_DeleteAPIKeyRequest as DeleteAPIKeyRequest, type index_gen$j_DeleteApplicationRequest as DeleteApplicationRequest, type index_gen$j_DeleteGroupRequest as DeleteGroupRequest, type index_gen$j_DeleteJWTRequest as DeleteJWTRequest, type index_gen$j_DeletePolicyRequest as DeletePolicyRequest, type index_gen$j_DeleteSSHKeyRequest as DeleteSSHKeyRequest, type DeleteUserRequest$1 as DeleteUserRequest, type index_gen$j_GetAPIKeyRequest as GetAPIKeyRequest, type index_gen$j_GetApplicationRequest as GetApplicationRequest, type index_gen$j_GetGroupRequest as GetGroupRequest, type index_gen$j_GetJWTRequest as GetJWTRequest, type index_gen$j_GetLogRequest as GetLogRequest, type index_gen$j_GetPolicyRequest as GetPolicyRequest, type GetQuotumRequest$1 as GetQuotumRequest, type index_gen$j_GetSSHKeyRequest as GetSSHKeyRequest, type index_gen$j_GetUserRequest as GetUserRequest, type index_gen$j_Group as Group, type index_gen$j_JWT as JWT, type ListAPIKeysRequest$1 as ListAPIKeysRequest, type index_gen$j_ListAPIKeysRequestOrderBy as ListAPIKeysRequestOrderBy, type index_gen$j_ListAPIKeysResponse as ListAPIKeysResponse, type ListApplicationsRequest$1 as ListApplicationsRequest, type index_gen$j_ListApplicationsRequestOrderBy as ListApplicationsRequestOrderBy, type index_gen$j_ListApplicationsResponse as ListApplicationsResponse, type ListGroupsRequest$1 as ListGroupsRequest, type index_gen$j_ListGroupsRequestOrderBy as ListGroupsRequestOrderBy, type index_gen$j_ListGroupsResponse as ListGroupsResponse, type ListJWTsRequest$1 as ListJWTsRequest, type index_gen$j_ListJWTsRequestOrderBy as ListJWTsRequestOrderBy, type index_gen$j_ListJWTsResponse as ListJWTsResponse, type ListLogsRequest$1 as ListLogsRequest, type index_gen$j_ListLogsRequestOrderBy as ListLogsRequestOrderBy, type index_gen$j_ListLogsResponse as ListLogsResponse, type ListPermissionSetsRequest$1 as ListPermissionSetsRequest, type index_gen$j_ListPermissionSetsRequestOrderBy as ListPermissionSetsRequestOrderBy, type index_gen$j_ListPermissionSetsResponse as ListPermissionSetsResponse, type ListPoliciesRequest$1 as ListPoliciesRequest, type index_gen$j_ListPoliciesRequestOrderBy as ListPoliciesRequestOrderBy, type index_gen$j_ListPoliciesResponse as ListPoliciesResponse, type ListQuotaRequest$1 as ListQuotaRequest, type index_gen$j_ListQuotaRequestOrderBy as ListQuotaRequestOrderBy, type index_gen$j_ListQuotaResponse as ListQuotaResponse, type ListRulesRequest$1 as ListRulesRequest, type index_gen$j_ListRulesResponse as ListRulesResponse, type ListSSHKeysRequest$1 as ListSSHKeysRequest, type index_gen$j_ListSSHKeysRequestOrderBy as ListSSHKeysRequestOrderBy, type index_gen$j_ListSSHKeysResponse as ListSSHKeysResponse, type ListUsersRequest$2 as ListUsersRequest, type ListUsersRequestOrderBy$1 as ListUsersRequestOrderBy, type ListUsersResponse$1 as ListUsersResponse, type index_gen$j_Log as Log, type index_gen$j_LogAction as LogAction, type index_gen$j_LogResourceType as LogResourceType, type index_gen$j_PermissionSet as PermissionSet, type index_gen$j_PermissionSetScopeType as PermissionSetScopeType, type index_gen$j_Policy as Policy, type index_gen$j_Quotum as Quotum, type index_gen$j_RemoveGroupMemberRequest as RemoveGroupMemberRequest, type index_gen$j_Rule as Rule, type index_gen$j_RuleSpecs as RuleSpecs, type index_gen$j_SSHKey as SSHKey, type index_gen$j_SetGroupMembersRequest as SetGroupMembersRequest, type index_gen$j_SetRulesRequest as SetRulesRequest, type index_gen$j_SetRulesResponse as SetRulesResponse, type UpdateAPIKeyRequest$1 as UpdateAPIKeyRequest, type UpdateApplicationRequest$1 as UpdateApplicationRequest, type UpdateGroupRequest$1 as UpdateGroupRequest, type UpdatePolicyRequest$1 as UpdatePolicyRequest, type UpdateSSHKeyRequest$1 as UpdateSSHKeyRequest, type UpdateUserRequest$1 as UpdateUserRequest, type User$1 as User, type index_gen$j_UserStatus as UserStatus, type index_gen$j_UserType as UserType, validationRules_gen$7 as ValidationRules };
}

declare namespace index$m {
  export { index_gen$j as v1alpha1 };
}

type Arch = 'unknown_arch' | 'x86_64' | 'arm' | 'arm64';
type AttachServerVolumeRequestVolumeType = 'unknown_volume_type' | 'l_ssd' | 'b_ssd' | 'sbs_volume';
type BootType = 'local' | 'bootscript' | 'rescue';
type ImageState = 'available' | 'creating' | 'error';
type IpState = 'unknown_state' | 'detached' | 'attached' | 'pending' | 'error';
type IpType = 'unknown_iptype' | 'nat' | 'routed_ipv4' | 'routed_ipv6';
type ListServersRequestOrder = 'creation_date_desc' | 'creation_date_asc' | 'modification_date_desc' | 'modification_date_asc';
type PlacementGroupPolicyMode = 'optional' | 'enforced';
type PlacementGroupPolicyType = 'max_availability' | 'low_latency';
type PrivateNICState = 'available' | 'syncing' | 'syncing_error';
type SecurityGroupPolicy = 'unknown_policy' | 'accept' | 'drop';
type SecurityGroupRuleAction = 'unknown_action' | 'accept' | 'drop';
type SecurityGroupRuleDirection = 'unknown_direction' | 'inbound' | 'outbound';
type SecurityGroupRuleProtocol = 'unknown_protocol' | 'TCP' | 'UDP' | 'ICMP' | 'ANY';
type SecurityGroupState = 'available' | 'syncing' | 'syncing_error';
type ServerAction = 'poweron' | 'backup' | 'stop_in_place' | 'poweroff' | 'terminate' | 'reboot' | 'enable_routed_ip';
type ServerIpIpFamily = 'inet' | 'inet6';
type ServerIpProvisioningMode = 'manual' | 'dhcp' | 'slaac';
type ServerIpState = 'unknown_state' | 'detached' | 'attached' | 'pending' | 'error';
type ServerState = 'running' | 'stopped' | 'stopped in place' | 'starting' | 'stopping' | 'locked';
type ServerTypesAvailability = 'available' | 'scarce' | 'shortage';
type SnapshotState = 'available' | 'snapshotting' | 'error' | 'invalid_data' | 'importing' | 'exporting';
type SnapshotVolumeType$1 = 'unknown_volume_type' | 'l_ssd' | 'b_ssd' | 'unified';
type TaskStatus = 'pending' | 'started' | 'success' | 'failure' | 'retry';
type VolumeServerState = 'available' | 'snapshotting' | 'fetching' | 'resizing' | 'saving' | 'hotsyncing' | 'error';
type VolumeServerVolumeType = 'l_ssd' | 'b_ssd' | 'sbs_volume' | 'scratch';
type VolumeState = 'available' | 'snapshotting' | 'fetching' | 'resizing' | 'saving' | 'hotsyncing' | 'error';
type VolumeVolumeType = 'l_ssd' | 'b_ssd' | 'unified' | 'scratch' | 'sbs_volume' | 'sbs_snapshot';
interface ServerSummary {
    id: string;
    name: string;
}
interface Bootscript {
    /** Bootscript arguments. */
    bootcmdargs: string;
    /**
     * Display if the bootscript is the default bootscript (if no other boot
     * option is configured).
     */
    default: boolean;
    /**
     * Provide information regarding a Device Tree Binary (DTB) for use with C1
     * servers.
     */
    dtb: string;
    /** Bootscript ID. */
    id: string;
    /** Initrd (initial ramdisk) configuration. */
    initrd: string;
    /** Instance kernel version. */
    kernel: string;
    /** Bootscript Organization ID. */
    organization: string;
    /** Bootscript Project ID. */
    project: string;
    /** Provide information if the bootscript is public. */
    public: boolean;
    /** Bootscript title. */
    title: string;
    /** Bootscript architecture. */
    arch: Arch;
    /** Zone in which the bootscript is located. */
    zone: Zone;
}
interface Volume$2 {
    /** Volume unique ID. */
    id: string;
    /** Volume name. */
    name: string;
    /** @deprecated Show the volume NBD export URI. */
    exportUri?: string;
    /** Volume disk size. */
    size: number;
    /** Volume type. */
    volumeType: VolumeVolumeType;
    /** Volume creation date. */
    creationDate?: Date;
    /** Volume modification date. */
    modificationDate?: Date;
    /** Volume Organization ID. */
    organization: string;
    /** Volume Project ID. */
    project: string;
    /** Volume tags. */
    tags: string[];
    /** Instance attached to the volume. */
    server?: ServerSummary;
    /** Volume state. */
    state: VolumeState;
    /** Zone in which the volume is located. */
    zone: Zone;
}
interface VolumeSummary {
    id: string;
    name: string;
    size: number;
    volumeType: VolumeVolumeType;
}
interface ServerTypeNetworkInterface {
    /** Maximum internal bandwidth in bits per seconds. */
    internalBandwidth?: number;
    /** Maximum internet bandwidth in bits per seconds. */
    internetBandwidth?: number;
}
interface ServerTypeVolumeConstraintSizes {
    /** Minimum volume size in bytes. */
    minSize: number;
    /** Maximum volume size in bytes. */
    maxSize: number;
}
interface Image$3 {
    id: string;
    name: string;
    arch: Arch;
    creationDate?: Date;
    modificationDate?: Date;
    /** @deprecated */
    defaultBootscript?: Bootscript;
    extraVolumes: Record<string, Volume$2>;
    fromServer: string;
    organization: string;
    public: boolean;
    rootVolume?: VolumeSummary;
    state: ImageState;
    project: string;
    tags: string[];
    /** Zone to target. If none is passed will use default zone from the config. */
    zone: Zone;
}
interface PlacementGroup {
    /** Placement group unique ID. */
    id: string;
    /** Placement group name. */
    name: string;
    /** Placement group Organization ID. */
    organization: string;
    /** Placement group Project ID. */
    project: string;
    /** Placement group tags. */
    tags: string[];
    /**
     * Select the failure mode when the placement cannot be respected, either
     * optional or enforced.
     */
    policyMode: PlacementGroupPolicyMode;
    /**
     * Select the behavior of the placement group, either low_latency (group) or
     * max_availability (spread).
     */
    policyType: PlacementGroupPolicyType;
    /** Returns true if the policy is respected, false otherwise. */
    policyRespected: boolean;
    /** Zone in which the placement group is located. */
    zone: Zone;
}
interface PrivateNIC {
    /** Private NIC unique ID. */
    id: string;
    /** Instance to which the private NIC is attached. */
    serverId: string;
    /** Private Network the private NIC is attached to. */
    privateNetworkId: string;
    /** Private NIC MAC address. */
    macAddress: string;
    /** Private NIC state. */
    state: PrivateNICState;
    /** Private NIC tags. */
    tags: string[];
}
interface SecurityGroupSummary {
    id: string;
    name: string;
}
interface ServerIp {
    /** Unique ID of the IP address. */
    id: string;
    /** Instance's public IP-Address. */
    address: string;
    /** Gateway's IP address. */
    gateway: string;
    /** CIDR netmask. */
    netmask: string;
    /** IP address family (inet or inet6). */
    family: ServerIpIpFamily;
    /** True if the IP address is dynamic. */
    dynamic: boolean;
    /** Information about this address provisioning mode. */
    provisioningMode: ServerIpProvisioningMode;
    /** Tags associated with the IP. */
    tags: string[];
    /** The ip_id of an IPAM ip if the ip is created from IPAM, null if not. */
    ipamId: string;
    /** IP address state. */
    state: ServerIpState;
}
interface ServerIpv6 {
    /** Instance IPv6 IP-Address. */
    address: string;
    /** IPv6 IP-addresses gateway. */
    gateway: string;
    /** IPv6 IP-addresses CIDR netmask. */
    netmask: string;
}
interface ServerLocation {
    clusterId: string;
    hypervisorId: string;
    nodeId: string;
    platformId: string;
    zoneId: string;
}
interface ServerMaintenance {
    reason: string;
    startDate?: Date;
}
interface VolumeServer {
    id: string;
    name: string;
    exportUri: string;
    organization: string;
    server?: ServerSummary;
    size: number;
    volumeType: VolumeServerVolumeType;
    creationDate?: Date;
    modificationDate?: Date;
    state: VolumeServerState;
    project: string;
    boot: boolean;
    /** Zone to target. If none is passed will use default zone from the config. */
    zone: Zone;
}
interface SnapshotBaseVolume {
    /** Volume ID on which the snapshot is based. */
    id: string;
    /** Volume name on which the snapshot is based on. */
    name: string;
}
interface ServerTypeCapabilities {
    /** Defines whether the Instance supports block storage. */
    blockStorage?: boolean;
    /** List of supported boot types. */
    bootTypes: BootType[];
}
interface ServerTypeNetwork {
    /** List of available network interfaces. */
    interfaces: ServerTypeNetworkInterface[];
    /** Total maximum internal bandwidth in bits per seconds. */
    sumInternalBandwidth?: number;
    /** Total maximum internet bandwidth in bits per seconds. */
    sumInternetBandwidth?: number;
    /** True if IPv6 is enabled. */
    ipv6Support: boolean;
}
interface ServerTypeVolumeConstraintsByType {
    /** Local SSD volumes. */
    lSsd?: ServerTypeVolumeConstraintSizes;
}
interface VolumeTypeCapabilities {
    snapshot: boolean;
}
interface VolumeTypeConstraints {
    min: number;
    max: number;
}
interface Server {
    /** Instance unique ID. */
    id: string;
    /** Instance name. */
    name: string;
    /** Instance Organization ID. */
    organization: string;
    /** Instance Project ID. */
    project: string;
    /** List of allowed actions on the Instance. */
    allowedActions: ServerAction[];
    /** Tags associated with the Instance. */
    tags: string[];
    /** Instance commercial type (eg. GP1-M). */
    commercialType: string;
    /** Instance creation date. */
    creationDate?: Date;
    /** True if a dynamic IPv4 is required. */
    dynamicIpRequired: boolean;
    /** True to configure the instance so it uses the new routed IP mode. */
    routedIpEnabled: boolean;
    /** True if IPv6 is enabled. */
    enableIpv6: boolean;
    /** Instance host name. */
    hostname: string;
    /** Information about the Instance image. */
    image?: Image$3;
    /** Defines whether the Instance protection option is activated. */
    protected: boolean;
    /** Private IP address of the Instance. */
    privateIp?: string;
    /** Information about the public IP. */
    publicIp?: ServerIp;
    /** Information about all the public IPs attached to the server. */
    publicIps: ServerIp[];
    /** The server's MAC address. */
    macAddress: string;
    /** Instance modification date. */
    modificationDate?: Date;
    /** Instance state. */
    state: ServerState;
    /** Instance location. */
    location?: ServerLocation;
    /** Instance IPv6 address. */
    ipv6?: ServerIpv6;
    /** @deprecated Instance bootscript. */
    bootscript?: Bootscript;
    /** Instance boot type. */
    bootType: BootType;
    /** Instance volumes. */
    volumes: Record<string, VolumeServer>;
    /** Instance security group. */
    securityGroup?: SecurityGroupSummary;
    /** Instance planned maintenance. */
    maintenances: ServerMaintenance[];
    /** Detailed information about the Instance state. */
    stateDetail: string;
    /** Instance architecture. */
    arch: Arch;
    /** Instance placement group. */
    placementGroup?: PlacementGroup;
    /** Instance private NICs. */
    privateNics: PrivateNIC[];
    /** Zone in which the Instance is located. */
    zone: Zone;
}
interface VolumeTemplate {
    /** UUID of the volume. */
    id: string;
    /** Name of the volume. */
    name: string;
    /** Disk size of the volume, must be a multiple of 512. */
    size: number;
    /** Type of the volume. */
    volumeType: VolumeVolumeType;
    /**
     * @deprecated Organization ID of the volume.
     *
     *   One-of ('projectIdentifier'): at most one of 'project', 'organization'
     *   could be set.
     */
    organization?: string;
    /**
     * Project ID of the volume.
     *
     * One-of ('projectIdentifier'): at most one of 'project', 'organization'
     * could be set.
     */
    project?: string;
}
interface Ip$1 {
    id: string;
    address: string;
    reverse?: string;
    server?: ServerSummary;
    organization: string;
    tags: string[];
    project: string;
    type: IpType;
    state: IpState;
    prefix: string;
    ipamId: string;
    /** Zone to target. If none is passed will use default zone from the config. */
    zone: Zone;
}
interface SecurityGroup {
    /** Security group unique ID. */
    id: string;
    /** Security group name. */
    name: string;
    /** Security group description. */
    description: string;
    /**
     * True if SMTP is blocked on IPv4 and IPv6. This feature is read only, please
     * open a support ticket if you need to make it configurable.
     */
    enableDefaultSecurity: boolean;
    /** Default inbound policy. */
    inboundDefaultPolicy: SecurityGroupPolicy;
    /** Default outbound policy. */
    outboundDefaultPolicy: SecurityGroupPolicy;
    /** Security group Organization ID. */
    organization: string;
    /** Security group Project ID. */
    project: string;
    /** Security group tags. */
    tags: string[];
    /**
     * @deprecated True if it is your default security group for this Organization
     *   ID.
     */
    organizationDefault?: boolean;
    /** True if it is your default security group for this Project ID. */
    projectDefault: boolean;
    /** Security group creation date. */
    creationDate?: Date;
    /** Security group modification date. */
    modificationDate?: Date;
    /** List of Instances attached to this security group. */
    servers: ServerSummary[];
    /** Defines whether the security group is stateful. */
    stateful: boolean;
    /** Security group state. */
    state: SecurityGroupState;
    /** Zone in which the security group is located. */
    zone: Zone;
}
interface SecurityGroupRule {
    id: string;
    protocol: SecurityGroupRuleProtocol;
    direction: SecurityGroupRuleDirection;
    action: SecurityGroupRuleAction;
    ipRange: string;
    destPortFrom?: number;
    destPortTo?: number;
    position: number;
    editable: boolean;
    /** Zone to target. If none is passed will use default zone from the config. */
    zone: Zone;
}
interface VolumeServerTemplate {
    /** UUID of the volume. */
    id?: string;
    /** Force the Instance to boot on this volume. */
    boot?: boolean;
    /** Name of the volume. */
    name?: string;
    /** Disk size of the volume, must be a multiple of 512. */
    size?: number;
    /** Type of the volume. */
    volumeType: VolumeVolumeType;
    /** ID of the snapshot on which this volume will be based. */
    baseSnapshot?: string;
    /** Organization ID of the volume. */
    organization?: string;
    /** Project ID of the volume. */
    project?: string;
}
interface Snapshot$1 {
    /** Snapshot ID. */
    id: string;
    /** Snapshot name. */
    name: string;
    /** Snapshot Organization ID. */
    organization: string;
    /** Snapshot Project ID. */
    project: string;
    /** Snapshot tags. */
    tags: string[];
    /** Snapshot volume type. */
    volumeType: VolumeVolumeType;
    /** Snapshot size. */
    size: number;
    /** Snapshot state. */
    state: SnapshotState;
    /** Volume on which the snapshot is based on. */
    baseVolume?: SnapshotBaseVolume;
    /** Snapshot creation date. */
    creationDate?: Date;
    /** Snapshot modification date. */
    modificationDate?: Date;
    /** Snapshot zone. */
    zone: Zone;
    /** Reason for the failed snapshot import. */
    errorReason?: string;
}
interface Task {
    /** Unique ID of the task. */
    id: string;
    /** Description of the task. */
    description: string;
    /** Progress of the task in percent. */
    progress: number;
    /** Task start date. */
    startedAt?: Date;
    /** Task end date. */
    terminatedAt?: Date;
    /** Task status. */
    status: TaskStatus;
    hrefFrom: string;
    hrefResult: string;
    /** Zone in which the task is excecuted. */
    zone: Zone;
}
interface Dashboard {
    volumesCount: number;
    runningServersCount: number;
    serversByTypes: Record<string, number>;
    imagesCount: number;
    snapshotsCount: number;
    serversCount: number;
    ipsCount: number;
    securityGroupsCount: number;
    ipsUnused: number;
    volumesLSsdCount: number;
    volumesBSsdCount: number;
    volumesLSsdTotalSize: number;
    volumesBSsdTotalSize: number;
    privateNicsCount: number;
    placementGroupsCount: number;
}
interface PlacementGroupServer {
    /** Instance UUID. */
    id: string;
    /** Instance name. */
    name: string;
    /** Defines whether the placement group policy is respected (either 1 or 0). */
    policyRespected: boolean;
}
interface GetServerTypesAvailabilityResponseAvailability {
    availability: ServerTypesAvailability;
}
interface ServerType {
    /** @deprecated Estimated monthly price, for a 30 days month, in Euro. */
    monthlyPrice?: number;
    /** Hourly price in Euro. */
    hourlyPrice: number;
    /** Alternative Instance name, if any. */
    altNames: string[];
    /** Additional volume constraints. */
    perVolumeConstraint?: ServerTypeVolumeConstraintsByType;
    /** Initial volume constraints. */
    volumesConstraint?: ServerTypeVolumeConstraintSizes;
    /** Number of CPU. */
    ncpus: number;
    /** Number of GPU. */
    gpu?: number;
    /** Available RAM in bytes. */
    ram: number;
    /** CPU architecture. */
    arch: Arch;
    /** True if it is a baremetal Instance. */
    baremetal: boolean;
    /** Network available for the Instance. */
    network?: ServerTypeNetwork;
    /** Capabilities. */
    capabilities?: ServerTypeCapabilities;
    /** Maximum available scratch storage. */
    scratchStorageMaxSize?: number;
}
interface VolumeType$1 {
    displayName: string;
    capabilities?: VolumeTypeCapabilities;
    constraints?: VolumeTypeConstraints;
}
interface ServerActionRequestVolumeBackupTemplate {
    /**
     * Overrides the `volume_type` of the snapshot for this volume. If omitted,
     * the volume type of the original volume will be used.
     */
    volumeType: SnapshotVolumeType$1;
}
interface SetSecurityGroupRulesRequestRule {
    /**
     * UUID of the security rule to update. If no value is provided, a new rule
     * will be created.
     */
    id?: string;
    /** Action to apply when the rule matches a packet. */
    action: SecurityGroupRuleAction;
    /** Protocol family this rule applies to. */
    protocol: SecurityGroupRuleProtocol;
    /** Direction the rule applies to. */
    direction: SecurityGroupRuleDirection;
    /** Range of IP addresses these rules apply to. */
    ipRange: string;
    /**
     * Beginning of the range of ports this rule applies to (inclusive). This
     * value will be set to null if protocol is ICMP or ANY.
     */
    destPortFrom?: number;
    /**
     * End of the range of ports this rule applies to (inclusive). This value will
     * be set to null if protocol is ICMP or ANY, or if it is equal to
     * dest_port_from.
     */
    destPortTo?: number;
    /**
     * Position of this rule in the security group rules list. If several rules
     * are passed with the same position, the resulting order is undefined.
     */
    position: number;
    /**
     * Indicates if this rule is editable. Rules with the value false will be
     * ignored.
     */
    editable?: boolean;
    /** Zone of the rule. This field is ignored. */
    zone?: Zone;
}
interface VolumeImageUpdateTemplate {
    /** UUID of the snapshot. */
    id: string;
}
interface SecurityGroupTemplate {
    id: string;
    name: string;
}
type ApplyBlockMigrationRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * The volume to migrate, along with potentially other resources, according to
     * the migration plan generated with a call to PlanBlockMigration.
     *
     * One-of ('resource'): at most one of 'volumeId', 'snapshotId' could be set.
     */
    volumeId?: string;
    /**
     * The snapshot to migrate, along with potentially other resources, according
     * to the migration plan generated with a call to PlanBlockMigration.
     *
     * One-of ('resource'): at most one of 'volumeId', 'snapshotId' could be set.
     */
    snapshotId?: string;
    /**
     * A value to be retrieved from a call to PlanBlockMigration, to confirm that
     * the volume and/or snapshots specified in said plan should be migrated.
     */
    validationKey: string;
};
type AttachServerVolumeRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    serverId: string;
    volumeId: string;
    volumeType?: AttachServerVolumeRequestVolumeType;
    boot?: boolean;
};
interface AttachServerVolumeResponse {
    server?: Server;
}
type CreateImageRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Name of the image. */
    name?: string;
    /** UUID of the snapshot. */
    rootVolume: string;
    /** Architecture of the image. */
    arch: Arch;
    /** @deprecated Default bootscript of the image. */
    defaultBootscript?: string;
    /** Additional volumes of the image. */
    extraVolumes?: Record<string, VolumeTemplate>;
    /**
     * @deprecated Organization ID of the image.
     *
     *   One-of ('projectIdentifier'): at most one of 'project', 'organization'
     *   could be set.
     */
    organization?: string;
    /**
     * Project ID of the image.
     *
     * One-of ('projectIdentifier'): at most one of 'project', 'organization'
     * could be set.
     */
    project?: string;
    /** Tags of the image. */
    tags?: string[];
    /** True to create a public image. */
    public?: boolean;
};
interface CreateImageResponse {
    image?: Image$3;
}
type CreateIpRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * @deprecated Organization ID in which the IP is reserved.
     *
     *   One-of ('projectIdentifier'): at most one of 'project', 'organization'
     *   could be set.
     */
    organization?: string;
    /**
     * Project ID in which the IP is reserved.
     *
     * One-of ('projectIdentifier'): at most one of 'project', 'organization'
     * could be set.
     */
    project?: string;
    /** Tags of the IP. */
    tags?: string[];
    /** UUID of the Instance you want to attach the IP to. */
    server?: string;
    /** IP type to reserve (either 'nat', 'routed_ipv4' or 'routed_ipv6'). */
    type?: IpType;
};
interface CreateIpResponse {
    ip?: Ip$1;
}
type CreatePlacementGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Name of the placement group. */
    name?: string;
    /**
     * @deprecated Organization ID of the placement group.
     *
     *   One-of ('projectIdentifier'): at most one of 'project', 'organization'
     *   could be set.
     */
    organization?: string;
    /**
     * Project ID of the placement group.
     *
     * One-of ('projectIdentifier'): at most one of 'project', 'organization'
     * could be set.
     */
    project?: string;
    /** Tags of the placement group. */
    tags?: string[];
    /** Operating mode of the placement group. */
    policyMode?: PlacementGroupPolicyMode;
    /** Policy type of the placement group. */
    policyType?: PlacementGroupPolicyType;
};
interface CreatePlacementGroupResponse {
    placementGroup?: PlacementGroup;
}
type CreatePrivateNICRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Instance the private NIC will be attached to. */
    serverId: string;
    /** UUID of the private network where the private NIC will be attached. */
    privateNetworkId: string;
    /** Private NIC tags. */
    tags?: string[];
    /** Ip_ids defined from IPAM. */
    ipIds?: string[];
};
interface CreatePrivateNICResponse {
    privateNic?: PrivateNIC;
}
type CreateSecurityGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Name of the security group. */
    name?: string;
    /** Description of the security group. */
    description: string;
    /**
     * @deprecated Organization ID the security group belongs to.
     *
     *   One-of ('projectIdentifier'): at most one of 'project', 'organization'
     *   could be set.
     */
    organization?: string;
    /**
     * Project ID the security group belong to.
     *
     * One-of ('projectIdentifier'): at most one of 'project', 'organization'
     * could be set.
     */
    project?: string;
    /** Tags of the security group. */
    tags?: string[];
    /**
     * @deprecated Defines whether this security group becomes the default
     *   security group for new Instances.
     *
     *   One-of ('defaultIdentifier'): at most one of 'organizationDefault',
     *   'projectDefault' could be set.
     */
    organizationDefault?: boolean;
    /**
     * Whether this security group becomes the default security group for new
     * Instances.
     *
     * One-of ('defaultIdentifier'): at most one of 'organizationDefault',
     * 'projectDefault' could be set.
     */
    projectDefault?: boolean;
    /** Whether the security group is stateful or not. */
    stateful: boolean;
    /** Default policy for inbound rules. */
    inboundDefaultPolicy?: SecurityGroupPolicy;
    /** Default policy for outbound rules. */
    outboundDefaultPolicy?: SecurityGroupPolicy;
    /**
     * True to block SMTP on IPv4 and IPv6. This feature is read only, please open
     * a support ticket if you need to make it configurable.
     */
    enableDefaultSecurity?: boolean;
};
interface CreateSecurityGroupResponse {
    securityGroup?: SecurityGroup;
}
type CreateSecurityGroupRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the security group. */
    securityGroupId: string;
    protocol: SecurityGroupRuleProtocol;
    direction: SecurityGroupRuleDirection;
    action: SecurityGroupRuleAction;
    ipRange: string;
    /** Beginning of the range of ports to apply this rule to (inclusive). */
    destPortFrom?: number;
    /** End of the range of ports to apply this rule to (inclusive). */
    destPortTo?: number;
    /** Position of this rule in the security group rules list. */
    position: number;
    /** Indicates if this rule is editable (will be ignored). */
    editable: boolean;
};
interface CreateSecurityGroupRuleResponse {
    rule?: SecurityGroupRule;
}
type CreateServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Instance name. */
    name?: string;
    /** Define if a dynamic IPv4 is required for the Instance. */
    dynamicIpRequired?: boolean;
    /** If true, configure the Instance so it uses the new routed IP mode. */
    routedIpEnabled?: boolean;
    /** Define the Instance commercial type (i.e. GP1-S). */
    commercialType: string;
    /** Instance image ID or label. */
    image: string;
    /** Volumes attached to the server. */
    volumes?: Record<string, VolumeServerTemplate>;
    /** True if IPv6 is enabled on the server. */
    enableIpv6: boolean;
    /** ID of the reserved IP to attach to the Instance. */
    publicIp?: string;
    /** A list of reserved IP IDs to attach to the Instance. */
    publicIps?: string[];
    /** Boot type to use. */
    bootType?: BootType;
    /** @deprecated Bootscript ID to use when `boot_type` is set to `bootscript`. */
    bootscript?: string;
    /**
     * @deprecated Instance Organization ID.
     *
     *   One-of ('projectIdentifier'): at most one of 'project', 'organization'
     *   could be set.
     */
    organization?: string;
    /**
     * Instance Project ID.
     *
     * One-of ('projectIdentifier'): at most one of 'project', 'organization'
     * could be set.
     */
    project?: string;
    /** Instance tags. */
    tags?: string[];
    /** Security group ID. */
    securityGroup?: string;
    /** Placement group ID if Instance must be part of a placement group. */
    placementGroup?: string;
};
interface CreateServerResponse {
    server?: Server;
}
type CreateSnapshotRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Name of the snapshot. */
    name?: string;
    /** UUID of the volume. */
    volumeId?: string;
    /** Tags of the snapshot. */
    tags?: string[];
    /**
     * @deprecated Organization ID of the snapshot.
     *
     *   One-of ('projectIdentifier'): at most one of 'project', 'organization'
     *   could be set.
     */
    organization?: string;
    /**
     * Project ID of the snapshot.
     *
     * One-of ('projectIdentifier'): at most one of 'project', 'organization'
     * could be set.
     */
    project?: string;
    /**
     * Overrides the volume_type of the snapshot. If omitted, the volume type of
     * the original volume will be used.
     */
    volumeType?: SnapshotVolumeType$1;
    /** Bucket name for snapshot imports. */
    bucket?: string;
    /** Object key for snapshot imports. */
    key?: string;
    /** Imported snapshot size, must be a multiple of 512. */
    size?: number;
};
interface CreateSnapshotResponse {
    snapshot?: Snapshot$1;
    task?: Task;
}
type CreateVolumeRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Volume name. */
    name?: string;
    /**
     * @deprecated Volume Organization ID.
     *
     *   One-of ('projectIdentifier'): at most one of 'project', 'organization'
     *   could be set.
     */
    organization?: string;
    /**
     * Volume Project ID.
     *
     * One-of ('projectIdentifier'): at most one of 'project', 'organization'
     * could be set.
     */
    project?: string;
    /** Volume tags. */
    tags?: string[];
    /** Volume type. */
    volumeType?: VolumeVolumeType;
    /**
     * Volume disk size, must be a multiple of 512.
     *
     * One-of ('from'): at most one of 'size', 'baseSnapshot' could be set.
     */
    size?: number;
    /**
     * ID of the snapshot on which this volume will be based.
     *
     * One-of ('from'): at most one of 'size', 'baseSnapshot' could be set.
     */
    baseSnapshot?: string;
};
interface CreateVolumeResponse {
    volume?: Volume$2;
}
type DeleteImageRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the image you want to delete. */
    imageId: string;
};
type DeleteIpRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID or address of the IP to delete. */
    ip: string;
};
type DeletePlacementGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the placement group you want to delete. */
    placementGroupId: string;
};
type DeletePrivateNICRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Instance to which the private NIC is attached. */
    serverId: string;
    /** Private NIC unique ID. */
    privateNicId: string;
};
type DeleteSecurityGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the security group you want to delete. */
    securityGroupId: string;
};
type DeleteSecurityGroupRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    securityGroupId: string;
    securityGroupRuleId: string;
};
type DeleteServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    serverId: string;
};
type DeleteServerUserDataRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Instance. */
    serverId: string;
    /** Key of the user data to delete. */
    key: string;
};
type DeleteSnapshotRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the snapshot you want to delete. */
    snapshotId: string;
};
type DeleteVolumeRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the volume you want to delete. */
    volumeId: string;
};
type DetachServerVolumeRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    serverId: string;
    volumeId: string;
};
interface DetachServerVolumeResponse {
    server?: Server;
}
type ExportSnapshotRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Snapshot ID. */
    snapshotId: string;
    /** S3 bucket name. */
    bucket: string;
    /** S3 object key. */
    key: string;
};
interface ExportSnapshotResponse {
    task?: Task;
}
type GetBootscriptRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    bootscriptId: string;
};
interface GetBootscriptResponse {
    bootscript?: Bootscript;
}
type GetDashboardRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    organization?: string;
    project?: string;
};
interface GetDashboardResponse {
    dashboard?: Dashboard;
}
type GetImageRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the image you want to get. */
    imageId: string;
};
interface GetImageResponse$1 {
    image?: Image$3;
}
type GetIpRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** IP ID or address to get. */
    ip: string;
};
interface GetIpResponse {
    ip?: Ip$1;
}
type GetPlacementGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the placement group you want to get. */
    placementGroupId: string;
};
interface GetPlacementGroupResponse {
    placementGroup?: PlacementGroup;
}
type GetPlacementGroupServersRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the placement group you want to get. */
    placementGroupId: string;
};
interface GetPlacementGroupServersResponse {
    /** Instances attached to the placement group. */
    servers: PlacementGroupServer[];
}
type GetPrivateNICRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Instance to which the private NIC is attached. */
    serverId: string;
    /** Private NIC unique ID. */
    privateNicId: string;
};
interface GetPrivateNICResponse {
    privateNic?: PrivateNIC;
}
type GetSecurityGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the security group you want to get. */
    securityGroupId: string;
};
interface GetSecurityGroupResponse {
    securityGroup?: SecurityGroup;
}
type GetSecurityGroupRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    securityGroupId: string;
    securityGroupRuleId: string;
};
interface GetSecurityGroupRuleResponse {
    rule?: SecurityGroupRule;
}
type GetServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Instance you want to get. */
    serverId: string;
};
interface GetServerResponse {
    server?: Server;
}
type GetServerTypesAvailabilityRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    perPage?: number;
    /** A positive integer to choose the page to return. */
    page?: number;
};
interface GetServerTypesAvailabilityResponse {
    /** Map of server types. */
    servers: Record<string, GetServerTypesAvailabilityResponseAvailability>;
    totalCount: number;
}
type GetSnapshotRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the snapshot you want to get. */
    snapshotId: string;
};
interface GetSnapshotResponse {
    snapshot?: Snapshot$1;
}
type GetVolumeRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the volume you want to get. */
    volumeId: string;
};
interface GetVolumeResponse {
    volume?: Volume$2;
}
type ListBootscriptsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    arch?: string;
    title?: string;
    default?: boolean;
    public?: boolean;
    perPage?: number;
    page?: number;
};
interface ListBootscriptsResponse {
    /** Total number of bootscripts. */
    totalCount: number;
    /** List of bootscripts. */
    bootscripts: Bootscript[];
}
type ListDefaultSecurityGroupRulesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
};
type ListImagesRequest$3 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    organization?: string;
    perPage?: number;
    page?: number;
    name?: string;
    public?: boolean;
    arch?: string;
    project?: string;
    tags?: string;
};
interface ListImagesResponse$3 {
    /** Total number of images. */
    totalCount: number;
    /** List of images. */
    images: Image$3[];
}
type ListIpsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Project ID in which the IPs are reserved. */
    project?: string;
    /** Organization ID in which the IPs are reserved. */
    organization?: string;
    /**
     * Filter IPs with these exact tags (to filter with several tags, use commas
     * to separate them).
     */
    tags?: string[];
    /** Filter on the IP address (Works as a LIKE operation on the IP address). */
    name?: string;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    perPage?: number;
    /** A positive integer to choose the page to return. */
    page?: number;
    /**
     * Filter on the IP Mobility IP type (whose value should be either 'nat',
     * 'routed_ipv4' or 'routed_ipv6').
     */
    type?: string;
};
interface ListIpsResponse$1 {
    /** Total number of ips. */
    totalCount: number;
    /** List of ips. */
    ips: Ip$1[];
}
type ListPlacementGroupsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    perPage?: number;
    /** A positive integer to choose the page to return. */
    page?: number;
    /** List only placement groups of this Organization ID. */
    organization?: string;
    /** List only placement groups of this Project ID. */
    project?: string;
    /**
     * List placement groups with these exact tags (to filter with several tags,
     * use commas to separate them).
     */
    tags?: string[];
    /**
     * Filter placement groups by name (for eg. "cluster1" will return
     * "cluster100" and "cluster1" but not "foo").
     */
    name?: string;
};
interface ListPlacementGroupsResponse {
    /** Total number of placement groups. */
    totalCount: number;
    /** List of placement groups. */
    placementGroups: PlacementGroup[];
}
type ListPrivateNICsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Instance to which the private NIC is attached. */
    serverId: string;
    /** Private NIC tags. */
    tags?: string[];
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    perPage?: number;
    /** A positive integer to choose the page to return. */
    page?: number;
};
interface ListPrivateNICsResponse {
    privateNics: PrivateNIC[];
    totalCount: number;
}
type ListSecurityGroupRulesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the security group. */
    securityGroupId: string;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    perPage?: number;
    /** A positive integer to choose the page to return. */
    page?: number;
};
interface ListSecurityGroupRulesResponse {
    /** Total number of security groups. */
    totalCount: number;
    /** List of security rules. */
    rules: SecurityGroupRule[];
}
type ListSecurityGroupsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Name of the security group. */
    name?: string;
    /** Security group Organization ID. */
    organization?: string;
    /** Security group Project ID. */
    project?: string;
    /**
     * List security groups with these exact tags (to filter with several tags,
     * use commas to separate them).
     */
    tags?: string[];
    /** Filter security groups with this value for project_default. */
    projectDefault?: boolean;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    perPage?: number;
    /** A positive integer to choose the page to return. */
    page?: number;
};
interface ListSecurityGroupsResponse {
    /** Total number of security groups. */
    totalCount: number;
    /** List of security groups. */
    securityGroups: SecurityGroup[];
}
type ListServerActionsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    serverId: string;
};
interface ListServerActionsResponse {
    actions: ServerAction[];
}
type ListServerUserDataRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Instance. */
    serverId: string;
};
interface ListServerUserDataResponse {
    userData: string[];
}
type ListServersRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    perPage?: number;
    /** A positive integer to choose the page to return. */
    page?: number;
    /** List only Instances of this Organization ID. */
    organization?: string;
    /** List only Instances of this Project ID. */
    project?: string;
    /**
     * Filter Instances by name (eg. "server1" will return "server100" and
     * "server1" but not "foo").
     */
    name?: string;
    /** List Instances by private_ip. */
    privateIp?: string;
    /** List Instances that are not attached to a public IP. */
    withoutIp?: boolean;
    /** List Instances of this commercial type. */
    commercialType?: string;
    /** List Instances in this state. */
    state?: ServerState;
    /**
     * List Instances with these exact tags (to filter with several tags, use
     * commas to separate them).
     */
    tags?: string[];
    /** List Instances in this Private Network. */
    privateNetwork?: string;
    /** Define the order of the returned servers. */
    order?: ListServersRequestOrder;
    /**
     * List Instances from the given Private Networks (use commas to separate
     * them).
     */
    privateNetworks?: string[];
    /** List Instances associated with the given private NIC MAC address. */
    privateNicMacAddress?: string;
    /** List Instances from these server ids (use commas to separate them). */
    servers?: string[];
};
interface ListServersResponse {
    /** Total number of Instances. */
    totalCount: number;
    /** List of Instances. */
    servers: Server[];
}
type ListServersTypesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    perPage?: number;
    page?: number;
};
interface ListServersTypesResponse {
    /** Total number of Instance types. */
    totalCount: number;
    /** List of Instance types. */
    servers: Record<string, ServerType>;
}
type ListSnapshotsRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** List snapshots only for this Organization ID. */
    organization?: string;
    /** List snapshots only for this Project ID. */
    project?: string;
    /**
     * Number of snapshots returned per page (positive integer lower or equal to
     * 100).
     */
    perPage?: number;
    /** Page to be returned. */
    page?: number;
    /** List snapshots of the requested name. */
    name?: string;
    /** List snapshots that have the requested tag. */
    tags?: string;
    /** List snapshots originating only from this volume. */
    baseVolumeId?: string;
};
interface ListSnapshotsResponse$1 {
    /** Total number of snapshots. */
    totalCount: number;
    /** List of snapshots. */
    snapshots: Snapshot$1[];
}
type ListVolumesRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Filter by volume type. */
    volumeType?: VolumeVolumeType;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * return.
     */
    perPage?: number;
    /** A positive integer to choose the page to return. */
    page?: number;
    /** Filter volume by Organization ID. */
    organization?: string;
    /** Filter volume by Project ID. */
    project?: string;
    /**
     * Filter volumes with these exact tags (to filter with several tags, use
     * commas to separate them).
     */
    tags?: string[];
    /**
     * Filter volume by name (for eg. "vol" will return "myvolume" but not
     * "data").
     */
    name?: string;
};
interface ListVolumesResponse$1 {
    /** Total number of volumes. */
    totalCount: number;
    /** List of volumes. */
    volumes: Volume$2[];
}
type ListVolumesTypesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    perPage?: number;
    page?: number;
};
interface ListVolumesTypesResponse {
    /** Total number of volume types. */
    totalCount: number;
    /** Map of volume types. */
    volumes: Record<string, VolumeType$1>;
}
interface MigrationPlan {
    /**
     * A volume which will be migrated to SBS together with the snapshots, if
     * present.
     */
    volume?: Volume$2;
    /**
     * A list of snapshots which will be migrated to SBS together and with the
     * volume, if present.
     */
    snapshots: Snapshot$1[];
    /**
     * A value to be passed to ApplyBlockMigrationRequest, to confirm that the
     * execution of the plan is being requested.
     */
    validationKey: string;
}
type PlanBlockMigrationRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * The volume for which the migration plan will be generated.
     *
     * One-of ('resource'): at most one of 'volumeId', 'snapshotId' could be set.
     */
    volumeId?: string;
    /**
     * The snapshot for which the migration plan will be generated.
     *
     * One-of ('resource'): at most one of 'volumeId', 'snapshotId' could be set.
     */
    snapshotId?: string;
};
type ServerActionRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Instance. */
    serverId: string;
    /** Action to perform on the Instance. */
    action?: ServerAction;
    /**
     * Name of the backup you want to create. This field should only be specified
     * when performing a backup action.
     */
    name?: string;
    /**
     * For each volume UUID, the snapshot parameters of the volume. This field
     * should only be specified when performing a backup action.
     */
    volumes?: Record<string, ServerActionRequestVolumeBackupTemplate>;
};
interface ServerActionResponse {
    task?: Task;
}
type SetImageRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    id: string;
    name: string;
    arch?: Arch;
    creationDate?: Date;
    modificationDate?: Date;
    /** @deprecated */
    defaultBootscript?: Bootscript;
    extraVolumes?: Record<string, Volume$2>;
    fromServer: string;
    organization?: string;
    public: boolean;
    rootVolume?: VolumeSummary;
    state?: ImageState;
    project?: string;
    tags?: string[];
};
type SetPlacementGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    placementGroupId: string;
    name: string;
    organization?: string;
    policyMode?: PlacementGroupPolicyMode;
    policyType?: PlacementGroupPolicyType;
    project?: string;
    tags?: string[];
};
interface SetPlacementGroupResponse {
    placementGroup?: PlacementGroup;
}
type SetPlacementGroupServersRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the placement group you want to set. */
    placementGroupId: string;
    /** An array of the Instances' UUIDs you want to configure. */
    servers: string[];
};
interface SetPlacementGroupServersResponse {
    /** Instances attached to the placement group. */
    servers: PlacementGroupServer[];
}
type SetSecurityGroupRulesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the security group to update the rules on. */
    securityGroupId: string;
    /** List of rules to update in the security group. */
    rules?: SetSecurityGroupRulesRequestRule[];
};
interface SetSecurityGroupRulesResponse {
    rules: SecurityGroupRule[];
}
type UpdateImageRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the image. */
    imageId: string;
    /** Name of the image. */
    name?: string;
    /** Architecture of the image. */
    arch?: Arch;
    /**
     * Additional snapshots of the image, with extra_volumeKey being the position
     * of the snapshot in the image.
     */
    extraVolumes?: Record<string, VolumeImageUpdateTemplate>;
    /** Tags of the image. */
    tags?: string[];
    /** True to set the image as public. */
    public?: boolean;
};
interface UpdateImageResponse {
    image?: Image$3;
}
type UpdateIpRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** IP ID or IP address. */
    ip: string;
    /** Reverse domain name. */
    reverse?: string | null;
    /** Convert a 'nat' IP to a 'routed_ipv4'. */
    type?: IpType;
    /** An array of keywords you want to tag this IP with. */
    tags?: string[];
    server?: string | null;
};
interface UpdateIpResponse {
    ip?: Ip$1;
}
type UpdatePlacementGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the placement group. */
    placementGroupId: string;
    /** Name of the placement group. */
    name?: string;
    /** Tags of the placement group. */
    tags?: string[];
    /** Operating mode of the placement group. */
    policyMode?: PlacementGroupPolicyMode;
    /** Policy type of the placement group. */
    policyType?: PlacementGroupPolicyType;
};
interface UpdatePlacementGroupResponse {
    placementGroup?: PlacementGroup;
}
type UpdatePlacementGroupServersRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the placement group you want to update. */
    placementGroupId: string;
    /** An array of the Instances' UUIDs you want to configure. */
    servers: string[];
};
interface UpdatePlacementGroupServersResponse {
    /** Instances attached to the placement group. */
    servers: PlacementGroupServer[];
}
type UpdatePrivateNICRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Instance the private NIC will be attached to. */
    serverId: string;
    /** Private NIC unique ID. */
    privateNicId: string;
    /** Tags used to select private NIC/s. */
    tags?: string[];
};
type UpdateSecurityGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the security group. */
    securityGroupId: string;
    /** Name of the security group. */
    name?: string;
    /** Description of the security group. */
    description?: string;
    /**
     * True to block SMTP on IPv4 and IPv6. This feature is read only, please open
     * a support ticket if you need to make it configurable.
     */
    enableDefaultSecurity?: boolean;
    /** Default inbound policy. */
    inboundDefaultPolicy?: SecurityGroupPolicy;
    /** Tags of the security group. */
    tags?: string[];
    /** @deprecated Please use project_default instead. */
    organizationDefault?: boolean;
    /** True use this security group for future Instances created in this project. */
    projectDefault?: boolean;
    /** Default outbound policy. */
    outboundDefaultPolicy?: SecurityGroupPolicy;
    /** True to set the security group as stateful. */
    stateful?: boolean;
};
interface UpdateSecurityGroupResponse {
    securityGroup?: SecurityGroup;
}
type UpdateSecurityGroupRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the security group. */
    securityGroupId: string;
    /** UUID of the rule. */
    securityGroupRuleId: string;
    /** Protocol family this rule applies to. */
    protocol?: SecurityGroupRuleProtocol;
    /** Direction the rule applies to. */
    direction?: SecurityGroupRuleDirection;
    /** Action to apply when the rule matches a packet. */
    action?: SecurityGroupRuleAction;
    /** Range of IP addresses these rules apply to. */
    ipRange?: string;
    /**
     * Beginning of the range of ports this rule applies to (inclusive). If 0 is
     * provided, unset the parameter.
     */
    destPortFrom?: number;
    /**
     * End of the range of ports this rule applies to (inclusive). If 0 is
     * provided, unset the parameter.
     */
    destPortTo?: number;
    /** Position of this rule in the security group rules list. */
    position?: number;
};
interface UpdateSecurityGroupRuleResponse {
    rule?: SecurityGroupRule;
}
type UpdateServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Instance. */
    serverId: string;
    /** Name of the Instance. */
    name?: string;
    bootType?: BootType;
    /** Tags of the Instance. */
    tags?: string[];
    volumes?: Record<string, VolumeServerTemplate>;
    /** @deprecated */
    bootscript?: string;
    dynamicIpRequired?: boolean;
    /**
     * True to configure the instance so it uses the new routed IP mode (once this
     * is set to True you cannot set it back to False).
     */
    routedIpEnabled?: boolean;
    /** A list of reserved IP IDs to attach to the Instance. */
    publicIps?: string[];
    enableIpv6?: boolean;
    protected?: boolean;
    securityGroup?: SecurityGroupTemplate;
    /** Placement group ID if Instance must be part of a placement group. */
    placementGroup?: string | null;
    /** Instance private NICs. */
    privateNics?: string[];
    /**
     * Warning: This field has some restrictions:
     *
     * - Cannot be changed if the Instance is not in `stopped` state.
     * - Cannot be changed if the Instance is in a placement group.
     * - Local storage requirements of the target commercial_types must be fulfilled
     *   (i.e. if an Instance has 80GB of local storage, it can be changed into a
     *   GP1-XS, which has a maximum of 150GB, but it cannot be changed into a
     *   DEV1-S, which has only 20GB).
     */
    commercialType?: string;
};
interface UpdateServerResponse {
    server?: Server;
}
type UpdateSnapshotRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the snapshot. */
    snapshotId: string;
    /** Name of the snapshot. */
    name?: string;
    /** Tags of the snapshot. */
    tags?: string[];
};
interface UpdateSnapshotResponse {
    snapshot?: Snapshot$1;
}
type UpdateVolumeRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the volume. */
    volumeId: string;
    /** Volume name. */
    name?: string;
    /** Tags of the volume. */
    tags?: string[];
    /** Volume disk size, must be a multiple of 512. */
    size?: number;
};
interface UpdateVolumeResponse {
    volume?: Volume$2;
}

interface SetImageResponse {
    image?: Image$3;
}
type SetSecurityGroupRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the security group. */
    id: string;
    /** Name of the security group. */
    name: string;
    /** Tags of the security group. */
    tags?: string[];
    /** Creation date of the security group (will be ignored). */
    creationDate?: Date;
    /** Modification date of the security group (will be ignored). */
    modificationDate?: Date;
    /** Description of the security group. */
    description: string;
    /**
     * True to block SMTP on IPv4 and IPv6. This feature is read only, please open
     * a support ticket if you need to make it configurable.
     */
    enableDefaultSecurity: boolean;
    /** Default inbound policy. */
    inboundDefaultPolicy?: SecurityGroupPolicy;
    /** Default outbound policy. */
    outboundDefaultPolicy?: SecurityGroupPolicy;
    /** Security groups Organization ID. */
    organization?: string;
    /** Security group Project ID. */
    project?: string;
    /** @deprecated Please use project_default instead. */
    organizationDefault?: boolean;
    /** True use this security group for future Instances created in this project. */
    projectDefault: boolean;
    /** Instances attached to this security group. */
    servers?: ServerSummary[];
    /** True to set the security group as stateful. */
    stateful: boolean;
};
interface SetSecurityGroupResponse {
    securityGroup?: SecurityGroup;
}
type SetSecurityGroupRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    securityGroupId: string;
    securityGroupRuleId: string;
    id: string;
    protocol?: SecurityGroupRuleProtocol;
    direction?: SecurityGroupRuleDirection;
    action?: SecurityGroupRuleAction;
    ipRange: string;
    destPortFrom?: number;
    destPortTo?: number;
    position: number;
    editable: boolean;
};
interface SetSecurityGroupRuleResponse {
    rule?: SecurityGroupRule;
}
type SetServerRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Instance unique ID. */
    id: string;
    /** Instance name. */
    name: string;
    /** Instance Organization ID. */
    organization?: string;
    /** Instance Project ID. */
    project?: string;
    /** Provide a list of allowed actions on the server. */
    allowedActions?: ServerAction[];
    /** Tags associated with the Instance. */
    tags?: string[];
    /** Instance commercial type (eg. GP1-M). */
    commercialType: string;
    /** Instance creation date. */
    creationDate?: Date;
    /** True if a dynamic IPv4 is required. */
    dynamicIpRequired: boolean;
    /**
     * True to configure the instance so it uses the new routed IP mode (once this
     * is set to True you cannot set it back to False).
     */
    routedIpEnabled?: boolean;
    /** True if IPv6 is enabled. */
    enableIpv6: boolean;
    /** Instance host name. */
    hostname: string;
    /** Provide information on the Instance image. */
    image?: Image$3;
    /** Instance protection option is activated. */
    protected: boolean;
    /** Instance private IP address. */
    privateIp?: string;
    /** Information about the public IP. */
    publicIp?: ServerIp;
    /** Information about all the public IPs attached to the server. */
    publicIps?: ServerIp[];
    /** Instance modification date. */
    modificationDate?: Date;
    /** Instance state. */
    state?: ServerState;
    /** Instance location. */
    location?: ServerLocation;
    /** Instance IPv6 address. */
    ipv6?: ServerIpv6;
    /** @deprecated Instance bootscript. */
    bootscript?: Bootscript;
    /** Instance boot type. */
    bootType?: BootType;
    /** Instance volumes. */
    volumes?: Record<string, Volume$2>;
    /** Instance security group. */
    securityGroup?: SecurityGroupSummary;
    /** Instance planned maintenances. */
    maintenances?: ServerMaintenance[];
    /** Instance state_detail. */
    stateDetail: string;
    /**
     * Instance architecture (refers to the CPU architecture used for the
     * Instance, e.g. x86_64, arm64).
     */
    arch?: Arch;
    /** Instance placement group. */
    placementGroup?: PlacementGroup;
    /** Instance private NICs. */
    privateNics?: PrivateNIC[];
};
interface SetServerResponse {
    server?: Server;
}
type SetSnapshotRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    snapshotId: string;
    id: string;
    name: string;
    organization?: string;
    volumeType?: VolumeVolumeType;
    size: number;
    state?: SnapshotState;
    baseVolume?: SnapshotBaseVolume;
    creationDate?: Date;
    modificationDate?: Date;
    project?: string;
    tags?: string[];
};
interface SetSnapshotResponse {
    snapshot?: Snapshot$1;
}

/** Instance API. */
declare class API$j extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    /**
     * Get availability. Get availability for all Instance types.
     *
     * @param request - The request {@link GetServerTypesAvailabilityRequest}
     * @returns A Promise of GetServerTypesAvailabilityResponse
     */
    getServerTypesAvailability: (request?: Readonly<GetServerTypesAvailabilityRequest>) => Promise<GetServerTypesAvailabilityResponse>;
    /**
     * List Instance types. List available Instance types and their technical
     * details.
     *
     * @param request - The request {@link ListServersTypesRequest}
     * @returns A Promise of ListServersTypesResponse
     */
    listServersTypes: (request?: Readonly<ListServersTypesRequest>) => Promise<ListServersTypesResponse>;
    /**
     * List volume types. List all volume types and their technical details.
     *
     * @param request - The request {@link ListVolumesTypesRequest}
     * @returns A Promise of ListVolumesTypesResponse
     */
    listVolumesTypes: (request?: Readonly<ListVolumesTypesRequest>) => Promise<ListVolumesTypesResponse>;
    protected pageOfListServers: (request?: Readonly<ListServersRequest>) => Promise<ListServersResponse>;
    /**
     * List all Instances. List all Instances in a specified Availability Zone,
     * e.g. `fr-par-1`.
     *
     * @param request - The request {@link ListServersRequest}
     * @returns A Promise of ListServersResponse
     */
    listServers: (request?: Readonly<ListServersRequest>) => Promise<ListServersResponse> & {
        all: () => Promise<Server[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Server[], void, void>;
    };
    protected _createServer: (request: Readonly<CreateServerRequest>) => Promise<CreateServerResponse>;
    /**
     * Delete an Instance. Delete the Instance with the specified ID.
     *
     * @param request - The request {@link DeleteServerRequest}
     */
    deleteServer: (request: Readonly<DeleteServerRequest>) => Promise<void>;
    /**
     * Get an Instance. Get the details of a specified Instance.
     *
     * @param request - The request {@link GetServerRequest}
     * @returns A Promise of GetServerResponse
     */
    getServer: (request: Readonly<GetServerRequest>) => Promise<GetServerResponse>;
    protected _setServer: (request: Readonly<SetServerRequest>) => Promise<SetServerResponse>;
    protected _updateServer: (request: Readonly<UpdateServerRequest>) => Promise<UpdateServerResponse>;
    /**
     * List Instance actions. List all actions (e.g. power on, power off, reboot)
     * that can currently be performed on an Instance.
     *
     * @param request - The request {@link ListServerActionsRequest}
     * @returns A Promise of ListServerActionsResponse
     */
    listServerActions: (request: Readonly<ListServerActionsRequest>) => Promise<ListServerActionsResponse>;
    /**
     * Perform action. Perform an action on an Instance. Available actions are:
     * `poweron`: Start a stopped Instance. `poweroff`: Fully stop the Instance
     * and release the hypervisor slot. `stop_in_place`: Stop the Instance, but
     * keep the slot on the hypervisor. `reboot`: Stop the instance and restart
     * it. `backup`: Create an image with all the volumes of an Instance.
     * `terminate`: Delete the Instance along with all attached volumes.
     * `enable_routed_ip`: Migrate the Instance to the new network stack.
     *
     * Keep in mind that terminating an Instance will result in the deletion of
     * all attached volumes, including local and block storage. If you want to
     * preserve your local volumes, you should use the `archive` action instead of
     * `terminate`. Similarly, if you want to keep your block storage volumes, you
     * must first detach them before issuing the `terminate` command. For more
     * information, read the [Volumes](#path-volumes-list-volumes) documentation.
     *
     * @param request - The request {@link ServerActionRequest}
     * @returns A Promise of ServerActionResponse
     */
    serverAction: (request: Readonly<ServerActionRequest>) => Promise<ServerActionResponse>;
    /**
     * List user data. List all user data keys registered on a specified Instance.
     *
     * @param request - The request {@link ListServerUserDataRequest}
     * @returns A Promise of ListServerUserDataResponse
     */
    listServerUserData: (request: Readonly<ListServerUserDataRequest>) => Promise<ListServerUserDataResponse>;
    /**
     * Delete user data. Delete the specified key from an Instance's user data.
     *
     * @param request - The request {@link DeleteServerUserDataRequest}
     */
    deleteServerUserData: (request: Readonly<DeleteServerUserDataRequest>) => Promise<void>;
    attachServerVolume: (request: Readonly<AttachServerVolumeRequest>) => Promise<AttachServerVolumeResponse>;
    detachServerVolume: (request: Readonly<DetachServerVolumeRequest>) => Promise<DetachServerVolumeResponse>;
    protected pageOfListImages: (request?: Readonly<ListImagesRequest$3>) => Promise<ListImagesResponse$3>;
    /**
     * List Instance images. List all existing Instance images.
     *
     * @param request - The request {@link ListImagesRequest}
     * @returns A Promise of ListImagesResponse
     */
    listImages: (request?: Readonly<ListImagesRequest$3>) => Promise<ListImagesResponse$3> & {
        all: () => Promise<Image$3[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Image$3[], void, void>;
    };
    /**
     * Get an Instance image. Get details of an image with the specified ID.
     *
     * @param request - The request {@link GetImageRequest}
     * @returns A Promise of GetImageResponse
     */
    getImage: (request: Readonly<GetImageRequest$3>) => Promise<GetImageResponse$1>;
    /**
     * Create an Instance image. Create an Instance image from the specified
     * snapshot ID.
     *
     * @param request - The request {@link CreateImageRequest}
     * @returns A Promise of CreateImageResponse
     */
    createImage: (request: Readonly<CreateImageRequest>) => Promise<CreateImageResponse>;
    protected _setImage: (request: Readonly<SetImageRequest>) => Promise<SetImageResponse>;
    /**
     * Update image. Update the properties of an image.
     *
     * @param request - The request {@link UpdateImageRequest}
     * @returns A Promise of UpdateImageResponse
     */
    updateImage: (request: Readonly<UpdateImageRequest$1>) => Promise<UpdateImageResponse>;
    /**
     * Delete an Instance image. Delete the image with the specified ID.
     *
     * @param request - The request {@link DeleteImageRequest}
     */
    deleteImage: (request: Readonly<DeleteImageRequest$1>) => Promise<void>;
    protected pageOfListSnapshots: (request?: Readonly<ListSnapshotsRequest$1>) => Promise<ListSnapshotsResponse$1>;
    /**
     * List snapshots. List all snapshots of an Organization in a specified
     * Availability Zone.
     *
     * @param request - The request {@link ListSnapshotsRequest}
     * @returns A Promise of ListSnapshotsResponse
     */
    listSnapshots: (request?: Readonly<ListSnapshotsRequest$1>) => Promise<ListSnapshotsResponse$1> & {
        all: () => Promise<Snapshot$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Snapshot$1[], void, void>;
    };
    /**
     * Create a snapshot from a specified volume or from a QCOW2 file. Create a
     * snapshot from a specified volume or from a QCOW2 file in a specified
     * Availability Zone.
     *
     * @param request - The request {@link CreateSnapshotRequest}
     * @returns A Promise of CreateSnapshotResponse
     */
    createSnapshot: (request?: Readonly<CreateSnapshotRequest$1>) => Promise<CreateSnapshotResponse>;
    /**
     * Get a snapshot. Get details of a snapshot with the specified ID.
     *
     * @param request - The request {@link GetSnapshotRequest}
     * @returns A Promise of GetSnapshotResponse
     */
    getSnapshot: (request: Readonly<GetSnapshotRequest$1>) => Promise<GetSnapshotResponse>;
    protected _setSnapshot: (request: Readonly<SetSnapshotRequest>) => Promise<SetSnapshotResponse>;
    /**
     * Update a snapshot. Update the properties of a snapshot.
     *
     * @param request - The request {@link UpdateSnapshotRequest}
     * @returns A Promise of UpdateSnapshotResponse
     */
    updateSnapshot: (request: Readonly<UpdateSnapshotRequest$1>) => Promise<UpdateSnapshotResponse>;
    /**
     * Delete a snapshot. Delete the snapshot with the specified ID.
     *
     * @param request - The request {@link DeleteSnapshotRequest}
     */
    deleteSnapshot: (request: Readonly<DeleteSnapshotRequest$1>) => Promise<void>;
    /**
     * Export a snapshot. Export a snapshot to a specified S3 bucket in the same
     * region.
     *
     * @param request - The request {@link ExportSnapshotRequest}
     * @returns A Promise of ExportSnapshotResponse
     */
    exportSnapshot: (request: Readonly<ExportSnapshotRequest>) => Promise<ExportSnapshotResponse>;
    protected pageOfListVolumes: (request?: Readonly<ListVolumesRequest$1>) => Promise<ListVolumesResponse$1>;
    /**
     * List volumes. List volumes in the specified Availability Zone. You can
     * filter the output by volume type.
     *
     * @param request - The request {@link ListVolumesRequest}
     * @returns A Promise of ListVolumesResponse
     */
    listVolumes: (request?: Readonly<ListVolumesRequest$1>) => Promise<ListVolumesResponse$1> & {
        all: () => Promise<Volume$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Volume$2[], void, void>;
    };
    /**
     * Create a volume. Create a volume of a specified type in an Availability
     * Zone.
     *
     * @param request - The request {@link CreateVolumeRequest}
     * @returns A Promise of CreateVolumeResponse
     */
    createVolume: (request?: Readonly<CreateVolumeRequest$1>) => Promise<CreateVolumeResponse>;
    /**
     * Get a volume. Get details of a volume with the specified ID.
     *
     * @param request - The request {@link GetVolumeRequest}
     * @returns A Promise of GetVolumeResponse
     */
    getVolume: (request: Readonly<GetVolumeRequest$1>) => Promise<GetVolumeResponse>;
    /**
     * Update a volume. Replace the name and/or size properties of a volume
     * specified by its ID, with the specified value(s). Any volume name can be
     * changed, however only `b_ssd` volumes can currently be increased in size.
     *
     * @param request - The request {@link UpdateVolumeRequest}
     * @returns A Promise of UpdateVolumeResponse
     */
    updateVolume: (request: Readonly<UpdateVolumeRequest$1>) => Promise<UpdateVolumeResponse>;
    /**
     * Delete a volume. Delete the volume with the specified ID.
     *
     * @param request - The request {@link DeleteVolumeRequest}
     */
    deleteVolume: (request: Readonly<DeleteVolumeRequest$1>) => Promise<void>;
    protected pageOfListSecurityGroups: (request?: Readonly<ListSecurityGroupsRequest>) => Promise<ListSecurityGroupsResponse>;
    /**
     * List security groups. List all existing security groups.
     *
     * @param request - The request {@link ListSecurityGroupsRequest}
     * @returns A Promise of ListSecurityGroupsResponse
     */
    listSecurityGroups: (request?: Readonly<ListSecurityGroupsRequest>) => Promise<ListSecurityGroupsResponse> & {
        all: () => Promise<SecurityGroup[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<SecurityGroup[], void, void>;
    };
    /**
     * Create a security group. Create a security group with a specified name and
     * description.
     *
     * @param request - The request {@link CreateSecurityGroupRequest}
     * @returns A Promise of CreateSecurityGroupResponse
     */
    createSecurityGroup: (request: Readonly<CreateSecurityGroupRequest>) => Promise<CreateSecurityGroupResponse>;
    /**
     * Get a security group. Get the details of a security group with the
     * specified ID.
     *
     * @param request - The request {@link GetSecurityGroupRequest}
     * @returns A Promise of GetSecurityGroupResponse
     */
    getSecurityGroup: (request: Readonly<GetSecurityGroupRequest>) => Promise<GetSecurityGroupResponse>;
    /**
     * Delete a security group. Delete a security group with the specified ID.
     *
     * @param request - The request {@link DeleteSecurityGroupRequest}
     */
    deleteSecurityGroup: (request: Readonly<DeleteSecurityGroupRequest>) => Promise<void>;
    protected _setSecurityGroup: (request: Readonly<SetSecurityGroupRequest>) => Promise<SetSecurityGroupResponse>;
    /**
     * Update a security group. Update the properties of security group.
     *
     * @param request - The request {@link UpdateSecurityGroupRequest}
     * @returns A Promise of UpdateSecurityGroupResponse
     */
    updateSecurityGroup: (request: Readonly<UpdateSecurityGroupRequest>) => Promise<UpdateSecurityGroupResponse>;
    /**
     * Get default rules. Lists the default rules applied to all the security
     * groups.
     *
     * @param request - The request {@link ListDefaultSecurityGroupRulesRequest}
     * @returns A Promise of ListSecurityGroupRulesResponse
     */
    listDefaultSecurityGroupRules: (request?: Readonly<ListDefaultSecurityGroupRulesRequest>) => Promise<ListSecurityGroupRulesResponse>;
    protected pageOfListSecurityGroupRules: (request: Readonly<ListSecurityGroupRulesRequest>) => Promise<ListSecurityGroupRulesResponse>;
    /**
     * List rules. List the rules of the a specified security group ID.
     *
     * @param request - The request {@link ListSecurityGroupRulesRequest}
     * @returns A Promise of ListSecurityGroupRulesResponse
     */
    listSecurityGroupRules: (request: Readonly<ListSecurityGroupRulesRequest>) => Promise<ListSecurityGroupRulesResponse> & {
        all: () => Promise<SecurityGroupRule[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<SecurityGroupRule[], void, void>;
    };
    /**
     * Create rule. Create a rule in the specified security group ID.
     *
     * @param request - The request {@link CreateSecurityGroupRuleRequest}
     * @returns A Promise of CreateSecurityGroupRuleResponse
     */
    createSecurityGroupRule: (request: Readonly<CreateSecurityGroupRuleRequest>) => Promise<CreateSecurityGroupRuleResponse>;
    /**
     * Update all the rules of a security group. Replaces the existing rules of
     * the security group with the rules provided. This endpoint supports the
     * update of existing rules, creation of new rules and deletion of existing
     * rules when they are not passed in the request.
     *
     * @param request - The request {@link SetSecurityGroupRulesRequest}
     * @returns A Promise of SetSecurityGroupRulesResponse
     */
    setSecurityGroupRules: (request: Readonly<SetSecurityGroupRulesRequest>) => Promise<SetSecurityGroupRulesResponse>;
    /**
     * Delete rule. Delete a security group rule with the specified ID.
     *
     * @param request - The request {@link DeleteSecurityGroupRuleRequest}
     */
    deleteSecurityGroupRule: (request: Readonly<DeleteSecurityGroupRuleRequest>) => Promise<void>;
    /**
     * Get rule. Get details of a security group rule with the specified ID.
     *
     * @param request - The request {@link GetSecurityGroupRuleRequest}
     * @returns A Promise of GetSecurityGroupRuleResponse
     */
    getSecurityGroupRule: (request: Readonly<GetSecurityGroupRuleRequest>) => Promise<GetSecurityGroupRuleResponse>;
    protected _setSecurityGroupRule: (request: Readonly<SetSecurityGroupRuleRequest>) => Promise<SetSecurityGroupRuleResponse>;
    /**
     * Update security group rule. Update the properties of a rule from a
     * specified security group.
     *
     * @param request - The request {@link UpdateSecurityGroupRuleRequest}
     * @returns A Promise of UpdateSecurityGroupRuleResponse
     */
    updateSecurityGroupRule: (request: Readonly<UpdateSecurityGroupRuleRequest>) => Promise<UpdateSecurityGroupRuleResponse>;
    protected pageOfListPlacementGroups: (request?: Readonly<ListPlacementGroupsRequest>) => Promise<ListPlacementGroupsResponse>;
    /**
     * List placement groups. List all placement groups in a specified
     * Availability Zone.
     *
     * @param request - The request {@link ListPlacementGroupsRequest}
     * @returns A Promise of ListPlacementGroupsResponse
     */
    listPlacementGroups: (request?: Readonly<ListPlacementGroupsRequest>) => Promise<ListPlacementGroupsResponse> & {
        all: () => Promise<PlacementGroup[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<PlacementGroup[], void, void>;
    };
    /**
     * Create a placement group. Create a new placement group in a specified
     * Availability Zone.
     *
     * @param request - The request {@link CreatePlacementGroupRequest}
     * @returns A Promise of CreatePlacementGroupResponse
     */
    createPlacementGroup: (request?: Readonly<CreatePlacementGroupRequest>) => Promise<CreatePlacementGroupResponse>;
    /**
     * Get a placement group. Get the specified placement group.
     *
     * @param request - The request {@link GetPlacementGroupRequest}
     * @returns A Promise of GetPlacementGroupResponse
     */
    getPlacementGroup: (request: Readonly<GetPlacementGroupRequest>) => Promise<GetPlacementGroupResponse>;
    /**
     * Set placement group. Set all parameters of the specified placement group.
     *
     * @param request - The request {@link SetPlacementGroupRequest}
     * @returns A Promise of SetPlacementGroupResponse
     */
    setPlacementGroup: (request: Readonly<SetPlacementGroupRequest>) => Promise<SetPlacementGroupResponse>;
    /**
     * Update a placement group. Update one or more parameter of the specified
     * placement group.
     *
     * @param request - The request {@link UpdatePlacementGroupRequest}
     * @returns A Promise of UpdatePlacementGroupResponse
     */
    updatePlacementGroup: (request: Readonly<UpdatePlacementGroupRequest>) => Promise<UpdatePlacementGroupResponse>;
    /**
     * Delete the specified placement group.
     *
     * @param request - The request {@link DeletePlacementGroupRequest}
     */
    deletePlacementGroup: (request: Readonly<DeletePlacementGroupRequest>) => Promise<void>;
    /**
     * Get placement group servers. Get all Instances belonging to the specified
     * placement group.
     *
     * @param request - The request {@link GetPlacementGroupServersRequest}
     * @returns A Promise of GetPlacementGroupServersResponse
     */
    getPlacementGroupServers: (request: Readonly<GetPlacementGroupServersRequest>) => Promise<GetPlacementGroupServersResponse>;
    /**
     * Set placement group servers. Set all Instances belonging to the specified
     * placement group.
     *
     * @param request - The request {@link SetPlacementGroupServersRequest}
     * @returns A Promise of SetPlacementGroupServersResponse
     */
    setPlacementGroupServers: (request: Readonly<SetPlacementGroupServersRequest>) => Promise<SetPlacementGroupServersResponse>;
    /**
     * Update placement group servers. Update all Instances belonging to the
     * specified placement group.
     *
     * @param request - The request {@link UpdatePlacementGroupServersRequest}
     * @returns A Promise of UpdatePlacementGroupServersResponse
     */
    updatePlacementGroupServers: (request: Readonly<UpdatePlacementGroupServersRequest>) => Promise<UpdatePlacementGroupServersResponse>;
    protected pageOfListIps: (request?: Readonly<ListIpsRequest>) => Promise<ListIpsResponse$1>;
    /**
     * List all flexible IPs. List all flexible IPs in a specified zone.
     *
     * @param request - The request {@link ListIpsRequest}
     * @returns A Promise of ListIpsResponse
     */
    listIps: (request?: Readonly<ListIpsRequest>) => Promise<ListIpsResponse$1> & {
        all: () => Promise<Ip$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Ip$1[], void, void>;
    };
    /**
     * Reserve a flexible IP. Reserve a flexible IP and attach it to the specified
     * Instance.
     *
     * @param request - The request {@link CreateIpRequest}
     * @returns A Promise of CreateIpResponse
     */
    createIp: (request?: Readonly<CreateIpRequest$1>) => Promise<CreateIpResponse>;
    /**
     * Get a flexible IP. Get details of an IP with the specified ID or address.
     *
     * @param request - The request {@link GetIpRequest}
     * @returns A Promise of GetIpResponse
     */
    getIp: (request: Readonly<GetIpRequest$1>) => Promise<GetIpResponse>;
    /**
     * Update a flexible IP. Update a flexible IP in the specified zone with the
     * specified ID.
     *
     * @param request - The request {@link UpdateIpRequest}
     * @returns A Promise of UpdateIpResponse
     */
    updateIp: (request: Readonly<UpdateIpRequest$1>) => Promise<UpdateIpResponse>;
    /**
     * Delete a flexible IP. Delete the IP with the specified ID.
     *
     * @param request - The request {@link DeleteIpRequest}
     */
    deleteIp: (request: Readonly<DeleteIpRequest>) => Promise<void>;
    protected pageOfListPrivateNICs: (request: Readonly<ListPrivateNICsRequest>) => Promise<ListPrivateNICsResponse>;
    /**
     * List all private NICs. List all private NICs of a specified Instance.
     *
     * @param request - The request {@link ListPrivateNICsRequest}
     * @returns A Promise of ListPrivateNICsResponse
     */
    listPrivateNICs: (request: Readonly<ListPrivateNICsRequest>) => Promise<ListPrivateNICsResponse> & {
        all: () => Promise<PrivateNIC[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<PrivateNIC[], void, void>;
    };
    /**
     * Create a private NIC connecting an Instance to a Private Network.
     *
     * @param request - The request {@link CreatePrivateNICRequest}
     * @returns A Promise of CreatePrivateNICResponse
     */
    createPrivateNIC: (request: Readonly<CreatePrivateNICRequest>) => Promise<CreatePrivateNICResponse>;
    /**
     * Get a private NIC. Get private NIC properties.
     *
     * @param request - The request {@link GetPrivateNICRequest}
     * @returns A Promise of GetPrivateNICResponse
     */
    getPrivateNIC: (request: Readonly<GetPrivateNICRequest>) => Promise<GetPrivateNICResponse>;
    /**
     * Update a private NIC. Update one or more parameter(s) of a specified
     * private NIC.
     *
     * @param request - The request {@link UpdatePrivateNICRequest}
     * @returns A Promise of PrivateNIC
     */
    updatePrivateNIC: (request: Readonly<UpdatePrivateNICRequest>) => Promise<PrivateNIC>;
    /**
     * Delete a private NIC.
     *
     * @param request - The request {@link DeletePrivateNICRequest}
     */
    deletePrivateNIC: (request: Readonly<DeletePrivateNICRequest>) => Promise<void>;
    protected pageOfListBootscripts: (request?: Readonly<ListBootscriptsRequest>) => Promise<ListBootscriptsResponse>;
    /**
     * List bootscripts.
     *
     * @deprecated
     * @param request - The request {@link ListBootscriptsRequest}
     * @returns A Promise of ListBootscriptsResponse
     */
    listBootscripts: (request?: Readonly<ListBootscriptsRequest>) => Promise<ListBootscriptsResponse> & {
        all: () => Promise<Bootscript[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Bootscript[], void, void>;
    };
    /**
     * Get bootscripts. Get details of a bootscript with the specified ID.
     *
     * @deprecated
     * @param request - The request {@link GetBootscriptRequest}
     * @returns A Promise of GetBootscriptResponse
     */
    getBootscript: (request: Readonly<GetBootscriptRequest>) => Promise<GetBootscriptResponse>;
    getDashboard: (request?: Readonly<GetDashboardRequest>) => Promise<GetDashboardResponse>;
    /**
     * Get a volume or snapshot's migration plan. Given a volume or snapshot,
     * returns the migration plan for a call to the RPC ApplyBlockMigration. This
     * plan will include zero or one volume, and zero or more snapshots, which
     * will need to be migrated together. This RPC does not perform the actual
     * migration itself, ApplyBlockMigration must be used. The validation_key
     * value returned by this call must be provided to the ApplyBlockMigration
     * call to confirm that all resources listed in the plan should be migrated.
     *
     * @param request - The request {@link PlanBlockMigrationRequest}
     * @returns A Promise of MigrationPlan
     */
    planBlockMigration: (request?: Readonly<PlanBlockMigrationRequest>) => Promise<MigrationPlan>;
    /**
     * Migrate a volume and/or snapshots to SBS (Scaleway Block Storage). To be
     * used, this RPC must be preceded by a call to PlanBlockMigration. To migrate
     * all resources mentioned in the MigrationPlan, the validation_key returned
     * in the MigrationPlan must be provided.
     *
     * @param request - The request {@link ApplyBlockMigrationRequest}
     */
    applyBlockMigration: (request: Readonly<ApplyBlockMigrationRequest>) => Promise<void>;
}

type AttachVolumeRequest = {
    zone?: Zone;
    serverId: string;
    volumeId: string;
};
/** Contains the updated server after attaching a volume. */
interface AttachVolumeResponse {
    server?: Server;
}
type DetachVolumeRequest = {
    zone?: Zone;
    volumeId: string;
};
/** Contains the updated server after detaching a volume. */
interface DetachVolumeResponse {
    server?: Server;
}
type GetServerUserDataRequest = {
    /** Zone to target. If none is passed will use default zone from the config */
    zone?: Zone;
    serverId: string;
    /** The user data key to get */
    key: string;
};
type SetServerUserDataRequest = {
    /** Zone to target. If none is passed will use default zone from the config */
    zone?: Zone;
    serverId: string;
    /** The user data key to set */
    key: string;
    /** The data encoded in base64 */
    content: string;
};

declare class InstanceV1UtilsAPI extends API$j {
    /**
     * Waits for {@link Image} to be in a final state.
     *
     * @param request - The request {@link GetImageRequest}
     * @param options - The waiting options
     * @returns A Promise of Image
     */
    waitForImage: (request: Readonly<GetImageRequest$3>, options?: Readonly<WaitForOptions<Image$3>>) => Promise<Image$3>;
    /**
     * Waits for {@link PrivateNIC} to be in a final state.
     *
     * @param request - The request {@link GetPrivateNICRequest}
     * @param options - The waiting options
     * @returns A Promise of PrivateNIC
     */
    waitForPrivateNIC: (request: Readonly<GetPrivateNICRequest>, options?: Readonly<WaitForOptions<PrivateNIC>>) => Promise<PrivateNIC>;
    /**
     * Waits for {@link Server} to be in a final state.
     *
     * @param request - The request {@link GetServerRequest}
     * @param options - The waiting options
     * @returns A Promise of Server
     */
    waitForServer: (request: Readonly<GetServerRequest>, options?: Readonly<WaitForOptions<Server>>) => Promise<Server>;
    /**
     * Waits for {@link Server} to be in a final state.
     *
     * @param request - The request {@link GetServerRequest}
     * @param options - The waiting options
     * @returns A Promise of Server
     */
    waitForSnapshot: (request: Readonly<GetSnapshotRequest$1>, options?: Readonly<WaitForOptions<Snapshot$1>>) => Promise<Snapshot$1>;
    /**
     * Waits for {@link Volume} to be in a final state.
     *
     * @param request - The request {@link GetVolumeRequest}
     * @param options - The waiting options
     * @returns A Promise of Volume
     */
    waitForVolume: (request: Readonly<GetVolumeRequest$1>, options?: Readonly<WaitForOptions<Volume$2>>) => Promise<Volume$2>;
    /**
     * Updates a server.
     *
     * @param request - The request {@link UpdateServerRequest}
     * @returns A Promise of UpdateServerResponse
     */
    updateServer: (request: UpdateServerRequest) => Promise<UpdateServerResponse>;
    /**
     * Creates a server.
     *
     * @param request - The request {@link CreateServerRequest}
     * @returns A Promise of CreateServerResponse
     */
    createServer: (request: Readonly<CreateServerRequest>) => Promise<CreateServerResponse>;
    /**
     * Starts an action and wait for the server to be in the correct "terminal
     * state" expected by this action.
     *
     * @param request - The request {@link ServerActionRequest}
     * @returns A Promise of Server
     */
    serverActionAndWait: (request: ServerActionRequest, options?: Readonly<WaitForOptions<Server>>) => Promise<Server>;
    /**
     * Attaches a volume to a server.
     *
     * @param request - The request {@link AttachVolumeRequest}
     * @returns A Promise of AttachVolumeResponse
     */
    attachVolume: (request: Readonly<AttachVolumeRequest>) => Promise<AttachVolumeResponse>;
    /**
     * Detaches a volume from a server.
     *
     * @param request - The request {@link DetachVolumeRequest}
     * @returns A Promise of DetachVolumeResponse
     */
    detachVolume: (request: DetachVolumeRequest) => Promise<DetachVolumeResponse>;
    /**
     * Get the content of a user data on a server for the given key.
     *
     * @param request - The request {@link GetServerUserDataRequest}
     * @returns The content of the key
     */
    getServerUserData: (request: Readonly<GetServerUserDataRequest>) => Promise<string>;
    /**
     * Sets the content of a user data on a server for the given key.
     *
     * @param request - The request {@link SetServerUserDataRequest}
     */
    setServerUserData: (request: Readonly<SetServerUserDataRequest>) => Promise<void>;
}

/** Lists transient statutes of the enum {@link ImageState}. */
declare const IMAGE_TRANSIENT_STATUSES$1: ImageState[];
/** Lists transient statutes of the enum {@link IpState}. */
declare const IP_TRANSIENT_STATUSES: IpState[];
/** Lists transient statutes of the enum {@link PrivateNICState}. */
declare const PRIVATE_NIC_TRANSIENT_STATUSES: PrivateNICState[];
/** Lists transient statutes of the enum {@link SecurityGroupState}. */
declare const SECURITY_GROUP_TRANSIENT_STATUSES: SecurityGroupState[];
/** Lists transient statutes of the enum {@link ServerIpState}. */
declare const SERVER_IP_TRANSIENT_STATUSES: ServerIpState[];
/** Lists transient statutes of the enum {@link ServerState}. */
declare const SERVER_TRANSIENT_STATUSES: ServerState[];
/** Lists transient statutes of the enum {@link SnapshotState}. */
declare const SNAPSHOT_TRANSIENT_STATUSES$1: SnapshotState[];
/** Lists transient statutes of the enum {@link TaskStatus}. */
declare const TASK_TRANSIENT_STATUSES: TaskStatus[];
/** Lists transient statutes of the enum {@link VolumeServerState}. */
declare const VOLUME_SERVER_TRANSIENT_STATUSES: VolumeServerState[];
/** Lists transient statutes of the enum {@link VolumeState}. */
declare const VOLUME_TRANSIENT_STATUSES: VolumeState[];

type index$l_ApplyBlockMigrationRequest = ApplyBlockMigrationRequest;
type index$l_Arch = Arch;
type index$l_AttachServerVolumeRequest = AttachServerVolumeRequest;
type index$l_AttachServerVolumeRequestVolumeType = AttachServerVolumeRequestVolumeType;
type index$l_AttachServerVolumeResponse = AttachServerVolumeResponse;
type index$l_AttachVolumeRequest = AttachVolumeRequest;
type index$l_AttachVolumeResponse = AttachVolumeResponse;
type index$l_BootType = BootType;
type index$l_Bootscript = Bootscript;
type index$l_CreateImageRequest = CreateImageRequest;
type index$l_CreateImageResponse = CreateImageResponse;
type index$l_CreateIpResponse = CreateIpResponse;
type index$l_CreatePlacementGroupRequest = CreatePlacementGroupRequest;
type index$l_CreatePlacementGroupResponse = CreatePlacementGroupResponse;
type index$l_CreatePrivateNICRequest = CreatePrivateNICRequest;
type index$l_CreatePrivateNICResponse = CreatePrivateNICResponse;
type index$l_CreateSecurityGroupRequest = CreateSecurityGroupRequest;
type index$l_CreateSecurityGroupResponse = CreateSecurityGroupResponse;
type index$l_CreateSecurityGroupRuleRequest = CreateSecurityGroupRuleRequest;
type index$l_CreateSecurityGroupRuleResponse = CreateSecurityGroupRuleResponse;
type index$l_CreateServerRequest = CreateServerRequest;
type index$l_CreateServerResponse = CreateServerResponse;
type index$l_CreateSnapshotResponse = CreateSnapshotResponse;
type index$l_CreateVolumeResponse = CreateVolumeResponse;
type index$l_Dashboard = Dashboard;
type index$l_DeleteIpRequest = DeleteIpRequest;
type index$l_DeletePlacementGroupRequest = DeletePlacementGroupRequest;
type index$l_DeletePrivateNICRequest = DeletePrivateNICRequest;
type index$l_DeleteSecurityGroupRequest = DeleteSecurityGroupRequest;
type index$l_DeleteSecurityGroupRuleRequest = DeleteSecurityGroupRuleRequest;
type index$l_DeleteServerRequest = DeleteServerRequest;
type index$l_DeleteServerUserDataRequest = DeleteServerUserDataRequest;
type index$l_DetachServerVolumeRequest = DetachServerVolumeRequest;
type index$l_DetachServerVolumeResponse = DetachServerVolumeResponse;
type index$l_DetachVolumeRequest = DetachVolumeRequest;
type index$l_DetachVolumeResponse = DetachVolumeResponse;
type index$l_ExportSnapshotRequest = ExportSnapshotRequest;
type index$l_ExportSnapshotResponse = ExportSnapshotResponse;
type index$l_GetBootscriptRequest = GetBootscriptRequest;
type index$l_GetBootscriptResponse = GetBootscriptResponse;
type index$l_GetDashboardRequest = GetDashboardRequest;
type index$l_GetDashboardResponse = GetDashboardResponse;
type index$l_GetIpResponse = GetIpResponse;
type index$l_GetPlacementGroupRequest = GetPlacementGroupRequest;
type index$l_GetPlacementGroupResponse = GetPlacementGroupResponse;
type index$l_GetPlacementGroupServersRequest = GetPlacementGroupServersRequest;
type index$l_GetPlacementGroupServersResponse = GetPlacementGroupServersResponse;
type index$l_GetPrivateNICRequest = GetPrivateNICRequest;
type index$l_GetPrivateNICResponse = GetPrivateNICResponse;
type index$l_GetSecurityGroupRequest = GetSecurityGroupRequest;
type index$l_GetSecurityGroupResponse = GetSecurityGroupResponse;
type index$l_GetSecurityGroupRuleRequest = GetSecurityGroupRuleRequest;
type index$l_GetSecurityGroupRuleResponse = GetSecurityGroupRuleResponse;
type index$l_GetServerRequest = GetServerRequest;
type index$l_GetServerResponse = GetServerResponse;
type index$l_GetServerTypesAvailabilityRequest = GetServerTypesAvailabilityRequest;
type index$l_GetServerTypesAvailabilityResponse = GetServerTypesAvailabilityResponse;
type index$l_GetServerTypesAvailabilityResponseAvailability = GetServerTypesAvailabilityResponseAvailability;
type index$l_GetServerUserDataRequest = GetServerUserDataRequest;
type index$l_GetSnapshotResponse = GetSnapshotResponse;
type index$l_GetVolumeResponse = GetVolumeResponse;
declare const index$l_IP_TRANSIENT_STATUSES: typeof IP_TRANSIENT_STATUSES;
type index$l_ImageState = ImageState;
type index$l_IpState = IpState;
type index$l_IpType = IpType;
type index$l_ListBootscriptsRequest = ListBootscriptsRequest;
type index$l_ListBootscriptsResponse = ListBootscriptsResponse;
type index$l_ListDefaultSecurityGroupRulesRequest = ListDefaultSecurityGroupRulesRequest;
type index$l_ListIpsRequest = ListIpsRequest;
type index$l_ListPlacementGroupsRequest = ListPlacementGroupsRequest;
type index$l_ListPlacementGroupsResponse = ListPlacementGroupsResponse;
type index$l_ListPrivateNICsRequest = ListPrivateNICsRequest;
type index$l_ListPrivateNICsResponse = ListPrivateNICsResponse;
type index$l_ListSecurityGroupRulesRequest = ListSecurityGroupRulesRequest;
type index$l_ListSecurityGroupRulesResponse = ListSecurityGroupRulesResponse;
type index$l_ListSecurityGroupsRequest = ListSecurityGroupsRequest;
type index$l_ListSecurityGroupsResponse = ListSecurityGroupsResponse;
type index$l_ListServerActionsRequest = ListServerActionsRequest;
type index$l_ListServerActionsResponse = ListServerActionsResponse;
type index$l_ListServerUserDataRequest = ListServerUserDataRequest;
type index$l_ListServerUserDataResponse = ListServerUserDataResponse;
type index$l_ListServersRequest = ListServersRequest;
type index$l_ListServersRequestOrder = ListServersRequestOrder;
type index$l_ListServersResponse = ListServersResponse;
type index$l_ListServersTypesRequest = ListServersTypesRequest;
type index$l_ListServersTypesResponse = ListServersTypesResponse;
type index$l_ListVolumesTypesRequest = ListVolumesTypesRequest;
type index$l_ListVolumesTypesResponse = ListVolumesTypesResponse;
type index$l_MigrationPlan = MigrationPlan;
declare const index$l_PRIVATE_NIC_TRANSIENT_STATUSES: typeof PRIVATE_NIC_TRANSIENT_STATUSES;
type index$l_PlacementGroup = PlacementGroup;
type index$l_PlacementGroupPolicyMode = PlacementGroupPolicyMode;
type index$l_PlacementGroupPolicyType = PlacementGroupPolicyType;
type index$l_PlacementGroupServer = PlacementGroupServer;
type index$l_PlanBlockMigrationRequest = PlanBlockMigrationRequest;
type index$l_PrivateNIC = PrivateNIC;
type index$l_PrivateNICState = PrivateNICState;
declare const index$l_SECURITY_GROUP_TRANSIENT_STATUSES: typeof SECURITY_GROUP_TRANSIENT_STATUSES;
declare const index$l_SERVER_IP_TRANSIENT_STATUSES: typeof SERVER_IP_TRANSIENT_STATUSES;
declare const index$l_SERVER_TRANSIENT_STATUSES: typeof SERVER_TRANSIENT_STATUSES;
type index$l_SecurityGroup = SecurityGroup;
type index$l_SecurityGroupPolicy = SecurityGroupPolicy;
type index$l_SecurityGroupRule = SecurityGroupRule;
type index$l_SecurityGroupRuleAction = SecurityGroupRuleAction;
type index$l_SecurityGroupRuleDirection = SecurityGroupRuleDirection;
type index$l_SecurityGroupRuleProtocol = SecurityGroupRuleProtocol;
type index$l_SecurityGroupState = SecurityGroupState;
type index$l_SecurityGroupSummary = SecurityGroupSummary;
type index$l_SecurityGroupTemplate = SecurityGroupTemplate;
type index$l_Server = Server;
type index$l_ServerAction = ServerAction;
type index$l_ServerActionRequest = ServerActionRequest;
type index$l_ServerActionRequestVolumeBackupTemplate = ServerActionRequestVolumeBackupTemplate;
type index$l_ServerActionResponse = ServerActionResponse;
type index$l_ServerIp = ServerIp;
type index$l_ServerIpIpFamily = ServerIpIpFamily;
type index$l_ServerIpProvisioningMode = ServerIpProvisioningMode;
type index$l_ServerIpState = ServerIpState;
type index$l_ServerIpv6 = ServerIpv6;
type index$l_ServerLocation = ServerLocation;
type index$l_ServerMaintenance = ServerMaintenance;
type index$l_ServerState = ServerState;
type index$l_ServerSummary = ServerSummary;
type index$l_ServerType = ServerType;
type index$l_ServerTypeCapabilities = ServerTypeCapabilities;
type index$l_ServerTypeNetwork = ServerTypeNetwork;
type index$l_ServerTypeNetworkInterface = ServerTypeNetworkInterface;
type index$l_ServerTypeVolumeConstraintSizes = ServerTypeVolumeConstraintSizes;
type index$l_ServerTypeVolumeConstraintsByType = ServerTypeVolumeConstraintsByType;
type index$l_ServerTypesAvailability = ServerTypesAvailability;
type index$l_SetImageRequest = SetImageRequest;
type index$l_SetPlacementGroupRequest = SetPlacementGroupRequest;
type index$l_SetPlacementGroupResponse = SetPlacementGroupResponse;
type index$l_SetPlacementGroupServersRequest = SetPlacementGroupServersRequest;
type index$l_SetPlacementGroupServersResponse = SetPlacementGroupServersResponse;
type index$l_SetSecurityGroupRulesRequest = SetSecurityGroupRulesRequest;
type index$l_SetSecurityGroupRulesRequestRule = SetSecurityGroupRulesRequestRule;
type index$l_SetSecurityGroupRulesResponse = SetSecurityGroupRulesResponse;
type index$l_SetServerUserDataRequest = SetServerUserDataRequest;
type index$l_SnapshotBaseVolume = SnapshotBaseVolume;
type index$l_SnapshotState = SnapshotState;
declare const index$l_TASK_TRANSIENT_STATUSES: typeof TASK_TRANSIENT_STATUSES;
type index$l_Task = Task;
type index$l_TaskStatus = TaskStatus;
type index$l_UpdateImageResponse = UpdateImageResponse;
type index$l_UpdateIpResponse = UpdateIpResponse;
type index$l_UpdatePlacementGroupRequest = UpdatePlacementGroupRequest;
type index$l_UpdatePlacementGroupResponse = UpdatePlacementGroupResponse;
type index$l_UpdatePlacementGroupServersRequest = UpdatePlacementGroupServersRequest;
type index$l_UpdatePlacementGroupServersResponse = UpdatePlacementGroupServersResponse;
type index$l_UpdatePrivateNICRequest = UpdatePrivateNICRequest;
type index$l_UpdateSecurityGroupRequest = UpdateSecurityGroupRequest;
type index$l_UpdateSecurityGroupResponse = UpdateSecurityGroupResponse;
type index$l_UpdateSecurityGroupRuleRequest = UpdateSecurityGroupRuleRequest;
type index$l_UpdateSecurityGroupRuleResponse = UpdateSecurityGroupRuleResponse;
type index$l_UpdateServerRequest = UpdateServerRequest;
type index$l_UpdateServerResponse = UpdateServerResponse;
type index$l_UpdateSnapshotResponse = UpdateSnapshotResponse;
type index$l_UpdateVolumeResponse = UpdateVolumeResponse;
declare const index$l_VOLUME_SERVER_TRANSIENT_STATUSES: typeof VOLUME_SERVER_TRANSIENT_STATUSES;
declare const index$l_VOLUME_TRANSIENT_STATUSES: typeof VOLUME_TRANSIENT_STATUSES;
type index$l_VolumeImageUpdateTemplate = VolumeImageUpdateTemplate;
type index$l_VolumeServer = VolumeServer;
type index$l_VolumeServerState = VolumeServerState;
type index$l_VolumeServerTemplate = VolumeServerTemplate;
type index$l_VolumeServerVolumeType = VolumeServerVolumeType;
type index$l_VolumeState = VolumeState;
type index$l_VolumeSummary = VolumeSummary;
type index$l_VolumeTemplate = VolumeTemplate;
type index$l_VolumeTypeCapabilities = VolumeTypeCapabilities;
type index$l_VolumeTypeConstraints = VolumeTypeConstraints;
type index$l_VolumeVolumeType = VolumeVolumeType;
declare namespace index$l {
  export { InstanceV1UtilsAPI as API, type index$l_ApplyBlockMigrationRequest as ApplyBlockMigrationRequest, type index$l_Arch as Arch, type index$l_AttachServerVolumeRequest as AttachServerVolumeRequest, type index$l_AttachServerVolumeRequestVolumeType as AttachServerVolumeRequestVolumeType, type index$l_AttachServerVolumeResponse as AttachServerVolumeResponse, type index$l_AttachVolumeRequest as AttachVolumeRequest, type index$l_AttachVolumeResponse as AttachVolumeResponse, type index$l_BootType as BootType, type index$l_Bootscript as Bootscript, type index$l_CreateImageRequest as CreateImageRequest, type index$l_CreateImageResponse as CreateImageResponse, type CreateIpRequest$1 as CreateIpRequest, type index$l_CreateIpResponse as CreateIpResponse, type index$l_CreatePlacementGroupRequest as CreatePlacementGroupRequest, type index$l_CreatePlacementGroupResponse as CreatePlacementGroupResponse, type index$l_CreatePrivateNICRequest as CreatePrivateNICRequest, type index$l_CreatePrivateNICResponse as CreatePrivateNICResponse, type index$l_CreateSecurityGroupRequest as CreateSecurityGroupRequest, type index$l_CreateSecurityGroupResponse as CreateSecurityGroupResponse, type index$l_CreateSecurityGroupRuleRequest as CreateSecurityGroupRuleRequest, type index$l_CreateSecurityGroupRuleResponse as CreateSecurityGroupRuleResponse, type index$l_CreateServerRequest as CreateServerRequest, type index$l_CreateServerResponse as CreateServerResponse, type CreateSnapshotRequest$1 as CreateSnapshotRequest, type index$l_CreateSnapshotResponse as CreateSnapshotResponse, type CreateVolumeRequest$1 as CreateVolumeRequest, type index$l_CreateVolumeResponse as CreateVolumeResponse, type index$l_Dashboard as Dashboard, type DeleteImageRequest$1 as DeleteImageRequest, type index$l_DeleteIpRequest as DeleteIpRequest, type index$l_DeletePlacementGroupRequest as DeletePlacementGroupRequest, type index$l_DeletePrivateNICRequest as DeletePrivateNICRequest, type index$l_DeleteSecurityGroupRequest as DeleteSecurityGroupRequest, type index$l_DeleteSecurityGroupRuleRequest as DeleteSecurityGroupRuleRequest, type index$l_DeleteServerRequest as DeleteServerRequest, type index$l_DeleteServerUserDataRequest as DeleteServerUserDataRequest, type DeleteSnapshotRequest$1 as DeleteSnapshotRequest, type DeleteVolumeRequest$1 as DeleteVolumeRequest, type index$l_DetachServerVolumeRequest as DetachServerVolumeRequest, type index$l_DetachServerVolumeResponse as DetachServerVolumeResponse, type index$l_DetachVolumeRequest as DetachVolumeRequest, type index$l_DetachVolumeResponse as DetachVolumeResponse, type index$l_ExportSnapshotRequest as ExportSnapshotRequest, type index$l_ExportSnapshotResponse as ExportSnapshotResponse, type index$l_GetBootscriptRequest as GetBootscriptRequest, type index$l_GetBootscriptResponse as GetBootscriptResponse, type index$l_GetDashboardRequest as GetDashboardRequest, type index$l_GetDashboardResponse as GetDashboardResponse, type GetImageRequest$3 as GetImageRequest, type GetImageResponse$1 as GetImageResponse, type GetIpRequest$1 as GetIpRequest, type index$l_GetIpResponse as GetIpResponse, type index$l_GetPlacementGroupRequest as GetPlacementGroupRequest, type index$l_GetPlacementGroupResponse as GetPlacementGroupResponse, type index$l_GetPlacementGroupServersRequest as GetPlacementGroupServersRequest, type index$l_GetPlacementGroupServersResponse as GetPlacementGroupServersResponse, type index$l_GetPrivateNICRequest as GetPrivateNICRequest, type index$l_GetPrivateNICResponse as GetPrivateNICResponse, type index$l_GetSecurityGroupRequest as GetSecurityGroupRequest, type index$l_GetSecurityGroupResponse as GetSecurityGroupResponse, type index$l_GetSecurityGroupRuleRequest as GetSecurityGroupRuleRequest, type index$l_GetSecurityGroupRuleResponse as GetSecurityGroupRuleResponse, type index$l_GetServerRequest as GetServerRequest, type index$l_GetServerResponse as GetServerResponse, type index$l_GetServerTypesAvailabilityRequest as GetServerTypesAvailabilityRequest, type index$l_GetServerTypesAvailabilityResponse as GetServerTypesAvailabilityResponse, type index$l_GetServerTypesAvailabilityResponseAvailability as GetServerTypesAvailabilityResponseAvailability, type index$l_GetServerUserDataRequest as GetServerUserDataRequest, type GetSnapshotRequest$1 as GetSnapshotRequest, type index$l_GetSnapshotResponse as GetSnapshotResponse, type GetVolumeRequest$1 as GetVolumeRequest, type index$l_GetVolumeResponse as GetVolumeResponse, IMAGE_TRANSIENT_STATUSES$1 as IMAGE_TRANSIENT_STATUSES, index$l_IP_TRANSIENT_STATUSES as IP_TRANSIENT_STATUSES, type Image$3 as Image, type index$l_ImageState as ImageState, type Ip$1 as Ip, type index$l_IpState as IpState, type index$l_IpType as IpType, type index$l_ListBootscriptsRequest as ListBootscriptsRequest, type index$l_ListBootscriptsResponse as ListBootscriptsResponse, type index$l_ListDefaultSecurityGroupRulesRequest as ListDefaultSecurityGroupRulesRequest, type ListImagesRequest$3 as ListImagesRequest, type ListImagesResponse$3 as ListImagesResponse, type index$l_ListIpsRequest as ListIpsRequest, type ListIpsResponse$1 as ListIpsResponse, type index$l_ListPlacementGroupsRequest as ListPlacementGroupsRequest, type index$l_ListPlacementGroupsResponse as ListPlacementGroupsResponse, type index$l_ListPrivateNICsRequest as ListPrivateNICsRequest, type index$l_ListPrivateNICsResponse as ListPrivateNICsResponse, type index$l_ListSecurityGroupRulesRequest as ListSecurityGroupRulesRequest, type index$l_ListSecurityGroupRulesResponse as ListSecurityGroupRulesResponse, type index$l_ListSecurityGroupsRequest as ListSecurityGroupsRequest, type index$l_ListSecurityGroupsResponse as ListSecurityGroupsResponse, type index$l_ListServerActionsRequest as ListServerActionsRequest, type index$l_ListServerActionsResponse as ListServerActionsResponse, type index$l_ListServerUserDataRequest as ListServerUserDataRequest, type index$l_ListServerUserDataResponse as ListServerUserDataResponse, type index$l_ListServersRequest as ListServersRequest, type index$l_ListServersRequestOrder as ListServersRequestOrder, type index$l_ListServersResponse as ListServersResponse, type index$l_ListServersTypesRequest as ListServersTypesRequest, type index$l_ListServersTypesResponse as ListServersTypesResponse, type ListSnapshotsRequest$1 as ListSnapshotsRequest, type ListSnapshotsResponse$1 as ListSnapshotsResponse, type ListVolumesRequest$1 as ListVolumesRequest, type ListVolumesResponse$1 as ListVolumesResponse, type index$l_ListVolumesTypesRequest as ListVolumesTypesRequest, type index$l_ListVolumesTypesResponse as ListVolumesTypesResponse, type index$l_MigrationPlan as MigrationPlan, index$l_PRIVATE_NIC_TRANSIENT_STATUSES as PRIVATE_NIC_TRANSIENT_STATUSES, type index$l_PlacementGroup as PlacementGroup, type index$l_PlacementGroupPolicyMode as PlacementGroupPolicyMode, type index$l_PlacementGroupPolicyType as PlacementGroupPolicyType, type index$l_PlacementGroupServer as PlacementGroupServer, type index$l_PlanBlockMigrationRequest as PlanBlockMigrationRequest, type index$l_PrivateNIC as PrivateNIC, type index$l_PrivateNICState as PrivateNICState, index$l_SECURITY_GROUP_TRANSIENT_STATUSES as SECURITY_GROUP_TRANSIENT_STATUSES, index$l_SERVER_IP_TRANSIENT_STATUSES as SERVER_IP_TRANSIENT_STATUSES, index$l_SERVER_TRANSIENT_STATUSES as SERVER_TRANSIENT_STATUSES, SNAPSHOT_TRANSIENT_STATUSES$1 as SNAPSHOT_TRANSIENT_STATUSES, type index$l_SecurityGroup as SecurityGroup, type index$l_SecurityGroupPolicy as SecurityGroupPolicy, type index$l_SecurityGroupRule as SecurityGroupRule, type index$l_SecurityGroupRuleAction as SecurityGroupRuleAction, type index$l_SecurityGroupRuleDirection as SecurityGroupRuleDirection, type index$l_SecurityGroupRuleProtocol as SecurityGroupRuleProtocol, type index$l_SecurityGroupState as SecurityGroupState, type index$l_SecurityGroupSummary as SecurityGroupSummary, type index$l_SecurityGroupTemplate as SecurityGroupTemplate, type index$l_Server as Server, type index$l_ServerAction as ServerAction, type index$l_ServerActionRequest as ServerActionRequest, type index$l_ServerActionRequestVolumeBackupTemplate as ServerActionRequestVolumeBackupTemplate, type index$l_ServerActionResponse as ServerActionResponse, type index$l_ServerIp as ServerIp, type index$l_ServerIpIpFamily as ServerIpIpFamily, type index$l_ServerIpProvisioningMode as ServerIpProvisioningMode, type index$l_ServerIpState as ServerIpState, type index$l_ServerIpv6 as ServerIpv6, type index$l_ServerLocation as ServerLocation, type index$l_ServerMaintenance as ServerMaintenance, type index$l_ServerState as ServerState, type index$l_ServerSummary as ServerSummary, type index$l_ServerType as ServerType, type index$l_ServerTypeCapabilities as ServerTypeCapabilities, type index$l_ServerTypeNetwork as ServerTypeNetwork, type index$l_ServerTypeNetworkInterface as ServerTypeNetworkInterface, type index$l_ServerTypeVolumeConstraintSizes as ServerTypeVolumeConstraintSizes, type index$l_ServerTypeVolumeConstraintsByType as ServerTypeVolumeConstraintsByType, type index$l_ServerTypesAvailability as ServerTypesAvailability, type index$l_SetImageRequest as SetImageRequest, type index$l_SetPlacementGroupRequest as SetPlacementGroupRequest, type index$l_SetPlacementGroupResponse as SetPlacementGroupResponse, type index$l_SetPlacementGroupServersRequest as SetPlacementGroupServersRequest, type index$l_SetPlacementGroupServersResponse as SetPlacementGroupServersResponse, type index$l_SetSecurityGroupRulesRequest as SetSecurityGroupRulesRequest, type index$l_SetSecurityGroupRulesRequestRule as SetSecurityGroupRulesRequestRule, type index$l_SetSecurityGroupRulesResponse as SetSecurityGroupRulesResponse, type index$l_SetServerUserDataRequest as SetServerUserDataRequest, type Snapshot$1 as Snapshot, type index$l_SnapshotBaseVolume as SnapshotBaseVolume, type index$l_SnapshotState as SnapshotState, type SnapshotVolumeType$1 as SnapshotVolumeType, index$l_TASK_TRANSIENT_STATUSES as TASK_TRANSIENT_STATUSES, type index$l_Task as Task, type index$l_TaskStatus as TaskStatus, type UpdateImageRequest$1 as UpdateImageRequest, type index$l_UpdateImageResponse as UpdateImageResponse, type UpdateIpRequest$1 as UpdateIpRequest, type index$l_UpdateIpResponse as UpdateIpResponse, type index$l_UpdatePlacementGroupRequest as UpdatePlacementGroupRequest, type index$l_UpdatePlacementGroupResponse as UpdatePlacementGroupResponse, type index$l_UpdatePlacementGroupServersRequest as UpdatePlacementGroupServersRequest, type index$l_UpdatePlacementGroupServersResponse as UpdatePlacementGroupServersResponse, type index$l_UpdatePrivateNICRequest as UpdatePrivateNICRequest, type index$l_UpdateSecurityGroupRequest as UpdateSecurityGroupRequest, type index$l_UpdateSecurityGroupResponse as UpdateSecurityGroupResponse, type index$l_UpdateSecurityGroupRuleRequest as UpdateSecurityGroupRuleRequest, type index$l_UpdateSecurityGroupRuleResponse as UpdateSecurityGroupRuleResponse, type index$l_UpdateServerRequest as UpdateServerRequest, type index$l_UpdateServerResponse as UpdateServerResponse, type UpdateSnapshotRequest$1 as UpdateSnapshotRequest, type index$l_UpdateSnapshotResponse as UpdateSnapshotResponse, type UpdateVolumeRequest$1 as UpdateVolumeRequest, type index$l_UpdateVolumeResponse as UpdateVolumeResponse, index$l_VOLUME_SERVER_TRANSIENT_STATUSES as VOLUME_SERVER_TRANSIENT_STATUSES, index$l_VOLUME_TRANSIENT_STATUSES as VOLUME_TRANSIENT_STATUSES, type Volume$2 as Volume, type index$l_VolumeImageUpdateTemplate as VolumeImageUpdateTemplate, type index$l_VolumeServer as VolumeServer, type index$l_VolumeServerState as VolumeServerState, type index$l_VolumeServerTemplate as VolumeServerTemplate, type index$l_VolumeServerVolumeType as VolumeServerVolumeType, type index$l_VolumeState as VolumeState, type index$l_VolumeSummary as VolumeSummary, type index$l_VolumeTemplate as VolumeTemplate, type VolumeType$1 as VolumeType, type index$l_VolumeTypeCapabilities as VolumeTypeCapabilities, type index$l_VolumeTypeConstraints as VolumeTypeConstraints, type index$l_VolumeVolumeType as VolumeVolumeType };
}

declare namespace index$k {
  export { index$l as v1 };
}

type DeviceMessageFiltersRulePolicy = 'unknown' | 'accept' | 'reject';
type DeviceStatus = 'unknown' | 'error' | 'enabled' | 'disabled';
type HubProductPlan = 'plan_unknown' | 'plan_shared' | 'plan_dedicated' | 'plan_ha';
type HubStatus = 'unknown' | 'error' | 'enabling' | 'ready' | 'disabling' | 'disabled';
type ListDevicesRequestOrderBy = 'name_asc' | 'name_desc' | 'status_asc' | 'status_desc' | 'hub_id_asc' | 'hub_id_desc' | 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'allow_insecure_asc' | 'allow_insecure_desc';
type ListHubsRequestOrderBy = 'name_asc' | 'name_desc' | 'status_asc' | 'status_desc' | 'product_plan_asc' | 'product_plan_desc' | 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc';
type ListNetworksRequestOrderBy = 'name_asc' | 'name_desc' | 'type_asc' | 'type_desc' | 'created_at_asc' | 'created_at_desc';
type ListRoutesRequestOrderBy$1 = 'name_asc' | 'name_desc' | 'hub_id_asc' | 'hub_id_desc' | 'type_asc' | 'type_desc' | 'created_at_asc' | 'created_at_desc';
type NetworkNetworkType = 'unknown' | 'sigfox' | 'rest';
type RouteDatabaseConfigEngine = 'unknown' | 'postgresql' | 'mysql';
type RouteRestConfigHttpVerb = 'unknown' | 'get' | 'post' | 'put' | 'patch' | 'delete';
type RouteRouteType = 'unknown' | 's3' | 'database' | 'rest';
type RouteS3ConfigS3Strategy = 'unknown' | 'per_topic' | 'per_message';
interface DeviceMessageFiltersRule {
    /**
     * If set to `accept`, all topics in the topics list will be allowed, with all
     * other topics being denied. If set to `reject`, all topics in the topics
     * list will be denied, with all other topics being allowed.
     */
    policy: DeviceMessageFiltersRulePolicy;
    /**
     * List of topics to accept or reject. It must be valid MQTT topics and up to
     * 65535 characters.
     */
    topics?: string[];
}
interface DeviceMessageFilters {
    /** Filtering rule to restrict topics the device can publish to. */
    publish?: DeviceMessageFiltersRule;
    /** Filtering rule to restrict topics the device can subscribe to. */
    subscribe?: DeviceMessageFiltersRule;
}
interface HubTwinsGraphiteConfig {
    pushUri: string;
}
interface Certificate$1 {
    crt: string;
    key: string;
}
interface Device {
    /** Device ID, also used as MQTT Client ID or username. */
    id: string;
    /** Device name. */
    name: string;
    /** Device description. */
    description: string;
    /** Device status. */
    status: DeviceStatus;
    /** Hub ID. */
    hubId: string;
    /** Last connection/activity date of a device. */
    lastActivityAt?: Date;
    /** Defines whether the device is connected to the Hub. */
    isConnected: boolean;
    /**
     * Defines whether to allow the device to connect to the Hub without TLS
     * mutual authentication.
     */
    allowInsecure: boolean;
    /**
     * Defines whether to allow multiple physical devices to connect to the Hub
     * with this device's credentials.
     */
    allowMultipleConnections: boolean;
    /** Filter-sets to restrict the topics the device can publish/subscribe to. */
    messageFilters?: DeviceMessageFilters;
    /**
     * Assigning a custom certificate allows a device to authenticate using that
     * specific certificate without checking the Hub's CA certificate.
     */
    hasCustomCertificate: boolean;
    /** Date at which the device was added. */
    createdAt?: Date;
    /** Date at which the device was last modified. */
    updatedAt?: Date;
}
interface Network {
    /** Network ID. */
    id: string;
    /** Network name. */
    name: string;
    /** Type of network to connect with. */
    type: NetworkNetworkType;
    /** Endpoint to use for interacting with the network. */
    endpoint: string;
    /** Hub ID to connect the Network to. */
    hubId: string;
    /** Date at which the network was created. */
    createdAt?: Date;
    /** This prefix will be prepended to all topics for this Network. */
    topicPrefix: string;
}
interface CreateRouteRequestDatabaseConfig {
    host: string;
    port: number;
    dbname: string;
    username: string;
    password: string;
    query: string;
    engine: RouteDatabaseConfigEngine;
}
interface CreateRouteRequestRestConfig {
    verb: RouteRestConfigHttpVerb;
    uri: string;
    headers: Record<string, string>;
}
interface CreateRouteRequestS3Config {
    bucketRegion: string;
    bucketName: string;
    objectPrefix: string;
    strategy: RouteS3ConfigS3Strategy;
}
interface Hub {
    /** Hub ID. */
    id: string;
    /** Hub name. */
    name: string;
    /** Current status of the Hub. */
    status: HubStatus;
    /** Hub feature set. */
    productPlan: HubProductPlan;
    /** Defines whether the hub has been enabled. */
    enabled: boolean;
    /** Number of registered devices. */
    deviceCount: number;
    /** Number of currently connected devices. */
    connectedDeviceCount: number;
    /**
     * Devices should be connected to this host. Port may be 1883 (MQTT), 8883
     * (MQTT over TLS), 80 (MQTT over Websocket) or 443 (MQTT over Websocket over
     * TLS).
     */
    endpoint: string;
    /** Defines whether to disable Hub events. */
    disableEvents: boolean;
    /** Hub events topic prefix. */
    eventsTopicPrefix: string;
    /** Region of the Hub. */
    region: Region;
    /** Hub creation date. */
    createdAt?: Date;
    /** Hub last modification date. */
    updatedAt?: Date;
    /** Project owning the resource. */
    projectId: string;
    /** Organization owning the resource. */
    organizationId: string;
    /**
     * When an unknown device connects to your hub using a valid certificate
     * chain, it will be automatically provisioned inside your Hub. The Hub uses
     * the common name of the device certifcate to find out if a device with the
     * same name already exists. This setting can only be enabled on a hub with a
     * custom certificate authority.
     */
    enableDeviceAutoProvisioning: boolean;
    /**
     * Flag is automatically set to `false` after Hub creation, as Hub
     * certificates are managed by Scaleway. Once a custom certificate authority
     * is set, the flag will be set to `true`.
     */
    hasCustomCa: boolean;
    /**
     * BETA - not implemented yet.
     *
     * One-of ('twinsDbConfig'): at most one of 'twinsGraphiteConfig' could be
     * set.
     */
    twinsGraphiteConfig?: HubTwinsGraphiteConfig;
}
interface RouteSummary {
    /** Route ID. */
    id: string;
    /** Route name. */
    name: string;
    /** Hub ID of the route. */
    hubId: string;
    /**
     * Topic the route subscribes to. It must be a valid MQTT topic and up to
     * 65535 characters.
     */
    topic: string;
    /** Route type. */
    type: RouteRouteType;
    /** Date at which the route was created. */
    createdAt?: Date;
    /** Date at which the route was last updated. */
    updatedAt?: Date;
}
interface ListTwinDocumentsResponseDocumentSummary {
    /** Name of the document. */
    documentName: string;
}
interface RouteDatabaseConfig {
    /**
     * Database engine the route will connect to. If not specified, the default
     * database will be 'PostgreSQL'.
     */
    engine: RouteDatabaseConfigEngine;
    /** Database host. */
    host: string;
    /** Database port. */
    port: number;
    /** Database name. */
    dbname: string;
    /** Database username. Make sure this account can execute the provided query. */
    username: string;
    /** Database password. */
    password: string;
    /**
     * SQL query to be executed ($TOPIC and $PAYLOAD variables are available, see
     * documentation).
     */
    query: string;
}
interface RouteRestConfig {
    /** HTTP verb used to call REST URI. */
    verb: RouteRestConfigHttpVerb;
    /** URI of the REST endpoint. */
    uri: string;
    /** HTTP call extra headers. */
    headers: Record<string, string>;
}
interface RouteS3Config {
    /** Region of the S3 route's destination bucket (e.g., 'fr-par'). */
    bucketRegion: string;
    /** Destination bucket name of the S3 route. */
    bucketName: string;
    /** Optional string to prefix object names with. */
    objectPrefix: string;
    /**
     * How the S3 route's objects will be created: one per topic or one per
     * message.
     */
    strategy: RouteS3ConfigS3Strategy;
}
interface UpdateRouteRequestDatabaseConfig {
    host?: string;
    port?: number;
    dbname?: string;
    username?: string;
    password?: string;
    query?: string;
    engine: RouteDatabaseConfigEngine;
}
interface UpdateRouteRequestRestConfig {
    verb: RouteRestConfigHttpVerb;
    uri?: string;
    headers?: Record<string, string>;
}
interface UpdateRouteRequestS3Config {
    bucketRegion?: string;
    bucketName?: string;
    objectPrefix?: string;
    strategy: RouteS3ConfigS3Strategy;
}
type CreateDeviceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device name. */
    name?: string;
    /** Hub ID of the device. */
    hubId: string;
    /**
     * Defines whether to allow plain and server-authenticated SSL connections in
     * addition to mutually-authenticated ones.
     */
    allowInsecure: boolean;
    /**
     * Defines whether to allow multiple physical devices to connect with this
     * device's credentials.
     */
    allowMultipleConnections: boolean;
    /**
     * Filter-sets to authorize or deny the device to publish/subscribe to
     * specific topics.
     */
    messageFilters?: DeviceMessageFilters;
    /** Device description. */
    description?: string;
};
interface CreateDeviceResponse {
    /** Information related to the created device. */
    device?: Device;
    /** Device certificate. */
    certificate?: Certificate$1;
}
type CreateHubRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hub name (up to 255 characters). */
    name?: string;
    /**
     * Project/Organization ID to filter for, only Hubs from this
     * Project/Organization will be returned.
     */
    projectId?: string;
    /** Hub product plan. */
    productPlan: HubProductPlan;
    /** Disable Hub events. */
    disableEvents?: boolean;
    /** Topic prefix (default '$SCW/events') of Hub events. */
    eventsTopicPrefix?: string;
    /**
     * BETA - not implemented yet.
     *
     * One-of ('twinsDbConfig'): at most one of 'twinsGraphiteConfig' could be
     * set.
     */
    twinsGraphiteConfig?: HubTwinsGraphiteConfig;
};
type CreateNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Network name. */
    name?: string;
    /** Type of network to connect with. */
    type: NetworkNetworkType;
    /** Hub ID to connect the Network to. */
    hubId: string;
    /** Topic prefix for the Network. */
    topicPrefix: string;
};
interface CreateNetworkResponse {
    /** Information related to the created network. */
    network?: Network;
    /** Endpoint Key to keep secret. This cannot be retrieved later. */
    secret: string;
}
type CreateRouteRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Route name. */
    name?: string;
    /** Hub ID of the route. */
    hubId: string;
    /**
     * Topic the route subscribes to. It must be a valid MQTT topic and up to
     * 65535 characters.
     */
    topic: string;
    /**
     * If creating S3 Route, S3-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    s3Config?: CreateRouteRequestS3Config;
    /**
     * If creating Database Route, DB-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    dbConfig?: CreateRouteRequestDatabaseConfig;
    /**
     * If creating Rest Route, Rest-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    restConfig?: CreateRouteRequestRestConfig;
};
type DeleteDeviceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
};
type DeleteHubRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hub ID. */
    hubId: string;
    /**
     * Defines whether to force the deletion of devices added to this Hub or
     * reject the operation.
     */
    deleteDevices?: boolean;
};
type DeleteNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Network ID. */
    networkId: string;
};
type DeleteRouteRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Route ID. */
    routeId: string;
};
type DeleteTwinDocumentRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Twin ID. */
    twinId: string;
    /** Name of the document. */
    documentName: string;
};
type DeleteTwinDocumentsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Twin ID. */
    twinId: string;
};
type DisableDeviceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
};
type DisableHubRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hub ID. */
    hubId: string;
};
type EnableDeviceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
};
type EnableHubRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hub ID. */
    hubId: string;
};
type GetDeviceCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
};
interface GetDeviceCertificateResponse {
    /** Information related to the created device. */
    device?: Device;
    /** Device certificate. */
    certificatePem: string;
}
type GetDeviceMetricsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
    /** Start date used to compute the best scale for the returned metrics. */
    startDate?: Date;
};
interface GetDeviceMetricsResponse {
    /** Metrics for a device over the requested period. */
    metrics: TimeSeries[];
}
type GetDeviceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
};
type GetHubCARequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    hubId: string;
};
interface GetHubCAResponse {
    caCertPem: string;
}
type GetHubMetricsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hub ID. */
    hubId: string;
    /** Start date used to compute the best scale for returned metrics. */
    startDate?: Date;
};
interface GetHubMetricsResponse {
    /** Metrics for a Hub over the requested period. */
    metrics: TimeSeries[];
}
type GetHubRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hub ID. */
    hubId: string;
};
type GetNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Network ID. */
    networkId: string;
};
type GetRouteRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Route ID. */
    routeId: string;
};
type GetTwinDocumentRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Twin ID. */
    twinId: string;
    /** Name of the document. */
    documentName: string;
};
type ListDevicesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Number of devices to return within a page. Maximum value is 100. */
    pageSize?: number;
    /** Ordering of requested devices. */
    orderBy?: ListDevicesRequestOrderBy;
    /** Name to filter for, only devices with this name will be returned. */
    name?: string;
    /** Hub ID to filter for, only devices attached to this Hub will be returned. */
    hubId?: string;
    /** Defines wheter to filter the allow_insecure flag. */
    allowInsecure?: boolean;
    /** Device status (enabled, disabled, etc.). */
    status?: DeviceStatus;
};
interface ListDevicesResponse {
    /** Total number of devices. */
    totalCount: number;
    /** Page of devices. */
    devices: Device[];
}
type ListHubsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Number of Hubs to return within a page. Maximum value is 100. */
    pageSize?: number;
    /** Sort order of Hubs in the response. */
    orderBy?: ListHubsRequestOrderBy;
    /** Only list Hubs of this Project ID. */
    projectId?: string;
    /** Only list Hubs of this Organization ID. */
    organizationId?: string;
    /** Hub name. */
    name?: string;
};
interface ListHubsResponse {
    /** Total number of Hubs. */
    totalCount: number;
    /** A page of hubs. */
    hubs: Hub[];
}
type ListNetworksRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Number of networks to return. The maximum value is 100. */
    pageSize?: number;
    /** Ordering of requested routes. */
    orderBy?: ListNetworksRequestOrderBy;
    /** Network name to filter for. */
    name?: string;
    /** Hub ID to filter for. */
    hubId?: string;
    /** Topic prefix to filter for. */
    topicPrefix?: string;
};
interface ListNetworksResponse {
    /** Total number of Networks. */
    totalCount: number;
    /** Page of networks. */
    networks: Network[];
}
type ListRoutesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Number of routes to return within a page. Maximum value is 100. */
    pageSize?: number;
    /** Ordering of requested routes. */
    orderBy?: ListRoutesRequestOrderBy$1;
    /** Hub ID to filter for. */
    hubId?: string;
    /** Route name to filter for. */
    name?: string;
};
interface ListRoutesResponse$1 {
    /** Total number of routes. */
    totalCount: number;
    /** Page of routes. */
    routes: RouteSummary[];
}
type ListTwinDocumentsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Twin ID. */
    twinId: string;
};
interface ListTwinDocumentsResponse {
    /** List of the twin document. */
    documents: ListTwinDocumentsResponseDocumentSummary[];
}
type PatchTwinDocumentRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Twin ID. */
    twinId: string;
    /** Name of the document. */
    documentName: string;
    /**
     * If set, ensures that the current version of the document matches before
     * persisting the update.
     */
    version?: number;
    /**
     * A json data that will be applied on the document's current data. Patching
     * rules: The patch goes recursively through the patch objects. If the patch
     * object property is null, it is removed from the final object. If the patch
     * object property is a value (number, strings, bool, arrays), it is replaced.
     * If the patch object property is an object, the previous rules will be
     * applied recursively on it.
     */
    data?: Record<string, unknown>;
};
type PutTwinDocumentRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Twin ID. */
    twinId: string;
    /** Name of the document. */
    documentName: string;
    /**
     * If set, ensures that the current version of the document matches before
     * persisting the update.
     */
    version?: number;
    /** New data that will replace the contents of the document. */
    data?: Record<string, unknown>;
};
type RenewDeviceCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
};
interface RenewDeviceCertificateResponse {
    /** Information related to the created device. */
    device?: Device;
    /** Device certificate. */
    certificate?: Certificate$1;
}
interface Route$1 {
    /** Route ID. */
    id: string;
    /** Route name. */
    name: string;
    /** Hub ID of the route. */
    hubId: string;
    /**
     * Topic the route subscribes to. It must be a valid MQTT topic and up to
     * 65535 characters.
     */
    topic: string;
    /** Route type. */
    type: RouteRouteType;
    /** Date at which the route was created. */
    createdAt?: Date;
    /**
     * When using S3 Route, S3-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    s3Config?: RouteS3Config;
    /**
     * When using Database Route, DB-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    dbConfig?: RouteDatabaseConfig;
    /**
     * When using Rest Route, Rest-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    restConfig?: RouteRestConfig;
    /** Date at which the route was last updated. */
    updatedAt?: Date;
}
type SetDeviceCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
    /** PEM-encoded custom certificate. */
    certificatePem: string;
};
interface SetDeviceCertificateResponse {
    device?: Device;
    certificatePem: string;
}
type SetHubCARequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hub ID. */
    hubId: string;
    /** CA's PEM-encoded certificate. */
    caCertPem: string;
    /**
     * Challenge is a PEM-encoded certificate that acts as proof of possession of
     * the CA. It must be signed by the CA, and have a Common Name equal to the
     * Hub ID.
     */
    challengeCertPem: string;
};
interface TwinDocument {
    /** Parent twin ID of the document. */
    twinId: string;
    /** Name of the document. */
    documentName: string;
    /** New version of the document. */
    version: number;
    /** New data related to the document. */
    data?: Record<string, unknown>;
}
type UpdateDeviceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Device ID. */
    deviceId: string;
    /** Description for the device. */
    description?: string;
    /**
     * Defines whether to allow plain and server-authenticated SSL connections in
     * addition to mutually-authenticated ones.
     */
    allowInsecure?: boolean;
    /**
     * Defines whether to allow multiple physical devices to connect with this
     * device's credentials.
     */
    allowMultipleConnections?: boolean;
    /** Filter-sets to restrict the topics the device can publish/subscribe to. */
    messageFilters?: DeviceMessageFilters;
    /** Change Hub for this device, additional fees may apply, see IoT Hub pricing. */
    hubId?: string;
};
type UpdateHubRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the Hub you want to update. */
    hubId: string;
    /** Hub name (up to 255 characters). */
    name?: string;
    /** Hub product plan. */
    productPlan?: HubProductPlan;
    /** Disable Hub events. */
    disableEvents?: boolean;
    /** Topic prefix of Hub events. */
    eventsTopicPrefix?: string;
    /** Enable device auto provisioning. */
    enableDeviceAutoProvisioning?: boolean;
    /**
     * BETA - not implemented yet.
     *
     * One-of ('twinsDbConfig'): at most one of 'twinsGraphiteConfig' could be
     * set.
     */
    twinsGraphiteConfig?: HubTwinsGraphiteConfig;
};
type UpdateRouteRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Route id. */
    routeId: string;
    /** Route name. */
    name?: string;
    /**
     * Topic the route subscribes to. It must be a valid MQTT topic and up to
     * 65535 characters.
     */
    topic?: string;
    /**
     * When updating S3 Route, S3-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    s3Config?: UpdateRouteRequestS3Config;
    /**
     * When updating Database Route, DB-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    dbConfig?: UpdateRouteRequestDatabaseConfig;
    /**
     * When updating Rest Route, Rest-specific configuration fields.
     *
     * One-of ('config'): at most one of 's3Config', 'dbConfig', 'restConfig'
     * could be set.
     */
    restConfig?: UpdateRouteRequestRestConfig;
};

/**
 * IoT Hub API.
 *
 * This API allows you to manage IoT hubs and devices.
 */
declare class API$i extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListHubs: (request?: Readonly<ListHubsRequest>) => Promise<ListHubsResponse>;
    /**
     * List hubs. List all Hubs in the specified zone. By default, returned Hubs
     * are ordered by creation date in ascending order, though this can be
     * modified via the `order_by` field.
     *
     * @param request - The request {@link ListHubsRequest}
     * @returns A Promise of ListHubsResponse
     */
    listHubs: (request?: Readonly<ListHubsRequest>) => Promise<ListHubsResponse> & {
        all: () => Promise<Hub[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Hub[], void, void>;
    };
    /**
     * Create a hub. Create a new Hub in the targeted region, specifying its
     * configuration including name and product plan.
     *
     * @param request - The request {@link CreateHubRequest}
     * @returns A Promise of Hub
     */
    createHub: (request: Readonly<CreateHubRequest>) => Promise<Hub>;
    /**
     * Get a hub. Retrieve information about an existing IoT Hub, specified by its
     * Hub ID. Its full details, including name, status and endpoint, are returned
     * in the response object.
     *
     * @param request - The request {@link GetHubRequest}
     * @returns A Promise of Hub
     */
    getHub: (request: Readonly<GetHubRequest>) => Promise<Hub>;
    /**
     * Waits for {@link Hub} to be in a final state.
     *
     * @param request - The request {@link GetHubRequest}
     * @param options - The waiting options
     * @returns A Promise of Hub
     */
    waitForHub: (request: Readonly<GetHubRequest>, options?: Readonly<WaitForOptions<Hub>>) => Promise<Hub>;
    /**
     * Update a hub. Update the parameters of an existing IoT Hub, specified by
     * its Hub ID.
     *
     * @param request - The request {@link UpdateHubRequest}
     * @returns A Promise of Hub
     */
    updateHub: (request: Readonly<UpdateHubRequest>) => Promise<Hub>;
    /**
     * Enable a hub. Enable an existing IoT Hub, specified by its Hub ID.
     *
     * @param request - The request {@link EnableHubRequest}
     * @returns A Promise of Hub
     */
    enableHub: (request: Readonly<EnableHubRequest>) => Promise<Hub>;
    /**
     * Disable a hub. Disable an existing IoT Hub, specified by its Hub ID.
     *
     * @param request - The request {@link DisableHubRequest}
     * @returns A Promise of Hub
     */
    disableHub: (request: Readonly<DisableHubRequest>) => Promise<Hub>;
    /**
     * Delete a hub. Delete an existing IoT Hub, specified by its Hub ID. Deleting
     * a Hub is permanent, and cannot be undone.
     *
     * @param request - The request {@link DeleteHubRequest}
     */
    deleteHub: (request: Readonly<DeleteHubRequest>) => Promise<void>;
    /**
     * Get a hub's metrics. Get the metrics of an existing IoT Hub, specified by
     * its Hub ID.
     *
     * @deprecated
     * @param request - The request {@link GetHubMetricsRequest}
     * @returns A Promise of GetHubMetricsResponse
     */
    getHubMetrics: (request: Readonly<GetHubMetricsRequest>) => Promise<GetHubMetricsResponse>;
    /**
     * Set the certificate authority of a hub. Set a particular PEM-encoded
     * certificate, specified by the Hub ID.
     *
     * @param request - The request {@link SetHubCARequest}
     * @returns A Promise of Hub
     */
    setHubCA: (request: Readonly<SetHubCARequest>) => Promise<Hub>;
    /**
     * Get the certificate authority of a hub. Get information for a particular
     * PEM-encoded certificate, specified by the Hub ID.
     *
     * @param request - The request {@link GetHubCARequest}
     * @returns A Promise of GetHubCAResponse
     */
    getHubCA: (request: Readonly<GetHubCARequest>) => Promise<GetHubCAResponse>;
    protected pageOfListDevices: (request?: Readonly<ListDevicesRequest>) => Promise<ListDevicesResponse>;
    /**
     * List devices. List all devices in the specified region. By default,
     * returned devices are ordered by creation date in ascending order, though
     * this can be modified via the `order_by` field.
     *
     * @param request - The request {@link ListDevicesRequest}
     * @returns A Promise of ListDevicesResponse
     */
    listDevices: (request?: Readonly<ListDevicesRequest>) => Promise<ListDevicesResponse> & {
        all: () => Promise<Device[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Device[], void, void>;
    };
    /**
     * Add a device. Attach a device to a given Hub.
     *
     * @param request - The request {@link CreateDeviceRequest}
     * @returns A Promise of CreateDeviceResponse
     */
    createDevice: (request: Readonly<CreateDeviceRequest>) => Promise<CreateDeviceResponse>;
    /**
     * Get a device. Retrieve information about an existing device, specified by
     * its device ID. Its full details, including name, status and ID, are
     * returned in the response object.
     *
     * @param request - The request {@link GetDeviceRequest}
     * @returns A Promise of Device
     */
    getDevice: (request: Readonly<GetDeviceRequest>) => Promise<Device>;
    /**
     * Update a device. Update the parameters of an existing device, specified by
     * its device ID.
     *
     * @param request - The request {@link UpdateDeviceRequest}
     * @returns A Promise of Device
     */
    updateDevice: (request: Readonly<UpdateDeviceRequest>) => Promise<Device>;
    /**
     * Enable a device. Enable a specific device, specified by its device ID.
     *
     * @param request - The request {@link EnableDeviceRequest}
     * @returns A Promise of Device
     */
    enableDevice: (request: Readonly<EnableDeviceRequest>) => Promise<Device>;
    /**
     * Disable a device. Disable an existing device, specified by its device ID.
     *
     * @param request - The request {@link DisableDeviceRequest}
     * @returns A Promise of Device
     */
    disableDevice: (request: Readonly<DisableDeviceRequest>) => Promise<Device>;
    /**
     * Renew a device certificate. Renew the certificate of an existing device,
     * specified by its device ID.
     *
     * @param request - The request {@link RenewDeviceCertificateRequest}
     * @returns A Promise of RenewDeviceCertificateResponse
     */
    renewDeviceCertificate: (request: Readonly<RenewDeviceCertificateRequest>) => Promise<RenewDeviceCertificateResponse>;
    /**
     * Set a custom certificate on a device. Switch the existing certificate of a
     * given device with an EM-encoded custom certificate.
     *
     * @param request - The request {@link SetDeviceCertificateRequest}
     * @returns A Promise of SetDeviceCertificateResponse
     */
    setDeviceCertificate: (request: Readonly<SetDeviceCertificateRequest>) => Promise<SetDeviceCertificateResponse>;
    /**
     * Get a device's certificate. Get information for a particular PEM-encoded
     * certificate, specified by the device ID. The response returns full details
     * of the device, including its type of certificate.
     *
     * @param request - The request {@link GetDeviceCertificateRequest}
     * @returns A Promise of GetDeviceCertificateResponse
     */
    getDeviceCertificate: (request: Readonly<GetDeviceCertificateRequest>) => Promise<GetDeviceCertificateResponse>;
    /**
     * Remove a device. Remove a specific device from the specific Hub it is
     * attached to.
     *
     * @param request - The request {@link DeleteDeviceRequest}
     */
    deleteDevice: (request: Readonly<DeleteDeviceRequest>) => Promise<void>;
    /**
     * Get a device's metrics. Get the metrics of an existing device, specified by
     * its device ID.
     *
     * @deprecated
     * @param request - The request {@link GetDeviceMetricsRequest}
     * @returns A Promise of GetDeviceMetricsResponse
     */
    getDeviceMetrics: (request: Readonly<GetDeviceMetricsRequest>) => Promise<GetDeviceMetricsResponse>;
    protected pageOfListRoutes: (request?: Readonly<ListRoutesRequest$1>) => Promise<ListRoutesResponse$1>;
    /**
     * List routes. List all routes in the specified region. By default, returned
     * routes are ordered by creation date in ascending order, though this can be
     * modified via the `order_by` field.
     *
     * @param request - The request {@link ListRoutesRequest}
     * @returns A Promise of ListRoutesResponse
     */
    listRoutes: (request?: Readonly<ListRoutesRequest$1>) => Promise<ListRoutesResponse$1> & {
        all: () => Promise<RouteSummary[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<RouteSummary[], void, void>;
    };
    /**
     * Create a route. Multiple kinds of routes can be created, such as:
     *
     * - Database Route Create a route that will record subscribed MQTT messages
     *   into your database. <b>You need to manage the database by yourself</b>.
     * - REST Route. Create a route that will call a REST API on received subscribed
     *   MQTT messages.
     * - S3 Routes. Create a route that will put subscribed MQTT messages into an S3
     *   bucket. You need to create the bucket yourself and grant write access.
     *   Granting can be done with s3cmd (`s3cmd setacl s3://<my-bucket>
     *   --acl-grant=write:555c69c3-87d0-4bf8-80f1-99a2f757d031:555c69c3-87d0-4bf8-80f1-99a2f757d031`).
     *
     * @param request - The request {@link CreateRouteRequest}
     * @returns A Promise of Route
     */
    createRoute: (request: Readonly<CreateRouteRequest$1>) => Promise<Route$1>;
    /**
     * Update a route. Update the parameters of an existing route, specified by
     * its route ID.
     *
     * @param request - The request {@link UpdateRouteRequest}
     * @returns A Promise of Route
     */
    updateRoute: (request: Readonly<UpdateRouteRequest$1>) => Promise<Route$1>;
    /**
     * Get a route. Get information for a particular route, specified by the route
     * ID. The response returns full details of the route, including its type, the
     * topic it subscribes to and its configuration.
     *
     * @param request - The request {@link GetRouteRequest}
     * @returns A Promise of Route
     */
    getRoute: (request: Readonly<GetRouteRequest$1>) => Promise<Route$1>;
    /**
     * Delete a route. Delete an existing route, specified by its route ID.
     * Deleting a route is permanent, and cannot be undone.
     *
     * @param request - The request {@link DeleteRouteRequest}
     */
    deleteRoute: (request: Readonly<DeleteRouteRequest$1>) => Promise<void>;
    protected pageOfListNetworks: (request?: Readonly<ListNetworksRequest>) => Promise<ListNetworksResponse>;
    /**
     * List the networks.
     *
     * @param request - The request {@link ListNetworksRequest}
     * @returns A Promise of ListNetworksResponse
     */
    listNetworks: (request?: Readonly<ListNetworksRequest>) => Promise<ListNetworksResponse> & {
        all: () => Promise<Network[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Network[], void, void>;
    };
    /**
     * Create a new network. Create a new network for an existing hub. Beside the
     * default network, you can add networks for different data providers.
     * Possible network types are Sigfox and REST.
     *
     * @param request - The request {@link CreateNetworkRequest}
     * @returns A Promise of CreateNetworkResponse
     */
    createNetwork: (request: Readonly<CreateNetworkRequest>) => Promise<CreateNetworkResponse>;
    /**
     * Retrieve a specific network. Retrieve an existing network, specified by its
     * network ID. The response returns full details of the network, including its
     * type, the topic prefix and its endpoint.
     *
     * @param request - The request {@link GetNetworkRequest}
     * @returns A Promise of Network
     */
    getNetwork: (request: Readonly<GetNetworkRequest>) => Promise<Network>;
    /**
     * Delete a Network. Delete an existing network, specified by its network ID.
     * Deleting a network is permanent, and cannot be undone.
     *
     * @param request - The request {@link DeleteNetworkRequest}
     */
    deleteNetwork: (request: Readonly<DeleteNetworkRequest>) => Promise<void>;
    /**
     * BETA - Get a Cloud Twin Document.
     *
     * @param request - The request {@link GetTwinDocumentRequest}
     * @returns A Promise of TwinDocument
     */
    getTwinDocument: (request: Readonly<GetTwinDocumentRequest>) => Promise<TwinDocument>;
    /**
     * BETA - Update a Cloud Twin Document.
     *
     * @param request - The request {@link PutTwinDocumentRequest}
     * @returns A Promise of TwinDocument
     */
    putTwinDocument: (request: Readonly<PutTwinDocumentRequest>) => Promise<TwinDocument>;
    /**
     * BETA - Patch a Cloud Twin Document.
     *
     * @param request - The request {@link PatchTwinDocumentRequest}
     * @returns A Promise of TwinDocument
     */
    patchTwinDocument: (request: Readonly<PatchTwinDocumentRequest>) => Promise<TwinDocument>;
    /**
     * BETA - Delete a Cloud Twin Document.
     *
     * @param request - The request {@link DeleteTwinDocumentRequest}
     */
    deleteTwinDocument: (request: Readonly<DeleteTwinDocumentRequest>) => Promise<void>;
    /**
     * BETA - List the documents of a Cloud Twin.
     *
     * @param request - The request {@link ListTwinDocumentsRequest}
     * @returns A Promise of ListTwinDocumentsResponse
     */
    listTwinDocuments: (request: Readonly<ListTwinDocumentsRequest>) => Promise<ListTwinDocumentsResponse>;
    /**
     * BETA - Delete all the documents of a Cloud Twin.
     *
     * @param request - The request {@link DeleteTwinDocumentsRequest}
     */
    deleteTwinDocuments: (request: Readonly<DeleteTwinDocumentsRequest>) => Promise<void>;
}

/** Lists transient statutes of the enum {@link HubStatus}. */
declare const HUB_TRANSIENT_STATUSES: HubStatus[];

type index_gen$i_CreateDeviceRequest = CreateDeviceRequest;
type index_gen$i_CreateDeviceResponse = CreateDeviceResponse;
type index_gen$i_CreateHubRequest = CreateHubRequest;
type index_gen$i_CreateNetworkRequest = CreateNetworkRequest;
type index_gen$i_CreateNetworkResponse = CreateNetworkResponse;
type index_gen$i_CreateRouteRequestDatabaseConfig = CreateRouteRequestDatabaseConfig;
type index_gen$i_CreateRouteRequestRestConfig = CreateRouteRequestRestConfig;
type index_gen$i_CreateRouteRequestS3Config = CreateRouteRequestS3Config;
type index_gen$i_DeleteDeviceRequest = DeleteDeviceRequest;
type index_gen$i_DeleteHubRequest = DeleteHubRequest;
type index_gen$i_DeleteNetworkRequest = DeleteNetworkRequest;
type index_gen$i_DeleteTwinDocumentRequest = DeleteTwinDocumentRequest;
type index_gen$i_DeleteTwinDocumentsRequest = DeleteTwinDocumentsRequest;
type index_gen$i_Device = Device;
type index_gen$i_DeviceMessageFilters = DeviceMessageFilters;
type index_gen$i_DeviceMessageFiltersRule = DeviceMessageFiltersRule;
type index_gen$i_DeviceMessageFiltersRulePolicy = DeviceMessageFiltersRulePolicy;
type index_gen$i_DeviceStatus = DeviceStatus;
type index_gen$i_DisableDeviceRequest = DisableDeviceRequest;
type index_gen$i_DisableHubRequest = DisableHubRequest;
type index_gen$i_EnableDeviceRequest = EnableDeviceRequest;
type index_gen$i_EnableHubRequest = EnableHubRequest;
type index_gen$i_GetDeviceCertificateRequest = GetDeviceCertificateRequest;
type index_gen$i_GetDeviceCertificateResponse = GetDeviceCertificateResponse;
type index_gen$i_GetDeviceMetricsRequest = GetDeviceMetricsRequest;
type index_gen$i_GetDeviceMetricsResponse = GetDeviceMetricsResponse;
type index_gen$i_GetDeviceRequest = GetDeviceRequest;
type index_gen$i_GetHubCARequest = GetHubCARequest;
type index_gen$i_GetHubCAResponse = GetHubCAResponse;
type index_gen$i_GetHubMetricsRequest = GetHubMetricsRequest;
type index_gen$i_GetHubMetricsResponse = GetHubMetricsResponse;
type index_gen$i_GetHubRequest = GetHubRequest;
type index_gen$i_GetNetworkRequest = GetNetworkRequest;
type index_gen$i_GetTwinDocumentRequest = GetTwinDocumentRequest;
declare const index_gen$i_HUB_TRANSIENT_STATUSES: typeof HUB_TRANSIENT_STATUSES;
type index_gen$i_Hub = Hub;
type index_gen$i_HubProductPlan = HubProductPlan;
type index_gen$i_HubStatus = HubStatus;
type index_gen$i_HubTwinsGraphiteConfig = HubTwinsGraphiteConfig;
type index_gen$i_ListDevicesRequest = ListDevicesRequest;
type index_gen$i_ListDevicesRequestOrderBy = ListDevicesRequestOrderBy;
type index_gen$i_ListDevicesResponse = ListDevicesResponse;
type index_gen$i_ListHubsRequest = ListHubsRequest;
type index_gen$i_ListHubsRequestOrderBy = ListHubsRequestOrderBy;
type index_gen$i_ListHubsResponse = ListHubsResponse;
type index_gen$i_ListNetworksRequest = ListNetworksRequest;
type index_gen$i_ListNetworksRequestOrderBy = ListNetworksRequestOrderBy;
type index_gen$i_ListNetworksResponse = ListNetworksResponse;
type index_gen$i_ListTwinDocumentsRequest = ListTwinDocumentsRequest;
type index_gen$i_ListTwinDocumentsResponse = ListTwinDocumentsResponse;
type index_gen$i_ListTwinDocumentsResponseDocumentSummary = ListTwinDocumentsResponseDocumentSummary;
type index_gen$i_Network = Network;
type index_gen$i_NetworkNetworkType = NetworkNetworkType;
type index_gen$i_PatchTwinDocumentRequest = PatchTwinDocumentRequest;
type index_gen$i_PutTwinDocumentRequest = PutTwinDocumentRequest;
type index_gen$i_RenewDeviceCertificateRequest = RenewDeviceCertificateRequest;
type index_gen$i_RenewDeviceCertificateResponse = RenewDeviceCertificateResponse;
type index_gen$i_RouteDatabaseConfig = RouteDatabaseConfig;
type index_gen$i_RouteDatabaseConfigEngine = RouteDatabaseConfigEngine;
type index_gen$i_RouteRestConfig = RouteRestConfig;
type index_gen$i_RouteRestConfigHttpVerb = RouteRestConfigHttpVerb;
type index_gen$i_RouteRouteType = RouteRouteType;
type index_gen$i_RouteS3Config = RouteS3Config;
type index_gen$i_RouteS3ConfigS3Strategy = RouteS3ConfigS3Strategy;
type index_gen$i_RouteSummary = RouteSummary;
type index_gen$i_SetDeviceCertificateRequest = SetDeviceCertificateRequest;
type index_gen$i_SetDeviceCertificateResponse = SetDeviceCertificateResponse;
type index_gen$i_SetHubCARequest = SetHubCARequest;
type index_gen$i_TwinDocument = TwinDocument;
type index_gen$i_UpdateDeviceRequest = UpdateDeviceRequest;
type index_gen$i_UpdateHubRequest = UpdateHubRequest;
type index_gen$i_UpdateRouteRequestDatabaseConfig = UpdateRouteRequestDatabaseConfig;
type index_gen$i_UpdateRouteRequestRestConfig = UpdateRouteRequestRestConfig;
type index_gen$i_UpdateRouteRequestS3Config = UpdateRouteRequestS3Config;
declare namespace index_gen$i {
  export { API$i as API, type Certificate$1 as Certificate, type index_gen$i_CreateDeviceRequest as CreateDeviceRequest, type index_gen$i_CreateDeviceResponse as CreateDeviceResponse, type index_gen$i_CreateHubRequest as CreateHubRequest, type index_gen$i_CreateNetworkRequest as CreateNetworkRequest, type index_gen$i_CreateNetworkResponse as CreateNetworkResponse, type CreateRouteRequest$1 as CreateRouteRequest, type index_gen$i_CreateRouteRequestDatabaseConfig as CreateRouteRequestDatabaseConfig, type index_gen$i_CreateRouteRequestRestConfig as CreateRouteRequestRestConfig, type index_gen$i_CreateRouteRequestS3Config as CreateRouteRequestS3Config, type index_gen$i_DeleteDeviceRequest as DeleteDeviceRequest, type index_gen$i_DeleteHubRequest as DeleteHubRequest, type index_gen$i_DeleteNetworkRequest as DeleteNetworkRequest, type DeleteRouteRequest$1 as DeleteRouteRequest, type index_gen$i_DeleteTwinDocumentRequest as DeleteTwinDocumentRequest, type index_gen$i_DeleteTwinDocumentsRequest as DeleteTwinDocumentsRequest, type index_gen$i_Device as Device, type index_gen$i_DeviceMessageFilters as DeviceMessageFilters, type index_gen$i_DeviceMessageFiltersRule as DeviceMessageFiltersRule, type index_gen$i_DeviceMessageFiltersRulePolicy as DeviceMessageFiltersRulePolicy, type index_gen$i_DeviceStatus as DeviceStatus, type index_gen$i_DisableDeviceRequest as DisableDeviceRequest, type index_gen$i_DisableHubRequest as DisableHubRequest, type index_gen$i_EnableDeviceRequest as EnableDeviceRequest, type index_gen$i_EnableHubRequest as EnableHubRequest, type index_gen$i_GetDeviceCertificateRequest as GetDeviceCertificateRequest, type index_gen$i_GetDeviceCertificateResponse as GetDeviceCertificateResponse, type index_gen$i_GetDeviceMetricsRequest as GetDeviceMetricsRequest, type index_gen$i_GetDeviceMetricsResponse as GetDeviceMetricsResponse, type index_gen$i_GetDeviceRequest as GetDeviceRequest, type index_gen$i_GetHubCARequest as GetHubCARequest, type index_gen$i_GetHubCAResponse as GetHubCAResponse, type index_gen$i_GetHubMetricsRequest as GetHubMetricsRequest, type index_gen$i_GetHubMetricsResponse as GetHubMetricsResponse, type index_gen$i_GetHubRequest as GetHubRequest, type index_gen$i_GetNetworkRequest as GetNetworkRequest, type GetRouteRequest$1 as GetRouteRequest, type index_gen$i_GetTwinDocumentRequest as GetTwinDocumentRequest, index_gen$i_HUB_TRANSIENT_STATUSES as HUB_TRANSIENT_STATUSES, type index_gen$i_Hub as Hub, type index_gen$i_HubProductPlan as HubProductPlan, type index_gen$i_HubStatus as HubStatus, type index_gen$i_HubTwinsGraphiteConfig as HubTwinsGraphiteConfig, type index_gen$i_ListDevicesRequest as ListDevicesRequest, type index_gen$i_ListDevicesRequestOrderBy as ListDevicesRequestOrderBy, type index_gen$i_ListDevicesResponse as ListDevicesResponse, type index_gen$i_ListHubsRequest as ListHubsRequest, type index_gen$i_ListHubsRequestOrderBy as ListHubsRequestOrderBy, type index_gen$i_ListHubsResponse as ListHubsResponse, type index_gen$i_ListNetworksRequest as ListNetworksRequest, type index_gen$i_ListNetworksRequestOrderBy as ListNetworksRequestOrderBy, type index_gen$i_ListNetworksResponse as ListNetworksResponse, type ListRoutesRequest$1 as ListRoutesRequest, type ListRoutesRequestOrderBy$1 as ListRoutesRequestOrderBy, type ListRoutesResponse$1 as ListRoutesResponse, type index_gen$i_ListTwinDocumentsRequest as ListTwinDocumentsRequest, type index_gen$i_ListTwinDocumentsResponse as ListTwinDocumentsResponse, type index_gen$i_ListTwinDocumentsResponseDocumentSummary as ListTwinDocumentsResponseDocumentSummary, type index_gen$i_Network as Network, type index_gen$i_NetworkNetworkType as NetworkNetworkType, type index_gen$i_PatchTwinDocumentRequest as PatchTwinDocumentRequest, type index_gen$i_PutTwinDocumentRequest as PutTwinDocumentRequest, type index_gen$i_RenewDeviceCertificateRequest as RenewDeviceCertificateRequest, type index_gen$i_RenewDeviceCertificateResponse as RenewDeviceCertificateResponse, type Route$1 as Route, type index_gen$i_RouteDatabaseConfig as RouteDatabaseConfig, type index_gen$i_RouteDatabaseConfigEngine as RouteDatabaseConfigEngine, type index_gen$i_RouteRestConfig as RouteRestConfig, type index_gen$i_RouteRestConfigHttpVerb as RouteRestConfigHttpVerb, type index_gen$i_RouteRouteType as RouteRouteType, type index_gen$i_RouteS3Config as RouteS3Config, type index_gen$i_RouteS3ConfigS3Strategy as RouteS3ConfigS3Strategy, type index_gen$i_RouteSummary as RouteSummary, type index_gen$i_SetDeviceCertificateRequest as SetDeviceCertificateRequest, type index_gen$i_SetDeviceCertificateResponse as SetDeviceCertificateResponse, type index_gen$i_SetHubCARequest as SetHubCARequest, type index_gen$i_TwinDocument as TwinDocument, type index_gen$i_UpdateDeviceRequest as UpdateDeviceRequest, type index_gen$i_UpdateHubRequest as UpdateHubRequest, type UpdateRouteRequest$1 as UpdateRouteRequest, type index_gen$i_UpdateRouteRequestDatabaseConfig as UpdateRouteRequestDatabaseConfig, type index_gen$i_UpdateRouteRequestRestConfig as UpdateRouteRequestRestConfig, type index_gen$i_UpdateRouteRequestS3Config as UpdateRouteRequestS3Config };
}

declare namespace index$j {
  export { index_gen$i as v1 };
}

type ListIPsRequestOrderBy$1 = 'created_at_desc' | 'created_at_asc' | 'updated_at_desc' | 'updated_at_asc' | 'attached_at_desc' | 'attached_at_asc';
type ResourceType = 'unknown_type' | 'instance_server' | 'instance_ip' | 'instance_private_nic' | 'lb_server' | 'fip_ip' | 'vpc_gateway' | 'vpc_gateway_network' | 'k8s_node' | 'k8s_cluster' | 'rdb_instance' | 'redis_cluster' | 'baremetal_server' | 'baremetal_private_nic' | 'llm_deployment';
interface Resource {
    /** Type of resource the IP is attached to. */
    type: ResourceType;
    /** ID of the resource the IP is attached to. */
    id: string;
    /** MAC of the resource the IP is attached to. */
    macAddress?: string;
    /**
     * When the IP is in a Private Network, then a DNS record is available to
     * resolve the resource name to this IP.
     */
    name?: string;
}
interface Reverse {
    /** Reverse domain name. */
    hostname: string;
    /** IP corresponding to the hostname. */
    address?: string;
}
interface Source {
    /**
     * This source is global.
     *
     * One-of ('source'): at most one of 'zonal', 'privateNetworkId', 'subnetId'
     * could be set.
     */
    zonal?: string;
    /**
     * This source is specific.
     *
     * One-of ('source'): at most one of 'zonal', 'privateNetworkId', 'subnetId'
     * could be set.
     */
    privateNetworkId?: string;
    /**
     * This source is specific.
     *
     * One-of ('source'): at most one of 'zonal', 'privateNetworkId', 'subnetId'
     * could be set.
     */
    subnetId?: string;
}
interface IP$1 {
    /** IP ID. */
    id: string;
    /** IPv4 or IPv6 address in CIDR notation. */
    address: string;
    /** Scaleway Project the IP belongs to. */
    projectId: string;
    /** Defines whether the IP is an IPv6 (false = IPv4). */
    isIpv6: boolean;
    /** Date the IP was booked. */
    createdAt?: Date;
    /** Date the IP was last modified. */
    updatedAt?: Date;
    /** Source pool where the IP was booked in. */
    source: Source;
    /** Resource which the IP is attached to. */
    resource?: Resource;
    /** Tags for the IP. */
    tags: string[];
    /** Array of reverses associated with the IP. */
    reverses: Reverse[];
    /** Region of the IP. */
    region: Region;
    /** Zone of the IP, if zonal. */
    zone?: Zone;
}
type BookIPRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * When creating an IP in a Private Network, the Project must match the
     * Private Network's Project.
     */
    projectId?: string;
    /** Source in which to book the IP. Not all sources are available for booking. */
    source: Source;
    /** Request an IPv6 instead of an IPv4. */
    isIpv6: boolean;
    /**
     * Note that only the Private Network source allows you to pick a specific IP.
     * If the requested IP is already booked, then the call will fail.
     */
    address?: string;
    /** Tags for the IP. */
    tags?: string[];
};
type GetIPRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** IP ID. */
    ipId: string;
};
type ListIPsRequest$3 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Sort order of the returned IPs. */
    orderBy?: ListIPsRequestOrderBy$1;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Maximum number of IPs to return per page. */
    pageSize?: number;
    /**
     * Project ID to filter for. Only IPs belonging to this Project will be
     * returned.
     */
    projectId?: string;
    /**
     * Zone to filter for. Only IPs that are zonal, and in this zone, will be
     * returned.
     *
     * One-of ('source'): at most one of 'zonal', 'privateNetworkId' could be set.
     */
    zonal?: string;
    /**
     * Only IPs that are private, and in this Private Network, will be returned.
     *
     * One-of ('source'): at most one of 'zonal', 'privateNetworkId' could be set.
     */
    privateNetworkId?: string;
    /** Defines whether to filter only for IPs which are attached to a resource. */
    attached?: boolean;
    /**
     * Resource ID to filter for. Only IPs attached to this resource will be
     * returned.
     */
    resourceId?: string;
    /**
     * Resource type to filter for. Only IPs attached to this type of resource
     * will be returned.
     */
    resourceType?: ResourceType;
    /**
     * MAC address to filter for. Only IPs attached to a resource with this MAC
     * address will be returned.
     */
    macAddress?: string;
    /**
     * Tags to filter for, only IPs with one or more matching tags will be
     * returned.
     */
    tags?: string[];
    /**
     * Organization ID to filter for. Only IPs belonging to this Organization will
     * be returned.
     */
    organizationId?: string;
    /** Defines whether to filter only for IPv4s or IPv6s. */
    isIpv6?: boolean;
    /**
     * Attached resource name to filter for, only IPs attached to a resource with
     * this string within their name will be returned.
     */
    resourceName?: string;
};
interface ListIPsResponse$1 {
    totalCount: number;
    ips: IP$1[];
}
type ReleaseIPRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** IP ID. */
    ipId: string;
};
type UpdateIPRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** IP ID. */
    ipId: string;
    /** Tags for the IP. */
    tags?: string[];
    /**
     * Array of reverse domain names associated with an IP in the subnet of the
     * current IP.
     */
    reverses?: Reverse[];
};

/**
 * IPAM API.
 *
 * This API allows you to manage IP addresses with Scaleway's IP Address
 * Management tool.
 */
declare class API$h extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Book a new IP. Book a new IP from the specified source. Currently IPs can
     * only be booked from a Private Network.
     *
     * @param request - The request {@link BookIPRequest}
     * @returns A Promise of IP
     */
    bookIP: (request: Readonly<BookIPRequest>) => Promise<IP$1>;
    /**
     * Release an IP. Release an IP not currently attached to a resource, and
     * returns it to the available IP pool.
     *
     * @param request - The request {@link ReleaseIPRequest}
     */
    releaseIP: (request: Readonly<ReleaseIPRequest>) => Promise<void>;
    /**
     * Get an IP. Retrieve details of an existing IP, specified by its IP ID.
     *
     * @param request - The request {@link GetIPRequest}
     * @returns A Promise of IP
     */
    getIP: (request: Readonly<GetIPRequest$1>) => Promise<IP$1>;
    /**
     * Update an IP. Update parameters including tags of the specified IP.
     *
     * @param request - The request {@link UpdateIPRequest}
     * @returns A Promise of IP
     */
    updateIP: (request: Readonly<UpdateIPRequest$1>) => Promise<IP$1>;
    protected pageOfListIPs: (request?: Readonly<ListIPsRequest$3>) => Promise<ListIPsResponse$1>;
    /**
     * List existing IPs. List existing IPs in the specified region using various
     * filters. For example, you can filter for IPs within a specified Private
     * Network, or for public IPs within a specified Project. By default, the IPs
     * returned in the list are ordered by creation date in ascending order,
     * though this can be modified via the order_by field.
     *
     * @param request - The request {@link ListIPsRequest}
     * @returns A Promise of ListIPsResponse
     */
    listIPs: (request?: Readonly<ListIPsRequest$3>) => Promise<ListIPsResponse$1> & {
        all: () => Promise<IP$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<IP$1[], void, void>;
    };
}

declare const ListIPsRequest$2: {
    resourceName: {
        maxLength: number;
        minLength: number;
    };
};

declare namespace validationRules_gen$6 {
  export { ListIPsRequest$2 as ListIPsRequest };
}

type index_gen$h_BookIPRequest = BookIPRequest;
type index_gen$h_ReleaseIPRequest = ReleaseIPRequest;
type index_gen$h_Resource = Resource;
type index_gen$h_ResourceType = ResourceType;
type index_gen$h_Reverse = Reverse;
type index_gen$h_Source = Source;
declare namespace index_gen$h {
  export { API$h as API, type index_gen$h_BookIPRequest as BookIPRequest, type GetIPRequest$1 as GetIPRequest, type IP$1 as IP, type ListIPsRequest$3 as ListIPsRequest, type ListIPsRequestOrderBy$1 as ListIPsRequestOrderBy, type ListIPsResponse$1 as ListIPsResponse, type index_gen$h_ReleaseIPRequest as ReleaseIPRequest, type index_gen$h_Resource as Resource, type index_gen$h_ResourceType as ResourceType, type index_gen$h_Reverse as Reverse, type index_gen$h_Source as Source, type UpdateIPRequest$1 as UpdateIPRequest, validationRules_gen$6 as ValidationRules };
}

declare namespace index$i {
  export { index_gen$h as v1 };
}

type ListNamesRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListPinsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListVolumesRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type NameStatus = 'unknown_status' | 'queued' | 'publishing' | 'failed' | 'published';
type PinDetails = 'unknown_details' | 'pinning_looking_for_provider' | 'pinning_in_progress' | 'pinning_blocks_fetched' | 'pinning_fetching_url_data' | 'pinned_ok' | 'unpinned_ok' | 'unpinning_in_progress' | 'failed_contains_banned_cid' | 'failed_pinning' | 'failed_pinning_no_provider' | 'failed_pinning_bad_cid_format' | 'failed_pinning_timeout' | 'failed_pinning_too_big_content' | 'failed_pinning_unreachable_url' | 'failed_pinning_bad_url_format' | 'failed_pinning_no_url_content_length' | 'failed_pinning_bad_url_status_code' | 'failed_unpinning' | 'checking_coherence' | 'rescheduled';
type PinStatus = 'unknown_status' | 'queued' | 'pinning' | 'failed' | 'pinned';
interface PinCIDMeta {
    id?: string;
}
interface PinCID {
    cid?: string;
    name?: string;
    origins: string[];
    meta?: PinCIDMeta;
}
interface PinInfo {
    id?: string;
    url?: string;
    size?: number;
    progress?: number;
    statusDetails: PinDetails;
}
interface Name {
    nameId: string;
    projectId: string;
    createdAt?: Date;
    updatedAt?: Date;
    tags: string[];
    name: string;
    key: string;
    status: NameStatus;
    value: string;
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region: Region;
}
interface Pin {
    pinId: string;
    status: PinStatus;
    createdAt?: Date;
    cid?: PinCID;
    delegates: string[];
    info?: PinInfo;
}
interface Volume$1 {
    id: string;
    projectId: string;
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region: Region;
    countPin: number;
    createdAt?: Date;
    updatedAt?: Date;
    tags: string[];
    name: string;
    size?: number;
}
type CreatePinByCIDRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Volume ID on which you want to pin your content. */
    volumeId: string;
    /** CID containing the content you want to pin. */
    cid: string;
    /** Node containing the content you want to pin. */
    origins?: string[];
    /** Pin name. */
    name?: string;
};
type CreatePinByURLRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Volume ID on which you want to pin your content. */
    volumeId: string;
    /** URL containing the content you want to pin. */
    url: string;
    /** Pin name. */
    name?: string;
};
type CreateVolumeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project ID. */
    projectId?: string;
    /** Volume name. */
    name: string;
};
type DeletePinRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Pin ID you want to remove from the volume. */
    pinId: string;
    /** Volume ID. */
    volumeId: string;
};
type DeleteVolumeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Volume ID. */
    volumeId: string;
};
interface ExportKeyNameResponse {
    nameId: string;
    projectId: string;
    createdAt?: Date;
    updatedAt?: Date;
    publicKey: string;
    privateKey: string;
}
type GetPinRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Pin ID of which you want to obtain information. */
    pinId: string;
    /** Volume ID. */
    volumeId: string;
};
type GetVolumeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Volume ID. */
    volumeId: string;
};
type IpnsApiCreateNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project ID. */
    projectId?: string;
    /** Name for your records. */
    name: string;
    /** Value you want to associate with your records, CID or IPNS key. */
    value: string;
};
type IpnsApiDeleteNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name ID you wish to delete. */
    nameId: string;
};
type IpnsApiExportKeyNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name ID whose keys you want to export. */
    nameId: string;
};
type IpnsApiGetNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name ID whose information you want to retrieve. */
    nameId: string;
};
type IpnsApiImportKeyNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project ID. */
    projectId?: string;
    /** Name for your records. */
    name: string;
    /** Base64 private key. */
    privateKey: string;
    /** Value you want to associate with your records, CID or IPNS key. */
    value: string;
};
type IpnsApiListNamesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project ID. */
    projectId?: string;
    /** Organization ID. */
    organizationId?: string;
    /** Sort the order of the returned names. */
    orderBy?: ListNamesRequestOrderBy;
    /** Page number. */
    page?: number;
    /** Maximum number of names to return per page. */
    pageSize?: number;
};
type IpnsApiUpdateNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name ID you wish to update. */
    nameId: string;
    /** New name you want to associate with your record. */
    name?: string;
    /** New tags you want to associate with your record. */
    tags?: string[];
    /** Value you want to associate with your records, CID or IPNS key. */
    value?: string;
};
interface ListNamesResponse {
    names: Name[];
    totalCount: number;
}
type ListPinsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Volume ID of which you want to list the pins. */
    volumeId: string;
    /** Project ID. */
    projectId?: string;
    /** Organization ID. */
    organizationId?: string;
    /** Sort order of the returned Volume. */
    orderBy?: ListPinsRequestOrderBy;
    /** Page number. */
    page?: number;
    /** Maximum number of volumes to return per page. */
    pageSize?: number;
    /** List pins by status. */
    status?: PinStatus;
};
interface ListPinsResponse {
    totalCount: number;
    pins: Pin[];
}
type ListVolumesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project ID, only volumes belonging to this project will be listed. */
    projectId?: string;
    /** Sort the order of the returned volumes. */
    orderBy?: ListVolumesRequestOrderBy;
    /** Page number. */
    page?: number;
    /** Maximum number of volumes to return per page. */
    pageSize?: number;
};
interface ListVolumesResponse {
    volumes: Volume$1[];
    totalCount: number;
}
type ReplacePinRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Pin ID whose information you wish to replace. */
    pinId: string;
    /** Volume ID. */
    volumeId: string;
    /** New CID you want to pin in place of the old one. */
    cid: string;
    /** New name to replace. */
    name?: string;
    /** Node containing the content you want to pin. */
    origins?: string[];
};
interface ReplacePinResponse {
    pin?: Pin;
}
type UpdateVolumeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Volume ID. */
    volumeId: string;
    /** Volume name. */
    name?: string;
    /** Tags of the volume. */
    tags?: string[];
};

/** IPFS Pinning service API. */
declare class API$g extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Create a new volume. Create a new volume from a Project ID. Volume is
     * identified by an ID and used to host pin references. Volume is personal (at
     * least to your organization) even if IPFS blocks and CID are available to
     * anyone. Should be the first command you made because every pin must be
     * attached to a volume.
     *
     * @param request - The request {@link CreateVolumeRequest}
     * @returns A Promise of Volume
     */
    createVolume: (request: Readonly<CreateVolumeRequest>) => Promise<Volume$1>;
    /**
     * Get information about a volume. Retrieve information about a specific
     * volume.
     *
     * @param request - The request {@link GetVolumeRequest}
     * @returns A Promise of Volume
     */
    getVolume: (request: Readonly<GetVolumeRequest>) => Promise<Volume$1>;
    protected pageOfListVolumes: (request?: Readonly<ListVolumesRequest>) => Promise<ListVolumesResponse>;
    /**
     * List all volumes by a Project ID. Retrieve information about all volumes
     * from a Project ID.
     *
     * @param request - The request {@link ListVolumesRequest}
     * @returns A Promise of ListVolumesResponse
     */
    listVolumes: (request?: Readonly<ListVolumesRequest>) => Promise<ListVolumesResponse> & {
        all: () => Promise<Volume$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Volume$1[], void, void>;
    };
    /**
     * Update volume information. Update volume information (tag, name...).
     *
     * @param request - The request {@link UpdateVolumeRequest}
     * @returns A Promise of Volume
     */
    updateVolume: (request: Readonly<UpdateVolumeRequest>) => Promise<Volume$1>;
    /**
     * Delete an existing volume. Delete a volume by its ID and every pin attached
     * to this volume. This process can take a while to conclude, depending on the
     * size of your pinned content.
     *
     * @param request - The request {@link DeleteVolumeRequest}
     */
    deleteVolume: (request: Readonly<DeleteVolumeRequest>) => Promise<void>;
    /**
     * Create a pin by URL. Will fetch and store the content pointed by the
     * provided URL. The content must be available on the public IPFS network. The
     * content (IPFS blocks) will be host by the pinning service until pin
     * deletion. From that point, any other IPFS peer can fetch and host your
     * content: Make sure to pin public or encrypted content. Many pin requests
     * (from different users) can target the same CID. A pin is defined by its ID
     * (UUID), its status (queued, pinning, pinned or failed) and target CID.
     *
     * @param request - The request {@link CreatePinByURLRequest}
     * @returns A Promise of Pin
     */
    createPinByURL: (request: Readonly<CreatePinByURLRequest>) => Promise<Pin>;
    /**
     * Create a pin by CID. Will fetch and store the content pointed by the
     * provided CID. The content must be available on the public IPFS network. The
     * content (IPFS blocks) will be host by the pinning service until pin
     * deletion. From that point, any other IPFS peer can fetch and host your
     * content: Make sure to pin public or encrypted content. Many pin requests
     * (from different users) can target the same CID. A pin is defined by its ID
     * (UUID), its status (queued, pinning, pinned or failed) and target CID.
     *
     * @param request - The request {@link CreatePinByCIDRequest}
     * @returns A Promise of Pin
     */
    createPinByCID: (request: Readonly<CreatePinByCIDRequest>) => Promise<Pin>;
    /**
     * Replace pin by CID. Deletes the given resource ID and pins the new CID in
     * its place. Will fetch and store the content pointed by the provided CID.
     * The content must be available on the public IPFS network. The content (IPFS
     * blocks) is hosted by the pinning service until the pin is deleted. While
     * the content is available any other IPFS peer can fetch and host your
     * content. For this reason, we recommend that you pin either public or
     * encrypted content. Several different pin requests can target the same CID.
     * A pin is defined by its ID (UUID), its status (queued, pinning, pinned or
     * failed) and target CID.
     *
     * @param request - The request {@link ReplacePinRequest}
     * @returns A Promise of ReplacePinResponse
     */
    replacePin: (request: Readonly<ReplacePinRequest>) => Promise<ReplacePinResponse>;
    /**
     * Get pin information. Retrieve information about the provided **pin ID**,
     * such as status, last modification, and CID.
     *
     * @param request - The request {@link GetPinRequest}
     * @returns A Promise of Pin
     */
    getPin: (request: Readonly<GetPinRequest>) => Promise<Pin>;
    /**
     * Waits for {@link Pin} to be in a final state.
     *
     * @param request - The request {@link GetPinRequest}
     * @param options - The waiting options
     * @returns A Promise of Pin
     */
    waitForPin: (request: Readonly<GetPinRequest>, options?: Readonly<WaitForOptions<Pin>>) => Promise<Pin>;
    protected pageOfListPins: (request: Readonly<ListPinsRequest>) => Promise<ListPinsResponse>;
    /**
     * List all pins within a volume. Retrieve information about all pins within a
     * volume.
     *
     * @param request - The request {@link ListPinsRequest}
     * @returns A Promise of ListPinsResponse
     */
    listPins: (request: Readonly<ListPinsRequest>) => Promise<ListPinsResponse> & {
        all: () => Promise<Pin[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Pin[], void, void>;
    };
    /**
     * Create an unpin request. An unpin request means that you no longer own the
     * content. This content can therefore be removed and no longer provided on
     * the IPFS network.
     *
     * @param request - The request {@link DeletePinRequest}
     */
    deletePin: (request: Readonly<DeletePinRequest>) => Promise<void>;
}
/** IPFS Naming service API. */
declare class IpnsAPI extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Create a new name. You can use the `ipns key` command to list and generate
     * more names and their respective keys.
     *
     * @param request - The request {@link IpnsApiCreateNameRequest}
     * @returns A Promise of Name
     */
    createName: (request: Readonly<IpnsApiCreateNameRequest>) => Promise<Name>;
    /**
     * Get information about a name. Retrieve information about a specific name.
     *
     * @param request - The request {@link IpnsApiGetNameRequest}
     * @returns A Promise of Name
     */
    getName: (request: Readonly<IpnsApiGetNameRequest>) => Promise<Name>;
    /**
     * Waits for {@link Name} to be in a final state.
     *
     * @param request - The request {@link IpnsApiGetNameRequest}
     * @param options - The waiting options
     * @returns A Promise of Name
     */
    waitForName: (request: Readonly<IpnsApiGetNameRequest>, options?: Readonly<WaitForOptions<Name>>) => Promise<Name>;
    /**
     * Delete an existing name. Delete a name by its ID.
     *
     * @param request - The request {@link IpnsApiDeleteNameRequest}
     */
    deleteName: (request: Readonly<IpnsApiDeleteNameRequest>) => Promise<void>;
    protected pageOfListNames: (request?: Readonly<IpnsApiListNamesRequest>) => Promise<ListNamesResponse>;
    /**
     * List all names by a Project ID. Retrieve information about all names from a
     * Project ID.
     *
     * @param request - The request {@link IpnsApiListNamesRequest}
     * @returns A Promise of ListNamesResponse
     */
    listNames: (request?: Readonly<IpnsApiListNamesRequest>) => Promise<ListNamesResponse> & {
        all: () => Promise<Name[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Name[], void, void>;
    };
    /**
     * Update name information. Update name information (CID, tag, name...).
     *
     * @param request - The request {@link IpnsApiUpdateNameRequest}
     * @returns A Promise of Name
     */
    updateName: (request: Readonly<IpnsApiUpdateNameRequest>) => Promise<Name>;
    /**
     * Export your private key. Export a private key by its ID.
     *
     * @param request - The request {@link IpnsApiExportKeyNameRequest}
     * @returns A Promise of ExportKeyNameResponse
     */
    exportKeyName: (request: Readonly<IpnsApiExportKeyNameRequest>) => Promise<ExportKeyNameResponse>;
    /**
     * Import your private key. Import a private key.
     *
     * @param request - The request {@link IpnsApiImportKeyNameRequest}
     * @returns A Promise of Name
     */
    importKeyName: (request: Readonly<IpnsApiImportKeyNameRequest>) => Promise<Name>;
}

/** Lists transient statutes of the enum {@link NameStatus}. */
declare const NAME_TRANSIENT_STATUSES: NameStatus[];
/** Lists transient statutes of the enum {@link PinStatus}. */
declare const PIN_TRANSIENT_STATUSES: PinStatus[];

type index_gen$g_CreatePinByCIDRequest = CreatePinByCIDRequest;
type index_gen$g_CreatePinByURLRequest = CreatePinByURLRequest;
type index_gen$g_CreateVolumeRequest = CreateVolumeRequest;
type index_gen$g_DeletePinRequest = DeletePinRequest;
type index_gen$g_DeleteVolumeRequest = DeleteVolumeRequest;
type index_gen$g_ExportKeyNameResponse = ExportKeyNameResponse;
type index_gen$g_GetPinRequest = GetPinRequest;
type index_gen$g_GetVolumeRequest = GetVolumeRequest;
type index_gen$g_IpnsAPI = IpnsAPI;
declare const index_gen$g_IpnsAPI: typeof IpnsAPI;
type index_gen$g_IpnsApiCreateNameRequest = IpnsApiCreateNameRequest;
type index_gen$g_IpnsApiDeleteNameRequest = IpnsApiDeleteNameRequest;
type index_gen$g_IpnsApiExportKeyNameRequest = IpnsApiExportKeyNameRequest;
type index_gen$g_IpnsApiGetNameRequest = IpnsApiGetNameRequest;
type index_gen$g_IpnsApiImportKeyNameRequest = IpnsApiImportKeyNameRequest;
type index_gen$g_IpnsApiListNamesRequest = IpnsApiListNamesRequest;
type index_gen$g_IpnsApiUpdateNameRequest = IpnsApiUpdateNameRequest;
type index_gen$g_ListNamesRequestOrderBy = ListNamesRequestOrderBy;
type index_gen$g_ListNamesResponse = ListNamesResponse;
type index_gen$g_ListPinsRequest = ListPinsRequest;
type index_gen$g_ListPinsRequestOrderBy = ListPinsRequestOrderBy;
type index_gen$g_ListPinsResponse = ListPinsResponse;
type index_gen$g_ListVolumesRequest = ListVolumesRequest;
type index_gen$g_ListVolumesRequestOrderBy = ListVolumesRequestOrderBy;
type index_gen$g_ListVolumesResponse = ListVolumesResponse;
declare const index_gen$g_NAME_TRANSIENT_STATUSES: typeof NAME_TRANSIENT_STATUSES;
type index_gen$g_Name = Name;
type index_gen$g_NameStatus = NameStatus;
declare const index_gen$g_PIN_TRANSIENT_STATUSES: typeof PIN_TRANSIENT_STATUSES;
type index_gen$g_Pin = Pin;
type index_gen$g_PinCID = PinCID;
type index_gen$g_PinCIDMeta = PinCIDMeta;
type index_gen$g_PinDetails = PinDetails;
type index_gen$g_PinInfo = PinInfo;
type index_gen$g_PinStatus = PinStatus;
type index_gen$g_ReplacePinRequest = ReplacePinRequest;
type index_gen$g_ReplacePinResponse = ReplacePinResponse;
type index_gen$g_UpdateVolumeRequest = UpdateVolumeRequest;
declare namespace index_gen$g {
  export { API$g as API, type index_gen$g_CreatePinByCIDRequest as CreatePinByCIDRequest, type index_gen$g_CreatePinByURLRequest as CreatePinByURLRequest, type index_gen$g_CreateVolumeRequest as CreateVolumeRequest, type index_gen$g_DeletePinRequest as DeletePinRequest, type index_gen$g_DeleteVolumeRequest as DeleteVolumeRequest, type index_gen$g_ExportKeyNameResponse as ExportKeyNameResponse, type index_gen$g_GetPinRequest as GetPinRequest, type index_gen$g_GetVolumeRequest as GetVolumeRequest, index_gen$g_IpnsAPI as IpnsAPI, type index_gen$g_IpnsApiCreateNameRequest as IpnsApiCreateNameRequest, type index_gen$g_IpnsApiDeleteNameRequest as IpnsApiDeleteNameRequest, type index_gen$g_IpnsApiExportKeyNameRequest as IpnsApiExportKeyNameRequest, type index_gen$g_IpnsApiGetNameRequest as IpnsApiGetNameRequest, type index_gen$g_IpnsApiImportKeyNameRequest as IpnsApiImportKeyNameRequest, type index_gen$g_IpnsApiListNamesRequest as IpnsApiListNamesRequest, type index_gen$g_IpnsApiUpdateNameRequest as IpnsApiUpdateNameRequest, type index_gen$g_ListNamesRequestOrderBy as ListNamesRequestOrderBy, type index_gen$g_ListNamesResponse as ListNamesResponse, type index_gen$g_ListPinsRequest as ListPinsRequest, type index_gen$g_ListPinsRequestOrderBy as ListPinsRequestOrderBy, type index_gen$g_ListPinsResponse as ListPinsResponse, type index_gen$g_ListVolumesRequest as ListVolumesRequest, type index_gen$g_ListVolumesRequestOrderBy as ListVolumesRequestOrderBy, type index_gen$g_ListVolumesResponse as ListVolumesResponse, index_gen$g_NAME_TRANSIENT_STATUSES as NAME_TRANSIENT_STATUSES, type index_gen$g_Name as Name, type index_gen$g_NameStatus as NameStatus, index_gen$g_PIN_TRANSIENT_STATUSES as PIN_TRANSIENT_STATUSES, type index_gen$g_Pin as Pin, type index_gen$g_PinCID as PinCID, type index_gen$g_PinCIDMeta as PinCIDMeta, type index_gen$g_PinDetails as PinDetails, type index_gen$g_PinInfo as PinInfo, type index_gen$g_PinStatus as PinStatus, type index_gen$g_ReplacePinRequest as ReplacePinRequest, type index_gen$g_ReplacePinResponse as ReplacePinResponse, type index_gen$g_UpdateVolumeRequest as UpdateVolumeRequest, type Volume$1 as Volume };
}

declare namespace index$h {
  export { index_gen$g as v1alpha1 };
}

type JobRunState = 'unknown_state' | 'queued' | 'scheduled' | 'running' | 'succeeded' | 'failed' | 'canceled' | 'internal_error';
type ListJobDefinitionsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListJobRunsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
interface CronSchedule$1 {
    schedule: string;
    timezone: string;
}
interface CreateJobDefinitionRequestCronScheduleConfig$1 {
    schedule: string;
    timezone: string;
}
interface JobDefinition {
    id: string;
    name: string;
    createdAt?: Date;
    updatedAt?: Date;
    cpuLimit: number;
    memoryLimit: number;
    imageUri: string;
    command: string;
    projectId: string;
    environmentVariables: Record<string, string>;
    description: string;
    jobTimeout?: string;
    cronSchedule?: CronSchedule$1;
    localStorageCapacity: number;
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region: Region;
}
interface JobRun {
    id: string;
    jobDefinitionId: string;
    state: JobRunState;
    createdAt?: Date;
    updatedAt?: Date;
    terminatedAt?: Date;
    exitCode?: number;
    runDuration?: string;
    errorMessage: string;
    cpuLimit: number;
    memoryLimit: number;
    command: string;
    environmentVariables: Record<string, string>;
    localStorageCapacity: number;
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region: Region;
}
interface UpdateJobDefinitionRequestCronScheduleConfig$1 {
    schedule?: string;
    timezone?: string;
}
type CreateJobDefinitionRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the job definition. */
    name?: string;
    /** CPU limit of the job. */
    cpuLimit: number;
    /** Memory limit of the job (in MiB). */
    memoryLimit: number;
    /** Local storage capacity of the job (in MiB). */
    localStorageCapacity?: number;
    /** Image to use for the job. */
    imageUri: string;
    /** Startup command. */
    command: string;
    /** UUID of the Scaleway Project containing the job. */
    projectId?: string;
    /** Environment variables of the job. */
    environmentVariables?: Record<string, string>;
    /** Description of the job. */
    description: string;
    /** Timeout of the job in seconds. */
    jobTimeout?: string;
    cronSchedule?: CreateJobDefinitionRequestCronScheduleConfig$1;
};
type DeleteJobDefinitionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the job definition to delete. */
    jobDefinitionId: string;
};
type GetJobDefinitionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the job definition to get. */
    jobDefinitionId: string;
};
type GetJobRunRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the job run to get. */
    jobRunId: string;
};
type ListJobDefinitionsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    page?: number;
    pageSize?: number;
    orderBy?: ListJobDefinitionsRequestOrderBy;
    projectId?: string;
};
interface ListJobDefinitionsResponse {
    jobDefinitions: JobDefinition[];
    totalCount: number;
}
type ListJobRunsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    page?: number;
    pageSize?: number;
    orderBy?: ListJobRunsRequestOrderBy;
    jobDefinitionId?: string;
    projectId?: string;
};
interface ListJobRunsResponse {
    jobRuns: JobRun[];
    totalCount: number;
}
type StartJobDefinitionRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the job definition to start. */
    jobDefinitionId: string;
    /** Contextual startup command for this specific job run. */
    command?: string;
    /** Contextual environment variables for this specific job run. */
    environmentVariables?: Record<string, string>;
    /** Number of jobs to run. */
    replicas?: number;
};
interface StartJobDefinitionResponse {
    jobRuns: JobRun[];
}
type StopJobRunRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the job run to stop. */
    jobRunId: string;
};
type UpdateJobDefinitionRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the job definition to update. */
    jobDefinitionId: string;
    /** Name of the job definition. */
    name?: string;
    /** CPU limit of the job. */
    cpuLimit?: number;
    /** Memory limit of the job (in MiB). */
    memoryLimit?: number;
    /** Local storage capacity of the job (in MiB). */
    localStorageCapacity?: number;
    /** Image to use for the job. */
    imageUri?: string;
    /** Startup command. */
    command?: string;
    /** Environment variables of the job. */
    environmentVariables?: Record<string, string>;
    /** Description of the job. */
    description?: string;
    /** Timeout of the job in seconds. */
    jobTimeout?: string;
    cronSchedule?: UpdateJobDefinitionRequestCronScheduleConfig$1;
};

/** Serverless Jobs API. */
declare class API$f extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Create a new job definition in a specified Project.
     *
     * @param request - The request {@link CreateJobDefinitionRequest}
     * @returns A Promise of JobDefinition
     */
    createJobDefinition: (request: Readonly<CreateJobDefinitionRequest$1>) => Promise<JobDefinition>;
    /**
     * Get a job definition by its unique identifier.
     *
     * @param request - The request {@link GetJobDefinitionRequest}
     * @returns A Promise of JobDefinition
     */
    getJobDefinition: (request: Readonly<GetJobDefinitionRequest>) => Promise<JobDefinition>;
    protected pageOfListJobDefinitions: (request?: Readonly<ListJobDefinitionsRequest$1>) => Promise<ListJobDefinitionsResponse>;
    /**
     * List all your job definitions with filters.
     *
     * @param request - The request {@link ListJobDefinitionsRequest}
     * @returns A Promise of ListJobDefinitionsResponse
     */
    listJobDefinitions: (request?: Readonly<ListJobDefinitionsRequest$1>) => Promise<ListJobDefinitionsResponse> & {
        all: () => Promise<JobDefinition[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<JobDefinition[], void, void>;
    };
    /**
     * Update an existing job definition associated with the specified unique
     * identifier.
     *
     * @param request - The request {@link UpdateJobDefinitionRequest}
     * @returns A Promise of JobDefinition
     */
    updateJobDefinition: (request: Readonly<UpdateJobDefinitionRequest$1>) => Promise<JobDefinition>;
    /**
     * Delete an exsisting job definition by its unique identifier.
     *
     * @param request - The request {@link DeleteJobDefinitionRequest}
     */
    deleteJobDefinition: (request: Readonly<DeleteJobDefinitionRequest>) => Promise<void>;
    /**
     * Run an existing job definition by its unique identifier. This will create a
     * new job run.
     *
     * @param request - The request {@link StartJobDefinitionRequest}
     * @returns A Promise of StartJobDefinitionResponse
     */
    startJobDefinition: (request: Readonly<StartJobDefinitionRequest$1>) => Promise<StartJobDefinitionResponse>;
    /**
     * Get a job run by its unique identifier.
     *
     * @param request - The request {@link GetJobRunRequest}
     * @returns A Promise of JobRun
     */
    getJobRun: (request: Readonly<GetJobRunRequest>) => Promise<JobRun>;
    /**
     * Stop a job run by its unique identifier.
     *
     * @param request - The request {@link StopJobRunRequest}
     * @returns A Promise of JobRun
     */
    stopJobRun: (request: Readonly<StopJobRunRequest>) => Promise<JobRun>;
    protected pageOfListJobRuns: (request?: Readonly<ListJobRunsRequest$1>) => Promise<ListJobRunsResponse>;
    /**
     * List all job runs with filters.
     *
     * @param request - The request {@link ListJobRunsRequest}
     * @returns A Promise of ListJobRunsResponse
     */
    listJobRuns: (request?: Readonly<ListJobRunsRequest$1>) => Promise<ListJobRunsResponse> & {
        all: () => Promise<JobRun[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<JobRun[], void, void>;
    };
}

/** Lists transient statutes of the enum {@link JobRunState}. */
declare const JOB_RUN_TRANSIENT_STATUSES: JobRunState[];

declare const CreateJobDefinitionRequest: {
    cpuLimit: {
        greaterThan: number;
    };
    imageUri: {
        pattern: RegExp;
    };
    localStorageCapacity: {
        greaterThan: number;
    };
    memoryLimit: {
        greaterThan: number;
    };
    name: {
        pattern: RegExp;
    };
};
declare const CreateJobDefinitionRequestCronScheduleConfig: {
    schedule: {
        maxLength: number;
        minLength: number;
    };
    timezone: {
        maxLength: number;
        minLength: number;
    };
};
declare const CronSchedule: {
    schedule: {
        maxLength: number;
        minLength: number;
    };
    timezone: {
        maxLength: number;
        minLength: number;
    };
};
declare const ListJobDefinitionsRequest: {
    page: {
        greaterThanOrEqual: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListJobRunsRequest: {
    page: {
        greaterThanOrEqual: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const StartJobDefinitionRequest: {
    replicas: {
        greaterThan: number;
    };
};
declare const UpdateJobDefinitionRequest: {
    cpuLimit: {
        greaterThan: number;
    };
    imageUri: {
        pattern: RegExp;
    };
    localStorageCapacity: {
        greaterThan: number;
    };
    memoryLimit: {
        greaterThan: number;
    };
    name: {
        pattern: RegExp;
    };
};
declare const UpdateJobDefinitionRequestCronScheduleConfig: {
    schedule: {
        maxLength: number;
        minLength: number;
    };
    timezone: {
        maxLength: number;
        minLength: number;
    };
};

declare const validationRules_gen$5_CreateJobDefinitionRequest: typeof CreateJobDefinitionRequest;
declare const validationRules_gen$5_CreateJobDefinitionRequestCronScheduleConfig: typeof CreateJobDefinitionRequestCronScheduleConfig;
declare const validationRules_gen$5_CronSchedule: typeof CronSchedule;
declare const validationRules_gen$5_ListJobDefinitionsRequest: typeof ListJobDefinitionsRequest;
declare const validationRules_gen$5_ListJobRunsRequest: typeof ListJobRunsRequest;
declare const validationRules_gen$5_StartJobDefinitionRequest: typeof StartJobDefinitionRequest;
declare const validationRules_gen$5_UpdateJobDefinitionRequest: typeof UpdateJobDefinitionRequest;
declare const validationRules_gen$5_UpdateJobDefinitionRequestCronScheduleConfig: typeof UpdateJobDefinitionRequestCronScheduleConfig;
declare namespace validationRules_gen$5 {
  export { validationRules_gen$5_CreateJobDefinitionRequest as CreateJobDefinitionRequest, validationRules_gen$5_CreateJobDefinitionRequestCronScheduleConfig as CreateJobDefinitionRequestCronScheduleConfig, validationRules_gen$5_CronSchedule as CronSchedule, validationRules_gen$5_ListJobDefinitionsRequest as ListJobDefinitionsRequest, validationRules_gen$5_ListJobRunsRequest as ListJobRunsRequest, validationRules_gen$5_StartJobDefinitionRequest as StartJobDefinitionRequest, validationRules_gen$5_UpdateJobDefinitionRequest as UpdateJobDefinitionRequest, validationRules_gen$5_UpdateJobDefinitionRequestCronScheduleConfig as UpdateJobDefinitionRequestCronScheduleConfig };
}

type index_gen$f_DeleteJobDefinitionRequest = DeleteJobDefinitionRequest;
type index_gen$f_GetJobDefinitionRequest = GetJobDefinitionRequest;
type index_gen$f_GetJobRunRequest = GetJobRunRequest;
declare const index_gen$f_JOB_RUN_TRANSIENT_STATUSES: typeof JOB_RUN_TRANSIENT_STATUSES;
type index_gen$f_JobDefinition = JobDefinition;
type index_gen$f_JobRun = JobRun;
type index_gen$f_JobRunState = JobRunState;
type index_gen$f_ListJobDefinitionsRequestOrderBy = ListJobDefinitionsRequestOrderBy;
type index_gen$f_ListJobDefinitionsResponse = ListJobDefinitionsResponse;
type index_gen$f_ListJobRunsRequestOrderBy = ListJobRunsRequestOrderBy;
type index_gen$f_ListJobRunsResponse = ListJobRunsResponse;
type index_gen$f_StartJobDefinitionResponse = StartJobDefinitionResponse;
type index_gen$f_StopJobRunRequest = StopJobRunRequest;
declare namespace index_gen$f {
  export { API$f as API, type CreateJobDefinitionRequest$1 as CreateJobDefinitionRequest, type CreateJobDefinitionRequestCronScheduleConfig$1 as CreateJobDefinitionRequestCronScheduleConfig, type CronSchedule$1 as CronSchedule, type index_gen$f_DeleteJobDefinitionRequest as DeleteJobDefinitionRequest, type index_gen$f_GetJobDefinitionRequest as GetJobDefinitionRequest, type index_gen$f_GetJobRunRequest as GetJobRunRequest, index_gen$f_JOB_RUN_TRANSIENT_STATUSES as JOB_RUN_TRANSIENT_STATUSES, type index_gen$f_JobDefinition as JobDefinition, type index_gen$f_JobRun as JobRun, type index_gen$f_JobRunState as JobRunState, type ListJobDefinitionsRequest$1 as ListJobDefinitionsRequest, type index_gen$f_ListJobDefinitionsRequestOrderBy as ListJobDefinitionsRequestOrderBy, type index_gen$f_ListJobDefinitionsResponse as ListJobDefinitionsResponse, type ListJobRunsRequest$1 as ListJobRunsRequest, type index_gen$f_ListJobRunsRequestOrderBy as ListJobRunsRequestOrderBy, type index_gen$f_ListJobRunsResponse as ListJobRunsResponse, type StartJobDefinitionRequest$1 as StartJobDefinitionRequest, type index_gen$f_StartJobDefinitionResponse as StartJobDefinitionResponse, type index_gen$f_StopJobRunRequest as StopJobRunRequest, type UpdateJobDefinitionRequest$1 as UpdateJobDefinitionRequest, type UpdateJobDefinitionRequestCronScheduleConfig$1 as UpdateJobDefinitionRequestCronScheduleConfig, validationRules_gen$5 as ValidationRules };
}

declare namespace index$g {
  export { index_gen$f as v1alpha1 };
}

type AutoscalerEstimator = 'unknown_estimator' | 'binpacking';
type AutoscalerExpander = 'unknown_expander' | 'random' | 'most_pods' | 'least_waste' | 'priority' | 'price';
type CNI = 'unknown_cni' | 'cilium' | 'calico' | 'weave' | 'flannel' | 'kilo';
type ClusterStatus$1 = 'unknown' | 'creating' | 'ready' | 'deleting' | 'deleted' | 'updating' | 'locked' | 'pool_required';
type ClusterTypeAvailability = 'available' | 'scarce' | 'shortage';
type ClusterTypeResiliency = 'unknown_resiliency' | 'standard' | 'high_availability';
type ListClustersRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc' | 'status_asc' | 'status_desc' | 'version_asc' | 'version_desc';
type ListNodesRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListPoolsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc' | 'status_asc' | 'status_desc' | 'version_asc' | 'version_desc';
type MaintenanceWindowDayOfTheWeek = 'any' | 'monday' | 'tuesday' | 'wednesday' | 'thursday' | 'friday' | 'saturday' | 'sunday';
type NodeStatus = 'unknown' | 'creating' | 'not_ready' | 'ready' | 'deleting' | 'deleted' | 'locked' | 'rebooting' | 'creation_error' | 'upgrading' | 'starting' | 'registering';
type PoolStatus = 'unknown' | 'ready' | 'deleting' | 'deleted' | 'scaling' | 'warning' | 'locked' | 'upgrading';
type PoolVolumeType = 'default_volume_type' | 'l_ssd' | 'b_ssd';
type Runtime = 'unknown_runtime' | 'docker' | 'containerd' | 'crio';
interface MaintenanceWindow$1 {
    /** Start time of the two-hour maintenance window. */
    startHour: number;
    /** Day of the week for the maintenance window. */
    day: MaintenanceWindowDayOfTheWeek;
}
interface PoolUpgradePolicy {
    maxUnavailable: number;
    maxSurge: number;
}
interface CreateClusterRequestPoolConfigUpgradePolicy {
    /** The maximum number of nodes that can be not ready at the same time. */
    maxUnavailable?: number;
    /** The maximum number of nodes to be created during the upgrade. */
    maxSurge?: number;
}
interface ClusterAutoUpgrade {
    /** Defines whether auto upgrade is enabled for the cluster. */
    enabled: boolean;
    /** Maintenance window of the cluster auto upgrades. */
    maintenanceWindow?: MaintenanceWindow$1;
}
interface ClusterAutoscalerConfig {
    /** Disable the cluster autoscaler. */
    scaleDownDisabled: boolean;
    /** How long after scale up that scale down evaluation resumes. */
    scaleDownDelayAfterAdd: string;
    /** Type of resource estimator to be used in scale up. */
    estimator: AutoscalerEstimator;
    /** Type of node group expander to be used in scale up. */
    expander: AutoscalerExpander;
    /**
     * Ignore DaemonSet pods when calculating resource utilization for scaling
     * down.
     */
    ignoreDaemonsetsUtilization: boolean;
    /** Detect similar node groups and balance the number of nodes between them. */
    balanceSimilarNodeGroups: boolean;
    /**
     * Pods with priority below cutoff will be expendable. They can be killed
     * without any consideration during scale down and they won't cause scale up.
     * Pods with null priority (PodPriority disabled) are non expendable.
     */
    expendablePodsPriorityCutoff: number;
    /** How long a node should be unneeded before it is eligible to be scaled down. */
    scaleDownUnneededTime: string;
    /**
     * Node utilization level, defined as a sum of requested resources divided by
     * capacity, below which a node can be considered for scale down.
     */
    scaleDownUtilizationThreshold: number;
    /**
     * Maximum number of seconds the cluster autoscaler waits for pod termination
     * when trying to scale down a node.
     */
    maxGracefulTerminationSec: number;
}
interface ClusterOpenIDConnectConfig {
    /**
     * URL of the provider which allows the API server to discover public signing
     * keys. Only URLs using the `https://` scheme are accepted. This is typically
     * the provider's discovery URL without a path, for example
     * "https://accounts.google.com" or "https://login.salesforce.com".
     */
    issuerUrl: string;
    /** A client ID that all tokens must be issued for. */
    clientId: string;
    /**
     * JWT claim to use as the user name. The default is `sub`, which is expected
     * to be the end user's unique identifier. Admins can choose other claims,
     * such as `email` or `name`, depending on their provider. However, claims
     * other than `email` will be prefixed with the issuer URL to prevent name
     * collision.
     */
    usernameClaim: string;
    /**
     * Prefix prepended to username claims to prevent name collision (such as
     * `system:` users). For example, the value `oidc:` will create usernames like
     * `oidc:jane.doe`. If this flag is not provided and `username_claim` is a
     * value other than `email`, the prefix defaults to `( Issuer URL )#` where `(
     * Issuer URL )` is the value of `issuer_url`. The value `-` can be used to
     * disable all prefixing.
     */
    usernamePrefix: string;
    /** JWT claim to use as the user's group. */
    groupsClaim: string[];
    /**
     * Prefix prepended to group claims to prevent name collision (such as
     * `system:` groups). For example, the value `oidc:` will create group names
     * like `oidc:engineering` and `oidc:infra`.
     */
    groupsPrefix: string;
    /**
     * Multiple key=value pairs describing a required claim in the ID token. If
     * set, the claims are verified to be present in the ID token with a matching
     * value.
     */
    requiredClaim: string[];
}
interface Pool {
    /** Pool ID. */
    id: string;
    /** Cluster ID of the pool. */
    clusterId: string;
    /** Date on which the pool was created. */
    createdAt?: Date;
    /** Date on which the pool was last updated. */
    updatedAt?: Date;
    /** Pool name. */
    name: string;
    /** Pool status. */
    status: PoolStatus;
    /** Pool version. */
    version: string;
    /**
     * Node type is the type of Scaleway Instance wanted for the pool. Nodes with
     * insufficient memory are not eligible (DEV1-S, PLAY2-PICO, STARDUST).
     * 'external' is a special node type used to provision instances from other
     * cloud providers in a Kosmos Cluster.
     */
    nodeType: string;
    /** Defines whether the autoscaling feature is enabled for the pool. */
    autoscaling: boolean;
    /** Size (number of nodes) of the pool. */
    size: number;
    /**
     * Defines the minimum size of the pool. Note that this field is only used
     * when autoscaling is enabled on the pool.
     */
    minSize: number;
    /**
     * Defines the maximum size of the pool. Note that this field is only used
     * when autoscaling is enabled on the pool.
     */
    maxSize: number;
    /**
     * Customization of the container runtime is available for each pool. Note
     * that `docker` has been deprecated since version 1.20 and will be removed by
     * version 1.24.
     */
    containerRuntime: Runtime;
    /** Defines whether the autohealing feature is enabled for the pool. */
    autohealing: boolean;
    /** Tags associated with the pool. */
    tags: string[];
    /** Placement group ID in which all the nodes of the pool will be created. */
    placementGroupId?: string;
    /**
     * Kubelet arguments to be used by this pool. Note that this feature is
     * experimental.
     */
    kubeletArgs: Record<string, string>;
    /** Pool upgrade policy. */
    upgradePolicy?: PoolUpgradePolicy;
    /** Zone in which the pool's nodes will be spawned. */
    zone: Zone;
    /**
     * Defines the system volume disk type. Two different types of volume
     * (`volume_type`) are provided: `l_ssd` is a local block storage which means
     * your system is stored locally on your node's hypervisor. `b_ssd` is a
     * remote block storage which means your system is stored on a centralized and
     * resilient cluster.
     */
    rootVolumeType: PoolVolumeType;
    /** System volume disk size. */
    rootVolumeSize?: number;
    /**
     * Defines if the public IP should be removed from Nodes. To use this feature,
     * your Cluster must have an attached Private Network set up with a Public
     * Gateway.
     */
    publicIpDisabled: boolean;
    /** Cluster region of the pool. */
    region: Region;
}
interface CreateClusterRequestAutoUpgrade {
    /** Defines whether auto upgrade is enabled for the cluster. */
    enable: boolean;
    /** Maintenance window of the cluster auto upgrades. */
    maintenanceWindow?: MaintenanceWindow$1;
}
interface CreateClusterRequestAutoscalerConfig$1 {
    /** Disable the cluster autoscaler. */
    scaleDownDisabled?: boolean;
    /** How long after scale up that scale down evaluation resumes. */
    scaleDownDelayAfterAdd?: string;
    /** Type of resource estimator to be used in scale up. */
    estimator: AutoscalerEstimator;
    /** Type of node group expander to be used in scale up. */
    expander: AutoscalerExpander;
    /**
     * Ignore DaemonSet pods when calculating resource utilization for scaling
     * down.
     */
    ignoreDaemonsetsUtilization?: boolean;
    /** Detect similar node groups and balance the number of nodes between them. */
    balanceSimilarNodeGroups?: boolean;
    /**
     * Pods with priority below cutoff will be expendable. They can be killed
     * without any consideration during scale down and they won't cause scale up.
     * Pods with null priority (PodPriority disabled) are non expendable.
     */
    expendablePodsPriorityCutoff?: number;
    /** How long a node should be unneeded before it is eligible to be scaled down. */
    scaleDownUnneededTime?: string;
    /**
     * Node utilization level, defined as a sum of requested resources divided by
     * capacity, below which a node can be considered for scale down.
     */
    scaleDownUtilizationThreshold?: number;
    /**
     * Maximum number of seconds the cluster autoscaler waits for pod termination
     * when trying to scale down a node.
     */
    maxGracefulTerminationSec?: number;
}
interface CreateClusterRequestOpenIDConnectConfig$1 {
    /**
     * URL of the provider which allows the API server to discover public signing
     * keys. Only URLs using the `https://` scheme are accepted. This is typically
     * the provider's discovery URL without a path, for example
     * "https://accounts.google.com" or "https://login.salesforce.com".
     */
    issuerUrl: string;
    /** A client ID that all tokens must be issued for. */
    clientId: string;
    /**
     * JWT claim to use as the user name. The default is `sub`, which is expected
     * to be the end user's unique identifier. Admins can choose other claims,
     * such as `email` or `name`, depending on their provider. However, claims
     * other than `email` will be prefixed with the issuer URL to prevent name
     * collision.
     */
    usernameClaim?: string;
    /**
     * Prefix prepended to username claims to prevent name collision (such as
     * `system:` users). For example, the value `oidc:` will create usernames like
     * `oidc:jane.doe`. If this flag is not provided and `username_claim` is a
     * value other than `email`, the prefix defaults to `( Issuer URL )#` where `(
     * Issuer URL )` is the value of `issuer_url`. The value `-` can be used to
     * disable all prefixing.
     */
    usernamePrefix?: string;
    /** JWT claim to use as the user's group. */
    groupsClaim?: string[];
    /**
     * Prefix prepended to group claims to prevent name collision (such as
     * `system:` groups). For example, the value `oidc:` will create group names
     * like `oidc:engineering` and `oidc:infra`.
     */
    groupsPrefix?: string;
    /**
     * Multiple key=value pairs describing a required claim in the ID token. If
     * set, the claims are verified to be present in the ID token with a matching
     * value.
     */
    requiredClaim?: string[];
}
interface CreateClusterRequestPoolConfig$1 {
    /** Name of the pool. */
    name: string;
    /**
     * Node type is the type of Scaleway Instance wanted for the pool. Nodes with
     * insufficient memory are not eligible (DEV1-S, PLAY2-PICO, STARDUST).
     * 'external' is a special node type used to provision instances from other
     * cloud providers in a Kosmos Cluster.
     */
    nodeType: string;
    /** Placement group ID in which all the nodes of the pool will be created. */
    placementGroupId?: string;
    /** Defines whether the autoscaling feature is enabled for the pool. */
    autoscaling: boolean;
    /** Size (number of nodes) of the pool. */
    size: number;
    /**
     * Defines the minimum size of the pool. Note that this field is only used
     * when autoscaling is enabled on the pool.
     */
    minSize?: number;
    /**
     * Defines the maximum size of the pool. Note that this field is only used
     * when autoscaling is enabled on the pool.
     */
    maxSize?: number;
    /**
     * Customization of the container runtime is available for each pool. Note
     * that `docker` has been deprecated since version 1.20 and will be removed by
     * version 1.24.
     */
    containerRuntime: Runtime;
    /** Defines whether the autohealing feature is enabled for the pool. */
    autohealing: boolean;
    /** Tags associated with the pool. */
    tags: string[];
    /**
     * Kubelet arguments to be used by this pool. Note that this feature is
     * experimental.
     */
    kubeletArgs: Record<string, string>;
    /** Pool upgrade policy. */
    upgradePolicy?: CreateClusterRequestPoolConfigUpgradePolicy;
    /** Zone in which the pool's nodes will be spawned. */
    zone: Zone;
    /**
     * Defines the system volume disk type. Two different types of volume
     * (`volume_type`) are provided: `l_ssd` is a local block storage which means
     * your system is stored locally on your node's hypervisor. `b_ssd` is a
     * remote block storage which means your system is stored on a centralized and
     * resilient cluster.
     */
    rootVolumeType: PoolVolumeType;
    /** System volume disk size. */
    rootVolumeSize?: number;
    /**
     * Defines if the public IP should be removed from Nodes. To use this feature,
     * your Cluster must have an attached Private Network set up with a Public
     * Gateway.
     */
    publicIpDisabled: boolean;
}
interface CreatePoolRequestUpgradePolicy {
    maxUnavailable?: number;
    maxSurge?: number;
}
interface ExternalNodeCoreV1Taint {
    key: string;
    value: string;
    effect: string;
}
interface ClusterType {
    /** Cluster type name. */
    name: string;
    /** Cluster type availability. */
    availability: ClusterTypeAvailability;
    /** Maximum number of nodes supported by the offer. */
    maxNodes: number;
    /** Time period during which you can no longer switch to a lower offer. */
    commitmentDelay?: string;
    /** Value of the Service Level Agreement of the offer. */
    sla: number;
    /** Resiliency offered by the offer. */
    resiliency: ClusterTypeResiliency;
    /** Max RAM allowed for the control plane. */
    memory: number;
    /** Returns information if this offer uses dedicated resources. */
    dedicated: boolean;
    /**
     * True if the offer allows activation of the audit log functionality. Please
     * note that audit logs are sent to Cockpit.
     */
    auditLogsSupported: boolean;
}
interface Version$2 {
    /** Name of the Kubernetes version. */
    name: string;
    /** Label of the Kubernetes version. */
    label: string;
    /** Region in which this version is available. */
    region: Region;
    /** Supported Container Network Interface (CNI) plugins for this version. */
    availableCnis: CNI[];
    /** Supported container runtimes for this version. */
    availableContainerRuntimes: Runtime[];
    /** Supported feature gates for this version. */
    availableFeatureGates: string[];
    /** Supported admission plugins for this version. */
    availableAdmissionPlugins: string[];
    /** Supported kubelet arguments for this version. */
    availableKubeletArgs: Record<string, string>;
}
interface Cluster$1 {
    /** Cluster ID. */
    id: string;
    /** Cluster type. */
    type: string;
    /** Cluster name. */
    name: string;
    /** Status of the cluster. */
    status: ClusterStatus$1;
    /** Kubernetes version of the cluster. */
    version: string;
    /** Region in which the cluster is deployed. */
    region: Region;
    /** ID of the Organization owning the cluster. */
    organizationId: string;
    /** ID of the Project owning the cluster. */
    projectId: string;
    /** Tags associated with the cluster. */
    tags: string[];
    /** Container Network Interface (CNI) plugin running in the cluster. */
    cni: CNI;
    /** Cluster description. */
    description: string;
    /** Kubernetes API server URL of the cluster. */
    clusterUrl: string;
    /** Wildcard DNS resolving all the ready cluster nodes. */
    dnsWildcard: string;
    /** Date on which the cluster was created. */
    createdAt?: Date;
    /** Date on which the cluster was last updated. */
    updatedAt?: Date;
    /** Autoscaler config for the cluster. */
    autoscalerConfig?: ClusterAutoscalerConfig;
    /** Auto upgrade configuration of the cluster. */
    autoUpgrade?: ClusterAutoUpgrade;
    /** Defines whether a new Kubernetes version is available. */
    upgradeAvailable: boolean;
    /** List of enabled feature gates. */
    featureGates: string[];
    /** List of enabled admission plugins. */
    admissionPlugins: string[];
    /**
     * This configuration enables to update the OpenID Connect configuration of
     * the Kubernetes API server.
     */
    openIdConnectConfig?: ClusterOpenIDConnectConfig;
    /**
     * Additional Subject Alternative Names for the Kubernetes API server
     * certificate.
     */
    apiserverCertSans: string[];
    /** Private network ID for internal cluster communication. */
    privateNetworkId?: string;
    /** Date on which it will be possible to switch to a smaller offer. */
    commitmentEndsAt?: Date;
}
interface Node {
    /** Node ID. */
    id: string;
    /** Pool ID of the node. */
    poolId: string;
    /** Cluster ID of the node. */
    clusterId: string;
    /**
     * Underlying instance ID. It is prefixed by instance type and location
     * information (see
     * https://pkg.go.dev/k8s.io/api/core/v1#NodeSpec.ProviderID).
     */
    providerId: string;
    /** Cluster region of the node. */
    region: Region;
    /** Name of the node. */
    name: string;
    /** @deprecated Public IPv4 address of the node. */
    publicIpV4?: string;
    /** @deprecated Public IPv6 address of the node. */
    publicIpV6?: string;
    /**
     * @deprecated Conditions of the node. These conditions contain the Node
     *   Problem Detector conditions, as well as some in house conditions.
     */
    conditions?: Record<string, string>;
    /** Status of the node. */
    status: NodeStatus;
    /** Details of the error, if any occurred when managing the node. */
    errorMessage?: string;
    /** Date on which the node was created. */
    createdAt?: Date;
    /** Date on which the node was last updated. */
    updatedAt?: Date;
}
interface UpdateClusterRequestAutoUpgrade {
    /** Defines whether auto upgrade is enabled for the cluster. */
    enable?: boolean;
    /** Maintenance window of the cluster auto upgrades. */
    maintenanceWindow?: MaintenanceWindow$1;
}
interface UpdateClusterRequestAutoscalerConfig$1 {
    /** Disable the cluster autoscaler. */
    scaleDownDisabled?: boolean;
    /** How long after scale up that scale down evaluation resumes. */
    scaleDownDelayAfterAdd?: string;
    /** Type of resource estimator to be used in scale up. */
    estimator: AutoscalerEstimator;
    /** Type of node group expander to be used in scale up. */
    expander: AutoscalerExpander;
    /**
     * Ignore DaemonSet pods when calculating resource utilization for scaling
     * down.
     */
    ignoreDaemonsetsUtilization?: boolean;
    /** Detect similar node groups and balance the number of nodes between them. */
    balanceSimilarNodeGroups?: boolean;
    /**
     * Pods with priority below cutoff will be expendable. They can be killed
     * without any consideration during scale down and they won't cause scale up.
     * Pods with null priority (PodPriority disabled) are non expendable.
     */
    expendablePodsPriorityCutoff?: number;
    /** How long a node should be unneeded before it is eligible to be scaled down. */
    scaleDownUnneededTime?: string;
    /**
     * Node utilization level, defined as a sum of requested resources divided by
     * capacity, below which a node can be considered for scale down.
     */
    scaleDownUtilizationThreshold?: number;
    /**
     * Maximum number of seconds the cluster autoscaler waits for pod termination
     * when trying to scale down a node.
     */
    maxGracefulTerminationSec?: number;
}
interface UpdateClusterRequestOpenIDConnectConfig {
    /**
     * URL of the provider which allows the API server to discover public signing
     * keys. Only URLs using the `https://` scheme are accepted. This is typically
     * the provider's discovery URL without a path, for example
     * "https://accounts.google.com" or "https://login.salesforce.com".
     */
    issuerUrl?: string;
    /** A client ID that all tokens must be issued for. */
    clientId?: string;
    /**
     * JWT claim to use as the user name. The default is `sub`, which is expected
     * to be the end user's unique identifier. Admins can choose other claims,
     * such as `email` or `name`, depending on their provider. However, claims
     * other than `email` will be prefixed with the issuer URL to prevent name
     * collision.
     */
    usernameClaim?: string;
    /**
     * Prefix prepended to username claims to prevent name collision (such as
     * `system:` users). For example, the value `oidc:` will create usernames like
     * `oidc:jane.doe`. If this flag is not provided and `username_claim` is a
     * value other than `email`, the prefix defaults to `( Issuer URL )#` where `(
     * Issuer URL )` is the value of `issuer_url`. The value `-` can be used to
     * disable all prefixing.
     */
    usernamePrefix?: string;
    /** JWT claim to use as the user's group. */
    groupsClaim?: string[];
    /**
     * Prefix prepended to group claims to prevent name collision (such as
     * `system:` groups). For example, the value `oidc:` will create group names
     * like `oidc:engineering` and `oidc:infra`.
     */
    groupsPrefix?: string;
    /**
     * Multiple key=value pairs describing a required claim in the ID token. If
     * set, the claims are verified to be present in the ID token with a matching
     * value.
     */
    requiredClaim?: string[];
}
interface UpdatePoolRequestUpgradePolicy {
    maxUnavailable?: number;
    maxSurge?: number;
}
type CreateClusterRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * @deprecated Organization ID in which the cluster will be created.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Project ID in which the cluster will be created.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
    /**
     * Type of the cluster (possible values are kapsule, multicloud,
     * kapsule-dedicated-8, kapsule-dedicated-16).
     */
    type: string;
    /** Cluster name. */
    name?: string;
    /** Cluster description. */
    description: string;
    /** Tags associated with the cluster. */
    tags?: string[];
    /** Kubernetes version of the cluster. */
    version: string;
    /** Container Network Interface (CNI) plugin running in the cluster. */
    cni: CNI;
    /** Pools created along with the cluster. */
    pools?: CreateClusterRequestPoolConfig$1[];
    /**
     * Autoscaler configuration for the cluster. It allows you to set (to an
     * extent) your preferred autoscaler configuration, which is an implementation
     * of the cluster-autoscaler
     * (https://github.com/kubernetes/autoscaler/tree/master/cluster-autoscaler/).
     */
    autoscalerConfig?: CreateClusterRequestAutoscalerConfig$1;
    /**
     * Auto upgrade configuration of the cluster. This configuration enables to
     * set a specific 2-hour time window in which the cluster can be automatically
     * updated to the latest patch version.
     */
    autoUpgrade?: CreateClusterRequestAutoUpgrade;
    /** List of feature gates to enable. */
    featureGates?: string[];
    /** List of admission plugins to enable. */
    admissionPlugins?: string[];
    /**
     * OpenID Connect configuration of the cluster. This configuration enables to
     * update the OpenID Connect configuration of the Kubernetes API server.
     */
    openIdConnectConfig?: CreateClusterRequestOpenIDConnectConfig$1;
    /**
     * Additional Subject Alternative Names for the Kubernetes API server
     * certificate.
     */
    apiserverCertSans?: string[];
    /**
     * Private network ID for internal cluster communication (cannot be changed
     * later).
     */
    privateNetworkId?: string;
};
type CreateExternalNodeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    poolId: string;
};
type CreatePoolRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Cluster ID to which the pool will be attached. */
    clusterId: string;
    /** Pool name. */
    name?: string;
    /**
     * Node type is the type of Scaleway Instance wanted for the pool. Nodes with
     * insufficient memory are not eligible (DEV1-S, PLAY2-PICO, STARDUST).
     * 'external' is a special node type used to provision instances from other
     * cloud providers in a Kosmos Cluster.
     */
    nodeType: string;
    /** Placement group ID in which all the nodes of the pool will be created. */
    placementGroupId?: string;
    /** Defines whether the autoscaling feature is enabled for the pool. */
    autoscaling: boolean;
    /** Size (number of nodes) of the pool. */
    size: number;
    /**
     * Defines the minimum size of the pool. Note that this field is only used
     * when autoscaling is enabled on the pool.
     */
    minSize?: number;
    /**
     * Defines the maximum size of the pool. Note that this field is only used
     * when autoscaling is enabled on the pool.
     */
    maxSize?: number;
    /**
     * Customization of the container runtime is available for each pool. Note
     * that `docker` has been deprecated since version 1.20 and will be removed by
     * version 1.24.
     */
    containerRuntime?: Runtime;
    /** Defines whether the autohealing feature is enabled for the pool. */
    autohealing: boolean;
    /** Tags associated with the pool. */
    tags?: string[];
    /**
     * Kubelet arguments to be used by this pool. Note that this feature is
     * experimental.
     */
    kubeletArgs?: Record<string, string>;
    /** Pool upgrade policy. */
    upgradePolicy?: CreatePoolRequestUpgradePolicy;
    /** Zone in which the pool's nodes will be spawned. */
    zone?: Zone;
    /**
     * Defines the system volume disk type. Two different types of volume
     * (`volume_type`) are provided: `l_ssd` is a local block storage which means
     * your system is stored locally on your node's hypervisor. `b_ssd` is a
     * remote block storage which means your system is stored on a centralized and
     * resilient cluster.
     */
    rootVolumeType?: PoolVolumeType;
    /** System volume disk size. */
    rootVolumeSize?: number;
    /**
     * Defines if the public IP should be removed from Nodes. To use this feature,
     * your Cluster must have an attached Private Network set up with a Public
     * Gateway.
     */
    publicIpDisabled: boolean;
};
type DeleteClusterRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the cluster to delete. */
    clusterId: string;
    /**
     * Defines whether all volumes (including retain volume type), empty Private
     * Networks and Load Balancers with a name starting with the cluster ID will
     * also be deleted.
     */
    withAdditionalResources: boolean;
};
type DeleteNodeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the node to replace. */
    nodeId: string;
    /**
     * Skip draining node from its workload (Note: this parameter is currently
     * inactive).
     */
    skipDrain: boolean;
    /** Add a new node after the deletion of this node. */
    replace: boolean;
};
type DeletePoolRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the pool to delete. */
    poolId: string;
};
interface ExternalNode {
    id: string;
    name: string;
    clusterUrl: string;
    poolVersion: string;
    clusterCa: string;
    kubeToken: string;
    kubeletConfig: string;
    externalIp: string;
    containerdVersion: string;
    runcVersion: string;
    cniPluginsVersion: string;
    nodeLabels: Record<string, string>;
    nodeTaints: ExternalNodeCoreV1Taint[];
}
type GetClusterKubeConfigRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Cluster ID for which to download the kubeconfig. */
    clusterId: string;
    /** Hide the legacy token from the kubeconfig. */
    redacted?: boolean;
};
type GetClusterRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the requested cluster. */
    clusterId: string;
};
type GetNodeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the requested node. */
    nodeId: string;
};
type GetPoolRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the requested pool. */
    poolId: string;
};
type GetVersionRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Requested version name. */
    versionName: string;
};
type ListClusterAvailableTypesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Cluster ID for which the available Kubernetes types will be listed. */
    clusterId: string;
};
interface ListClusterAvailableTypesResponse {
    /** Available cluster types for the cluster. */
    clusterTypes: ClusterType[];
    /** Total number of types. */
    totalCount: number;
}
type ListClusterAvailableVersionsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Cluster ID for which the available Kubernetes versions will be listed. */
    clusterId: string;
};
interface ListClusterAvailableVersionsResponse {
    /** Available Kubernetes versions for the cluster. */
    versions: Version$2[];
}
type ListClusterTypesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Page number, from the paginated results, to return for cluster-types. */
    page?: number;
    /** Maximum number of clusters per page. */
    pageSize?: number;
};
interface ListClusterTypesResponse {
    /** Total number of cluster-types. */
    totalCount: number;
    /** Paginated returned cluster-types. */
    clusterTypes: ClusterType[];
}
type ListClustersRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Organization ID on which to filter the returned clusters. */
    organizationId?: string;
    /** Project ID on which to filter the returned clusters. */
    projectId?: string;
    /** Sort order of returned clusters. */
    orderBy?: ListClustersRequestOrderBy$1;
    /** Page number to return for clusters, from the paginated results. */
    page?: number;
    /** Maximum number of clusters per page. */
    pageSize?: number;
    /**
     * Name to filter on, only clusters containing this substring in their name
     * will be returned.
     */
    name?: string;
    /** Status to filter on, only clusters with this status will be returned. */
    status?: ClusterStatus$1;
    /** Type to filter on, only clusters with this type will be returned. */
    type?: string;
    /**
     * Private Network ID to filter on, only clusters within this Private Network
     * will be returned.
     */
    privateNetworkId?: string;
};
interface ListClustersResponse$1 {
    /** Total number of clusters. */
    totalCount: number;
    /** Paginated returned clusters. */
    clusters: Cluster$1[];
}
type ListNodesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Cluster ID from which the nodes will be listed from. */
    clusterId: string;
    /** Pool ID on which to filter the returned nodes. */
    poolId?: string;
    /** Sort order of the returned nodes. */
    orderBy?: ListNodesRequestOrderBy;
    /** Page number for the returned nodes. */
    page?: number;
    /** Maximum number of nodes per page. */
    pageSize?: number;
    /**
     * Name to filter on, only nodes containing this substring in their name will
     * be returned.
     */
    name?: string;
    /** Status to filter on, only nodes with this status will be returned. */
    status?: NodeStatus;
};
interface ListNodesResponse {
    /** Total number of nodes. */
    totalCount: number;
    /** Paginated returned nodes. */
    nodes: Node[];
}
type ListPoolsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the cluster whose pools will be listed. */
    clusterId: string;
    /** Sort order of returned pools. */
    orderBy?: ListPoolsRequestOrderBy;
    /** Page number for the returned pools. */
    page?: number;
    /** Maximum number of pools per page. */
    pageSize?: number;
    /**
     * Name to filter on, only pools containing this substring in their name will
     * be returned.
     */
    name?: string;
    /** Status to filter on, only pools with this status will be returned. */
    status?: PoolStatus;
};
interface ListPoolsResponse {
    /** Total number of pools that exists for the cluster. */
    totalCount: number;
    /** Paginated returned pools. */
    pools: Pool[];
}
type ListVersionsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
};
interface ListVersionsResponse$1 {
    /** Available Kubernetes versions. */
    versions: Version$2[];
}
type MigrateToPrivateNetworkClusterRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the cluster to migrate. */
    clusterId: string;
    /** ID of the Private Network to link to the cluster. */
    privateNetworkId: string;
};
type RebootNodeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the node to reboot. */
    nodeId: string;
};
type ReplaceNodeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the node to replace. */
    nodeId: string;
};
type ResetClusterAdminTokenRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Cluster ID on which the admin token will be renewed. */
    clusterId: string;
};
type SetClusterTypeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the cluster to migrate from one type to another. */
    clusterId: string;
    /**
     * Type of the cluster. Note that some migrations are not possible (please
     * refer to product documentation).
     */
    type: string;
};
type UpdateClusterRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the cluster to update. */
    clusterId: string;
    /** New external name for the cluster. */
    name?: string;
    /** New description for the cluster. */
    description?: string;
    /** New tags associated with the cluster. */
    tags?: string[];
    /** New autoscaler config for the cluster. */
    autoscalerConfig?: UpdateClusterRequestAutoscalerConfig$1;
    /**
     * New auto upgrade configuration for the cluster. Note that all fields need
     * to be set.
     */
    autoUpgrade?: UpdateClusterRequestAutoUpgrade;
    /** List of feature gates to enable. */
    featureGates?: string[];
    /** List of admission plugins to enable. */
    admissionPlugins?: string[];
    /**
     * OpenID Connect configuration of the cluster. This configuration enables to
     * update the OpenID Connect configuration of the Kubernetes API server.
     */
    openIdConnectConfig?: UpdateClusterRequestOpenIDConnectConfig;
    /**
     * Additional Subject Alternative Names for the Kubernetes API server
     * certificate.
     */
    apiserverCertSans?: string[];
};
type UpdatePoolRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the pool to update. */
    poolId: string;
    /** New value for the pool autoscaling enablement. */
    autoscaling?: boolean;
    /** New desired pool size. */
    size?: number;
    /** New minimum size for the pool. */
    minSize?: number;
    /** New maximum size for the pool. */
    maxSize?: number;
    /** New value for the pool autohealing enablement. */
    autohealing?: boolean;
    /** New tags associated with the pool. */
    tags?: string[];
    /**
     * New Kubelet arguments to be used by this pool. Note that this feature is
     * experimental.
     */
    kubeletArgs?: Record<string, string>;
    /** New upgrade policy for the pool. */
    upgradePolicy?: UpdatePoolRequestUpgradePolicy;
};
type UpgradeClusterRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the cluster to upgrade. */
    clusterId: string;
    /**
     * New Kubernetes version of the cluster. Note that the version should either
     * be a higher patch version of the same minor version or the direct minor
     * version after the current one.
     */
    version: string;
    /**
     * Defines whether pools will also be upgraded once the control plane is
     * upgraded.
     */
    upgradePools: boolean;
};
type UpgradePoolRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the pool to upgrade. */
    poolId: string;
    /** New Kubernetes version for the pool. */
    version: string;
};

/** Kubernetes API. */
declare class API$e extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListClusters: (request?: Readonly<ListClustersRequest$2>) => Promise<ListClustersResponse$1>;
    /**
     * List Clusters. List all existing Kubernetes clusters in a specific region.
     *
     * @param request - The request {@link ListClustersRequest}
     * @returns A Promise of ListClustersResponse
     */
    listClusters: (request?: Readonly<ListClustersRequest$2>) => Promise<ListClustersResponse$1> & {
        all: () => Promise<Cluster$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Cluster$1[], void, void>;
    };
    /**
     * Create a new Cluster. Create a new Kubernetes cluster in a Scaleway region.
     *
     * @param request - The request {@link CreateClusterRequest}
     * @returns A Promise of Cluster
     */
    createCluster: (request: Readonly<CreateClusterRequest$2>) => Promise<Cluster$1>;
    /**
     * Get a Cluster. Retrieve information about a specific Kubernetes cluster.
     *
     * @param request - The request {@link GetClusterRequest}
     * @returns A Promise of Cluster
     */
    getCluster: (request: Readonly<GetClusterRequest$1>) => Promise<Cluster$1>;
    /**
     * Waits for {@link Cluster} to be in a final state.
     *
     * @param request - The request {@link GetClusterRequest}
     * @param options - The waiting options
     * @returns A Promise of Cluster
     */
    waitForCluster: (request: Readonly<GetClusterRequest$1>, options?: Readonly<WaitForOptions<Cluster$1>>) => Promise<Cluster$1>;
    /**
     * Update a Cluster. Update information on a specific Kubernetes cluster. You
     * can update details such as its name, description, tags and configuration.
     * To upgrade a cluster, you will need to use the dedicated endpoint.
     *
     * @param request - The request {@link UpdateClusterRequest}
     * @returns A Promise of Cluster
     */
    updateCluster: (request: Readonly<UpdateClusterRequest$2>) => Promise<Cluster$1>;
    /**
     * Delete a Cluster. Delete a specific Kubernetes cluster and all its
     * associated pools and nodes. Note that this method will not delete any Load
     * Balancer or Block Volume that are associated with the cluster.
     *
     * @param request - The request {@link DeleteClusterRequest}
     * @returns A Promise of Cluster
     */
    deleteCluster: (request: Readonly<DeleteClusterRequest$1>) => Promise<Cluster$1>;
    /**
     * Upgrade a Cluster. Upgrade a specific Kubernetes cluster and possibly its
     * associated pools to a specific and supported Kubernetes version.
     *
     * @param request - The request {@link UpgradeClusterRequest}
     * @returns A Promise of Cluster
     */
    upgradeCluster: (request: Readonly<UpgradeClusterRequest>) => Promise<Cluster$1>;
    /**
     * Change the Cluster type. Change the type of a specific Kubernetes cluster.
     * To see the possible values you can enter for the `type` field, [list
     * available cluster
     * types](#path-clusters-list-available-cluster-types-for-a-cluster).
     *
     * @param request - The request {@link SetClusterTypeRequest}
     * @returns A Promise of Cluster
     */
    setClusterType: (request: Readonly<SetClusterTypeRequest>) => Promise<Cluster$1>;
    /**
     * List available versions for a Cluster. List the versions that a specific
     * Kubernetes cluster is allowed to upgrade to. Results will include every
     * patch version greater than the current patch, as well as one minor version
     * ahead of the current version. Any upgrade skipping a minor version will not
     * work.
     *
     * @param request - The request {@link ListClusterAvailableVersionsRequest}
     * @returns A Promise of ListClusterAvailableVersionsResponse
     */
    listClusterAvailableVersions: (request: Readonly<ListClusterAvailableVersionsRequest>) => Promise<ListClusterAvailableVersionsResponse>;
    /**
     * List available cluster types for a cluster. List the cluster types that a
     * specific Kubernetes cluster is allowed to switch to.
     *
     * @param request - The request {@link ListClusterAvailableTypesRequest}
     * @returns A Promise of ListClusterAvailableTypesResponse
     */
    listClusterAvailableTypes: (request: Readonly<ListClusterAvailableTypesRequest>) => Promise<ListClusterAvailableTypesResponse>;
    protected _getClusterKubeConfig: (request: Readonly<GetClusterKubeConfigRequest>) => Promise<Blob>;
    /**
     * Reset the admin token of a Cluster. Reset the admin token for a specific
     * Kubernetes cluster. This will revoke the old admin token (which will not be
     * usable afterwards) and create a new one. Note that you will need to
     * download kubeconfig again to keep interacting with the cluster.
     *
     * @param request - The request {@link ResetClusterAdminTokenRequest}
     */
    resetClusterAdminToken: (request: Readonly<ResetClusterAdminTokenRequest>) => Promise<void>;
    /**
     * Migrate an existing cluster to a Private Network cluster. Migrate a cluster
     * that was created before the release of Private Network clusters to a new
     * one with a Private Network.
     *
     * @param request - The request {@link MigrateToPrivateNetworkClusterRequest}
     * @returns A Promise of Cluster
     */
    migrateToPrivateNetworkCluster: (request: Readonly<MigrateToPrivateNetworkClusterRequest>) => Promise<Cluster$1>;
    protected pageOfListPools: (request: Readonly<ListPoolsRequest$1>) => Promise<ListPoolsResponse>;
    /**
     * List Pools in a Cluster. List all the existing pools for a specific
     * Kubernetes cluster.
     *
     * @param request - The request {@link ListPoolsRequest}
     * @returns A Promise of ListPoolsResponse
     */
    listPools: (request: Readonly<ListPoolsRequest$1>) => Promise<ListPoolsResponse> & {
        all: () => Promise<Pool[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Pool[], void, void>;
    };
    /**
     * Create a new Pool in a Cluster. Create a new pool in a specific Kubernetes
     * cluster.
     *
     * @param request - The request {@link CreatePoolRequest}
     * @returns A Promise of Pool
     */
    createPool: (request: Readonly<CreatePoolRequest$1>) => Promise<Pool>;
    /**
     * Get a Pool in a Cluster. Retrieve details about a specific pool in a
     * Kubernetes cluster.
     *
     * @param request - The request {@link GetPoolRequest}
     * @returns A Promise of Pool
     */
    getPool: (request: Readonly<GetPoolRequest>) => Promise<Pool>;
    /**
     * Waits for {@link Pool} to be in a final state.
     *
     * @param request - The request {@link GetPoolRequest}
     * @param options - The waiting options
     * @returns A Promise of Pool
     */
    waitForPool: (request: Readonly<GetPoolRequest>, options?: Readonly<WaitForOptions<Pool>>) => Promise<Pool>;
    /**
     * Upgrade a Pool in a Cluster. Upgrade the Kubernetes version of a specific
     * pool. Note that it only works if the targeted version matches the cluster's
     * version.
     *
     * @param request - The request {@link UpgradePoolRequest}
     * @returns A Promise of Pool
     */
    upgradePool: (request: Readonly<UpgradePoolRequest>) => Promise<Pool>;
    /**
     * Update a Pool in a Cluster. Update the attributes of a specific pool, such
     * as its desired size, autoscaling settings, and tags.
     *
     * @param request - The request {@link UpdatePoolRequest}
     * @returns A Promise of Pool
     */
    updatePool: (request: Readonly<UpdatePoolRequest>) => Promise<Pool>;
    /**
     * Delete a Pool in a Cluster. Delete a specific pool from a cluster. Note
     * that all the pool's nodes will also be deleted.
     *
     * @param request - The request {@link DeletePoolRequest}
     * @returns A Promise of Pool
     */
    deletePool: (request: Readonly<DeletePoolRequest>) => Promise<Pool>;
    /**
     * Create a Kosmos node. Retrieve metadata for a Kosmos node. This method is
     * not intended to be called by end users but rather programmatically by the
     * kapsule-node-agent.
     *
     * @param request - The request {@link CreateExternalNodeRequest}
     * @returns A Promise of ExternalNode
     */
    createExternalNode: (request: Readonly<CreateExternalNodeRequest>) => Promise<ExternalNode>;
    protected pageOfListNodes: (request: Readonly<ListNodesRequest$1>) => Promise<ListNodesResponse>;
    /**
     * List Nodes in a Cluster. List all the existing nodes for a specific
     * Kubernetes cluster.
     *
     * @param request - The request {@link ListNodesRequest}
     * @returns A Promise of ListNodesResponse
     */
    listNodes: (request: Readonly<ListNodesRequest$1>) => Promise<ListNodesResponse> & {
        all: () => Promise<Node[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Node[], void, void>;
    };
    /**
     * Get a Node in a Cluster. Retrieve details about a specific Kubernetes Node.
     *
     * @param request - The request {@link GetNodeRequest}
     * @returns A Promise of Node
     */
    getNode: (request: Readonly<GetNodeRequest>) => Promise<Node>;
    /**
     * Waits for {@link Node} to be in a final state.
     *
     * @param request - The request {@link GetNodeRequest}
     * @param options - The waiting options
     * @returns A Promise of Node
     */
    waitForNode: (request: Readonly<GetNodeRequest>, options?: Readonly<WaitForOptions<Node>>) => Promise<Node>;
    /**
     * Replace a Node in a Cluster. Replace a specific Node. The node will first
     * be cordoned (scheduling will be disabled on it). The existing pods on the
     * node will then be drained and rescheduled onto another schedulable node.
     * Note that when there is not enough space to reschedule all the pods (such
     * as in a one-node cluster), disruption of your applications can be
     * expected.
     *
     * @deprecated
     * @param request - The request {@link ReplaceNodeRequest}
     * @returns A Promise of Node
     */
    replaceNode: (request: Readonly<ReplaceNodeRequest>) => Promise<Node>;
    /**
     * Reboot a Node in a Cluster. Reboot a specific Node. The node will first be
     * cordoned (scheduling will be disabled on it). The existing pods on the node
     * will then be drained and rescheduled onto another schedulable node. Note
     * that when there is not enough space to reschedule all the pods (such as in
     * a one-node cluster), disruption of your applications can be expected.
     *
     * @param request - The request {@link RebootNodeRequest}
     * @returns A Promise of Node
     */
    rebootNode: (request: Readonly<RebootNodeRequest>) => Promise<Node>;
    /**
     * Delete a Node in a Cluster. Delete a specific Node. Note that when there is
     * not enough space to reschedule all the pods (such as in a one-node
     * cluster), disruption of your applications can be expected.
     *
     * @param request - The request {@link DeleteNodeRequest}
     * @returns A Promise of Node
     */
    deleteNode: (request: Readonly<DeleteNodeRequest>) => Promise<Node>;
    /**
     * List all available Versions. List all available versions for the creation
     * of a new Kubernetes cluster.
     *
     * @param request - The request {@link ListVersionsRequest}
     * @returns A Promise of ListVersionsResponse
     */
    listVersions: (request?: Readonly<ListVersionsRequest$1>) => Promise<ListVersionsResponse$1>;
    /**
     * Get a Version. Retrieve a specific Kubernetes version and its details.
     *
     * @param request - The request {@link GetVersionRequest}
     * @returns A Promise of Version
     */
    getVersion: (request: Readonly<GetVersionRequest$1>) => Promise<Version$2>;
    protected pageOfListClusterTypes: (request?: Readonly<ListClusterTypesRequest$1>) => Promise<ListClusterTypesResponse>;
    /**
     * List cluster types. List available cluster types and their technical
     * details.
     *
     * @param request - The request {@link ListClusterTypesRequest}
     * @returns A Promise of ListClusterTypesResponse
     */
    listClusterTypes: (request?: Readonly<ListClusterTypesRequest$1>) => Promise<ListClusterTypesResponse> & {
        all: () => Promise<ClusterType[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ClusterType[], void, void>;
    };
}

declare class K8SUtilsAPI extends API$e {
    /**
     * Get configuration of a kube cluster.
     *
     * @param request - The request {@link GetClusterKubeConfigRequest}
     * @returns A Promise of Blob
     */
    getClusterKubeConfig: (request: Readonly<GetClusterKubeConfigRequest>) => Promise<Blob>;
}

/** Lists transient statutes of the enum {@link ClusterStatus}. */
declare const CLUSTER_TRANSIENT_STATUSES$1: ClusterStatus$1[];
/** Lists transient statutes of the enum {@link NodeStatus}. */
declare const NODE_TRANSIENT_STATUSES: NodeStatus[];
/** Lists transient statutes of the enum {@link PoolStatus}. */
declare const POOL_TRANSIENT_STATUSES: PoolStatus[];

declare const CreateClusterRequest$1: {
    name: {
        minLength: number;
    };
};
declare const CreateClusterRequestAutoscalerConfig: {
    scaleDownUtilizationThreshold: {
        greaterThan: number;
        lessThan: number;
    };
};
declare const CreateClusterRequestOpenIDConnectConfig: {
    clientId: {
        minLength: number;
    };
    groupsPrefix: {
        minLength: number;
    };
    usernameClaim: {
        minLength: number;
    };
    usernamePrefix: {
        minLength: number;
    };
};
declare const CreateClusterRequestPoolConfig: {
    name: {
        minLength: number;
    };
};
declare const CreatePoolRequest: {
    name: {
        minLength: number;
    };
};
declare const ListClusterTypesRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListClustersRequest$1: {
    name: {
        minLength: number;
    };
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListNodesRequest: {
    name: {
        minLength: number;
    };
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListPoolsRequest: {
    name: {
        minLength: number;
    };
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const MaintenanceWindow: {
    startHour: {
        lessThanOrEqual: number;
    };
};
declare const UpdateClusterRequest$1: {
    name: {
        minLength: number;
    };
};
declare const UpdateClusterRequestAutoscalerConfig: {
    scaleDownUtilizationThreshold: {
        greaterThan: number;
        lessThan: number;
    };
};

declare const validationRules_gen$4_CreateClusterRequestAutoscalerConfig: typeof CreateClusterRequestAutoscalerConfig;
declare const validationRules_gen$4_CreateClusterRequestOpenIDConnectConfig: typeof CreateClusterRequestOpenIDConnectConfig;
declare const validationRules_gen$4_CreateClusterRequestPoolConfig: typeof CreateClusterRequestPoolConfig;
declare const validationRules_gen$4_CreatePoolRequest: typeof CreatePoolRequest;
declare const validationRules_gen$4_ListClusterTypesRequest: typeof ListClusterTypesRequest;
declare const validationRules_gen$4_ListNodesRequest: typeof ListNodesRequest;
declare const validationRules_gen$4_ListPoolsRequest: typeof ListPoolsRequest;
declare const validationRules_gen$4_MaintenanceWindow: typeof MaintenanceWindow;
declare const validationRules_gen$4_UpdateClusterRequestAutoscalerConfig: typeof UpdateClusterRequestAutoscalerConfig;
declare namespace validationRules_gen$4 {
  export { CreateClusterRequest$1 as CreateClusterRequest, validationRules_gen$4_CreateClusterRequestAutoscalerConfig as CreateClusterRequestAutoscalerConfig, validationRules_gen$4_CreateClusterRequestOpenIDConnectConfig as CreateClusterRequestOpenIDConnectConfig, validationRules_gen$4_CreateClusterRequestPoolConfig as CreateClusterRequestPoolConfig, validationRules_gen$4_CreatePoolRequest as CreatePoolRequest, validationRules_gen$4_ListClusterTypesRequest as ListClusterTypesRequest, ListClustersRequest$1 as ListClustersRequest, validationRules_gen$4_ListNodesRequest as ListNodesRequest, validationRules_gen$4_ListPoolsRequest as ListPoolsRequest, validationRules_gen$4_MaintenanceWindow as MaintenanceWindow, UpdateClusterRequest$1 as UpdateClusterRequest, validationRules_gen$4_UpdateClusterRequestAutoscalerConfig as UpdateClusterRequestAutoscalerConfig };
}

type index$f_AutoscalerEstimator = AutoscalerEstimator;
type index$f_AutoscalerExpander = AutoscalerExpander;
type index$f_CNI = CNI;
type index$f_ClusterAutoUpgrade = ClusterAutoUpgrade;
type index$f_ClusterAutoscalerConfig = ClusterAutoscalerConfig;
type index$f_ClusterOpenIDConnectConfig = ClusterOpenIDConnectConfig;
type index$f_ClusterType = ClusterType;
type index$f_ClusterTypeAvailability = ClusterTypeAvailability;
type index$f_ClusterTypeResiliency = ClusterTypeResiliency;
type index$f_CreateClusterRequestAutoUpgrade = CreateClusterRequestAutoUpgrade;
type index$f_CreateClusterRequestPoolConfigUpgradePolicy = CreateClusterRequestPoolConfigUpgradePolicy;
type index$f_CreateExternalNodeRequest = CreateExternalNodeRequest;
type index$f_CreatePoolRequestUpgradePolicy = CreatePoolRequestUpgradePolicy;
type index$f_DeleteNodeRequest = DeleteNodeRequest;
type index$f_DeletePoolRequest = DeletePoolRequest;
type index$f_ExternalNode = ExternalNode;
type index$f_ExternalNodeCoreV1Taint = ExternalNodeCoreV1Taint;
type index$f_GetClusterKubeConfigRequest = GetClusterKubeConfigRequest;
type index$f_GetNodeRequest = GetNodeRequest;
type index$f_GetPoolRequest = GetPoolRequest;
type index$f_ListClusterAvailableTypesRequest = ListClusterAvailableTypesRequest;
type index$f_ListClusterAvailableTypesResponse = ListClusterAvailableTypesResponse;
type index$f_ListClusterAvailableVersionsRequest = ListClusterAvailableVersionsRequest;
type index$f_ListClusterAvailableVersionsResponse = ListClusterAvailableVersionsResponse;
type index$f_ListClusterTypesResponse = ListClusterTypesResponse;
type index$f_ListNodesRequestOrderBy = ListNodesRequestOrderBy;
type index$f_ListNodesResponse = ListNodesResponse;
type index$f_ListPoolsRequestOrderBy = ListPoolsRequestOrderBy;
type index$f_ListPoolsResponse = ListPoolsResponse;
type index$f_MaintenanceWindowDayOfTheWeek = MaintenanceWindowDayOfTheWeek;
type index$f_MigrateToPrivateNetworkClusterRequest = MigrateToPrivateNetworkClusterRequest;
declare const index$f_NODE_TRANSIENT_STATUSES: typeof NODE_TRANSIENT_STATUSES;
type index$f_Node = Node;
type index$f_NodeStatus = NodeStatus;
declare const index$f_POOL_TRANSIENT_STATUSES: typeof POOL_TRANSIENT_STATUSES;
type index$f_Pool = Pool;
type index$f_PoolStatus = PoolStatus;
type index$f_PoolUpgradePolicy = PoolUpgradePolicy;
type index$f_PoolVolumeType = PoolVolumeType;
type index$f_RebootNodeRequest = RebootNodeRequest;
type index$f_ReplaceNodeRequest = ReplaceNodeRequest;
type index$f_ResetClusterAdminTokenRequest = ResetClusterAdminTokenRequest;
type index$f_Runtime = Runtime;
type index$f_SetClusterTypeRequest = SetClusterTypeRequest;
type index$f_UpdateClusterRequestAutoUpgrade = UpdateClusterRequestAutoUpgrade;
type index$f_UpdateClusterRequestOpenIDConnectConfig = UpdateClusterRequestOpenIDConnectConfig;
type index$f_UpdatePoolRequest = UpdatePoolRequest;
type index$f_UpdatePoolRequestUpgradePolicy = UpdatePoolRequestUpgradePolicy;
type index$f_UpgradeClusterRequest = UpgradeClusterRequest;
type index$f_UpgradePoolRequest = UpgradePoolRequest;
declare namespace index$f {
  export { K8SUtilsAPI as API, type index$f_AutoscalerEstimator as AutoscalerEstimator, type index$f_AutoscalerExpander as AutoscalerExpander, CLUSTER_TRANSIENT_STATUSES$1 as CLUSTER_TRANSIENT_STATUSES, type index$f_CNI as CNI, type Cluster$1 as Cluster, type index$f_ClusterAutoUpgrade as ClusterAutoUpgrade, type index$f_ClusterAutoscalerConfig as ClusterAutoscalerConfig, type index$f_ClusterOpenIDConnectConfig as ClusterOpenIDConnectConfig, type ClusterStatus$1 as ClusterStatus, type index$f_ClusterType as ClusterType, type index$f_ClusterTypeAvailability as ClusterTypeAvailability, type index$f_ClusterTypeResiliency as ClusterTypeResiliency, type CreateClusterRequest$2 as CreateClusterRequest, type index$f_CreateClusterRequestAutoUpgrade as CreateClusterRequestAutoUpgrade, type CreateClusterRequestAutoscalerConfig$1 as CreateClusterRequestAutoscalerConfig, type CreateClusterRequestOpenIDConnectConfig$1 as CreateClusterRequestOpenIDConnectConfig, type CreateClusterRequestPoolConfig$1 as CreateClusterRequestPoolConfig, type index$f_CreateClusterRequestPoolConfigUpgradePolicy as CreateClusterRequestPoolConfigUpgradePolicy, type index$f_CreateExternalNodeRequest as CreateExternalNodeRequest, type CreatePoolRequest$1 as CreatePoolRequest, type index$f_CreatePoolRequestUpgradePolicy as CreatePoolRequestUpgradePolicy, type DeleteClusterRequest$1 as DeleteClusterRequest, type index$f_DeleteNodeRequest as DeleteNodeRequest, type index$f_DeletePoolRequest as DeletePoolRequest, type index$f_ExternalNode as ExternalNode, type index$f_ExternalNodeCoreV1Taint as ExternalNodeCoreV1Taint, type index$f_GetClusterKubeConfigRequest as GetClusterKubeConfigRequest, type GetClusterRequest$1 as GetClusterRequest, type index$f_GetNodeRequest as GetNodeRequest, type index$f_GetPoolRequest as GetPoolRequest, type GetVersionRequest$1 as GetVersionRequest, type index$f_ListClusterAvailableTypesRequest as ListClusterAvailableTypesRequest, type index$f_ListClusterAvailableTypesResponse as ListClusterAvailableTypesResponse, type index$f_ListClusterAvailableVersionsRequest as ListClusterAvailableVersionsRequest, type index$f_ListClusterAvailableVersionsResponse as ListClusterAvailableVersionsResponse, type ListClusterTypesRequest$1 as ListClusterTypesRequest, type index$f_ListClusterTypesResponse as ListClusterTypesResponse, type ListClustersRequest$2 as ListClustersRequest, type ListClustersRequestOrderBy$1 as ListClustersRequestOrderBy, type ListClustersResponse$1 as ListClustersResponse, type ListNodesRequest$1 as ListNodesRequest, type index$f_ListNodesRequestOrderBy as ListNodesRequestOrderBy, type index$f_ListNodesResponse as ListNodesResponse, type ListPoolsRequest$1 as ListPoolsRequest, type index$f_ListPoolsRequestOrderBy as ListPoolsRequestOrderBy, type index$f_ListPoolsResponse as ListPoolsResponse, type ListVersionsRequest$1 as ListVersionsRequest, type ListVersionsResponse$1 as ListVersionsResponse, type MaintenanceWindow$1 as MaintenanceWindow, type index$f_MaintenanceWindowDayOfTheWeek as MaintenanceWindowDayOfTheWeek, type index$f_MigrateToPrivateNetworkClusterRequest as MigrateToPrivateNetworkClusterRequest, index$f_NODE_TRANSIENT_STATUSES as NODE_TRANSIENT_STATUSES, type index$f_Node as Node, type index$f_NodeStatus as NodeStatus, index$f_POOL_TRANSIENT_STATUSES as POOL_TRANSIENT_STATUSES, type index$f_Pool as Pool, type index$f_PoolStatus as PoolStatus, type index$f_PoolUpgradePolicy as PoolUpgradePolicy, type index$f_PoolVolumeType as PoolVolumeType, type index$f_RebootNodeRequest as RebootNodeRequest, type index$f_ReplaceNodeRequest as ReplaceNodeRequest, type index$f_ResetClusterAdminTokenRequest as ResetClusterAdminTokenRequest, type index$f_Runtime as Runtime, type index$f_SetClusterTypeRequest as SetClusterTypeRequest, type UpdateClusterRequest$2 as UpdateClusterRequest, type index$f_UpdateClusterRequestAutoUpgrade as UpdateClusterRequestAutoUpgrade, type UpdateClusterRequestAutoscalerConfig$1 as UpdateClusterRequestAutoscalerConfig, type index$f_UpdateClusterRequestOpenIDConnectConfig as UpdateClusterRequestOpenIDConnectConfig, type index$f_UpdatePoolRequest as UpdatePoolRequest, type index$f_UpdatePoolRequestUpgradePolicy as UpdatePoolRequestUpgradePolicy, type index$f_UpgradeClusterRequest as UpgradeClusterRequest, type index$f_UpgradePoolRequest as UpgradePoolRequest, validationRules_gen$4 as ValidationRules, type Version$2 as Version };
}

declare namespace index$e {
  export { index$f as v1 };
}

type AclActionRedirectRedirectType = 'location' | 'scheme';
type AclActionType = 'allow' | 'deny' | 'redirect';
type AclHttpFilter = 'acl_http_filter_none' | 'path_begin' | 'path_end' | 'regex' | 'http_header_match';
type BackendServerStatsHealthCheckStatus = 'unknown' | 'neutral' | 'failed' | 'passed' | 'condpass';
type BackendServerStatsServerState = 'stopped' | 'starting' | 'running' | 'stopping';
type CertificateStatus = 'pending' | 'ready' | 'error';
type CertificateType = 'letsencryt' | 'custom';
type ForwardPortAlgorithm = 'roundrobin' | 'leastconn' | 'first';
type InstanceStatus$1 = 'unknown' | 'ready' | 'pending' | 'stopped' | 'error' | 'locked' | 'migrating';
type LbStatus = 'unknown' | 'ready' | 'pending' | 'stopped' | 'error' | 'locked' | 'migrating' | 'to_create' | 'creating' | 'to_delete' | 'deleting';
type LbTypeStock = 'unknown' | 'low_stock' | 'out_of_stock' | 'available';
type ListAclRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListBackendsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListCertificatesRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListFrontendsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListIpsRequestIpType = 'all' | 'ipv4' | 'ipv6';
type ListLbsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListPrivateNetworksRequestOrderBy$2 = 'created_at_asc' | 'created_at_desc';
type ListRoutesRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListSubscriberRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type OnMarkedDownAction = 'on_marked_down_action_none' | 'shutdown_sessions';
type PrivateNetworkStatus = 'unknown' | 'ready' | 'pending' | 'error';
type Protocol = 'tcp' | 'http';
type ProxyProtocol = 'proxy_protocol_unknown' | 'proxy_protocol_none' | 'proxy_protocol_v1' | 'proxy_protocol_v2' | 'proxy_protocol_v2_ssl' | 'proxy_protocol_v2_ssl_cn';
type SSLCompatibilityLevel = 'ssl_compatibility_level_unknown' | 'ssl_compatibility_level_intermediate' | 'ssl_compatibility_level_modern' | 'ssl_compatibility_level_old';
type StickySessionsType = 'none' | 'cookie' | 'table';
interface SubscriberEmailConfig {
    /** Email address to send alerts to. */
    email: string;
}
/** Webhook alert of subscriber. */
interface SubscriberWebhookConfig {
    /** URI to receive POST requests. */
    uri: string;
}
interface HealthCheckHttpConfig {
    /** The HTTP URI to use when performing a health check on backend servers. */
    uri: string;
    /** The HTTP method used when performing a health check on backend servers. */
    method: string;
    /**
     * The HTTP response code that should be returned for a health check to be
     * considered successful.
     */
    code?: number;
    /**
     * The HTTP host header used when performing a health check on backend
     * servers.
     */
    hostHeader: string;
}
interface HealthCheckHttpsConfig {
    /** The HTTP URI to use when performing a health check on backend servers. */
    uri: string;
    /** The HTTP method used when performing a health check on backend servers. */
    method: string;
    /**
     * The HTTP response code that should be returned for a health check to be
     * considered successful.
     */
    code?: number;
    /**
     * The HTTP host header used when performing a health check on backend
     * servers.
     */
    hostHeader: string;
    /**
     * The SNI value used when performing a health check on backend servers over
     * SSL.
     */
    sni: string;
}
interface HealthCheckLdapConfig {
}
interface HealthCheckMysqlConfig {
    /** MySQL user to use for the health check. */
    user: string;
}
interface HealthCheckPgsqlConfig {
    /** PostgreSQL user to use for the health check. */
    user: string;
}
interface HealthCheckRedisConfig {
}
interface HealthCheckTcpConfig {
}
interface Instance$1 {
    /** Underlying Instance ID. */
    id: string;
    /** Instance status. */
    status: InstanceStatus$1;
    /** Instance IP address. */
    ipAddress: string;
    /** Date on which the Instance was created. */
    createdAt?: Date;
    /** Date on which the Instance was last updated. */
    updatedAt?: Date;
    /** @deprecated The region the Instance is in. */
    region?: Region;
    /** The zone the Instance is in. */
    zone: Zone;
}
interface Ip {
    /** IP address ID. */
    id: string;
    /** IP address. */
    ipAddress: string;
    /** Organization ID of the Scaleway Organization the IP address is in. */
    organizationId: string;
    /** Project ID of the Scaleway Project the IP address is in. */
    projectId: string;
    /** Load Balancer ID. */
    lbId?: string;
    /** Reverse DNS (domain name) of the IP address. */
    reverse: string;
    /** @deprecated The region the IP address is in. */
    region?: Region;
    /** The zone the IP address is in. */
    zone: Zone;
}
/** Subscriber. */
interface Subscriber {
    /** Subscriber ID. */
    id: string;
    /** Subscriber name. */
    name: string;
    /**
     * Email address of subscriber.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    emailConfig?: SubscriberEmailConfig;
    /**
     * Webhook URI of subscriber.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    webhookConfig?: SubscriberWebhookConfig;
}
interface HealthCheck {
    /** Port to use for the backend server health check. */
    port: number;
    /** Time to wait between two consecutive health checks. */
    checkDelay?: string;
    /** Maximum time a backend server has to reply to the health check. */
    checkTimeout?: string;
    /**
     * Number of consecutive unsuccessful health checks after which the server
     * will be considered dead.
     */
    checkMaxRetries: number;
    /**
     * Object to configure a basic TCP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    tcpConfig?: HealthCheckTcpConfig;
    /**
     * Object to configure a MySQL health check. The check requires MySQL >=3.22,
     * for older versions, use a TCP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    mysqlConfig?: HealthCheckMysqlConfig;
    /**
     * Object to configure a PostgreSQL health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    pgsqlConfig?: HealthCheckPgsqlConfig;
    /**
     * Object to configure an LDAP health check. The response is analyzed to find
     * the LDAPv3 response message.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    ldapConfig?: HealthCheckLdapConfig;
    /**
     * Object to configure a Redis health check. The response is analyzed to find
     * the +PONG response message.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    redisConfig?: HealthCheckRedisConfig;
    /**
     * Object to configure an HTTP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    httpConfig?: HealthCheckHttpConfig;
    /**
     * Object to configure an HTTPS health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    httpsConfig?: HealthCheckHttpsConfig;
    /** Defines whether proxy protocol should be activated for the health check. */
    checkSendProxy: boolean;
    /**
     * Time to wait between two consecutive health checks when a backend server is
     * in a transient state (going UP or DOWN).
     */
    transientCheckDelay?: string;
}
interface Lb {
    /** Underlying Instance ID. */
    id: string;
    /** Load Balancer name. */
    name: string;
    /** Load Balancer description. */
    description: string;
    /** Load Balancer status. */
    status: LbStatus;
    /** List of underlying Instances. */
    instances: Instance$1[];
    /** Scaleway Organization ID. */
    organizationId: string;
    /** Scaleway Project ID. */
    projectId: string;
    /** List of IP addresses attached to the Load Balancer. */
    ip: Ip[];
    /** Load Balancer tags. */
    tags: string[];
    /** Number of frontends the Load Balancer has. */
    frontendCount: number;
    /** Number of backends the Load Balancer has. */
    backendCount: number;
    /** Load Balancer offer type. */
    type: string;
    /** Subscriber information. */
    subscriber?: Subscriber;
    /**
     * Determines the minimal SSL version which needs to be supported on client
     * side.
     */
    sslCompatibilityLevel: SSLCompatibilityLevel;
    /** Date on which the Load Balancer was created. */
    createdAt?: Date;
    /** Date on which the Load Balancer was last updated. */
    updatedAt?: Date;
    /** Number of Private Networks attached to the Load Balancer. */
    privateNetworkCount: number;
    /** Number of routes configured on the Load Balancer. */
    routeCount: number;
    /** @deprecated The region the Load Balancer is in. */
    region?: Region;
    /** The zone the Load Balancer is in. */
    zone: Zone;
}
interface AclActionRedirect {
    /** Redirect type. */
    type: AclActionRedirectRedirectType;
    /**
     * Redirect target. For a location redirect, you can use a URL e.g.
     * `https://scaleway.com`. Using a scheme name (e.g. `https`, `http`, `ftp`,
     * `git`) will replace the request's original scheme. This can be useful to
     * implement HTTP to HTTPS redirects. Valid placeholders that can be used in a
     * `location` redirect to preserve parts of the original request in the
     * redirection URL are {{host}}, {{query}}, {{path}} and {{scheme}}.
     */
    target: string;
    /**
     * HTTP redirect code to use. Valid values are 301, 302, 303, 307 and 308.
     * Default value is 302.
     */
    code?: number;
}
interface Backend {
    /** Backend ID. */
    id: string;
    /** Name of the backend. */
    name: string;
    /** Protocol used by the backend when forwarding traffic to backend servers. */
    forwardProtocol: Protocol;
    /** Port used by the backend when forwarding traffic to backend servers. */
    forwardPort: number;
    /**
     * Load balancing algorithm to use when determining which backend server to
     * forward new traffic to.
     */
    forwardPortAlgorithm: ForwardPortAlgorithm;
    /**
     * Defines whether sticky sessions (binding a particular session to a
     * particular backend server) are activated and the method to use if so. None
     * disables sticky sessions. Cookie-based uses an HTTP cookie to stick a
     * session to a backend server. Table-based uses the source (client) IP
     * address to stick a session to a backend server.
     */
    stickySessions: StickySessionsType;
    /** Cookie name for cookie-based sticky sessions. */
    stickySessionsCookieName: string;
    /**
     * Object defining the health check to be carried out by the backend when
     * checking the status and health of backend servers.
     */
    healthCheck?: HealthCheck;
    /** List of IP addresses of backend servers attached to this backend. */
    pool: string[];
    /** Load Balancer the backend is attached to. */
    lb?: Lb;
    /** @deprecated Deprecated in favor of proxy_protocol field. */
    sendProxyV2?: boolean;
    /** Maximum allowed time for a backend server to process a request. */
    timeoutServer?: string;
    /** Maximum allowed time for establishing a connection to a backend server. */
    timeoutConnect?: string;
    /**
     * Maximum allowed tunnel inactivity time after Websocket is established
     * (takes precedence over client and server timeout).
     */
    timeoutTunnel?: string;
    /** Action to take when a backend server is marked as down. */
    onMarkedDownAction: OnMarkedDownAction;
    /**
     * Protocol to use between the Load Balancer and backend servers. Allows the
     * backend servers to be informed of the client's real IP address. The PROXY
     * protocol must be supported by the backend servers' software.
     */
    proxyProtocol: ProxyProtocol;
    /** Date at which the backend was created. */
    createdAt?: Date;
    /** Date at which the backend was updated. */
    updatedAt?: Date;
    /**
     * Scaleway S3 bucket website to be served as failover if all backend servers
     * are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
     */
    failoverHost?: string;
    /**
     * Defines whether to enable SSL bridging between the Load Balancer and
     * backend servers.
     */
    sslBridging?: boolean;
    /** Defines whether the server certificate verification should be ignored. */
    ignoreSslServerVerify?: boolean;
    /** Whether to use another backend server on each attempt. */
    redispatchAttemptCount?: number;
    /** Number of retries when a backend server connection failed. */
    maxRetries?: number;
    /** Maximum number of connections allowed per backend server. */
    maxConnections?: number;
    /**
     * Maximum time for a request to be left pending in queue when
     * `max_connections` is reached.
     */
    timeoutQueue?: string;
}
interface Certificate {
    /** Certificate type (Let's Encrypt or custom). */
    type: CertificateType;
    /** Certificate ID. */
    id: string;
    /** Main domain name of certificate. */
    commonName: string;
    /** Alternative domain names. */
    subjectAlternativeName: string[];
    /** Identifier (SHA-1) of the certificate. */
    fingerprint: string;
    /** Lower validity bound. */
    notValidBefore?: Date;
    /** Upper validity bound. */
    notValidAfter?: Date;
    /** Certificate status. */
    status: CertificateStatus;
    /** Load Balancer object the certificate is attached to. */
    lb?: Lb;
    /** Certificate name. */
    name: string;
    /** Date on which the certificate was created. */
    createdAt?: Date;
    /** Date on which the certificate was last updated. */
    updatedAt?: Date;
    /**
     * Additional information about the certificate status (useful in case of
     * certificate generation failure, for example).
     */
    statusDetails?: string;
}
interface AclAction {
    /** Action to take when incoming traffic matches an ACL filter. */
    type: AclActionType;
    /** Redirection parameters when using an ACL with a `redirect` action. */
    redirect?: AclActionRedirect;
}
interface AclMatch {
    /** List of IPs or CIDR v4/v6 addresses to filter for from the client side. */
    ipSubnet: string[];
    /**
     * Type of HTTP filter to match. Extracts the request's URL path, which starts
     * at the first slash and ends before the question mark (without the host
     * part). Defines where to filter for the http_filter_value. Only supported
     * for HTTP backends.
     */
    httpFilter: AclHttpFilter;
    /** List of values to filter for. */
    httpFilterValue: string[];
    /**
     * Name of the HTTP header to filter on if `http_header_match` was selected in
     * `http_filter`.
     */
    httpFilterOption?: string;
    /**
     * Defines whether to invert the match condition. If set to `true`, the ACL
     * carries out its action when the condition DOES NOT match.
     */
    invert: boolean;
}
interface Frontend {
    /** Frontend ID. */
    id: string;
    /** Name of the frontend. */
    name: string;
    /** Port the frontend listens on. */
    inboundPort: number;
    /** Backend object the frontend is attached to. */
    backend?: Backend;
    /** Load Balancer object the frontend is attached to. */
    lb?: Lb;
    /** Maximum allowed inactivity time on the client side. */
    timeoutClient?: string;
    /** @deprecated Certificate, deprecated in favor of certificate_ids array. */
    certificate?: Certificate;
    /** List of SSL/TLS certificate IDs to bind to the frontend. */
    certificateIds: string[];
    /** Date on which the frontend was created. */
    createdAt?: Date;
    /** Date on which the frontend was last updated. */
    updatedAt?: Date;
    /** Defines whether to enable HTTP/3 protocol on the frontend. */
    enableHttp3: boolean;
}
interface PrivateNetworkDHCPConfig {
    /** @deprecated */
    ipId?: string;
}
interface PrivateNetworkIpamConfig {
}
interface PrivateNetworkStaticConfig {
    /**
     * @deprecated Array of a local IP address for the Load Balancer on this
     *   Private Network.
     */
    ipAddress?: string[];
}
interface RouteMatch {
    /**
     * Value to match in the Server Name Indication TLS extension (SNI) field from
     * an incoming connection made via an SSL/TLS transport layer. This field
     * should be set for routes on TCP Load Balancers.
     *
     * One-of ('matchType'): at most one of 'sni', 'hostHeader' could be set.
     */
    sni?: string;
    /**
     * Value to match in the HTTP Host request header from an incoming connection.
     * This field should be set for routes on HTTP Load Balancers.
     *
     * One-of ('matchType'): at most one of 'sni', 'hostHeader' could be set.
     */
    hostHeader?: string;
}
interface CreateCertificateRequestCustomCertificate {
    /**
     * Full PEM-formatted certificate, consisting of the entire certificate chain
     * including public key, private key, and (optionally) Certificate
     * Authorities.
     */
    certificateChain: string;
}
interface CreateCertificateRequestLetsencryptConfig {
    /**
     * Main domain name of certificate (this domain must exist and resolve to your
     * Load Balancer IP address).
     */
    commonName: string;
    /**
     * Alternative domain names (all domain names must exist and resolve to your
     * Load Balancer IP address).
     */
    subjectAlternativeName: string[];
}
interface BackendServerStats {
    /** ID of your Load Balancer's underlying Instance. */
    instanceId: string;
    /** Backend ID. */
    backendId: string;
    /** IPv4 or IPv6 address of the backend server. */
    ip: string;
    /** Server operational state (stopped/starting/running/stopping). */
    serverState: BackendServerStatsServerState;
    /** Time since last operational change. */
    serverStateChangedAt?: Date;
    /** Last health check status (unknown/neutral/failed/passed/condpass). */
    lastHealthCheckStatus: BackendServerStatsHealthCheckStatus;
}
interface Acl {
    /** ACL ID. */
    id: string;
    /** ACL name. */
    name: string;
    /**
     * ACL match filter object. One of `ip_subnet` or `http_filter` &
     * `http_filter_value` are required.
     */
    match?: AclMatch;
    /** Action to take when incoming traffic matches an ACL filter. */
    action?: AclAction;
    /** ACL is attached to this frontend object. */
    frontend?: Frontend;
    /**
     * Priority of this ACL (ACLs are applied in ascending order, 0 is the first
     * ACL executed).
     */
    index: number;
    /** Date on which the ACL was created. */
    createdAt?: Date;
    /** Date on which the ACL was last updated. */
    updatedAt?: Date;
    /** ACL description. */
    description: string;
}
interface PrivateNetwork$3 {
    /** Load Balancer object which is attached to the Private Network. */
    lb?: Lb;
    /** IPAM IDs of the booked IP addresses. */
    ipamIds: string[];
    /**
     * @deprecated Object containing an array of a local IP address for the Load
     *   Balancer on this Private Network.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    staticConfig?: PrivateNetworkStaticConfig;
    /**
     * @deprecated Object containing DHCP-assigned IP addresses.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    dhcpConfig?: PrivateNetworkDHCPConfig;
    /**
     * @deprecated For internal use only.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    ipamConfig?: PrivateNetworkIpamConfig;
    /** Private Network ID. */
    privateNetworkId: string;
    /** Status of Private Network connection. */
    status: PrivateNetworkStatus;
    /** Date on which the Private Network was created. */
    createdAt?: Date;
    /** Date on which the PN was last updated. */
    updatedAt?: Date;
}
interface LbType {
    /** Load Balancer commercial offer type name. */
    name: string;
    /** Current stock status for a given Load Balancer type. */
    stockStatus: LbTypeStock;
    /** Load Balancer commercial offer type description. */
    description: string;
    /** @deprecated The region the Load Balancer stock is in. */
    region?: Region;
    /** The zone the Load Balancer stock is in. */
    zone: Zone;
}
interface Route {
    /** Route ID. */
    id: string;
    /** ID of the source frontend. */
    frontendId: string;
    /** ID of the target backend. */
    backendId: string;
    /**
     * Object defining the match condition for a route to be applied. If an
     * incoming client session matches the specified condition (i.e. it has a
     * matching SNI value or HTTP Host header value), it will be passed to the
     * target backend.
     */
    match?: RouteMatch;
    /** Date on which the route was created. */
    createdAt?: Date;
    /** Date on which the route was last updated. */
    updatedAt?: Date;
}
interface AclSpec {
    /** ACL name. */
    name: string;
    /** Action to take when incoming traffic matches an ACL filter. */
    action: AclAction;
    /**
     * ACL match filter object. One of `ip_subnet` or `http_filter` and
     * `http_filter_value` are required.
     */
    match?: AclMatch;
    /**
     * Priority of this ACL (ACLs are applied in ascending order, 0 is the first
     * ACL executed).
     */
    index: number;
    /** ACL description. */
    description: string;
}
type AddBackendServersRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Backend ID. */
    backendId: string;
    /** List of IP addresses to add to backend servers. */
    serverIp: string[];
};
type AttachPrivateNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Private Network ID. */
    privateNetworkId: string;
    /**
     * @deprecated Object containing an array of a local IP address for the Load
     *   Balancer on this Private Network.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    staticConfig?: PrivateNetworkStaticConfig;
    /**
     * @deprecated Defines whether to let DHCP assign IP addresses.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    dhcpConfig?: PrivateNetworkDHCPConfig;
    /**
     * @deprecated For internal use only.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    ipamConfig?: PrivateNetworkIpamConfig;
};
/** Add an ACL to a Load Balancer frontend. */
type CreateAclRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Frontend ID to attach the ACL to. */
    frontendId: string;
    /** ACL name. */
    name?: string;
    /** Action to take when incoming traffic matches an ACL filter. */
    action: AclAction;
    /**
     * ACL match filter object. One of `ip_subnet` or `http_filter` &
     * `http_filter_value` are required.
     */
    match?: AclMatch;
    /**
     * Priority of this ACL (ACLs are applied in ascending order, 0 is the first
     * ACL executed).
     */
    index: number;
    /** ACL description. */
    description: string;
};
type CreateBackendRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Name for the backend. */
    name?: string;
    /**
     * Protocol to be used by the backend when forwarding traffic to backend
     * servers.
     */
    forwardProtocol: Protocol;
    /** Port to be used by the backend when forwarding traffic to backend servers. */
    forwardPort: number;
    /**
     * Load balancing algorithm to be used when determining which backend server
     * to forward new traffic to.
     */
    forwardPortAlgorithm: ForwardPortAlgorithm;
    /**
     * Defines whether to activate sticky sessions (binding a particular session
     * to a particular backend server) and the method to use if so. None disables
     * sticky sessions. Cookie-based uses an HTTP cookie TO stick a session to a
     * backend server. Table-based uses the source (client) IP address to stick a
     * session to a backend server.
     */
    stickySessions: StickySessionsType;
    /** Cookie name for cookie-based sticky sessions. */
    stickySessionsCookieName: string;
    /**
     * Object defining the health check to be carried out by the backend when
     * checking the status and health of backend servers.
     */
    healthCheck: HealthCheck;
    /**
     * List of backend server IP addresses (IPv4 or IPv6) the backend should
     * forward traffic to.
     */
    serverIp: string[];
    /** @deprecated Deprecated in favor of proxy_protocol field. */
    sendProxyV2?: boolean;
    /** Maximum allowed time for a backend server to process a request. */
    timeoutServer?: string;
    /** Maximum allowed time for establishing a connection to a backend server. */
    timeoutConnect?: string;
    /**
     * Maximum allowed tunnel inactivity time after Websocket is established
     * (takes precedence over client and server timeout).
     */
    timeoutTunnel?: string;
    /** Action to take when a backend server is marked as down. */
    onMarkedDownAction?: OnMarkedDownAction;
    /**
     * Protocol to use between the Load Balancer and backend servers. Allows the
     * backend servers to be informed of the client's real IP address. The PROXY
     * protocol must be supported by the backend servers' software.
     */
    proxyProtocol?: ProxyProtocol;
    /**
     * Scaleway S3 bucket website to be served as failover if all backend servers
     * are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
     */
    failoverHost?: string;
    /**
     * Defines whether to enable SSL bridging between the Load Balancer and
     * backend servers.
     */
    sslBridging?: boolean;
    /** Defines whether the server certificate verification should be ignored. */
    ignoreSslServerVerify?: boolean;
    /** Whether to use another backend server on each attempt. */
    redispatchAttemptCount?: number;
    /** Number of retries when a backend server connection failed. */
    maxRetries?: number;
    /** Maximum number of connections allowed per backend server. */
    maxConnections?: number;
    /**
     * Maximum time for a request to be left pending in queue when
     * `max_connections` is reached.
     */
    timeoutQueue?: string;
};
type CreateCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Name for the certificate. */
    name?: string;
    /**
     * Object to define a new Let's Encrypt certificate to be generated.
     *
     * One-of ('type'): at most one of 'letsencrypt', 'customCertificate' could be
     * set.
     */
    letsencrypt?: CreateCertificateRequestLetsencryptConfig;
    /**
     * Object to define an existing custom certificate to be imported.
     *
     * One-of ('type'): at most one of 'letsencrypt', 'customCertificate' could be
     * set.
     */
    customCertificate?: CreateCertificateRequestCustomCertificate;
};
type CreateFrontendRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID (ID of the Load Balancer to attach the frontend to). */
    lbId: string;
    /** Name for the frontend. */
    name?: string;
    /** Port the frontend should listen on. */
    inboundPort: number;
    /** Backend ID (ID of the backend the frontend should pass traffic to). */
    backendId: string;
    /** Maximum allowed inactivity time on the client side. */
    timeoutClient?: string;
    /** @deprecated Certificate ID, deprecated in favor of certificate_ids array. */
    certificateId?: string;
    /** List of SSL/TLS certificate IDs to bind to the frontend. */
    certificateIds?: string[];
    /** Defines whether to enable HTTP/3 protocol on the frontend. */
    enableHttp3: boolean;
};
type CreateIpRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * @deprecated Organization ID of the Organization where the IP address should
     *   be created.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Project ID of the Project where the IP address should be created.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
    /** Reverse DNS (domain name) for the IP address. */
    reverse?: string;
    /** If true, creates a Flexible IP with an ipv6 address. */
    isIpv6: boolean;
};
type CreateLbRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * @deprecated Scaleway Organization to create the Load Balancer in.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Scaleway Project to create the Load Balancer in.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
    /** Name for the Load Balancer. */
    name?: string;
    /** Description for the Load Balancer. */
    description: string;
    /**
     * @deprecated ID of an existing flexible IP address to attach to the Load
     *   Balancer.
     */
    ipId?: string;
    /**
     * Defines whether to automatically assign a flexible public IP to lb. Default
     * value is `false` (do not assign).
     */
    assignFlexibleIp?: boolean;
    /**
     * Defines whether to automatically assign a flexible public IPv6 to the Load
     * Balancer. Default value is `false` (do not assign).
     */
    assignFlexibleIpv6?: boolean;
    /** List of IP IDs to attach to the Load Balancer. */
    ipIds?: string[];
    /** List of tags for the Load Balancer. */
    tags?: string[];
    /**
     * Load Balancer commercial offer type. Use the Load Balancer types endpoint
     * to retrieve a list of available offer types.
     */
    type: string;
    /**
     * Determines the minimal SSL version which needs to be supported on the
     * client side, in an SSL/TLS offloading context. Intermediate is suitable for
     * general-purpose servers with a variety of clients, recommended for almost
     * all systems. Modern is suitable for services with clients that support TLS
     * 1.3 and do not need backward compatibility. Old is compatible with a small
     * number of very old clients and should be used only as a last resort.
     */
    sslCompatibilityLevel?: SSLCompatibilityLevel;
};
type CreateRouteRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the source frontend to create the route on. */
    frontendId: string;
    /** ID of the target backend for the route. */
    backendId: string;
    /**
     * Object defining the match condition for a route to be applied. If an
     * incoming client session matches the specified condition (i.e. it has a
     * matching SNI value or HTTP Host header value), it will be passed to the
     * target backend.
     */
    match?: RouteMatch;
};
/** Create a new alert subscriber (webhook or email). */
type CreateSubscriberRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Subscriber name. */
    name: string;
    /**
     * Email address configuration.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    emailConfig?: SubscriberEmailConfig;
    /**
     * WebHook URI configuration.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    webhookConfig?: SubscriberWebhookConfig;
    /**
     * @deprecated Organization ID to create the subscriber in.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Project ID to create the subscriber in.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
};
type DeleteAclRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ACL ID. */
    aclId: string;
};
type DeleteBackendRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the backend to delete. */
    backendId: string;
};
type DeleteCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Certificate ID. */
    certificateId: string;
};
type DeleteFrontendRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the frontend to delete. */
    frontendId: string;
};
type DeleteLbRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the Load Balancer to delete. */
    lbId: string;
    /**
     * Defines whether the Load Balancer's flexible IP should be deleted. Set to
     * true to release the flexible IP, or false to keep it available in your
     * account for future Load Balancers.
     */
    releaseIp: boolean;
};
type DeleteRouteRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Route ID. */
    routeId: string;
};
type DeleteSubscriberRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Subscriber ID. */
    subscriberId: string;
};
type DetachPrivateNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load balancer ID. */
    lbId: string;
    /** Set your instance private network id. */
    privateNetworkId: string;
};
type GetAclRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ACL ID. */
    aclId: string;
};
type GetBackendRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Backend ID. */
    backendId: string;
};
type GetCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Certificate ID. */
    certificateId: string;
};
type GetFrontendRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Frontend ID. */
    frontendId: string;
};
type GetIpRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** IP address ID. */
    ipId: string;
};
type GetLbRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
};
/** Get Load Balancer stats. */
type GetLbStatsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** ID of the backend. */
    backendId?: string;
};
type GetRouteRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Route ID. */
    routeId: string;
};
type GetSubscriberRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Subscriber ID. */
    subscriberId: string;
};
interface LbStats {
    /** List of objects containing Load Balancer statistics. */
    backendServersStats: BackendServerStats[];
}
interface ListAclResponse {
    /** List of ACL objects. */
    acls: Acl[];
    /** The total number of objects. */
    totalCount: number;
}
type ListAclsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * Frontend ID (ACLs attached to this frontend will be returned in the
     * response).
     */
    frontendId: string;
    /** Sort order of ACLs in the response. */
    orderBy?: ListAclRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** The number of ACLs to return. */
    pageSize?: number;
    /** ACL name to filter for. */
    name?: string;
};
type ListBackendStatsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of items to return. */
    pageSize?: number;
    /** ID of the backend. */
    backendId?: string;
};
interface ListBackendStatsResponse {
    /** List of objects containing backend server statistics. */
    backendServersStats: BackendServerStats[];
    /** The total number of objects. */
    totalCount: number;
}
type ListBackendsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Name of the backend to filter for. */
    name?: string;
    /** Sort order of backends in the response. */
    orderBy?: ListBackendsRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of backends to return. */
    pageSize?: number;
};
interface ListBackendsResponse {
    /** List of backend objects of a given Load Balancer. */
    backends: Backend[];
    /** Total count of backend objects, without pagination. */
    totalCount: number;
}
type ListCertificatesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Sort order of certificates in the response. */
    orderBy?: ListCertificatesRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of certificates to return. */
    pageSize?: number;
    /**
     * Certificate name to filter for, only certificates of this name will be
     * returned.
     */
    name?: string;
};
interface ListCertificatesResponse {
    /** List of certificate objects. */
    certificates: Certificate[];
    /** The total number of objects. */
    totalCount: number;
}
type ListFrontendsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Name of the frontend to filter for. */
    name?: string;
    /** Sort order of frontends in the response. */
    orderBy?: ListFrontendsRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of frontends to return. */
    pageSize?: number;
};
interface ListFrontendsResponse {
    /** List of frontend objects of a given Load Balancer. */
    frontends: Frontend[];
    /** Total count of frontend objects, without pagination. */
    totalCount: number;
}
type ListIPsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of IP addresses to return. */
    pageSize?: number;
    /** IP address to filter for. */
    ipAddress?: string;
    /**
     * Organization ID to filter for, only Load Balancer IP addresses from this
     * Organization will be returned.
     */
    organizationId?: string;
    /**
     * Project ID to filter for, only Load Balancer IP addresses from this Project
     * will be returned.
     */
    projectId?: string;
    /** IP type to filter for. */
    ipType?: ListIpsRequestIpType;
};
interface ListIpsResponse {
    /** List of IP address objects. */
    ips: Ip[];
    /** Total count of IP address objects, without pagination. */
    totalCount: number;
}
type ListLbPrivateNetworksRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Sort order of Private Network objects in the response. */
    orderBy?: ListPrivateNetworksRequestOrderBy$2;
    /** Number of objects to return. */
    pageSize?: number;
    /** The page number to return, from the paginated results. */
    page?: number;
};
interface ListLbPrivateNetworksResponse {
    /** List of Private Network objects attached to the Load Balancer. */
    privateNetwork: PrivateNetwork$3[];
    /** Total number of objects in the response. */
    totalCount: number;
}
type ListLbTypesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** The number of items to return. */
    pageSize?: number;
};
interface ListLbTypesResponse {
    /** List of Load Balancer commercial offer type objects. */
    lbTypes: LbType[];
    /** Total number of Load Balancer offer type objects. */
    totalCount: number;
}
type ListLbsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer name to filter for. */
    name?: string;
    /** Sort order of Load Balancers in the response. */
    orderBy?: ListLbsRequestOrderBy;
    /** Number of Load Balancers to return. */
    pageSize?: number;
    /** Page number to return, from the paginated results. */
    page?: number;
    /**
     * Organization ID to filter for, only Load Balancers from this Organization
     * will be returned.
     */
    organizationId?: string;
    /**
     * Project ID to filter for, only Load Balancers from this Project will be
     * returned.
     */
    projectId?: string;
};
interface ListLbsResponse {
    /** List of Load Balancer objects. */
    lbs: Lb[];
    /** The total number of Load Balancer objects. */
    totalCount: number;
}
type ListRoutesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Sort order of routes in the response. */
    orderBy?: ListRoutesRequestOrderBy;
    /** The number of route objects to return. */
    pageSize?: number;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Frontend ID to filter for, only Routes from this Frontend will be returned. */
    frontendId?: string;
};
interface ListRoutesResponse {
    /** List of route objects. */
    routes: Route[];
    /** The total number of route objects. */
    totalCount: number;
}
type ListSubscriberRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Sort order of subscribers in the response. */
    orderBy?: ListSubscriberRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** The number of items to return. */
    pageSize?: number;
    /** Subscriber name to search for. */
    name?: string;
    /** Filter subscribers by Organization ID. */
    organizationId?: string;
    /** Filter subscribers by Project ID. */
    projectId?: string;
};
interface ListSubscriberResponse {
    /** List of subscriber objects. */
    subscribers: Subscriber[];
    /** The total number of objects. */
    totalCount: number;
}
type MigrateLbRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /**
     * Load Balancer type to migrate to (use the List all Load Balancer offer
     * types endpoint to get a list of available offer types).
     */
    type: string;
};
type ReleaseIpRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** IP address ID. */
    ipId: string;
};
type RemoveBackendServersRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Backend ID. */
    backendId: string;
    /** List of IP addresses to remove from backend servers. */
    serverIp: string[];
};
interface SetAclsResponse {
    /** List of ACL objects. */
    acls: Acl[];
    /** The total number of ACL objects. */
    totalCount: number;
}
type SetBackendServersRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Backend ID. */
    backendId: string;
    /**
     * List of IP addresses for backend servers. Any other existing backend
     * servers will be removed.
     */
    serverIp: string[];
};
type SubscribeToLbRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Subscriber ID. */
    subscriberId: string;
};
type UnsubscribeFromLbRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
};
type UpdateAclRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ACL ID. */
    aclId: string;
    /** ACL name. */
    name: string;
    /** Action to take when incoming traffic matches an ACL filter. */
    action: AclAction;
    /**
     * ACL match filter object. One of `ip_subnet` or `http_filter` &
     * `http_filter_value` are required.
     */
    match?: AclMatch;
    /**
     * Priority of this ACL (ACLs are applied in ascending order, 0 is the first
     * ACL executed).
     */
    index: number;
    /** ACL description. */
    description?: string;
};
type UpdateBackendRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Backend ID. */
    backendId: string;
    /** Backend name. */
    name: string;
    /**
     * Protocol to be used by the backend when forwarding traffic to backend
     * servers.
     */
    forwardProtocol: Protocol;
    /** Port to be used by the backend when forwarding traffic to backend servers. */
    forwardPort: number;
    /**
     * Load balancing algorithm to be used when determining which backend server
     * to forward new traffic to.
     */
    forwardPortAlgorithm: ForwardPortAlgorithm;
    /**
     * Defines whether to activate sticky sessions (binding a particular session
     * to a particular backend server) and the method to use if so. None disables
     * sticky sessions. Cookie-based uses an HTTP cookie to stick a session to a
     * backend server. Table-based uses the source (client) IP address to stick a
     * session to a backend server.
     */
    stickySessions: StickySessionsType;
    /** Cookie name for cookie-based sticky sessions. */
    stickySessionsCookieName: string;
    /** @deprecated Deprecated in favor of proxy_protocol field. */
    sendProxyV2?: boolean;
    /** Maximum allowed time for a backend server to process a request. */
    timeoutServer?: string;
    /** Maximum allowed time for establishing a connection to a backend server. */
    timeoutConnect?: string;
    /**
     * Maximum allowed tunnel inactivity time after Websocket is established
     * (takes precedence over client and server timeout).
     */
    timeoutTunnel?: string;
    /** Action to take when a backend server is marked as down. */
    onMarkedDownAction?: OnMarkedDownAction;
    /**
     * Protocol to use between the Load Balancer and backend servers. Allows the
     * backend servers to be informed of the client's real IP address. The PROXY
     * protocol must be supported by the backend servers' software.
     */
    proxyProtocol?: ProxyProtocol;
    /**
     * Scaleway S3 bucket website to be served as failover if all backend servers
     * are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
     */
    failoverHost?: string;
    /**
     * Defines whether to enable SSL bridging between the Load Balancer and
     * backend servers.
     */
    sslBridging?: boolean;
    /** Defines whether the server certificate verification should be ignored. */
    ignoreSslServerVerify?: boolean;
    /** Whether to use another backend server on each attempt. */
    redispatchAttemptCount?: number;
    /** Number of retries when a backend server connection failed. */
    maxRetries?: number;
    /** Maximum number of connections allowed per backend server. */
    maxConnections?: number;
    /**
     * Maximum time for a request to be left pending in queue when
     * `max_connections` is reached.
     */
    timeoutQueue?: string;
};
type UpdateCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Certificate ID. */
    certificateId: string;
    /** Certificate name. */
    name: string;
};
type UpdateFrontendRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Frontend ID. */
    frontendId: string;
    /** Frontend name. */
    name: string;
    /** Port the frontend should listen on. */
    inboundPort: number;
    /** Backend ID (ID of the backend the frontend should pass traffic to). */
    backendId: string;
    /** Maximum allowed inactivity time on the client side. */
    timeoutClient?: string;
    /** @deprecated Certificate ID, deprecated in favor of certificate_ids array. */
    certificateId?: string;
    /** List of SSL/TLS certificate IDs to bind to the frontend. */
    certificateIds?: string[];
    /** Defines whether to enable HTTP/3 protocol on the frontend. */
    enableHttp3: boolean;
};
type UpdateHealthCheckRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Backend ID. */
    backendId: string;
    /** Port to use for the backend server health check. */
    port: number;
    /** Time to wait between two consecutive health checks. */
    checkDelay?: string;
    /** Maximum time a backend server has to reply to the health check. */
    checkTimeout?: string;
    /**
     * Number of consecutive unsuccessful health checks after which the server
     * will be considered dead.
     */
    checkMaxRetries: number;
    /** Defines whether proxy protocol should be activated for the health check. */
    checkSendProxy: boolean;
    /**
     * Object to configure a basic TCP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    tcpConfig?: HealthCheckTcpConfig;
    /**
     * Object to configure a MySQL health check. The check requires MySQL >=3.22,
     * for older versions, use a TCP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    mysqlConfig?: HealthCheckMysqlConfig;
    /**
     * Object to configure a PostgreSQL health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    pgsqlConfig?: HealthCheckPgsqlConfig;
    /**
     * Object to configure an LDAP health check. The response is analyzed to find
     * the LDAPv3 response message.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    ldapConfig?: HealthCheckLdapConfig;
    /**
     * Object to configure a Redis health check. The response is analyzed to find
     * the +PONG response message.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    redisConfig?: HealthCheckRedisConfig;
    /**
     * Object to configure an HTTP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    httpConfig?: HealthCheckHttpConfig;
    /**
     * Object to configure an HTTPS health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    httpsConfig?: HealthCheckHttpsConfig;
    /**
     * Time to wait between two consecutive health checks when a backend server is
     * in a transient state (going UP or DOWN).
     */
    transientCheckDelay?: string;
};
type UpdateIpRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** IP address ID. */
    ipId: string;
    /** Reverse DNS (domain name) for the IP address. */
    reverse?: string;
    /** ID of the server on which to attach the flexible IP. */
    lbId?: string;
};
type UpdateLbRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Load Balancer ID. */
    lbId: string;
    /** Load Balancer name. */
    name: string;
    /** Load Balancer description. */
    description: string;
    /** List of tags for the Load Balancer. */
    tags?: string[];
    /**
     * Determines the minimal SSL version which needs to be supported on the
     * client side, in an SSL/TLS offloading context. Intermediate is suitable for
     * general-purpose servers with a variety of clients, recommended for almost
     * all systems. Modern is suitable for services with clients that support TLS
     * 1.3 and don't need backward compatibility. Old is compatible with a small
     * number of very old clients and should be used only as a last resort.
     */
    sslCompatibilityLevel?: SSLCompatibilityLevel;
};
type UpdateRouteRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Route ID. */
    routeId: string;
    /** ID of the target backend for the route. */
    backendId: string;
    /**
     * Object defining the match condition for a route to be applied. If an
     * incoming client session matches the specified condition (i.e. it has a
     * matching SNI value or HTTP Host header value), it will be passed to the
     * target backend.
     */
    match?: RouteMatch;
};
type UpdateSubscriberRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Subscriber ID. */
    subscriberId: string;
    /** Subscriber name. */
    name: string;
    /**
     * Email address configuration.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    emailConfig?: SubscriberEmailConfig;
    /**
     * Webhook URI configuration.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    webhookConfig?: SubscriberWebhookConfig;
};
type ZonedApiAddBackendServersRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Backend ID. */
    backendId: string;
    /** List of IP addresses to add to backend servers. */
    serverIp: string[];
};
type ZonedApiAttachPrivateNetworkRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Private Network ID. */
    privateNetworkId: string;
    /**
     * @deprecated Object containing an array of a local IP address for the Load
     *   Balancer on this Private Network.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    staticConfig?: PrivateNetworkStaticConfig;
    /**
     * @deprecated Defines whether to let DHCP assign IP addresses.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    dhcpConfig?: PrivateNetworkDHCPConfig;
    /**
     * @deprecated For internal use only.
     *
     *   One-of ('config'): at most one of 'staticConfig', 'dhcpConfig',
     *   'ipamConfig' could be set.
     */
    ipamConfig?: PrivateNetworkIpamConfig;
};
/** Add an ACL to a Load Balancer frontend. */
type ZonedApiCreateAclRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Frontend ID to attach the ACL to. */
    frontendId: string;
    /** ACL name. */
    name?: string;
    /** Action to take when incoming traffic matches an ACL filter. */
    action: AclAction;
    /**
     * ACL match filter object. One of `ip_subnet` or `http_filter` &
     * `http_filter_value` are required.
     */
    match?: AclMatch;
    /**
     * Priority of this ACL (ACLs are applied in ascending order, 0 is the first
     * ACL executed).
     */
    index: number;
    /** ACL description. */
    description: string;
};
type ZonedApiCreateBackendRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Name for the backend. */
    name?: string;
    /**
     * Protocol to be used by the backend when forwarding traffic to backend
     * servers.
     */
    forwardProtocol: Protocol;
    /** Port to be used by the backend when forwarding traffic to backend servers. */
    forwardPort: number;
    /**
     * Load balancing algorithm to be used when determining which backend server
     * to forward new traffic to.
     */
    forwardPortAlgorithm: ForwardPortAlgorithm;
    /**
     * Defines whether to activate sticky sessions (binding a particular session
     * to a particular backend server) and the method to use if so. None disables
     * sticky sessions. Cookie-based uses an HTTP cookie TO stick a session to a
     * backend server. Table-based uses the source (client) IP address to stick a
     * session to a backend server.
     */
    stickySessions: StickySessionsType;
    /** Cookie name for cookie-based sticky sessions. */
    stickySessionsCookieName: string;
    /**
     * Object defining the health check to be carried out by the backend when
     * checking the status and health of backend servers.
     */
    healthCheck: HealthCheck;
    /**
     * List of backend server IP addresses (IPv4 or IPv6) the backend should
     * forward traffic to.
     */
    serverIp: string[];
    /** @deprecated Deprecated in favor of proxy_protocol field. */
    sendProxyV2?: boolean;
    /** Maximum allowed time for a backend server to process a request. */
    timeoutServer?: string;
    /** Maximum allowed time for establishing a connection to a backend server. */
    timeoutConnect?: string;
    /**
     * Maximum allowed tunnel inactivity time after Websocket is established
     * (takes precedence over client and server timeout).
     */
    timeoutTunnel?: string;
    /** Action to take when a backend server is marked as down. */
    onMarkedDownAction?: OnMarkedDownAction;
    /**
     * Protocol to use between the Load Balancer and backend servers. Allows the
     * backend servers to be informed of the client's real IP address. The PROXY
     * protocol must be supported by the backend servers' software.
     */
    proxyProtocol?: ProxyProtocol;
    /**
     * Scaleway S3 bucket website to be served as failover if all backend servers
     * are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
     */
    failoverHost?: string;
    /**
     * Defines whether to enable SSL bridging between the Load Balancer and
     * backend servers.
     */
    sslBridging?: boolean;
    /** Defines whether the server certificate verification should be ignored. */
    ignoreSslServerVerify?: boolean;
    /** Whether to use another backend server on each attempt. */
    redispatchAttemptCount?: number;
    /** Number of retries when a backend server connection failed. */
    maxRetries?: number;
    /** Maximum number of connections allowed per backend server. */
    maxConnections?: number;
    /**
     * Maximum time for a request to be left pending in queue when
     * `max_connections` is reached.
     */
    timeoutQueue?: string;
};
type ZonedApiCreateCertificateRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Name for the certificate. */
    name?: string;
    /**
     * Object to define a new Let's Encrypt certificate to be generated.
     *
     * One-of ('type'): at most one of 'letsencrypt', 'customCertificate' could be
     * set.
     */
    letsencrypt?: CreateCertificateRequestLetsencryptConfig;
    /**
     * Object to define an existing custom certificate to be imported.
     *
     * One-of ('type'): at most one of 'letsencrypt', 'customCertificate' could be
     * set.
     */
    customCertificate?: CreateCertificateRequestCustomCertificate;
};
type ZonedApiCreateFrontendRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID (ID of the Load Balancer to attach the frontend to). */
    lbId: string;
    /** Name for the frontend. */
    name?: string;
    /** Port the frontend should listen on. */
    inboundPort: number;
    /** Backend ID (ID of the backend the frontend should pass traffic to). */
    backendId: string;
    /** Maximum allowed inactivity time on the client side. */
    timeoutClient?: string;
    /** @deprecated Certificate ID, deprecated in favor of certificate_ids array. */
    certificateId?: string;
    /** List of SSL/TLS certificate IDs to bind to the frontend. */
    certificateIds?: string[];
    /** Defines whether to enable HTTP/3 protocol on the frontend. */
    enableHttp3: boolean;
};
type ZonedApiCreateIpRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * @deprecated Organization ID of the Organization where the IP address should
     *   be created.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Project ID of the Project where the IP address should be created.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
    /** Reverse DNS (domain name) for the IP address. */
    reverse?: string;
    /** If true, creates a Flexible IP with an ipv6 address. */
    isIpv6: boolean;
};
type ZonedApiCreateLbRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * @deprecated Scaleway Organization to create the Load Balancer in.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Scaleway Project to create the Load Balancer in.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
    /** Name for the Load Balancer. */
    name?: string;
    /** Description for the Load Balancer. */
    description: string;
    /**
     * @deprecated ID of an existing flexible IP address to attach to the Load
     *   Balancer.
     */
    ipId?: string;
    /**
     * Defines whether to automatically assign a flexible public IP to lb. Default
     * value is `false` (do not assign).
     */
    assignFlexibleIp?: boolean;
    /**
     * Defines whether to automatically assign a flexible public IPv6 to the Load
     * Balancer. Default value is `false` (do not assign).
     */
    assignFlexibleIpv6?: boolean;
    /** List of IP IDs to attach to the Load Balancer. */
    ipIds?: string[];
    /** List of tags for the Load Balancer. */
    tags?: string[];
    /**
     * Load Balancer commercial offer type. Use the Load Balancer types endpoint
     * to retrieve a list of available offer types.
     */
    type: string;
    /**
     * Determines the minimal SSL version which needs to be supported on the
     * client side, in an SSL/TLS offloading context. Intermediate is suitable for
     * general-purpose servers with a variety of clients, recommended for almost
     * all systems. Modern is suitable for services with clients that support TLS
     * 1.3 and do not need backward compatibility. Old is compatible with a small
     * number of very old clients and should be used only as a last resort.
     */
    sslCompatibilityLevel?: SSLCompatibilityLevel;
};
type ZonedApiCreateRouteRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the source frontend to create the route on. */
    frontendId: string;
    /** ID of the target backend for the route. */
    backendId: string;
    /**
     * Object defining the match condition for a route to be applied. If an
     * incoming client session matches the specified condition (i.e. it has a
     * matching SNI value or HTTP Host header value), it will be passed to the
     * target backend.
     */
    match?: RouteMatch;
};
/** Create a new alert subscriber (webhook or email). */
type ZonedApiCreateSubscriberRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Subscriber name. */
    name: string;
    /**
     * Email address configuration.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    emailConfig?: SubscriberEmailConfig;
    /**
     * WebHook URI configuration.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    webhookConfig?: SubscriberWebhookConfig;
    /**
     * @deprecated Organization ID to create the subscriber in.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Project ID to create the subscriber in.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
};
type ZonedApiDeleteAclRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ACL ID. */
    aclId: string;
};
type ZonedApiDeleteBackendRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the backend to delete. */
    backendId: string;
};
type ZonedApiDeleteCertificateRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Certificate ID. */
    certificateId: string;
};
type ZonedApiDeleteFrontendRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the frontend to delete. */
    frontendId: string;
};
type ZonedApiDeleteLbRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the Load Balancer to delete. */
    lbId: string;
    /**
     * Defines whether the Load Balancer's flexible IP should be deleted. Set to
     * true to release the flexible IP, or false to keep it available in your
     * account for future Load Balancers.
     */
    releaseIp: boolean;
};
type ZonedApiDeleteRouteRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Route ID. */
    routeId: string;
};
type ZonedApiDeleteSubscriberRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Subscriber ID. */
    subscriberId: string;
};
type ZonedApiDetachPrivateNetworkRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load balancer ID. */
    lbId: string;
    /** Set your instance private network id. */
    privateNetworkId: string;
};
type ZonedApiGetAclRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ACL ID. */
    aclId: string;
};
type ZonedApiGetBackendRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Backend ID. */
    backendId: string;
};
type ZonedApiGetCertificateRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Certificate ID. */
    certificateId: string;
};
type ZonedApiGetFrontendRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Frontend ID. */
    frontendId: string;
};
type ZonedApiGetIpRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** IP address ID. */
    ipId: string;
};
type ZonedApiGetLbRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
};
/** Get Load Balancer stats. */
type ZonedApiGetLbStatsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** ID of the backend. */
    backendId?: string;
};
type ZonedApiGetRouteRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Route ID. */
    routeId: string;
};
type ZonedApiGetSubscriberRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Subscriber ID. */
    subscriberId: string;
};
type ZonedApiListAclsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /**
     * Frontend ID (ACLs attached to this frontend will be returned in the
     * response).
     */
    frontendId: string;
    /** Sort order of ACLs in the response. */
    orderBy?: ListAclRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** The number of ACLs to return. */
    pageSize?: number;
    /** ACL name to filter for. */
    name?: string;
};
type ZonedApiListBackendStatsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of items to return. */
    pageSize?: number;
    /** ID of the backend. */
    backendId?: string;
};
type ZonedApiListBackendsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Name of the backend to filter for. */
    name?: string;
    /** Sort order of backends in the response. */
    orderBy?: ListBackendsRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of backends to return. */
    pageSize?: number;
};
type ZonedApiListCertificatesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Sort order of certificates in the response. */
    orderBy?: ListCertificatesRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of certificates to return. */
    pageSize?: number;
    /**
     * Certificate name to filter for, only certificates of this name will be
     * returned.
     */
    name?: string;
};
type ZonedApiListFrontendsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Name of the frontend to filter for. */
    name?: string;
    /** Sort order of frontends in the response. */
    orderBy?: ListFrontendsRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of frontends to return. */
    pageSize?: number;
};
type ZonedApiListIPsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Number of IP addresses to return. */
    pageSize?: number;
    /** IP address to filter for. */
    ipAddress?: string;
    /**
     * Organization ID to filter for, only Load Balancer IP addresses from this
     * Organization will be returned.
     */
    organizationId?: string;
    /**
     * Project ID to filter for, only Load Balancer IP addresses from this Project
     * will be returned.
     */
    projectId?: string;
    /** IP type to filter for. */
    ipType?: ListIpsRequestIpType;
};
type ZonedApiListLbPrivateNetworksRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Sort order of Private Network objects in the response. */
    orderBy?: ListPrivateNetworksRequestOrderBy$2;
    /** Number of objects to return. */
    pageSize?: number;
    /** The page number to return, from the paginated results. */
    page?: number;
};
type ZonedApiListLbTypesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** The number of items to return. */
    pageSize?: number;
};
type ZonedApiListLbsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer name to filter for. */
    name?: string;
    /** Sort order of Load Balancers in the response. */
    orderBy?: ListLbsRequestOrderBy;
    /** Number of Load Balancers to return. */
    pageSize?: number;
    /** Page number to return, from the paginated results. */
    page?: number;
    /**
     * Organization ID to filter for, only Load Balancers from this Organization
     * will be returned.
     */
    organizationId?: string;
    /**
     * Project ID to filter for, only Load Balancers from this Project will be
     * returned.
     */
    projectId?: string;
};
type ZonedApiListRoutesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Sort order of routes in the response. */
    orderBy?: ListRoutesRequestOrderBy;
    /** The number of route objects to return. */
    pageSize?: number;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** Frontend ID to filter for, only Routes from this Frontend will be returned. */
    frontendId?: string;
};
type ZonedApiListSubscriberRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Sort order of subscribers in the response. */
    orderBy?: ListSubscriberRequestOrderBy;
    /** The page number to return, from the paginated results. */
    page?: number;
    /** The number of items to return. */
    pageSize?: number;
    /** Subscriber name to search for. */
    name?: string;
    /** Filter subscribers by Organization ID. */
    organizationId?: string;
    /** Filter subscribers by Project ID. */
    projectId?: string;
};
type ZonedApiMigrateLbRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /**
     * Load Balancer type to migrate to (use the List all Load Balancer offer
     * types endpoint to get a list of available offer types).
     */
    type: string;
};
type ZonedApiReleaseIpRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** IP address ID. */
    ipId: string;
};
type ZonedApiRemoveBackendServersRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Backend ID. */
    backendId: string;
    /** List of IP addresses to remove from backend servers. */
    serverIp: string[];
};
type ZonedApiSetAclsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Frontend ID. */
    frontendId: string;
    /**
     * List of ACLs for this frontend. Any other existing ACLs on this frontend
     * will be removed.
     */
    acls: AclSpec[];
};
type ZonedApiSetBackendServersRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Backend ID. */
    backendId: string;
    /**
     * List of IP addresses for backend servers. Any other existing backend
     * servers will be removed.
     */
    serverIp: string[];
};
type ZonedApiSubscribeToLbRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Subscriber ID. */
    subscriberId: string;
};
type ZonedApiUnsubscribeFromLbRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
};
type ZonedApiUpdateAclRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ACL ID. */
    aclId: string;
    /** ACL name. */
    name: string;
    /** Action to take when incoming traffic matches an ACL filter. */
    action: AclAction;
    /**
     * ACL match filter object. One of `ip_subnet` or `http_filter` &
     * `http_filter_value` are required.
     */
    match?: AclMatch;
    /**
     * Priority of this ACL (ACLs are applied in ascending order, 0 is the first
     * ACL executed).
     */
    index: number;
    /** ACL description. */
    description?: string;
};
type ZonedApiUpdateBackendRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Backend ID. */
    backendId: string;
    /** Backend name. */
    name: string;
    /**
     * Protocol to be used by the backend when forwarding traffic to backend
     * servers.
     */
    forwardProtocol: Protocol;
    /** Port to be used by the backend when forwarding traffic to backend servers. */
    forwardPort: number;
    /**
     * Load balancing algorithm to be used when determining which backend server
     * to forward new traffic to.
     */
    forwardPortAlgorithm: ForwardPortAlgorithm;
    /**
     * Defines whether to activate sticky sessions (binding a particular session
     * to a particular backend server) and the method to use if so. None disables
     * sticky sessions. Cookie-based uses an HTTP cookie to stick a session to a
     * backend server. Table-based uses the source (client) IP address to stick a
     * session to a backend server.
     */
    stickySessions: StickySessionsType;
    /** Cookie name for cookie-based sticky sessions. */
    stickySessionsCookieName: string;
    /** @deprecated Deprecated in favor of proxy_protocol field. */
    sendProxyV2?: boolean;
    /** Maximum allowed time for a backend server to process a request. */
    timeoutServer?: string;
    /** Maximum allowed time for establishing a connection to a backend server. */
    timeoutConnect?: string;
    /**
     * Maximum allowed tunnel inactivity time after Websocket is established
     * (takes precedence over client and server timeout).
     */
    timeoutTunnel?: string;
    /** Action to take when a backend server is marked as down. */
    onMarkedDownAction?: OnMarkedDownAction;
    /**
     * Protocol to use between the Load Balancer and backend servers. Allows the
     * backend servers to be informed of the client's real IP address. The PROXY
     * protocol must be supported by the backend servers' software.
     */
    proxyProtocol?: ProxyProtocol;
    /**
     * Scaleway S3 bucket website to be served as failover if all backend servers
     * are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
     */
    failoverHost?: string;
    /**
     * Defines whether to enable SSL bridging between the Load Balancer and
     * backend servers.
     */
    sslBridging?: boolean;
    /** Defines whether the server certificate verification should be ignored. */
    ignoreSslServerVerify?: boolean;
    /** Whether to use another backend server on each attempt. */
    redispatchAttemptCount?: number;
    /** Number of retries when a backend server connection failed. */
    maxRetries?: number;
    /** Maximum number of connections allowed per backend server. */
    maxConnections?: number;
    /**
     * Maximum time for a request to be left pending in queue when
     * `max_connections` is reached.
     */
    timeoutQueue?: string;
};
type ZonedApiUpdateCertificateRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Certificate ID. */
    certificateId: string;
    /** Certificate name. */
    name: string;
};
type ZonedApiUpdateFrontendRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Frontend ID. */
    frontendId: string;
    /** Frontend name. */
    name: string;
    /** Port the frontend should listen on. */
    inboundPort: number;
    /** Backend ID (ID of the backend the frontend should pass traffic to). */
    backendId: string;
    /** Maximum allowed inactivity time on the client side. */
    timeoutClient?: string;
    /** @deprecated Certificate ID, deprecated in favor of certificate_ids array. */
    certificateId?: string;
    /** List of SSL/TLS certificate IDs to bind to the frontend. */
    certificateIds?: string[];
    /** Defines whether to enable HTTP/3 protocol on the frontend. */
    enableHttp3: boolean;
};
type ZonedApiUpdateHealthCheckRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Backend ID. */
    backendId: string;
    /** Port to use for the backend server health check. */
    port: number;
    /** Time to wait between two consecutive health checks. */
    checkDelay?: string;
    /** Maximum time a backend server has to reply to the health check. */
    checkTimeout?: string;
    /**
     * Number of consecutive unsuccessful health checks after which the server
     * will be considered dead.
     */
    checkMaxRetries: number;
    /** Defines whether proxy protocol should be activated for the health check. */
    checkSendProxy: boolean;
    /**
     * Object to configure a basic TCP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    tcpConfig?: HealthCheckTcpConfig;
    /**
     * Object to configure a MySQL health check. The check requires MySQL >=3.22,
     * for older versions, use a TCP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    mysqlConfig?: HealthCheckMysqlConfig;
    /**
     * Object to configure a PostgreSQL health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    pgsqlConfig?: HealthCheckPgsqlConfig;
    /**
     * Object to configure an LDAP health check. The response is analyzed to find
     * the LDAPv3 response message.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    ldapConfig?: HealthCheckLdapConfig;
    /**
     * Object to configure a Redis health check. The response is analyzed to find
     * the +PONG response message.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    redisConfig?: HealthCheckRedisConfig;
    /**
     * Object to configure an HTTP health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    httpConfig?: HealthCheckHttpConfig;
    /**
     * Object to configure an HTTPS health check.
     *
     * One-of ('config'): at most one of 'tcpConfig', 'mysqlConfig',
     * 'pgsqlConfig', 'ldapConfig', 'redisConfig', 'httpConfig', 'httpsConfig'
     * could be set.
     */
    httpsConfig?: HealthCheckHttpsConfig;
    /**
     * Time to wait between two consecutive health checks when a backend server is
     * in a transient state (going UP or DOWN).
     */
    transientCheckDelay?: string;
};
type ZonedApiUpdateIpRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** IP address ID. */
    ipId: string;
    /** Reverse DNS (domain name) for the IP address. */
    reverse?: string;
    /** ID of the server on which to attach the flexible IP. */
    lbId?: string;
};
type ZonedApiUpdateLbRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Load Balancer ID. */
    lbId: string;
    /** Load Balancer name. */
    name: string;
    /** Load Balancer description. */
    description: string;
    /** List of tags for the Load Balancer. */
    tags?: string[];
    /**
     * Determines the minimal SSL version which needs to be supported on the
     * client side, in an SSL/TLS offloading context. Intermediate is suitable for
     * general-purpose servers with a variety of clients, recommended for almost
     * all systems. Modern is suitable for services with clients that support TLS
     * 1.3 and don't need backward compatibility. Old is compatible with a small
     * number of very old clients and should be used only as a last resort.
     */
    sslCompatibilityLevel?: SSLCompatibilityLevel;
};
type ZonedApiUpdateRouteRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Route ID. */
    routeId: string;
    /** ID of the target backend for the route. */
    backendId: string;
    /**
     * Object defining the match condition for a route to be applied. If an
     * incoming client session matches the specified condition (i.e. it has a
     * matching SNI value or HTTP Host header value), it will be passed to the
     * target backend.
     */
    match?: RouteMatch;
};
type ZonedApiUpdateSubscriberRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Subscriber ID. */
    subscriberId: string;
    /** Subscriber name. */
    name: string;
    /**
     * Email address configuration.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    emailConfig?: SubscriberEmailConfig;
    /**
     * Webhook URI configuration.
     *
     * One-of ('config'): at most one of 'emailConfig', 'webhookConfig' could be
     * set.
     */
    webhookConfig?: SubscriberWebhookConfig;
};

/**
 * Load Balancer API.
 *
 * This API allows you to manage your Scaleway Load Balancer services.
 */
declare class ZonedAPI extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    protected pageOfListLbs: (request?: Readonly<ZonedApiListLbsRequest>) => Promise<ListLbsResponse>;
    /**
     * List Load Balancers. List all Load Balancers in the specified zone, for a
     * Scaleway Organization or Scaleway Project. By default, the Load Balancers
     * returned in the list are ordered by creation date in ascending order,
     * though this can be modified via the `order_by` field.
     *
     * @param request - The request {@link ZonedApiListLbsRequest}
     * @returns A Promise of ListLbsResponse
     */
    listLbs: (request?: Readonly<ZonedApiListLbsRequest>) => Promise<ListLbsResponse> & {
        all: () => Promise<Lb[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Lb[], void, void>;
    };
    /**
     * Create a Load Balancer. Create a new Load Balancer. Note that the Load
     * Balancer will be created without frontends or backends; these must be
     * created separately via the dedicated endpoints.
     *
     * @param request - The request {@link ZonedApiCreateLbRequest}
     * @returns A Promise of Lb
     */
    createLb: (request: Readonly<ZonedApiCreateLbRequest>) => Promise<Lb>;
    /**
     * Get a Load Balancer. Retrieve information about an existing Load Balancer,
     * specified by its Load Balancer ID. Its full details, including name, status
     * and IP address, are returned in the response object.
     *
     * @param request - The request {@link ZonedApiGetLbRequest}
     * @returns A Promise of Lb
     */
    getLb: (request: Readonly<ZonedApiGetLbRequest>) => Promise<Lb>;
    /**
     * Waits for {@link Lb} to be in a final state.
     *
     * @param request - The request {@link ZonedApiGetLbRequest}
     * @param options - The waiting options
     * @returns A Promise of Lb
     */
    waitForLb: (request: Readonly<ZonedApiGetLbRequest>, options?: Readonly<WaitForOptions<Lb>>) => Promise<Lb>;
    /**
     * Update a Load Balancer. Update the parameters of an existing Load Balancer,
     * specified by its Load Balancer ID. Note that the request type is PUT and
     * not PATCH. You must set all parameters.
     *
     * @param request - The request {@link ZonedApiUpdateLbRequest}
     * @returns A Promise of Lb
     */
    updateLb: (request: Readonly<ZonedApiUpdateLbRequest>) => Promise<Lb>;
    /**
     * Delete a Load Balancer. Delete an existing Load Balancer, specified by its
     * Load Balancer ID. Deleting a Load Balancer is permanent, and cannot be
     * undone. The Load Balancer's flexible IP address can either be deleted with
     * the Load Balancer, or kept in your account for future use.
     *
     * @param request - The request {@link ZonedApiDeleteLbRequest}
     */
    deleteLb: (request: Readonly<ZonedApiDeleteLbRequest>) => Promise<void>;
    /**
     * Migrate a Load Balancer. Migrate an existing Load Balancer from one
     * commercial type to another. Allows you to scale your Load Balancer up or
     * down in terms of bandwidth or multi-cloud provision.
     *
     * @param request - The request {@link ZonedApiMigrateLbRequest}
     * @returns A Promise of Lb
     */
    migrateLb: (request: Readonly<ZonedApiMigrateLbRequest>) => Promise<Lb>;
    protected pageOfListIPs: (request?: Readonly<ZonedApiListIPsRequest>) => Promise<ListIpsResponse>;
    /**
     * List IP addresses. List the Load Balancer flexible IP addresses held in the
     * account (filtered by Organization ID or Project ID). It is also possible to
     * search for a specific IP address.
     *
     * @param request - The request {@link ZonedApiListIPsRequest}
     * @returns A Promise of ListIpsResponse
     */
    listIPs: (request?: Readonly<ZonedApiListIPsRequest>) => Promise<ListIpsResponse> & {
        all: () => Promise<Ip[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Ip[], void, void>;
    };
    /**
     * Create an IP address. Create a new Load Balancer flexible IP address, in
     * the specified Scaleway Project. This can be attached to new Load Balancers
     * created in the future.
     *
     * @param request - The request {@link ZonedApiCreateIpRequest}
     * @returns A Promise of Ip
     */
    createIp: (request: Readonly<ZonedApiCreateIpRequest>) => Promise<Ip>;
    /**
     * Get an IP address. Retrieve the full details of a Load Balancer flexible IP
     * address.
     *
     * @param request - The request {@link ZonedApiGetIpRequest}
     * @returns A Promise of Ip
     */
    getIp: (request: Readonly<ZonedApiGetIpRequest>) => Promise<Ip>;
    /**
     * Delete an IP address. Delete a Load Balancer flexible IP address. This
     * action is irreversible, and cannot be undone.
     *
     * @param request - The request {@link ZonedApiReleaseIpRequest}
     */
    releaseIp: (request: Readonly<ZonedApiReleaseIpRequest>) => Promise<void>;
    /**
     * Update an IP address. Update the reverse DNS of a Load Balancer flexible IP
     * address.
     *
     * @param request - The request {@link ZonedApiUpdateIpRequest}
     * @returns A Promise of Ip
     */
    updateIp: (request: Readonly<ZonedApiUpdateIpRequest>) => Promise<Ip>;
    protected pageOfListBackends: (request: Readonly<ZonedApiListBackendsRequest>) => Promise<ListBackendsResponse>;
    /**
     * List the backends of a given Load Balancer. List all the backends of a Load
     * Balancer, specified by its Load Balancer ID. By default, results are
     * returned in ascending order by the creation date of each backend. The
     * response is an array of backend objects, containing full details of each
     * one including their configuration parameters such as protocol, port and
     * forwarding algorithm.
     *
     * @param request - The request {@link ZonedApiListBackendsRequest}
     * @returns A Promise of ListBackendsResponse
     */
    listBackends: (request: Readonly<ZonedApiListBackendsRequest>) => Promise<ListBackendsResponse> & {
        all: () => Promise<Backend[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Backend[], void, void>;
    };
    /**
     * Create a backend for a given Load Balancer. Create a new backend for a
     * given Load Balancer, specifying its full configuration including protocol,
     * port and forwarding algorithm.
     *
     * @param request - The request {@link ZonedApiCreateBackendRequest}
     * @returns A Promise of Backend
     */
    createBackend: (request: Readonly<ZonedApiCreateBackendRequest>) => Promise<Backend>;
    /**
     * Get a backend of a given Load Balancer. Get the full details of a given
     * backend, specified by its backend ID. The response contains the backend's
     * full configuration parameters including protocol, port and forwarding
     * algorithm.
     *
     * @param request - The request {@link ZonedApiGetBackendRequest}
     * @returns A Promise of Backend
     */
    getBackend: (request: Readonly<ZonedApiGetBackendRequest>) => Promise<Backend>;
    /**
     * Update a backend of a given Load Balancer. Update a backend of a given Load
     * Balancer, specified by its backend ID. Note that the request type is PUT
     * and not PATCH. You must set all parameters.
     *
     * @param request - The request {@link ZonedApiUpdateBackendRequest}
     * @returns A Promise of Backend
     */
    updateBackend: (request: Readonly<ZonedApiUpdateBackendRequest>) => Promise<Backend>;
    /**
     * Delete a backend of a given Load Balancer. Delete a backend of a given Load
     * Balancer, specified by its backend ID. This action is irreversible and
     * cannot be undone.
     *
     * @param request - The request {@link ZonedApiDeleteBackendRequest}
     */
    deleteBackend: (request: Readonly<ZonedApiDeleteBackendRequest>) => Promise<void>;
    /**
     * Add a set of backend servers to a given backend. For a given backend
     * specified by its backend ID, add a set of backend servers (identified by
     * their IP addresses) it should forward traffic to. These will be appended to
     * any existing set of backend servers for this backend.
     *
     * @param request - The request {@link ZonedApiAddBackendServersRequest}
     * @returns A Promise of Backend
     */
    addBackendServers: (request: Readonly<ZonedApiAddBackendServersRequest>) => Promise<Backend>;
    /**
     * Remove a set of servers for a given backend. For a given backend specified
     * by its backend ID, remove the specified backend servers (identified by
     * their IP addresses) so that it no longer forwards traffic to them.
     *
     * @param request - The request {@link ZonedApiRemoveBackendServersRequest}
     * @returns A Promise of Backend
     */
    removeBackendServers: (request: Readonly<ZonedApiRemoveBackendServersRequest>) => Promise<Backend>;
    /**
     * Define all backend servers for a given backend. For a given backend
     * specified by its backend ID, define the set of backend servers (identified
     * by their IP addresses) that it should forward traffic to. Any existing
     * backend servers configured for this backend will be removed.
     *
     * @param request - The request {@link ZonedApiSetBackendServersRequest}
     * @returns A Promise of Backend
     */
    setBackendServers: (request: Readonly<ZonedApiSetBackendServersRequest>) => Promise<Backend>;
    /**
     * Update a health check for a given backend. Update the configuration of the
     * health check performed by a given backend to verify the health of its
     * backend servers, identified by its backend ID. Note that the request type
     * is PUT and not PATCH. You must set all parameters.
     *
     * @param request - The request {@link ZonedApiUpdateHealthCheckRequest}
     * @returns A Promise of HealthCheck
     */
    updateHealthCheck: (request: Readonly<ZonedApiUpdateHealthCheckRequest>) => Promise<HealthCheck>;
    protected pageOfListFrontends: (request: Readonly<ZonedApiListFrontendsRequest>) => Promise<ListFrontendsResponse>;
    /**
     * List frontends of a given Load Balancer. List all the frontends of a Load
     * Balancer, specified by its Load Balancer ID. By default, results are
     * returned in ascending order by the creation date of each frontend. The
     * response is an array of frontend objects, containing full details of each
     * one including the port they listen on and the backend they are attached
     * to.
     *
     * @param request - The request {@link ZonedApiListFrontendsRequest}
     * @returns A Promise of ListFrontendsResponse
     */
    listFrontends: (request: Readonly<ZonedApiListFrontendsRequest>) => Promise<ListFrontendsResponse> & {
        all: () => Promise<Frontend[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Frontend[], void, void>;
    };
    /**
     * Create a frontend in a given Load Balancer. Create a new frontend for a
     * given Load Balancer, specifying its configuration including the port it
     * should listen on and the backend to attach it to.
     *
     * @param request - The request {@link ZonedApiCreateFrontendRequest}
     * @returns A Promise of Frontend
     */
    createFrontend: (request: Readonly<ZonedApiCreateFrontendRequest>) => Promise<Frontend>;
    /**
     * Get a frontend. Get the full details of a given frontend, specified by its
     * frontend ID. The response contains the frontend's full configuration
     * parameters including the backend it is attached to, the port it listens on,
     * and any certificates it has.
     *
     * @param request - The request {@link ZonedApiGetFrontendRequest}
     * @returns A Promise of Frontend
     */
    getFrontend: (request: Readonly<ZonedApiGetFrontendRequest>) => Promise<Frontend>;
    /**
     * Update a frontend. Update a given frontend, specified by its frontend ID.
     * You can update configuration parameters including its name and the port it
     * listens on. Note that the request type is PUT and not PATCH. You must set
     * all parameters.
     *
     * @param request - The request {@link ZonedApiUpdateFrontendRequest}
     * @returns A Promise of Frontend
     */
    updateFrontend: (request: Readonly<ZonedApiUpdateFrontendRequest>) => Promise<Frontend>;
    /**
     * Delete a frontend. Delete a given frontend, specified by its frontend ID.
     * This action is irreversible and cannot be undone.
     *
     * @param request - The request {@link ZonedApiDeleteFrontendRequest}
     */
    deleteFrontend: (request: Readonly<ZonedApiDeleteFrontendRequest>) => Promise<void>;
    protected pageOfListRoutes: (request?: Readonly<ZonedApiListRoutesRequest>) => Promise<ListRoutesResponse>;
    /**
     * List all routes. List all routes for a given frontend. The response is an
     * array of routes, each one with a specified backend to direct to if a
     * certain condition is matched (based on the value of the SNI field or HTTP
     * Host header).
     *
     * @param request - The request {@link ZonedApiListRoutesRequest}
     * @returns A Promise of ListRoutesResponse
     */
    listRoutes: (request?: Readonly<ZonedApiListRoutesRequest>) => Promise<ListRoutesResponse> & {
        all: () => Promise<Route[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Route[], void, void>;
    };
    /**
     * Create a route. Create a new route on a given frontend. To configure a
     * route, specify the backend to direct to if a certain condition is matched
     * (based on the value of the SNI field or HTTP Host header).
     *
     * @param request - The request {@link ZonedApiCreateRouteRequest}
     * @returns A Promise of Route
     */
    createRoute: (request: Readonly<ZonedApiCreateRouteRequest>) => Promise<Route>;
    /**
     * Get a route. Retrieve information about an existing route, specified by its
     * route ID. Its full details, origin frontend, target backend and match
     * condition, are returned in the response object.
     *
     * @param request - The request {@link ZonedApiGetRouteRequest}
     * @returns A Promise of Route
     */
    getRoute: (request: Readonly<ZonedApiGetRouteRequest>) => Promise<Route>;
    /**
     * Update a route. Update the configuration of an existing route, specified by
     * its route ID.
     *
     * @param request - The request {@link ZonedApiUpdateRouteRequest}
     * @returns A Promise of Route
     */
    updateRoute: (request: Readonly<ZonedApiUpdateRouteRequest>) => Promise<Route>;
    /**
     * Delete a route. Delete an existing route, specified by its route ID.
     * Deleting a route is permanent, and cannot be undone.
     *
     * @param request - The request {@link ZonedApiDeleteRouteRequest}
     */
    deleteRoute: (request: Readonly<ZonedApiDeleteRouteRequest>) => Promise<void>;
    /**
     * Get usage statistics of a given Load Balancer.
     *
     * @deprecated
     * @param request - The request {@link ZonedApiGetLbStatsRequest}
     * @returns A Promise of LbStats
     */
    getLbStats: (request: Readonly<ZonedApiGetLbStatsRequest>) => Promise<LbStats>;
    protected pageOfListBackendStats: (request: Readonly<ZonedApiListBackendStatsRequest>) => Promise<ListBackendStatsResponse>;
    /**
     * List backend server statistics. List information about your backend
     * servers, including their state and the result of their last health check.
     *
     * @param request - The request {@link ZonedApiListBackendStatsRequest}
     * @returns A Promise of ListBackendStatsResponse
     */
    listBackendStats: (request: Readonly<ZonedApiListBackendStatsRequest>) => Promise<ListBackendStatsResponse> & {
        all: () => Promise<BackendServerStats[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<BackendServerStats[], void, void>;
    };
    protected pageOfListAcls: (request: Readonly<ZonedApiListAclsRequest>) => Promise<ListAclResponse>;
    /**
     * List ACLs for a given frontend. List the ACLs for a given frontend,
     * specified by its frontend ID. The response is an array of ACL objects, each
     * one representing an ACL that denies or allows traffic based on certain
     * conditions.
     *
     * @param request - The request {@link ZonedApiListAclsRequest}
     * @returns A Promise of ListAclResponse
     */
    listAcls: (request: Readonly<ZonedApiListAclsRequest>) => Promise<ListAclResponse> & {
        all: () => Promise<Acl[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Acl[], void, void>;
    };
    /**
     * Create an ACL for a given frontend. Create a new ACL for a given frontend.
     * Each ACL must have a name, an action to perform (allow or deny), and a
     * match rule (the action is carried out when the incoming traffic matches the
     * rule).
     *
     * @param request - The request {@link ZonedApiCreateAclRequest}
     * @returns A Promise of Acl
     */
    createAcl: (request: Readonly<ZonedApiCreateAclRequest>) => Promise<Acl>;
    /**
     * Get an ACL. Get information for a particular ACL, specified by its ACL ID.
     * The response returns full details of the ACL, including its name, action,
     * match rule and frontend.
     *
     * @param request - The request {@link ZonedApiGetAclRequest}
     * @returns A Promise of Acl
     */
    getAcl: (request: Readonly<ZonedApiGetAclRequest>) => Promise<Acl>;
    /**
     * Update an ACL. Update a particular ACL, specified by its ACL ID. You can
     * update details including its name, action and match rule.
     *
     * @param request - The request {@link ZonedApiUpdateAclRequest}
     * @returns A Promise of Acl
     */
    updateAcl: (request: Readonly<ZonedApiUpdateAclRequest>) => Promise<Acl>;
    /**
     * Delete an ACL. Delete an ACL, specified by its ACL ID. Deleting an ACL is
     * irreversible and cannot be undone.
     *
     * @param request - The request {@link ZonedApiDeleteAclRequest}
     */
    deleteAcl: (request: Readonly<ZonedApiDeleteAclRequest>) => Promise<void>;
    /**
     * Define all ACLs for a given frontend. For a given frontend specified by its
     * frontend ID, define and add the complete set of ACLS for that frontend. Any
     * existing ACLs on this frontend will be removed.
     *
     * @param request - The request {@link ZonedApiSetAclsRequest}
     * @returns A Promise of SetAclsResponse
     */
    setAcls: (request: Readonly<ZonedApiSetAclsRequest>) => Promise<SetAclsResponse>;
    /**
     * Create an SSL/TLS certificate. Generate a new SSL/TLS certificate for a
     * given Load Balancer. You can choose to create a Let's Encrypt certificate,
     * or import a custom certificate.
     *
     * @param request - The request {@link ZonedApiCreateCertificateRequest}
     * @returns A Promise of Certificate
     */
    createCertificate: (request: Readonly<ZonedApiCreateCertificateRequest>) => Promise<Certificate>;
    protected pageOfListCertificates: (request: Readonly<ZonedApiListCertificatesRequest>) => Promise<ListCertificatesResponse>;
    /**
     * List all SSL/TLS certificates on a given Load Balancer. List all the
     * SSL/TLS certificates on a given Load Balancer. The response is an array of
     * certificate objects, which are by default listed in ascending order of
     * creation date.
     *
     * @param request - The request {@link ZonedApiListCertificatesRequest}
     * @returns A Promise of ListCertificatesResponse
     */
    listCertificates: (request: Readonly<ZonedApiListCertificatesRequest>) => Promise<ListCertificatesResponse> & {
        all: () => Promise<Certificate[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Certificate[], void, void>;
    };
    /**
     * Get an SSL/TLS certificate. Get information for a particular SSL/TLS
     * certificate, specified by its certificate ID. The response returns full
     * details of the certificate, including its type, main domain name, and
     * alternative domain names.
     *
     * @param request - The request {@link ZonedApiGetCertificateRequest}
     * @returns A Promise of Certificate
     */
    getCertificate: (request: Readonly<ZonedApiGetCertificateRequest>) => Promise<Certificate>;
    /**
     * Waits for {@link Certificate} to be in a final state.
     *
     * @param request - The request {@link ZonedApiGetCertificateRequest}
     * @param options - The waiting options
     * @returns A Promise of Certificate
     */
    waitForCertificate: (request: Readonly<ZonedApiGetCertificateRequest>, options?: Readonly<WaitForOptions<Certificate>>) => Promise<Certificate>;
    /**
     * Update an SSL/TLS certificate. Update the name of a particular SSL/TLS
     * certificate, specified by its certificate ID.
     *
     * @param request - The request {@link ZonedApiUpdateCertificateRequest}
     * @returns A Promise of Certificate
     */
    updateCertificate: (request: Readonly<ZonedApiUpdateCertificateRequest>) => Promise<Certificate>;
    /**
     * Delete an SSL/TLS certificate. Delete an SSL/TLS certificate, specified by
     * its certificate ID. Deleting a certificate is irreversible and cannot be
     * undone.
     *
     * @param request - The request {@link ZonedApiDeleteCertificateRequest}
     */
    deleteCertificate: (request: Readonly<ZonedApiDeleteCertificateRequest>) => Promise<void>;
    protected pageOfListLbTypes: (request?: Readonly<ZonedApiListLbTypesRequest>) => Promise<ListLbTypesResponse>;
    /**
     * List all Load Balancer offer types. List all the different commercial Load
     * Balancer types. The response includes an array of offer types, each with a
     * name, description, and information about its stock availability.
     *
     * @param request - The request {@link ZonedApiListLbTypesRequest}
     * @returns A Promise of ListLbTypesResponse
     */
    listLbTypes: (request?: Readonly<ZonedApiListLbTypesRequest>) => Promise<ListLbTypesResponse> & {
        all: () => Promise<LbType[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<LbType[], void, void>;
    };
    /**
     * Create a subscriber. Create a new subscriber, either with an email
     * configuration or a webhook configuration, for a specified Scaleway
     * Project.
     *
     * @param request - The request {@link ZonedApiCreateSubscriberRequest}
     * @returns A Promise of Subscriber
     */
    createSubscriber: (request: Readonly<ZonedApiCreateSubscriberRequest>) => Promise<Subscriber>;
    /**
     * Get a subscriber. Retrieve information about an existing subscriber,
     * specified by its subscriber ID. Its full details, including name and
     * email/webhook configuration, are returned in the response object.
     *
     * @param request - The request {@link ZonedApiGetSubscriberRequest}
     * @returns A Promise of Subscriber
     */
    getSubscriber: (request: Readonly<ZonedApiGetSubscriberRequest>) => Promise<Subscriber>;
    protected pageOfListSubscriber: (request?: Readonly<ZonedApiListSubscriberRequest>) => Promise<ListSubscriberResponse>;
    /**
     * List all subscribers. List all subscribers to Load Balancer alerts. By
     * default, returns all subscribers to Load Balancer alerts for the
     * Organization associated with the authentication token used for the
     * request.
     *
     * @param request - The request {@link ZonedApiListSubscriberRequest}
     * @returns A Promise of ListSubscriberResponse
     */
    listSubscriber: (request?: Readonly<ZonedApiListSubscriberRequest>) => Promise<ListSubscriberResponse> & {
        all: () => Promise<Subscriber[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Subscriber[], void, void>;
    };
    /**
     * Update a subscriber. Update the parameters of a given subscriber (e.g.
     * name, webhook configuration, email configuration), specified by its
     * subscriber ID.
     *
     * @param request - The request {@link ZonedApiUpdateSubscriberRequest}
     * @returns A Promise of Subscriber
     */
    updateSubscriber: (request: Readonly<ZonedApiUpdateSubscriberRequest>) => Promise<Subscriber>;
    /**
     * Delete a subscriber. Delete an existing subscriber, specified by its
     * subscriber ID. Deleting a subscriber is permanent, and cannot be undone.
     *
     * @param request - The request {@link ZonedApiDeleteSubscriberRequest}
     */
    deleteSubscriber: (request: Readonly<ZonedApiDeleteSubscriberRequest>) => Promise<void>;
    /**
     * Subscribe a subscriber to alerts for a given Load Balancer. Subscribe an
     * existing subscriber to alerts for a given Load Balancer.
     *
     * @param request - The request {@link ZonedApiSubscribeToLbRequest}
     * @returns A Promise of Lb
     */
    subscribeToLb: (request: Readonly<ZonedApiSubscribeToLbRequest>) => Promise<Lb>;
    /**
     * Unsubscribe a subscriber from alerts for a given Load Balancer. Unsubscribe
     * a subscriber from alerts for a given Load Balancer. The subscriber is not
     * deleted, and can be resubscribed in the future if necessary.
     *
     * @param request - The request {@link ZonedApiUnsubscribeFromLbRequest}
     * @returns A Promise of Lb
     */
    unsubscribeFromLb: (request: Readonly<ZonedApiUnsubscribeFromLbRequest>) => Promise<Lb>;
    protected pageOfListLbPrivateNetworks: (request: Readonly<ZonedApiListLbPrivateNetworksRequest>) => Promise<ListLbPrivateNetworksResponse>;
    /**
     * List Private Networks attached to a Load Balancer. List the Private
     * Networks attached to a given Load Balancer, specified by its Load Balancer
     * ID. The response is an array of Private Network objects, giving information
     * including the status, configuration, name and creation date of each Private
     * Network.
     *
     * @param request - The request {@link ZonedApiListLbPrivateNetworksRequest}
     * @returns A Promise of ListLbPrivateNetworksResponse
     */
    listLbPrivateNetworks: (request: Readonly<ZonedApiListLbPrivateNetworksRequest>) => Promise<ListLbPrivateNetworksResponse> & {
        all: () => Promise<PrivateNetwork$3[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<PrivateNetwork$3[], void, void>;
    };
    /**
     * Attach a Load Balancer to a Private Network. Attach a specified Load
     * Balancer to a specified Private Network, defining a static or DHCP
     * configuration for the Load Balancer on the network.
     *
     * @param request - The request {@link ZonedApiAttachPrivateNetworkRequest}
     * @returns A Promise of PrivateNetwork
     */
    attachPrivateNetwork: (request: Readonly<ZonedApiAttachPrivateNetworkRequest>) => Promise<PrivateNetwork$3>;
    /**
     * Detach Load Balancer from Private Network. Detach a specified Load Balancer
     * from a specified Private Network.
     *
     * @param request - The request {@link ZonedApiDetachPrivateNetworkRequest}
     */
    detachPrivateNetwork: (request: Readonly<ZonedApiDetachPrivateNetworkRequest>) => Promise<void>;
}
/**
 * Load balancer API.
 *
 * This API allows you to manage your load balancer service.
 */
declare class API$d extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListLbs: (request?: Readonly<ListLbsRequest>) => Promise<ListLbsResponse>;
    /**
     * List load balancers.
     *
     * @param request - The request {@link ListLbsRequest}
     * @returns A Promise of ListLbsResponse
     */
    listLbs: (request?: Readonly<ListLbsRequest>) => Promise<ListLbsResponse> & {
        all: () => Promise<Lb[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Lb[], void, void>;
    };
    /**
     * Create a load balancer.
     *
     * @param request - The request {@link CreateLbRequest}
     * @returns A Promise of Lb
     */
    createLb: (request: Readonly<CreateLbRequest>) => Promise<Lb>;
    /**
     * Get a load balancer.
     *
     * @param request - The request {@link GetLbRequest}
     * @returns A Promise of Lb
     */
    getLb: (request: Readonly<GetLbRequest>) => Promise<Lb>;
    /**
     * Waits for {@link Lb} to be in a final state.
     *
     * @param request - The request {@link GetLbRequest}
     * @param options - The waiting options
     * @returns A Promise of Lb
     */
    waitForLb: (request: Readonly<GetLbRequest>, options?: Readonly<WaitForOptions<Lb>>) => Promise<Lb>;
    /**
     * Update a load balancer.
     *
     * @param request - The request {@link UpdateLbRequest}
     * @returns A Promise of Lb
     */
    updateLb: (request: Readonly<UpdateLbRequest>) => Promise<Lb>;
    /**
     * Delete a load balancer.
     *
     * @param request - The request {@link DeleteLbRequest}
     */
    deleteLb: (request: Readonly<DeleteLbRequest>) => Promise<void>;
    /**
     * Migrate a load balancer.
     *
     * @param request - The request {@link MigrateLbRequest}
     * @returns A Promise of Lb
     */
    migrateLb: (request: Readonly<MigrateLbRequest>) => Promise<Lb>;
    protected pageOfListIPs: (request?: Readonly<ListIPsRequest$1>) => Promise<ListIpsResponse>;
    /**
     * List IPs.
     *
     * @param request - The request {@link ListIPsRequest}
     * @returns A Promise of ListIpsResponse
     */
    listIPs: (request?: Readonly<ListIPsRequest$1>) => Promise<ListIpsResponse> & {
        all: () => Promise<Ip[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Ip[], void, void>;
    };
    /**
     * Create an IP.
     *
     * @param request - The request {@link CreateIpRequest}
     * @returns A Promise of Ip
     */
    createIp: (request: Readonly<CreateIpRequest>) => Promise<Ip>;
    /**
     * Get an IP.
     *
     * @param request - The request {@link GetIpRequest}
     * @returns A Promise of Ip
     */
    getIp: (request: Readonly<GetIpRequest>) => Promise<Ip>;
    /**
     * Delete an IP.
     *
     * @param request - The request {@link ReleaseIpRequest}
     */
    releaseIp: (request: Readonly<ReleaseIpRequest>) => Promise<void>;
    /**
     * Update an IP.
     *
     * @param request - The request {@link UpdateIpRequest}
     * @returns A Promise of Ip
     */
    updateIp: (request: Readonly<UpdateIpRequest>) => Promise<Ip>;
    protected pageOfListBackends: (request: Readonly<ListBackendsRequest>) => Promise<ListBackendsResponse>;
    /**
     * List backends in a given load balancer.
     *
     * @param request - The request {@link ListBackendsRequest}
     * @returns A Promise of ListBackendsResponse
     */
    listBackends: (request: Readonly<ListBackendsRequest>) => Promise<ListBackendsResponse> & {
        all: () => Promise<Backend[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Backend[], void, void>;
    };
    /**
     * Create a backend in a given load balancer.
     *
     * @param request - The request {@link CreateBackendRequest}
     * @returns A Promise of Backend
     */
    createBackend: (request: Readonly<CreateBackendRequest>) => Promise<Backend>;
    /**
     * Get a backend in a given load balancer.
     *
     * @param request - The request {@link GetBackendRequest}
     * @returns A Promise of Backend
     */
    getBackend: (request: Readonly<GetBackendRequest>) => Promise<Backend>;
    /**
     * Update a backend in a given load balancer.
     *
     * @param request - The request {@link UpdateBackendRequest}
     * @returns A Promise of Backend
     */
    updateBackend: (request: Readonly<UpdateBackendRequest>) => Promise<Backend>;
    /**
     * Delete a backend in a given load balancer.
     *
     * @param request - The request {@link DeleteBackendRequest}
     */
    deleteBackend: (request: Readonly<DeleteBackendRequest>) => Promise<void>;
    /**
     * Add a set of servers in a given backend.
     *
     * @param request - The request {@link AddBackendServersRequest}
     * @returns A Promise of Backend
     */
    addBackendServers: (request: Readonly<AddBackendServersRequest>) => Promise<Backend>;
    /**
     * Remove a set of servers for a given backend.
     *
     * @param request - The request {@link RemoveBackendServersRequest}
     * @returns A Promise of Backend
     */
    removeBackendServers: (request: Readonly<RemoveBackendServersRequest>) => Promise<Backend>;
    /**
     * Define all servers in a given backend.
     *
     * @param request - The request {@link SetBackendServersRequest}
     * @returns A Promise of Backend
     */
    setBackendServers: (request: Readonly<SetBackendServersRequest>) => Promise<Backend>;
    /**
     * Update an health check for a given backend.
     *
     * @param request - The request {@link UpdateHealthCheckRequest}
     * @returns A Promise of HealthCheck
     */
    updateHealthCheck: (request: Readonly<UpdateHealthCheckRequest>) => Promise<HealthCheck>;
    protected pageOfListFrontends: (request: Readonly<ListFrontendsRequest>) => Promise<ListFrontendsResponse>;
    /**
     * List frontends in a given load balancer.
     *
     * @param request - The request {@link ListFrontendsRequest}
     * @returns A Promise of ListFrontendsResponse
     */
    listFrontends: (request: Readonly<ListFrontendsRequest>) => Promise<ListFrontendsResponse> & {
        all: () => Promise<Frontend[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Frontend[], void, void>;
    };
    /**
     * Create a frontend in a given load balancer.
     *
     * @param request - The request {@link CreateFrontendRequest}
     * @returns A Promise of Frontend
     */
    createFrontend: (request: Readonly<CreateFrontendRequest>) => Promise<Frontend>;
    /**
     * Get a frontend.
     *
     * @param request - The request {@link GetFrontendRequest}
     * @returns A Promise of Frontend
     */
    getFrontend: (request: Readonly<GetFrontendRequest>) => Promise<Frontend>;
    /**
     * Update a frontend.
     *
     * @param request - The request {@link UpdateFrontendRequest}
     * @returns A Promise of Frontend
     */
    updateFrontend: (request: Readonly<UpdateFrontendRequest>) => Promise<Frontend>;
    /**
     * Delete a frontend.
     *
     * @param request - The request {@link DeleteFrontendRequest}
     */
    deleteFrontend: (request: Readonly<DeleteFrontendRequest>) => Promise<void>;
    protected pageOfListRoutes: (request?: Readonly<ListRoutesRequest>) => Promise<ListRoutesResponse>;
    /**
     * List all backend redirections.
     *
     * @param request - The request {@link ListRoutesRequest}
     * @returns A Promise of ListRoutesResponse
     */
    listRoutes: (request?: Readonly<ListRoutesRequest>) => Promise<ListRoutesResponse> & {
        all: () => Promise<Route[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Route[], void, void>;
    };
    /**
     * Create a backend redirection.
     *
     * @param request - The request {@link CreateRouteRequest}
     * @returns A Promise of Route
     */
    createRoute: (request: Readonly<CreateRouteRequest>) => Promise<Route>;
    /**
     * Get single backend redirection.
     *
     * @param request - The request {@link GetRouteRequest}
     * @returns A Promise of Route
     */
    getRoute: (request: Readonly<GetRouteRequest>) => Promise<Route>;
    /**
     * Edit a backend redirection.
     *
     * @param request - The request {@link UpdateRouteRequest}
     * @returns A Promise of Route
     */
    updateRoute: (request: Readonly<UpdateRouteRequest>) => Promise<Route>;
    /**
     * Delete a backend redirection.
     *
     * @param request - The request {@link DeleteRouteRequest}
     */
    deleteRoute: (request: Readonly<DeleteRouteRequest>) => Promise<void>;
    /**
     * Get usage statistics of a given load balancer.
     *
     * @deprecated
     * @param request - The request {@link GetLbStatsRequest}
     * @returns A Promise of LbStats
     */
    getLbStats: (request: Readonly<GetLbStatsRequest>) => Promise<LbStats>;
    protected pageOfListBackendStats: (request: Readonly<ListBackendStatsRequest>) => Promise<ListBackendStatsResponse>;
    /**
     * List backend server statistics.
     *
     * @param request - The request {@link ListBackendStatsRequest}
     * @returns A Promise of ListBackendStatsResponse
     */
    listBackendStats: (request: Readonly<ListBackendStatsRequest>) => Promise<ListBackendStatsResponse> & {
        all: () => Promise<BackendServerStats[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<BackendServerStats[], void, void>;
    };
    protected pageOfListAcls: (request: Readonly<ListAclsRequest>) => Promise<ListAclResponse>;
    /**
     * List ACL for a given frontend.
     *
     * @param request - The request {@link ListAclsRequest}
     * @returns A Promise of ListAclResponse
     */
    listAcls: (request: Readonly<ListAclsRequest>) => Promise<ListAclResponse> & {
        all: () => Promise<Acl[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Acl[], void, void>;
    };
    /**
     * Create an ACL for a given frontend.
     *
     * @param request - The request {@link CreateAclRequest}
     * @returns A Promise of Acl
     */
    createAcl: (request: Readonly<CreateAclRequest>) => Promise<Acl>;
    /**
     * Get an ACL.
     *
     * @param request - The request {@link GetAclRequest}
     * @returns A Promise of Acl
     */
    getAcl: (request: Readonly<GetAclRequest>) => Promise<Acl>;
    /**
     * Update an ACL.
     *
     * @param request - The request {@link UpdateAclRequest}
     * @returns A Promise of Acl
     */
    updateAcl: (request: Readonly<UpdateAclRequest>) => Promise<Acl>;
    /**
     * Delete an ACL.
     *
     * @param request - The request {@link DeleteAclRequest}
     */
    deleteAcl: (request: Readonly<DeleteAclRequest>) => Promise<void>;
    /**
     * Create a TLS certificate. Generate a new TLS certificate using Let's
     * Encrypt or import your certificate.
     *
     * @param request - The request {@link CreateCertificateRequest}
     * @returns A Promise of Certificate
     */
    createCertificate: (request: Readonly<CreateCertificateRequest>) => Promise<Certificate>;
    protected pageOfListCertificates: (request: Readonly<ListCertificatesRequest>) => Promise<ListCertificatesResponse>;
    /**
     * List all TLS certificates on a given load balancer.
     *
     * @param request - The request {@link ListCertificatesRequest}
     * @returns A Promise of ListCertificatesResponse
     */
    listCertificates: (request: Readonly<ListCertificatesRequest>) => Promise<ListCertificatesResponse> & {
        all: () => Promise<Certificate[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Certificate[], void, void>;
    };
    /**
     * Get a TLS certificate.
     *
     * @param request - The request {@link GetCertificateRequest}
     * @returns A Promise of Certificate
     */
    getCertificate: (request: Readonly<GetCertificateRequest>) => Promise<Certificate>;
    /**
     * Waits for {@link Certificate} to be in a final state.
     *
     * @param request - The request {@link GetCertificateRequest}
     * @param options - The waiting options
     * @returns A Promise of Certificate
     */
    waitForCertificate: (request: Readonly<GetCertificateRequest>, options?: Readonly<WaitForOptions<Certificate>>) => Promise<Certificate>;
    /**
     * Update a TLS certificate.
     *
     * @param request - The request {@link UpdateCertificateRequest}
     * @returns A Promise of Certificate
     */
    updateCertificate: (request: Readonly<UpdateCertificateRequest>) => Promise<Certificate>;
    /**
     * Delete a TLS certificate.
     *
     * @param request - The request {@link DeleteCertificateRequest}
     */
    deleteCertificate: (request: Readonly<DeleteCertificateRequest>) => Promise<void>;
    protected pageOfListLbTypes: (request?: Readonly<ListLbTypesRequest>) => Promise<ListLbTypesResponse>;
    /**
     * List all load balancer offer type.
     *
     * @param request - The request {@link ListLbTypesRequest}
     * @returns A Promise of ListLbTypesResponse
     */
    listLbTypes: (request?: Readonly<ListLbTypesRequest>) => Promise<ListLbTypesResponse> & {
        all: () => Promise<LbType[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<LbType[], void, void>;
    };
    /**
     * Create a subscriber, webhook or email.
     *
     * @param request - The request {@link CreateSubscriberRequest}
     * @returns A Promise of Subscriber
     */
    createSubscriber: (request: Readonly<CreateSubscriberRequest>) => Promise<Subscriber>;
    /**
     * Get a subscriber.
     *
     * @param request - The request {@link GetSubscriberRequest}
     * @returns A Promise of Subscriber
     */
    getSubscriber: (request: Readonly<GetSubscriberRequest>) => Promise<Subscriber>;
    protected pageOfListSubscriber: (request?: Readonly<ListSubscriberRequest>) => Promise<ListSubscriberResponse>;
    /**
     * List all subscriber.
     *
     * @param request - The request {@link ListSubscriberRequest}
     * @returns A Promise of ListSubscriberResponse
     */
    listSubscriber: (request?: Readonly<ListSubscriberRequest>) => Promise<ListSubscriberResponse> & {
        all: () => Promise<Subscriber[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Subscriber[], void, void>;
    };
    /**
     * Update a subscriber.
     *
     * @param request - The request {@link UpdateSubscriberRequest}
     * @returns A Promise of Subscriber
     */
    updateSubscriber: (request: Readonly<UpdateSubscriberRequest>) => Promise<Subscriber>;
    /**
     * Delete a subscriber.
     *
     * @param request - The request {@link DeleteSubscriberRequest}
     */
    deleteSubscriber: (request: Readonly<DeleteSubscriberRequest>) => Promise<void>;
    /**
     * Subscribe a subscriber to a given load balancer.
     *
     * @param request - The request {@link SubscribeToLbRequest}
     * @returns A Promise of Lb
     */
    subscribeToLb: (request: Readonly<SubscribeToLbRequest>) => Promise<Lb>;
    /**
     * Unsubscribe a subscriber from a given load balancer.
     *
     * @param request - The request {@link UnsubscribeFromLbRequest}
     * @returns A Promise of Lb
     */
    unsubscribeFromLb: (request: Readonly<UnsubscribeFromLbRequest>) => Promise<Lb>;
    protected pageOfListLbPrivateNetworks: (request: Readonly<ListLbPrivateNetworksRequest>) => Promise<ListLbPrivateNetworksResponse>;
    /**
     * List attached private network of load balancer.
     *
     * @param request - The request {@link ListLbPrivateNetworksRequest}
     * @returns A Promise of ListLbPrivateNetworksResponse
     */
    listLbPrivateNetworks: (request: Readonly<ListLbPrivateNetworksRequest>) => Promise<ListLbPrivateNetworksResponse> & {
        all: () => Promise<PrivateNetwork$3[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<PrivateNetwork$3[], void, void>;
    };
    /**
     * Add load balancer on instance private network.
     *
     * @param request - The request {@link AttachPrivateNetworkRequest}
     * @returns A Promise of PrivateNetwork
     */
    attachPrivateNetwork: (request: Readonly<AttachPrivateNetworkRequest>) => Promise<PrivateNetwork$3>;
    /**
     * Remove load balancer of private network.
     *
     * @param request - The request {@link DetachPrivateNetworkRequest}
     */
    detachPrivateNetwork: (request: Readonly<DetachPrivateNetworkRequest>) => Promise<void>;
}

type WaitForLbPrivateNetworksRequest = {
    /** Region to target. If none is passed will use default region from the config */
    region?: Region;
    /** Load balancer ID */
    lbId: string;
};
type ZonedWaitForLbPrivateNetworksRequest = {
    /** Zone to target. If none is passed will use default zone from the config */
    zone?: Zone;
    /** Load balancer ID */
    lbId: string;
};

declare class LbV1UtilsAPI extends API$d {
    /**
     * Waits for all private networks of a load balancer to be in a final state.
     *
     * @param request - The request {@link WaitForLbPrivateNetworksRequest}
     * @param options - The waiting options
     * @returns A Promise of ListLbPrivateNetworksResponse
     */
    waitForLbPrivateNetworks: (request: Readonly<WaitForLbPrivateNetworksRequest>, options?: Readonly<WaitForOptions<PrivateNetwork$3>>) => Promise<ListLbPrivateNetworksResponse>;
    /**
     * Waits for all instances of a load balancer to be in a final state.
     *
     * @param request - The request {@link GetLbRequest}
     * @param options - The waiting options
     * @returns A Promise of Lb
     */
    waitForLbInstances: (request: Readonly<GetLbRequest>, options?: Readonly<WaitForOptions<Lb>>) => Promise<Lb>;
}
declare class LbZonedV1UtilsAPI extends ZonedAPI {
    /**
     * Waits for all private networks of a (zoned) load balancer to be in a final
     * state.
     *
     * @param request - The request {@link ZonedWaitForLbPrivateNetworksRequest}
     * @param options - The waiting options
     * @returns A Promise of ListLbPrivateNetworksResponse
     */
    waitForLbPrivateNetworks: (request: Readonly<ZonedWaitForLbPrivateNetworksRequest>, options?: Readonly<WaitForOptions<ListLbPrivateNetworksResponse>>) => Promise<ListLbPrivateNetworksResponse>;
    /**
     * Waits for all instances of a (zoned) load balancer to be in a final state.
     *
     * @param request - The request {@link GetLbRequest}
     * @param options - The waiting options
     * @returns A Promise of Lb
     */
    waitForLbInstances: (request: Readonly<GetLbRequest>, options?: Readonly<WaitForOptions<Lb>>) => Promise<Lb>;
}

/** Lists transient statutes of the enum {@link CertificateStatus}. */
declare const CERTIFICATE_TRANSIENT_STATUSES: CertificateStatus[];
/** Lists transient statutes of the enum {@link InstanceStatus}. */
declare const INSTANCE_TRANSIENT_STATUSES$1: InstanceStatus$1[];
/** Lists transient statutes of the enum {@link LbStatus}. */
declare const LB_TRANSIENT_STATUSES: LbStatus[];
/** Lists transient statutes of the enum {@link PrivateNetworkStatus}. */
declare const PRIVATE_NETWORK_TRANSIENT_STATUSES: PrivateNetworkStatus[];

type index$d_Acl = Acl;
type index$d_AclAction = AclAction;
type index$d_AclActionRedirect = AclActionRedirect;
type index$d_AclActionRedirectRedirectType = AclActionRedirectRedirectType;
type index$d_AclActionType = AclActionType;
type index$d_AclHttpFilter = AclHttpFilter;
type index$d_AclMatch = AclMatch;
type index$d_AclSpec = AclSpec;
type index$d_AddBackendServersRequest = AddBackendServersRequest;
type index$d_AttachPrivateNetworkRequest = AttachPrivateNetworkRequest;
type index$d_Backend = Backend;
type index$d_BackendServerStats = BackendServerStats;
type index$d_BackendServerStatsHealthCheckStatus = BackendServerStatsHealthCheckStatus;
type index$d_BackendServerStatsServerState = BackendServerStatsServerState;
declare const index$d_CERTIFICATE_TRANSIENT_STATUSES: typeof CERTIFICATE_TRANSIENT_STATUSES;
type index$d_Certificate = Certificate;
type index$d_CertificateStatus = CertificateStatus;
type index$d_CertificateType = CertificateType;
type index$d_CreateAclRequest = CreateAclRequest;
type index$d_CreateBackendRequest = CreateBackendRequest;
type index$d_CreateCertificateRequest = CreateCertificateRequest;
type index$d_CreateCertificateRequestCustomCertificate = CreateCertificateRequestCustomCertificate;
type index$d_CreateCertificateRequestLetsencryptConfig = CreateCertificateRequestLetsencryptConfig;
type index$d_CreateFrontendRequest = CreateFrontendRequest;
type index$d_CreateIpRequest = CreateIpRequest;
type index$d_CreateLbRequest = CreateLbRequest;
type index$d_CreateRouteRequest = CreateRouteRequest;
type index$d_CreateSubscriberRequest = CreateSubscriberRequest;
type index$d_DeleteAclRequest = DeleteAclRequest;
type index$d_DeleteBackendRequest = DeleteBackendRequest;
type index$d_DeleteCertificateRequest = DeleteCertificateRequest;
type index$d_DeleteFrontendRequest = DeleteFrontendRequest;
type index$d_DeleteLbRequest = DeleteLbRequest;
type index$d_DeleteRouteRequest = DeleteRouteRequest;
type index$d_DeleteSubscriberRequest = DeleteSubscriberRequest;
type index$d_DetachPrivateNetworkRequest = DetachPrivateNetworkRequest;
type index$d_ForwardPortAlgorithm = ForwardPortAlgorithm;
type index$d_Frontend = Frontend;
type index$d_GetAclRequest = GetAclRequest;
type index$d_GetBackendRequest = GetBackendRequest;
type index$d_GetCertificateRequest = GetCertificateRequest;
type index$d_GetFrontendRequest = GetFrontendRequest;
type index$d_GetIpRequest = GetIpRequest;
type index$d_GetLbRequest = GetLbRequest;
type index$d_GetLbStatsRequest = GetLbStatsRequest;
type index$d_GetRouteRequest = GetRouteRequest;
type index$d_GetSubscriberRequest = GetSubscriberRequest;
type index$d_HealthCheck = HealthCheck;
type index$d_HealthCheckHttpConfig = HealthCheckHttpConfig;
type index$d_HealthCheckHttpsConfig = HealthCheckHttpsConfig;
type index$d_HealthCheckLdapConfig = HealthCheckLdapConfig;
type index$d_HealthCheckMysqlConfig = HealthCheckMysqlConfig;
type index$d_HealthCheckPgsqlConfig = HealthCheckPgsqlConfig;
type index$d_HealthCheckRedisConfig = HealthCheckRedisConfig;
type index$d_HealthCheckTcpConfig = HealthCheckTcpConfig;
type index$d_Ip = Ip;
declare const index$d_LB_TRANSIENT_STATUSES: typeof LB_TRANSIENT_STATUSES;
type index$d_Lb = Lb;
type index$d_LbStats = LbStats;
type index$d_LbStatus = LbStatus;
type index$d_LbType = LbType;
type index$d_LbTypeStock = LbTypeStock;
type index$d_ListAclRequestOrderBy = ListAclRequestOrderBy;
type index$d_ListAclResponse = ListAclResponse;
type index$d_ListAclsRequest = ListAclsRequest;
type index$d_ListBackendStatsRequest = ListBackendStatsRequest;
type index$d_ListBackendStatsResponse = ListBackendStatsResponse;
type index$d_ListBackendsRequest = ListBackendsRequest;
type index$d_ListBackendsRequestOrderBy = ListBackendsRequestOrderBy;
type index$d_ListBackendsResponse = ListBackendsResponse;
type index$d_ListCertificatesRequest = ListCertificatesRequest;
type index$d_ListCertificatesRequestOrderBy = ListCertificatesRequestOrderBy;
type index$d_ListCertificatesResponse = ListCertificatesResponse;
type index$d_ListFrontendsRequest = ListFrontendsRequest;
type index$d_ListFrontendsRequestOrderBy = ListFrontendsRequestOrderBy;
type index$d_ListFrontendsResponse = ListFrontendsResponse;
type index$d_ListIpsRequestIpType = ListIpsRequestIpType;
type index$d_ListIpsResponse = ListIpsResponse;
type index$d_ListLbPrivateNetworksRequest = ListLbPrivateNetworksRequest;
type index$d_ListLbPrivateNetworksResponse = ListLbPrivateNetworksResponse;
type index$d_ListLbTypesRequest = ListLbTypesRequest;
type index$d_ListLbTypesResponse = ListLbTypesResponse;
type index$d_ListLbsRequest = ListLbsRequest;
type index$d_ListLbsRequestOrderBy = ListLbsRequestOrderBy;
type index$d_ListLbsResponse = ListLbsResponse;
type index$d_ListRoutesRequest = ListRoutesRequest;
type index$d_ListRoutesRequestOrderBy = ListRoutesRequestOrderBy;
type index$d_ListRoutesResponse = ListRoutesResponse;
type index$d_ListSubscriberRequest = ListSubscriberRequest;
type index$d_ListSubscriberRequestOrderBy = ListSubscriberRequestOrderBy;
type index$d_ListSubscriberResponse = ListSubscriberResponse;
type index$d_MigrateLbRequest = MigrateLbRequest;
type index$d_OnMarkedDownAction = OnMarkedDownAction;
declare const index$d_PRIVATE_NETWORK_TRANSIENT_STATUSES: typeof PRIVATE_NETWORK_TRANSIENT_STATUSES;
type index$d_PrivateNetworkDHCPConfig = PrivateNetworkDHCPConfig;
type index$d_PrivateNetworkIpamConfig = PrivateNetworkIpamConfig;
type index$d_PrivateNetworkStaticConfig = PrivateNetworkStaticConfig;
type index$d_PrivateNetworkStatus = PrivateNetworkStatus;
type index$d_Protocol = Protocol;
type index$d_ProxyProtocol = ProxyProtocol;
type index$d_ReleaseIpRequest = ReleaseIpRequest;
type index$d_RemoveBackendServersRequest = RemoveBackendServersRequest;
type index$d_Route = Route;
type index$d_RouteMatch = RouteMatch;
type index$d_SSLCompatibilityLevel = SSLCompatibilityLevel;
type index$d_SetAclsResponse = SetAclsResponse;
type index$d_SetBackendServersRequest = SetBackendServersRequest;
type index$d_StickySessionsType = StickySessionsType;
type index$d_SubscribeToLbRequest = SubscribeToLbRequest;
type index$d_Subscriber = Subscriber;
type index$d_SubscriberEmailConfig = SubscriberEmailConfig;
type index$d_SubscriberWebhookConfig = SubscriberWebhookConfig;
type index$d_UnsubscribeFromLbRequest = UnsubscribeFromLbRequest;
type index$d_UpdateAclRequest = UpdateAclRequest;
type index$d_UpdateBackendRequest = UpdateBackendRequest;
type index$d_UpdateCertificateRequest = UpdateCertificateRequest;
type index$d_UpdateFrontendRequest = UpdateFrontendRequest;
type index$d_UpdateHealthCheckRequest = UpdateHealthCheckRequest;
type index$d_UpdateIpRequest = UpdateIpRequest;
type index$d_UpdateLbRequest = UpdateLbRequest;
type index$d_UpdateRouteRequest = UpdateRouteRequest;
type index$d_UpdateSubscriberRequest = UpdateSubscriberRequest;
type index$d_WaitForLbPrivateNetworksRequest = WaitForLbPrivateNetworksRequest;
type index$d_ZonedApiAddBackendServersRequest = ZonedApiAddBackendServersRequest;
type index$d_ZonedApiAttachPrivateNetworkRequest = ZonedApiAttachPrivateNetworkRequest;
type index$d_ZonedApiCreateAclRequest = ZonedApiCreateAclRequest;
type index$d_ZonedApiCreateBackendRequest = ZonedApiCreateBackendRequest;
type index$d_ZonedApiCreateCertificateRequest = ZonedApiCreateCertificateRequest;
type index$d_ZonedApiCreateFrontendRequest = ZonedApiCreateFrontendRequest;
type index$d_ZonedApiCreateIpRequest = ZonedApiCreateIpRequest;
type index$d_ZonedApiCreateLbRequest = ZonedApiCreateLbRequest;
type index$d_ZonedApiCreateRouteRequest = ZonedApiCreateRouteRequest;
type index$d_ZonedApiCreateSubscriberRequest = ZonedApiCreateSubscriberRequest;
type index$d_ZonedApiDeleteAclRequest = ZonedApiDeleteAclRequest;
type index$d_ZonedApiDeleteBackendRequest = ZonedApiDeleteBackendRequest;
type index$d_ZonedApiDeleteCertificateRequest = ZonedApiDeleteCertificateRequest;
type index$d_ZonedApiDeleteFrontendRequest = ZonedApiDeleteFrontendRequest;
type index$d_ZonedApiDeleteLbRequest = ZonedApiDeleteLbRequest;
type index$d_ZonedApiDeleteRouteRequest = ZonedApiDeleteRouteRequest;
type index$d_ZonedApiDeleteSubscriberRequest = ZonedApiDeleteSubscriberRequest;
type index$d_ZonedApiDetachPrivateNetworkRequest = ZonedApiDetachPrivateNetworkRequest;
type index$d_ZonedApiGetAclRequest = ZonedApiGetAclRequest;
type index$d_ZonedApiGetBackendRequest = ZonedApiGetBackendRequest;
type index$d_ZonedApiGetCertificateRequest = ZonedApiGetCertificateRequest;
type index$d_ZonedApiGetFrontendRequest = ZonedApiGetFrontendRequest;
type index$d_ZonedApiGetIpRequest = ZonedApiGetIpRequest;
type index$d_ZonedApiGetLbRequest = ZonedApiGetLbRequest;
type index$d_ZonedApiGetLbStatsRequest = ZonedApiGetLbStatsRequest;
type index$d_ZonedApiGetRouteRequest = ZonedApiGetRouteRequest;
type index$d_ZonedApiGetSubscriberRequest = ZonedApiGetSubscriberRequest;
type index$d_ZonedApiListAclsRequest = ZonedApiListAclsRequest;
type index$d_ZonedApiListBackendStatsRequest = ZonedApiListBackendStatsRequest;
type index$d_ZonedApiListBackendsRequest = ZonedApiListBackendsRequest;
type index$d_ZonedApiListCertificatesRequest = ZonedApiListCertificatesRequest;
type index$d_ZonedApiListFrontendsRequest = ZonedApiListFrontendsRequest;
type index$d_ZonedApiListIPsRequest = ZonedApiListIPsRequest;
type index$d_ZonedApiListLbPrivateNetworksRequest = ZonedApiListLbPrivateNetworksRequest;
type index$d_ZonedApiListLbTypesRequest = ZonedApiListLbTypesRequest;
type index$d_ZonedApiListLbsRequest = ZonedApiListLbsRequest;
type index$d_ZonedApiListRoutesRequest = ZonedApiListRoutesRequest;
type index$d_ZonedApiListSubscriberRequest = ZonedApiListSubscriberRequest;
type index$d_ZonedApiMigrateLbRequest = ZonedApiMigrateLbRequest;
type index$d_ZonedApiReleaseIpRequest = ZonedApiReleaseIpRequest;
type index$d_ZonedApiRemoveBackendServersRequest = ZonedApiRemoveBackendServersRequest;
type index$d_ZonedApiSetAclsRequest = ZonedApiSetAclsRequest;
type index$d_ZonedApiSetBackendServersRequest = ZonedApiSetBackendServersRequest;
type index$d_ZonedApiSubscribeToLbRequest = ZonedApiSubscribeToLbRequest;
type index$d_ZonedApiUnsubscribeFromLbRequest = ZonedApiUnsubscribeFromLbRequest;
type index$d_ZonedApiUpdateAclRequest = ZonedApiUpdateAclRequest;
type index$d_ZonedApiUpdateBackendRequest = ZonedApiUpdateBackendRequest;
type index$d_ZonedApiUpdateCertificateRequest = ZonedApiUpdateCertificateRequest;
type index$d_ZonedApiUpdateFrontendRequest = ZonedApiUpdateFrontendRequest;
type index$d_ZonedApiUpdateHealthCheckRequest = ZonedApiUpdateHealthCheckRequest;
type index$d_ZonedApiUpdateIpRequest = ZonedApiUpdateIpRequest;
type index$d_ZonedApiUpdateLbRequest = ZonedApiUpdateLbRequest;
type index$d_ZonedApiUpdateRouteRequest = ZonedApiUpdateRouteRequest;
type index$d_ZonedApiUpdateSubscriberRequest = ZonedApiUpdateSubscriberRequest;
type index$d_ZonedWaitForLbPrivateNetworksRequest = ZonedWaitForLbPrivateNetworksRequest;
declare namespace index$d {
  export { LbV1UtilsAPI as API, type index$d_Acl as Acl, type index$d_AclAction as AclAction, type index$d_AclActionRedirect as AclActionRedirect, type index$d_AclActionRedirectRedirectType as AclActionRedirectRedirectType, type index$d_AclActionType as AclActionType, type index$d_AclHttpFilter as AclHttpFilter, type index$d_AclMatch as AclMatch, type index$d_AclSpec as AclSpec, type index$d_AddBackendServersRequest as AddBackendServersRequest, type index$d_AttachPrivateNetworkRequest as AttachPrivateNetworkRequest, type index$d_Backend as Backend, type index$d_BackendServerStats as BackendServerStats, type index$d_BackendServerStatsHealthCheckStatus as BackendServerStatsHealthCheckStatus, type index$d_BackendServerStatsServerState as BackendServerStatsServerState, index$d_CERTIFICATE_TRANSIENT_STATUSES as CERTIFICATE_TRANSIENT_STATUSES, type index$d_Certificate as Certificate, type index$d_CertificateStatus as CertificateStatus, type index$d_CertificateType as CertificateType, type index$d_CreateAclRequest as CreateAclRequest, type index$d_CreateBackendRequest as CreateBackendRequest, type index$d_CreateCertificateRequest as CreateCertificateRequest, type index$d_CreateCertificateRequestCustomCertificate as CreateCertificateRequestCustomCertificate, type index$d_CreateCertificateRequestLetsencryptConfig as CreateCertificateRequestLetsencryptConfig, type index$d_CreateFrontendRequest as CreateFrontendRequest, type index$d_CreateIpRequest as CreateIpRequest, type index$d_CreateLbRequest as CreateLbRequest, type index$d_CreateRouteRequest as CreateRouteRequest, type index$d_CreateSubscriberRequest as CreateSubscriberRequest, type index$d_DeleteAclRequest as DeleteAclRequest, type index$d_DeleteBackendRequest as DeleteBackendRequest, type index$d_DeleteCertificateRequest as DeleteCertificateRequest, type index$d_DeleteFrontendRequest as DeleteFrontendRequest, type index$d_DeleteLbRequest as DeleteLbRequest, type index$d_DeleteRouteRequest as DeleteRouteRequest, type index$d_DeleteSubscriberRequest as DeleteSubscriberRequest, type index$d_DetachPrivateNetworkRequest as DetachPrivateNetworkRequest, type index$d_ForwardPortAlgorithm as ForwardPortAlgorithm, type index$d_Frontend as Frontend, type index$d_GetAclRequest as GetAclRequest, type index$d_GetBackendRequest as GetBackendRequest, type index$d_GetCertificateRequest as GetCertificateRequest, type index$d_GetFrontendRequest as GetFrontendRequest, type index$d_GetIpRequest as GetIpRequest, type index$d_GetLbRequest as GetLbRequest, type index$d_GetLbStatsRequest as GetLbStatsRequest, type index$d_GetRouteRequest as GetRouteRequest, type index$d_GetSubscriberRequest as GetSubscriberRequest, type index$d_HealthCheck as HealthCheck, type index$d_HealthCheckHttpConfig as HealthCheckHttpConfig, type index$d_HealthCheckHttpsConfig as HealthCheckHttpsConfig, type index$d_HealthCheckLdapConfig as HealthCheckLdapConfig, type index$d_HealthCheckMysqlConfig as HealthCheckMysqlConfig, type index$d_HealthCheckPgsqlConfig as HealthCheckPgsqlConfig, type index$d_HealthCheckRedisConfig as HealthCheckRedisConfig, type index$d_HealthCheckTcpConfig as HealthCheckTcpConfig, INSTANCE_TRANSIENT_STATUSES$1 as INSTANCE_TRANSIENT_STATUSES, type Instance$1 as Instance, type InstanceStatus$1 as InstanceStatus, type index$d_Ip as Ip, index$d_LB_TRANSIENT_STATUSES as LB_TRANSIENT_STATUSES, type index$d_Lb as Lb, type index$d_LbStats as LbStats, type index$d_LbStatus as LbStatus, type index$d_LbType as LbType, type index$d_LbTypeStock as LbTypeStock, type index$d_ListAclRequestOrderBy as ListAclRequestOrderBy, type index$d_ListAclResponse as ListAclResponse, type index$d_ListAclsRequest as ListAclsRequest, type index$d_ListBackendStatsRequest as ListBackendStatsRequest, type index$d_ListBackendStatsResponse as ListBackendStatsResponse, type index$d_ListBackendsRequest as ListBackendsRequest, type index$d_ListBackendsRequestOrderBy as ListBackendsRequestOrderBy, type index$d_ListBackendsResponse as ListBackendsResponse, type index$d_ListCertificatesRequest as ListCertificatesRequest, type index$d_ListCertificatesRequestOrderBy as ListCertificatesRequestOrderBy, type index$d_ListCertificatesResponse as ListCertificatesResponse, type index$d_ListFrontendsRequest as ListFrontendsRequest, type index$d_ListFrontendsRequestOrderBy as ListFrontendsRequestOrderBy, type index$d_ListFrontendsResponse as ListFrontendsResponse, type ListIPsRequest$1 as ListIPsRequest, type index$d_ListIpsRequestIpType as ListIpsRequestIpType, type index$d_ListIpsResponse as ListIpsResponse, type index$d_ListLbPrivateNetworksRequest as ListLbPrivateNetworksRequest, type index$d_ListLbPrivateNetworksResponse as ListLbPrivateNetworksResponse, type index$d_ListLbTypesRequest as ListLbTypesRequest, type index$d_ListLbTypesResponse as ListLbTypesResponse, type index$d_ListLbsRequest as ListLbsRequest, type index$d_ListLbsRequestOrderBy as ListLbsRequestOrderBy, type index$d_ListLbsResponse as ListLbsResponse, type ListPrivateNetworksRequestOrderBy$2 as ListPrivateNetworksRequestOrderBy, type index$d_ListRoutesRequest as ListRoutesRequest, type index$d_ListRoutesRequestOrderBy as ListRoutesRequestOrderBy, type index$d_ListRoutesResponse as ListRoutesResponse, type index$d_ListSubscriberRequest as ListSubscriberRequest, type index$d_ListSubscriberRequestOrderBy as ListSubscriberRequestOrderBy, type index$d_ListSubscriberResponse as ListSubscriberResponse, type index$d_MigrateLbRequest as MigrateLbRequest, type index$d_OnMarkedDownAction as OnMarkedDownAction, index$d_PRIVATE_NETWORK_TRANSIENT_STATUSES as PRIVATE_NETWORK_TRANSIENT_STATUSES, type PrivateNetwork$3 as PrivateNetwork, type index$d_PrivateNetworkDHCPConfig as PrivateNetworkDHCPConfig, type index$d_PrivateNetworkIpamConfig as PrivateNetworkIpamConfig, type index$d_PrivateNetworkStaticConfig as PrivateNetworkStaticConfig, type index$d_PrivateNetworkStatus as PrivateNetworkStatus, type index$d_Protocol as Protocol, type index$d_ProxyProtocol as ProxyProtocol, type index$d_ReleaseIpRequest as ReleaseIpRequest, type index$d_RemoveBackendServersRequest as RemoveBackendServersRequest, type index$d_Route as Route, type index$d_RouteMatch as RouteMatch, type index$d_SSLCompatibilityLevel as SSLCompatibilityLevel, type index$d_SetAclsResponse as SetAclsResponse, type index$d_SetBackendServersRequest as SetBackendServersRequest, type index$d_StickySessionsType as StickySessionsType, type index$d_SubscribeToLbRequest as SubscribeToLbRequest, type index$d_Subscriber as Subscriber, type index$d_SubscriberEmailConfig as SubscriberEmailConfig, type index$d_SubscriberWebhookConfig as SubscriberWebhookConfig, type index$d_UnsubscribeFromLbRequest as UnsubscribeFromLbRequest, type index$d_UpdateAclRequest as UpdateAclRequest, type index$d_UpdateBackendRequest as UpdateBackendRequest, type index$d_UpdateCertificateRequest as UpdateCertificateRequest, type index$d_UpdateFrontendRequest as UpdateFrontendRequest, type index$d_UpdateHealthCheckRequest as UpdateHealthCheckRequest, type index$d_UpdateIpRequest as UpdateIpRequest, type index$d_UpdateLbRequest as UpdateLbRequest, type index$d_UpdateRouteRequest as UpdateRouteRequest, type index$d_UpdateSubscriberRequest as UpdateSubscriberRequest, type index$d_WaitForLbPrivateNetworksRequest as WaitForLbPrivateNetworksRequest, LbZonedV1UtilsAPI as ZonedAPI, type index$d_ZonedApiAddBackendServersRequest as ZonedApiAddBackendServersRequest, type index$d_ZonedApiAttachPrivateNetworkRequest as ZonedApiAttachPrivateNetworkRequest, type index$d_ZonedApiCreateAclRequest as ZonedApiCreateAclRequest, type index$d_ZonedApiCreateBackendRequest as ZonedApiCreateBackendRequest, type index$d_ZonedApiCreateCertificateRequest as ZonedApiCreateCertificateRequest, type index$d_ZonedApiCreateFrontendRequest as ZonedApiCreateFrontendRequest, type index$d_ZonedApiCreateIpRequest as ZonedApiCreateIpRequest, type index$d_ZonedApiCreateLbRequest as ZonedApiCreateLbRequest, type index$d_ZonedApiCreateRouteRequest as ZonedApiCreateRouteRequest, type index$d_ZonedApiCreateSubscriberRequest as ZonedApiCreateSubscriberRequest, type index$d_ZonedApiDeleteAclRequest as ZonedApiDeleteAclRequest, type index$d_ZonedApiDeleteBackendRequest as ZonedApiDeleteBackendRequest, type index$d_ZonedApiDeleteCertificateRequest as ZonedApiDeleteCertificateRequest, type index$d_ZonedApiDeleteFrontendRequest as ZonedApiDeleteFrontendRequest, type index$d_ZonedApiDeleteLbRequest as ZonedApiDeleteLbRequest, type index$d_ZonedApiDeleteRouteRequest as ZonedApiDeleteRouteRequest, type index$d_ZonedApiDeleteSubscriberRequest as ZonedApiDeleteSubscriberRequest, type index$d_ZonedApiDetachPrivateNetworkRequest as ZonedApiDetachPrivateNetworkRequest, type index$d_ZonedApiGetAclRequest as ZonedApiGetAclRequest, type index$d_ZonedApiGetBackendRequest as ZonedApiGetBackendRequest, type index$d_ZonedApiGetCertificateRequest as ZonedApiGetCertificateRequest, type index$d_ZonedApiGetFrontendRequest as ZonedApiGetFrontendRequest, type index$d_ZonedApiGetIpRequest as ZonedApiGetIpRequest, type index$d_ZonedApiGetLbRequest as ZonedApiGetLbRequest, type index$d_ZonedApiGetLbStatsRequest as ZonedApiGetLbStatsRequest, type index$d_ZonedApiGetRouteRequest as ZonedApiGetRouteRequest, type index$d_ZonedApiGetSubscriberRequest as ZonedApiGetSubscriberRequest, type index$d_ZonedApiListAclsRequest as ZonedApiListAclsRequest, type index$d_ZonedApiListBackendStatsRequest as ZonedApiListBackendStatsRequest, type index$d_ZonedApiListBackendsRequest as ZonedApiListBackendsRequest, type index$d_ZonedApiListCertificatesRequest as ZonedApiListCertificatesRequest, type index$d_ZonedApiListFrontendsRequest as ZonedApiListFrontendsRequest, type index$d_ZonedApiListIPsRequest as ZonedApiListIPsRequest, type index$d_ZonedApiListLbPrivateNetworksRequest as ZonedApiListLbPrivateNetworksRequest, type index$d_ZonedApiListLbTypesRequest as ZonedApiListLbTypesRequest, type index$d_ZonedApiListLbsRequest as ZonedApiListLbsRequest, type index$d_ZonedApiListRoutesRequest as ZonedApiListRoutesRequest, type index$d_ZonedApiListSubscriberRequest as ZonedApiListSubscriberRequest, type index$d_ZonedApiMigrateLbRequest as ZonedApiMigrateLbRequest, type index$d_ZonedApiReleaseIpRequest as ZonedApiReleaseIpRequest, type index$d_ZonedApiRemoveBackendServersRequest as ZonedApiRemoveBackendServersRequest, type index$d_ZonedApiSetAclsRequest as ZonedApiSetAclsRequest, type index$d_ZonedApiSetBackendServersRequest as ZonedApiSetBackendServersRequest, type index$d_ZonedApiSubscribeToLbRequest as ZonedApiSubscribeToLbRequest, type index$d_ZonedApiUnsubscribeFromLbRequest as ZonedApiUnsubscribeFromLbRequest, type index$d_ZonedApiUpdateAclRequest as ZonedApiUpdateAclRequest, type index$d_ZonedApiUpdateBackendRequest as ZonedApiUpdateBackendRequest, type index$d_ZonedApiUpdateCertificateRequest as ZonedApiUpdateCertificateRequest, type index$d_ZonedApiUpdateFrontendRequest as ZonedApiUpdateFrontendRequest, type index$d_ZonedApiUpdateHealthCheckRequest as ZonedApiUpdateHealthCheckRequest, type index$d_ZonedApiUpdateIpRequest as ZonedApiUpdateIpRequest, type index$d_ZonedApiUpdateLbRequest as ZonedApiUpdateLbRequest, type index$d_ZonedApiUpdateRouteRequest as ZonedApiUpdateRouteRequest, type index$d_ZonedApiUpdateSubscriberRequest as ZonedApiUpdateSubscriberRequest, type index$d_ZonedWaitForLbPrivateNetworksRequest as ZonedWaitForLbPrivateNetworksRequest };
}

declare namespace index$c {
  export { index$d as v1 };
}

interface LocalImage$1 {
    /** Version you will typically use to define an image in an API call. */
    id: string;
    /** List of all commercial types that are compatible with this local image. */
    compatibleCommercialTypes: string[];
    /** Supported architecture for this local image. */
    arch: string;
    /** Availability Zone where this local image is available. */
    zone: Zone;
}
interface Organization {
    id: string;
    name: string;
}
interface Version$1 {
    /** UUID of this version. */
    id: string;
    /** Name of this version. */
    name: string;
    /** Creation date of this image version. */
    creationDate?: Date;
    /** Date of the last modification of this version. */
    modificationDate?: Date;
    /** List of local images available in this version. */
    localImages: LocalImage$1[];
}
interface Image$2 {
    /** UUID of this image. */
    id: string;
    /** Name of the image. */
    name: string;
    /** Text description of this image. */
    description: string;
    /** URL of this image's logo. */
    logo: string;
    /** List of categories this image belongs to. */
    categories: string[];
    /** Creation date of this image. */
    creationDate?: Date;
    /** Date of the last modification of this image. */
    modificationDate?: Date;
    /** Expiration date of this image. */
    validUntil?: Date;
    /** Typically an identifier for a distribution (ex. "ubuntu_focal"). */
    label: string;
    /** List of versions of this image. */
    versions: Version$1[];
    /** Organization this image belongs to. */
    organization?: Organization;
    currentPublicVersion: string;
}
type GetImageRequest$2 = {
    /** Display the image name. */
    imageId: string;
};
interface GetImageResponse {
    image?: Image$2;
}
type ListImagesRequest$2 = {
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * display.
     */
    perPage?: number;
    /** A positive integer to choose the page to display. */
    page?: number;
};
interface ListImagesResponse$2 {
    images: Image$2[];
    totalCount: number;
}

/** Marketplace API. */
declare class API$c extends API$x {
    protected pageOfListImages: (request?: Readonly<ListImagesRequest$2>) => Promise<ListImagesResponse$2>;
    /**
     * List marketplace images.
     *
     * @param request - The request {@link ListImagesRequest}
     * @returns A Promise of ListImagesResponse
     */
    listImages: (request?: Readonly<ListImagesRequest$2>) => Promise<ListImagesResponse$2> & {
        all: () => Promise<Image$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Image$2[], void, void>;
    };
    /**
     * Get a specific marketplace image.
     *
     * @param request - The request {@link GetImageRequest}
     * @returns A Promise of GetImageResponse
     */
    getImage: (request: Readonly<GetImageRequest$2>) => Promise<GetImageResponse>;
}

type index_gen$e_GetImageResponse = GetImageResponse;
type index_gen$e_Organization = Organization;
declare namespace index_gen$e {
  export { API$c as API, type GetImageRequest$2 as GetImageRequest, type index_gen$e_GetImageResponse as GetImageResponse, type Image$2 as Image, type ListImagesRequest$2 as ListImagesRequest, type ListImagesResponse$2 as ListImagesResponse, type LocalImage$1 as LocalImage, type index_gen$e_Organization as Organization, type Version$1 as Version };
}

type ListImagesRequestOrderBy$1 = 'name_asc' | 'name_desc' | 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc';
type ListLocalImagesRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListVersionsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type LocalImageType = 'unknown_type' | 'instance_local' | 'instance_sbs';
interface Category {
    id: string;
    name: string;
    description: string;
}
interface Image$1 {
    /** UUID of this image. */
    id: string;
    /** Name of the image. */
    name: string;
    /** Text description of this image. */
    description: string;
    /** URL of this image's logo. */
    logo: string;
    /** List of categories this image belongs to. */
    categories: string[];
    /** Creation date of this image. */
    createdAt?: Date;
    /** Date of the last modification of this image. */
    updatedAt?: Date;
    /** Expiration date of this image. */
    validUntil?: Date;
    /** Typically an identifier for a distribution (ex. "ubuntu_focal"). */
    label: string;
}
interface LocalImage {
    /** Version you will typically use to define an image in an API call. */
    id: string;
    /** List of all commercial types that are compatible with this local image. */
    compatibleCommercialTypes: string[];
    /** Supported architecture for this local image. */
    arch: string;
    /** Availability Zone where this local image is available. */
    zone: Zone;
    /** Image label this image belongs to. */
    label: string;
    /** Type of this local image. */
    type: LocalImageType;
}
interface Version {
    /** UUID of this version. */
    id: string;
    /** Name of this version. */
    name: string;
    /** Creation date of this image version. */
    createdAt?: Date;
    /** Date of the last modification of this version. */
    updatedAt?: Date;
    /** Date this version was officially published. */
    publishedAt?: Date;
}
type GetCategoryRequest = {
    categoryId: string;
};
type GetImageRequest$1 = {
    /** Display the image name. */
    imageId: string;
};
type GetLocalImageRequest = {
    localImageId: string;
};
type GetVersionRequest = {
    versionId: string;
};
type ListCategoriesRequest = {
    pageSize?: number;
    page?: number;
};
interface ListCategoriesResponse {
    categories: Category[];
    totalCount: number;
}
type ListImagesRequest$1 = {
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * display.
     */
    pageSize?: number;
    /** A positive integer to choose the page to display. */
    page?: number;
    /** Ordering to use. */
    orderBy?: ListImagesRequestOrderBy$1;
    /** Choose for which machine architecture to return images. */
    arch?: string;
    /** Choose the category of images to get. */
    category?: string;
    /** Choose to include end-of-life images. */
    includeEol: boolean;
};
interface ListImagesResponse$1 {
    images: Image$1[];
    totalCount: number;
}
type ListLocalImagesRequest = {
    /**
     * One-of ('scope'): at most one of 'imageId', 'versionId', 'imageLabel' could
     * be set.
     */
    imageId?: string;
    /**
     * One-of ('scope'): at most one of 'imageId', 'versionId', 'imageLabel' could
     * be set.
     */
    versionId?: string;
    pageSize?: number;
    page?: number;
    orderBy?: ListLocalImagesRequestOrderBy;
    /**
     * One-of ('scope'): at most one of 'imageId', 'versionId', 'imageLabel' could
     * be set.
     */
    imageLabel?: string;
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    type?: LocalImageType;
};
interface ListLocalImagesResponse {
    localImages: LocalImage[];
    totalCount: number;
}
type ListVersionsRequest = {
    imageId: string;
    pageSize?: number;
    page?: number;
    orderBy?: ListVersionsRequestOrderBy;
};
interface ListVersionsResponse {
    versions: Version[];
    totalCount: number;
}

/** Marketplace API. */
declare class API$b extends API$x {
    protected pageOfListImages: (request: Readonly<ListImagesRequest$1>) => Promise<ListImagesResponse$1>;
    /**
     * List marketplace images. List all available images on the marketplace,
     * their UUID, CPU architecture and description.
     *
     * @param request - The request {@link ListImagesRequest}
     * @returns A Promise of ListImagesResponse
     */
    listImages: (request: Readonly<ListImagesRequest$1>) => Promise<ListImagesResponse$1> & {
        all: () => Promise<Image$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Image$1[], void, void>;
    };
    /**
     * Get a specific marketplace image. Get detailed information about a
     * marketplace image, specified by its `image_id` (UUID format).
     *
     * @param request - The request {@link GetImageRequest}
     * @returns A Promise of Image
     */
    getImage: (request: Readonly<GetImageRequest$1>) => Promise<Image$1>;
    protected pageOfListVersions: (request: Readonly<ListVersionsRequest>) => Promise<ListVersionsResponse>;
    /**
     * List versions of an Image. Get a list of all available version of an image,
     * specified by its `image_id` (UUID format).
     *
     * @param request - The request {@link ListVersionsRequest}
     * @returns A Promise of ListVersionsResponse
     */
    listVersions: (request: Readonly<ListVersionsRequest>) => Promise<ListVersionsResponse> & {
        all: () => Promise<Version[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Version[], void, void>;
    };
    /**
     * Get a specific image version. Get information such as the name, creation
     * date, last update and published date for an image version specified by its
     * `version_id` (UUID format).
     *
     * @param request - The request {@link GetVersionRequest}
     * @returns A Promise of Version
     */
    getVersion: (request: Readonly<GetVersionRequest>) => Promise<Version>;
    protected pageOfListLocalImages: (request?: Readonly<ListLocalImagesRequest>) => Promise<ListLocalImagesResponse>;
    /**
     * List local images from a specific image or version. List information about
     * local images in a specific Availability Zone, specified by its `image_id`
     * (UUID format), `version_id` (UUID format) or `image_label`. Only one of
     * these three parameters may be set.
     *
     * @param request - The request {@link ListLocalImagesRequest}
     * @returns A Promise of ListLocalImagesResponse
     */
    listLocalImages: (request?: Readonly<ListLocalImagesRequest>) => Promise<ListLocalImagesResponse> & {
        all: () => Promise<LocalImage[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<LocalImage[], void, void>;
    };
    /**
     * Get a specific local image by ID. Get detailed information about a local
     * image, including compatible commercial types, supported architecture,
     * labels and the Availability Zone of the image, specified by its
     * `local_image_id` (UUID format).
     *
     * @param request - The request {@link GetLocalImageRequest}
     * @returns A Promise of LocalImage
     */
    getLocalImage: (request: Readonly<GetLocalImageRequest>) => Promise<LocalImage>;
    protected pageOfListCategories: (request?: Readonly<ListCategoriesRequest>) => Promise<ListCategoriesResponse>;
    /**
     * List existing image categories. Get a list of all existing categories. The
     * output can be paginated.
     *
     * @param request - The request {@link ListCategoriesRequest}
     * @returns A Promise of ListCategoriesResponse
     */
    listCategories: (request?: Readonly<ListCategoriesRequest>) => Promise<ListCategoriesResponse> & {
        all: () => Promise<Category[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Category[], void, void>;
    };
    /**
     * Get a specific category. Get information about a specific category of the
     * marketplace catalog, specified by its `category_id` (UUID format).
     *
     * @param request - The request {@link GetCategoryRequest}
     * @returns A Promise of Category
     */
    getCategory: (request: Readonly<GetCategoryRequest>) => Promise<Category>;
}

type index_gen$d_Category = Category;
type index_gen$d_GetCategoryRequest = GetCategoryRequest;
type index_gen$d_GetLocalImageRequest = GetLocalImageRequest;
type index_gen$d_GetVersionRequest = GetVersionRequest;
type index_gen$d_ListCategoriesRequest = ListCategoriesRequest;
type index_gen$d_ListCategoriesResponse = ListCategoriesResponse;
type index_gen$d_ListLocalImagesRequest = ListLocalImagesRequest;
type index_gen$d_ListLocalImagesRequestOrderBy = ListLocalImagesRequestOrderBy;
type index_gen$d_ListLocalImagesResponse = ListLocalImagesResponse;
type index_gen$d_ListVersionsRequest = ListVersionsRequest;
type index_gen$d_ListVersionsRequestOrderBy = ListVersionsRequestOrderBy;
type index_gen$d_ListVersionsResponse = ListVersionsResponse;
type index_gen$d_LocalImage = LocalImage;
type index_gen$d_LocalImageType = LocalImageType;
type index_gen$d_Version = Version;
declare namespace index_gen$d {
  export { API$b as API, type index_gen$d_Category as Category, type index_gen$d_GetCategoryRequest as GetCategoryRequest, type GetImageRequest$1 as GetImageRequest, type index_gen$d_GetLocalImageRequest as GetLocalImageRequest, type index_gen$d_GetVersionRequest as GetVersionRequest, type Image$1 as Image, type index_gen$d_ListCategoriesRequest as ListCategoriesRequest, type index_gen$d_ListCategoriesResponse as ListCategoriesResponse, type ListImagesRequest$1 as ListImagesRequest, type ListImagesRequestOrderBy$1 as ListImagesRequestOrderBy, type ListImagesResponse$1 as ListImagesResponse, type index_gen$d_ListLocalImagesRequest as ListLocalImagesRequest, type index_gen$d_ListLocalImagesRequestOrderBy as ListLocalImagesRequestOrderBy, type index_gen$d_ListLocalImagesResponse as ListLocalImagesResponse, type index_gen$d_ListVersionsRequest as ListVersionsRequest, type index_gen$d_ListVersionsRequestOrderBy as ListVersionsRequestOrderBy, type index_gen$d_ListVersionsResponse as ListVersionsResponse, type index_gen$d_LocalImage as LocalImage, type index_gen$d_LocalImageType as LocalImageType, type index_gen$d_Version as Version };
}

declare namespace index$b {
  export { index_gen$e as v1, index_gen$d as v2 };
}

type ListNatsAccountsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc';
type ListNatsCredentialsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc';
type ListSnsCredentialsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc';
type ListSqsCredentialsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'name_asc' | 'name_desc';
type SnsInfoStatus = 'unknown_status' | 'enabled' | 'disabled';
type SqsInfoStatus = 'unknown_status' | 'enabled' | 'disabled';
interface File {
    /** File name. */
    name: string;
    /** File content. */
    content: string;
}
interface SnsPermissions {
    /**
     * Defines whether the credentials bearer can publish messages to the service
     * (publish to SNS topics).
     */
    canPublish?: boolean;
    /**
     * Defines whether the credentials bearer can receive messages from the
     * service (configure subscriptions).
     */
    canReceive?: boolean;
    /**
     * Defines whether the credentials bearer can manage the associated SNS topics
     * or subscriptions.
     */
    canManage?: boolean;
}
interface SqsPermissions {
    /**
     * Defines whether the credentials bearer can publish messages to the service
     * (send messages to SQS queues).
     */
    canPublish?: boolean;
    /**
     * Defines whether the credentials bearer can receive messages from SQS
     * queues.
     */
    canReceive?: boolean;
    /**
     * Defines whether the credentials bearer can manage the associated SQS
     * queues.
     */
    canManage?: boolean;
}
interface NatsAccount$1 {
    /** NATS account ID. */
    id: string;
    /** NATS account name. */
    name: string;
    /** Endpoint of the NATS service for this account. */
    endpoint: string;
    /** Project ID of the Project containing the NATS account. */
    projectId: string;
    /** Region where the NATS account is deployed. */
    region: Region;
    /** NATS account creation date. */
    createdAt?: Date;
    /** NATS account last modification date. */
    updatedAt?: Date;
}
interface NatsCredentials$1 {
    /** ID of the credentials. */
    id: string;
    /** Name of the credentials. */
    name: string;
    /** NATS account containing the credentials. */
    natsAccountId: string;
    /** NATS credentials creation date. */
    createdAt?: Date;
    /** NATS credentials last modification date. */
    updatedAt?: Date;
    /**
     * Object containing the credentials file (Only returned by **Create Nats
     * Credentials** call).
     */
    credentials?: File;
    /** Checksum of the credentials file. */
    checksum: string;
}
interface SnsCredentials$1 {
    /** ID of the credentials. */
    id: string;
    /** Name of the credentials. */
    name: string;
    /** Project ID of the Project containing the credentials. */
    projectId: string;
    /** Region where the credentials exists. */
    region: Region;
    /** Credentials creation date. */
    createdAt?: Date;
    /** Credentials last modification date. */
    updatedAt?: Date;
    /** Access key ID. */
    accessKey: string;
    /** Secret key ID (Only returned by **Create SNS Credentials** call). */
    secretKey: string;
    /** Checksum of the Secret key. */
    secretChecksum: string;
    /** Permissions associated with these credentials. */
    permissions?: SnsPermissions;
}
interface SqsCredentials$1 {
    /** ID of the credentials. */
    id: string;
    /** Name of the credentials. */
    name: string;
    /** Project ID of the Project containing the credentials. */
    projectId: string;
    /** Region where the credentials exists. */
    region: Region;
    /** Credentials creation date. */
    createdAt?: Date;
    /** Credentials last modification date. */
    updatedAt?: Date;
    /** Access key ID. */
    accessKey: string;
    /** Secret key ID (Only returned by **Create SQS Credentials** call). */
    secretKey: string;
    /** Checksum of the Secret key. */
    secretChecksum: string;
    /** Permissions associated with these credentials. */
    permissions?: SqsPermissions;
}
interface ListNatsAccountsResponse {
    /** Total count of existing NATS accounts (matching any filters specified). */
    totalCount: number;
    /** NATS accounts on this page. */
    natsAccounts: NatsAccount$1[];
}
interface ListNatsCredentialsResponse {
    /** Total count of existing credentials (matching any filters specified). */
    totalCount: number;
    /** Credentials on this page. */
    natsCredentials: NatsCredentials$1[];
}
interface ListSnsCredentialsResponse {
    /** Total count of existing credentials (matching any filters specified). */
    totalCount: number;
    /** SNS credentials on this page. */
    snsCredentials: SnsCredentials$1[];
}
interface ListSqsCredentialsResponse {
    /** Total count of existing credentials (matching any filters specified). */
    totalCount: number;
    /** SQS credentials on this page. */
    sqsCredentials: SqsCredentials$1[];
}
type NatsApiCreateNatsAccountRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** NATS account name. */
    name?: string;
    /** Project containing the NATS account. */
    projectId?: string;
};
type NatsApiCreateNatsCredentialsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** NATS account containing the credentials. */
    natsAccountId: string;
    /** Name of the credentials. */
    name?: string;
};
type NatsApiDeleteNatsAccountRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the NATS account to delete. */
    natsAccountId: string;
};
type NatsApiDeleteNatsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the credentials to delete. */
    natsCredentialsId: string;
};
type NatsApiGetNatsAccountRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the NATS account to get. */
    natsAccountId: string;
};
type NatsApiGetNatsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the credentials to get. */
    natsCredentialsId: string;
};
type NatsApiListNatsAccountsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Include only NATS accounts in this Project. */
    projectId?: string;
    /** Page number to return. */
    page?: number;
    /** Maximum number of NATS accounts to return per page. */
    pageSize?: number;
    /** Order in which to return results. */
    orderBy?: ListNatsAccountsRequestOrderBy;
};
type NatsApiListNatsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Include only credentials for this NATS account. */
    natsAccountId?: string;
    /** Page number to return. */
    page?: number;
    /** Maximum number of credentials to return per page. */
    pageSize?: number;
    /** Order in which to return results. */
    orderBy?: ListNatsCredentialsRequestOrderBy;
};
type NatsApiUpdateNatsAccountRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the NATS account to update. */
    natsAccountId: string;
    /** NATS account name. */
    name?: string;
};
type SnsApiActivateSnsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project on which to activate the SNS service. */
    projectId?: string;
};
type SnsApiCreateSnsCredentialsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project containing the SNS credentials. */
    projectId?: string;
    /** Name of the credentials. */
    name?: string;
    /** Permissions associated with these credentials. */
    permissions?: SnsPermissions;
};
type SnsApiDeactivateSnsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project on which to deactivate the SNS service. */
    projectId?: string;
};
type SnsApiDeleteSnsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the credentials to delete. */
    snsCredentialsId: string;
};
type SnsApiGetSnsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the SNS credentials to get. */
    snsCredentialsId: string;
};
type SnsApiGetSnsInfoRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project to retrieve SNS info from. */
    projectId?: string;
};
type SnsApiListSnsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Include only SNS credentials in this Project. */
    projectId?: string;
    /** Page number to return. */
    page?: number;
    /** Maximum number of credentials to return per page. */
    pageSize?: number;
    /** Order in which to return results. */
    orderBy?: ListSnsCredentialsRequestOrderBy;
};
type SnsApiUpdateSnsCredentialsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the SNS credentials to update. */
    snsCredentialsId: string;
    /** Name of the credentials. */
    name?: string;
    /** Permissions associated with these credentials. */
    permissions?: SnsPermissions;
};
interface SnsInfo {
    /** Project ID of the Project containing the service. */
    projectId: string;
    /** Region of the service. */
    region: Region;
    /** SNS creation date. */
    createdAt?: Date;
    /** SNS last modification date. */
    updatedAt?: Date;
    /** SNS activation status. */
    status: SnsInfoStatus;
    /** Endpoint of the SNS service for this region and project. */
    snsEndpointUrl: string;
}
type SqsApiActivateSqsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project on which to activate the SQS service. */
    projectId?: string;
};
type SqsApiCreateSqsCredentialsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project containing the SQS credentials. */
    projectId?: string;
    /** Name of the credentials. */
    name?: string;
    /** Permissions associated with these credentials. */
    permissions?: SqsPermissions;
};
type SqsApiDeactivateSqsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project on which to deactivate the SQS service. */
    projectId?: string;
};
type SqsApiDeleteSqsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the credentials to delete. */
    sqsCredentialsId: string;
};
type SqsApiGetSqsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the SQS credentials to get. */
    sqsCredentialsId: string;
};
type SqsApiGetSqsInfoRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Project to retrieve SQS info from. */
    projectId?: string;
};
type SqsApiListSqsCredentialsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Include only SQS credentials in this Project. */
    projectId?: string;
    /** Page number to return. */
    page?: number;
    /** Maximum number of credentials to return per page. */
    pageSize?: number;
    /** Order in which to return results. */
    orderBy?: ListSqsCredentialsRequestOrderBy;
};
type SqsApiUpdateSqsCredentialsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the SQS credentials to update. */
    sqsCredentialsId: string;
    /** Name of the credentials. */
    name?: string;
    /** Permissions associated with these credentials. */
    permissions?: SqsPermissions;
};
interface SqsInfo {
    /** Project ID of the Project containing the service. */
    projectId: string;
    /** Region of the service. */
    region: Region;
    /** SQS creation date. */
    createdAt?: Date;
    /** SQS last modification date. */
    updatedAt?: Date;
    /** SQS activation status. */
    status: SqsInfoStatus;
    /** Endpoint of the SQS service for this region and project. */
    sqsEndpointUrl: string;
}

/**
 * Messaging and Queuing NATS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing NATS accounts.
 */
declare class NatsAPI extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Create a NATS account. Create a NATS account associated with a Project.
     *
     * @param request - The request {@link NatsApiCreateNatsAccountRequest}
     * @returns A Promise of NatsAccount
     */
    createNatsAccount: (request?: Readonly<NatsApiCreateNatsAccountRequest$1>) => Promise<NatsAccount$1>;
    /**
     * Delete a NATS account. Delete a NATS account, specified by its NATS account
     * ID. Note that deleting a NATS account is irreversible, and any credentials,
     * streams, consumer and stored messages belonging to this NATS account will
     * also be deleted.
     *
     * @param request - The request {@link NatsApiDeleteNatsAccountRequest}
     */
    deleteNatsAccount: (request: Readonly<NatsApiDeleteNatsAccountRequest>) => Promise<void>;
    /**
     * Update the name of a NATS account. Update the name of a NATS account,
     * specified by its NATS account ID.
     *
     * @param request - The request {@link NatsApiUpdateNatsAccountRequest}
     * @returns A Promise of NatsAccount
     */
    updateNatsAccount: (request: Readonly<NatsApiUpdateNatsAccountRequest$1>) => Promise<NatsAccount$1>;
    /**
     * Get a NATS account. Retrieve information about an existing NATS account
     * identified by its NATS account ID. Its full details, including name and
     * endpoint, are returned in the response.
     *
     * @param request - The request {@link NatsApiGetNatsAccountRequest}
     * @returns A Promise of NatsAccount
     */
    getNatsAccount: (request: Readonly<NatsApiGetNatsAccountRequest>) => Promise<NatsAccount$1>;
    protected pageOfListNatsAccounts: (request?: Readonly<NatsApiListNatsAccountsRequest>) => Promise<ListNatsAccountsResponse>;
    /**
     * List NATS accounts. List all NATS accounts in the specified region, for a
     * Scaleway Organization or Project. By default, the NATS accounts returned in
     * the list are ordered by creation date in ascending order, though this can
     * be modified via the `order_by` field.
     *
     * @param request - The request {@link NatsApiListNatsAccountsRequest}
     * @returns A Promise of ListNatsAccountsResponse
     */
    listNatsAccounts: (request?: Readonly<NatsApiListNatsAccountsRequest>) => Promise<ListNatsAccountsResponse> & {
        all: () => Promise<NatsAccount$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<NatsAccount$1[], void, void>;
    };
    /**
     * Create NATS credentials. Create a set of credentials for a NATS account,
     * specified by its NATS account ID.
     *
     * @param request - The request {@link NatsApiCreateNatsCredentialsRequest}
     * @returns A Promise of NatsCredentials
     */
    createNatsCredentials: (request: Readonly<NatsApiCreateNatsCredentialsRequest$1>) => Promise<NatsCredentials$1>;
    /**
     * Delete NATS credentials. Delete a set of credentials, specified by their
     * credentials ID. Deleting credentials is irreversible and cannot be undone.
     * The credentials can no longer be used to access the NATS account, and
     * active connections using this credentials will be closed.
     *
     * @param request - The request {@link NatsApiDeleteNatsCredentialsRequest}
     */
    deleteNatsCredentials: (request: Readonly<NatsApiDeleteNatsCredentialsRequest>) => Promise<void>;
    /**
     * Get NATS credentials. Retrieve an existing set of credentials, identified
     * by the `nats_credentials_id`. The credentials themselves are NOT returned,
     * only their metadata (NATS account ID, credentials name, etc), are returned
     * in the response.
     *
     * @param request - The request {@link NatsApiGetNatsCredentialsRequest}
     * @returns A Promise of NatsCredentials
     */
    getNatsCredentials: (request: Readonly<NatsApiGetNatsCredentialsRequest>) => Promise<NatsCredentials$1>;
    protected pageOfListNatsCredentials: (request?: Readonly<NatsApiListNatsCredentialsRequest>) => Promise<ListNatsCredentialsResponse>;
    /**
     * List NATS credentials. List existing credentials in the specified NATS
     * account. The response contains only the metadata for the credentials, not
     * the credentials themselves, which are only returned after a **Create
     * Credentials** call.
     *
     * @param request - The request {@link NatsApiListNatsCredentialsRequest}
     * @returns A Promise of ListNatsCredentialsResponse
     */
    listNatsCredentials: (request?: Readonly<NatsApiListNatsCredentialsRequest>) => Promise<ListNatsCredentialsResponse> & {
        all: () => Promise<NatsCredentials$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<NatsCredentials$1[], void, void>;
    };
}
/**
 * Messaging and Queuing SNS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing SNS brokers.
 */
declare class SnsAPI extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Activate SNS. Activate SNS for the specified Project ID. SNS must be
     * activated before any usage. Activating SNS does not trigger any billing,
     * and you can deactivate at any time.
     *
     * @param request - The request {@link SnsApiActivateSnsRequest}
     * @returns A Promise of SnsInfo
     */
    activateSns: (request?: Readonly<SnsApiActivateSnsRequest>) => Promise<SnsInfo>;
    /**
     * Get SNS info. Retrieve the SNS information of the specified Project ID.
     * Informations include the activation status and the SNS API endpoint URL.
     *
     * @param request - The request {@link SnsApiGetSnsInfoRequest}
     * @returns A Promise of SnsInfo
     */
    getSnsInfo: (request?: Readonly<SnsApiGetSnsInfoRequest>) => Promise<SnsInfo>;
    /**
     * Deactivate SNS. Deactivate SNS for the specified Project ID.You must delete
     * all topics and credentials before this call or you need to set the
     * force_delete parameter.
     *
     * @param request - The request {@link SnsApiDeactivateSnsRequest}
     * @returns A Promise of SnsInfo
     */
    deactivateSns: (request?: Readonly<SnsApiDeactivateSnsRequest>) => Promise<SnsInfo>;
    /**
     * Create SNS credentials. Create a set of credentials for SNS, specified by a
     * Project ID. Credentials give the bearer access to topics, and the level of
     * permissions can be defined granularly.
     *
     * @param request - The request {@link SnsApiCreateSnsCredentialsRequest}
     * @returns A Promise of SnsCredentials
     */
    createSnsCredentials: (request?: Readonly<SnsApiCreateSnsCredentialsRequest$1>) => Promise<SnsCredentials$1>;
    /**
     * Delete SNS credentials. Delete a set of SNS credentials, specified by their
     * credentials ID. Deleting credentials is irreversible and cannot be undone.
     * The credentials can then no longer be used to access SNS.
     *
     * @param request - The request {@link SnsApiDeleteSnsCredentialsRequest}
     */
    deleteSnsCredentials: (request: Readonly<SnsApiDeleteSnsCredentialsRequest>) => Promise<void>;
    /**
     * Update SNS credentials. Update a set of SNS credentials. You can update the
     * credentials' name, or their permissions.
     *
     * @param request - The request {@link SnsApiUpdateSnsCredentialsRequest}
     * @returns A Promise of SnsCredentials
     */
    updateSnsCredentials: (request: Readonly<SnsApiUpdateSnsCredentialsRequest$1>) => Promise<SnsCredentials$1>;
    /**
     * Get SNS credentials. Retrieve an existing set of credentials, identified by
     * the `credentials_id`. The credentials themselves, as well as their metadata
     * (name, project ID etc), are returned in the response.
     *
     * @param request - The request {@link SnsApiGetSnsCredentialsRequest}
     * @returns A Promise of SnsCredentials
     */
    getSnsCredentials: (request: Readonly<SnsApiGetSnsCredentialsRequest>) => Promise<SnsCredentials$1>;
    protected pageOfListSnsCredentials: (request?: Readonly<SnsApiListSnsCredentialsRequest>) => Promise<ListSnsCredentialsResponse>;
    /**
     * List SNS credentials. List existing SNS credentials in the specified
     * region. The response contains only the metadata for the credentials, not
     * the credentials themselves.
     *
     * @param request - The request {@link SnsApiListSnsCredentialsRequest}
     * @returns A Promise of ListSnsCredentialsResponse
     */
    listSnsCredentials: (request?: Readonly<SnsApiListSnsCredentialsRequest>) => Promise<ListSnsCredentialsResponse> & {
        all: () => Promise<SnsCredentials$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<SnsCredentials$1[], void, void>;
    };
}
/**
 * Messaging and Queuing SQS API.
 *
 * This API allows you to manage Scaleway Messaging and Queueing SQS brokers.
 */
declare class SqsAPI extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Activate SQS. Activate SQS for the specified Project ID. SQS must be
     * activated before any usage such as creating credentials and queues.
     * Activating SQS does not trigger any billing, and you can deactivate at any
     * time.
     *
     * @param request - The request {@link SqsApiActivateSqsRequest}
     * @returns A Promise of SqsInfo
     */
    activateSqs: (request?: Readonly<SqsApiActivateSqsRequest>) => Promise<SqsInfo>;
    /**
     * Get SQS info. Retrieve the SQS information of the specified Project ID.
     * Informations include the activation status and the SQS API endpoint URL.
     *
     * @param request - The request {@link SqsApiGetSqsInfoRequest}
     * @returns A Promise of SqsInfo
     */
    getSqsInfo: (request?: Readonly<SqsApiGetSqsInfoRequest>) => Promise<SqsInfo>;
    /**
     * Deactivate SQS. Deactivate SQS for the specified Project ID. You must
     * delete all queues and credentials before this call or you need to set the
     * force_delete parameter.
     *
     * @param request - The request {@link SqsApiDeactivateSqsRequest}
     * @returns A Promise of SqsInfo
     */
    deactivateSqs: (request?: Readonly<SqsApiDeactivateSqsRequest>) => Promise<SqsInfo>;
    /**
     * Create SQS credentials. Create a set of credentials for SQS, specified by a
     * Project ID. Credentials give the bearer access to queues, and the level of
     * permissions can be defined granularly.
     *
     * @param request - The request {@link SqsApiCreateSqsCredentialsRequest}
     * @returns A Promise of SqsCredentials
     */
    createSqsCredentials: (request?: Readonly<SqsApiCreateSqsCredentialsRequest$1>) => Promise<SqsCredentials$1>;
    /**
     * Delete SQS credentials. Delete a set of SQS credentials, specified by their
     * credentials ID. Deleting credentials is irreversible and cannot be undone.
     * The credentials can then no longer be used to access SQS.
     *
     * @param request - The request {@link SqsApiDeleteSqsCredentialsRequest}
     */
    deleteSqsCredentials: (request: Readonly<SqsApiDeleteSqsCredentialsRequest>) => Promise<void>;
    /**
     * Update SQS credentials. Update a set of SQS credentials. You can update the
     * credentials' name, or their permissions.
     *
     * @param request - The request {@link SqsApiUpdateSqsCredentialsRequest}
     * @returns A Promise of SqsCredentials
     */
    updateSqsCredentials: (request: Readonly<SqsApiUpdateSqsCredentialsRequest$1>) => Promise<SqsCredentials$1>;
    /**
     * Get SQS credentials. Retrieve an existing set of credentials, identified by
     * the `credentials_id`. The credentials themselves, as well as their metadata
     * (name, project ID etc), are returned in the response.
     *
     * @param request - The request {@link SqsApiGetSqsCredentialsRequest}
     * @returns A Promise of SqsCredentials
     */
    getSqsCredentials: (request: Readonly<SqsApiGetSqsCredentialsRequest>) => Promise<SqsCredentials$1>;
    protected pageOfListSqsCredentials: (request?: Readonly<SqsApiListSqsCredentialsRequest>) => Promise<ListSqsCredentialsResponse>;
    /**
     * List SQS credentials. List existing SQS credentials in the specified
     * region. The response contains only the metadata for the credentials, not
     * the credentials themselves.
     *
     * @param request - The request {@link SqsApiListSqsCredentialsRequest}
     * @returns A Promise of ListSqsCredentialsResponse
     */
    listSqsCredentials: (request?: Readonly<SqsApiListSqsCredentialsRequest>) => Promise<ListSqsCredentialsResponse> & {
        all: () => Promise<SqsCredentials$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<SqsCredentials$1[], void, void>;
    };
}

declare const NatsAccount: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const NatsApiCreateNatsAccountRequest: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const NatsApiCreateNatsCredentialsRequest: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const NatsApiUpdateNatsAccountRequest: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const NatsCredentials: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const SnsApiCreateSnsCredentialsRequest: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const SnsApiUpdateSnsCredentialsRequest: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const SnsCredentials: {
    accessKey: {
        minLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const SqsApiCreateSqsCredentialsRequest: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const SqsApiUpdateSqsCredentialsRequest: {
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const SqsCredentials: {
    accessKey: {
        minLength: number;
    };
    name: {
        maxLength: number;
        minLength: number;
    };
};

declare const validationRules_gen$3_NatsAccount: typeof NatsAccount;
declare const validationRules_gen$3_NatsApiCreateNatsAccountRequest: typeof NatsApiCreateNatsAccountRequest;
declare const validationRules_gen$3_NatsApiCreateNatsCredentialsRequest: typeof NatsApiCreateNatsCredentialsRequest;
declare const validationRules_gen$3_NatsApiUpdateNatsAccountRequest: typeof NatsApiUpdateNatsAccountRequest;
declare const validationRules_gen$3_NatsCredentials: typeof NatsCredentials;
declare const validationRules_gen$3_SnsApiCreateSnsCredentialsRequest: typeof SnsApiCreateSnsCredentialsRequest;
declare const validationRules_gen$3_SnsApiUpdateSnsCredentialsRequest: typeof SnsApiUpdateSnsCredentialsRequest;
declare const validationRules_gen$3_SnsCredentials: typeof SnsCredentials;
declare const validationRules_gen$3_SqsApiCreateSqsCredentialsRequest: typeof SqsApiCreateSqsCredentialsRequest;
declare const validationRules_gen$3_SqsApiUpdateSqsCredentialsRequest: typeof SqsApiUpdateSqsCredentialsRequest;
declare const validationRules_gen$3_SqsCredentials: typeof SqsCredentials;
declare namespace validationRules_gen$3 {
  export { validationRules_gen$3_NatsAccount as NatsAccount, validationRules_gen$3_NatsApiCreateNatsAccountRequest as NatsApiCreateNatsAccountRequest, validationRules_gen$3_NatsApiCreateNatsCredentialsRequest as NatsApiCreateNatsCredentialsRequest, validationRules_gen$3_NatsApiUpdateNatsAccountRequest as NatsApiUpdateNatsAccountRequest, validationRules_gen$3_NatsCredentials as NatsCredentials, validationRules_gen$3_SnsApiCreateSnsCredentialsRequest as SnsApiCreateSnsCredentialsRequest, validationRules_gen$3_SnsApiUpdateSnsCredentialsRequest as SnsApiUpdateSnsCredentialsRequest, validationRules_gen$3_SnsCredentials as SnsCredentials, validationRules_gen$3_SqsApiCreateSqsCredentialsRequest as SqsApiCreateSqsCredentialsRequest, validationRules_gen$3_SqsApiUpdateSqsCredentialsRequest as SqsApiUpdateSqsCredentialsRequest, validationRules_gen$3_SqsCredentials as SqsCredentials };
}

type index_gen$c_File = File;
type index_gen$c_ListNatsAccountsRequestOrderBy = ListNatsAccountsRequestOrderBy;
type index_gen$c_ListNatsAccountsResponse = ListNatsAccountsResponse;
type index_gen$c_ListNatsCredentialsRequestOrderBy = ListNatsCredentialsRequestOrderBy;
type index_gen$c_ListNatsCredentialsResponse = ListNatsCredentialsResponse;
type index_gen$c_ListSnsCredentialsRequestOrderBy = ListSnsCredentialsRequestOrderBy;
type index_gen$c_ListSnsCredentialsResponse = ListSnsCredentialsResponse;
type index_gen$c_ListSqsCredentialsRequestOrderBy = ListSqsCredentialsRequestOrderBy;
type index_gen$c_ListSqsCredentialsResponse = ListSqsCredentialsResponse;
type index_gen$c_NatsAPI = NatsAPI;
declare const index_gen$c_NatsAPI: typeof NatsAPI;
type index_gen$c_NatsApiDeleteNatsAccountRequest = NatsApiDeleteNatsAccountRequest;
type index_gen$c_NatsApiDeleteNatsCredentialsRequest = NatsApiDeleteNatsCredentialsRequest;
type index_gen$c_NatsApiGetNatsAccountRequest = NatsApiGetNatsAccountRequest;
type index_gen$c_NatsApiGetNatsCredentialsRequest = NatsApiGetNatsCredentialsRequest;
type index_gen$c_NatsApiListNatsAccountsRequest = NatsApiListNatsAccountsRequest;
type index_gen$c_NatsApiListNatsCredentialsRequest = NatsApiListNatsCredentialsRequest;
type index_gen$c_SnsAPI = SnsAPI;
declare const index_gen$c_SnsAPI: typeof SnsAPI;
type index_gen$c_SnsApiActivateSnsRequest = SnsApiActivateSnsRequest;
type index_gen$c_SnsApiDeactivateSnsRequest = SnsApiDeactivateSnsRequest;
type index_gen$c_SnsApiDeleteSnsCredentialsRequest = SnsApiDeleteSnsCredentialsRequest;
type index_gen$c_SnsApiGetSnsCredentialsRequest = SnsApiGetSnsCredentialsRequest;
type index_gen$c_SnsApiGetSnsInfoRequest = SnsApiGetSnsInfoRequest;
type index_gen$c_SnsApiListSnsCredentialsRequest = SnsApiListSnsCredentialsRequest;
type index_gen$c_SnsInfo = SnsInfo;
type index_gen$c_SnsInfoStatus = SnsInfoStatus;
type index_gen$c_SnsPermissions = SnsPermissions;
type index_gen$c_SqsAPI = SqsAPI;
declare const index_gen$c_SqsAPI: typeof SqsAPI;
type index_gen$c_SqsApiActivateSqsRequest = SqsApiActivateSqsRequest;
type index_gen$c_SqsApiDeactivateSqsRequest = SqsApiDeactivateSqsRequest;
type index_gen$c_SqsApiDeleteSqsCredentialsRequest = SqsApiDeleteSqsCredentialsRequest;
type index_gen$c_SqsApiGetSqsCredentialsRequest = SqsApiGetSqsCredentialsRequest;
type index_gen$c_SqsApiGetSqsInfoRequest = SqsApiGetSqsInfoRequest;
type index_gen$c_SqsApiListSqsCredentialsRequest = SqsApiListSqsCredentialsRequest;
type index_gen$c_SqsInfo = SqsInfo;
type index_gen$c_SqsInfoStatus = SqsInfoStatus;
type index_gen$c_SqsPermissions = SqsPermissions;
declare namespace index_gen$c {
  export { type index_gen$c_File as File, type index_gen$c_ListNatsAccountsRequestOrderBy as ListNatsAccountsRequestOrderBy, type index_gen$c_ListNatsAccountsResponse as ListNatsAccountsResponse, type index_gen$c_ListNatsCredentialsRequestOrderBy as ListNatsCredentialsRequestOrderBy, type index_gen$c_ListNatsCredentialsResponse as ListNatsCredentialsResponse, type index_gen$c_ListSnsCredentialsRequestOrderBy as ListSnsCredentialsRequestOrderBy, type index_gen$c_ListSnsCredentialsResponse as ListSnsCredentialsResponse, type index_gen$c_ListSqsCredentialsRequestOrderBy as ListSqsCredentialsRequestOrderBy, type index_gen$c_ListSqsCredentialsResponse as ListSqsCredentialsResponse, index_gen$c_NatsAPI as NatsAPI, type NatsAccount$1 as NatsAccount, type NatsApiCreateNatsAccountRequest$1 as NatsApiCreateNatsAccountRequest, type NatsApiCreateNatsCredentialsRequest$1 as NatsApiCreateNatsCredentialsRequest, type index_gen$c_NatsApiDeleteNatsAccountRequest as NatsApiDeleteNatsAccountRequest, type index_gen$c_NatsApiDeleteNatsCredentialsRequest as NatsApiDeleteNatsCredentialsRequest, type index_gen$c_NatsApiGetNatsAccountRequest as NatsApiGetNatsAccountRequest, type index_gen$c_NatsApiGetNatsCredentialsRequest as NatsApiGetNatsCredentialsRequest, type index_gen$c_NatsApiListNatsAccountsRequest as NatsApiListNatsAccountsRequest, type index_gen$c_NatsApiListNatsCredentialsRequest as NatsApiListNatsCredentialsRequest, type NatsApiUpdateNatsAccountRequest$1 as NatsApiUpdateNatsAccountRequest, type NatsCredentials$1 as NatsCredentials, index_gen$c_SnsAPI as SnsAPI, type index_gen$c_SnsApiActivateSnsRequest as SnsApiActivateSnsRequest, type SnsApiCreateSnsCredentialsRequest$1 as SnsApiCreateSnsCredentialsRequest, type index_gen$c_SnsApiDeactivateSnsRequest as SnsApiDeactivateSnsRequest, type index_gen$c_SnsApiDeleteSnsCredentialsRequest as SnsApiDeleteSnsCredentialsRequest, type index_gen$c_SnsApiGetSnsCredentialsRequest as SnsApiGetSnsCredentialsRequest, type index_gen$c_SnsApiGetSnsInfoRequest as SnsApiGetSnsInfoRequest, type index_gen$c_SnsApiListSnsCredentialsRequest as SnsApiListSnsCredentialsRequest, type SnsApiUpdateSnsCredentialsRequest$1 as SnsApiUpdateSnsCredentialsRequest, type SnsCredentials$1 as SnsCredentials, type index_gen$c_SnsInfo as SnsInfo, type index_gen$c_SnsInfoStatus as SnsInfoStatus, type index_gen$c_SnsPermissions as SnsPermissions, index_gen$c_SqsAPI as SqsAPI, type index_gen$c_SqsApiActivateSqsRequest as SqsApiActivateSqsRequest, type SqsApiCreateSqsCredentialsRequest$1 as SqsApiCreateSqsCredentialsRequest, type index_gen$c_SqsApiDeactivateSqsRequest as SqsApiDeactivateSqsRequest, type index_gen$c_SqsApiDeleteSqsCredentialsRequest as SqsApiDeleteSqsCredentialsRequest, type index_gen$c_SqsApiGetSqsCredentialsRequest as SqsApiGetSqsCredentialsRequest, type index_gen$c_SqsApiGetSqsInfoRequest as SqsApiGetSqsInfoRequest, type index_gen$c_SqsApiListSqsCredentialsRequest as SqsApiListSqsCredentialsRequest, type SqsApiUpdateSqsCredentialsRequest$1 as SqsApiUpdateSqsCredentialsRequest, type SqsCredentials$1 as SqsCredentials, type index_gen$c_SqsInfo as SqsInfo, type index_gen$c_SqsInfoStatus as SqsInfoStatus, type index_gen$c_SqsPermissions as SqsPermissions, validationRules_gen$3 as ValidationRules };
}

declare namespace index$a {
  export { index_gen$c as v1beta1 };
}

type ACLRuleAction = 'allow' | 'deny';
type ACLRuleDirection = 'inbound' | 'outbound';
type ACLRuleProtocol = 'tcp' | 'udp' | 'icmp';
type DatabaseBackupStatus$1 = 'unknown' | 'creating' | 'ready' | 'restoring' | 'deleting' | 'error' | 'exporting' | 'locked';
type EngineSettingPropertyType = 'BOOLEAN' | 'INT' | 'STRING' | 'FLOAT';
type InstanceLogStatus = 'unknown' | 'ready' | 'creating' | 'error';
type InstanceStatus = 'unknown' | 'ready' | 'provisioning' | 'configuring' | 'deleting' | 'error' | 'autohealing' | 'locked' | 'initializing' | 'disk_full' | 'backuping' | 'snapshotting' | 'restarting';
type ListDatabaseBackupsRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc' | 'status_asc' | 'status_desc';
type ListDatabasesRequestOrderBy$1 = 'name_asc' | 'name_desc' | 'size_asc' | 'size_desc';
type ListInstanceLogsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListInstancesRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc' | 'region' | 'status_asc' | 'status_desc';
type ListPrivilegesRequestOrderBy = 'user_name_asc' | 'user_name_desc' | 'database_name_asc' | 'database_name_desc';
type ListSnapshotsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc' | 'expires_at_asc' | 'expires_at_desc';
type ListUsersRequestOrderBy = 'name_asc' | 'name_desc' | 'is_admin_asc' | 'is_admin_desc';
type MaintenanceStatus = 'unknown' | 'pending' | 'done' | 'canceled';
type NodeTypeGeneration = 'unknown_generation' | 'generation_v1' | 'generation_v2';
type NodeTypeStock$1 = 'unknown' | 'low_stock' | 'out_of_stock' | 'available';
type Permission = 'readonly' | 'readwrite' | 'all' | 'custom' | 'none';
type ReadReplicaStatus = 'unknown' | 'provisioning' | 'initializing' | 'ready' | 'deleting' | 'error' | 'locked' | 'configuring' | 'promoting';
type SnapshotStatus = 'unknown' | 'creating' | 'ready' | 'restoring' | 'deleting' | 'error' | 'locked';
type StorageClass = 'unknown_storage_class' | 'lssd' | 'bssd' | 'sbs';
type VolumeType = 'lssd' | 'bssd' | 'sbs_5k' | 'sbs_15k';
interface EndpointDirectAccessDetails {
}
interface EndpointLoadBalancerDetails {
}
interface EndpointPrivateNetworkDetails {
    /** UUID of the private network. */
    privateNetworkId: string;
    /** CIDR notation of the endpoint IPv4 address. */
    serviceIp: string;
    /** Private network zone. */
    zone: Zone;
}
interface EndpointSpecPrivateNetworkIpamConfig {
}
interface ReadReplicaEndpointSpecPrivateNetworkIpamConfig {
}
interface EngineSetting {
    /** Setting name from the database engine. */
    name: string;
    /** Value set when not specified. */
    defaultValue: string;
    /** Setting can be applied without restarting. */
    hotConfigurable: boolean;
    /** Setting description. */
    description: string;
    /** Setting type. */
    propertyType: EngineSettingPropertyType;
    /** Setting base unit. */
    unit?: string;
    /** Validation regex for string type settings. */
    stringConstraint?: string;
    /** Minimum value for int types. */
    intMin?: number;
    /** Maximum value for int types. */
    intMax?: number;
    /** Minimum value for float types. */
    floatMin?: number;
    /** Maximum value for float types. */
    floatMax?: number;
}
interface Endpoint$1 {
    /** UUID of the endpoint. */
    id: string;
    /**
     * IPv4 address of the endpoint.
     *
     * One-of ('address'): at most one of 'ip', 'hostname' could be set.
     */
    ip?: string;
    /** TCP port of the endpoint. */
    port: number;
    /** Name of the endpoint. */
    name?: string;
    /**
     * Private Network details. One maximum per Database Instance or Read Replica
     * (a Database Instance and its Read Replica can have different Private
     * Networks). Cannot be updated (has to be deleted and recreated).
     *
     * One-of ('details'): at most one of 'privateNetwork', 'loadBalancer',
     * 'directAccess' could be set.
     */
    privateNetwork?: EndpointPrivateNetworkDetails;
    /**
     * Load balancer details. Public endpoint for Database Instance which is
     * systematically present. One per Database Instance.
     *
     * One-of ('details'): at most one of 'privateNetwork', 'loadBalancer',
     * 'directAccess' could be set.
     */
    loadBalancer?: EndpointLoadBalancerDetails;
    /**
     * Direct access details. Public endpoint reserved for Read Replicas. One per
     * Read Replica.
     *
     * One-of ('details'): at most one of 'privateNetwork', 'loadBalancer',
     * 'directAccess' could be set.
     */
    directAccess?: EndpointDirectAccessDetails;
    /**
     * Hostname of the endpoint.
     *
     * One-of ('address'): at most one of 'ip', 'hostname' could be set.
     */
    hostname?: string;
}
interface EndpointSpecLoadBalancer {
}
interface EndpointSpecPrivateNetwork {
    /** UUID of the Private Network to be connected to the Database Instance. */
    privateNetworkId: string;
    /**
     * Endpoint IPv4 address with a CIDR notation. Refer to the official Scaleway
     * documentation to learn more about IP and subnet limitations.
     *
     * One-of ('config'): at most one of 'serviceIp', 'ipamConfig' could be set.
     */
    serviceIp?: string;
    /**
     * Automated configuration of your Private Network endpoint with Scaleway IPAM
     * service. One at the most per Database Instance or Read Replica (a Database
     * Instance and its Read Replica can have different Private Networks). Cannot
     * be updated (has to be deleted and recreated).
     *
     * One-of ('config'): at most one of 'serviceIp', 'ipamConfig' could be set.
     */
    ipamConfig?: EndpointSpecPrivateNetworkIpamConfig;
}
interface ReadReplicaEndpointSpecDirectAccess {
}
interface ReadReplicaEndpointSpecPrivateNetwork {
    /** UUID of the Private Network to be connected to the Read Replica. */
    privateNetworkId: string;
    /**
     * Endpoint IPv4 address with a CIDR notation. Refer to the official Scaleway
     * documentation to learn more about IP and subnet limitations.
     *
     * One-of ('config'): at most one of 'serviceIp', 'ipamConfig' could be set.
     */
    serviceIp?: string;
    /**
     * Automated configuration of your Private Network endpoint with Scaleway IPAM
     * service. One at the most per Database Instance or Read Replica (a Database
     * Instance and its Read Replica can have different private networks). Cannot
     * be updated (has to be deleted and recreated).
     *
     * One-of ('config'): at most one of 'serviceIp', 'ipamConfig' could be set.
     */
    ipamConfig?: ReadReplicaEndpointSpecPrivateNetworkIpamConfig;
}
interface EngineVersion {
    /** Database engine version. */
    version: string;
    /** Database engine name. */
    name: string;
    /** End of life date. */
    endOfLife?: Date;
    /** Engine settings available to be set. */
    availableSettings: EngineSetting[];
    /** Disabled versions cannot be created. */
    disabled: boolean;
    /** Beta status of engine version. */
    beta: boolean;
    /** Engine settings available to be set at database initialization. */
    availableInitSettings: EngineSetting[];
}
interface BackupSchedule {
    /** Frequency of the backup schedule (in hours). */
    frequency: number;
    /** Default retention period of backups (in days). */
    retention: number;
    /** Defines whether the backup schedule feature is disabled. */
    disabled: boolean;
    /** Next run of the backup schedule (accurate to 10 minutes). */
    nextRunAt?: Date;
}
interface InstanceSetting {
    name: string;
    value: string;
}
interface LogsPolicy {
    /** Max age (in days) of remote logs to keep on the Database Instance. */
    maxAgeRetention?: number;
    /** Max disk size of remote logs to keep on the Database Instance. */
    totalDiskRetention?: number;
}
interface Maintenance {
    /** Start date of the maintenance window. */
    startsAt?: Date;
    /** End date of the maintenance window. */
    stopsAt?: Date;
    /** Closed maintenance date. */
    closedAt?: Date;
    /** Maintenance information message. */
    reason: string;
    /** Status of the maintenance. */
    status: MaintenanceStatus;
}
interface ReadReplica {
    /** UUID of the Read Replica. */
    id: string;
    /** Display Read Replica connection information. */
    endpoints: Endpoint$1[];
    /** Read replica status. */
    status: ReadReplicaStatus;
    /** Region the Read Replica is in. */
    region: Region;
    /**
     * Whether the replica is in the same availability zone as the main instance
     * nodes or not.
     */
    sameZone: boolean;
    /** UUID of the Database Instance. */
    instanceId: string;
}
interface UpgradableVersion {
    id: string;
    name: string;
    version: string;
    minorVersion: string;
}
interface Volume {
    type: VolumeType;
    size: number;
    class: StorageClass;
}
interface NodeTypeVolumeConstraintSizes {
    /** [deprecated] Mimimum size required for the Volume. */
    minSize: number;
    /** [deprecated] Maximum size required for the Volume. */
    maxSize: number;
}
interface NodeTypeVolumeType {
    /** Volume Type. */
    type: VolumeType;
    /** The description of the Volume. */
    description: string;
    /** Mimimum size required for the Volume. */
    minSize: number;
    /** Maximum size required for the Volume. */
    maxSize: number;
    /** Minimum increment level for a Block Storage volume size. */
    chunkSize: number;
    /** The storage class of the volume. */
    class: StorageClass;
}
interface SnapshotVolumeType {
    type: VolumeType;
    class: StorageClass;
}
interface ACLRuleRequest {
    ip: string;
    description: string;
}
interface ACLRule$1 {
    ip: string;
    /** @deprecated */
    port?: number;
    protocol: ACLRuleProtocol;
    direction: ACLRuleDirection;
    action: ACLRuleAction;
    description: string;
}
interface EndpointSpec$1 {
    /**
     * Load balancer endpoint specifications. Public endpoint for Database
     * Instance which is systematically present. One per RDB instance.
     *
     * One-of ('spec'): at most one of 'loadBalancer', 'privateNetwork' could be
     * set.
     */
    loadBalancer?: EndpointSpecLoadBalancer;
    /**
     * Private Network endpoint specifications. One maximum per Database Instance
     * or Read Replica (a Database Instance and its Read Replica can have
     * different Private Networks). Cannot be updated (has to be deleted and
     * recreated).
     *
     * One-of ('spec'): at most one of 'loadBalancer', 'privateNetwork' could be
     * set.
     */
    privateNetwork?: EndpointSpecPrivateNetwork;
}
interface ReadReplicaEndpointSpec {
    /**
     * Direct access endpoint specifications. Public endpoint reserved for Read
     * Replicas. One per Read Replica.
     *
     * One-of ('spec'): at most one of 'directAccess', 'privateNetwork' could be
     * set.
     */
    directAccess?: ReadReplicaEndpointSpecDirectAccess;
    /**
     * Private Network endpoint specifications. One at the most per Read Replica.
     * Cannot be updated (has to be deleted and recreated).
     *
     * One-of ('spec'): at most one of 'directAccess', 'privateNetwork' could be
     * set.
     */
    privateNetwork?: ReadReplicaEndpointSpecPrivateNetwork;
}
interface DatabaseBackup$1 {
    /** UUID of the database backup. */
    id: string;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of backed up database. */
    databaseName: string;
    /** Name of the backup. */
    name: string;
    /** Status of the backup. */
    status: DatabaseBackupStatus$1;
    /** Size of the database backup. */
    size?: number;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
    /** Creation date (must follow the ISO 8601 format). */
    createdAt?: Date;
    /** Updated date (must follow the ISO 8601 format). */
    updatedAt?: Date;
    /** Name of the Database Instance of the backup. */
    instanceName: string;
    /** URL you can download the backup from. */
    downloadUrl?: string;
    /** Expiration date of the download link. */
    downloadUrlExpiresAt?: Date;
    /** Region of the database backup. */
    region: Region;
    /** Store logical backups in the same region as the source Database Instance. */
    sameRegion: boolean;
}
interface DatabaseEngine {
    /** Engine name. */
    name: string;
    /** Engine logo URL. */
    logoUrl: string;
    /** Available versions. */
    versions: EngineVersion[];
    /** Region of this Database Instance. */
    region: Region;
}
interface Database$2 {
    /** Name of the database. */
    name: string;
    /** Name of the database owner. */
    owner: string;
    /** Defines whether the database is managed or not. */
    managed: boolean;
    /** Size of the database. */
    size: number;
}
interface ListInstanceLogsDetailsResponseInstanceLogDetail {
    logName: string;
    size: number;
}
interface InstanceLog {
    /** Presigned S3 URL to download your log file. */
    downloadUrl?: string;
    /** UUID of the Database Instance log. */
    id: string;
    /** Status of the logs in a Database Instance. */
    status: InstanceLogStatus;
    /** Name of the underlying node. */
    nodeName: string;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
    /** Creation date (must follow the ISO 8601 format). */
    createdAt?: Date;
    /** Region the Database Instance is in. */
    region: Region;
}
interface Instance {
    /** Creation date (must follow the ISO 8601 format). */
    createdAt?: Date;
    /** Volumes of the Database Instance. */
    volume?: Volume;
    /** Region the Database Instance is in. */
    region: Region;
    /** UUID of the Database Instance. */
    id: string;
    /** Name of the Database Instance. */
    name: string;
    /** Organization ID the Database Instance belongs to. */
    organizationId: string;
    /** Project ID the Database Instance belongs to. */
    projectId: string;
    /** Status of the Database Instance. */
    status: InstanceStatus;
    /** Database engine of the database (PostgreSQL, MySQL, ...). */
    engine: string;
    /** Available database engine versions for upgrade. */
    upgradableVersion: UpgradableVersion[];
    /** @deprecated Endpoint of the Database Instance. */
    endpoint?: Endpoint$1;
    /** List of tags applied to the Database Instance. */
    tags: string[];
    /** Advanced settings of the Database Instance. */
    settings: InstanceSetting[];
    /** Backup schedule of the Database Instance. */
    backupSchedule?: BackupSchedule;
    /** Defines whether or not High-Availability is enabled. */
    isHaCluster: boolean;
    /** Read Replicas of the Database Instance. */
    readReplicas: ReadReplica[];
    /** Node type of the Database Instance. */
    nodeType: string;
    /** List of engine settings to be set at database initialization. */
    initSettings: InstanceSetting[];
    /** List of Database Instance endpoints. */
    endpoints: Endpoint$1[];
    /** Logs policy of the Database Instance. */
    logsPolicy?: LogsPolicy;
    /** Store logical backups in the same region as the Database Instance. */
    backupSameRegion: boolean;
    /** List of Database Instance maintenance events. */
    maintenances: Maintenance[];
}
interface NodeType$1 {
    /** Node Type name identifier. */
    name: string;
    /** Current stock status for the Node Type. */
    stockStatus: NodeTypeStock$1;
    /** Current specs of the offer. */
    description: string;
    /** Number of virtual CPUs. */
    vcpus: number;
    /** Quantity of RAM. */
    memory: number;
    /** @deprecated {undefined} Deprecated Node Type volume constraints. */
    volumeConstraint?: NodeTypeVolumeConstraintSizes;
    /** @deprecated The Node Type is compliant with Block Storage. */
    isBssdCompatible?: boolean;
    /** The Node Type is currently disabled. */
    disabled: boolean;
    /** The Node Type is currently in beta. */
    beta: boolean;
    /** Available storage options for the Node Type. */
    availableVolumeTypes: NodeTypeVolumeType[];
    /** The Node Type can be used only with high availability option. */
    isHaRequired: boolean;
    /** Generation associated with the NodeType offer. */
    generation: NodeTypeGeneration;
    /** Instance range associated with the NodeType offer. */
    instanceRange: string;
    /** Region the Node Type is in. */
    region: Region;
}
interface Privilege {
    /** Permission (Read, Read/Write, All, Custom). */
    permission: Permission;
    /** Name of the database. */
    databaseName: string;
    /** Name of the user. */
    userName: string;
}
interface Snapshot {
    /** UUID of the snapshot. */
    id: string;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the snapshot. */
    name: string;
    /** Status of the snapshot. */
    status: SnapshotStatus;
    /** Size of the snapshot. */
    size?: number;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
    /** Creation date (must follow the ISO 8601 format). */
    createdAt?: Date;
    /** Updated date (must follow the ISO 8601 format). */
    updatedAt?: Date;
    /** Name of the Database Instance of the snapshot. */
    instanceName: string;
    /** Source node type. */
    nodeType: string;
    /** Type of volume where data is stored (lssd, bssd or sbs). */
    volumeType?: SnapshotVolumeType;
    /** Region of this snapshot. */
    region: Region;
}
interface User {
    /**
     * Name of the user (Length must be between 1 and 63 characters for PostgreSQL
     * and between 1 and 32 characters for MySQL. First character must be an
     * alphabet character (a-zA-Z). Your username cannot start with '_rdb' or in
     * PostgreSQL, 'pg_'. Only a-zA-Z0-9_$- characters are accepted).
     */
    name: string;
    /**
     * Defines whether or not a user got administrative privileges on the Database
     * Instance.
     */
    isAdmin: boolean;
}
interface UpgradeInstanceRequestMajorUpgradeWorkflow {
    /**
     * This will create a new Database Instance with same specifications as the
     * current one and perform a Database Engine upgrade.
     */
    upgradableVersionId: string;
    /**
     * At the end of the migration procedure this option let you migrate all your
     * database endpoint to the upgraded instance.
     */
    withEndpoints: boolean;
}
type AddInstanceACLRulesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to add ACL rules to. */
    instanceId: string;
    /** ACL rules to add to the Database Instance. */
    rules: ACLRuleRequest[];
};
interface AddInstanceACLRulesResponse {
    /** ACL Rules enabled for the Database Instance. */
    rules: ACLRule$1[];
}
type AddInstanceSettingsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to add settings to. */
    instanceId: string;
    /** Settings to add to the Database Instance. */
    settings: InstanceSetting[];
};
interface AddInstanceSettingsResponse {
    /** Settings available on the Database Instance. */
    settings: InstanceSetting[];
}
type CloneInstanceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to clone. */
    instanceId: string;
    /** Name of the Database Instance clone. */
    name: string;
    /** Node type of the clone. */
    nodeType?: string;
};
type CreateDatabaseBackupRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the database you want to back up. */
    databaseName: string;
    /** Name of the backup. */
    name?: string;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
};
type CreateDatabaseRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance where to create the database. */
    instanceId: string;
    /** Name of the database. */
    name: string;
};
type CreateEndpointRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you to which you want to add an endpoint. */
    instanceId: string;
    /** Specification of the endpoint you want to create. */
    endpointSpec?: EndpointSpec$1;
};
type CreateInstanceFromSnapshotRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Block snapshot of the Database Instance. */
    snapshotId: string;
    /** Name of the Database Instance created with the snapshot. */
    instanceName: string;
    /**
     * Defines whether or not High-Availability is enabled on the new Database
     * Instance.
     */
    isHaCluster?: boolean;
    /** The node type used to restore the snapshot. */
    nodeType?: string;
};
type CreateInstanceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * @deprecated Please use project_id instead.
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * The Project ID on which the Database Instance will be created.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
    /** Name of the Database Instance. */
    name?: string;
    /** Database engine of the Database Instance (PostgreSQL, MySQL, ...). */
    engine: string;
    /** Username created when the Database Instance is created. */
    userName: string;
    /** Password of the user. */
    password: string;
    /** Type of node to use for the Database Instance. */
    nodeType: string;
    /** Defines whether or not High-Availability is enabled. */
    isHaCluster: boolean;
    /** Defines whether or not backups are disabled. */
    disableBackup: boolean;
    /** Tags to apply to the Database Instance. */
    tags?: string[];
    /** List of engine settings to be set upon Database Instance initialization. */
    initSettings?: InstanceSetting[];
    /** Type of volume where data is stored (lssd, bssd, ...). */
    volumeType?: VolumeType;
    /** Volume size when volume_type is not lssd. */
    volumeSize: number;
    /**
     * One or multiple EndpointSpec used to expose your Database Instance. A
     * load_balancer public endpoint is systematically created.
     */
    initEndpoints?: EndpointSpec$1[];
    /**
     * Defines whether to or not to store logical backups in the same region as
     * the Database Instance.
     */
    backupSameRegion: boolean;
};
type CreateReadReplicaEndpointRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
    /** Specification of the endpoint you want to create. */
    endpointSpec: ReadReplicaEndpointSpec[];
};
type CreateReadReplicaRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to create a Read Replica from. */
    instanceId: string;
    /** Specification of the endpoint you want to create. */
    endpointSpec?: ReadReplicaEndpointSpec[];
    /**
     * Defines whether to create the replica in the same availability zone as the
     * main instance nodes or not.
     */
    sameZone?: boolean;
};
type CreateSnapshotRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the snapshot. */
    name?: string;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
};
type CreateUserRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance in which you want to create a user. */
    instanceId: string;
    /** Name of the user you want to create. */
    name: string;
    /** Password of the user you want to create. */
    password: string;
    /** Defines whether the user will have administrative privileges. */
    isAdmin: boolean;
};
type DeleteDatabaseBackupRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the database backup to delete. */
    databaseBackupId: string;
};
type DeleteDatabaseRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance where to delete the database. */
    instanceId: string;
    /** Name of the database to delete. */
    name: string;
};
type DeleteEndpointRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** This endpoint can also be used to delete a Read Replica endpoint. */
    endpointId: string;
};
type DeleteInstanceACLRulesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to delete an ACL rule from. */
    instanceId: string;
    /** IP addresses defined in the ACL rules of the Database Instance. */
    aclRuleIps: string[];
};
interface DeleteInstanceACLRulesResponse {
    /** IP addresses defined in the ACL rules of the Database Instance. */
    rules: ACLRule$1[];
}
type DeleteInstanceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to delete. */
    instanceId: string;
};
type DeleteInstanceSettingsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to delete settings from. */
    instanceId: string;
    /** Settings names to delete. */
    settingNames: string[];
};
interface DeleteInstanceSettingsResponse {
    /** Settings names to delete from the Database Instance. */
    settings: InstanceSetting[];
}
type DeleteReadReplicaRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
};
type DeleteSnapshotRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the snapshot to delete. */
    snapshotId: string;
};
type DeleteUserRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to delete the user from. */
    instanceId: string;
    /** Name of the user. */
    name: string;
};
type ExportDatabaseBackupRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the database backup you want to export. */
    databaseBackupId: string;
};
type GetDatabaseBackupRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the database backup. */
    databaseBackupId: string;
};
type GetEndpointRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the endpoint you want to get. */
    endpointId: string;
};
type GetInstanceCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
};
type GetInstanceLogRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the instance_log you want. */
    instanceLogId: string;
};
type GetInstanceMetricsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Start date to gather metrics from. */
    startDate?: Date;
    /** End date to gather metrics from. */
    endDate?: Date;
    /** Name of the metric to gather. */
    metricName?: string;
};
type GetInstanceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
};
type GetReadReplicaRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
};
type GetSnapshotRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the snapshot. */
    snapshotId: string;
};
interface InstanceMetrics {
    /** Time series of metrics of a Database Instance. */
    timeseries: TimeSeries[];
}
type ListDatabaseBackupsRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the database backups. */
    name?: string;
    /** Criteria to use when ordering database backups listing. */
    orderBy?: ListDatabaseBackupsRequestOrderBy$1;
    /** UUID of the Database Instance. */
    instanceId?: string;
    /** Organization ID of the Organization the database backups belong to. */
    organizationId?: string;
    /** Project ID of the Project the database backups belong to. */
    projectId?: string;
    page?: number;
    pageSize?: number;
};
interface ListDatabaseBackupsResponse$1 {
    /** List of database backups. */
    databaseBackups: DatabaseBackup$1[];
    /** Total count of database backups available. */
    totalCount: number;
}
type ListDatabaseEnginesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the database engine. */
    name?: string;
    /** Version of the database engine. */
    version?: string;
    page?: number;
    pageSize?: number;
};
interface ListDatabaseEnginesResponse {
    /** List of the available database engines. */
    engines: DatabaseEngine[];
    /** Total count of database engines available. */
    totalCount: number;
}
type ListDatabasesRequest$2 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to list the databases of. */
    instanceId: string;
    /** Name of the database. */
    name?: string;
    /** Defines whether or not the database is managed. */
    managed?: boolean;
    /** User that owns this database. */
    owner?: string;
    /** Criteria to use when ordering database listing. */
    orderBy?: ListDatabasesRequestOrderBy$1;
    page?: number;
    pageSize?: number;
};
interface ListDatabasesResponse$1 {
    /** List of the databases. */
    databases: Database$2[];
    /** Total count of databases present on a Database Instance. */
    totalCount: number;
}
type ListInstanceACLRulesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    page?: number;
    pageSize?: number;
};
interface ListInstanceACLRulesResponse {
    /** List of ACL rules present on a Database Instance. */
    rules: ACLRule$1[];
    /** Total count of ACL rules present on a Database Instance. */
    totalCount: number;
}
type ListInstanceLogsDetailsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
};
interface ListInstanceLogsDetailsResponse {
    /** Remote Database Instance logs details. */
    details: ListInstanceLogsDetailsResponseInstanceLogDetail[];
}
type ListInstanceLogsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
    /** Criteria to use when ordering Database Instance logs listing. */
    orderBy?: ListInstanceLogsRequestOrderBy;
};
interface ListInstanceLogsResponse {
    /** Available logs in a Database Instance. */
    instanceLogs: InstanceLog[];
}
type ListInstancesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** List Database Instances that have a given tag. */
    tags?: string[];
    /** Lists Database Instances that match a name pattern. */
    name?: string;
    /** Criteria to use when ordering Database Instance listings. */
    orderBy?: ListInstancesRequestOrderBy;
    /** Please use project_id instead. */
    organizationId?: string;
    /** Project ID to list the Database Instance of. */
    projectId?: string;
    page?: number;
    pageSize?: number;
};
interface ListInstancesResponse {
    /** List of all Database Instances available in an Organization or Project. */
    instances: Instance[];
    /** Total count of Database Instances available in a Organization or Project. */
    totalCount: number;
}
type ListNodeTypesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Defines whether or not to include disabled types. */
    includeDisabledTypes: boolean;
    page?: number;
    pageSize?: number;
};
interface ListNodeTypesResponse$1 {
    /** Types of the node. */
    nodeTypes: NodeType$1[];
    /** Total count of node-types available. */
    totalCount: number;
}
type ListPrivilegesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Criteria to use when ordering privileges listing. */
    orderBy?: ListPrivilegesRequestOrderBy;
    page?: number;
    pageSize?: number;
    /** Name of the database. */
    databaseName?: string;
    /** Name of the user. */
    userName?: string;
};
interface ListPrivilegesResponse {
    /** Privileges of a user in a database in a Database Instance. */
    privileges: Privilege[];
    /** Total count of privileges present on a database. */
    totalCount: number;
}
type ListSnapshotsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the snapshot. */
    name?: string;
    /** Criteria to use when ordering snapshot listing. */
    orderBy?: ListSnapshotsRequestOrderBy;
    /** UUID of the Database Instance. */
    instanceId?: string;
    /** Organization ID the snapshots belongs to. */
    organizationId?: string;
    /** Project ID the snapshots belongs to. */
    projectId?: string;
    page?: number;
    pageSize?: number;
};
interface ListSnapshotsResponse {
    /** List of snapshots. */
    snapshots: Snapshot[];
    /** Total count of snapshots available. */
    totalCount: number;
}
type ListUsersRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the user. */
    name?: string;
    /** Criteria to use when requesting user listing. */
    orderBy?: ListUsersRequestOrderBy;
    page?: number;
    pageSize?: number;
};
interface ListUsersResponse {
    /** List of users in a Database Instance. */
    users: User[];
    /** Total count of users present on a Database Instance. */
    totalCount: number;
}
type MigrateEndpointRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the endpoint you want to migrate. */
    endpointId: string;
    /** UUID of the instance you want to attach the endpoint to. */
    instanceId: string;
};
type PrepareInstanceLogsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
    /** Start datetime of your log. (RFC 3339 format). */
    startDate?: Date;
    /** End datetime of your log. (RFC 3339 format). */
    endDate?: Date;
};
interface PrepareInstanceLogsResponse {
    /** Instance logs for a Database Instance between a start and an end date. */
    instanceLogs: InstanceLog[];
}
type PromoteReadReplicaRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
};
type PurgeInstanceLogsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
    /** Given log name to purge. */
    logName?: string;
};
type RenewInstanceCertificateRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want logs of. */
    instanceId: string;
};
type ResetReadReplicaRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Read Replica. */
    readReplicaId: string;
};
type RestartInstanceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to restart. */
    instanceId: string;
};
type RestoreDatabaseBackupRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Backup of a logical database. */
    databaseBackupId: string;
    /**
     * Defines the destination database to restore into a specified database (the
     * default destination is set to the origin database of the backup).
     */
    databaseName?: string;
    /** Defines the Database Instance where the backup has to be restored. */
    instanceId: string;
};
type SetInstanceACLRulesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance where the ACL rules must be set. */
    instanceId: string;
    /** ACL rules to define for the Database Instance. */
    rules: ACLRuleRequest[];
};
interface SetInstanceACLRulesResponse {
    /** ACLs rules configured for a Database Instance. */
    rules: ACLRule$1[];
}
type SetInstanceSettingsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance where the settings must be set. */
    instanceId: string;
    /** Settings to define for the Database Instance. */
    settings: InstanceSetting[];
};
interface SetInstanceSettingsResponse {
    /** Settings configured for a Database Instance. */
    settings: InstanceSetting[];
}
type SetPrivilegeRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance. */
    instanceId: string;
    /** Name of the database. */
    databaseName: string;
    /** Name of the user. */
    userName: string;
    /** Permission to set (Read, Read/Write, All, Custom). */
    permission?: Permission;
};
type UpdateDatabaseBackupRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the database backup to update. */
    databaseBackupId: string;
    /** Name of the Database Backup. */
    name?: string;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
};
type UpdateInstanceRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance to update. */
    instanceId: string;
    /** In hours. */
    backupScheduleFrequency?: number;
    /** In days. */
    backupScheduleRetention?: number;
    /** Defines whether or not the backup schedule is disabled. */
    isBackupScheduleDisabled?: boolean;
    /** Name of the Database Instance. */
    name?: string;
    /** Tags of a Database Instance. */
    tags?: string[];
    /** Logs policy of the Database Instance. */
    logsPolicy?: LogsPolicy;
    /** Store logical backups in the same region as the Database Instance. */
    backupSameRegion?: boolean;
    /** Defines the start time of the autobackup. */
    backupScheduleStartHour?: number;
};
type UpdateSnapshotRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the snapshot to update. */
    snapshotId: string;
    /** Name of the snapshot. */
    name?: string;
    /** Expiration date (must follow the ISO 8601 format). */
    expiresAt?: Date;
};
type UpdateUserRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance the user belongs to. */
    instanceId: string;
    /** Name of the database user. */
    name: string;
    /** Password of the database user. */
    password?: string;
    /** Defines whether or not this user got administrative privileges. */
    isAdmin?: boolean;
};
type UpgradeInstanceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Database Instance you want to upgrade. */
    instanceId: string;
    /**
     * Node type of the Database Instance you want to upgrade to.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    nodeType?: string;
    /**
     * Defines whether or not high availability should be enabled on the Database
     * Instance.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    enableHa?: boolean;
    /**
     * Increase your block storage volume size.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    volumeSize?: number;
    /**
     * Change your Database Instance storage type.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    volumeType?: VolumeType;
    /**
     * This will create a new Database Instance with same specifications as the
     * current one and perform a Database Engine upgrade.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    upgradableVersionId?: string;
    /**
     * Upgrade your database engine to a new major version including instance
     * endpoints.
     *
     * One-of ('upgradeTarget'): at most one of 'nodeType', 'enableHa',
     * 'volumeSize', 'volumeType', 'upgradableVersionId', 'majorUpgradeWorkflow'
     * could be set.
     */
    majorUpgradeWorkflow?: UpgradeInstanceRequestMajorUpgradeWorkflow;
};

/** Managed Database for PostgreSQL and MySQL API. */
declare class API$a extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListDatabaseEngines: (request?: Readonly<ListDatabaseEnginesRequest>) => Promise<ListDatabaseEnginesResponse>;
    /**
     * List available database engines. List the PostgreSQL and MySQL database
     * engines available at Scaleway.
     *
     * @param request - The request {@link ListDatabaseEnginesRequest}
     * @returns A Promise of ListDatabaseEnginesResponse
     */
    listDatabaseEngines: (request?: Readonly<ListDatabaseEnginesRequest>) => Promise<ListDatabaseEnginesResponse> & {
        all: () => Promise<DatabaseEngine[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DatabaseEngine[], void, void>;
    };
    protected pageOfListNodeTypes: (request: Readonly<ListNodeTypesRequest$1>) => Promise<ListNodeTypesResponse$1>;
    /**
     * List available node types. List all available node types. By default, the
     * node types returned in the list are ordered by creation date in ascending
     * order, though this can be modified via the `order_by` field.
     *
     * @param request - The request {@link ListNodeTypesRequest}
     * @returns A Promise of ListNodeTypesResponse
     */
    listNodeTypes: (request: Readonly<ListNodeTypesRequest$1>) => Promise<ListNodeTypesResponse$1> & {
        all: () => Promise<NodeType$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<NodeType$1[], void, void>;
    };
    protected pageOfListDatabaseBackups: (request?: Readonly<ListDatabaseBackupsRequest$2>) => Promise<ListDatabaseBackupsResponse$1>;
    /**
     * List database backups. List all backups in a specified region, for a given
     * Scaleway Organization or Scaleway Project. By default, the backups listed
     * are ordered by creation date in ascending order. This can be modified via
     * the `order_by` field.
     *
     * @param request - The request {@link ListDatabaseBackupsRequest}
     * @returns A Promise of ListDatabaseBackupsResponse
     */
    listDatabaseBackups: (request?: Readonly<ListDatabaseBackupsRequest$2>) => Promise<ListDatabaseBackupsResponse$1> & {
        all: () => Promise<DatabaseBackup$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DatabaseBackup$1[], void, void>;
    };
    /**
     * Create a database backup. Create a new backup. You must set the
     * `instance_id`, `database_name`, `name` and `expires_at` parameters.
     *
     * @param request - The request {@link CreateDatabaseBackupRequest}
     * @returns A Promise of DatabaseBackup
     */
    createDatabaseBackup: (request: Readonly<CreateDatabaseBackupRequest>) => Promise<DatabaseBackup$1>;
    /**
     * Get a database backup. Retrieve information about a given backup, specified
     * by its database backup ID and region. Full details about the backup, like
     * size, URL and expiration date, are returned in the response.
     *
     * @param request - The request {@link GetDatabaseBackupRequest}
     * @returns A Promise of DatabaseBackup
     */
    getDatabaseBackup: (request: Readonly<GetDatabaseBackupRequest$1>) => Promise<DatabaseBackup$1>;
    /**
     * Waits for {@link DatabaseBackup} to be in a final state.
     *
     * @param request - The request {@link GetDatabaseBackupRequest}
     * @param options - The waiting options
     * @returns A Promise of DatabaseBackup
     */
    waitForDatabaseBackup: (request: Readonly<GetDatabaseBackupRequest$1>, options?: Readonly<WaitForOptions<DatabaseBackup$1>>) => Promise<DatabaseBackup$1>;
    /**
     * Update a database backup. Update the parameters of a backup, including name
     * and expiration date.
     *
     * @param request - The request {@link UpdateDatabaseBackupRequest}
     * @returns A Promise of DatabaseBackup
     */
    updateDatabaseBackup: (request: Readonly<UpdateDatabaseBackupRequest>) => Promise<DatabaseBackup$1>;
    /**
     * Delete a database backup. Delete a backup, specified by its database backup
     * ID and region. Deleting a backup is permanent, and cannot be undone.
     *
     * @param request - The request {@link DeleteDatabaseBackupRequest}
     * @returns A Promise of DatabaseBackup
     */
    deleteDatabaseBackup: (request: Readonly<DeleteDatabaseBackupRequest>) => Promise<DatabaseBackup$1>;
    /**
     * Restore a database backup. Launch the process of restoring database backup.
     * You must specify the `instance_id` of the Database Instance of destination,
     * where the backup will be restored. Note that large database backups can
     * take up to several hours to restore.
     *
     * @param request - The request {@link RestoreDatabaseBackupRequest}
     * @returns A Promise of DatabaseBackup
     */
    restoreDatabaseBackup: (request: Readonly<RestoreDatabaseBackupRequest>) => Promise<DatabaseBackup$1>;
    /**
     * Export a database backup. Export a backup, specified by the
     * `database_backup_id` and the `region` parameters. The download URL is
     * returned in the response.
     *
     * @param request - The request {@link ExportDatabaseBackupRequest}
     * @returns A Promise of DatabaseBackup
     */
    exportDatabaseBackup: (request: Readonly<ExportDatabaseBackupRequest$1>) => Promise<DatabaseBackup$1>;
    /**
     * Upgrade a Database Instance. Upgrade your current Database Instance
     * specifications like node type, high availability, volume, or the database
     * engine version. Note that upon upgrade the `enable_ha` parameter can only
     * be set to `true`.
     *
     * @param request - The request {@link UpgradeInstanceRequest}
     * @returns A Promise of Instance
     */
    upgradeInstance: (request: Readonly<UpgradeInstanceRequest>) => Promise<Instance>;
    protected pageOfListInstances: (request?: Readonly<ListInstancesRequest>) => Promise<ListInstancesResponse>;
    /**
     * List Database Instances. List all Database Instances in the specified
     * region, for a given Scaleway Organization or Scaleway Project. By default,
     * the Database Instances returned in the list are ordered by creation date in
     * ascending order, though this can be modified via the order_by field. You
     * can define additional parameters for your query, such as `tags` and `name`.
     * For the `name` parameter, the value you include will be checked against the
     * whole name string to see if it includes the string you put in the
     * parameter.
     *
     * @param request - The request {@link ListInstancesRequest}
     * @returns A Promise of ListInstancesResponse
     */
    listInstances: (request?: Readonly<ListInstancesRequest>) => Promise<ListInstancesResponse> & {
        all: () => Promise<Instance[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Instance[], void, void>;
    };
    /**
     * Get a Database Instance. Retrieve information about a given Database
     * Instance, specified by the `region` and `instance_id` parameters. Its full
     * details, including name, status, IP address and port, are returned in the
     * response object.
     *
     * @param request - The request {@link GetInstanceRequest}
     * @returns A Promise of Instance
     */
    getInstance: (request: Readonly<GetInstanceRequest>) => Promise<Instance>;
    /**
     * Waits for {@link Instance} to be in a final state.
     *
     * @param request - The request {@link GetInstanceRequest}
     * @param options - The waiting options
     * @returns A Promise of Instance
     */
    waitForInstance: (request: Readonly<GetInstanceRequest>, options?: Readonly<WaitForOptions<Instance>>) => Promise<Instance>;
    /**
     * Create a Database Instance. Create a new Database Instance. You must set
     * the `engine`, `user_name`, `password` and `node_type` parameters.
     * Optionally, you can specify the volume type and size.
     *
     * @param request - The request {@link CreateInstanceRequest}
     * @returns A Promise of Instance
     */
    createInstance: (request: Readonly<CreateInstanceRequest>) => Promise<Instance>;
    /**
     * Update a Database Instance. Update the parameters of a Database Instance,
     * including name, tags and backup schedule details.
     *
     * @param request - The request {@link UpdateInstanceRequest}
     * @returns A Promise of Instance
     */
    updateInstance: (request: Readonly<UpdateInstanceRequest$1>) => Promise<Instance>;
    /**
     * Delete a Database Instance. Delete a given Database Instance, specified by
     * the `region` and `instance_id` parameters. Deleting a Database Instance is
     * permanent, and cannot be undone. Note that upon deletion all your data will
     * be lost.
     *
     * @param request - The request {@link DeleteInstanceRequest}
     * @returns A Promise of Instance
     */
    deleteInstance: (request: Readonly<DeleteInstanceRequest>) => Promise<Instance>;
    /**
     * Clone a Database Instance. Clone a given Database Instance, specified by
     * the `region` and `instance_id` parameters. The clone feature allows you to
     * create a new Database Instance from an existing one. The clone includes all
     * existing databases, users and permissions. You can create a clone on a
     * Database Instance bigger than your current one.
     *
     * @param request - The request {@link CloneInstanceRequest}
     * @returns A Promise of Instance
     */
    cloneInstance: (request: Readonly<CloneInstanceRequest>) => Promise<Instance>;
    /**
     * Restart Database Instance. Restart a given Database Instance, specified by
     * the `region` and `instance_id` parameters. The status of the Database
     * Instance returned in the response.
     *
     * @param request - The request {@link RestartInstanceRequest}
     * @returns A Promise of Instance
     */
    restartInstance: (request: Readonly<RestartInstanceRequest>) => Promise<Instance>;
    /**
     * Get the TLS certificate of a Database Instance. Retrieve information about
     * the TLS certificate of a given Database Instance. Details like name and
     * content are returned in the response.
     *
     * @param request - The request {@link GetInstanceCertificateRequest}
     * @returns A Promise of Blob
     */
    getInstanceCertificate: (request: Readonly<GetInstanceCertificateRequest>) => Promise<Blob>;
    /**
     * Renew the TLS certificate of a Database Instance. Renew a TLS for a
     * Database Instance. Renewing a certificate means that you will not be able
     * to connect to your Database Instance using the previous certificate. You
     * will also need to download and update the new certificate for all database
     * clients.
     *
     * @param request - The request {@link RenewInstanceCertificateRequest}
     */
    renewInstanceCertificate: (request: Readonly<RenewInstanceCertificateRequest>) => Promise<void>;
    /**
     * Get Database Instance metrics. Retrieve the time series metrics of a given
     * Database Instance. You can define the period from which to retrieve metrics
     * by specifying the `start_date` and `end_date`.
     *
     * @param request - The request {@link GetInstanceMetricsRequest}
     * @returns A Promise of InstanceMetrics
     */
    getInstanceMetrics: (request: Readonly<GetInstanceMetricsRequest>) => Promise<InstanceMetrics>;
    /**
     * Create a Read Replica. Create a new Read Replica of a Database Instance.
     * You must specify the `region` and the `instance_id`. You can only create a
     * maximum of 3 Read Replicas per Database Instance.
     *
     * @param request - The request {@link CreateReadReplicaRequest}
     * @returns A Promise of ReadReplica
     */
    createReadReplica: (request: Readonly<CreateReadReplicaRequest>) => Promise<ReadReplica>;
    /**
     * Get a Read Replica. Retrieve information about a Database Instance Read
     * Replica. Full details about the Read Replica, like `endpoints`, `status`
     * and `region` are returned in the response.
     *
     * @param request - The request {@link GetReadReplicaRequest}
     * @returns A Promise of ReadReplica
     */
    getReadReplica: (request: Readonly<GetReadReplicaRequest>) => Promise<ReadReplica>;
    /**
     * Waits for {@link ReadReplica} to be in a final state.
     *
     * @param request - The request {@link GetReadReplicaRequest}
     * @param options - The waiting options
     * @returns A Promise of ReadReplica
     */
    waitForReadReplica: (request: Readonly<GetReadReplicaRequest>, options?: Readonly<WaitForOptions<ReadReplica>>) => Promise<ReadReplica>;
    /**
     * Delete a Read Replica. Delete a Read Replica of a Database Instance. You
     * must specify the `region` and `read_replica_id` parameters of the Read
     * Replica you want to delete.
     *
     * @param request - The request {@link DeleteReadReplicaRequest}
     * @returns A Promise of ReadReplica
     */
    deleteReadReplica: (request: Readonly<DeleteReadReplicaRequest>) => Promise<ReadReplica>;
    /**
     * Resync a Read Replica. When you resync a Read Replica, first it is reset,
     * then its data is resynchronized from the primary node. Your Read Replica
     * remains unavailable during the resync process. The duration of this process
     * is proportional to the size of your Database Instance. The configured
     * endpoints do not change.
     *
     * @param request - The request {@link ResetReadReplicaRequest}
     * @returns A Promise of ReadReplica
     */
    resetReadReplica: (request: Readonly<ResetReadReplicaRequest>) => Promise<ReadReplica>;
    /**
     * Promote a Read Replica. Promote a Read Replica to Database Instance
     * automatically.
     *
     * @param request - The request {@link PromoteReadReplicaRequest}
     * @returns A Promise of Instance
     */
    promoteReadReplica: (request: Readonly<PromoteReadReplicaRequest>) => Promise<Instance>;
    /**
     * Create an endpoint for a Read Replica. Create a new endpoint for a Read
     * Replica. Read Replicas can have at most one direct access and one Private
     * Network endpoint.
     *
     * @param request - The request {@link CreateReadReplicaEndpointRequest}
     * @returns A Promise of ReadReplica
     */
    createReadReplicaEndpoint: (request: Readonly<CreateReadReplicaEndpointRequest>) => Promise<ReadReplica>;
    /**
     * Prepare logs of a Database Instance. Prepare your Database Instance logs.
     * You can define the `start_date` and `end_date` parameters for your query.
     * The download URL is returned in the response. Logs are recorded from 00h00
     * to 23h59 and then aggregated in a `.log` file once a day. Therefore, even
     * if you specify a timeframe from which you want to get the logs, you will
     * receive logs from the full 24 hours.
     *
     * @param request - The request {@link PrepareInstanceLogsRequest}
     * @returns A Promise of PrepareInstanceLogsResponse
     */
    prepareInstanceLogs: (request: Readonly<PrepareInstanceLogsRequest>) => Promise<PrepareInstanceLogsResponse>;
    /**
     * List available logs of a Database Instance. List the available logs of a
     * Database Instance. By default, the logs returned in the list are ordered by
     * creation date in ascending order, though this can be modified via the
     * order_by field.
     *
     * @param request - The request {@link ListInstanceLogsRequest}
     * @returns A Promise of ListInstanceLogsResponse
     */
    listInstanceLogs: (request: Readonly<ListInstanceLogsRequest>) => Promise<ListInstanceLogsResponse>;
    /**
     * Get given logs of a Database Instance. Retrieve information about the logs
     * of a Database Instance. Specify the `instance_log_id` and `region` in your
     * request to get information such as `download_url`, `status`, `expires_at`
     * and `created_at` about your logs in the response.
     *
     * @param request - The request {@link GetInstanceLogRequest}
     * @returns A Promise of InstanceLog
     */
    getInstanceLog: (request: Readonly<GetInstanceLogRequest>) => Promise<InstanceLog>;
    /**
     * Waits for {@link InstanceLog} to be in a final state.
     *
     * @param request - The request {@link GetInstanceLogRequest}
     * @param options - The waiting options
     * @returns A Promise of InstanceLog
     */
    waitForInstanceLog: (request: Readonly<GetInstanceLogRequest>, options?: Readonly<WaitForOptions<InstanceLog>>) => Promise<InstanceLog>;
    /**
     * Purge remote Database Instance logs. Purge a given remote log from a
     * Database Instance. You can specify the `log_name` of the log you wish to
     * clean from your Database Instance.
     *
     * @param request - The request {@link PurgeInstanceLogsRequest}
     */
    purgeInstanceLogs: (request: Readonly<PurgeInstanceLogsRequest>) => Promise<void>;
    /**
     * List remote Database Instance logs details. List remote log details. By
     * default, the details returned in the list are ordered by creation date in
     * ascending order, though this can be modified via the order_by field.
     *
     * @param request - The request {@link ListInstanceLogsDetailsRequest}
     * @returns A Promise of ListInstanceLogsDetailsResponse
     */
    listInstanceLogsDetails: (request: Readonly<ListInstanceLogsDetailsRequest>) => Promise<ListInstanceLogsDetailsResponse>;
    /**
     * Add Database Instance advanced settings. Add an advanced setting to a
     * Database Instance. You must set the `name` and the `value` of each
     * setting.
     *
     * @param request - The request {@link AddInstanceSettingsRequest}
     * @returns A Promise of AddInstanceSettingsResponse
     */
    addInstanceSettings: (request: Readonly<AddInstanceSettingsRequest>) => Promise<AddInstanceSettingsResponse>;
    /**
     * Delete Database Instance advanced settings. Delete an advanced setting in a
     * Database Instance. You must specify the names of the settings you want to
     * delete in the request.
     *
     * @param request - The request {@link DeleteInstanceSettingsRequest}
     * @returns A Promise of DeleteInstanceSettingsResponse
     */
    deleteInstanceSettings: (request: Readonly<DeleteInstanceSettingsRequest>) => Promise<DeleteInstanceSettingsResponse>;
    /**
     * Set Database Instance advanced settings. Update an advanced setting for a
     * Database Instance. Settings added upon database engine initalization can
     * only be defined once, and cannot, therefore, be updated.
     *
     * @param request - The request {@link SetInstanceSettingsRequest}
     * @returns A Promise of SetInstanceSettingsResponse
     */
    setInstanceSettings: (request: Readonly<SetInstanceSettingsRequest>) => Promise<SetInstanceSettingsResponse>;
    protected pageOfListInstanceACLRules: (request: Readonly<ListInstanceACLRulesRequest>) => Promise<ListInstanceACLRulesResponse>;
    /**
     * List ACL rules of a Database Instance. List the ACL rules for a given
     * Database Instance. The response is an array of ACL objects, each one
     * representing an ACL that denies, allows or redirects traffic based on
     * certain conditions.
     *
     * @param request - The request {@link ListInstanceACLRulesRequest}
     * @returns A Promise of ListInstanceACLRulesResponse
     */
    listInstanceACLRules: (request: Readonly<ListInstanceACLRulesRequest>) => Promise<ListInstanceACLRulesResponse> & {
        all: () => Promise<ACLRule$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ACLRule$1[], void, void>;
    };
    /**
     * Add an ACL rule to a Database Instance. Add an additional ACL rule to a
     * Database Instance.
     *
     * @param request - The request {@link AddInstanceACLRulesRequest}
     * @returns A Promise of AddInstanceACLRulesResponse
     */
    addInstanceACLRules: (request: Readonly<AddInstanceACLRulesRequest>) => Promise<AddInstanceACLRulesResponse>;
    /**
     * Set ACL rules for a Database Instance. Replace all the ACL rules of a
     * Database Instance.
     *
     * @param request - The request {@link SetInstanceACLRulesRequest}
     * @returns A Promise of SetInstanceACLRulesResponse
     */
    setInstanceACLRules: (request: Readonly<SetInstanceACLRulesRequest>) => Promise<SetInstanceACLRulesResponse>;
    /**
     * Delete ACL rules of a Database Instance. Delete one or more ACL rules of a
     * Database Instance.
     *
     * @param request - The request {@link DeleteInstanceACLRulesRequest}
     * @returns A Promise of DeleteInstanceACLRulesResponse
     */
    deleteInstanceACLRules: (request: Readonly<DeleteInstanceACLRulesRequest>) => Promise<DeleteInstanceACLRulesResponse>;
    protected pageOfListUsers: (request: Readonly<ListUsersRequest>) => Promise<ListUsersResponse>;
    /**
     * List users of a Database Instance. List all users of a given Database
     * Instance. By default, the users returned in the list are ordered by
     * creation date in ascending order, though this can be modified via the
     * order_by field.
     *
     * @param request - The request {@link ListUsersRequest}
     * @returns A Promise of ListUsersResponse
     */
    listUsers: (request: Readonly<ListUsersRequest>) => Promise<ListUsersResponse> & {
        all: () => Promise<User[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<User[], void, void>;
    };
    /**
     * Create a user for a Database Instance. Create a new user for a Database
     * Instance. You must define the `name`, `password` and `is_admin`
     * parameters.
     *
     * @param request - The request {@link CreateUserRequest}
     * @returns A Promise of User
     */
    createUser: (request: Readonly<CreateUserRequest>) => Promise<User>;
    /**
     * Update a user on a Database Instance. Update the parameters of a user on a
     * Database Instance. You can update the `password` and `is_admin` parameters,
     * but you cannot change the name of the user.
     *
     * @param request - The request {@link UpdateUserRequest}
     * @returns A Promise of User
     */
    updateUser: (request: Readonly<UpdateUserRequest>) => Promise<User>;
    /**
     * Delete a user on a Database Instance. Delete a given user on a Database
     * Instance. You must specify, in the endpoint, the `region`, `instance_id`
     * and `name` parameters of the user you want to delete.
     *
     * @param request - The request {@link DeleteUserRequest}
     */
    deleteUser: (request: Readonly<DeleteUserRequest>) => Promise<void>;
    protected pageOfListDatabases: (request: Readonly<ListDatabasesRequest$2>) => Promise<ListDatabasesResponse$1>;
    /**
     * List databases in a Database Instance. List all databases of a given
     * Database Instance. By default, the databases returned in the list are
     * ordered by creation date in ascending order, though this can be modified
     * via the order_by field. You can define additional parameters for your
     * query, such as `name`, `managed` and `owner`.
     *
     * @param request - The request {@link ListDatabasesRequest}
     * @returns A Promise of ListDatabasesResponse
     */
    listDatabases: (request: Readonly<ListDatabasesRequest$2>) => Promise<ListDatabasesResponse$1> & {
        all: () => Promise<Database$2[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Database$2[], void, void>;
    };
    /**
     * Create a database in a Database Instance. Create a new database. You must
     * define the `name` parameter in the request.
     *
     * @param request - The request {@link CreateDatabaseRequest}
     * @returns A Promise of Database
     */
    createDatabase: (request: Readonly<CreateDatabaseRequest$2>) => Promise<Database$2>;
    /**
     * Delete a database in a Database Instance. Delete a given database on a
     * Database Instance. You must specify, in the endpoint, the `region`,
     * `instance_id` and `name` parameters of the database you want to delete.
     *
     * @param request - The request {@link DeleteDatabaseRequest}
     */
    deleteDatabase: (request: Readonly<DeleteDatabaseRequest$1>) => Promise<void>;
    protected pageOfListPrivileges: (request: Readonly<ListPrivilegesRequest>) => Promise<ListPrivilegesResponse>;
    /**
     * List user privileges for a database. List privileges of a user on a
     * database. By default, the details returned in the list are ordered by
     * creation date in ascending order, though this can be modified via the
     * order_by field. You can define additional parameters for your query, such
     * as `database_name` and `user_name`.
     *
     * @param request - The request {@link ListPrivilegesRequest}
     * @returns A Promise of ListPrivilegesResponse
     */
    listPrivileges: (request: Readonly<ListPrivilegesRequest>) => Promise<ListPrivilegesResponse> & {
        all: () => Promise<Privilege[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Privilege[], void, void>;
    };
    /**
     * Set user privileges for a database. Set the privileges of a user on a
     * database. You must define `database_name`, `user_name` and `permission` in
     * the request body.
     *
     * @param request - The request {@link SetPrivilegeRequest}
     * @returns A Promise of Privilege
     */
    setPrivilege: (request: Readonly<SetPrivilegeRequest>) => Promise<Privilege>;
    protected pageOfListSnapshots: (request?: Readonly<ListSnapshotsRequest>) => Promise<ListSnapshotsResponse>;
    /**
     * List snapshots. List snapshots. You can include the `instance_id` or
     * `project_id` in your query to get the list of snapshots for specific
     * Database Instances and/or Projects. By default, the details returned in the
     * list are ordered by creation date in ascending order, though this can be
     * modified via the `order_by` field.
     *
     * @param request - The request {@link ListSnapshotsRequest}
     * @returns A Promise of ListSnapshotsResponse
     */
    listSnapshots: (request?: Readonly<ListSnapshotsRequest>) => Promise<ListSnapshotsResponse> & {
        all: () => Promise<Snapshot[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Snapshot[], void, void>;
    };
    /**
     * Get a Database Instance snapshot. Retrieve information about a given
     * snapshot, specified by its `snapshot_id` and `region`. Full details about
     * the snapshot, like size and expiration date, are returned in the response.
     *
     * @param request - The request {@link GetSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    getSnapshot: (request: Readonly<GetSnapshotRequest>) => Promise<Snapshot>;
    /**
     * Waits for {@link Snapshot} to be in a final state.
     *
     * @param request - The request {@link GetSnapshotRequest}
     * @param options - The waiting options
     * @returns A Promise of Snapshot
     */
    waitForSnapshot: (request: Readonly<GetSnapshotRequest>, options?: Readonly<WaitForOptions<Snapshot>>) => Promise<Snapshot>;
    /**
     * Create a Database Instance snapshot. Create a new snapshot of a Database
     * Instance. You must define the `name` parameter in the request.
     *
     * @param request - The request {@link CreateSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    createSnapshot: (request: Readonly<CreateSnapshotRequest>) => Promise<Snapshot>;
    /**
     * Update a Database Instance snapshot. Update the parameters of a snapshot of
     * a Database Instance. You can update the `name` and `expires_at`
     * parameters.
     *
     * @param request - The request {@link UpdateSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    updateSnapshot: (request: Readonly<UpdateSnapshotRequest>) => Promise<Snapshot>;
    /**
     * Delete a Database Instance snapshot. Delete a given snapshot of a Database
     * Instance. You must specify, in the endpoint, the `region` and `snapshot_id`
     * parameters of the snapshot you want to delete.
     *
     * @param request - The request {@link DeleteSnapshotRequest}
     * @returns A Promise of Snapshot
     */
    deleteSnapshot: (request: Readonly<DeleteSnapshotRequest>) => Promise<Snapshot>;
    /**
     * Create a new Database Instance from a snapshot. Restore a snapshot. When
     * you restore a snapshot, a new Instance is created and billed to your
     * account. Note that is possible to select a larger node type for your new
     * Database Instance. However, the Block volume size will be the same as the
     * size of the restored snapshot. All Instance settings will be restored if
     * you chose a node type with the same or more memory size than the initial
     * Instance. Settings will be reset to the default if your node type has less
     * memory.
     *
     * @param request - The request {@link CreateInstanceFromSnapshotRequest}
     * @returns A Promise of Instance
     */
    createInstanceFromSnapshot: (request: Readonly<CreateInstanceFromSnapshotRequest>) => Promise<Instance>;
    /**
     * Create a new Database Instance endpoint. Create a new endpoint for a
     * Database Instance. You can add `load_balancer` and `private_network`
     * specifications to the body of the request.
     *
     * @param request - The request {@link CreateEndpointRequest}
     * @returns A Promise of Endpoint
     */
    createEndpoint: (request: Readonly<CreateEndpointRequest>) => Promise<Endpoint$1>;
    /**
     * Delete a Database Instance endpoint. Delete the endpoint of a Database
     * Instance. You must specify the `region` and `endpoint_id` parameters of the
     * endpoint you want to delete. Note that might need to update any environment
     * configurations that point to the deleted endpoint.
     *
     * @param request - The request {@link DeleteEndpointRequest}
     */
    deleteEndpoint: (request: Readonly<DeleteEndpointRequest$1>) => Promise<void>;
    /**
     * Get a Database Instance endpoint. Retrieve information about a Database
     * Instance endpoint. Full details about the endpoint, like `ip`, `port`,
     * `private_network` and `load_balancer` specifications are returned in the
     * response.
     *
     * @param request - The request {@link GetEndpointRequest}
     * @returns A Promise of Endpoint
     */
    getEndpoint: (request: Readonly<GetEndpointRequest$1>) => Promise<Endpoint$1>;
    /**
     * Migrate an existing instance endpoint to another instance.
     *
     * @param request - The request {@link MigrateEndpointRequest}
     * @returns A Promise of Endpoint
     */
    migrateEndpoint: (request: Readonly<MigrateEndpointRequest>) => Promise<Endpoint$1>;
}

/** Lists transient statutes of the enum {@link DatabaseBackupStatus}. */
declare const DATABASE_BACKUP_TRANSIENT_STATUSES: DatabaseBackupStatus$1[];
/** Lists transient statutes of the enum {@link InstanceLogStatus}. */
declare const INSTANCE_LOG_TRANSIENT_STATUSES: InstanceLogStatus[];
/** Lists transient statutes of the enum {@link InstanceStatus}. */
declare const INSTANCE_TRANSIENT_STATUSES: InstanceStatus[];
/** Lists transient statutes of the enum {@link MaintenanceStatus}. */
declare const MAINTENANCE_TRANSIENT_STATUSES: MaintenanceStatus[];
/** Lists transient statutes of the enum {@link ReadReplicaStatus}. */
declare const READ_REPLICA_TRANSIENT_STATUSES: ReadReplicaStatus[];
/** Lists transient statutes of the enum {@link SnapshotStatus}. */
declare const SNAPSHOT_TRANSIENT_STATUSES: SnapshotStatus[];

declare const UpdateInstanceRequest: {
    backupScheduleStartHour: {
        lessThanOrEqual: number;
    };
};

declare const validationRules_gen$2_UpdateInstanceRequest: typeof UpdateInstanceRequest;
declare namespace validationRules_gen$2 {
  export { validationRules_gen$2_UpdateInstanceRequest as UpdateInstanceRequest };
}

type index_gen$b_ACLRuleAction = ACLRuleAction;
type index_gen$b_ACLRuleDirection = ACLRuleDirection;
type index_gen$b_ACLRuleProtocol = ACLRuleProtocol;
type index_gen$b_ACLRuleRequest = ACLRuleRequest;
type index_gen$b_AddInstanceACLRulesRequest = AddInstanceACLRulesRequest;
type index_gen$b_AddInstanceACLRulesResponse = AddInstanceACLRulesResponse;
type index_gen$b_AddInstanceSettingsRequest = AddInstanceSettingsRequest;
type index_gen$b_AddInstanceSettingsResponse = AddInstanceSettingsResponse;
type index_gen$b_BackupSchedule = BackupSchedule;
type index_gen$b_CloneInstanceRequest = CloneInstanceRequest;
type index_gen$b_CreateDatabaseBackupRequest = CreateDatabaseBackupRequest;
type index_gen$b_CreateEndpointRequest = CreateEndpointRequest;
type index_gen$b_CreateInstanceFromSnapshotRequest = CreateInstanceFromSnapshotRequest;
type index_gen$b_CreateInstanceRequest = CreateInstanceRequest;
type index_gen$b_CreateReadReplicaEndpointRequest = CreateReadReplicaEndpointRequest;
type index_gen$b_CreateReadReplicaRequest = CreateReadReplicaRequest;
type index_gen$b_CreateSnapshotRequest = CreateSnapshotRequest;
type index_gen$b_CreateUserRequest = CreateUserRequest;
declare const index_gen$b_DATABASE_BACKUP_TRANSIENT_STATUSES: typeof DATABASE_BACKUP_TRANSIENT_STATUSES;
type index_gen$b_DatabaseEngine = DatabaseEngine;
type index_gen$b_DeleteDatabaseBackupRequest = DeleteDatabaseBackupRequest;
type index_gen$b_DeleteInstanceACLRulesRequest = DeleteInstanceACLRulesRequest;
type index_gen$b_DeleteInstanceACLRulesResponse = DeleteInstanceACLRulesResponse;
type index_gen$b_DeleteInstanceRequest = DeleteInstanceRequest;
type index_gen$b_DeleteInstanceSettingsRequest = DeleteInstanceSettingsRequest;
type index_gen$b_DeleteInstanceSettingsResponse = DeleteInstanceSettingsResponse;
type index_gen$b_DeleteReadReplicaRequest = DeleteReadReplicaRequest;
type index_gen$b_DeleteSnapshotRequest = DeleteSnapshotRequest;
type index_gen$b_DeleteUserRequest = DeleteUserRequest;
type index_gen$b_EndpointDirectAccessDetails = EndpointDirectAccessDetails;
type index_gen$b_EndpointLoadBalancerDetails = EndpointLoadBalancerDetails;
type index_gen$b_EndpointPrivateNetworkDetails = EndpointPrivateNetworkDetails;
type index_gen$b_EndpointSpecLoadBalancer = EndpointSpecLoadBalancer;
type index_gen$b_EndpointSpecPrivateNetwork = EndpointSpecPrivateNetwork;
type index_gen$b_EndpointSpecPrivateNetworkIpamConfig = EndpointSpecPrivateNetworkIpamConfig;
type index_gen$b_EngineSetting = EngineSetting;
type index_gen$b_EngineSettingPropertyType = EngineSettingPropertyType;
type index_gen$b_EngineVersion = EngineVersion;
type index_gen$b_GetInstanceCertificateRequest = GetInstanceCertificateRequest;
type index_gen$b_GetInstanceLogRequest = GetInstanceLogRequest;
type index_gen$b_GetInstanceMetricsRequest = GetInstanceMetricsRequest;
type index_gen$b_GetInstanceRequest = GetInstanceRequest;
type index_gen$b_GetReadReplicaRequest = GetReadReplicaRequest;
type index_gen$b_GetSnapshotRequest = GetSnapshotRequest;
declare const index_gen$b_INSTANCE_LOG_TRANSIENT_STATUSES: typeof INSTANCE_LOG_TRANSIENT_STATUSES;
declare const index_gen$b_INSTANCE_TRANSIENT_STATUSES: typeof INSTANCE_TRANSIENT_STATUSES;
type index_gen$b_Instance = Instance;
type index_gen$b_InstanceLog = InstanceLog;
type index_gen$b_InstanceLogStatus = InstanceLogStatus;
type index_gen$b_InstanceMetrics = InstanceMetrics;
type index_gen$b_InstanceSetting = InstanceSetting;
type index_gen$b_InstanceStatus = InstanceStatus;
type index_gen$b_ListDatabaseEnginesRequest = ListDatabaseEnginesRequest;
type index_gen$b_ListDatabaseEnginesResponse = ListDatabaseEnginesResponse;
type index_gen$b_ListInstanceACLRulesRequest = ListInstanceACLRulesRequest;
type index_gen$b_ListInstanceACLRulesResponse = ListInstanceACLRulesResponse;
type index_gen$b_ListInstanceLogsDetailsRequest = ListInstanceLogsDetailsRequest;
type index_gen$b_ListInstanceLogsDetailsResponse = ListInstanceLogsDetailsResponse;
type index_gen$b_ListInstanceLogsDetailsResponseInstanceLogDetail = ListInstanceLogsDetailsResponseInstanceLogDetail;
type index_gen$b_ListInstanceLogsRequest = ListInstanceLogsRequest;
type index_gen$b_ListInstanceLogsRequestOrderBy = ListInstanceLogsRequestOrderBy;
type index_gen$b_ListInstanceLogsResponse = ListInstanceLogsResponse;
type index_gen$b_ListInstancesRequest = ListInstancesRequest;
type index_gen$b_ListInstancesRequestOrderBy = ListInstancesRequestOrderBy;
type index_gen$b_ListInstancesResponse = ListInstancesResponse;
type index_gen$b_ListPrivilegesRequest = ListPrivilegesRequest;
type index_gen$b_ListPrivilegesRequestOrderBy = ListPrivilegesRequestOrderBy;
type index_gen$b_ListPrivilegesResponse = ListPrivilegesResponse;
type index_gen$b_ListSnapshotsRequest = ListSnapshotsRequest;
type index_gen$b_ListSnapshotsRequestOrderBy = ListSnapshotsRequestOrderBy;
type index_gen$b_ListSnapshotsResponse = ListSnapshotsResponse;
type index_gen$b_ListUsersRequest = ListUsersRequest;
type index_gen$b_ListUsersRequestOrderBy = ListUsersRequestOrderBy;
type index_gen$b_ListUsersResponse = ListUsersResponse;
type index_gen$b_LogsPolicy = LogsPolicy;
declare const index_gen$b_MAINTENANCE_TRANSIENT_STATUSES: typeof MAINTENANCE_TRANSIENT_STATUSES;
type index_gen$b_Maintenance = Maintenance;
type index_gen$b_MaintenanceStatus = MaintenanceStatus;
type index_gen$b_MigrateEndpointRequest = MigrateEndpointRequest;
type index_gen$b_NodeTypeGeneration = NodeTypeGeneration;
type index_gen$b_NodeTypeVolumeConstraintSizes = NodeTypeVolumeConstraintSizes;
type index_gen$b_NodeTypeVolumeType = NodeTypeVolumeType;
type index_gen$b_Permission = Permission;
type index_gen$b_PrepareInstanceLogsRequest = PrepareInstanceLogsRequest;
type index_gen$b_PrepareInstanceLogsResponse = PrepareInstanceLogsResponse;
type index_gen$b_Privilege = Privilege;
type index_gen$b_PromoteReadReplicaRequest = PromoteReadReplicaRequest;
type index_gen$b_PurgeInstanceLogsRequest = PurgeInstanceLogsRequest;
declare const index_gen$b_READ_REPLICA_TRANSIENT_STATUSES: typeof READ_REPLICA_TRANSIENT_STATUSES;
type index_gen$b_ReadReplica = ReadReplica;
type index_gen$b_ReadReplicaEndpointSpec = ReadReplicaEndpointSpec;
type index_gen$b_ReadReplicaEndpointSpecDirectAccess = ReadReplicaEndpointSpecDirectAccess;
type index_gen$b_ReadReplicaEndpointSpecPrivateNetwork = ReadReplicaEndpointSpecPrivateNetwork;
type index_gen$b_ReadReplicaEndpointSpecPrivateNetworkIpamConfig = ReadReplicaEndpointSpecPrivateNetworkIpamConfig;
type index_gen$b_ReadReplicaStatus = ReadReplicaStatus;
type index_gen$b_RenewInstanceCertificateRequest = RenewInstanceCertificateRequest;
type index_gen$b_ResetReadReplicaRequest = ResetReadReplicaRequest;
type index_gen$b_RestartInstanceRequest = RestartInstanceRequest;
type index_gen$b_RestoreDatabaseBackupRequest = RestoreDatabaseBackupRequest;
declare const index_gen$b_SNAPSHOT_TRANSIENT_STATUSES: typeof SNAPSHOT_TRANSIENT_STATUSES;
type index_gen$b_SetInstanceACLRulesRequest = SetInstanceACLRulesRequest;
type index_gen$b_SetInstanceACLRulesResponse = SetInstanceACLRulesResponse;
type index_gen$b_SetInstanceSettingsRequest = SetInstanceSettingsRequest;
type index_gen$b_SetInstanceSettingsResponse = SetInstanceSettingsResponse;
type index_gen$b_SetPrivilegeRequest = SetPrivilegeRequest;
type index_gen$b_Snapshot = Snapshot;
type index_gen$b_SnapshotStatus = SnapshotStatus;
type index_gen$b_SnapshotVolumeType = SnapshotVolumeType;
type index_gen$b_StorageClass = StorageClass;
type index_gen$b_UpdateDatabaseBackupRequest = UpdateDatabaseBackupRequest;
type index_gen$b_UpdateSnapshotRequest = UpdateSnapshotRequest;
type index_gen$b_UpdateUserRequest = UpdateUserRequest;
type index_gen$b_UpgradableVersion = UpgradableVersion;
type index_gen$b_UpgradeInstanceRequest = UpgradeInstanceRequest;
type index_gen$b_UpgradeInstanceRequestMajorUpgradeWorkflow = UpgradeInstanceRequestMajorUpgradeWorkflow;
type index_gen$b_User = User;
type index_gen$b_Volume = Volume;
type index_gen$b_VolumeType = VolumeType;
declare namespace index_gen$b {
  export { type ACLRule$1 as ACLRule, type index_gen$b_ACLRuleAction as ACLRuleAction, type index_gen$b_ACLRuleDirection as ACLRuleDirection, type index_gen$b_ACLRuleProtocol as ACLRuleProtocol, type index_gen$b_ACLRuleRequest as ACLRuleRequest, API$a as API, type index_gen$b_AddInstanceACLRulesRequest as AddInstanceACLRulesRequest, type index_gen$b_AddInstanceACLRulesResponse as AddInstanceACLRulesResponse, type index_gen$b_AddInstanceSettingsRequest as AddInstanceSettingsRequest, type index_gen$b_AddInstanceSettingsResponse as AddInstanceSettingsResponse, type index_gen$b_BackupSchedule as BackupSchedule, type index_gen$b_CloneInstanceRequest as CloneInstanceRequest, type index_gen$b_CreateDatabaseBackupRequest as CreateDatabaseBackupRequest, type CreateDatabaseRequest$2 as CreateDatabaseRequest, type index_gen$b_CreateEndpointRequest as CreateEndpointRequest, type index_gen$b_CreateInstanceFromSnapshotRequest as CreateInstanceFromSnapshotRequest, type index_gen$b_CreateInstanceRequest as CreateInstanceRequest, type index_gen$b_CreateReadReplicaEndpointRequest as CreateReadReplicaEndpointRequest, type index_gen$b_CreateReadReplicaRequest as CreateReadReplicaRequest, type index_gen$b_CreateSnapshotRequest as CreateSnapshotRequest, type index_gen$b_CreateUserRequest as CreateUserRequest, index_gen$b_DATABASE_BACKUP_TRANSIENT_STATUSES as DATABASE_BACKUP_TRANSIENT_STATUSES, type Database$2 as Database, type DatabaseBackup$1 as DatabaseBackup, type DatabaseBackupStatus$1 as DatabaseBackupStatus, type index_gen$b_DatabaseEngine as DatabaseEngine, type index_gen$b_DeleteDatabaseBackupRequest as DeleteDatabaseBackupRequest, type DeleteDatabaseRequest$1 as DeleteDatabaseRequest, type DeleteEndpointRequest$1 as DeleteEndpointRequest, type index_gen$b_DeleteInstanceACLRulesRequest as DeleteInstanceACLRulesRequest, type index_gen$b_DeleteInstanceACLRulesResponse as DeleteInstanceACLRulesResponse, type index_gen$b_DeleteInstanceRequest as DeleteInstanceRequest, type index_gen$b_DeleteInstanceSettingsRequest as DeleteInstanceSettingsRequest, type index_gen$b_DeleteInstanceSettingsResponse as DeleteInstanceSettingsResponse, type index_gen$b_DeleteReadReplicaRequest as DeleteReadReplicaRequest, type index_gen$b_DeleteSnapshotRequest as DeleteSnapshotRequest, type index_gen$b_DeleteUserRequest as DeleteUserRequest, type Endpoint$1 as Endpoint, type index_gen$b_EndpointDirectAccessDetails as EndpointDirectAccessDetails, type index_gen$b_EndpointLoadBalancerDetails as EndpointLoadBalancerDetails, type index_gen$b_EndpointPrivateNetworkDetails as EndpointPrivateNetworkDetails, type EndpointSpec$1 as EndpointSpec, type index_gen$b_EndpointSpecLoadBalancer as EndpointSpecLoadBalancer, type index_gen$b_EndpointSpecPrivateNetwork as EndpointSpecPrivateNetwork, type index_gen$b_EndpointSpecPrivateNetworkIpamConfig as EndpointSpecPrivateNetworkIpamConfig, type index_gen$b_EngineSetting as EngineSetting, type index_gen$b_EngineSettingPropertyType as EngineSettingPropertyType, type index_gen$b_EngineVersion as EngineVersion, type ExportDatabaseBackupRequest$1 as ExportDatabaseBackupRequest, type GetDatabaseBackupRequest$1 as GetDatabaseBackupRequest, type GetEndpointRequest$1 as GetEndpointRequest, type index_gen$b_GetInstanceCertificateRequest as GetInstanceCertificateRequest, type index_gen$b_GetInstanceLogRequest as GetInstanceLogRequest, type index_gen$b_GetInstanceMetricsRequest as GetInstanceMetricsRequest, type index_gen$b_GetInstanceRequest as GetInstanceRequest, type index_gen$b_GetReadReplicaRequest as GetReadReplicaRequest, type index_gen$b_GetSnapshotRequest as GetSnapshotRequest, index_gen$b_INSTANCE_LOG_TRANSIENT_STATUSES as INSTANCE_LOG_TRANSIENT_STATUSES, index_gen$b_INSTANCE_TRANSIENT_STATUSES as INSTANCE_TRANSIENT_STATUSES, type index_gen$b_Instance as Instance, type index_gen$b_InstanceLog as InstanceLog, type index_gen$b_InstanceLogStatus as InstanceLogStatus, type index_gen$b_InstanceMetrics as InstanceMetrics, type index_gen$b_InstanceSetting as InstanceSetting, type index_gen$b_InstanceStatus as InstanceStatus, type ListDatabaseBackupsRequest$2 as ListDatabaseBackupsRequest, type ListDatabaseBackupsRequestOrderBy$1 as ListDatabaseBackupsRequestOrderBy, type ListDatabaseBackupsResponse$1 as ListDatabaseBackupsResponse, type index_gen$b_ListDatabaseEnginesRequest as ListDatabaseEnginesRequest, type index_gen$b_ListDatabaseEnginesResponse as ListDatabaseEnginesResponse, type ListDatabasesRequest$2 as ListDatabasesRequest, type ListDatabasesRequestOrderBy$1 as ListDatabasesRequestOrderBy, type ListDatabasesResponse$1 as ListDatabasesResponse, type index_gen$b_ListInstanceACLRulesRequest as ListInstanceACLRulesRequest, type index_gen$b_ListInstanceACLRulesResponse as ListInstanceACLRulesResponse, type index_gen$b_ListInstanceLogsDetailsRequest as ListInstanceLogsDetailsRequest, type index_gen$b_ListInstanceLogsDetailsResponse as ListInstanceLogsDetailsResponse, type index_gen$b_ListInstanceLogsDetailsResponseInstanceLogDetail as ListInstanceLogsDetailsResponseInstanceLogDetail, type index_gen$b_ListInstanceLogsRequest as ListInstanceLogsRequest, type index_gen$b_ListInstanceLogsRequestOrderBy as ListInstanceLogsRequestOrderBy, type index_gen$b_ListInstanceLogsResponse as ListInstanceLogsResponse, type index_gen$b_ListInstancesRequest as ListInstancesRequest, type index_gen$b_ListInstancesRequestOrderBy as ListInstancesRequestOrderBy, type index_gen$b_ListInstancesResponse as ListInstancesResponse, type ListNodeTypesRequest$1 as ListNodeTypesRequest, type ListNodeTypesResponse$1 as ListNodeTypesResponse, type index_gen$b_ListPrivilegesRequest as ListPrivilegesRequest, type index_gen$b_ListPrivilegesRequestOrderBy as ListPrivilegesRequestOrderBy, type index_gen$b_ListPrivilegesResponse as ListPrivilegesResponse, type index_gen$b_ListSnapshotsRequest as ListSnapshotsRequest, type index_gen$b_ListSnapshotsRequestOrderBy as ListSnapshotsRequestOrderBy, type index_gen$b_ListSnapshotsResponse as ListSnapshotsResponse, type index_gen$b_ListUsersRequest as ListUsersRequest, type index_gen$b_ListUsersRequestOrderBy as ListUsersRequestOrderBy, type index_gen$b_ListUsersResponse as ListUsersResponse, type index_gen$b_LogsPolicy as LogsPolicy, index_gen$b_MAINTENANCE_TRANSIENT_STATUSES as MAINTENANCE_TRANSIENT_STATUSES, type index_gen$b_Maintenance as Maintenance, type index_gen$b_MaintenanceStatus as MaintenanceStatus, type index_gen$b_MigrateEndpointRequest as MigrateEndpointRequest, type NodeType$1 as NodeType, type index_gen$b_NodeTypeGeneration as NodeTypeGeneration, type NodeTypeStock$1 as NodeTypeStock, type index_gen$b_NodeTypeVolumeConstraintSizes as NodeTypeVolumeConstraintSizes, type index_gen$b_NodeTypeVolumeType as NodeTypeVolumeType, type index_gen$b_Permission as Permission, type index_gen$b_PrepareInstanceLogsRequest as PrepareInstanceLogsRequest, type index_gen$b_PrepareInstanceLogsResponse as PrepareInstanceLogsResponse, type index_gen$b_Privilege as Privilege, type index_gen$b_PromoteReadReplicaRequest as PromoteReadReplicaRequest, type index_gen$b_PurgeInstanceLogsRequest as PurgeInstanceLogsRequest, index_gen$b_READ_REPLICA_TRANSIENT_STATUSES as READ_REPLICA_TRANSIENT_STATUSES, type index_gen$b_ReadReplica as ReadReplica, type index_gen$b_ReadReplicaEndpointSpec as ReadReplicaEndpointSpec, type index_gen$b_ReadReplicaEndpointSpecDirectAccess as ReadReplicaEndpointSpecDirectAccess, type index_gen$b_ReadReplicaEndpointSpecPrivateNetwork as ReadReplicaEndpointSpecPrivateNetwork, type index_gen$b_ReadReplicaEndpointSpecPrivateNetworkIpamConfig as ReadReplicaEndpointSpecPrivateNetworkIpamConfig, type index_gen$b_ReadReplicaStatus as ReadReplicaStatus, type index_gen$b_RenewInstanceCertificateRequest as RenewInstanceCertificateRequest, type index_gen$b_ResetReadReplicaRequest as ResetReadReplicaRequest, type index_gen$b_RestartInstanceRequest as RestartInstanceRequest, type index_gen$b_RestoreDatabaseBackupRequest as RestoreDatabaseBackupRequest, index_gen$b_SNAPSHOT_TRANSIENT_STATUSES as SNAPSHOT_TRANSIENT_STATUSES, type index_gen$b_SetInstanceACLRulesRequest as SetInstanceACLRulesRequest, type index_gen$b_SetInstanceACLRulesResponse as SetInstanceACLRulesResponse, type index_gen$b_SetInstanceSettingsRequest as SetInstanceSettingsRequest, type index_gen$b_SetInstanceSettingsResponse as SetInstanceSettingsResponse, type index_gen$b_SetPrivilegeRequest as SetPrivilegeRequest, type index_gen$b_Snapshot as Snapshot, type index_gen$b_SnapshotStatus as SnapshotStatus, type index_gen$b_SnapshotVolumeType as SnapshotVolumeType, type index_gen$b_StorageClass as StorageClass, type index_gen$b_UpdateDatabaseBackupRequest as UpdateDatabaseBackupRequest, type UpdateInstanceRequest$1 as UpdateInstanceRequest, type index_gen$b_UpdateSnapshotRequest as UpdateSnapshotRequest, type index_gen$b_UpdateUserRequest as UpdateUserRequest, type index_gen$b_UpgradableVersion as UpgradableVersion, type index_gen$b_UpgradeInstanceRequest as UpgradeInstanceRequest, type index_gen$b_UpgradeInstanceRequestMajorUpgradeWorkflow as UpgradeInstanceRequestMajorUpgradeWorkflow, type index_gen$b_User as User, validationRules_gen$2 as ValidationRules, type index_gen$b_Volume as Volume, type index_gen$b_VolumeType as VolumeType };
}

declare namespace index$9 {
  export { index_gen$b as v1 };
}

type AvailableClusterSettingPropertyType = 'UNKNOWN' | 'BOOLEAN' | 'INT' | 'STRING';
type ClusterStatus = 'unknown' | 'ready' | 'provisioning' | 'configuring' | 'deleting' | 'error' | 'autohealing' | 'locked' | 'suspended' | 'initializing';
type ListClustersRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type NodeTypeStock = 'unknown' | 'low_stock' | 'out_of_stock' | 'available';
type PrivateNetworkProvisioningMode = 'static' | 'ipam';
interface EndpointSpecPrivateNetworkSpecIpamConfig {
}
interface PrivateNetwork$2 {
    /** UUID of the Private Network. */
    id: string;
    /** List of IPv4 CIDR notation addresses of the endpoint. */
    serviceIps: string[];
    /** Zone of the Private Network. */
    zone: Zone;
    /** How your endpoint ips are provisioned. */
    provisioningMode: PrivateNetworkProvisioningMode;
}
interface PublicNetwork {
}
interface EndpointSpecPrivateNetworkSpec {
    /** UUID of the Private Network to connect to the Database Instance. */
    id: string;
    /**
     * Endpoint IPv4 address with a CIDR notation. You must provide at least one
     * IPv4 per node.
     */
    serviceIps: string[];
    /**
     * Automated configuration of your Private Network endpoint with Scaleway IPAM
     * service.
     */
    ipamConfig?: EndpointSpecPrivateNetworkSpecIpamConfig;
}
interface EndpointSpecPublicNetworkSpec {
}
interface AvailableClusterSetting {
    /** Name of the setting. */
    name: string;
    /** Default value of the setting. */
    defaultValue?: string;
    /** Type of setting. */
    type: AvailableClusterSettingPropertyType;
    /** Description of the setting. */
    description: string;
    /** Optional maximum value of the setting. */
    maxValue?: number;
    /** Optional minimum value of the setting. */
    minValue?: number;
    /** Optional validation rule of the setting. */
    regex?: string;
    /** Defines whether or not the setting is deprecated. */
    deprecated: boolean;
}
interface ACLRule {
    /** ID of the rule. */
    id: string;
    /** IPv4 network address of the rule. */
    ipCidr?: string;
    /** Description of the rule. */
    description?: string;
}
interface ClusterSetting {
    /** Value of the setting. */
    value: string;
    /** Name of the setting. */
    name: string;
}
interface Endpoint {
    /** TCP port of the endpoint. */
    port: number;
    /**
     * Private Network details.
     *
     * One-of ('details'): at most one of 'privateNetwork', 'publicNetwork' could
     * be set.
     */
    privateNetwork?: PrivateNetwork$2;
    /**
     * Public network details.
     *
     * One-of ('details'): at most one of 'privateNetwork', 'publicNetwork' could
     * be set.
     */
    publicNetwork?: PublicNetwork;
    /** List of IPv4 addresses of the endpoint. */
    ips: string[];
    /** UUID of the endpoint. */
    id: string;
}
interface ACLRuleSpec {
    /** IPv4 network address of the rule. */
    ipCidr: string;
    /** Description of the rule. */
    description: string;
}
interface EndpointSpec {
    /**
     * Private Network specification details.
     *
     * One-of ('endpointType'): at most one of 'privateNetwork', 'publicNetwork'
     * could be set.
     */
    privateNetwork?: EndpointSpecPrivateNetworkSpec;
    /**
     * Public network specification details.
     *
     * One-of ('endpointType'): at most one of 'privateNetwork', 'publicNetwork'
     * could be set.
     */
    publicNetwork?: EndpointSpecPublicNetworkSpec;
}
interface ClusterVersion {
    /** Redis™ engine version. */
    version: string;
    /** Date of End of Life. */
    endOfLifeAt?: Date;
    /** Cluster settings available to be updated. */
    availableSettings: AvailableClusterSetting[];
    /** Redis™ logo url. */
    logoUrl: string;
    /** Zone of the Redis™ Database Instance. */
    zone: Zone;
}
interface Cluster {
    /** UUID of the Database Instance. */
    id: string;
    /** Name of the Database Instance. */
    name: string;
    /** Project ID the Database Instance belongs to. */
    projectId: string;
    /** Status of the Database Instance. */
    status: ClusterStatus;
    /** Redis™ engine version of the Database Instance. */
    version: string;
    /** List of Database Instance endpoints. */
    endpoints: Endpoint[];
    /** List of tags applied to the Database Instance. */
    tags: string[];
    /** Node type of the Database Instance. */
    nodeType: string;
    /** Creation date (Format ISO 8601). */
    createdAt?: Date;
    /** Update date (Format ISO 8601). */
    updatedAt?: Date;
    /** Defines whether or not TLS is enabled. */
    tlsEnabled: boolean;
    /** List of Database Instance settings. */
    clusterSettings: ClusterSetting[];
    /** List of ACL rules. */
    aclRules: ACLRule[];
    /** Number of nodes of the Database Instance cluster. */
    clusterSize: number;
    /** Zone of the Database Instance. */
    zone: Zone;
    /** Name of the user associated to the cluster. */
    userName: string;
    /** List of engine versions the Database Instance can upgrade to. */
    upgradableVersions: string[];
}
interface NodeType {
    /** Node type name. */
    name: string;
    /** Current stock status of the node type. */
    stockStatus: NodeTypeStock;
    /** Current specifications of the offer. */
    description: string;
    /** Number of virtual CPUs. */
    vcpus: number;
    /** Quantity of RAM. */
    memory: number;
    /** Defines whether node type is currently disabled or not. */
    disabled: boolean;
    /** Defines whether node type is currently in beta. */
    beta: boolean;
    /** Zone of the node type. */
    zone: Zone;
}
type AddAclRulesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance you want to add ACL rules to. */
    clusterId: string;
    /** ACLs rules to add to the cluster. */
    aclRules: ACLRuleSpec[];
};
interface AddAclRulesResponse {
    /** ACL Rules enabled for the Database Instance. */
    aclRules: ACLRule[];
    /** Total count of ACL rules of the Database Instance. */
    totalCount: number;
}
type AddClusterSettingsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance you want to add settings to. */
    clusterId: string;
    /** Settings to add to the cluster. */
    settings: ClusterSetting[];
};
type AddEndpointsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance you want to add endpoints to. */
    clusterId: string;
    /** Endpoints to add to the Database Instance. */
    endpoints: EndpointSpec[];
};
interface AddEndpointsResponse {
    /** Endpoints defined on the Database Instance. */
    endpoints: Endpoint[];
    /** Total count of endpoints of the Database Instance. */
    totalCount: number;
}
interface ClusterMetricsResponse {
    /** Time series of metrics of a given cluster. */
    timeseries: TimeSeries[];
}
interface ClusterSettingsResponse {
    /** Settings configured for a given Database Instance. */
    settings: ClusterSetting[];
}
type CreateClusterRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Project ID in which to create the Database Instance. */
    projectId?: string;
    /** Name of the Database Instance. */
    name?: string;
    /** Redis™ engine version of the Database Instance. */
    version: string;
    /** Tags to apply to the Database Instance. */
    tags?: string[];
    /** Type of node to use for the Database Instance. */
    nodeType: string;
    /** Name of the user created upon Database Instance creation. */
    userName: string;
    /** Password of the user. */
    password: string;
    /** Number of nodes in the Redis™ cluster. */
    clusterSize?: number;
    /** List of ACLRuleSpec used to secure your publicly exposed cluster. */
    aclRules?: ACLRuleSpec[];
    /**
     * Zero or multiple EndpointSpec used to expose your cluster publicly and
     * inside private networks. If no EndpoindSpec is given the cluster will be
     * publicly exposed by default.
     */
    endpoints?: EndpointSpec[];
    /** Defines whether or not TLS is enabled. */
    tlsEnabled: boolean;
    /** List of advanced settings to be set upon Database Instance initialization. */
    clusterSettings?: ClusterSetting[];
};
type DeleteAclRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the ACL rule you want to delete. */
    aclId: string;
};
type DeleteClusterRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance to delete. */
    clusterId: string;
};
type DeleteClusterSettingRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance where the settings must be set. */
    clusterId: string;
    /** Setting name to delete. */
    settingName: string;
};
type DeleteEndpointRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the endpoint you want to delete. */
    endpointId: string;
};
type GetAclRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the ACL rule you want to get. */
    aclId: string;
};
type GetClusterCertificateRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the cluster. */
    clusterId: string;
};
type GetClusterMetricsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the cluster. */
    clusterId: string;
    /** Start date. */
    startAt?: Date;
    /** End date. */
    endAt?: Date;
    /** Name of the metric to gather. */
    metricName?: string;
};
type GetClusterRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the cluster. */
    clusterId: string;
};
type GetEndpointRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the endpoint you want to get. */
    endpointId: string;
};
type ListClusterVersionsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Defines whether or not to include disabled Redis™ engine versions. */
    includeDisabled: boolean;
    /** Defines whether or not to include beta Redis™ engine versions. */
    includeBeta: boolean;
    /** Defines whether or not to include deprecated Redis™ engine versions. */
    includeDeprecated: boolean;
    /** List Redis™ engine versions that match a given name pattern. */
    version?: string;
    page?: number;
    pageSize?: number;
};
interface ListClusterVersionsResponse {
    /** List of available Redis™ engine versions. */
    versions: ClusterVersion[];
    /** Total count of available Redis™ engine versions. */
    totalCount: number;
}
type ListClustersRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Filter by Database Instance tags. */
    tags?: string[];
    /** Filter by Database Instance names. */
    name?: string;
    /** Criteria to use when ordering the list. */
    orderBy?: ListClustersRequestOrderBy;
    /** Filter by Project ID. */
    projectId?: string;
    /** Filter by Organization ID. */
    organizationId?: string;
    /** Filter by Redis™ engine version. */
    version?: string;
    page?: number;
    pageSize?: number;
};
interface ListClustersResponse {
    /** List all Database Instances. */
    clusters: Cluster[];
    /** Total count of Database Instances. */
    totalCount: number;
}
type ListNodeTypesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Defines whether or not to include disabled types. */
    includeDisabledTypes: boolean;
    page?: number;
    pageSize?: number;
};
interface ListNodeTypesResponse {
    /** Types of node. */
    nodeTypes: NodeType[];
    /** Total count of node types available. */
    totalCount: number;
}
type MigrateClusterRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance to update. */
    clusterId: string;
    /**
     * Redis™ engine version of the Database Instance.
     *
     * One-of ('action'): at most one of 'version', 'nodeType', 'clusterSize'
     * could be set.
     */
    version?: string;
    /**
     * Type of node to use for the Database Instance.
     *
     * One-of ('action'): at most one of 'version', 'nodeType', 'clusterSize'
     * could be set.
     */
    nodeType?: string;
    /**
     * Number of nodes for the Database Instance.
     *
     * One-of ('action'): at most one of 'version', 'nodeType', 'clusterSize'
     * could be set.
     */
    clusterSize?: number;
};
type RenewClusterCertificateRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the cluster. */
    clusterId: string;
};
type SetAclRulesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance where the ACL rules have to be set. */
    clusterId: string;
    /** ACLs rules to define for the cluster. */
    aclRules: ACLRuleSpec[];
};
interface SetAclRulesResponse {
    /** ACL Rules enabled for the Database Instance. */
    aclRules: ACLRule[];
}
type SetClusterSettingsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance where the settings must be set. */
    clusterId: string;
    /** Settings to define for the Database Instance. */
    settings: ClusterSetting[];
};
type SetEndpointsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance where the endpoints have to be set. */
    clusterId: string;
    /** Endpoints to define for the Database Instance. */
    endpoints: EndpointSpec[];
};
interface SetEndpointsResponse {
    /** Endpoints defined on the Database Instance. */
    endpoints: Endpoint[];
}
type UpdateClusterRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the Database Instance to update. */
    clusterId: string;
    /** Name of the Database Instance. */
    name?: string;
    /** Database Instance tags. */
    tags?: string[];
    /** Name of the Database Instance user. */
    userName?: string;
    /** Password of the Database Instance user. */
    password?: string;
};
type UpdateEndpointRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** UUID of the endpoint you want to get. */
    endpointId: string;
    /**
     * Private Network details.
     *
     * One-of ('endpointType'): at most one of 'privateNetwork', 'publicNetwork'
     * could be set.
     */
    privateNetwork?: EndpointSpecPrivateNetworkSpec;
    /**
     * Public network details.
     *
     * One-of ('endpointType'): at most one of 'privateNetwork', 'publicNetwork'
     * could be set.
     */
    publicNetwork?: EndpointSpecPublicNetworkSpec;
};

/** Managed Database for Redis™ API. */
declare class API$9 extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    /**
     * Create a Redis™ Database Instance. Create a new Redis™ Database Instance
     * (Redis™ cluster). You must set the `zone`, `project_id`, `version`,
     * `node_type`, `user_name` and `password` parameters. Optionally you can
     * define `acl_rules`, `endpoints`, `tls_enabled` and `cluster_settings`.
     *
     * @param request - The request {@link CreateClusterRequest}
     * @returns A Promise of Cluster
     */
    createCluster: (request: Readonly<CreateClusterRequest>) => Promise<Cluster>;
    /**
     * Update a Redis™ Database Instance. Update the parameters of a Redis™
     * Database Instance (Redis™ cluster), including `name`, `tags`, `user_name`
     * and `password`.
     *
     * @param request - The request {@link UpdateClusterRequest}
     * @returns A Promise of Cluster
     */
    updateCluster: (request: Readonly<UpdateClusterRequest>) => Promise<Cluster>;
    /**
     * Get a Redis™ Database Instance. Retrieve information about a Redis™
     * Database Instance (Redis™ cluster). Specify the `cluster_id` and `region`
     * in your request to get information such as `id`, `status`, `version`,
     * `tls_enabled`, `cluster_settings`, `upgradable_versions` and `endpoints`
     * about your cluster in the response.
     *
     * @param request - The request {@link GetClusterRequest}
     * @returns A Promise of Cluster
     */
    getCluster: (request: Readonly<GetClusterRequest>) => Promise<Cluster>;
    /**
     * Waits for {@link Cluster} to be in a final state.
     *
     * @param request - The request {@link GetClusterRequest}
     * @param options - The waiting options
     * @returns A Promise of Cluster
     */
    waitForCluster: (request: Readonly<GetClusterRequest>, options?: Readonly<WaitForOptions<Cluster>>) => Promise<Cluster>;
    protected pageOfListClusters: (request?: Readonly<ListClustersRequest>) => Promise<ListClustersResponse>;
    /**
     * List Redis™ Database Instances. List all Redis™ Database Instances (Redis™
     * cluster) in the specified zone. By default, the Database Instances returned
     * in the list are ordered by creation date in ascending order, though this
     * can be modified via the order_by field. You can define additional
     * parameters for your query, such as `tags`, `name`, `organization_id` and
     * `version`.
     *
     * @param request - The request {@link ListClustersRequest}
     * @returns A Promise of ListClustersResponse
     */
    listClusters: (request?: Readonly<ListClustersRequest>) => Promise<ListClustersResponse> & {
        all: () => Promise<Cluster[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Cluster[], void, void>;
    };
    /**
     * Scale up a Redis™ Database Instance. Upgrade your standalone Redis™
     * Database Instance node, either by upgrading to a bigger node type (vertical
     * scaling) or by adding more nodes to your Database Instance to increase your
     * number of endpoints and distribute cache (horizontal scaling). Note that
     * scaling horizontally your Redis™ Database Instance will not renew its TLS
     * certificate. In order to refresh the TLS certificate, you must use the
     * Renew TLS certificate endpoint.
     *
     * @param request - The request {@link MigrateClusterRequest}
     * @returns A Promise of Cluster
     */
    migrateCluster: (request: Readonly<MigrateClusterRequest>) => Promise<Cluster>;
    /**
     * Delete a Redis™ Database Instance. Delete a Redis™ Database Instance
     * (Redis™ cluster), specified by the `region` and `cluster_id` parameters.
     * Deleting a Database Instance is permanent, and cannot be undone. Note that
     * upon deletion all your data will be lost.
     *
     * @param request - The request {@link DeleteClusterRequest}
     * @returns A Promise of Cluster
     */
    deleteCluster: (request: Readonly<DeleteClusterRequest>) => Promise<Cluster>;
    /**
     * Get metrics of a Redis™ Database Instance. Retrieve the metrics of a Redis™
     * Database Instance (Redis™ cluster). You can define the period from which to
     * retrieve metrics by specifying the `start_date` and `end_date`.
     *
     * @param request - The request {@link GetClusterMetricsRequest}
     * @returns A Promise of ClusterMetricsResponse
     */
    getClusterMetrics: (request: Readonly<GetClusterMetricsRequest>) => Promise<ClusterMetricsResponse>;
    protected pageOfListNodeTypes: (request: Readonly<ListNodeTypesRequest>) => Promise<ListNodeTypesResponse>;
    /**
     * List available node types. List all available node types. By default, the
     * node types returned in the list are ordered by creation date in ascending
     * order, though this can be modified via the `order_by` field.
     *
     * @param request - The request {@link ListNodeTypesRequest}
     * @returns A Promise of ListNodeTypesResponse
     */
    listNodeTypes: (request: Readonly<ListNodeTypesRequest>) => Promise<ListNodeTypesResponse> & {
        all: () => Promise<NodeType[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<NodeType[], void, void>;
    };
    protected pageOfListClusterVersions: (request: Readonly<ListClusterVersionsRequest>) => Promise<ListClusterVersionsResponse>;
    /**
     * List available Redis™ versions. List the Redis™ database engine versions
     * available. You can define additional parameters for your query, such as
     * `include_disabled`, `include_beta`, `include_deprecated` and `version`.
     *
     * @param request - The request {@link ListClusterVersionsRequest}
     * @returns A Promise of ListClusterVersionsResponse
     */
    listClusterVersions: (request: Readonly<ListClusterVersionsRequest>) => Promise<ListClusterVersionsResponse> & {
        all: () => Promise<ClusterVersion[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ClusterVersion[], void, void>;
    };
    /**
     * Get the TLS certificate of a cluster. Retrieve information about the TLS
     * certificate of a Redis™ Database Instance (Redis™ cluster). Details like
     * name and content are returned in the response.
     *
     * @param request - The request {@link GetClusterCertificateRequest}
     * @returns A Promise of Blob
     */
    getClusterCertificate: (request: Readonly<GetClusterCertificateRequest>) => Promise<Blob>;
    /**
     * Renew the TLS certificate of a cluster. Renew a TLS certificate for a
     * Redis™ Database Instance (Redis™ cluster). Renewing a certificate means
     * that you will not be able to connect to your Database Instance using the
     * previous certificate. You will also need to download and update the new
     * certificate for all database clients.
     *
     * @param request - The request {@link RenewClusterCertificateRequest}
     * @returns A Promise of Cluster
     */
    renewClusterCertificate: (request: Readonly<RenewClusterCertificateRequest>) => Promise<Cluster>;
    /**
     * Add advanced settings. Add an advanced setting to a Redis™ Database
     * Instance (Redis™ cluster). You must set the `name` and the `value` of each
     * setting.
     *
     * @param request - The request {@link AddClusterSettingsRequest}
     * @returns A Promise of ClusterSettingsResponse
     */
    addClusterSettings: (request: Readonly<AddClusterSettingsRequest>) => Promise<ClusterSettingsResponse>;
    /**
     * Delete advanced setting. Delete an advanced setting in a Redis™ Database
     * Instance (Redis™ cluster). You must specify the names of the settings you
     * want to delete in the request body.
     *
     * @param request - The request {@link DeleteClusterSettingRequest}
     * @returns A Promise of Cluster
     */
    deleteClusterSetting: (request: Readonly<DeleteClusterSettingRequest>) => Promise<Cluster>;
    /**
     * Set advanced settings. Update an advanced setting for a Redis™ Database
     * Instance (Redis™ cluster). Settings added upon database engine
     * initalization can only be defined once, and cannot, therefore, be updated.
     *
     * @param request - The request {@link SetClusterSettingsRequest}
     * @returns A Promise of ClusterSettingsResponse
     */
    setClusterSettings: (request: Readonly<SetClusterSettingsRequest>) => Promise<ClusterSettingsResponse>;
    /**
     * Set ACL rules for a cluster. Replace all the ACL rules of a Redis™ Database
     * Instance (Redis™ cluster).
     *
     * @param request - The request {@link SetAclRulesRequest}
     * @returns A Promise of SetAclRulesResponse
     */
    setAclRules: (request: Readonly<SetAclRulesRequest>) => Promise<SetAclRulesResponse>;
    /**
     * Add ACL rules for a cluster. Add an additional ACL rule to a Redis™
     * Database Instance (Redis™ cluster).
     *
     * @param request - The request {@link AddAclRulesRequest}
     * @returns A Promise of AddAclRulesResponse
     */
    addAclRules: (request: Readonly<AddAclRulesRequest>) => Promise<AddAclRulesResponse>;
    /**
     * Delete an ACL rule for a cluster. Delete an ACL rule of a Redis™ Database
     * Instance (Redis™ cluster). You must specify the `acl_id` of the rule you
     * want to delete in your request.
     *
     * @param request - The request {@link DeleteAclRuleRequest}
     * @returns A Promise of Cluster
     */
    deleteAclRule: (request: Readonly<DeleteAclRuleRequest>) => Promise<Cluster>;
    /**
     * Get an ACL rule. Retrieve information about an ACL rule of a Redis™
     * Database Instance (Redis™ cluster). You must specify the `acl_id` of the
     * rule in your request.
     *
     * @param request - The request {@link GetAclRuleRequest}
     * @returns A Promise of ACLRule
     */
    getAclRule: (request: Readonly<GetAclRuleRequest>) => Promise<ACLRule>;
    /**
     * Set endpoints for a cluster. Update an endpoint for a Redis™ Database
     * Instance (Redis™ cluster). You must specify the `cluster_id` and the
     * `endpoints` parameters in your request.
     *
     * @param request - The request {@link SetEndpointsRequest}
     * @returns A Promise of SetEndpointsResponse
     */
    setEndpoints: (request: Readonly<SetEndpointsRequest>) => Promise<SetEndpointsResponse>;
    /**
     * Add endpoints for a cluster. Add a new endpoint for a Redis™ Database
     * Instance (Redis™ cluster). You can add `private_network` or
     * `public_network` specifications to the body of the request.
     *
     * @param request - The request {@link AddEndpointsRequest}
     * @returns A Promise of AddEndpointsResponse
     */
    addEndpoints: (request: Readonly<AddEndpointsRequest>) => Promise<AddEndpointsResponse>;
    /**
     * Delete an endpoint for a cluster. Delete the endpoint of a Redis™ Database
     * Instance (Redis™ cluster). You must specify the `region` and `endpoint_id`
     * parameters of the endpoint you want to delete. Note that might need to
     * update any environment configurations that point to the deleted endpoint.
     *
     * @param request - The request {@link DeleteEndpointRequest}
     * @returns A Promise of Cluster
     */
    deleteEndpoint: (request: Readonly<DeleteEndpointRequest>) => Promise<Cluster>;
    /**
     * Get an endpoint. Retrieve information about a Redis™ Database Instance
     * (Redis™ cluster) endpoint. Full details about the endpoint, like `ips`,
     * `port`, `private_network` and `public_network` specifications are returned
     * in the response.
     *
     * @param request - The request {@link GetEndpointRequest}
     * @returns A Promise of Endpoint
     */
    getEndpoint: (request: Readonly<GetEndpointRequest>) => Promise<Endpoint>;
    /**
     * Update an endpoint. Update information about a Redis™ Database Instance
     * (Redis™ cluster) endpoint. Full details about the endpoint, like `ips`,
     * `port`, `private_network` and `public_network` specifications are returned
     * in the response.
     *
     * @param request - The request {@link UpdateEndpointRequest}
     * @returns A Promise of Endpoint
     */
    updateEndpoint: (request: Readonly<UpdateEndpointRequest>) => Promise<Endpoint>;
}

/** Lists transient statutes of the enum {@link ClusterStatus}. */
declare const CLUSTER_TRANSIENT_STATUSES: ClusterStatus[];

type index_gen$a_ACLRule = ACLRule;
type index_gen$a_ACLRuleSpec = ACLRuleSpec;
type index_gen$a_AddAclRulesRequest = AddAclRulesRequest;
type index_gen$a_AddAclRulesResponse = AddAclRulesResponse;
type index_gen$a_AddClusterSettingsRequest = AddClusterSettingsRequest;
type index_gen$a_AddEndpointsRequest = AddEndpointsRequest;
type index_gen$a_AddEndpointsResponse = AddEndpointsResponse;
type index_gen$a_AvailableClusterSetting = AvailableClusterSetting;
type index_gen$a_AvailableClusterSettingPropertyType = AvailableClusterSettingPropertyType;
declare const index_gen$a_CLUSTER_TRANSIENT_STATUSES: typeof CLUSTER_TRANSIENT_STATUSES;
type index_gen$a_Cluster = Cluster;
type index_gen$a_ClusterMetricsResponse = ClusterMetricsResponse;
type index_gen$a_ClusterSetting = ClusterSetting;
type index_gen$a_ClusterSettingsResponse = ClusterSettingsResponse;
type index_gen$a_ClusterStatus = ClusterStatus;
type index_gen$a_ClusterVersion = ClusterVersion;
type index_gen$a_CreateClusterRequest = CreateClusterRequest;
type index_gen$a_DeleteAclRuleRequest = DeleteAclRuleRequest;
type index_gen$a_DeleteClusterRequest = DeleteClusterRequest;
type index_gen$a_DeleteClusterSettingRequest = DeleteClusterSettingRequest;
type index_gen$a_DeleteEndpointRequest = DeleteEndpointRequest;
type index_gen$a_Endpoint = Endpoint;
type index_gen$a_EndpointSpec = EndpointSpec;
type index_gen$a_EndpointSpecPrivateNetworkSpec = EndpointSpecPrivateNetworkSpec;
type index_gen$a_EndpointSpecPrivateNetworkSpecIpamConfig = EndpointSpecPrivateNetworkSpecIpamConfig;
type index_gen$a_EndpointSpecPublicNetworkSpec = EndpointSpecPublicNetworkSpec;
type index_gen$a_GetAclRuleRequest = GetAclRuleRequest;
type index_gen$a_GetClusterCertificateRequest = GetClusterCertificateRequest;
type index_gen$a_GetClusterMetricsRequest = GetClusterMetricsRequest;
type index_gen$a_GetClusterRequest = GetClusterRequest;
type index_gen$a_GetEndpointRequest = GetEndpointRequest;
type index_gen$a_ListClusterVersionsRequest = ListClusterVersionsRequest;
type index_gen$a_ListClusterVersionsResponse = ListClusterVersionsResponse;
type index_gen$a_ListClustersRequest = ListClustersRequest;
type index_gen$a_ListClustersRequestOrderBy = ListClustersRequestOrderBy;
type index_gen$a_ListClustersResponse = ListClustersResponse;
type index_gen$a_ListNodeTypesRequest = ListNodeTypesRequest;
type index_gen$a_ListNodeTypesResponse = ListNodeTypesResponse;
type index_gen$a_MigrateClusterRequest = MigrateClusterRequest;
type index_gen$a_NodeType = NodeType;
type index_gen$a_NodeTypeStock = NodeTypeStock;
type index_gen$a_PrivateNetworkProvisioningMode = PrivateNetworkProvisioningMode;
type index_gen$a_PublicNetwork = PublicNetwork;
type index_gen$a_RenewClusterCertificateRequest = RenewClusterCertificateRequest;
type index_gen$a_SetAclRulesRequest = SetAclRulesRequest;
type index_gen$a_SetAclRulesResponse = SetAclRulesResponse;
type index_gen$a_SetClusterSettingsRequest = SetClusterSettingsRequest;
type index_gen$a_SetEndpointsRequest = SetEndpointsRequest;
type index_gen$a_SetEndpointsResponse = SetEndpointsResponse;
type index_gen$a_UpdateClusterRequest = UpdateClusterRequest;
type index_gen$a_UpdateEndpointRequest = UpdateEndpointRequest;
declare namespace index_gen$a {
  export { type index_gen$a_ACLRule as ACLRule, type index_gen$a_ACLRuleSpec as ACLRuleSpec, API$9 as API, type index_gen$a_AddAclRulesRequest as AddAclRulesRequest, type index_gen$a_AddAclRulesResponse as AddAclRulesResponse, type index_gen$a_AddClusterSettingsRequest as AddClusterSettingsRequest, type index_gen$a_AddEndpointsRequest as AddEndpointsRequest, type index_gen$a_AddEndpointsResponse as AddEndpointsResponse, type index_gen$a_AvailableClusterSetting as AvailableClusterSetting, type index_gen$a_AvailableClusterSettingPropertyType as AvailableClusterSettingPropertyType, index_gen$a_CLUSTER_TRANSIENT_STATUSES as CLUSTER_TRANSIENT_STATUSES, type index_gen$a_Cluster as Cluster, type index_gen$a_ClusterMetricsResponse as ClusterMetricsResponse, type index_gen$a_ClusterSetting as ClusterSetting, type index_gen$a_ClusterSettingsResponse as ClusterSettingsResponse, type index_gen$a_ClusterStatus as ClusterStatus, type index_gen$a_ClusterVersion as ClusterVersion, type index_gen$a_CreateClusterRequest as CreateClusterRequest, type index_gen$a_DeleteAclRuleRequest as DeleteAclRuleRequest, type index_gen$a_DeleteClusterRequest as DeleteClusterRequest, type index_gen$a_DeleteClusterSettingRequest as DeleteClusterSettingRequest, type index_gen$a_DeleteEndpointRequest as DeleteEndpointRequest, type index_gen$a_Endpoint as Endpoint, type index_gen$a_EndpointSpec as EndpointSpec, type index_gen$a_EndpointSpecPrivateNetworkSpec as EndpointSpecPrivateNetworkSpec, type index_gen$a_EndpointSpecPrivateNetworkSpecIpamConfig as EndpointSpecPrivateNetworkSpecIpamConfig, type index_gen$a_EndpointSpecPublicNetworkSpec as EndpointSpecPublicNetworkSpec, type index_gen$a_GetAclRuleRequest as GetAclRuleRequest, type index_gen$a_GetClusterCertificateRequest as GetClusterCertificateRequest, type index_gen$a_GetClusterMetricsRequest as GetClusterMetricsRequest, type index_gen$a_GetClusterRequest as GetClusterRequest, type index_gen$a_GetEndpointRequest as GetEndpointRequest, type index_gen$a_ListClusterVersionsRequest as ListClusterVersionsRequest, type index_gen$a_ListClusterVersionsResponse as ListClusterVersionsResponse, type index_gen$a_ListClustersRequest as ListClustersRequest, type index_gen$a_ListClustersRequestOrderBy as ListClustersRequestOrderBy, type index_gen$a_ListClustersResponse as ListClustersResponse, type index_gen$a_ListNodeTypesRequest as ListNodeTypesRequest, type index_gen$a_ListNodeTypesResponse as ListNodeTypesResponse, type index_gen$a_MigrateClusterRequest as MigrateClusterRequest, type index_gen$a_NodeType as NodeType, type index_gen$a_NodeTypeStock as NodeTypeStock, type PrivateNetwork$2 as PrivateNetwork, type index_gen$a_PrivateNetworkProvisioningMode as PrivateNetworkProvisioningMode, type index_gen$a_PublicNetwork as PublicNetwork, type index_gen$a_RenewClusterCertificateRequest as RenewClusterCertificateRequest, type index_gen$a_SetAclRulesRequest as SetAclRulesRequest, type index_gen$a_SetAclRulesResponse as SetAclRulesResponse, type index_gen$a_SetClusterSettingsRequest as SetClusterSettingsRequest, type index_gen$a_SetEndpointsRequest as SetEndpointsRequest, type index_gen$a_SetEndpointsResponse as SetEndpointsResponse, type index_gen$a_UpdateClusterRequest as UpdateClusterRequest, type index_gen$a_UpdateEndpointRequest as UpdateEndpointRequest };
}

declare namespace index$8 {
  export { index_gen$a as v1 };
}

type ImageStatus = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked';
type ImageVisibility = 'visibility_unknown' | 'inherit' | 'public' | 'private';
type ListImagesRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListNamespacesRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'description_asc' | 'description_desc' | 'name_asc' | 'name_desc';
type ListTagsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type NamespaceStatus = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked';
type TagStatus = 'unknown' | 'ready' | 'deleting' | 'error' | 'locked';
interface Image {
    /** UUID of the image. */
    id: string;
    /** Name of the image, it must be unique within the namespace. */
    name: string;
    /** UUID of the namespace the image belongs to. */
    namespaceId: string;
    /** Status of the image. */
    status: ImageStatus;
    /** Details of the image status. */
    statusMessage?: string;
    /**
     * Set to `public` to allow the image to be pulled without authentication.
     * Else, set to `private`. Set to `inherit` to keep the same visibility
     * configuration as the namespace.
     */
    visibility: ImageVisibility;
    /**
     * Image size in bytes, calculated from the size of image layers. One layer
     * used in two tags of the same image is counted once but one layer used in
     * two images is counted twice.
     */
    size: number;
    /** Date and time of image creation. */
    createdAt?: Date;
    /** Date and time of last update. */
    updatedAt?: Date;
    /** List of docker tags of the image. */
    tags: string[];
}
interface Namespace {
    /** UUID of the namespace. */
    id: string;
    /** Name of the namespace, unique in a region accross all organizations. */
    name: string;
    /** Description of the namespace. */
    description: string;
    /** Owner of the namespace. */
    organizationId: string;
    /** Project of the namespace. */
    projectId: string;
    /** Namespace status. */
    status: NamespaceStatus;
    /** Namespace status details. */
    statusMessage: string;
    /** Endpoint reachable by docker. */
    endpoint: string;
    /** Defines whether or not namespace is public. */
    isPublic: boolean;
    /**
     * Total size of the namespace, calculated as the sum of the size of all
     * images in the namespace.
     */
    size: number;
    /** Date and time of creation. */
    createdAt?: Date;
    /** Date and time of last update. */
    updatedAt?: Date;
    /** Number of images in the namespace. */
    imageCount: number;
    /** Region the namespace belongs to. */
    region: Region;
}
interface Tag {
    /** UUID of the tag. */
    id: string;
    /** Tag name, unique to an image. */
    name: string;
    /** Image ID the of the image the tag belongs to. */
    imageId: string;
    /** Tag status. */
    status: TagStatus;
    /**
     * Hash of the tag content. Several tags of a same image may have the same
     * digest.
     */
    digest: string;
    /** Date and time of creation. */
    createdAt?: Date;
    /** Date and time of last update. */
    updatedAt?: Date;
}
type CreateNamespaceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the namespace. */
    name?: string;
    /** Description of the namespace. */
    description: string;
    /**
     * @deprecated Namespace owner (deprecated).
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    /**
     * Project ID on which the namespace will be created.
     *
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
    /** Defines whether or not namespace is public. */
    isPublic: boolean;
};
type DeleteImageRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the image. */
    imageId: string;
};
type DeleteNamespaceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespace. */
    namespaceId: string;
};
type DeleteTagRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the tag. */
    tagId: string;
    /**
     * @deprecated If two tags share the same digest the deletion will fail unless
     *   this parameter is set to true (deprecated).
     */
    force?: boolean;
};
type GetImageRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the image. */
    imageId: string;
};
type GetNamespaceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the namespace. */
    namespaceId: string;
};
type GetTagRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the tag. */
    tagId: string;
};
type ListImagesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** A positive integer to choose the page to display. */
    page?: number;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * display.
     */
    pageSize?: number;
    /**
     * Criteria to use when ordering image listings. Possible values are
     * `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`,
     * `status_asc` and `status_desc`. The default value is `created_at_asc`.
     */
    orderBy?: ListImagesRequestOrderBy;
    /** Filter by the namespace ID. */
    namespaceId?: string;
    /** Filter by the image name (exact match). */
    name?: string;
    /** Filter by Organization ID. */
    organizationId?: string;
    /** Filter by Project ID. */
    projectId?: string;
};
interface ListImagesResponse {
    /** Paginated list of images that match the selected filters. */
    images: Image[];
    /** Total number of images that match the selected filters. */
    totalCount: number;
}
type ListNamespacesRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** A positive integer to choose the page to display. */
    page?: number;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * display.
     */
    pageSize?: number;
    /**
     * Criteria to use when ordering namespace listings. Possible values are
     * `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`,
     * `status_asc` and `status_desc`. The default value is `created_at_asc`.
     */
    orderBy?: ListNamespacesRequestOrderBy;
    /** Filter by Organization ID. */
    organizationId?: string;
    /** Filter by Project ID. */
    projectId?: string;
    /** Filter by the namespace name (exact match). */
    name?: string;
};
interface ListNamespacesResponse {
    /** Paginated list of namespaces that match the selected filters. */
    namespaces: Namespace[];
    /** Total number of namespaces that match the selected filters. */
    totalCount: number;
}
type ListTagsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the image. */
    imageId: string;
    /** A positive integer to choose the page to display. */
    page?: number;
    /**
     * A positive integer lower or equal to 100 to select the number of items to
     * display.
     */
    pageSize?: number;
    /**
     * Criteria to use when ordering tag listings. Possible values are
     * `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`,
     * `status_asc` and `status_desc`. The default value is `created_at_asc`.
     */
    orderBy?: ListTagsRequestOrderBy;
    /** Filter by the tag name (exact match). */
    name?: string;
};
interface ListTagsResponse$1 {
    /** Paginated list of tags that match the selected filters. */
    tags: Tag[];
    /** Total number of tags that match the selected filters. */
    totalCount: number;
}
type UpdateImageRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the image to update. */
    imageId: string;
    /**
     * Set to `public` to allow the image to be pulled without authentication.
     * Else, set to `private`. Set to `inherit` to keep the same visibility
     * configuration as the namespace.
     */
    visibility?: ImageVisibility;
};
type UpdateNamespaceRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the namespace to update. */
    namespaceId: string;
    /** Namespace description. */
    description?: string;
    /** Defines whether or not the namespace is public. */
    isPublic?: boolean;
};

/** Container Registry API. */
declare class API$8 extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListNamespaces: (request?: Readonly<ListNamespacesRequest>) => Promise<ListNamespacesResponse>;
    /**
     * List namespaces. List all namespaces in a specified region. By default, the
     * namespaces listed are ordered by creation date in ascending order. This can
     * be modified via the order_by field. You can also define additional
     * parameters for your query, such as the `instance_id` and `project_id`
     * parameters.
     *
     * @param request - The request {@link ListNamespacesRequest}
     * @returns A Promise of ListNamespacesResponse
     */
    listNamespaces: (request?: Readonly<ListNamespacesRequest>) => Promise<ListNamespacesResponse> & {
        all: () => Promise<Namespace[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Namespace[], void, void>;
    };
    /**
     * Get a namespace. Retrieve information about a given namespace, specified by
     * its `namespace_id` and region. Full details about the namespace, such as
     * `description`, `project_id`, `status`, `endpoint`, `is_public`, `size`, and
     * `image_count` are returned in the response.
     *
     * @param request - The request {@link GetNamespaceRequest}
     * @returns A Promise of Namespace
     */
    getNamespace: (request: Readonly<GetNamespaceRequest>) => Promise<Namespace>;
    /**
     * Waits for {@link Namespace} to be in a final state.
     *
     * @param request - The request {@link GetNamespaceRequest}
     * @param options - The waiting options
     * @returns A Promise of Namespace
     */
    waitForNamespace: (request: Readonly<GetNamespaceRequest>, options?: Readonly<WaitForOptions<Namespace>>) => Promise<Namespace>;
    /**
     * Create a namespace. Create a new Container Registry namespace. You must
     * specify the namespace name and region in which you want it to be created.
     * Optionally, you can specify the `project_id` and `is_public` in the request
     * payload.
     *
     * @param request - The request {@link CreateNamespaceRequest}
     * @returns A Promise of Namespace
     */
    createNamespace: (request: Readonly<CreateNamespaceRequest>) => Promise<Namespace>;
    /**
     * Update a namespace. Update the parameters of a given namespace, specified
     * by its `namespace_id` and `region`. You can update the `description` and
     * `is_public` parameters.
     *
     * @param request - The request {@link UpdateNamespaceRequest}
     * @returns A Promise of Namespace
     */
    updateNamespace: (request: Readonly<UpdateNamespaceRequest>) => Promise<Namespace>;
    /**
     * Delete a namespace. Delete a given namespace. You must specify, in the
     * endpoint, the `region` and `namespace_id` parameters of the namespace you
     * want to delete.
     *
     * @param request - The request {@link DeleteNamespaceRequest}
     * @returns A Promise of Namespace
     */
    deleteNamespace: (request: Readonly<DeleteNamespaceRequest>) => Promise<Namespace>;
    protected pageOfListImages: (request?: Readonly<ListImagesRequest>) => Promise<ListImagesResponse>;
    /**
     * List images. List all images in a specified region. By default, the images
     * listed are ordered by creation date in ascending order. This can be
     * modified via the order_by field. You can also define additional parameters
     * for your query, such as the `namespace_id` and `project_id` parameters.
     *
     * @param request - The request {@link ListImagesRequest}
     * @returns A Promise of ListImagesResponse
     */
    listImages: (request?: Readonly<ListImagesRequest>) => Promise<ListImagesResponse> & {
        all: () => Promise<Image[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Image[], void, void>;
    };
    /**
     * Get an image. Retrieve information about a given container image, specified
     * by its `image_id` and region. Full details about the image, such as `name`,
     * `namespace_id`, `status`, `visibility`, and `size` are returned in the
     * response.
     *
     * @param request - The request {@link GetImageRequest}
     * @returns A Promise of Image
     */
    getImage: (request: Readonly<GetImageRequest>) => Promise<Image>;
    /**
     * Waits for {@link Image} to be in a final state.
     *
     * @param request - The request {@link GetImageRequest}
     * @param options - The waiting options
     * @returns A Promise of Image
     */
    waitForImage: (request: Readonly<GetImageRequest>, options?: Readonly<WaitForOptions<Image>>) => Promise<Image>;
    /**
     * Update an image. Update the parameters of a given image, specified by its
     * `image_id` and `region`. You can update the `visibility` parameter.
     *
     * @param request - The request {@link UpdateImageRequest}
     * @returns A Promise of Image
     */
    updateImage: (request: Readonly<UpdateImageRequest>) => Promise<Image>;
    /**
     * Delete an image. Delete a given image. You must specify, in the endpoint,
     * the `region` and `image_id` parameters of the image you want to delete.
     *
     * @param request - The request {@link DeleteImageRequest}
     * @returns A Promise of Image
     */
    deleteImage: (request: Readonly<DeleteImageRequest>) => Promise<Image>;
    protected pageOfListTags: (request: Readonly<ListTagsRequest$1>) => Promise<ListTagsResponse$1>;
    /**
     * List tags. List all tags for a given image, specified by region. By
     * default, the tags listed are ordered by creation date in ascending order.
     * This can be modified via the order_by field. You can also define additional
     * parameters for your query, such as the `name`.
     *
     * @param request - The request {@link ListTagsRequest}
     * @returns A Promise of ListTagsResponse
     */
    listTags: (request: Readonly<ListTagsRequest$1>) => Promise<ListTagsResponse$1> & {
        all: () => Promise<Tag[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Tag[], void, void>;
    };
    /**
     * Get a tag. Retrieve information about a given image tag, specified by its
     * `tag_id` and region. Full details about the tag, such as `name`,
     * `image_id`, `status`, and `digest` are returned in the response.
     *
     * @param request - The request {@link GetTagRequest}
     * @returns A Promise of Tag
     */
    getTag: (request: Readonly<GetTagRequest>) => Promise<Tag>;
    /**
     * Waits for {@link Tag} to be in a final state.
     *
     * @param request - The request {@link GetTagRequest}
     * @param options - The waiting options
     * @returns A Promise of Tag
     */
    waitForTag: (request: Readonly<GetTagRequest>, options?: Readonly<WaitForOptions<Tag>>) => Promise<Tag>;
    /**
     * Delete a tag. Delete a given image tag. You must specify, in the endpoint,
     * the `region` and `tag_id` parameters of the tag you want to delete.
     *
     * @param request - The request {@link DeleteTagRequest}
     * @returns A Promise of Tag
     */
    deleteTag: (request: Readonly<DeleteTagRequest>) => Promise<Tag>;
}

/** Lists transient statutes of the enum {@link ImageStatus}. */
declare const IMAGE_TRANSIENT_STATUSES: ImageStatus[];
/** Lists transient statutes of the enum {@link NamespaceStatus}. */
declare const NAMESPACE_TRANSIENT_STATUSES: NamespaceStatus[];
/** Lists transient statutes of the enum {@link TagStatus}. */
declare const TAG_TRANSIENT_STATUSES: TagStatus[];

type index_gen$9_CreateNamespaceRequest = CreateNamespaceRequest;
type index_gen$9_DeleteImageRequest = DeleteImageRequest;
type index_gen$9_DeleteNamespaceRequest = DeleteNamespaceRequest;
type index_gen$9_DeleteTagRequest = DeleteTagRequest;
type index_gen$9_GetImageRequest = GetImageRequest;
type index_gen$9_GetNamespaceRequest = GetNamespaceRequest;
type index_gen$9_GetTagRequest = GetTagRequest;
declare const index_gen$9_IMAGE_TRANSIENT_STATUSES: typeof IMAGE_TRANSIENT_STATUSES;
type index_gen$9_Image = Image;
type index_gen$9_ImageStatus = ImageStatus;
type index_gen$9_ImageVisibility = ImageVisibility;
type index_gen$9_ListImagesRequest = ListImagesRequest;
type index_gen$9_ListImagesRequestOrderBy = ListImagesRequestOrderBy;
type index_gen$9_ListImagesResponse = ListImagesResponse;
type index_gen$9_ListNamespacesRequest = ListNamespacesRequest;
type index_gen$9_ListNamespacesRequestOrderBy = ListNamespacesRequestOrderBy;
type index_gen$9_ListNamespacesResponse = ListNamespacesResponse;
type index_gen$9_ListTagsRequestOrderBy = ListTagsRequestOrderBy;
declare const index_gen$9_NAMESPACE_TRANSIENT_STATUSES: typeof NAMESPACE_TRANSIENT_STATUSES;
type index_gen$9_Namespace = Namespace;
type index_gen$9_NamespaceStatus = NamespaceStatus;
declare const index_gen$9_TAG_TRANSIENT_STATUSES: typeof TAG_TRANSIENT_STATUSES;
type index_gen$9_Tag = Tag;
type index_gen$9_TagStatus = TagStatus;
type index_gen$9_UpdateImageRequest = UpdateImageRequest;
type index_gen$9_UpdateNamespaceRequest = UpdateNamespaceRequest;
declare namespace index_gen$9 {
  export { API$8 as API, type index_gen$9_CreateNamespaceRequest as CreateNamespaceRequest, type index_gen$9_DeleteImageRequest as DeleteImageRequest, type index_gen$9_DeleteNamespaceRequest as DeleteNamespaceRequest, type index_gen$9_DeleteTagRequest as DeleteTagRequest, type index_gen$9_GetImageRequest as GetImageRequest, type index_gen$9_GetNamespaceRequest as GetNamespaceRequest, type index_gen$9_GetTagRequest as GetTagRequest, index_gen$9_IMAGE_TRANSIENT_STATUSES as IMAGE_TRANSIENT_STATUSES, type index_gen$9_Image as Image, type index_gen$9_ImageStatus as ImageStatus, type index_gen$9_ImageVisibility as ImageVisibility, type index_gen$9_ListImagesRequest as ListImagesRequest, type index_gen$9_ListImagesRequestOrderBy as ListImagesRequestOrderBy, type index_gen$9_ListImagesResponse as ListImagesResponse, type index_gen$9_ListNamespacesRequest as ListNamespacesRequest, type index_gen$9_ListNamespacesRequestOrderBy as ListNamespacesRequestOrderBy, type index_gen$9_ListNamespacesResponse as ListNamespacesResponse, type ListTagsRequest$1 as ListTagsRequest, type index_gen$9_ListTagsRequestOrderBy as ListTagsRequestOrderBy, type ListTagsResponse$1 as ListTagsResponse, index_gen$9_NAMESPACE_TRANSIENT_STATUSES as NAMESPACE_TRANSIENT_STATUSES, type index_gen$9_Namespace as Namespace, type index_gen$9_NamespaceStatus as NamespaceStatus, index_gen$9_TAG_TRANSIENT_STATUSES as TAG_TRANSIENT_STATUSES, type index_gen$9_Tag as Tag, type index_gen$9_TagStatus as TagStatus, type index_gen$9_UpdateImageRequest as UpdateImageRequest, type index_gen$9_UpdateNamespaceRequest as UpdateNamespaceRequest };
}

declare namespace index$7 {
  export { index_gen$9 as v1 };
}

type EphemeralPolicyAction = 'unknown_action' | 'delete' | 'disable';
type ListFoldersRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListSecretsRequestOrderBy = 'name_asc' | 'name_desc' | 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc';
type Product = 'unknown' | 'edge_services';
type SecretStatus = 'ready' | 'locked';
type SecretType = 'unknown_secret_type' | 'opaque' | 'certificate' | 'key_value';
type SecretVersionStatus = 'unknown' | 'enabled' | 'disabled' | 'destroyed';
interface EphemeralProperties {
    /** (Optional.) If not specified, the version does not have an expiration date. */
    expiresAt?: Date;
    /**
     * (Optional.) If not specified, the version can be accessed an unlimited
     * amount of times.
     */
    expiresOnceAccessed?: boolean;
    /** See `EphemeralPolicy.Action` enum for a description of values. */
    action: EphemeralPolicyAction;
}
interface EphemeralPolicy {
    /**
     * Time frame, from one second and up to one year, during which the secret's
     * versions are valid.
     */
    timeToLive?: string;
    /** Returns `true` if the version expires after a single user access. */
    expiresOnceAccessed?: boolean;
    /** See the `EphemeralPolicy.Action` enum for a description of values. */
    action: EphemeralPolicyAction;
}
interface PasswordGenerationParams {
    /** Length of the password to generate (between 1 and 1024). */
    length: number;
    /** Do not include lower case letters by default in the alphabet. */
    noLowercaseLetters: boolean;
    /** Do not include upper case letters by default in the alphabet. */
    noUppercaseLetters: boolean;
    /** Do not include digits by default in the alphabet. */
    noDigits: boolean;
    /** Additional ascii characters to be included in the alphabet. */
    additionalChars: string;
}
interface Folder {
    /** ID of the folder. */
    id: string;
    /** ID of the Project containing the folder. */
    projectId: string;
    /** Name of the folder. */
    name: string;
    /** Location of the folder in the directory structure. */
    path: string;
    /** Date and time of the folder's creation. */
    createdAt?: Date;
    /** Region of the folder. */
    region: Region;
}
interface SecretVersion {
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1.
     */
    revision: number;
    /** ID of the secret. */
    secretId: string;
    /**
     * - `unknown`: the version is in an invalid state. `enabled`: the version is
     *   accessible. `disabled`: the version is not accessible but can be enabled.
     *   `destroyed`: the version is permanently deleted. It is not possible to
     *   recover it.
     */
    status: SecretVersionStatus;
    /** Date and time of the version's creation. */
    createdAt?: Date;
    /** Last update of the version. */
    updatedAt?: Date;
    /** Description of the version. */
    description?: string;
    /** Returns `true` if the version is the latest. */
    isLatest: boolean;
    /**
     * Returns the version's expiration date, whether it expires after being
     * accessed once, and the action to perform (disable or delete) once the
     * version expires.
     */
    ephemeralProperties?: EphemeralProperties;
}
interface Secret {
    /** ID of the secret. */
    id: string;
    /** ID of the Project containing the secret. */
    projectId: string;
    /** Name of the secret. */
    name: string;
    /**
     * - `ready`: the secret can be read, modified and deleted. `locked`: no action
     *   can be performed on the secret. This status can only be applied and
     *   removed by Scaleway.
     */
    status: SecretStatus;
    /** Date and time of the secret's creation. */
    createdAt?: Date;
    /** Last update of the secret. */
    updatedAt?: Date;
    /** List of the secret's tags. */
    tags: string[];
    /** Number of versions for this secret. */
    versionCount: number;
    /** Updated description of the secret. */
    description?: string;
    /** Returns `true` for secrets that are managed by another product. */
    isManaged: boolean;
    /** Returns `true` for protected secrets that cannot be deleted. */
    isProtected: boolean;
    /** See `Secret.Type` enum for description of values. */
    type: SecretType;
    /** Location of the secret in the directory structure. */
    path: string;
    /**
     * (Optional.) Policy that defines whether/when a secret's versions expire. By
     * default, the policy is applied to all the secret's versions.
     */
    ephemeralPolicy?: EphemeralPolicy;
    /** Region of the secret. */
    region: Region;
}
type AccessSecretVersionByNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the secret. */
    secretName: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1. Value can be either:
     *
     * - A number (the revision number)
     * - "latest" (the latest revision)
     * - "latest_enabled" (the latest enabled revision).
     */
    revision: string;
    /**
     * (Optional.) If not specified, Secret Manager will look for the secret
     * version in all Projects.
     */
    projectId?: string;
};
type AccessSecretVersionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1. Value can be either:
     *
     * - A number (the revision number)
     * - "latest" (the latest revision)
     * - "latest_enabled" (the latest enabled revision).
     */
    revision: string;
};
interface AccessSecretVersionResponse {
    /** ID of the secret. */
    secretId: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1.
     */
    revision: number;
    /** The base64-encoded secret payload of the version. */
    data: string;
    /**
     * This field is only available if a CRC32 was supplied during the creation of
     * the version.
     */
    dataCrc32?: number;
}
type AddSecretOwnerRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /** @deprecated (Deprecated: use `product` field) Name of the product to add. */
    productName?: string;
    /** See `Product` enum for description of values. */
    product?: Product;
};
type CreateFolderRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the Project containing the folder. */
    projectId?: string;
    /** Name of the folder. */
    name: string;
    /**
     * (Optional.) Location of the folder in the directory structure. If not
     * specified, the path is `/`.
     */
    path?: string;
};
type CreateSecretRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the Project containing the secret. */
    projectId?: string;
    /** Name of the secret. */
    name: string;
    /** List of the secret's tags. */
    tags?: string[];
    /** Description of the secret. */
    description?: string;
    /**
     * (Optional.) See `Secret.Type` enum for description of values. If not
     * specified, the type is `Opaque`.
     */
    type?: SecretType;
    /**
     * (Optional.) Location of the secret in the directory structure. If not
     * specified, the path is `/`.
     */
    path?: string;
    /**
     * (Optional.) Policy that defines whether/when a secret's versions expire. By
     * default, the policy is applied to all the secret's versions.
     */
    ephemeralPolicy?: EphemeralPolicy;
    /** A protected secret cannot be deleted. */
    isProtected: boolean;
};
type CreateSecretVersionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /** The base64-encoded secret payload of the version. */
    data: string;
    /** Description of the version. */
    description?: string;
    /**
     * (Optional.) If there is no previous version or if the previous version was
     * already disabled, does nothing.
     */
    disablePrevious?: boolean;
    /**
     * @deprecated (Optional.) If specified, a random password will be generated.
     *   The `data` and `data_crc32` fields must be empty. By default, the
     *   generator will use upper and lower case letters, and digits. This
     *   behavior can be tuned using the generation parameters.
     */
    passwordGeneration?: PasswordGenerationParams;
    /**
     * If specified, Secret Manager will verify the integrity of the data received
     * against the given CRC32 checksum. An error is returned if the CRC32 does
     * not match. If, however, the CRC32 matches, it will be stored and returned
     * along with the SecretVersion on future access requests.
     */
    dataCrc32?: number;
};
type DeleteFolderRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the folder. */
    folderId: string;
};
type DeleteSecretRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
};
type DestroySecretVersionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1. Value can be either:
     *
     * - A number (the revision number)
     * - "latest" (the latest revision)
     * - "latest_enabled" (the latest enabled revision).
     */
    revision: string;
};
type DisableSecretVersionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1. Value can be either:
     *
     * - A number (the revision number)
     * - "latest" (the latest revision)
     * - "latest_enabled" (the latest enabled revision).
     */
    revision: string;
};
type EnableSecretVersionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1. Value can be either:
     *
     * - A number (the revision number)
     * - "latest" (the latest revision)
     * - "latest_enabled" (the latest enabled revision).
     */
    revision: string;
};
type GeneratePasswordRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /** Description of the version. */
    description?: string;
    /**
     * This has no effect if there is no previous version or if the previous
     * version was already disabled.
     */
    disablePrevious?: boolean;
    /** Length of the password to generate (between 1 and 1024 characters). */
    length: number;
    /**
     * (Optional.) Exclude lower case letters by default in the password character
     * set.
     */
    noLowercaseLetters?: boolean;
    /**
     * (Optional.) Exclude upper case letters by default in the password character
     * set.
     */
    noUppercaseLetters?: boolean;
    /** (Optional.) Exclude digits by default in the password character set. */
    noDigits?: boolean;
    /**
     * (Optional.) Additional ASCII characters to be included in the password
     * character set.
     */
    additionalChars?: string;
};
type GetSecretByNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the secret. */
    secretName: string;
    /**
     * (Optional.) If not specified, Secret Manager will look for the secret in
     * all Projects.
     */
    projectId?: string;
};
type GetSecretRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
};
type GetSecretVersionByNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the secret. */
    secretName: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1. Value can be either:
     *
     * - A number (the revision number)
     * - "latest" (the latest revision)
     * - "latest_enabled" (the latest enabled revision).
     */
    revision: string;
    /**
     * (Optional.) If not specified, Secret Manager will look for the secret
     * version in all Projects.
     */
    projectId?: string;
};
type GetSecretVersionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1. Value can be either:
     *
     * - A number (the revision number)
     * - "latest" (the latest revision)
     * - "latest_enabled" (the latest enabled revision).
     */
    revision: string;
};
type ListFoldersRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Filter by Project ID (optional). */
    projectId?: string;
    /** Filter by path (optional). */
    path?: string;
    page?: number;
    pageSize?: number;
    orderBy?: ListFoldersRequestOrderBy;
};
interface ListFoldersResponse {
    /** List of folders. */
    folders: Folder[];
    /** Count of all folders matching the requested criteria. */
    totalCount: number;
}
type ListSecretVersionsByNameRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name of the secret. */
    secretName: string;
    page?: number;
    pageSize?: number;
    /** Filter results by status. */
    status?: SecretVersionStatus[];
    /**
     * (Optional.) If not specified, Secret Manager will look for the secret in
     * all Projects.
     */
    projectId?: string;
};
type ListSecretVersionsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    page?: number;
    pageSize?: number;
    /** Filter results by status. */
    status?: SecretVersionStatus[];
};
interface ListSecretVersionsResponse {
    /** Single page of versions. */
    versions: SecretVersion[];
    /** Number of versions. */
    totalCount: number;
}
type ListSecretsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Filter by Organization ID (optional). */
    organizationId?: string;
    /** Filter by Project ID (optional). */
    projectId?: string;
    orderBy?: ListSecretsRequestOrderBy;
    page?: number;
    pageSize?: number;
    /** List of tags to filter on (optional). */
    tags?: string[];
    /** Filter by secret name (optional). */
    name?: string;
    /** Filter by managed / not managed (optional). */
    isManaged?: boolean;
    /** Filter by path (optional). */
    path?: string;
    /** Filter by ephemeral / not ephemeral (optional). */
    isEphemeral?: boolean;
};
interface ListSecretsResponse {
    /** Single page of secrets matching the requested criteria. */
    secrets: Secret[];
    /** Count of all secrets matching the requested criteria. */
    totalCount: number;
}
type ListTagsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * (Optional.) If not specified, Secret Manager will look for tags in all
     * Projects.
     */
    projectId?: string;
    page?: number;
    pageSize?: number;
};
interface ListTagsResponse {
    /** List of tags. */
    tags: string[];
    /** Count of all tags matching the requested criteria. */
    totalCount: number;
}
type ProtectSecretRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret to protect. */
    secretId: string;
};
type UnprotectSecretRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret to unprotect. */
    secretId: string;
};
type UpdateSecretRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /** Secret's updated name (optional). */
    name?: string;
    /** Secret's updated list of tags (optional). */
    tags?: string[];
    /** Description of the secret. */
    description?: string;
    /**
     * (Optional.) Location of the folder in the directory structure. If not
     * specified, the path is `/`.
     */
    path?: string;
    /** (Optional.) Policy that defines whether/when a secret's versions expire. */
    ephemeralPolicy?: EphemeralPolicy;
};
type UpdateSecretVersionRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the secret. */
    secretId: string;
    /**
     * The first version of the secret is numbered 1, and all subsequent revisions
     * augment by 1. Value can be either:
     *
     * - A number (the revision number)
     * - "latest" (the latest revision)
     * - "latest_enabled" (the latest enabled revision).
     */
    revision: string;
    /** Description of the version. */
    description?: string;
    /**
     * (Optional.) Properties that defines the version's expiration date, whether
     * it expires after being accessed once, and the action to perform (disable or
     * delete) once the version expires.
     */
    ephemeralProperties?: EphemeralProperties;
};

/**
 * Secret Manager API.
 *
 * Secret Manager API. This API allows you to conveniently store, access and
 * share sensitive data.
 */
declare class API$7 extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Create a secret. You must specify the `region` to create a secret.
     *
     * @param request - The request {@link CreateSecretRequest}
     * @returns A Promise of Secret
     */
    createSecret: (request: Readonly<CreateSecretRequest>) => Promise<Secret>;
    /**
     * Create folder.
     *
     * @param request - The request {@link CreateFolderRequest}
     * @returns A Promise of Folder
     */
    createFolder: (request: Readonly<CreateFolderRequest>) => Promise<Folder>;
    /**
     * Get metadata using the secret's ID. Retrieve the metadata of a secret
     * specified by the `region` and `secret_id` parameters.
     *
     * @param request - The request {@link GetSecretRequest}
     * @returns A Promise of Secret
     */
    getSecret: (request: Readonly<GetSecretRequest>) => Promise<Secret>;
    /**
     * Get metadata using the secret's name. Retrieve the metadata of a secret
     * specified by the `region` and `secret_name` parameters.
     *
     * GetSecretByName usage is now deprecated.
     *
     * Scaleway recommends that you use the `ListSecrets` request with the `name`
     * filter.
     *
     * @deprecated
     * @param request - The request {@link GetSecretByNameRequest}
     * @returns A Promise of Secret
     */
    getSecretByName: (request: Readonly<GetSecretByNameRequest>) => Promise<Secret>;
    /**
     * Update metadata of a secret. Edit a secret's metadata such as name, tag(s),
     * description and ephemeral policy. The secret to update is specified by the
     * `secret_id` and `region` parameters.
     *
     * @param request - The request {@link UpdateSecretRequest}
     * @returns A Promise of Secret
     */
    updateSecret: (request: Readonly<UpdateSecretRequest>) => Promise<Secret>;
    protected pageOfListSecrets: (request?: Readonly<ListSecretsRequest>) => Promise<ListSecretsResponse>;
    /**
     * List secrets. Retrieve the list of secrets created within an Organization
     * and/or Project. You must specify either the `organization_id` or the
     * `project_id` and the `region`.
     *
     * @param request - The request {@link ListSecretsRequest}
     * @returns A Promise of ListSecretsResponse
     */
    listSecrets: (request?: Readonly<ListSecretsRequest>) => Promise<ListSecretsResponse> & {
        all: () => Promise<Secret[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Secret[], void, void>;
    };
    protected pageOfListFolders: (request?: Readonly<ListFoldersRequest>) => Promise<ListFoldersResponse>;
    /**
     * List folders. Retrieve the list of folders created within a Project.
     *
     * @param request - The request {@link ListFoldersRequest}
     * @returns A Promise of ListFoldersResponse
     */
    listFolders: (request?: Readonly<ListFoldersRequest>) => Promise<ListFoldersResponse> & {
        all: () => Promise<Folder[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Folder[], void, void>;
    };
    /**
     * Delete a secret. Delete a given secret specified by the `region` and
     * `secret_id` parameters.
     *
     * @param request - The request {@link DeleteSecretRequest}
     */
    deleteSecret: (request: Readonly<DeleteSecretRequest>) => Promise<void>;
    /**
     * Delete a given folder specified by the `region` and `folder_id` parameters.
     *
     * @param request - The request {@link DeleteFolderRequest}
     */
    deleteFolder: (request: Readonly<DeleteFolderRequest>) => Promise<void>;
    /**
     * Protect a secret. Protect a given secret specified by the `secret_id`
     * parameter. A protected secret can be read and modified but cannot be
     * deleted.
     *
     * @param request - The request {@link ProtectSecretRequest}
     * @returns A Promise of Secret
     */
    protectSecret: (request: Readonly<ProtectSecretRequest>) => Promise<Secret>;
    /**
     * Unprotect a secret. Unprotect a given secret specified by the `secret_id`
     * parameter. An unprotected secret can be read, modified and deleted.
     *
     * @param request - The request {@link UnprotectSecretRequest}
     * @returns A Promise of Secret
     */
    unprotectSecret: (request: Readonly<UnprotectSecretRequest>) => Promise<Secret>;
    /**
     * Allow a product to use the secret.
     *
     * @param request - The request {@link AddSecretOwnerRequest}
     */
    addSecretOwner: (request: Readonly<AddSecretOwnerRequest>) => Promise<void>;
    /**
     * Create a version. Create a version of a given secret specified by the
     * `region` and `secret_id` parameters.
     *
     * @param request - The request {@link CreateSecretVersionRequest}
     * @returns A Promise of SecretVersion
     */
    createSecretVersion: (request: Readonly<CreateSecretVersionRequest>) => Promise<SecretVersion>;
    /**
     * Generate a password in a new version. Generate a password for the given
     * secret specified by the `region` and `secret_id` parameters. This will also
     * create a new version of the secret that will store the password.
     *
     * @param request - The request {@link GeneratePasswordRequest}
     * @returns A Promise of SecretVersion
     */
    generatePassword: (request: Readonly<GeneratePasswordRequest>) => Promise<SecretVersion>;
    /**
     * Get metadata of a secret's version using the secret's ID. Retrieve the
     * metadata of a secret's given version specified by the `region`, `secret_id`
     * and `revision` parameters.
     *
     * @param request - The request {@link GetSecretVersionRequest}
     * @returns A Promise of SecretVersion
     */
    getSecretVersion: (request: Readonly<GetSecretVersionRequest>) => Promise<SecretVersion>;
    /**
     * Get metadata of a secret's version using the secret's name. Retrieve the
     * metadata of a secret's given version specified by the `region`,
     * `secret_name`, `revision` and `project_id` parameters.
     *
     * This method is deprecated.
     *
     * Scaleway recommends that you use the `ListSecrets` request with the `name`
     * filter to specify the secret version desired, then use the
     * `GetSecretVersion` request.
     *
     * @deprecated
     * @param request - The request {@link GetSecretVersionByNameRequest}
     * @returns A Promise of SecretVersion
     */
    getSecretVersionByName: (request: Readonly<GetSecretVersionByNameRequest>) => Promise<SecretVersion>;
    /**
     * Update metadata of a version. Edit the metadata of a secret's given
     * version, specified by the `region`, `secret_id` and `revision` parameters.
     *
     * @param request - The request {@link UpdateSecretVersionRequest}
     * @returns A Promise of SecretVersion
     */
    updateSecretVersion: (request: Readonly<UpdateSecretVersionRequest>) => Promise<SecretVersion>;
    protected pageOfListSecretVersions: (request: Readonly<ListSecretVersionsRequest>) => Promise<ListSecretVersionsResponse>;
    /**
     * List versions of a secret using the secret's ID. Retrieve the list of a
     * given secret's versions specified by the `secret_id` and `region`
     * parameters.
     *
     * @param request - The request {@link ListSecretVersionsRequest}
     * @returns A Promise of ListSecretVersionsResponse
     */
    listSecretVersions: (request: Readonly<ListSecretVersionsRequest>) => Promise<ListSecretVersionsResponse> & {
        all: () => Promise<SecretVersion[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<SecretVersion[], void, void>;
    };
    protected pageOfListSecretVersionsByName: (request: Readonly<ListSecretVersionsByNameRequest>) => Promise<ListSecretVersionsResponse>;
    /**
     * List versions of a secret using the secret's name. Retrieve the list of a
     * given secret's versions specified by the `secret_name`,`region` and
     * `project_id` parameters.
     *
     * This method is deprecated.
     *
     * Scaleway recommends that you use the `ListSecrets` request with the `name`
     * filter to specify the secret version desired, then use the
     * `ListSecretVersions` request.
     *
     * @deprecated
     * @param request - The request {@link ListSecretVersionsByNameRequest}
     * @returns A Promise of ListSecretVersionsResponse
     */
    listSecretVersionsByName: (request: Readonly<ListSecretVersionsByNameRequest>) => Promise<ListSecretVersionsResponse> & {
        all: () => Promise<SecretVersion[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<SecretVersion[], void, void>;
    };
    /**
     * Enable a version. Make a specific version accessible. You must specify the
     * `region`, `secret_id` and `revision` parameters.
     *
     * @param request - The request {@link EnableSecretVersionRequest}
     * @returns A Promise of SecretVersion
     */
    enableSecretVersion: (request: Readonly<EnableSecretVersionRequest>) => Promise<SecretVersion>;
    /**
     * Disable a version. Make a specific version inaccessible. You must specify
     * the `region`, `secret_id` and `revision` parameters.
     *
     * @param request - The request {@link DisableSecretVersionRequest}
     * @returns A Promise of SecretVersion
     */
    disableSecretVersion: (request: Readonly<DisableSecretVersionRequest>) => Promise<SecretVersion>;
    /**
     * Access a secret's version using the secret's ID. Access sensitive data in a
     * secret's version specified by the `region`, `secret_id` and `revision`
     * parameters.
     *
     * @param request - The request {@link AccessSecretVersionRequest}
     * @returns A Promise of AccessSecretVersionResponse
     */
    accessSecretVersion: (request: Readonly<AccessSecretVersionRequest>) => Promise<AccessSecretVersionResponse>;
    /**
     * Access a secret's version using the secret's name. Access sensitive data in
     * a secret's version specified by the `region`, `secret_name`, `revision` and
     * `project_id` parameters.
     *
     * This method is deprecated.
     *
     * Scaleway recommends that you use the `ListSecrets` request with the `name`
     * filter to specify the secret version desired, then use the
     * `AccessSecretVersion` request.
     *
     * @deprecated
     * @param request - The request {@link AccessSecretVersionByNameRequest}
     * @returns A Promise of AccessSecretVersionResponse
     */
    accessSecretVersionByName: (request: Readonly<AccessSecretVersionByNameRequest>) => Promise<AccessSecretVersionResponse>;
    /**
     * Delete a version. Delete a secret's version and the sensitive data
     * contained in it. Deleting a version is permanent and cannot be undone.
     *
     * @param request - The request {@link DestroySecretVersionRequest}
     * @returns A Promise of SecretVersion
     */
    destroySecretVersion: (request: Readonly<DestroySecretVersionRequest>) => Promise<SecretVersion>;
    protected pageOfListTags: (request?: Readonly<ListTagsRequest>) => Promise<ListTagsResponse>;
    /**
     * List tags. List all tags associated with secrets within a given Project.
     *
     * @param request - The request {@link ListTagsRequest}
     * @returns A Promise of ListTagsResponse
     */
    listTags: (request?: Readonly<ListTagsRequest>) => Promise<ListTagsResponse> & {
        all: () => Promise<string[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<string[], void, void>;
    };
}

type index_gen$8_AccessSecretVersionByNameRequest = AccessSecretVersionByNameRequest;
type index_gen$8_AccessSecretVersionRequest = AccessSecretVersionRequest;
type index_gen$8_AccessSecretVersionResponse = AccessSecretVersionResponse;
type index_gen$8_AddSecretOwnerRequest = AddSecretOwnerRequest;
type index_gen$8_CreateFolderRequest = CreateFolderRequest;
type index_gen$8_CreateSecretRequest = CreateSecretRequest;
type index_gen$8_CreateSecretVersionRequest = CreateSecretVersionRequest;
type index_gen$8_DeleteFolderRequest = DeleteFolderRequest;
type index_gen$8_DeleteSecretRequest = DeleteSecretRequest;
type index_gen$8_DestroySecretVersionRequest = DestroySecretVersionRequest;
type index_gen$8_DisableSecretVersionRequest = DisableSecretVersionRequest;
type index_gen$8_EnableSecretVersionRequest = EnableSecretVersionRequest;
type index_gen$8_EphemeralPolicy = EphemeralPolicy;
type index_gen$8_EphemeralPolicyAction = EphemeralPolicyAction;
type index_gen$8_EphemeralProperties = EphemeralProperties;
type index_gen$8_Folder = Folder;
type index_gen$8_GeneratePasswordRequest = GeneratePasswordRequest;
type index_gen$8_GetSecretByNameRequest = GetSecretByNameRequest;
type index_gen$8_GetSecretRequest = GetSecretRequest;
type index_gen$8_GetSecretVersionByNameRequest = GetSecretVersionByNameRequest;
type index_gen$8_GetSecretVersionRequest = GetSecretVersionRequest;
type index_gen$8_ListFoldersRequest = ListFoldersRequest;
type index_gen$8_ListFoldersRequestOrderBy = ListFoldersRequestOrderBy;
type index_gen$8_ListFoldersResponse = ListFoldersResponse;
type index_gen$8_ListSecretVersionsByNameRequest = ListSecretVersionsByNameRequest;
type index_gen$8_ListSecretVersionsRequest = ListSecretVersionsRequest;
type index_gen$8_ListSecretVersionsResponse = ListSecretVersionsResponse;
type index_gen$8_ListSecretsRequest = ListSecretsRequest;
type index_gen$8_ListSecretsRequestOrderBy = ListSecretsRequestOrderBy;
type index_gen$8_ListSecretsResponse = ListSecretsResponse;
type index_gen$8_ListTagsRequest = ListTagsRequest;
type index_gen$8_ListTagsResponse = ListTagsResponse;
type index_gen$8_PasswordGenerationParams = PasswordGenerationParams;
type index_gen$8_Product = Product;
type index_gen$8_ProtectSecretRequest = ProtectSecretRequest;
type index_gen$8_Secret = Secret;
type index_gen$8_SecretStatus = SecretStatus;
type index_gen$8_SecretType = SecretType;
type index_gen$8_SecretVersion = SecretVersion;
type index_gen$8_SecretVersionStatus = SecretVersionStatus;
type index_gen$8_UnprotectSecretRequest = UnprotectSecretRequest;
type index_gen$8_UpdateSecretRequest = UpdateSecretRequest;
type index_gen$8_UpdateSecretVersionRequest = UpdateSecretVersionRequest;
declare namespace index_gen$8 {
  export { API$7 as API, type index_gen$8_AccessSecretVersionByNameRequest as AccessSecretVersionByNameRequest, type index_gen$8_AccessSecretVersionRequest as AccessSecretVersionRequest, type index_gen$8_AccessSecretVersionResponse as AccessSecretVersionResponse, type index_gen$8_AddSecretOwnerRequest as AddSecretOwnerRequest, type index_gen$8_CreateFolderRequest as CreateFolderRequest, type index_gen$8_CreateSecretRequest as CreateSecretRequest, type index_gen$8_CreateSecretVersionRequest as CreateSecretVersionRequest, type index_gen$8_DeleteFolderRequest as DeleteFolderRequest, type index_gen$8_DeleteSecretRequest as DeleteSecretRequest, type index_gen$8_DestroySecretVersionRequest as DestroySecretVersionRequest, type index_gen$8_DisableSecretVersionRequest as DisableSecretVersionRequest, type index_gen$8_EnableSecretVersionRequest as EnableSecretVersionRequest, type index_gen$8_EphemeralPolicy as EphemeralPolicy, type index_gen$8_EphemeralPolicyAction as EphemeralPolicyAction, type index_gen$8_EphemeralProperties as EphemeralProperties, type index_gen$8_Folder as Folder, type index_gen$8_GeneratePasswordRequest as GeneratePasswordRequest, type index_gen$8_GetSecretByNameRequest as GetSecretByNameRequest, type index_gen$8_GetSecretRequest as GetSecretRequest, type index_gen$8_GetSecretVersionByNameRequest as GetSecretVersionByNameRequest, type index_gen$8_GetSecretVersionRequest as GetSecretVersionRequest, type index_gen$8_ListFoldersRequest as ListFoldersRequest, type index_gen$8_ListFoldersRequestOrderBy as ListFoldersRequestOrderBy, type index_gen$8_ListFoldersResponse as ListFoldersResponse, type index_gen$8_ListSecretVersionsByNameRequest as ListSecretVersionsByNameRequest, type index_gen$8_ListSecretVersionsRequest as ListSecretVersionsRequest, type index_gen$8_ListSecretVersionsResponse as ListSecretVersionsResponse, type index_gen$8_ListSecretsRequest as ListSecretsRequest, type index_gen$8_ListSecretsRequestOrderBy as ListSecretsRequestOrderBy, type index_gen$8_ListSecretsResponse as ListSecretsResponse, type index_gen$8_ListTagsRequest as ListTagsRequest, type index_gen$8_ListTagsResponse as ListTagsResponse, type index_gen$8_PasswordGenerationParams as PasswordGenerationParams, type index_gen$8_Product as Product, type index_gen$8_ProtectSecretRequest as ProtectSecretRequest, type index_gen$8_Secret as Secret, type index_gen$8_SecretStatus as SecretStatus, type index_gen$8_SecretType as SecretType, type index_gen$8_SecretVersion as SecretVersion, type index_gen$8_SecretVersionStatus as SecretVersionStatus, type index_gen$8_UnprotectSecretRequest as UnprotectSecretRequest, type index_gen$8_UpdateSecretRequest as UpdateSecretRequest, type index_gen$8_UpdateSecretVersionRequest as UpdateSecretVersionRequest };
}

declare namespace index$6 {
  export { index_gen$8 as v1alpha1 };
}

type DatabaseBackupStatus = 'unknown_status' | 'error' | 'ready' | 'locked';
type DatabaseStatus = 'unknown_status' | 'error' | 'creating' | 'ready' | 'deleting' | 'restoring' | 'locked';
type ListDatabaseBackupsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListDatabasesRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
interface DatabaseBackup {
    /** UUID that uniquely identifies a Serverless SQL Database backup. */
    id: string;
    /**
     * Status of the Serverless SQL Database backup. One of `unknown_status` |
     * `error` | `ready` | `locked`.
     */
    status: DatabaseBackupStatus;
    /** The ID of your Scaleway organization. */
    organizationId: string;
    /** UUID of the Scaleway project. */
    projectId: string;
    /** UUID of the source Serverless SQL Database the backup is created from. */
    databaseId: string;
    /** Creation date. */
    createdAt?: Date;
    /** Expiration date. */
    expiresAt?: Date;
    /** Size of the database backup. */
    size?: number;
    /** Download URL of the exported database backup. */
    downloadUrl?: string;
    /** Expiration date of the download URL. */
    downloadUrlExpiresAt?: Date;
    /** Region of the database backup. */
    region: Region;
}
interface Database$1 {
    /** UUID that uniquely identifies your Serverless SQL DB Database. */
    id: string;
    /** Name of the database. */
    name: string;
    /**
     * Status of the Serverless SQL Ddatabase. One of `unknown_status` | `ready` |
     * `creating` | `deleting` | `error` | `restoring` | `locked`.
     */
    status: DatabaseStatus;
    /** Endpoint of the database. */
    endpoint: string;
    /** The ID of your Scaleway organization. */
    organizationId: string;
    /** Project ID the database belongs to. */
    projectId: string;
    /** Region of the database. */
    region: Region;
    /** Creation date. */
    createdAt?: Date;
    /** The minimum number of CPU units for your Serverless SQL Database. */
    cpuMin: number;
    /** The maximum number of CPU units for your Serverless SQL Database. */
    cpuMax: number;
    /** The current number of CPU units allocated to your Serverless SQL Database. */
    cpuCurrent: number;
    /** Whether your Serverless SQL Database is running or not. */
    started: boolean;
    /** The major version of the underlying database engine. */
    engineMajorVersion: number;
}
type CreateDatabaseRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** The ID of your Scaleway project. */
    projectId?: string;
    /** The name of the Serverless SQL Database to be created. */
    name: string;
    /** The minimum number of CPU units for your Serverless SQL Database. */
    cpuMin: number;
    /** The maximum number of CPU units for your Serverless SQL Database. */
    cpuMax: number;
    /** The ID of the backup to create the database from. */
    fromBackupId?: string;
};
type DeleteDatabaseRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Serverless SQL Database. */
    databaseId: string;
};
type ExportDatabaseBackupRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Serverless SQL Database backup. */
    backupId: string;
};
type GetDatabaseBackupRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Serverless SQL Database backup. */
    backupId: string;
};
type GetDatabaseRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Serverless SQL DB database. */
    databaseId: string;
};
type ListDatabaseBackupsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Filter by the UUID of the Scaleway organization. */
    organizationId?: string;
    /** Filter by the UUID of the Scaleway project. */
    projectId?: string;
    /** Filter by the UUID of the Serverless SQL Database. */
    databaseId: string;
    /** Page number. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    /** Sorting criteria. One of `created_at_asc`, `created_at_desc`. */
    orderBy?: ListDatabaseBackupsRequestOrderBy;
};
interface ListDatabaseBackupsResponse {
    /** List of the backups. */
    backups: DatabaseBackup[];
    /** Length of the backups list. */
    totalCount: number;
}
type ListDatabasesRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Filter by the UUID of the Scaleway organization. */
    organizationId?: string;
    /** UUID of the Scaleway project. */
    projectId?: string;
    /** Page number. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    /** Filter by the name of the database. */
    name?: string;
    /**
     * Sorting criteria. One of `created_at_asc`, `created_at_desc`, `name_asc`,
     * `name_desc`.
     */
    orderBy?: ListDatabasesRequestOrderBy;
};
interface ListDatabasesResponse {
    /** List of the databases. */
    databases: Database$1[];
    /** Total count of Serverless SQL Databases. */
    totalCount: number;
}
type RestoreDatabaseFromBackupRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Serverless SQL Database. */
    databaseId: string;
    /** UUID of the Serverless SQL Database backup to restore. */
    backupId: string;
};
type UpdateDatabaseRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** UUID of the Serverless SQL Database. */
    databaseId: string;
    /** The minimum number of CPU units for your Serverless SQL Database. */
    cpuMin?: number;
    /** The maximum number of CPU units for your Serverless SQL Database. */
    cpuMax?: number;
};

/**
 * Serverless SQL Databases API.
 *
 * This API allows you to manage your Serverless SQL DB databases.
 */
declare class API$6 extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Create a new Serverless SQL Database. You must provide the following
     * parameters: `organization_id`, `project_id`, `name`, `cpu_min`, `cpu_max`.
     * You can also provide `from_backup_id` to create a database from a backup.
     *
     * @param request - The request {@link CreateDatabaseRequest}
     * @returns A Promise of Database
     */
    createDatabase: (request: Readonly<CreateDatabaseRequest$1>) => Promise<Database$1>;
    /**
     * Get a database information. Retrieve information about your Serverless SQL
     * Database. You must provide the `database_id` parameter.
     *
     * @param request - The request {@link GetDatabaseRequest}
     * @returns A Promise of Database
     */
    getDatabase: (request: Readonly<GetDatabaseRequest>) => Promise<Database$1>;
    /**
     * Waits for {@link Database} to be in a final state.
     *
     * @param request - The request {@link GetDatabaseRequest}
     * @param options - The waiting options
     * @returns A Promise of Database
     */
    waitForDatabase: (request: Readonly<GetDatabaseRequest>, options?: Readonly<WaitForOptions<Database$1>>) => Promise<Database$1>;
    /**
     * Delete a database. Deletes a database. You must provide the `database_id`
     * parameter. All data stored in the database will be permanently deleted.
     *
     * @param request - The request {@link DeleteDatabaseRequest}
     * @returns A Promise of Database
     */
    deleteDatabase: (request: Readonly<DeleteDatabaseRequest>) => Promise<Database$1>;
    protected pageOfListDatabases: (request?: Readonly<ListDatabasesRequest$1>) => Promise<ListDatabasesResponse>;
    /**
     * List your Serverless SQL Databases. List all Serverless SQL Databases for a
     * given Scaleway Organization or Scaleway Project. By default, the databases
     * returned in the list are ordered by creation date in ascending order,
     * though this can be modified via the order_by field. For the `name`
     * parameter, the value you include will be checked against the whole name
     * string to see if it includes the string you put in the parameter.
     *
     * @param request - The request {@link ListDatabasesRequest}
     * @returns A Promise of ListDatabasesResponse
     */
    listDatabases: (request?: Readonly<ListDatabasesRequest$1>) => Promise<ListDatabasesResponse> & {
        all: () => Promise<Database$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Database$1[], void, void>;
    };
    /**
     * Update database information. Update CPU limits of your Serverless SQL
     * Database. You must provide the `database_id` parameter.
     *
     * @param request - The request {@link UpdateDatabaseRequest}
     * @returns A Promise of Database
     */
    updateDatabase: (request: Readonly<UpdateDatabaseRequest$1>) => Promise<Database$1>;
    /**
     * Restore a database from a backup. Restore a database from a backup. You
     * must provide the `backup_id` parameter.
     *
     * @param request - The request {@link RestoreDatabaseFromBackupRequest}
     * @returns A Promise of Database
     */
    restoreDatabaseFromBackup: (request: Readonly<RestoreDatabaseFromBackupRequest>) => Promise<Database$1>;
    /**
     * Get a database backup information. Retrieve information about your
     * Serverless SQL Database backup. You must provide the `backup_id`
     * parameter.
     *
     * @param request - The request {@link GetDatabaseBackupRequest}
     * @returns A Promise of DatabaseBackup
     */
    getDatabaseBackup: (request: Readonly<GetDatabaseBackupRequest>) => Promise<DatabaseBackup>;
    protected pageOfListDatabaseBackups: (request: Readonly<ListDatabaseBackupsRequest$1>) => Promise<ListDatabaseBackupsResponse>;
    /**
     * List your Serverless SQL Database backups. List all Serverless SQL Database
     * backups for a given Scaleway Project or Database. By default, the backups
     * returned in the list are ordered by creation date in ascending order,
     * though this can be modified via the order_by field.
     *
     * @param request - The request {@link ListDatabaseBackupsRequest}
     * @returns A Promise of ListDatabaseBackupsResponse
     */
    listDatabaseBackups: (request: Readonly<ListDatabaseBackupsRequest$1>) => Promise<ListDatabaseBackupsResponse> & {
        all: () => Promise<DatabaseBackup[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DatabaseBackup[], void, void>;
    };
    /**
     * Export a database backup. Export a database backup providing a download
     * link once the export process is completed. You must provide the `backup_id`
     * parameter.
     *
     * @param request - The request {@link ExportDatabaseBackupRequest}
     * @returns A Promise of DatabaseBackup
     */
    exportDatabaseBackup: (request: Readonly<ExportDatabaseBackupRequest>) => Promise<DatabaseBackup>;
}

/** Lists transient statutes of the enum {@link DatabaseStatus}. */
declare const DATABASE_TRANSIENT_STATUSES: DatabaseStatus[];

declare const CreateDatabaseRequest: {
    cpuMax: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
    cpuMin: {
        lessThanOrEqual: number;
    };
    fromBackupId: {
        ignoreEmpty: boolean;
    };
    name: {
        maxLength: number;
        minLength: number;
    };
};
declare const Database: {
    cpuCurrent: {
        lessThanOrEqual: number;
    };
    cpuMax: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
    cpuMin: {
        lessThanOrEqual: number;
    };
};
declare const ListDatabaseBackupsRequest: {
    page: {
        greaterThanOrEqual: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const ListDatabasesRequest: {
    page: {
        greaterThanOrEqual: number;
    };
    pageSize: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
};
declare const UpdateDatabaseRequest: {
    cpuMax: {
        greaterThanOrEqual: number;
        lessThanOrEqual: number;
    };
    cpuMin: {
        lessThanOrEqual: number;
    };
};

declare const validationRules_gen$1_CreateDatabaseRequest: typeof CreateDatabaseRequest;
declare const validationRules_gen$1_Database: typeof Database;
declare const validationRules_gen$1_ListDatabaseBackupsRequest: typeof ListDatabaseBackupsRequest;
declare const validationRules_gen$1_ListDatabasesRequest: typeof ListDatabasesRequest;
declare const validationRules_gen$1_UpdateDatabaseRequest: typeof UpdateDatabaseRequest;
declare namespace validationRules_gen$1 {
  export { validationRules_gen$1_CreateDatabaseRequest as CreateDatabaseRequest, validationRules_gen$1_Database as Database, validationRules_gen$1_ListDatabaseBackupsRequest as ListDatabaseBackupsRequest, validationRules_gen$1_ListDatabasesRequest as ListDatabasesRequest, validationRules_gen$1_UpdateDatabaseRequest as UpdateDatabaseRequest };
}

declare const index_gen$7_DATABASE_TRANSIENT_STATUSES: typeof DATABASE_TRANSIENT_STATUSES;
type index_gen$7_DatabaseBackup = DatabaseBackup;
type index_gen$7_DatabaseBackupStatus = DatabaseBackupStatus;
type index_gen$7_DatabaseStatus = DatabaseStatus;
type index_gen$7_DeleteDatabaseRequest = DeleteDatabaseRequest;
type index_gen$7_ExportDatabaseBackupRequest = ExportDatabaseBackupRequest;
type index_gen$7_GetDatabaseBackupRequest = GetDatabaseBackupRequest;
type index_gen$7_GetDatabaseRequest = GetDatabaseRequest;
type index_gen$7_ListDatabaseBackupsRequestOrderBy = ListDatabaseBackupsRequestOrderBy;
type index_gen$7_ListDatabaseBackupsResponse = ListDatabaseBackupsResponse;
type index_gen$7_ListDatabasesRequestOrderBy = ListDatabasesRequestOrderBy;
type index_gen$7_ListDatabasesResponse = ListDatabasesResponse;
type index_gen$7_RestoreDatabaseFromBackupRequest = RestoreDatabaseFromBackupRequest;
declare namespace index_gen$7 {
  export { API$6 as API, type CreateDatabaseRequest$1 as CreateDatabaseRequest, index_gen$7_DATABASE_TRANSIENT_STATUSES as DATABASE_TRANSIENT_STATUSES, type Database$1 as Database, type index_gen$7_DatabaseBackup as DatabaseBackup, type index_gen$7_DatabaseBackupStatus as DatabaseBackupStatus, type index_gen$7_DatabaseStatus as DatabaseStatus, type index_gen$7_DeleteDatabaseRequest as DeleteDatabaseRequest, type index_gen$7_ExportDatabaseBackupRequest as ExportDatabaseBackupRequest, type index_gen$7_GetDatabaseBackupRequest as GetDatabaseBackupRequest, type index_gen$7_GetDatabaseRequest as GetDatabaseRequest, type ListDatabaseBackupsRequest$1 as ListDatabaseBackupsRequest, type index_gen$7_ListDatabaseBackupsRequestOrderBy as ListDatabaseBackupsRequestOrderBy, type index_gen$7_ListDatabaseBackupsResponse as ListDatabaseBackupsResponse, type ListDatabasesRequest$1 as ListDatabasesRequest, type index_gen$7_ListDatabasesRequestOrderBy as ListDatabasesRequestOrderBy, type index_gen$7_ListDatabasesResponse as ListDatabasesResponse, type index_gen$7_RestoreDatabaseFromBackupRequest as RestoreDatabaseFromBackupRequest, type UpdateDatabaseRequest$1 as UpdateDatabaseRequest, validationRules_gen$1 as ValidationRules };
}

declare namespace index$5 {
  export { index_gen$7 as v1alpha1 };
}

type index_gen$6_LanguageCode = LanguageCode;
declare namespace index_gen$6 {
  export type { index_gen$6_LanguageCode as LanguageCode };
}

type EyeColors = 'unknown' | 'amber' | 'blue' | 'brown' | 'gray' | 'green' | 'hazel' | 'red' | 'violet';
type HumanStatus = 'unknown' | 'stopped' | 'running';
type ListHumansRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'updated_at_asc' | 'updated_at_desc' | 'height_asc' | 'height_desc';
interface Human {
    id: string;
    organizationId: string;
    createdAt?: Date;
    updatedAt?: Date;
    height: number;
    shoeSize: number;
    altitudeInMeter: number;
    altitudeInMillimeter: number;
    fingersCount: number;
    hairCount: number;
    isHappy: boolean;
    eyesColor: EyeColors;
    status: HumanStatus;
    name: string;
    projectId: string;
}
type CreateHumanRequest = {
    height: number;
    shoeSize: number;
    altitudeInMeter: number;
    altitudeInMillimeter: number;
    fingersCount: number;
    hairCount: number;
    isHappy: boolean;
    eyesColor?: EyeColors;
    /**
     * @deprecated
     *
     *   One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     *   could be set.
     */
    organizationId?: string;
    name: string;
    /**
     * One-of ('projectIdentifier'): at most one of 'projectId', 'organizationId'
     * could be set.
     */
    projectId?: string;
};
type DeleteHumanRequest = {
    /** UUID of the human you want to delete. */
    humanId: string;
};
type GetHumanRequest = {
    /** UUID of the human you want to get. */
    humanId: string;
};
type ListHumansRequest = {
    page?: number;
    pageSize?: number;
    orderBy?: ListHumansRequestOrderBy;
    organizationId?: string;
    projectId?: string;
};
interface ListHumansResponse {
    totalCount: number;
    humans: Human[];
}
type RegisterRequest = {
    username: string;
};
interface RegisterResponse {
    secretKey: string;
    accessKey: string;
}
type RunHumanRequest = {
    /** UUID of the human you want to make run. */
    humanId: string;
};
type SmokeHumanRequest = {
    /** @deprecated UUID of the human you want to make smoking. */
    humanId: string;
};
type UpdateHumanRequest = {
    /** UUID of the human you want to update. */
    humanId: string;
    /** Height of the human in meters. */
    height?: number;
    shoeSize?: number;
    altitudeInMeter?: number;
    altitudeInMillimeter?: number;
    fingersCount?: number;
    hairCount?: number;
    isHappy?: boolean;
    eyesColor?: EyeColors;
    name?: string;
};

/**
 * Fake API.
 *
 * No Auth Service for end-to-end testing.
 */
declare class API$5 extends API$x {
    /**
     * Register a user. Register a human and return a access-key and a secret-key
     * that must be used in all other commands.
     *
     * Hint: you can use other test commands by setting the SCW_SECRET_KEY env
     * variable.
     *
     * @param request - The request {@link RegisterRequest}
     * @returns A Promise of RegisterResponse
     */
    register: (request: Readonly<RegisterRequest>) => Promise<RegisterResponse>;
    protected pageOfListHumans: (request?: Readonly<ListHumansRequest>) => Promise<ListHumansResponse>;
    /**
     * List all your humans.
     *
     * @param request - The request {@link ListHumansRequest}
     * @returns A Promise of ListHumansResponse
     */
    listHumans: (request?: Readonly<ListHumansRequest>) => Promise<ListHumansResponse> & {
        all: () => Promise<Human[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Human[], void, void>;
    };
    /**
     * Get human details. Get the human details associated with the given id.
     *
     * @param request - The request {@link GetHumanRequest}
     * @returns A Promise of Human
     */
    getHuman: (request: Readonly<GetHumanRequest>) => Promise<Human>;
    /**
     * Waits for {@link Human} to be in a final state.
     *
     * @param request - The request {@link GetHumanRequest}
     * @param options - The waiting options
     * @returns A Promise of Human
     */
    waitForHuman: (request: Readonly<GetHumanRequest>, options?: Readonly<WaitForOptions<Human>>) => Promise<Human>;
    /**
     * Create a new human.
     *
     * @param request - The request {@link CreateHumanRequest}
     * @returns A Promise of Human
     */
    createHuman: (request: Readonly<CreateHumanRequest>) => Promise<Human>;
    /**
     * Update an existing human. Update the human associated with the given id.
     *
     * @param request - The request {@link UpdateHumanRequest}
     * @returns A Promise of Human
     */
    updateHuman: (request: Readonly<UpdateHumanRequest>) => Promise<Human>;
    /**
     * Delete an existing human. Delete the human associated with the given id.
     *
     * @param request - The request {@link DeleteHumanRequest}
     * @returns A Promise of Human
     */
    deleteHuman: (request: Readonly<DeleteHumanRequest>) => Promise<Human>;
    /**
     * Start a 1h running for the given human. Start a one hour running for the
     * given human.
     *
     * @param request - The request {@link RunHumanRequest}
     * @returns A Promise of Human
     */
    runHuman: (request: Readonly<RunHumanRequest>) => Promise<Human>;
    /**
     * Make a human smoke.
     *
     * @deprecated
     * @param request - The request {@link SmokeHumanRequest}
     * @returns A Promise of Human
     */
    smokeHuman: (request: Readonly<SmokeHumanRequest>) => Promise<Human>;
}

/** Lists transient statutes of the enum {@link HumanStatus}. */
declare const HUMAN_TRANSIENT_STATUSES: HumanStatus[];

type index_gen$5_CreateHumanRequest = CreateHumanRequest;
type index_gen$5_DeleteHumanRequest = DeleteHumanRequest;
type index_gen$5_EyeColors = EyeColors;
type index_gen$5_GetHumanRequest = GetHumanRequest;
declare const index_gen$5_HUMAN_TRANSIENT_STATUSES: typeof HUMAN_TRANSIENT_STATUSES;
type index_gen$5_Human = Human;
type index_gen$5_HumanStatus = HumanStatus;
type index_gen$5_ListHumansRequest = ListHumansRequest;
type index_gen$5_ListHumansRequestOrderBy = ListHumansRequestOrderBy;
type index_gen$5_ListHumansResponse = ListHumansResponse;
type index_gen$5_RegisterRequest = RegisterRequest;
type index_gen$5_RegisterResponse = RegisterResponse;
type index_gen$5_RunHumanRequest = RunHumanRequest;
type index_gen$5_SmokeHumanRequest = SmokeHumanRequest;
type index_gen$5_UpdateHumanRequest = UpdateHumanRequest;
declare namespace index_gen$5 {
  export { API$5 as API, type index_gen$5_CreateHumanRequest as CreateHumanRequest, type index_gen$5_DeleteHumanRequest as DeleteHumanRequest, type index_gen$5_EyeColors as EyeColors, type index_gen$5_GetHumanRequest as GetHumanRequest, index_gen$5_HUMAN_TRANSIENT_STATUSES as HUMAN_TRANSIENT_STATUSES, type index_gen$5_Human as Human, type index_gen$5_HumanStatus as HumanStatus, type index_gen$5_ListHumansRequest as ListHumansRequest, type index_gen$5_ListHumansRequestOrderBy as ListHumansRequestOrderBy, type index_gen$5_ListHumansResponse as ListHumansResponse, type index_gen$5_RegisterRequest as RegisterRequest, type index_gen$5_RegisterResponse as RegisterResponse, type index_gen$5_RunHumanRequest as RunHumanRequest, type index_gen$5_SmokeHumanRequest as SmokeHumanRequest, type index_gen$5_UpdateHumanRequest as UpdateHumanRequest };
}

declare namespace index$4 {
  export { index_gen$5 as v1 };
}

type DomainLastStatusRecordStatus = 'unknown_record_status' | 'valid' | 'invalid' | 'not_found';
type DomainReputationStatus = 'unknown_status' | 'excellent' | 'good' | 'average' | 'bad';
type DomainStatus = 'unknown' | 'checked' | 'unchecked' | 'invalid' | 'locked' | 'revoked' | 'pending';
type EmailFlag = 'unknown_flag' | 'soft_bounce' | 'hard_bounce' | 'spam' | 'mailbox_full' | 'mailbox_not_found' | 'greylisted' | 'send_before_expiration';
type EmailRcptType = 'unknown_rcpt_type' | 'to' | 'cc' | 'bcc';
type EmailStatus = 'unknown' | 'new' | 'sending' | 'sent' | 'failed' | 'canceled';
type ListEmailsRequestOrderBy = 'created_at_desc' | 'created_at_asc' | 'updated_at_desc' | 'updated_at_asc' | 'status_desc' | 'status_asc' | 'mail_from_desc' | 'mail_from_asc' | 'mail_rcpt_desc' | 'mail_rcpt_asc' | 'subject_desc' | 'subject_asc';
interface EmailTry {
    /** Rank number of this attempt to send the email. */
    rank: number;
    /** Date of the attempt to send the email. */
    triedAt?: Date;
    /**
     * The SMTP status code received after the attempt. 0 if the attempt did not
     * reach an SMTP server.
     */
    code: number;
    /**
     * The SMTP message received. If the attempt did not reach an SMTP server, the
     * message returned explains what happened.
     */
    message: string;
}
interface DomainReputation {
    /** Status of your domain's reputation. */
    status: DomainReputationStatus;
    /**
     * A range from 0 to 100 that determines your domain's reputation score. A
     * score of `0` means a bad domain reputation and a score of `100` means an
     * excellent domain reputation.
     */
    score: number;
    /** Time and date the score was calculated. */
    scoredAt?: Date;
    /** The previously-calculated domain's reputation score. */
    previousScore?: number;
    /** Time and date the previous reputation score was calculated. */
    previousScoredAt?: Date;
}
interface DomainStatistics {
    totalCount: number;
    sentCount: number;
    failedCount: number;
    canceledCount: number;
}
interface CreateEmailRequestAddress {
    /** Email address. */
    email: string;
    /** (Optional) Name displayed. */
    name?: string;
}
interface CreateEmailRequestAttachment {
    /** Filename of the attachment. */
    name: string;
    /** MIME type of the attachment. */
    type: string;
    /** Content of the attachment encoded in base64. */
    content: string;
}
interface CreateEmailRequestHeader {
    /** Email header key. */
    key: string;
    /** Email header value. */
    value: string;
}
interface Email {
    /** Technical ID of the email. */
    id: string;
    /** Message ID of the email. */
    messageId: string;
    /** ID of the Project to which the email belongs. */
    projectId: string;
    /** Email address of the sender. */
    mailFrom: string;
    /** @deprecated Email address of the recipient. */
    rcptTo?: string;
    /** Email address of the recipient. */
    mailRcpt: string;
    /** Type of recipient. */
    rcptType: EmailRcptType;
    /** Subject of the email. */
    subject: string;
    /** Creation date of the email object. */
    createdAt?: Date;
    /** Last update of the email object. */
    updatedAt?: Date;
    /** Status of the email. */
    status: EmailStatus;
    /** Additional status information. */
    statusDetails?: string;
    /** Number of attempts to send the email. */
    tryCount: number;
    /** Information about the last three attempts to send the email. */
    lastTries: EmailTry[];
    /**
     * Flags categorize emails. They allow you to obtain more information about
     * recurring errors, for example.
     */
    flags: EmailFlag[];
}
interface DomainLastStatusDkimRecord {
    /** Status of the DKIM record's configuration. */
    status: DomainLastStatusRecordStatus;
    /** Time and date the DKIM record was last valid. */
    lastValidAt?: Date;
    /** An error text displays in case the record is not valid. */
    error?: string;
}
interface DomainLastStatusDmarcRecord {
    /** Status of the DMARC record's configuration. */
    status: DomainLastStatusRecordStatus;
    /** Time and date the DMARC record was last valid. */
    lastValidAt?: Date;
    /** An error text displays in case the record is not valid. */
    error?: string;
}
interface DomainLastStatusSpfRecord {
    /** Status of the SPF record's configuration. */
    status: DomainLastStatusRecordStatus;
    /** Time and date the SPF record was last valid. */
    lastValidAt?: Date;
    /** An error text displays in case the record is not valid. */
    error?: string;
}
interface Domain {
    /** ID of the domain. */
    id: string;
    /** ID of the domain's Organization. */
    organizationId: string;
    /** ID of the domain's Project. */
    projectId: string;
    /** Domain name (example.com). */
    name: string;
    /** Status of the domain. */
    status: DomainStatus;
    /** Date and time of domain creation. */
    createdAt?: Date;
    /** Date and time of the next scheduled check. */
    nextCheckAt?: Date;
    /** Date and time the domain was last valid. */
    lastValidAt?: Date;
    /** Date and time of the domain's deletion. */
    revokedAt?: Date;
    /** @deprecated Error message returned if the last check failed. */
    lastError?: string;
    /** Snippet of the SPF record to register in the DNS zone. */
    spfConfig: string;
    /** DKIM public key to record in the DNS zone. */
    dkimConfig: string;
    /** Domain's statistics. */
    statistics?: DomainStatistics;
    /**
     * The domain's reputation is available when your domain is checked and has
     * sent enough emails.
     */
    reputation?: DomainReputation;
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region: Region;
}
type CancelEmailRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the email to cancel. */
    emailId: string;
};
type CheckDomainRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the domain to check. */
    domainId: string;
};
type CreateDomainRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the project to which the domain belongs. */
    projectId?: string;
    /** Fully qualified domain dame. */
    domainName: string;
    /** Accept Scaleway's Terms of Service. */
    acceptTos: boolean;
};
type CreateEmailRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Sender information. Must be from a checked domain declared in the Project. */
    from?: CreateEmailRequestAddress;
    /** An array of the primary recipient's information. */
    to?: CreateEmailRequestAddress[];
    /** An array of the carbon copy recipient's information. */
    cc?: CreateEmailRequestAddress[];
    /** An array of the blind carbon copy recipient's information. */
    bcc?: CreateEmailRequestAddress[];
    /** Subject of the email. */
    subject: string;
    /** Text content. */
    text: string;
    /** HTML content. */
    html: string;
    /** ID of the Project in which to create the email. */
    projectId?: string;
    /** Array of attachments. */
    attachments?: CreateEmailRequestAttachment[];
    /** Maximum date to deliver the email. */
    sendBefore?: Date;
    /** Array of additional headers as key-value. */
    additionalHeaders?: CreateEmailRequestHeader[];
};
interface CreateEmailResponse {
    /** Single page of emails matching the requested criteria. */
    emails: Email[];
}
interface DomainLastStatus {
    /** The ID of the domain. */
    domainId: string;
    /** The domain name (example.com). */
    domainName: string;
    /** The SPF record verification data. */
    spfRecord?: DomainLastStatusSpfRecord;
    /** The DKIM record verification data. */
    dkimRecord?: DomainLastStatusDkimRecord;
    /** The DMARC record verification data. */
    dmarcRecord?: DomainLastStatusDmarcRecord;
}
type GetDomainLastStatusRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the domain to delete. */
    domainId: string;
};
type GetDomainRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the domain. */
    domainId: string;
};
type GetEmailRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the email to retrieve. */
    emailId: string;
};
type GetStatisticsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** (Optional) Number of emails for this Project. */
    projectId?: string;
    /**
     * (Optional) Number of emails sent from this domain (must be coherent with
     * the `project_id` and the `organization_id`).
     */
    domainId?: string;
    /** (Optional) Number of emails created after this date. */
    since?: Date;
    /** (Optional) Number of emails created before this date. */
    until?: Date;
    /** (Optional) Number of emails sent with this sender's email address. */
    mailFrom?: string;
};
type ListDomainsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Requested page number. Value must be greater or equal to 1. */
    page?: number;
    /** Page size. */
    pageSize?: number;
    projectId?: string;
    status?: DomainStatus[];
    organizationId?: string;
    name?: string;
};
interface ListDomainsResponse {
    /** Number of domains that match the request (without pagination). */
    totalCount: number;
    domains: Domain[];
}
type ListEmailsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    page?: number;
    pageSize?: number;
    /** (Optional) ID of the Project in which to list the emails. */
    projectId?: string;
    /** (Optional) ID of the domain for which to list the emails. */
    domainId?: string;
    /** (Optional) ID of the message for which to list the emails. */
    messageId?: string;
    /** (Optional) List emails created after this date. */
    since?: Date;
    /** (Optional) List emails created before this date. */
    until?: Date;
    /** (Optional) List emails sent with this sender's email address. */
    mailFrom?: string;
    /** @deprecated List emails sent to this recipient's email address. */
    mailTo?: string;
    /** (Optional) List emails sent to this recipient's email address. */
    mailRcpt?: string;
    /** (Optional) List emails with any of these statuses. */
    statuses?: EmailStatus[];
    /** (Optional) List emails with this subject. */
    subject?: string;
    /** (Optional) List emails by searching to all fields. */
    search?: string;
    /** (Optional) List emails corresponding to specific criteria. */
    orderBy?: ListEmailsRequestOrderBy;
    /** (Optional) List emails containing only specific flags. */
    flags?: EmailFlag[];
};
interface ListEmailsResponse {
    /** Number of emails matching the requested criteria. */
    totalCount: number;
    /** Single page of emails matching the requested criteria. */
    emails: Email[];
}
type RevokeDomainRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the domain to delete. */
    domainId: string;
};
interface Statistics {
    /** Total number of emails matching the requested criteria. */
    totalCount: number;
    /**
     * Number of emails still in the `new` transient state. This means emails
     * received from the API but not yet processed.
     */
    newCount: number;
    /**
     * Number of emails still in the `sending` transient state. This means emails
     * received from the API but not yet in their final status.
     */
    sendingCount: number;
    /**
     * Number of emails in the final `sent` state. This means emails that have
     * been delivered to the target mail system.
     */
    sentCount: number;
    /**
     * Number of emails in the final `failed` state. This means emails that have
     * been refused by the target mail system with a final error status.
     */
    failedCount: number;
    /**
     * Number of emails in the final `canceled` state. This means emails that have
     * been canceled upon request.
     */
    canceledCount: number;
}

/** Transactional Email API. */
declare class API$4 extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Send an email. You must specify the `region`, the sender and the
     * recipient's information and the `project_id` to send an email from a
     * checked domain. The subject of the email must contain at least 6
     * characters.
     *
     * @param request - The request {@link CreateEmailRequest}
     * @returns A Promise of CreateEmailResponse
     */
    createEmail: (request: Readonly<CreateEmailRequest>) => Promise<CreateEmailResponse>;
    /**
     * Get an email. Retrieve information about a specific email using the
     * `email_id` and `region` parameters.
     *
     * @param request - The request {@link GetEmailRequest}
     * @returns A Promise of Email
     */
    getEmail: (request: Readonly<GetEmailRequest>) => Promise<Email>;
    /**
     * Waits for {@link Email} to be in a final state.
     *
     * @param request - The request {@link GetEmailRequest}
     * @param options - The waiting options
     * @returns A Promise of Email
     */
    waitForEmail: (request: Readonly<GetEmailRequest>, options?: Readonly<WaitForOptions<Email>>) => Promise<Email>;
    protected pageOfListEmails: (request?: Readonly<ListEmailsRequest>) => Promise<ListEmailsResponse>;
    /**
     * List emails. Retrieve the list of emails sent from a specific domain or for
     * a specific Project or Organization. You must specify the `region`.
     *
     * @param request - The request {@link ListEmailsRequest}
     * @returns A Promise of ListEmailsResponse
     */
    listEmails: (request?: Readonly<ListEmailsRequest>) => Promise<ListEmailsResponse> & {
        all: () => Promise<Email[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Email[], void, void>;
    };
    /**
     * Email statuses. Get information on your emails' statuses.
     *
     * @param request - The request {@link GetStatisticsRequest}
     * @returns A Promise of Statistics
     */
    getStatistics: (request?: Readonly<GetStatisticsRequest>) => Promise<Statistics>;
    /**
     * Cancel an email. You can cancel the sending of an email if it has not been
     * sent yet. You must specify the `region` and the `email_id` of the email you
     * want to cancel.
     *
     * @param request - The request {@link CancelEmailRequest}
     * @returns A Promise of Email
     */
    cancelEmail: (request: Readonly<CancelEmailRequest>) => Promise<Email>;
    /**
     * Register a domain in a project. You must specify the `region`, `project_id`
     * and `domain_name` to register a domain in a specific Project.
     *
     * @param request - The request {@link CreateDomainRequest}
     * @returns A Promise of Domain
     */
    createDomain: (request: Readonly<CreateDomainRequest>) => Promise<Domain>;
    /**
     * Get information about a domain. Retrieve information about a specific
     * domain using the `region` and `domain_id` parameters. Monitor your domain's
     * reputation and improve **average** and **bad** reputation statuses, using
     * your domain's **Email activity** tab on the [Scaleway
     * console](https://console.scaleway.com/transactional-email/domains) to get a
     * more detailed report. Check out our [dedicated
     * documentation](https://www.scaleway.com/en/docs/managed-services/transactional-email/reference-content/understanding-tem-reputation-score/)
     * to improve your domain's reputation.
     *
     * @param request - The request {@link GetDomainRequest}
     * @returns A Promise of Domain
     */
    getDomain: (request: Readonly<GetDomainRequest>) => Promise<Domain>;
    /**
     * Waits for {@link Domain} to be in a final state.
     *
     * @param request - The request {@link GetDomainRequest}
     * @param options - The waiting options
     * @returns A Promise of Domain
     */
    waitForDomain: (request: Readonly<GetDomainRequest>, options?: Readonly<WaitForOptions<Domain>>) => Promise<Domain>;
    protected pageOfListDomains: (request?: Readonly<ListDomainsRequest>) => Promise<ListDomainsResponse>;
    /**
     * List domains. Retrieve domains in a specific Project or in a specific
     * Organization using the `region` parameter.
     *
     * @param request - The request {@link ListDomainsRequest}
     * @returns A Promise of ListDomainsResponse
     */
    listDomains: (request?: Readonly<ListDomainsRequest>) => Promise<ListDomainsResponse> & {
        all: () => Promise<Domain[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Domain[], void, void>;
    };
    /**
     * Delete a domain. You must specify the domain you want to delete by the
     * `region` and `domain_id`. Deleting a domain is permanent and cannot be
     * undone.
     *
     * @param request - The request {@link RevokeDomainRequest}
     * @returns A Promise of Domain
     */
    revokeDomain: (request: Readonly<RevokeDomainRequest>) => Promise<Domain>;
    /**
     * Domain DNS check. Perform an immediate DNS check of a domain using the
     * `region` and `domain_id` parameters.
     *
     * @param request - The request {@link CheckDomainRequest}
     * @returns A Promise of Domain
     */
    checkDomain: (request: Readonly<CheckDomainRequest>) => Promise<Domain>;
    /**
     * Display SPF and DKIM records status and potential errors. Display SPF and
     * DKIM records status and potential errors, including the found records to
     * make debugging easier.
     *
     * @param request - The request {@link GetDomainLastStatusRequest}
     * @returns A Promise of DomainLastStatus
     */
    getDomainLastStatus: (request: Readonly<GetDomainLastStatusRequest>) => Promise<DomainLastStatus>;
}

/** Lists transient statutes of the enum {@link DomainStatus}. */
declare const DOMAIN_TRANSIENT_STATUSES: DomainStatus[];
/** Lists transient statutes of the enum {@link EmailStatus}. */
declare const EMAIL_TRANSIENT_STATUSES: EmailStatus[];

type index_gen$4_CancelEmailRequest = CancelEmailRequest;
type index_gen$4_CheckDomainRequest = CheckDomainRequest;
type index_gen$4_CreateDomainRequest = CreateDomainRequest;
type index_gen$4_CreateEmailRequest = CreateEmailRequest;
type index_gen$4_CreateEmailRequestAddress = CreateEmailRequestAddress;
type index_gen$4_CreateEmailRequestAttachment = CreateEmailRequestAttachment;
type index_gen$4_CreateEmailRequestHeader = CreateEmailRequestHeader;
type index_gen$4_CreateEmailResponse = CreateEmailResponse;
declare const index_gen$4_DOMAIN_TRANSIENT_STATUSES: typeof DOMAIN_TRANSIENT_STATUSES;
type index_gen$4_Domain = Domain;
type index_gen$4_DomainLastStatus = DomainLastStatus;
type index_gen$4_DomainLastStatusDkimRecord = DomainLastStatusDkimRecord;
type index_gen$4_DomainLastStatusDmarcRecord = DomainLastStatusDmarcRecord;
type index_gen$4_DomainLastStatusRecordStatus = DomainLastStatusRecordStatus;
type index_gen$4_DomainLastStatusSpfRecord = DomainLastStatusSpfRecord;
type index_gen$4_DomainReputation = DomainReputation;
type index_gen$4_DomainReputationStatus = DomainReputationStatus;
type index_gen$4_DomainStatistics = DomainStatistics;
type index_gen$4_DomainStatus = DomainStatus;
declare const index_gen$4_EMAIL_TRANSIENT_STATUSES: typeof EMAIL_TRANSIENT_STATUSES;
type index_gen$4_Email = Email;
type index_gen$4_EmailFlag = EmailFlag;
type index_gen$4_EmailRcptType = EmailRcptType;
type index_gen$4_EmailStatus = EmailStatus;
type index_gen$4_EmailTry = EmailTry;
type index_gen$4_GetDomainLastStatusRequest = GetDomainLastStatusRequest;
type index_gen$4_GetDomainRequest = GetDomainRequest;
type index_gen$4_GetEmailRequest = GetEmailRequest;
type index_gen$4_GetStatisticsRequest = GetStatisticsRequest;
type index_gen$4_ListDomainsRequest = ListDomainsRequest;
type index_gen$4_ListDomainsResponse = ListDomainsResponse;
type index_gen$4_ListEmailsRequest = ListEmailsRequest;
type index_gen$4_ListEmailsRequestOrderBy = ListEmailsRequestOrderBy;
type index_gen$4_ListEmailsResponse = ListEmailsResponse;
type index_gen$4_RevokeDomainRequest = RevokeDomainRequest;
type index_gen$4_Statistics = Statistics;
declare namespace index_gen$4 {
  export { API$4 as API, type index_gen$4_CancelEmailRequest as CancelEmailRequest, type index_gen$4_CheckDomainRequest as CheckDomainRequest, type index_gen$4_CreateDomainRequest as CreateDomainRequest, type index_gen$4_CreateEmailRequest as CreateEmailRequest, type index_gen$4_CreateEmailRequestAddress as CreateEmailRequestAddress, type index_gen$4_CreateEmailRequestAttachment as CreateEmailRequestAttachment, type index_gen$4_CreateEmailRequestHeader as CreateEmailRequestHeader, type index_gen$4_CreateEmailResponse as CreateEmailResponse, index_gen$4_DOMAIN_TRANSIENT_STATUSES as DOMAIN_TRANSIENT_STATUSES, type index_gen$4_Domain as Domain, type index_gen$4_DomainLastStatus as DomainLastStatus, type index_gen$4_DomainLastStatusDkimRecord as DomainLastStatusDkimRecord, type index_gen$4_DomainLastStatusDmarcRecord as DomainLastStatusDmarcRecord, type index_gen$4_DomainLastStatusRecordStatus as DomainLastStatusRecordStatus, type index_gen$4_DomainLastStatusSpfRecord as DomainLastStatusSpfRecord, type index_gen$4_DomainReputation as DomainReputation, type index_gen$4_DomainReputationStatus as DomainReputationStatus, type index_gen$4_DomainStatistics as DomainStatistics, type index_gen$4_DomainStatus as DomainStatus, index_gen$4_EMAIL_TRANSIENT_STATUSES as EMAIL_TRANSIENT_STATUSES, type index_gen$4_Email as Email, type index_gen$4_EmailFlag as EmailFlag, type index_gen$4_EmailRcptType as EmailRcptType, type index_gen$4_EmailStatus as EmailStatus, type index_gen$4_EmailTry as EmailTry, type index_gen$4_GetDomainLastStatusRequest as GetDomainLastStatusRequest, type index_gen$4_GetDomainRequest as GetDomainRequest, type index_gen$4_GetEmailRequest as GetEmailRequest, type index_gen$4_GetStatisticsRequest as GetStatisticsRequest, type index_gen$4_ListDomainsRequest as ListDomainsRequest, type index_gen$4_ListDomainsResponse as ListDomainsResponse, type index_gen$4_ListEmailsRequest as ListEmailsRequest, type index_gen$4_ListEmailsRequestOrderBy as ListEmailsRequestOrderBy, type index_gen$4_ListEmailsResponse as ListEmailsResponse, type index_gen$4_RevokeDomainRequest as RevokeDomainRequest, type index_gen$4_Statistics as Statistics };
}

declare namespace index$3 {
  export { index_gen$4 as v1alpha1 };
}

type ListPrivateNetworksRequestOrderBy$1 = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
interface PrivateNetwork$1 {
    /** Private Network ID. */
    id: string;
    /** Private Network name. */
    name: string;
    /** Scaleway Organization the Private Network belongs to. */
    organizationId: string;
    /** Scaleway Project the Private Network belongs to. */
    projectId: string;
    /** Availability Zone in which the Private Network is available. */
    zone: Zone;
    /** Tags of the Private Network. */
    tags: string[];
    /** Date the Private Network was created. */
    createdAt?: Date;
    /** Date the Private Network was last modified. */
    updatedAt?: Date;
    /** Private Network subnets CIDR. */
    subnets: string[];
}
type CreatePrivateNetworkRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Name for the Private Network. */
    name?: string;
    /** Scaleway Project in which to create the Private Network. */
    projectId?: string;
    /** Tags for the Private Network. */
    tags?: string[];
    /** Private Network subnets CIDR. */
    subnets?: string[];
};
type DeletePrivateNetworkRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Private Network ID. */
    privateNetworkId: string;
};
type GetPrivateNetworkRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Private Network ID. */
    privateNetworkId: string;
};
type ListPrivateNetworksRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Sort order of the returned Private Networks. */
    orderBy?: ListPrivateNetworksRequestOrderBy$1;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Maximum number of Private Networks to return per page. */
    pageSize?: number;
    /**
     * Name to filter for. Only Private Networks with names containing this string
     * will be returned.
     */
    name?: string;
    /**
     * Tags to filter for. Only Private Networks with one or more matching tags
     * will be returned.
     */
    tags?: string[];
    /**
     * Organization ID to filter for. Only Private Networks belonging to this
     * Organization will be returned.
     */
    organizationId?: string;
    /**
     * Project ID to filter for. Only Private Networks belonging to this Project
     * will be returned.
     */
    projectId?: string;
    /**
     * Private Network IDs to filter for. Only Private Networks with one of these
     * IDs will be returned.
     */
    privateNetworkIds?: string[];
    /** Defines whether to include regional Private Networks in the response. */
    includeRegional?: boolean;
};
interface ListPrivateNetworksResponse$1 {
    privateNetworks: PrivateNetwork$1[];
    totalCount: number;
}
type UpdatePrivateNetworkRequest$1 = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Private Network ID. */
    privateNetworkId: string;
    /** Name of the private network. */
    name?: string;
    /** Tags for the Private Network. */
    tags?: string[];
    /** @deprecated Private Network subnets CIDR (deprecated). */
    subnets?: string[];
};

/** VPC API. */
declare class API$3 extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    protected pageOfListPrivateNetworks: (request?: Readonly<ListPrivateNetworksRequest$1>) => Promise<ListPrivateNetworksResponse$1>;
    /**
     * List Private Networks. List existing Private Networks in a specified
     * Availability Zone. By default, the Private Networks returned in the list
     * are ordered by creation date in ascending order, though this can be
     * modified via the order_by field.
     *
     * @param request - The request {@link ListPrivateNetworksRequest}
     * @returns A Promise of ListPrivateNetworksResponse
     */
    listPrivateNetworks: (request?: Readonly<ListPrivateNetworksRequest$1>) => Promise<ListPrivateNetworksResponse$1> & {
        all: () => Promise<PrivateNetwork$1[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<PrivateNetwork$1[], void, void>;
    };
    /**
     * Create a Private Network. Create a new Private Network. Once created, you
     * can attach Scaleway resources in the same Availability Zone.
     *
     * @param request - The request {@link CreatePrivateNetworkRequest}
     * @returns A Promise of PrivateNetwork
     */
    createPrivateNetwork: (request?: Readonly<CreatePrivateNetworkRequest$1>) => Promise<PrivateNetwork$1>;
    /**
     * Get a Private Network. Retrieve information about an existing Private
     * Network, specified by its Private Network ID. Its full details are returned
     * in the response object.
     *
     * @param request - The request {@link GetPrivateNetworkRequest}
     * @returns A Promise of PrivateNetwork
     */
    getPrivateNetwork: (request: Readonly<GetPrivateNetworkRequest$1>) => Promise<PrivateNetwork$1>;
    /**
     * Update Private Network. Update parameters (such as name or tags) of an
     * existing Private Network, specified by its Private Network ID.
     *
     * @param request - The request {@link UpdatePrivateNetworkRequest}
     * @returns A Promise of PrivateNetwork
     */
    updatePrivateNetwork: (request: Readonly<UpdatePrivateNetworkRequest$1>) => Promise<PrivateNetwork$1>;
    /**
     * Delete a Private Network. Delete an existing Private Network. Note that you
     * must first detach all resources from the network, in order to delete it.
     *
     * @param request - The request {@link DeletePrivateNetworkRequest}
     */
    deletePrivateNetwork: (request: Readonly<DeletePrivateNetworkRequest$1>) => Promise<void>;
}

declare namespace index_gen$3 {
  export { API$3 as API, type CreatePrivateNetworkRequest$1 as CreatePrivateNetworkRequest, type DeletePrivateNetworkRequest$1 as DeletePrivateNetworkRequest, type GetPrivateNetworkRequest$1 as GetPrivateNetworkRequest, type ListPrivateNetworksRequest$1 as ListPrivateNetworksRequest, type ListPrivateNetworksRequestOrderBy$1 as ListPrivateNetworksRequestOrderBy, type ListPrivateNetworksResponse$1 as ListPrivateNetworksResponse, type PrivateNetwork$1 as PrivateNetwork, type UpdatePrivateNetworkRequest$1 as UpdatePrivateNetworkRequest };
}

type ListPrivateNetworksRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
type ListVPCsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc';
interface Subnet {
    /** ID of the subnet. */
    id: string;
    /** Subnet creation date. */
    createdAt?: Date;
    /** Subnet last modification date. */
    updatedAt?: Date;
    /** Subnet CIDR. */
    subnet: string;
}
interface PrivateNetwork {
    /** Private Network ID. */
    id: string;
    /** Private Network name. */
    name: string;
    /** Scaleway Organization the Private Network belongs to. */
    organizationId: string;
    /** Scaleway Project the Private Network belongs to. */
    projectId: string;
    /** Region in which the Private Network is available. */
    region: Region;
    /** Tags of the Private Network. */
    tags: string[];
    /** Date the Private Network was created. */
    createdAt?: Date;
    /** Date the Private Network was last modified. */
    updatedAt?: Date;
    /** Private Network subnets. */
    subnets: Subnet[];
    /** VPC the Private Network belongs to. */
    vpcId: string;
    /** Defines whether managed DHCP is enabled for this Private Network. */
    dhcpEnabled: boolean;
}
interface VPC {
    /** VPC ID. */
    id: string;
    /** VPC name. */
    name: string;
    /** Scaleway Organization the VPC belongs to. */
    organizationId: string;
    /** Scaleway Project the VPC belongs to. */
    projectId: string;
    /** Region of the VPC. */
    region: Region;
    /** Tags for the VPC. */
    tags: string[];
    /** Defines whether the VPC is the default one for its Project. */
    isDefault: boolean;
    /** Date the VPC was created. */
    createdAt?: Date;
    /** Date the VPC was last modified. */
    updatedAt?: Date;
    /** Number of Private Networks within this VPC. */
    privateNetworkCount: number;
}
type AddSubnetsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Private Network ID. */
    privateNetworkId: string;
    /** Private Network subnets CIDR. */
    subnets?: string[];
};
interface AddSubnetsResponse {
    subnets: string[];
}
type CreatePrivateNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name for the Private Network. */
    name?: string;
    /** Scaleway Project in which to create the Private Network. */
    projectId?: string;
    /** Tags for the Private Network. */
    tags?: string[];
    /** Private Network subnets CIDR. */
    subnets?: string[];
    /** VPC in which to create the Private Network. */
    vpcId?: string;
};
type CreateVPCRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Name for the VPC. */
    name?: string;
    /** Scaleway Project in which to create the VPC. */
    projectId?: string;
    /** Tags for the VPC. */
    tags?: string[];
};
type DeletePrivateNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Private Network ID. */
    privateNetworkId: string;
};
type DeleteSubnetsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Private Network ID. */
    privateNetworkId: string;
    /** Private Network subnets CIDR. */
    subnets?: string[];
};
interface DeleteSubnetsResponse {
    subnets: string[];
}
type DeleteVPCRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** VPC ID. */
    vpcId: string;
};
type EnableDHCPRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Private Network ID. */
    privateNetworkId: string;
};
type GetPrivateNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Private Network ID. */
    privateNetworkId: string;
};
type GetVPCRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** VPC ID. */
    vpcId: string;
};
type ListPrivateNetworksRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Sort order of the returned Private Networks. */
    orderBy?: ListPrivateNetworksRequestOrderBy;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Maximum number of Private Networks to return per page. */
    pageSize?: number;
    /**
     * Name to filter for. Only Private Networks with names containing this string
     * will be returned.
     */
    name?: string;
    /**
     * Tags to filter for. Only Private Networks with one or more matching tags
     * will be returned.
     */
    tags?: string[];
    /**
     * Organization ID to filter for. Only Private Networks belonging to this
     * Organization will be returned.
     */
    organizationId?: string;
    /**
     * Project ID to filter for. Only Private Networks belonging to this Project
     * will be returned.
     */
    projectId?: string;
    /**
     * Private Network IDs to filter for. Only Private Networks with one of these
     * IDs will be returned.
     */
    privateNetworkIds?: string[];
    /**
     * VPC ID to filter for. Only Private Networks belonging to this VPC will be
     * returned.
     */
    vpcId?: string;
    /**
     * DHCP status to filter for. When true, only Private Networks with managed
     * DHCP enabled will be returned.
     */
    dhcpEnabled?: boolean;
};
interface ListPrivateNetworksResponse {
    privateNetworks: PrivateNetwork[];
    totalCount: number;
}
type ListVPCsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Sort order of the returned VPCs. */
    orderBy?: ListVPCsRequestOrderBy;
    /** Page number to return, from the paginated results. */
    page?: number;
    /** Maximum number of VPCs to return per page. */
    pageSize?: number;
    /**
     * Name to filter for. Only VPCs with names containing this string will be
     * returned.
     */
    name?: string;
    /**
     * Tags to filter for. Only VPCs with one more more matching tags will be
     * returned.
     */
    tags?: string[];
    /**
     * Organization ID to filter for. Only VPCs belonging to this Organization
     * will be returned.
     */
    organizationId?: string;
    /**
     * Project ID to filter for. Only VPCs belonging to this Project will be
     * returned.
     */
    projectId?: string;
    /**
     * Defines whether to filter only for VPCs which are the default one for their
     * Project.
     */
    isDefault?: boolean;
};
interface ListVPCsResponse {
    vpcs: VPC[];
    totalCount: number;
}
type MigrateZonalPrivateNetworksRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * Organization ID to target. The specified zoned Private Networks within this
     * Organization will be migrated to regional.
     *
     * One-of ('scope'): at most one of 'organizationId', 'projectId' could be
     * set.
     */
    organizationId?: string;
    /**
     * Project to target. The specified zoned Private Networks within this Project
     * will be migrated to regional.
     *
     * One-of ('scope'): at most one of 'organizationId', 'projectId' could be
     * set.
     */
    projectId?: string;
    /** IDs of the Private Networks to migrate. */
    privateNetworkIds?: string[];
};
type SetSubnetsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Private Network ID. */
    privateNetworkId: string;
    /** Private Network subnets CIDR. */
    subnets?: string[];
};
interface SetSubnetsResponse {
    subnets: string[];
}
type UpdatePrivateNetworkRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Private Network ID. */
    privateNetworkId: string;
    /** Name for the Private Network. */
    name?: string;
    /** Tags for the Private Network. */
    tags?: string[];
};
type UpdateVPCRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** VPC ID. */
    vpcId: string;
    /** Name for the VPC. */
    name?: string;
    /** Tags for the VPC. */
    tags?: string[];
};

/** VPC API. */
declare class API$2 extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    protected pageOfListVPCs: (request?: Readonly<ListVPCsRequest>) => Promise<ListVPCsResponse>;
    /**
     * List VPCs. List existing VPCs in the specified region.
     *
     * @param request - The request {@link ListVPCsRequest}
     * @returns A Promise of ListVPCsResponse
     */
    listVPCs: (request?: Readonly<ListVPCsRequest>) => Promise<ListVPCsResponse> & {
        all: () => Promise<VPC[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<VPC[], void, void>;
    };
    /**
     * Create a VPC. Create a new VPC in the specified region.
     *
     * @param request - The request {@link CreateVPCRequest}
     * @returns A Promise of VPC
     */
    createVPC: (request?: Readonly<CreateVPCRequest>) => Promise<VPC>;
    /**
     * Get a VPC. Retrieve details of an existing VPC, specified by its VPC ID.
     *
     * @param request - The request {@link GetVPCRequest}
     * @returns A Promise of VPC
     */
    getVPC: (request: Readonly<GetVPCRequest>) => Promise<VPC>;
    /**
     * Update VPC. Update parameters including name and tags of the specified VPC.
     *
     * @param request - The request {@link UpdateVPCRequest}
     * @returns A Promise of VPC
     */
    updateVPC: (request: Readonly<UpdateVPCRequest>) => Promise<VPC>;
    /**
     * Delete a VPC. Delete a VPC specified by its VPC ID.
     *
     * @param request - The request {@link DeleteVPCRequest}
     */
    deleteVPC: (request: Readonly<DeleteVPCRequest>) => Promise<void>;
    protected pageOfListPrivateNetworks: (request?: Readonly<ListPrivateNetworksRequest>) => Promise<ListPrivateNetworksResponse>;
    /**
     * List Private Networks. List existing Private Networks in the specified
     * region. By default, the Private Networks returned in the list are ordered
     * by creation date in ascending order, though this can be modified via the
     * order_by field.
     *
     * @param request - The request {@link ListPrivateNetworksRequest}
     * @returns A Promise of ListPrivateNetworksResponse
     */
    listPrivateNetworks: (request?: Readonly<ListPrivateNetworksRequest>) => Promise<ListPrivateNetworksResponse> & {
        all: () => Promise<PrivateNetwork[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<PrivateNetwork[], void, void>;
    };
    /**
     * Create a Private Network. Create a new Private Network. Once created, you
     * can attach Scaleway resources which are in the same region.
     *
     * @param request - The request {@link CreatePrivateNetworkRequest}
     * @returns A Promise of PrivateNetwork
     */
    createPrivateNetwork: (request?: Readonly<CreatePrivateNetworkRequest>) => Promise<PrivateNetwork>;
    /**
     * Get a Private Network. Retrieve information about an existing Private
     * Network, specified by its Private Network ID. Its full details are returned
     * in the response object.
     *
     * @param request - The request {@link GetPrivateNetworkRequest}
     * @returns A Promise of PrivateNetwork
     */
    getPrivateNetwork: (request: Readonly<GetPrivateNetworkRequest>) => Promise<PrivateNetwork>;
    /**
     * Update Private Network. Update parameters (such as name or tags) of an
     * existing Private Network, specified by its Private Network ID.
     *
     * @param request - The request {@link UpdatePrivateNetworkRequest}
     * @returns A Promise of PrivateNetwork
     */
    updatePrivateNetwork: (request: Readonly<UpdatePrivateNetworkRequest>) => Promise<PrivateNetwork>;
    /**
     * Delete a Private Network. Delete an existing Private Network. Note that you
     * must first detach all resources from the network, in order to delete it.
     *
     * @param request - The request {@link DeletePrivateNetworkRequest}
     */
    deletePrivateNetwork: (request: Readonly<DeletePrivateNetworkRequest>) => Promise<void>;
    /**
     * Migrate Private Networks from zoned to regional. Transform multiple
     * existing zoned Private Networks (scoped to a single Availability Zone) into
     * regional Private Networks, scoped to an entire region. You can transform
     * one or many Private Networks (specified by their Private Network IDs)
     * within a single Scaleway Organization or Project, with the same call.
     *
     * @param request - The request {@link MigrateZonalPrivateNetworksRequest}
     */
    migrateZonalPrivateNetworks: (request?: Readonly<MigrateZonalPrivateNetworksRequest>) => Promise<void>;
    /**
     * Enable DHCP on a Private Network. Enable DHCP managed on an existing
     * Private Network. Note that you will not be able to deactivate it
     * afterwards.
     *
     * @param request - The request {@link EnableDHCPRequest}
     * @returns A Promise of PrivateNetwork
     */
    enableDHCP: (request: Readonly<EnableDHCPRequest>) => Promise<PrivateNetwork>;
    /**
     * Set the subnets of a Private Network. Set subnets for an existing Private
     * Network. Note that the method is PUT and not PATCH. Any existing subnets
     * will be removed in favor of the new specified set of subnets.
     *
     * @param request - The request {@link SetSubnetsRequest}
     * @returns A Promise of SetSubnetsResponse
     */
    setSubnets: (request: Readonly<SetSubnetsRequest>) => Promise<SetSubnetsResponse>;
    /**
     * Add subnets to a Private Network. Add new subnets to an existing Private
     * Network.
     *
     * @param request - The request {@link AddSubnetsRequest}
     * @returns A Promise of AddSubnetsResponse
     */
    addSubnets: (request: Readonly<AddSubnetsRequest>) => Promise<AddSubnetsResponse>;
    /**
     * Delete subnets from a Private Network. Delete the specified subnets from a
     * Private Network.
     *
     * @param request - The request {@link DeleteSubnetsRequest}
     * @returns A Promise of DeleteSubnetsResponse
     */
    deleteSubnets: (request: Readonly<DeleteSubnetsRequest>) => Promise<DeleteSubnetsResponse>;
}

type index_gen$2_AddSubnetsRequest = AddSubnetsRequest;
type index_gen$2_AddSubnetsResponse = AddSubnetsResponse;
type index_gen$2_CreatePrivateNetworkRequest = CreatePrivateNetworkRequest;
type index_gen$2_CreateVPCRequest = CreateVPCRequest;
type index_gen$2_DeletePrivateNetworkRequest = DeletePrivateNetworkRequest;
type index_gen$2_DeleteSubnetsRequest = DeleteSubnetsRequest;
type index_gen$2_DeleteSubnetsResponse = DeleteSubnetsResponse;
type index_gen$2_DeleteVPCRequest = DeleteVPCRequest;
type index_gen$2_EnableDHCPRequest = EnableDHCPRequest;
type index_gen$2_GetPrivateNetworkRequest = GetPrivateNetworkRequest;
type index_gen$2_GetVPCRequest = GetVPCRequest;
type index_gen$2_ListPrivateNetworksRequest = ListPrivateNetworksRequest;
type index_gen$2_ListPrivateNetworksRequestOrderBy = ListPrivateNetworksRequestOrderBy;
type index_gen$2_ListPrivateNetworksResponse = ListPrivateNetworksResponse;
type index_gen$2_ListVPCsRequest = ListVPCsRequest;
type index_gen$2_ListVPCsRequestOrderBy = ListVPCsRequestOrderBy;
type index_gen$2_ListVPCsResponse = ListVPCsResponse;
type index_gen$2_MigrateZonalPrivateNetworksRequest = MigrateZonalPrivateNetworksRequest;
type index_gen$2_PrivateNetwork = PrivateNetwork;
type index_gen$2_SetSubnetsRequest = SetSubnetsRequest;
type index_gen$2_SetSubnetsResponse = SetSubnetsResponse;
type index_gen$2_Subnet = Subnet;
type index_gen$2_UpdatePrivateNetworkRequest = UpdatePrivateNetworkRequest;
type index_gen$2_UpdateVPCRequest = UpdateVPCRequest;
type index_gen$2_VPC = VPC;
declare namespace index_gen$2 {
  export { API$2 as API, type index_gen$2_AddSubnetsRequest as AddSubnetsRequest, type index_gen$2_AddSubnetsResponse as AddSubnetsResponse, type index_gen$2_CreatePrivateNetworkRequest as CreatePrivateNetworkRequest, type index_gen$2_CreateVPCRequest as CreateVPCRequest, type index_gen$2_DeletePrivateNetworkRequest as DeletePrivateNetworkRequest, type index_gen$2_DeleteSubnetsRequest as DeleteSubnetsRequest, type index_gen$2_DeleteSubnetsResponse as DeleteSubnetsResponse, type index_gen$2_DeleteVPCRequest as DeleteVPCRequest, type index_gen$2_EnableDHCPRequest as EnableDHCPRequest, type index_gen$2_GetPrivateNetworkRequest as GetPrivateNetworkRequest, type index_gen$2_GetVPCRequest as GetVPCRequest, type index_gen$2_ListPrivateNetworksRequest as ListPrivateNetworksRequest, type index_gen$2_ListPrivateNetworksRequestOrderBy as ListPrivateNetworksRequestOrderBy, type index_gen$2_ListPrivateNetworksResponse as ListPrivateNetworksResponse, type index_gen$2_ListVPCsRequest as ListVPCsRequest, type index_gen$2_ListVPCsRequestOrderBy as ListVPCsRequestOrderBy, type index_gen$2_ListVPCsResponse as ListVPCsResponse, type index_gen$2_MigrateZonalPrivateNetworksRequest as MigrateZonalPrivateNetworksRequest, type index_gen$2_PrivateNetwork as PrivateNetwork, type index_gen$2_SetSubnetsRequest as SetSubnetsRequest, type index_gen$2_SetSubnetsResponse as SetSubnetsResponse, type index_gen$2_Subnet as Subnet, type index_gen$2_UpdatePrivateNetworkRequest as UpdatePrivateNetworkRequest, type index_gen$2_UpdateVPCRequest as UpdateVPCRequest, type index_gen$2_VPC as VPC };
}

declare namespace index$2 {
  export { index_gen$3 as v1, index_gen$2 as v2 };
}

type DHCPEntryType = 'unknown' | 'reservation' | 'lease';
type GatewayNetworkStatus = 'unknown' | 'created' | 'attaching' | 'configuring' | 'ready' | 'detaching' | 'deleted';
type GatewayStatus = 'unknown' | 'stopped' | 'allocating' | 'configuring' | 'running' | 'stopping' | 'failed' | 'deleting' | 'deleted' | 'locked';
type ListDHCPEntriesRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'ip_address_asc' | 'ip_address_desc' | 'hostname_asc' | 'hostname_desc';
type ListDHCPsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'subnet_asc' | 'subnet_desc';
type ListGatewayNetworksRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'status_asc' | 'status_desc';
type ListGatewaysRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'name_asc' | 'name_desc' | 'type_asc' | 'type_desc' | 'status_asc' | 'status_desc';
type ListIPsRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'ip_asc' | 'ip_desc' | 'reverse_asc' | 'reverse_desc';
type ListPATRulesRequestOrderBy = 'created_at_asc' | 'created_at_desc' | 'public_port_asc' | 'public_port_desc';
type PATRuleProtocol = 'unknown' | 'both' | 'tcp' | 'udp';
interface DHCP {
    /** ID of the DHCP config. */
    id: string;
    /** Owning Organization. */
    organizationId: string;
    /** Owning Project. */
    projectId: string;
    /** Date the DHCP configuration was created. */
    createdAt?: Date;
    /** Configuration last modification date. */
    updatedAt?: Date;
    /** Subnet for the DHCP server. */
    subnet: string;
    /**
     * IP address of the DHCP server. This will be the Public Gateway's address in
     * the Private Network. It must be part of config's subnet.
     */
    address: string;
    /**
     * Low IP (inclusive) of the dynamic address pool. Must be in the config's
     * subnet.
     */
    poolLow: string;
    /**
     * High IP (inclusive) of the dynamic address pool. Must be in the config's
     * subnet.
     */
    poolHigh: string;
    /**
     * Defines whether to enable dynamic pooling of IPs. When false, only
     * pre-existing DHCP reservations will be handed out.
     */
    enableDynamic: boolean;
    /** How long DHCP entries will be valid for. */
    validLifetime?: string;
    /**
     * After how long a renew will be attempted. Must be 30s lower than
     * `rebind_timer`.
     */
    renewTimer?: string;
    /**
     * After how long a DHCP client will query for a new lease if previous renews
     * fail. Must be 30s lower than `valid_lifetime`.
     */
    rebindTimer?: string;
    /**
     * Defines whether the gateway should push a default route to DHCP clients, or
     * only hand out IPs.
     */
    pushDefaultRoute: boolean;
    /**
     * Defines whether the gateway should push custom DNS servers to clients. This
     * allows for instance hostname -> IP resolution.
     */
    pushDnsServer: boolean;
    /**
     * Array of DNS server IP addresses used to override the DNS server list
     * pushed to DHCP clients, instead of the gateway itself.
     */
    dnsServersOverride: string[];
    /** Array of search paths in addition to the pushed DNS configuration. */
    dnsSearch: string[];
    /**
     * TLD given to hostnames in the Private Networks. If an Instance with
     * hostname `foo` gets a lease, and this is set to `bar`, `foo.bar` will
     * resolve.
     */
    dnsLocalName: string;
    /** Zone of this DHCP configuration. */
    zone: Zone;
}
interface IpamConfig {
    /** Defines whether the default route is enabled on that Gateway Network. */
    pushDefaultRoute: boolean;
    /** IPAM-booked IP ID as the Gateway's IP in this Private Network. */
    ipamIpId: string;
}
interface GatewayNetwork {
    /** ID of the Public Gateway-Private Network connection. */
    id: string;
    /** Connection creation date. */
    createdAt?: Date;
    /** Connection last modification date. */
    updatedAt?: Date;
    /** ID of the connected Public Gateway. */
    gatewayId: string;
    /** ID of the connected Private Network. */
    privateNetworkId: string;
    /**
     * MAC address of the gateway in the Private Network (if the gateway is up and
     * running).
     */
    macAddress?: string;
    /**
     * Defines whether the gateway masquerades traffic for this Private Network
     * (Dynamic NAT).
     */
    enableMasquerade: boolean;
    /** Current status of the Public Gateway's connection to the Private Network. */
    status: GatewayNetworkStatus;
    /** DHCP configuration for the connected Private Network. */
    dhcp?: DHCP;
    /** Defines whether DHCP is enabled on the connected Private Network. */
    enableDhcp: boolean;
    /** Address of the Gateway (in CIDR form) to use when DHCP is not used. */
    address?: string;
    /** IPAM IP configuration used. */
    ipamConfig?: IpamConfig;
    /** Zone of the GatewayNetwork connection. */
    zone: Zone;
}
interface GatewayType {
    /** Public Gateway type name. */
    name: string;
    /**
     * Bandwidth, in bps, of the Public Gateway. This is the public bandwidth to
     * the outer Internet, and the internal bandwidth to each connected Private
     * Networks.
     */
    bandwidth: number;
    /** Zone the Public Gateway type is available in. */
    zone: Zone;
}
interface IP {
    /** IP address ID. */
    id: string;
    /** Owning Organization. */
    organizationId: string;
    /** Owning Project. */
    projectId: string;
    /** IP address creation date. */
    createdAt?: Date;
    /** IP address last modification date. */
    updatedAt?: Date;
    /** Tags associated with the IP address. */
    tags: string[];
    /** The IP address itself. */
    address: string;
    /** Reverse domain name for the IP address. */
    reverse?: string;
    /** Public Gateway associated with the IP address. */
    gatewayId?: string;
    /** Zone of the IP address. */
    zone: Zone;
}
type CreateDHCPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Project to create the DHCP configuration in. */
    projectId?: string;
    /** Subnet for the DHCP server. */
    subnet: string;
    /**
     * IP address of the DHCP server. This will be the gateway's address in the
     * Private Network. Defaults to the first address of the subnet.
     */
    address?: string;
    /**
     * Low IP (inclusive) of the dynamic address pool. Must be in the config's
     * subnet. Defaults to the second address of the subnet.
     */
    poolLow?: string;
    /**
     * High IP (inclusive) of the dynamic address pool. Must be in the config's
     * subnet. Defaults to the last address of the subnet.
     */
    poolHigh?: string;
    /**
     * Defines whether to enable dynamic pooling of IPs. When false, only
     * pre-existing DHCP reservations will be handed out. Defaults to true.
     */
    enableDynamic?: boolean;
    /** How long DHCP entries will be valid for. Defaults to 1h (3600s). */
    validLifetime?: string;
    /**
     * After how long a renew will be attempted. Must be 30s lower than
     * `rebind_timer`. Defaults to 50m (3000s).
     */
    renewTimer?: string;
    /**
     * After how long a DHCP client will query for a new lease if previous renews
     * fail. Must be 30s lower than `valid_lifetime`. Defaults to 51m (3060s).
     */
    rebindTimer?: string;
    /**
     * Defines whether the gateway should push a default route to DHCP clients or
     * only hand out IPs. Defaults to true.
     */
    pushDefaultRoute?: boolean;
    /**
     * Defines whether the gateway should push custom DNS servers to clients. This
     * allows for Instance hostname -> IP resolution. Defaults to true.
     */
    pushDnsServer?: boolean;
    /**
     * Array of DNS server IP addresses used to override the DNS server list
     * pushed to DHCP clients, instead of the gateway itself.
     */
    dnsServersOverride?: string[];
    /** Array of search paths in addition to the pushed DNS configuration. */
    dnsSearch?: string[];
    /**
     * TLD given to hostnames in the Private Network. Allowed characters are
     * `a-z0-9-.`. Defaults to the slugified Private Network name if created along
     * a GatewayNetwork, or else to `priv`.
     */
    dnsLocalName?: string;
};
interface CreateGatewayNetworkRequestIpamConfig {
    /** Enabling the default route also enables masquerading. */
    pushDefaultRoute: boolean;
    /** Use this IPAM-booked IP ID as the Gateway's IP in this Private Network. */
    ipamIpId?: string;
}
interface DHCPEntry {
    /** DHCP entry ID. */
    id: string;
    /** DHCP entry creation date. */
    createdAt?: Date;
    /** DHCP entry last modification date. */
    updatedAt?: Date;
    /** Owning GatewayNetwork. */
    gatewayNetworkId: string;
    /** MAC address of the client device. */
    macAddress: string;
    /** Assigned IP address. */
    ipAddress: string;
    /** Hostname of the client device. */
    hostname: string;
    /** Entry type, either static (DHCP reservation) or dynamic (DHCP lease). */
    type: DHCPEntryType;
    /** Zone of this DHCP entry. */
    zone: Zone;
}
interface Gateway {
    /** ID of the gateway. */
    id: string;
    /** Owning Organization. */
    organizationId: string;
    /** Owning Project. */
    projectId: string;
    /** Gateway creation date. */
    createdAt?: Date;
    /** Gateway last modification date. */
    updatedAt?: Date;
    /** Gateway type (commercial offer). */
    type?: GatewayType;
    /** Current status of the gateway. */
    status: GatewayStatus;
    /** Name of the gateway. */
    name: string;
    /** Tags associated with the gateway. */
    tags: string[];
    /** Public IP address of the gateway. */
    ip?: IP;
    /**
     * GatewayNetwork objects attached to the gateway (each one represents a
     * connection to a Private Network).
     */
    gatewayNetworks: GatewayNetwork[];
    /**
     * Array of DNS server IP addresses to override the gateway's default
     * recursive DNS servers.
     */
    upstreamDnsServers: string[];
    /** Version of the running gateway software. */
    version?: string;
    /** Newly available gateway software version that can be updated to. */
    canUpgradeTo?: string;
    /** Defines whether SSH bastion is enabled on the gateway. */
    bastionEnabled: boolean;
    /** Port of the SSH bastion. */
    bastionPort: number;
    /** Defines whether SMTP traffic is allowed to pass through the gateway. */
    smtpEnabled: boolean;
    /** Whether this uses non-IPAM IP configurations. */
    isLegacy: boolean;
    /** Zone of the gateway. */
    zone: Zone;
}
interface PATRule {
    /** PAT rule ID. */
    id: string;
    /** Gateway the PAT rule applies to. */
    gatewayId: string;
    /** PAT rule creation date. */
    createdAt?: Date;
    /** PAT rule last modification date. */
    updatedAt?: Date;
    /** Public port to listen on. */
    publicPort: number;
    /** Private IP address to forward data to. */
    privateIp: string;
    /** Private port to translate to. */
    privatePort: number;
    /** Protocol the rule applies to. */
    protocol: PATRuleProtocol;
    /** Zone of the PAT rule. */
    zone: Zone;
}
interface SetDHCPEntriesRequestEntry {
    /**
     * MAC address to give a static entry to. A matching entry will be upgraded to
     * a reservation, and a matching reservation will be updated.
     */
    macAddress: string;
    /** IP address to give to the device. */
    ipAddress: string;
}
interface SetPATRulesRequestRule {
    /**
     * Public port to listen on. Uniquely identifies the rule, and a matching rule
     * will be updated with the new parameters.
     */
    publicPort: number;
    /** Private IP to forward data to. */
    privateIp: string;
    /** Private port to translate to. */
    privatePort: number;
    /** Protocol the rule should apply to. */
    protocol: PATRuleProtocol;
}
interface UpdateGatewayNetworkRequestIpamConfig {
    /** Enabling the default route also enables masquerading. */
    pushDefaultRoute?: boolean;
    /** Use this IPAM-booked IP ID as the Gateway's IP in this Private Network. */
    ipamIpId?: string;
}
type CreateDHCPEntryRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** GatewayNetwork on which to create a DHCP reservation. */
    gatewayNetworkId: string;
    /** MAC address to give a static entry to. */
    macAddress: string;
    /** IP address to give to the device. */
    ipAddress: string;
};
type CreateGatewayNetworkRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Public Gateway to connect. */
    gatewayId: string;
    /** Private Network to connect. */
    privateNetworkId: string;
    /** Note: this setting is ignored when passing `ipam_config`. */
    enableMasquerade: boolean;
    /**
     * Defaults to `true` if either `dhcp_id` or `dhcp` are present. If set to
     * `true`, either `dhcp_id` or `dhcp` must be present. Note: this setting is
     * ignored when passing `ipam_config`.
     */
    enableDhcp?: boolean;
    /**
     * ID of an existing DHCP configuration object to use for this GatewayNetwork.
     *
     * One-of ('ipConfig'): at most one of 'dhcpId', 'dhcp', 'address',
     * 'ipamConfig' could be set.
     */
    dhcpId?: string;
    /**
     * New DHCP configuration object to use for this GatewayNetwork.
     *
     * One-of ('ipConfig'): at most one of 'dhcpId', 'dhcp', 'address',
     * 'ipamConfig' could be set.
     */
    dhcp?: CreateDHCPRequest;
    /**
     * Static IP address in CIDR format to to use without DHCP.
     *
     * One-of ('ipConfig'): at most one of 'dhcpId', 'dhcp', 'address',
     * 'ipamConfig' could be set.
     */
    address?: string;
    /**
     * Note: all or none of the GatewayNetworks for a single gateway can use the
     * IPAM. DHCP and IPAM configurations cannot be mixed. Some products may
     * require that the Public Gateway uses the IPAM, to ensure correct
     * functionality.
     *
     * One-of ('ipConfig'): at most one of 'dhcpId', 'dhcp', 'address',
     * 'ipamConfig' could be set.
     */
    ipamConfig?: CreateGatewayNetworkRequestIpamConfig;
};
type CreateGatewayRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Scaleway Project to create the gateway in. */
    projectId?: string;
    /** Name for the gateway. */
    name?: string;
    /** Tags for the gateway. */
    tags?: string[];
    /** Gateway type (commercial offer type). */
    type: string;
    /**
     * Array of DNS server IP addresses to override the gateway's default
     * recursive DNS servers.
     */
    upstreamDnsServers?: string[];
    /** Existing IP address to attach to the gateway. */
    ipId?: string;
    /** Defines whether SMTP traffic should be allowed pass through the gateway. */
    enableSmtp: boolean;
    /** Defines whether SSH bastion should be enabled the gateway. */
    enableBastion: boolean;
    /** Port of the SSH bastion. */
    bastionPort?: number;
};
type CreateIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Project to create the IP address in. */
    projectId?: string;
    /** Tags to give to the IP address. */
    tags?: string[];
};
type CreatePATRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the Gateway on which to create the rule. */
    gatewayId: string;
    /** Public port to listen on. */
    publicPort: number;
    /** Private IP to forward data to. */
    privateIp: string;
    /** Private port to translate to. */
    privatePort: number;
    /** Protocol the rule should apply to. */
    protocol?: PATRuleProtocol;
};
type DeleteDHCPEntryRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the DHCP entry to delete. */
    dhcpEntryId: string;
};
type DeleteDHCPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** DHCP configuration ID to delete. */
    dhcpId: string;
};
type DeleteGatewayNetworkRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the GatewayNetwork to delete. */
    gatewayNetworkId: string;
    /**
     * Defines whether to clean up attached DHCP configurations (if any, and if
     * not attached to another Gateway Network).
     */
    cleanupDhcp: boolean;
};
type DeleteGatewayRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the gateway to delete. */
    gatewayId: string;
    /**
     * Defines whether to clean up attached DHCP configurations (if any, and if
     * not attached to another Gateway Network).
     */
    cleanupDhcp: boolean;
};
type DeleteIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the IP address to delete. */
    ipId: string;
};
type DeletePATRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the PAT rule to delete. */
    patRuleId: string;
};
type GetDHCPEntryRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the DHCP entry to fetch. */
    dhcpEntryId: string;
};
type GetDHCPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the DHCP configuration to fetch. */
    dhcpId: string;
};
type GetGatewayNetworkRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the GatewayNetwork to fetch. */
    gatewayNetworkId: string;
};
type GetGatewayRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the gateway to fetch. */
    gatewayId: string;
};
type GetIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the IP address to get. */
    ipId: string;
};
type GetPATRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the PAT rule to get. */
    patRuleId: string;
};
type ListDHCPEntriesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Order in which to return results. */
    orderBy?: ListDHCPEntriesRequestOrderBy;
    /** Page number. */
    page?: number;
    /** DHCP entries per page. */
    pageSize?: number;
    /** Filter for entries on this GatewayNetwork. */
    gatewayNetworkId?: string;
    /** Filter for entries with this MAC address. */
    macAddress?: string;
    /** Filter for entries with this IP address. */
    ipAddress?: string;
    /** Filter for entries with this hostname substring. */
    hostname?: string;
    /** Filter for entries of this type. */
    type?: DHCPEntryType;
};
interface ListDHCPEntriesResponse {
    /** DHCP entries in this page. */
    dhcpEntries: DHCPEntry[];
    /** Total count of DHCP entries matching the filter. */
    totalCount: number;
}
type ListDHCPsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Order in which to return results. */
    orderBy?: ListDHCPsRequestOrderBy;
    /** Page number. */
    page?: number;
    /** DHCP configurations per page. */
    pageSize?: number;
    /** Include only DHCP configuration objects in this Organization. */
    organizationId?: string;
    /** Include only DHCP configuration objects in this Project. */
    projectId?: string;
    /**
     * Filter for DHCP configuration objects with this DHCP server IP address (the
     * gateway's address in the Private Network).
     */
    address?: string;
    /**
     * Filter for DHCP configuration objects with subnets containing this IP
     * address.
     */
    hasAddress?: string;
};
interface ListDHCPsResponse {
    /** First page of DHCP configuration objects. */
    dhcps: DHCP[];
    /** Total count of DHCP configuration objects matching the filter. */
    totalCount: number;
}
type ListGatewayNetworksRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Order in which to return results. */
    orderBy?: ListGatewayNetworksRequestOrderBy;
    /** Page number. */
    page?: number;
    /** GatewayNetworks per page. */
    pageSize?: number;
    /** Filter for GatewayNetworks connected to this gateway. */
    gatewayId?: string;
    /** Filter for GatewayNetworks connected to this Private Network. */
    privateNetworkId?: string;
    /** Filter for GatewayNetworks with this `enable_masquerade` setting. */
    enableMasquerade?: boolean;
    /** Filter for GatewayNetworks using this DHCP configuration. */
    dhcpId?: string;
    /**
     * Filter for GatewayNetworks with this current status this status. Use
     * `unknown` to include all statuses.
     */
    status?: GatewayNetworkStatus;
};
interface ListGatewayNetworksResponse {
    /** GatewayNetworks on this page. */
    gatewayNetworks: GatewayNetwork[];
    /** Total GatewayNetworks count matching the filter. */
    totalCount: number;
}
type ListGatewayTypesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
};
interface ListGatewayTypesResponse {
    /** Available types of Public Gateway. */
    types: GatewayType[];
}
type ListGatewaysRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Order in which to return results. */
    orderBy?: ListGatewaysRequestOrderBy;
    /** Page number to return. */
    page?: number;
    /** Gateways per page. */
    pageSize?: number;
    /** Include only gateways in this Organization. */
    organizationId?: string;
    /** Include only gateways in this Project. */
    projectId?: string;
    /** Filter for gateways which have this search term in their name. */
    name?: string;
    /** Filter for gateways with these tags. */
    tags?: string[];
    /** Filter for gateways of this type. */
    type?: string;
    /**
     * Filter for gateways with this current status. Use `unknown` to include all
     * statuses.
     */
    status?: GatewayStatus;
    /** Filter for gateways attached to this Private nNetwork. */
    privateNetworkId?: string;
};
interface ListGatewaysResponse {
    /** Gateways on this page. */
    gateways: Gateway[];
    /** Total count of gateways matching the filter. */
    totalCount: number;
}
type ListIPsRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Order in which to return results. */
    orderBy?: ListIPsRequestOrderBy;
    /** Page number. */
    page?: number;
    /** IP addresses per page. */
    pageSize?: number;
    /** Filter for IP addresses in this Organization. */
    organizationId?: string;
    /** Filter for IP addresses in this Project. */
    projectId?: string;
    /** Filter for IP addresses with these tags. */
    tags?: string[];
    /** Filter for IP addresses that have a reverse containing this string. */
    reverse?: string;
    /** Filter based on whether the IP is attached to a gateway or not. */
    isFree?: boolean;
};
interface ListIPsResponse {
    /** IP addresses on this page. */
    ips: IP[];
    /** Total count of IP addresses matching the filter. */
    totalCount: number;
}
type ListPATRulesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** Order in which to return results. */
    orderBy?: ListPATRulesRequestOrderBy;
    /** Page number. */
    page?: number;
    /** PAT rules per page. */
    pageSize?: number;
    /** Filter for PAT rules on this Gateway. */
    gatewayId?: string;
    /** Filter for PAT rules targeting this private ip. */
    privateIp?: string;
    /** Filter for PAT rules with this protocol. */
    protocol?: PATRuleProtocol;
};
interface ListPATRulesResponse {
    /** Array of PAT rules matching the filter. */
    patRules: PATRule[];
    /** Total count of PAT rules matching the filter. */
    totalCount: number;
}
type RefreshSSHKeysRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the gateway to refresh SSH keys on. */
    gatewayId: string;
};
type SetDHCPEntriesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the Gateway Network on which to set DHCP reservation list. */
    gatewayNetworkId: string;
    /** New list of DHCP reservations. */
    dhcpEntries?: SetDHCPEntriesRequestEntry[];
};
interface SetDHCPEntriesResponse {
    /** List of DHCP entries. */
    dhcpEntries: DHCPEntry[];
}
type SetPATRulesRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the gateway on which to set the PAT rules. */
    gatewayId: string;
    /** New list of PAT rules. */
    patRules: SetPATRulesRequestRule[];
};
interface SetPATRulesResponse {
    /** List of PAT rules. */
    patRules: PATRule[];
}
type UpdateDHCPEntryRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the DHCP entry to update. */
    dhcpEntryId: string;
    /** New IP address to give to the device. */
    ipAddress?: string;
};
type UpdateDHCPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** DHCP configuration to update. */
    dhcpId: string;
    /** Subnet for the DHCP server. */
    subnet?: string;
    /**
     * IP address of the DHCP server. This will be the Public Gateway's address in
     * the Private Network. It must be part of config's subnet.
     */
    address?: string;
    /**
     * Low IP (inclusive) of the dynamic address pool. Must be in the config's
     * subnet.
     */
    poolLow?: string;
    /**
     * High IP (inclusive) of the dynamic address pool. Must be in the config's
     * subnet.
     */
    poolHigh?: string;
    /**
     * Defines whether to enable dynamic pooling of IPs. When false, only
     * pre-existing DHCP reservations will be handed out. Defaults to true.
     */
    enableDynamic?: boolean;
    /** How long DHCP entries will be valid for. */
    validLifetime?: string;
    /**
     * After how long a renew will be attempted. Must be 30s lower than
     * `rebind_timer`.
     */
    renewTimer?: string;
    /**
     * After how long a DHCP client will query for a new lease if previous renews
     * fail. Must be 30s lower than `valid_lifetime`.
     */
    rebindTimer?: string;
    /**
     * Defines whether the gateway should push a default route to DHCP clients, or
     * only hand out IPs.
     */
    pushDefaultRoute?: boolean;
    /**
     * Defines whether the gateway should push custom DNS servers to clients. This
     * allows for instance hostname -> IP resolution.
     */
    pushDnsServer?: boolean;
    /**
     * Array of DNS server IP addresses used to override the DNS server list
     * pushed to DHCP clients, instead of the gateway itself.
     */
    dnsServersOverride?: string[];
    /** Array of search paths in addition to the pushed DNS configuration. */
    dnsSearch?: string[];
    /**
     * TLD given to hostnames in the Private Networks. If an instance with
     * hostname `foo` gets a lease, and this is set to `bar`, `foo.bar` will
     * resolve. Allowed characters are `a-z0-9-.`.
     */
    dnsLocalName?: string;
};
type UpdateGatewayNetworkRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the GatewayNetwork to update. */
    gatewayNetworkId: string;
    /** Note: this setting is ignored when passing `ipam_config`. */
    enableMasquerade?: boolean;
    /**
     * Defaults to `true` if `dhcp_id` is present. If set to `true`, `dhcp_id`
     * must be present. Note: this setting is ignored when passing `ipam_config`.
     */
    enableDhcp?: boolean;
    /**
     * ID of the new DHCP configuration object to use with this GatewayNetwork.
     *
     * One-of ('ipConfig'): at most one of 'dhcpId', 'address', 'ipamConfig' could
     * be set.
     */
    dhcpId?: string;
    /**
     * New static IP address.
     *
     * One-of ('ipConfig'): at most one of 'dhcpId', 'address', 'ipamConfig' could
     * be set.
     */
    address?: string;
    /**
     * Note: all or none of the GatewayNetworks for a single gateway can use the
     * IPAM. DHCP and IPAM configurations cannot be mixed. Some products may
     * require that the Public Gateway uses the IPAM, to ensure correct
     * functionality.
     *
     * One-of ('ipConfig'): at most one of 'dhcpId', 'address', 'ipamConfig' could
     * be set.
     */
    ipamConfig?: UpdateGatewayNetworkRequestIpamConfig;
};
type UpdateGatewayRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the gateway to update. */
    gatewayId: string;
    /** Name for the gateway. */
    name?: string;
    /** Tags for the gateway. */
    tags?: string[];
    /**
     * Array of DNS server IP addresses to override the gateway's default
     * recursive DNS servers.
     */
    upstreamDnsServers?: string[];
    /** Defines whether SSH bastion should be enabled the gateway. */
    enableBastion?: boolean;
    /** Port of the SSH bastion. */
    bastionPort?: number;
    /** Defines whether SMTP traffic should be allowed to pass through the gateway. */
    enableSmtp?: boolean;
};
type UpdateIPRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the IP address to update. */
    ipId: string;
    /** Tags to give to the IP address. */
    tags?: string[];
    /** Reverse to set on the address. Empty string to unset. */
    reverse?: string;
    /** Gateway to attach the IP address to. Empty string to detach. */
    gatewayId?: string;
};
type UpdatePATRuleRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the PAT rule to update. */
    patRuleId: string;
    /** Public port to listen on. */
    publicPort?: number;
    /** Private IP to forward data to. */
    privateIp?: string;
    /** Private port to translate to. */
    privatePort?: number;
    /** Protocol the rule should apply to. */
    protocol?: PATRuleProtocol;
};
type UpgradeGatewayRequest = {
    /** Zone to target. If none is passed will use default zone from the config. */
    zone?: Zone;
    /** ID of the gateway to upgrade. */
    gatewayId: string;
};

/** Public Gateways API. */
declare class API$1 extends API$x {
    /** Lists the available zones of the API. */
    static readonly LOCALITIES: Zone[];
    protected pageOfListGateways: (request?: Readonly<ListGatewaysRequest>) => Promise<ListGatewaysResponse>;
    /**
     * List Public Gateways. List Public Gateways in a given Scaleway Organization
     * or Project. By default, results are displayed in ascending order of
     * creation date.
     *
     * @param request - The request {@link ListGatewaysRequest}
     * @returns A Promise of ListGatewaysResponse
     */
    listGateways: (request?: Readonly<ListGatewaysRequest>) => Promise<ListGatewaysResponse> & {
        all: () => Promise<Gateway[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Gateway[], void, void>;
    };
    /**
     * Get a Public Gateway. Get details of a Public Gateway, specified by its
     * gateway ID. The response object contains full details of the gateway,
     * including its **name**, **type**, **status** and more.
     *
     * @param request - The request {@link GetGatewayRequest}
     * @returns A Promise of Gateway
     */
    getGateway: (request: Readonly<GetGatewayRequest>) => Promise<Gateway>;
    /**
     * Waits for {@link Gateway} to be in a final state.
     *
     * @param request - The request {@link GetGatewayRequest}
     * @param options - The waiting options
     * @returns A Promise of Gateway
     */
    waitForGateway: (request: Readonly<GetGatewayRequest>, options?: Readonly<WaitForOptions<Gateway>>) => Promise<Gateway>;
    /**
     * Create a Public Gateway. Create a new Public Gateway in the specified
     * Scaleway Project, defining its **name**, **type** and other configuration
     * details such as whether to enable SSH bastion.
     *
     * @param request - The request {@link CreateGatewayRequest}
     * @returns A Promise of Gateway
     */
    createGateway: (request: Readonly<CreateGatewayRequest>) => Promise<Gateway>;
    /**
     * Update a Public Gateway. Update the parameters of an existing Public
     * Gateway, for example, its **name**, **tags**, **SSH bastion
     * configuration**, and **DNS servers**.
     *
     * @param request - The request {@link UpdateGatewayRequest}
     * @returns A Promise of Gateway
     */
    updateGateway: (request: Readonly<UpdateGatewayRequest>) => Promise<Gateway>;
    /**
     * Delete a Public Gateway. Delete an existing Public Gateway, specified by
     * its gateway ID. This action is irreversible.
     *
     * @param request - The request {@link DeleteGatewayRequest}
     */
    deleteGateway: (request: Readonly<DeleteGatewayRequest>) => Promise<void>;
    /**
     * Upgrade a Public Gateway to the latest version. Upgrade a given Public
     * Gateway to the newest software version. This applies the latest bugfixes
     * and features to your Public Gateway, but its service will be interrupted
     * during the update.
     *
     * @param request - The request {@link UpgradeGatewayRequest}
     * @returns A Promise of Gateway
     */
    upgradeGateway: (request: Readonly<UpgradeGatewayRequest>) => Promise<Gateway>;
    protected pageOfListGatewayNetworks: (request?: Readonly<ListGatewayNetworksRequest>) => Promise<ListGatewayNetworksResponse>;
    /**
     * List Public Gateway connections to Private Networks. List the connections
     * between Public Gateways and Private Networks (a connection = a
     * GatewayNetwork). You can choose to filter by `gateway-id` to list all
     * Private Networks attached to the specified Public Gateway, or by
     * `private_network_id` to list all Public Gateways attached to the specified
     * Private Network. Other query parameters are also available. The result is
     * an array of GatewayNetwork objects, each giving details of the connection
     * between a given Public Gateway and a given Private Network.
     *
     * @param request - The request {@link ListGatewayNetworksRequest}
     * @returns A Promise of ListGatewayNetworksResponse
     */
    listGatewayNetworks: (request?: Readonly<ListGatewayNetworksRequest>) => Promise<ListGatewayNetworksResponse> & {
        all: () => Promise<GatewayNetwork[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<GatewayNetwork[], void, void>;
    };
    /**
     * Get a Public Gateway connection to a Private Network. Get details of a
     * given connection between a Public Gateway and a Private Network (this
     * connection = a GatewayNetwork), specified by its `gateway_network_id`. The
     * response object contains details of the connection including the IDs of the
     * Public Gateway and Private Network, the dates the connection was
     * created/updated and its configuration settings.
     *
     * @param request - The request {@link GetGatewayNetworkRequest}
     * @returns A Promise of GatewayNetwork
     */
    getGatewayNetwork: (request: Readonly<GetGatewayNetworkRequest>) => Promise<GatewayNetwork>;
    /**
     * Waits for {@link GatewayNetwork} to be in a final state.
     *
     * @param request - The request {@link GetGatewayNetworkRequest}
     * @param options - The waiting options
     * @returns A Promise of GatewayNetwork
     */
    waitForGatewayNetwork: (request: Readonly<GetGatewayNetworkRequest>, options?: Readonly<WaitForOptions<GatewayNetwork>>) => Promise<GatewayNetwork>;
    /**
     * Attach a Public Gateway to a Private Network. Attach a specific Public
     * Gateway to a specific Private Network (create a GatewayNetwork). You can
     * configure parameters for the connection including DHCP settings, whether to
     * enable masquerade (dynamic NAT), and more.
     *
     * @param request - The request {@link CreateGatewayNetworkRequest}
     * @returns A Promise of GatewayNetwork
     */
    createGatewayNetwork: (request: Readonly<CreateGatewayNetworkRequest>) => Promise<GatewayNetwork>;
    /**
     * Update a Public Gateway's connection to a Private Network. Update the
     * configuration parameters of a connection between a given Public Gateway and
     * Private Network (the connection = a GatewayNetwork). Updatable parameters
     * include DHCP settings and whether to enable traffic masquerade (dynamic
     * NAT).
     *
     * @param request - The request {@link UpdateGatewayNetworkRequest}
     * @returns A Promise of GatewayNetwork
     */
    updateGatewayNetwork: (request: Readonly<UpdateGatewayNetworkRequest>) => Promise<GatewayNetwork>;
    /**
     * Detach a Public Gateway from a Private Network. Detach a given Public
     * Gateway from a given Private Network, i.e. delete a GatewayNetwork
     * specified by a gateway_network_id.
     *
     * @param request - The request {@link DeleteGatewayNetworkRequest}
     */
    deleteGatewayNetwork: (request: Readonly<DeleteGatewayNetworkRequest>) => Promise<void>;
    protected pageOfListDHCPs: (request?: Readonly<ListDHCPsRequest>) => Promise<ListDHCPsResponse>;
    /**
     * List DHCP configurations. List DHCP configurations, optionally filtering by
     * Organization, Project, Public Gateway IP address or more. The response is
     * an array of DHCP configuration objects, each identified by a DHCP ID and
     * containing configuration settings for the assignment of IP addresses to
     * devices on a Private Network attached to a Public Gateway. Note that the
     * response does not contain the IDs of any Private Network / Public Gateway
     * the configuration is attached to. Use the `List Public Gateway connections
     * to Private Networks` method for that purpose, filtering on DHCP ID.
     *
     * @param request - The request {@link ListDHCPsRequest}
     * @returns A Promise of ListDHCPsResponse
     */
    listDHCPs: (request?: Readonly<ListDHCPsRequest>) => Promise<ListDHCPsResponse> & {
        all: () => Promise<DHCP[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DHCP[], void, void>;
    };
    /**
     * Get a DHCP configuration. Get a DHCP configuration object, identified by
     * its DHCP ID. The response object contains configuration settings for the
     * assignment of IP addresses to devices on a Private Network attached to a
     * Public Gateway. Note that the response does not contain the IDs of any
     * Private Network / Public Gateway the configuration is attached to. Use the
     * `List Public Gateway connections to Private Networks` method for that
     * purpose, filtering on DHCP ID.
     *
     * @param request - The request {@link GetDHCPRequest}
     * @returns A Promise of DHCP
     */
    getDHCP: (request: Readonly<GetDHCPRequest>) => Promise<DHCP>;
    /**
     * Create a DHCP configuration. Create a new DHCP configuration object,
     * containing settings for the assignment of IP addresses to devices on a
     * Private Network attached to a Public Gateway. The response object includes
     * the ID of the DHCP configuration object. You can use this ID as part of a
     * call to `Create a Public Gateway connection to a Private Network` or
     * `Update a Public Gateway connection to a Private Network` to directly apply
     * this DHCP configuration.
     *
     * @param request - The request {@link CreateDHCPRequest}
     * @returns A Promise of DHCP
     */
    createDHCP: (request: Readonly<CreateDHCPRequest>) => Promise<DHCP>;
    /**
     * Update a DHCP configuration. Update a DHCP configuration object, identified
     * by its DHCP ID.
     *
     * @param request - The request {@link UpdateDHCPRequest}
     * @returns A Promise of DHCP
     */
    updateDHCP: (request: Readonly<UpdateDHCPRequest>) => Promise<DHCP>;
    /**
     * Delete a DHCP configuration. Delete a DHCP configuration object, identified
     * by its DHCP ID. Note that you cannot delete a DHCP configuration object
     * that is currently being used by a Gateway Network.
     *
     * @param request - The request {@link DeleteDHCPRequest}
     */
    deleteDHCP: (request: Readonly<DeleteDHCPRequest>) => Promise<void>;
    protected pageOfListDHCPEntries: (request?: Readonly<ListDHCPEntriesRequest>) => Promise<ListDHCPEntriesResponse>;
    /**
     * List DHCP entries. List DHCP entries, whether dynamically assigned and/or
     * statically reserved. DHCP entries can be filtered by the Gateway Network
     * they are on, their MAC address, IP address, type or hostname.
     *
     * @param request - The request {@link ListDHCPEntriesRequest}
     * @returns A Promise of ListDHCPEntriesResponse
     */
    listDHCPEntries: (request?: Readonly<ListDHCPEntriesRequest>) => Promise<ListDHCPEntriesResponse> & {
        all: () => Promise<DHCPEntry[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<DHCPEntry[], void, void>;
    };
    /**
     * Get a DHCP entry. Get a DHCP entry, specified by its DHCP entry ID.
     *
     * @param request - The request {@link GetDHCPEntryRequest}
     * @returns A Promise of DHCPEntry
     */
    getDHCPEntry: (request: Readonly<GetDHCPEntryRequest>) => Promise<DHCPEntry>;
    /**
     * Create a DHCP entry. Create a static DHCP reservation, specifying the
     * Gateway Network for the reservation, the MAC address of the target device
     * and the IP address to assign this device. The response is a DHCP entry
     * object, confirming the ID and configuration details of the static DHCP
     * reservation.
     *
     * @param request - The request {@link CreateDHCPEntryRequest}
     * @returns A Promise of DHCPEntry
     */
    createDHCPEntry: (request: Readonly<CreateDHCPEntryRequest>) => Promise<DHCPEntry>;
    /**
     * Update a DHCP entry. Update the IP address for a DHCP entry, specified by
     * its DHCP entry ID. You can update an existing DHCP entry of any type
     * (`reservation` (static), `lease` (dynamic) or `unknown`), but in manually
     * updating the IP address the entry will necessarily be of type `reservation`
     * after the update.
     *
     * @param request - The request {@link UpdateDHCPEntryRequest}
     * @returns A Promise of DHCPEntry
     */
    updateDHCPEntry: (request: Readonly<UpdateDHCPEntryRequest>) => Promise<DHCPEntry>;
    /**
     * Set all DHCP reservations on a Gateway Network. Set the list of DHCP
     * reservations attached to a Gateway Network. Reservations are identified by
     * their MAC address, and will sync the current DHCP entry list to the given
     * list, creating, updating or deleting DHCP entries accordingly.
     *
     * @param request - The request {@link SetDHCPEntriesRequest}
     * @returns A Promise of SetDHCPEntriesResponse
     */
    setDHCPEntries: (request: Readonly<SetDHCPEntriesRequest>) => Promise<SetDHCPEntriesResponse>;
    /**
     * Delete a DHCP entry. Delete a static DHCP reservation, identified by its
     * DHCP entry ID. Note that you cannot delete DHCP entries of type `lease`,
     * these are deleted automatically when their time-to-live expires.
     *
     * @param request - The request {@link DeleteDHCPEntryRequest}
     */
    deleteDHCPEntry: (request: Readonly<DeleteDHCPEntryRequest>) => Promise<void>;
    protected pageOfListPATRules: (request?: Readonly<ListPATRulesRequest>) => Promise<ListPATRulesResponse>;
    /**
     * List PAT rules. List PAT rules. You can filter by gateway ID to list all
     * PAT rules for a particular gateway, or filter for PAT rules targeting a
     * specific IP address or using a specific protocol.
     *
     * @param request - The request {@link ListPATRulesRequest}
     * @returns A Promise of ListPATRulesResponse
     */
    listPATRules: (request?: Readonly<ListPATRulesRequest>) => Promise<ListPATRulesResponse> & {
        all: () => Promise<PATRule[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<PATRule[], void, void>;
    };
    /**
     * Get a PAT rule. Get a PAT rule, specified by its PAT rule ID. The response
     * object gives full details of the PAT rule, including the Public Gateway it
     * belongs to and the configuration settings in terms of public / private
     * ports, private IP and protocol.
     *
     * @param request - The request {@link GetPATRuleRequest}
     * @returns A Promise of PATRule
     */
    getPATRule: (request: Readonly<GetPATRuleRequest>) => Promise<PATRule>;
    /**
     * Create a PAT rule. Create a new PAT rule on a specified Public Gateway,
     * defining the protocol to use, public port to listen on, and private port /
     * IP address to map to.
     *
     * @param request - The request {@link CreatePATRuleRequest}
     * @returns A Promise of PATRule
     */
    createPATRule: (request: Readonly<CreatePATRuleRequest>) => Promise<PATRule>;
    /**
     * Update a PAT rule. Update a PAT rule, specified by its PAT rule ID.
     * Configuration settings including private/public port, private IP address
     * and protocol can all be updated.
     *
     * @param request - The request {@link UpdatePATRuleRequest}
     * @returns A Promise of PATRule
     */
    updatePATRule: (request: Readonly<UpdatePATRuleRequest>) => Promise<PATRule>;
    /**
     * Set all PAT rules. Set a definitive list of PAT rules attached to a Public
     * Gateway. Each rule is identified by its public port and protocol. This will
     * sync the current PAT rule list on the gateway with the new list, creating,
     * updating or deleting PAT rules accordingly.
     *
     * @param request - The request {@link SetPATRulesRequest}
     * @returns A Promise of SetPATRulesResponse
     */
    setPATRules: (request: Readonly<SetPATRulesRequest>) => Promise<SetPATRulesResponse>;
    /**
     * Delete a PAT rule. Delete a PAT rule, identified by its PAT rule ID. This
     * action is irreversible.
     *
     * @param request - The request {@link DeletePATRuleRequest}
     */
    deletePATRule: (request: Readonly<DeletePATRuleRequest>) => Promise<void>;
    /**
     * List Public Gateway types. List the different Public Gateway commercial
     * offer types available at Scaleway. The response is an array of objects
     * describing the name and technical details of each available gateway type.
     *
     * @param request - The request {@link ListGatewayTypesRequest}
     * @returns A Promise of ListGatewayTypesResponse
     */
    listGatewayTypes: (request?: Readonly<ListGatewayTypesRequest>) => Promise<ListGatewayTypesResponse>;
    protected pageOfListIPs: (request?: Readonly<ListIPsRequest>) => Promise<ListIPsResponse>;
    /**
     * List IPs. List Public Gateway flexible IP addresses. A number of filter
     * options are available for limiting results in the response.
     *
     * @param request - The request {@link ListIPsRequest}
     * @returns A Promise of ListIPsResponse
     */
    listIPs: (request?: Readonly<ListIPsRequest>) => Promise<ListIPsResponse> & {
        all: () => Promise<IP[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<IP[], void, void>;
    };
    /**
     * Get an IP. Get details of a Public Gateway flexible IP address, identified
     * by its IP ID. The response object contains information including which (if
     * any) Public Gateway using this IP address, the reverse and various other
     * metadata.
     *
     * @param request - The request {@link GetIPRequest}
     * @returns A Promise of IP
     */
    getIP: (request: Readonly<GetIPRequest>) => Promise<IP>;
    /**
     * Reserve an IP. Create (reserve) a new flexible IP address that can be used
     * for a Public Gateway in a specified Scaleway Project.
     *
     * @param request - The request {@link CreateIPRequest}
     * @returns A Promise of IP
     */
    createIP: (request?: Readonly<CreateIPRequest>) => Promise<IP>;
    /**
     * Update an IP. Update details of an existing flexible IP address, including
     * its tags, reverse and the Public Gateway it is assigned to.
     *
     * @param request - The request {@link UpdateIPRequest}
     * @returns A Promise of IP
     */
    updateIP: (request: Readonly<UpdateIPRequest>) => Promise<IP>;
    /**
     * Delete an IP. Delete a flexible IP address from your account. This action
     * is irreversible.
     *
     * @param request - The request {@link DeleteIPRequest}
     */
    deleteIP: (request: Readonly<DeleteIPRequest>) => Promise<void>;
    /**
     * Refresh a Public Gateway's SSH keys. Refresh the SSH keys of a given Public
     * Gateway, specified by its gateway ID. This adds any new SSH keys in the
     * gateway's Scaleway Project to the gateway itself.
     *
     * @param request - The request {@link RefreshSSHKeysRequest}
     * @returns A Promise of Gateway
     */
    refreshSSHKeys: (request: Readonly<RefreshSSHKeysRequest>) => Promise<Gateway>;
}

/** Lists transient statutes of the enum {@link GatewayNetworkStatus}. */
declare const GATEWAY_NETWORK_TRANSIENT_STATUSES: GatewayNetworkStatus[];
/** Lists transient statutes of the enum {@link GatewayStatus}. */
declare const GATEWAY_TRANSIENT_STATUSES: GatewayStatus[];

type index_gen$1_CreateDHCPEntryRequest = CreateDHCPEntryRequest;
type index_gen$1_CreateDHCPRequest = CreateDHCPRequest;
type index_gen$1_CreateGatewayNetworkRequest = CreateGatewayNetworkRequest;
type index_gen$1_CreateGatewayNetworkRequestIpamConfig = CreateGatewayNetworkRequestIpamConfig;
type index_gen$1_CreateGatewayRequest = CreateGatewayRequest;
type index_gen$1_CreateIPRequest = CreateIPRequest;
type index_gen$1_CreatePATRuleRequest = CreatePATRuleRequest;
type index_gen$1_DHCP = DHCP;
type index_gen$1_DHCPEntry = DHCPEntry;
type index_gen$1_DHCPEntryType = DHCPEntryType;
type index_gen$1_DeleteDHCPEntryRequest = DeleteDHCPEntryRequest;
type index_gen$1_DeleteDHCPRequest = DeleteDHCPRequest;
type index_gen$1_DeleteGatewayNetworkRequest = DeleteGatewayNetworkRequest;
type index_gen$1_DeleteGatewayRequest = DeleteGatewayRequest;
type index_gen$1_DeleteIPRequest = DeleteIPRequest;
type index_gen$1_DeletePATRuleRequest = DeletePATRuleRequest;
declare const index_gen$1_GATEWAY_NETWORK_TRANSIENT_STATUSES: typeof GATEWAY_NETWORK_TRANSIENT_STATUSES;
declare const index_gen$1_GATEWAY_TRANSIENT_STATUSES: typeof GATEWAY_TRANSIENT_STATUSES;
type index_gen$1_Gateway = Gateway;
type index_gen$1_GatewayNetwork = GatewayNetwork;
type index_gen$1_GatewayNetworkStatus = GatewayNetworkStatus;
type index_gen$1_GatewayStatus = GatewayStatus;
type index_gen$1_GatewayType = GatewayType;
type index_gen$1_GetDHCPEntryRequest = GetDHCPEntryRequest;
type index_gen$1_GetDHCPRequest = GetDHCPRequest;
type index_gen$1_GetGatewayNetworkRequest = GetGatewayNetworkRequest;
type index_gen$1_GetGatewayRequest = GetGatewayRequest;
type index_gen$1_GetIPRequest = GetIPRequest;
type index_gen$1_GetPATRuleRequest = GetPATRuleRequest;
type index_gen$1_IP = IP;
type index_gen$1_IpamConfig = IpamConfig;
type index_gen$1_ListDHCPEntriesRequest = ListDHCPEntriesRequest;
type index_gen$1_ListDHCPEntriesRequestOrderBy = ListDHCPEntriesRequestOrderBy;
type index_gen$1_ListDHCPEntriesResponse = ListDHCPEntriesResponse;
type index_gen$1_ListDHCPsRequest = ListDHCPsRequest;
type index_gen$1_ListDHCPsRequestOrderBy = ListDHCPsRequestOrderBy;
type index_gen$1_ListDHCPsResponse = ListDHCPsResponse;
type index_gen$1_ListGatewayNetworksRequest = ListGatewayNetworksRequest;
type index_gen$1_ListGatewayNetworksRequestOrderBy = ListGatewayNetworksRequestOrderBy;
type index_gen$1_ListGatewayNetworksResponse = ListGatewayNetworksResponse;
type index_gen$1_ListGatewayTypesRequest = ListGatewayTypesRequest;
type index_gen$1_ListGatewayTypesResponse = ListGatewayTypesResponse;
type index_gen$1_ListGatewaysRequest = ListGatewaysRequest;
type index_gen$1_ListGatewaysRequestOrderBy = ListGatewaysRequestOrderBy;
type index_gen$1_ListGatewaysResponse = ListGatewaysResponse;
type index_gen$1_ListIPsRequest = ListIPsRequest;
type index_gen$1_ListIPsRequestOrderBy = ListIPsRequestOrderBy;
type index_gen$1_ListIPsResponse = ListIPsResponse;
type index_gen$1_ListPATRulesRequest = ListPATRulesRequest;
type index_gen$1_ListPATRulesRequestOrderBy = ListPATRulesRequestOrderBy;
type index_gen$1_ListPATRulesResponse = ListPATRulesResponse;
type index_gen$1_PATRule = PATRule;
type index_gen$1_PATRuleProtocol = PATRuleProtocol;
type index_gen$1_RefreshSSHKeysRequest = RefreshSSHKeysRequest;
type index_gen$1_SetDHCPEntriesRequest = SetDHCPEntriesRequest;
type index_gen$1_SetDHCPEntriesRequestEntry = SetDHCPEntriesRequestEntry;
type index_gen$1_SetDHCPEntriesResponse = SetDHCPEntriesResponse;
type index_gen$1_SetPATRulesRequest = SetPATRulesRequest;
type index_gen$1_SetPATRulesRequestRule = SetPATRulesRequestRule;
type index_gen$1_SetPATRulesResponse = SetPATRulesResponse;
type index_gen$1_UpdateDHCPEntryRequest = UpdateDHCPEntryRequest;
type index_gen$1_UpdateDHCPRequest = UpdateDHCPRequest;
type index_gen$1_UpdateGatewayNetworkRequest = UpdateGatewayNetworkRequest;
type index_gen$1_UpdateGatewayNetworkRequestIpamConfig = UpdateGatewayNetworkRequestIpamConfig;
type index_gen$1_UpdateGatewayRequest = UpdateGatewayRequest;
type index_gen$1_UpdateIPRequest = UpdateIPRequest;
type index_gen$1_UpdatePATRuleRequest = UpdatePATRuleRequest;
type index_gen$1_UpgradeGatewayRequest = UpgradeGatewayRequest;
declare namespace index_gen$1 {
  export { API$1 as API, type index_gen$1_CreateDHCPEntryRequest as CreateDHCPEntryRequest, type index_gen$1_CreateDHCPRequest as CreateDHCPRequest, type index_gen$1_CreateGatewayNetworkRequest as CreateGatewayNetworkRequest, type index_gen$1_CreateGatewayNetworkRequestIpamConfig as CreateGatewayNetworkRequestIpamConfig, type index_gen$1_CreateGatewayRequest as CreateGatewayRequest, type index_gen$1_CreateIPRequest as CreateIPRequest, type index_gen$1_CreatePATRuleRequest as CreatePATRuleRequest, type index_gen$1_DHCP as DHCP, type index_gen$1_DHCPEntry as DHCPEntry, type index_gen$1_DHCPEntryType as DHCPEntryType, type index_gen$1_DeleteDHCPEntryRequest as DeleteDHCPEntryRequest, type index_gen$1_DeleteDHCPRequest as DeleteDHCPRequest, type index_gen$1_DeleteGatewayNetworkRequest as DeleteGatewayNetworkRequest, type index_gen$1_DeleteGatewayRequest as DeleteGatewayRequest, type index_gen$1_DeleteIPRequest as DeleteIPRequest, type index_gen$1_DeletePATRuleRequest as DeletePATRuleRequest, index_gen$1_GATEWAY_NETWORK_TRANSIENT_STATUSES as GATEWAY_NETWORK_TRANSIENT_STATUSES, index_gen$1_GATEWAY_TRANSIENT_STATUSES as GATEWAY_TRANSIENT_STATUSES, type index_gen$1_Gateway as Gateway, type index_gen$1_GatewayNetwork as GatewayNetwork, type index_gen$1_GatewayNetworkStatus as GatewayNetworkStatus, type index_gen$1_GatewayStatus as GatewayStatus, type index_gen$1_GatewayType as GatewayType, type index_gen$1_GetDHCPEntryRequest as GetDHCPEntryRequest, type index_gen$1_GetDHCPRequest as GetDHCPRequest, type index_gen$1_GetGatewayNetworkRequest as GetGatewayNetworkRequest, type index_gen$1_GetGatewayRequest as GetGatewayRequest, type index_gen$1_GetIPRequest as GetIPRequest, type index_gen$1_GetPATRuleRequest as GetPATRuleRequest, type index_gen$1_IP as IP, type index_gen$1_IpamConfig as IpamConfig, type index_gen$1_ListDHCPEntriesRequest as ListDHCPEntriesRequest, type index_gen$1_ListDHCPEntriesRequestOrderBy as ListDHCPEntriesRequestOrderBy, type index_gen$1_ListDHCPEntriesResponse as ListDHCPEntriesResponse, type index_gen$1_ListDHCPsRequest as ListDHCPsRequest, type index_gen$1_ListDHCPsRequestOrderBy as ListDHCPsRequestOrderBy, type index_gen$1_ListDHCPsResponse as ListDHCPsResponse, type index_gen$1_ListGatewayNetworksRequest as ListGatewayNetworksRequest, type index_gen$1_ListGatewayNetworksRequestOrderBy as ListGatewayNetworksRequestOrderBy, type index_gen$1_ListGatewayNetworksResponse as ListGatewayNetworksResponse, type index_gen$1_ListGatewayTypesRequest as ListGatewayTypesRequest, type index_gen$1_ListGatewayTypesResponse as ListGatewayTypesResponse, type index_gen$1_ListGatewaysRequest as ListGatewaysRequest, type index_gen$1_ListGatewaysRequestOrderBy as ListGatewaysRequestOrderBy, type index_gen$1_ListGatewaysResponse as ListGatewaysResponse, type index_gen$1_ListIPsRequest as ListIPsRequest, type index_gen$1_ListIPsRequestOrderBy as ListIPsRequestOrderBy, type index_gen$1_ListIPsResponse as ListIPsResponse, type index_gen$1_ListPATRulesRequest as ListPATRulesRequest, type index_gen$1_ListPATRulesRequestOrderBy as ListPATRulesRequestOrderBy, type index_gen$1_ListPATRulesResponse as ListPATRulesResponse, type index_gen$1_PATRule as PATRule, type index_gen$1_PATRuleProtocol as PATRuleProtocol, type index_gen$1_RefreshSSHKeysRequest as RefreshSSHKeysRequest, type index_gen$1_SetDHCPEntriesRequest as SetDHCPEntriesRequest, type index_gen$1_SetDHCPEntriesRequestEntry as SetDHCPEntriesRequestEntry, type index_gen$1_SetDHCPEntriesResponse as SetDHCPEntriesResponse, type index_gen$1_SetPATRulesRequest as SetPATRulesRequest, type index_gen$1_SetPATRulesRequestRule as SetPATRulesRequestRule, type index_gen$1_SetPATRulesResponse as SetPATRulesResponse, type index_gen$1_UpdateDHCPEntryRequest as UpdateDHCPEntryRequest, type index_gen$1_UpdateDHCPRequest as UpdateDHCPRequest, type index_gen$1_UpdateGatewayNetworkRequest as UpdateGatewayNetworkRequest, type index_gen$1_UpdateGatewayNetworkRequestIpamConfig as UpdateGatewayNetworkRequestIpamConfig, type index_gen$1_UpdateGatewayRequest as UpdateGatewayRequest, type index_gen$1_UpdateIPRequest as UpdateIPRequest, type index_gen$1_UpdatePATRuleRequest as UpdatePATRuleRequest, type index_gen$1_UpgradeGatewayRequest as UpgradeGatewayRequest };
}

declare namespace index$1 {
  export { index_gen$1 as v1 };
}

type DnsRecordStatus = 'unknown_status' | 'valid' | 'invalid';
type DnsRecordType = 'unknown_type' | 'a' | 'cname' | 'mx' | 'txt' | 'ns' | 'aaaa';
type DnsRecordsStatus = 'unknown' | 'valid' | 'invalid';
type HostingDnsStatus = 'unknown_dns_status' | 'valid' | 'invalid';
type HostingStatus = 'unknown_status' | 'delivering' | 'ready' | 'deleting' | 'error' | 'locked' | 'migrating';
type ListHostingsRequestOrderBy = 'created_at_asc' | 'created_at_desc';
type ListOffersRequestOrderBy = 'price_asc';
type NameserverStatus = 'unknown_status' | 'valid' | 'invalid';
type OfferQuotaWarning = 'unknown_quota_warning' | 'email_count_exceeded' | 'database_count_exceeded' | 'disk_usage_exceeded';
interface HostingCpanelUrls {
    dashboard: string;
    webmail: string;
}
interface HostingOption {
    /** Option ID. */
    id: string;
    /** Option name. */
    name: string;
}
interface OfferProduct {
    /** Product name. */
    name: string;
    /** Product option. */
    option: boolean;
    /** Limit number of email accounts. */
    emailAccountsQuota: number;
    /** Limit quantity of email storage in gigabytes. */
    emailStorageQuota: number;
    /** Limit number of databases. */
    databasesQuota: number;
    /** Limit quantity of hosting storage in gigabytes. */
    hostingStorageQuota: number;
    /** Whether or not support is included. */
    supportIncluded: boolean;
    /** Limit number of virtual CPU. */
    vCpu: number;
    /** Limit quantity of memory in gigabytes. */
    ram: number;
    /** Limit number of add-on domains. */
    maxAddonDomains: number;
}
interface DnsRecord {
    /** Record name. */
    name: string;
    /** Record type. */
    type: DnsRecordType;
    /** Record time-to-live. */
    ttl: number;
    /** Record value. */
    value: string;
    /** Record priority level. */
    priority?: number;
    /** Record status. */
    status: DnsRecordStatus;
}
interface Nameserver {
    /** Hostname of the nameserver. */
    hostname: string;
    /** Status of the nameserver. */
    status: NameserverStatus;
    /** Defines whether the nameserver is the default one. */
    isDefault: boolean;
}
interface ControlPanel {
    /** Control panel name. */
    name: string;
    /** Define if the control panel type is available to order. */
    available: boolean;
    /** URL of this control panel's logo. */
    logoUrl: string;
}
interface Hosting {
    /** ID of the Web Hosting plan. */
    id: string;
    /** ID of the Scaleway Organization the Web Hosting plan belongs to. */
    organizationId: string;
    /** ID of the Scaleway Project the Web Hosting plan belongs to. */
    projectId: string;
    /** Date on which the Web Hosting plan was last updated. */
    updatedAt?: Date;
    /** Date on which the Web Hosting plan was created. */
    createdAt?: Date;
    /** Status of the Web Hosting plan. */
    status: HostingStatus;
    /** Hostname of the host platform. */
    platformHostname: string;
    /** Number of the host platform. */
    platformNumber?: number;
    /** ID of the active offer for the Web Hosting plan. */
    offerId: string;
    /** Name of the active offer for the Web Hosting plan. */
    offerName: string;
    /** Main domain associated with the Web Hosting plan. */
    domain: string;
    /** List of tags associated with the Web Hosting plan. */
    tags: string[];
    /** Array of any options activated for the Web Hosting plan. */
    options: HostingOption[];
    /** DNS status of the Web Hosting plan. */
    dnsStatus: HostingDnsStatus;
    /** URL to connect to cPanel dashboard and to Webmail interface. */
    cpanelUrls?: HostingCpanelUrls;
    /** Main Web Hosting cPanel username. */
    username: string;
    /** Indicates if the hosting offer has reached its end of life. */
    offerEndOfLife: boolean;
    /** Name of the control panel. */
    controlPanelName: string;
    /** Group of the hosting's host server/platform. */
    platformGroup: string;
    /** IPv4 address of the hosting's host server. */
    ipv4: string;
    /** IPv6 address of the hosting's host server. */
    ipv6: string;
    /** Region where the Web Hosting plan is hosted. */
    region: Region;
}
interface Offer {
    /** Offer ID. */
    id: string;
    /** Unique identifier used for billing. */
    billingOperationPath: string;
    /** Product constituting this offer. */
    product?: OfferProduct;
    /** Price of this offer. */
    price?: Money;
    /**
     * If a hosting_id was specified in the call, defines whether this offer is
     * available for that Web Hosting plan to migrate (update) to.
     */
    available: boolean;
    /** Quota warnings, if the offer is not available for the specified hosting_id. */
    quotaWarnings: OfferQuotaWarning[];
    /** Indicates if the offer has reached its end of life. */
    endOfLife: boolean;
    /** Name of the control panel. */
    controlPanelName: string;
}
type CreateHostingRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** ID of the selected offer for the Web Hosting plan. */
    offerId: string;
    /** ID of the Scaleway Project in which to create the Web Hosting plan. */
    projectId?: string;
    /** Contact email for the Web Hosting client. */
    email?: string;
    /** List of tags for the Web Hosting plan. */
    tags?: string[];
    /**
     * Domain name to link to the Web Hosting plan. You must already own this
     * domain name, and have completed the DNS validation process beforehand.
     */
    domain: string;
    /** IDs of any selected additional options for the Web Hosting plan. */
    optionIds?: string[];
    /** Default language for the control panel interface. */
    language?: LanguageCode;
};
type DeleteHostingRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hosting ID. */
    hostingId: string;
};
interface DnsRecords {
    /** List of DNS records. */
    records: DnsRecord[];
    /** List of nameservers. */
    nameServers: Nameserver[];
    /** Status of the records. */
    status: DnsRecordsStatus;
}
type GetDomainDnsRecordsRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Domain associated with the DNS records. */
    domain: string;
};
type GetHostingRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hosting ID. */
    hostingId: string;
};
type ListControlPanelsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * Page number to return, from the paginated results (must be a positive
     * integer).
     */
    page?: number;
    /**
     * Number of control panels to return (must be a positive integer lower or
     * equal to 100).
     */
    pageSize?: number;
};
interface ListControlPanelsResponse {
    /** Number of control panels returned. */
    totalCount: number;
    /** List of control panels. */
    controlPanels: ControlPanel[];
}
type ListHostingsRequest$1 = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /**
     * Page number to return, from the paginated results (must be a positive
     * integer).
     */
    page?: number;
    /**
     * Number of Web Hosting plans to return (must be a positive integer lower or
     * equal to 100).
     */
    pageSize?: number;
    /** Sort order for Web Hosting plans in the response. */
    orderBy?: ListHostingsRequestOrderBy;
    /**
     * Tags to filter for, only Web Hosting plans with matching tags will be
     * returned.
     */
    tags?: string[];
    /**
     * Statuses to filter for, only Web Hosting plans with matching statuses will
     * be returned.
     */
    statuses?: HostingStatus[];
    /**
     * Domain to filter for, only Web Hosting plans associated with this domain
     * will be returned.
     */
    domain?: string;
    /**
     * Project ID to filter for, only Web Hosting plans from this Project will be
     * returned.
     */
    projectId?: string;
    /**
     * Organization ID to filter for, only Web Hosting plans from this
     * Organization will be returned.
     */
    organizationId?: string;
    /**
     * Name of the control panel to filter for, only Web Hosting plans from this
     * control panel will be returned.
     */
    controlPanels?: string[];
};
interface ListHostingsResponse {
    /** Number of Web Hosting plans returned. */
    totalCount: number;
    /** List of Web Hosting plans. */
    hostings: Hosting[];
}
type ListOffersRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Sort order of offers in the response. */
    orderBy?: ListOffersRequestOrderBy;
    /**
     * Defines whether the response should consist of offers only, without
     * options.
     */
    withoutOptions: boolean;
    /**
     * Defines whether the response should consist of options only, without
     * offers.
     */
    onlyOptions: boolean;
    /**
     * ID of a Web Hosting plan, to check compatibility with returned offers (in
     * case of wanting to update the plan).
     */
    hostingId?: string;
    /** Name of the control panel to filter for. */
    controlPanels?: string[];
};
interface ListOffersResponse {
    /** List of offers. */
    offers: Offer[];
}
type RestoreHostingRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hosting ID. */
    hostingId: string;
};
type UpdateHostingRequest = {
    /**
     * Region to target. If none is passed will use default region from the
     * config.
     */
    region?: Region;
    /** Hosting ID. */
    hostingId: string;
    /** New contact email for the Web Hosting plan. */
    email?: string;
    /** New tags for the Web Hosting plan. */
    tags?: string[];
    /** IDs of the new options for the Web Hosting plan. */
    optionIds?: string[];
    /** ID of the new offer for the Web Hosting plan. */
    offerId?: string;
};

/** Web Hosting API. */
declare class API extends API$x {
    /** Lists the available regions of the API. */
    static readonly LOCALITIES: Region[];
    /**
     * Order a Web Hosting plan. Order a Web Hosting plan, specifying the offer
     * type required via the `offer_id` parameter.
     *
     * @param request - The request {@link CreateHostingRequest}
     * @returns A Promise of Hosting
     */
    createHosting: (request: Readonly<CreateHostingRequest>) => Promise<Hosting>;
    protected pageOfListHostings: (request?: Readonly<ListHostingsRequest$1>) => Promise<ListHostingsResponse>;
    /**
     * List all Web Hosting plans. List all of your existing Web Hosting plans.
     * Various filters are available to limit the results, including filtering by
     * domain, status, tag and Project ID.
     *
     * @param request - The request {@link ListHostingsRequest}
     * @returns A Promise of ListHostingsResponse
     */
    listHostings: (request?: Readonly<ListHostingsRequest$1>) => Promise<ListHostingsResponse> & {
        all: () => Promise<Hosting[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<Hosting[], void, void>;
    };
    /**
     * Get a Web Hosting plan. Get the details of one of your existing Web Hosting
     * plans, specified by its `hosting_id`.
     *
     * @param request - The request {@link GetHostingRequest}
     * @returns A Promise of Hosting
     */
    getHosting: (request: Readonly<GetHostingRequest>) => Promise<Hosting>;
    /**
     * Waits for {@link Hosting} to be in a final state.
     *
     * @param request - The request {@link GetHostingRequest}
     * @param options - The waiting options
     * @returns A Promise of Hosting
     */
    waitForHosting: (request: Readonly<GetHostingRequest>, options?: Readonly<WaitForOptions<Hosting>>) => Promise<Hosting>;
    /**
     * Update a Web Hosting plan. Update the details of one of your existing Web
     * Hosting plans, specified by its `hosting_id`. You can update parameters
     * including the contact email address, tags, options and offer.
     *
     * @param request - The request {@link UpdateHostingRequest}
     * @returns A Promise of Hosting
     */
    updateHosting: (request: Readonly<UpdateHostingRequest>) => Promise<Hosting>;
    /**
     * Delete a Web Hosting plan. Delete a Web Hosting plan, specified by its
     * `hosting_id`. Note that deletion is not immediate: it will take place at
     * the end of the calendar month, after which time your Web Hosting plan and
     * all its data (files and emails) will be irreversibly lost.
     *
     * @param request - The request {@link DeleteHostingRequest}
     * @returns A Promise of Hosting
     */
    deleteHosting: (request: Readonly<DeleteHostingRequest>) => Promise<Hosting>;
    /**
     * Restore a Web Hosting plan. When you [delete a Web Hosting
     * plan](#path-hostings-delete-a-hosting), definitive deletion does not take
     * place until the end of the calendar month. In the time between initiating
     * the deletion, and definitive deletion at the end of the month, you can
     * choose to **restore** the Web Hosting plan, using this endpoint and
     * specifying its `hosting_id`.
     *
     * @param request - The request {@link RestoreHostingRequest}
     * @returns A Promise of Hosting
     */
    restoreHosting: (request: Readonly<RestoreHostingRequest>) => Promise<Hosting>;
    /**
     * Get DNS records. Get the set of DNS records of a specified domain
     * associated with a Web Hosting plan.
     *
     * @param request - The request {@link GetDomainDnsRecordsRequest}
     * @returns A Promise of DnsRecords
     */
    getDomainDnsRecords: (request: Readonly<GetDomainDnsRecordsRequest>) => Promise<DnsRecords>;
    /**
     * List all offers. List the different Web Hosting offers, and their options,
     * available to order from Scaleway.
     *
     * @param request - The request {@link ListOffersRequest}
     * @returns A Promise of ListOffersResponse
     */
    listOffers: (request: Readonly<ListOffersRequest>) => Promise<ListOffersResponse>;
    protected pageOfListControlPanels: (request?: Readonly<ListControlPanelsRequest$1>) => Promise<ListControlPanelsResponse>;
    /**
     * List all control panels type. List the control panels type: cpanel or
     * plesk.
     *
     * @param request - The request {@link ListControlPanelsRequest}
     * @returns A Promise of ListControlPanelsResponse
     */
    listControlPanels: (request?: Readonly<ListControlPanelsRequest$1>) => Promise<ListControlPanelsResponse> & {
        all: () => Promise<ControlPanel[]>;
        [Symbol.asyncIterator]: () => AsyncGenerator<ControlPanel[], void, void>;
    };
}

/** Lists transient statutes of the enum {@link HostingStatus}. */
declare const HOSTING_TRANSIENT_STATUSES: HostingStatus[];

declare const ListControlPanelsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};
declare const ListHostingsRequest: {
    page: {
        greaterThan: number;
    };
    pageSize: {
        greaterThan: number;
        lessThanOrEqual: number;
    };
};

declare const validationRules_gen_ListControlPanelsRequest: typeof ListControlPanelsRequest;
declare const validationRules_gen_ListHostingsRequest: typeof ListHostingsRequest;
declare namespace validationRules_gen {
  export { validationRules_gen_ListControlPanelsRequest as ListControlPanelsRequest, validationRules_gen_ListHostingsRequest as ListHostingsRequest };
}

type index_gen_API = API;
declare const index_gen_API: typeof API;
type index_gen_ControlPanel = ControlPanel;
type index_gen_CreateHostingRequest = CreateHostingRequest;
type index_gen_DeleteHostingRequest = DeleteHostingRequest;
type index_gen_DnsRecord = DnsRecord;
type index_gen_DnsRecordStatus = DnsRecordStatus;
type index_gen_DnsRecordType = DnsRecordType;
type index_gen_DnsRecords = DnsRecords;
type index_gen_DnsRecordsStatus = DnsRecordsStatus;
type index_gen_GetDomainDnsRecordsRequest = GetDomainDnsRecordsRequest;
type index_gen_GetHostingRequest = GetHostingRequest;
declare const index_gen_HOSTING_TRANSIENT_STATUSES: typeof HOSTING_TRANSIENT_STATUSES;
type index_gen_Hosting = Hosting;
type index_gen_HostingCpanelUrls = HostingCpanelUrls;
type index_gen_HostingDnsStatus = HostingDnsStatus;
type index_gen_HostingOption = HostingOption;
type index_gen_HostingStatus = HostingStatus;
type index_gen_ListControlPanelsResponse = ListControlPanelsResponse;
type index_gen_ListHostingsRequestOrderBy = ListHostingsRequestOrderBy;
type index_gen_ListHostingsResponse = ListHostingsResponse;
type index_gen_ListOffersRequest = ListOffersRequest;
type index_gen_ListOffersRequestOrderBy = ListOffersRequestOrderBy;
type index_gen_ListOffersResponse = ListOffersResponse;
type index_gen_Nameserver = Nameserver;
type index_gen_NameserverStatus = NameserverStatus;
type index_gen_Offer = Offer;
type index_gen_OfferProduct = OfferProduct;
type index_gen_OfferQuotaWarning = OfferQuotaWarning;
type index_gen_RestoreHostingRequest = RestoreHostingRequest;
type index_gen_UpdateHostingRequest = UpdateHostingRequest;
declare namespace index_gen {
  export { index_gen_API as API, type index_gen_ControlPanel as ControlPanel, type index_gen_CreateHostingRequest as CreateHostingRequest, type index_gen_DeleteHostingRequest as DeleteHostingRequest, type index_gen_DnsRecord as DnsRecord, type index_gen_DnsRecordStatus as DnsRecordStatus, type index_gen_DnsRecordType as DnsRecordType, type index_gen_DnsRecords as DnsRecords, type index_gen_DnsRecordsStatus as DnsRecordsStatus, type index_gen_GetDomainDnsRecordsRequest as GetDomainDnsRecordsRequest, type index_gen_GetHostingRequest as GetHostingRequest, index_gen_HOSTING_TRANSIENT_STATUSES as HOSTING_TRANSIENT_STATUSES, type index_gen_Hosting as Hosting, type index_gen_HostingCpanelUrls as HostingCpanelUrls, type index_gen_HostingDnsStatus as HostingDnsStatus, type index_gen_HostingOption as HostingOption, type index_gen_HostingStatus as HostingStatus, type ListControlPanelsRequest$1 as ListControlPanelsRequest, type index_gen_ListControlPanelsResponse as ListControlPanelsResponse, type ListHostingsRequest$1 as ListHostingsRequest, type index_gen_ListHostingsRequestOrderBy as ListHostingsRequestOrderBy, type index_gen_ListHostingsResponse as ListHostingsResponse, type index_gen_ListOffersRequest as ListOffersRequest, type index_gen_ListOffersRequestOrderBy as ListOffersRequestOrderBy, type index_gen_ListOffersResponse as ListOffersResponse, type index_gen_Nameserver as Nameserver, type index_gen_NameserverStatus as NameserverStatus, type index_gen_Offer as Offer, type index_gen_OfferProduct as OfferProduct, type index_gen_OfferQuotaWarning as OfferQuotaWarning, type index_gen_RestoreHostingRequest as RestoreHostingRequest, type index_gen_UpdateHostingRequest as UpdateHostingRequest, validationRules_gen as ValidationRules };
}

declare namespace index {
  export { index_gen as v1alpha1 };
}

export { API$x as API, index$y as Account, index$x as AppleSilicon, index$v as BareMetal, index$u as Billing, index$t as Block, type Client, type ClientConfig, index$s as Cockpit, index$r as Container, type DefaultValues, index$q as DocumentDB, index$p as Domain, index$z as Errors, index$o as FlexibleIP, index$n as Function, index$m as IAM, index$j as IOT, index$i as IPAM, index$h as IPFS, index$k as Instance, index$g as Jobs, index$e as K8S, index$c as LB, type Logger, index$a as MNQ, index$b as Marketplace, type Money, type NetworkInterceptors, type Profile, index$9 as RDB, index$8 as Redis, type Region, index$7 as Registry, type RequestInterceptor, type ResponseErrorInterceptor, type ResponseInterceptor, type ScwFile, index$6 as Secret, index$5 as ServerlessSQLDB, type ServiceInfo, type Settings, index_gen$6 as Std, index$4 as Test, type TimeSeries, index$3 as TransactionalEmail, index$2 as VPC, index$1 as VPCGW, type WaitForOptions, type WaitForStopCondition, index as Webhosting, type Zone, addAsyncHeaderInterceptor, authenticateWithSessionToken, createAdvancedClient, createClient, createExponentialBackoffStrategy, enableConsoleLogger, enrichForPagination, isJSONObject, marshalMoney, marshalScwFile, marshalTimeSeries, resolveOneOf, setLogger, tryAtIntervals, unmarshalArrayOfObject, unmarshalDate, unmarshalMapOfObject, unmarshalMoney, unmarshalScwFile, unmarshalServiceInfo, unmarshalTimeSeries, unmarshalTimeSeriesPoint, urlParams, validatePathParam, waitForResource, withAdditionalInterceptors, withDefaultPageSize, withHTTPClient, withProfile, withUserAgent, withUserAgentSuffix };
